//-----------------------------------------------------------------------
//  This file is part of the UC Merced Microsoft Robotics Developer Studio 
// Project Code Base.
//
//  Created by Benjamin Balaguer under the supervision of Stefano Carpin.
//
//  $File: ComStation2009ControllerTypes.cs $
//-----------------------------------------------------------------------

using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using System;
using System.Collections.Generic;
using W3C.Soap;
using comstation2009controller = Robotics.ComStation2009Controller;
using Microsoft.Dss.Core.DsspHttp;
using System.Collections;
using System.IO;
using System.Drawing;


namespace Robotics.ComStation2009Controller
{
    /// <summary>
    /// ComStation2009Controller Contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// The Dss Service contract
        /// </summary>
        public const String Identifier = "http://schemas.tempuri.org/2008/04/comstation2009controller.html";
    }
    
    /// <summary>
    /// The ComStation2009Controller State
    /// </summary>
    [DataContract()]
    public class ComStation2009ControllerState
    {
    }

    #region Main Operations
    /// <summary>
    /// ComStation2009Controller Main Operations Port
    /// </summary>
    [ServicePort()]
    public class ComStation2009ControllerOperations : PortSet<
        DsspDefaultLookup, 
        DsspDefaultDrop, 
        Get, 
        HttpGet, 
        Subscribe,
        CONNECTED_MSG,
        DISCONNECTED_MSG,
        START_POSES_MSG,
        INITIALIZED_MSG,
        MIS_STA_MSG,
        IMAGE_MSG,
        CONNECT_CMD,
        GET_POSES_CMD,
        INIT_CMD,
        GET_CONF_CMD,
        DRIVE_AERIAL_CMD,
        MIS_PKG_CMD,
        IMG_SVR_CAM_CMD,
        IMG_SVR_OK_CMD>
    {
    }
    #endregion

    /// <summary>
    /// ComStation2009Controller Get Operation
    /// </summary>
    public class Get : Get<GetRequestType, PortSet<ComStation2009ControllerState, Fault>>
    {
        
        /// <summary>
        /// ComStation2009Controller Get Operation
        /// </summary>
        public Get()
        {
        }
        
        /// <summary>
        /// ComStation2009Controller Get Operation
        /// </summary>
        public Get(Microsoft.Dss.ServiceModel.Dssp.GetRequestType body) : 
                base(body)
        {
        }
        
        /// <summary>
        /// ComStation2009Controller Get Operation
        /// </summary>
        public Get(Microsoft.Dss.ServiceModel.Dssp.GetRequestType body, Microsoft.Ccr.Core.PortSet<ComStation2009ControllerState, W3C.Soap.Fault> responsePort) : 
                base(body, responsePort)
        {
        }
    }

    // To handle subscriptions
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }

    #region Messages Going to GUI
    #region CONNECTED MESSAGE
    public class CONNECTED_MSG : Update<ConnectedMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {

        public CONNECTED_MSG()
            : base(new ConnectedMSGRequest())
        {
        }
    }

    [DataContract]
    public class ConnectedMSGRequest
    {
        public ConnectedMSGRequest()
        {
        }
    }
    #endregion

    #region DISCONNECTED MESSAGE
    public class DISCONNECTED_MSG : Update<DisconnectedMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {

        public DISCONNECTED_MSG()
            : base(new DisconnectedMSGRequest())
        {
        }
    }

    [DataContract]
    public class DisconnectedMSGRequest
    {
        public DisconnectedMSGRequest()
        {
        }
    }
    #endregion

    #region INITIALIZED MESSAGE
    public class INITIALIZED_MSG : Update<InitializedMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public INITIALIZED_MSG()
            : base(new InitializedMSGRequest())
        {
        }
    }

    [DataContract]
    public class InitializedMSGRequest
    {
        public InitializedMSGRequest()
        {
        }
    }
    #endregion

    #region STARTPOSES MESSAGE
    public class START_POSES_MSG : Update<StartPosesMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {

        public START_POSES_MSG() : base(new StartPosesMSGRequest())
        {
        }
    }

    [DataContract]
    public class StartPosesMSGRequest
    {
        public StartPosesMSGRequest()
        {
            _startPoses = new List<StartPoses>();
        }
        
        private List<StartPoses> _startPoses;

        [DataMember]
        public List<StartPoses> startPoses
        {
            get { return _startPoses; }
            set { _startPoses = value; }
        }
    }
    #endregion
 
    #region MISSTA MESSAGE
    public class MIS_STA_MSG : Update<MisStaMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public MIS_STA_MSG() : base(new MisStaMSGRequest())
        {
        }
    }

    [DataContract]
    public class MisStaMSGRequest
    {
        public MisStaMSGRequest()
        {
            _misPkg = new List<MisStatus>();
        }

        private List<MisStatus> _misPkg;

        [DataMember]
        public List<MisStatus> misPkg
        {
            get { return _misPkg; }
            set { _misPkg = value; }
        }
    }
    #endregion
   
    #region GPS MESSAGE
    public class GPS_MSG : Update<GPSMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public GPS_MSG() : base(new GPSMSGRequest())
        {
        }
    }

    [DataContract]
    public class GPSMSGRequest
    {
        public GPSMSGRequest()
        {
            _gps = new GPS();
        }

        private GPS _gps;

        [DataMember]
        public GPS gps
        {
            get { return _gps; }
            set { _gps = value; }
        }
    }
    #endregion

    #region IMAGE MESSAGE
    public class IMAGE_MSG : Update<ImageMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public IMAGE_MSG()
            : base(new ImageMSGRequest())
        {
        }

        public IMAGE_MSG(Image jpeg)
            : base(new ImageMSGRequest(jpeg))
        {
        }
    }

    [DataContract]
    public class ImageMSGRequest
    {
        public ImageMSGRequest()
        {
        }

        public ImageMSGRequest(Image jpeg)
        {
            _jpeg = jpeg;
        }

        private Image _jpeg;

        [DataMember]
        public Image JPEG
        {
            get { return _jpeg; }
            set { _jpeg = value; }
        }
    }
    #endregion

    #region CONNECT CMD
    public class CONNECT_CMD : Update<ConnectCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public CONNECT_CMD()
            : base(new ConnectCMDRequest())
        {
        }

        public CONNECT_CMD(string IP, int gameBotPort, int imgSvrPort)
            : base(new ConnectCMDRequest(IP, gameBotPort, imgSvrPort))
        {
        }
    }

    [DataContract]
    public class ConnectCMDRequest
    {
        public ConnectCMDRequest()
        {
        }

        public ConnectCMDRequest(string ip, int gameBotPort, int imgSvrPort)
        {
            _ip = ip;
            _gameBotPort = gameBotPort;
            _imgSvrPort = imgSvrPort;
        }

        private string _ip;
        private int _gameBotPort;
        private int _imgSvrPort;

        [DataMember]
        public string IP
        {
            get { return _ip; }
            set { _ip = value; }
        }

        [DataMember]
        public int gameBotPort
        {
            get { return _gameBotPort; }
            set { _gameBotPort = value; }
        }

        [DataMember]
        public int imgSvrPort
        {
            get { return _imgSvrPort; }
            set { _imgSvrPort = value; }
        }
    }
    #endregion

    #region GETSTARTPOSES COMMAND
    public class GET_POSES_CMD : Update<GetPosesCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public GET_POSES_CMD()
            : base(new GetPosesCMDRequest())
        {
        }
    }

    [DataContract]
    public class GetPosesCMDRequest
    {
        public GetPosesCMDRequest()
        {
        }
    }
    #endregion

    #region INIT COMMAND
    public class INIT_CMD : Update<InitCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public INIT_CMD()
            : base(new InitCMDRequest())
        {
        }

        public INIT_CMD(string Robot, string Name, string Location, string Rotation)
            : base(new InitCMDRequest(Robot, Name, Location, Rotation))
        {
        }
    }

    [DataContract]
    public class InitCMDRequest
    {
        public InitCMDRequest()
        {
        }

        public InitCMDRequest(string robot, string name, string location, string rotation)
        {
            _robot = robot;
            _name = name;
            _location = location;
            _rotation = rotation;
        }

        private string _robot;
        private string _name;
        private string _location;
        private string _rotation;

        [DataMember]
        public string Robot
        {
            get { return _robot; }
            set { _name = value; }
        }

        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public string Location
        {
            get { return _location; }
            set { _location = value; }
        }

        [DataMember]
        public string Rotation
        {
            get { return _rotation; }
            set { _rotation = value; }
        }
    }
    #endregion

    #region GETCONF COMMAND
    public class GET_CONF_CMD : Update<GetConfCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public GET_CONF_CMD()
            : base(new GetConfCMDRequest())
        {
        }

        public GET_CONF_CMD(string Type, string Name)
            : base(new GetConfCMDRequest(Type, Name))
        {
        }
    }

    [DataContract]
    public class GetConfCMDRequest
    {
        public GetConfCMDRequest()
        {
        }

        public GetConfCMDRequest(string type, string name)
        {
            _type = type;
            _name = name;
        }

        private string _type;
        private string _name;

        [DataMember]
        public string Type
        {
            get { return _type; }
            set { _type = value; }
        }

        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }
    }
    #endregion

    #region AERIAL DRIVE COMMAND
    public class DRIVE_AERIAL_CMD : Update<DriveAerialCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public DRIVE_AERIAL_CMD()
            : base(new DriveAerialCMDRequest())
        {
        }

        public DRIVE_AERIAL_CMD(string AltitudeVelocity, string LinearVelocity, string LateralVelocity, string RotationalVelocity, bool Normalized)
            : base(new DriveAerialCMDRequest(AltitudeVelocity, LinearVelocity, LateralVelocity, RotationalVelocity, Normalized))
        {
        }
    }

    [DataContract]
    public class DriveAerialCMDRequest
    {
        public DriveAerialCMDRequest()
        {
        }

        public DriveAerialCMDRequest(string altitudeVelocity, string linearVelocity, string lateralVelocity, string rotationalVelocity, bool normalized)
        {
            _altitudeVelocity = altitudeVelocity;
            _linearVelocity = linearVelocity;
            _lateralVelocity = lateralVelocity;
            _rotationalVelocity = rotationalVelocity;
            _normalized = normalized;
        }

        private string _altitudeVelocity;
        private string _linearVelocity;
        private string _lateralVelocity;
        private string _rotationalVelocity;
        private bool _normalized;

        [DataMember]
        public string AltitudeVelocity
        {
            get { return _altitudeVelocity; }
            set { _altitudeVelocity = value; }
        }

        [DataMember]
        public string LinearVelocity
        {
            get { return _linearVelocity; }
            set { _linearVelocity = value; }
        }

        [DataMember]
        public string LateralVelocity
        {
            get { return _lateralVelocity; }
            set { _lateralVelocity = value; }
        }

        [DataMember]
        public string RotationalVelocity
        {
            get { return _rotationalVelocity; }
            set { _rotationalVelocity = value; }
        }

        [DataMember]
        public bool Normalized
        {
            get { return _normalized; }
            set { _normalized = value; }
        }
    }
    #endregion

    #region MISSION PACKAGE COMMAND
    public class MIS_PKG_CMD : Update<MisPkgCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public MIS_PKG_CMD()
            : base(new MisPkgCMDRequest())
        {
        }

        public MIS_PKG_CMD(string Name, List<LinkProperty> Link)
            : base(new MisPkgCMDRequest(Name, Link))
        {
        }
    }
    
    [DataContract]
    public class MisPkgCMDRequest
    {
        public MisPkgCMDRequest()
        {
        }

        public MisPkgCMDRequest(string name, List<LinkProperty> link)
        {
            _name = name;
            _link = link;
        }

        private string _name;
        private List<LinkProperty> _link;

        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public List<LinkProperty> Link
        {
            get { return _link; }
            set { _link = value; }
        }
    }
    #endregion

    #region IMAGE SERVER COMMAND
    public class IMG_SVR_CAM_CMD : Update<ImgSvrCamCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public IMG_SVR_CAM_CMD()
            : base(new ImgSvrCamCMDRequest())
        {
        }

        public IMG_SVR_CAM_CMD(string Robot, string Cam, string IP)
            : base(new ImgSvrCamCMDRequest(Robot, Cam, IP))
        {
        }
    }

    [DataContract]
    public class ImgSvrCamCMDRequest
    {
        public ImgSvrCamCMDRequest()
        {
        }

        public ImgSvrCamCMDRequest(string robot, string cam, string ip)
        {
            _robot = robot;
            _cam = cam;
            _ip = ip;
        }

        private string _robot;
        private string _cam;
        private string _ip;

        [DataMember]
        public string RobotName
        {
            get { return _robot; }
            set { _robot = value; }
        }

        [DataMember]
        public string CamName
        {
            get { return _cam; }
            set { _cam = value; }
        }

        [DataMember]
        public string IP
        {
            get { return _ip; }
            set { _ip = value; }
        }

    }
    #endregion

    #region IMAGE SERVER OK COMMAND
    public class IMG_SVR_OK_CMD : Update<ImgSvrOKCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public IMG_SVR_OK_CMD()
            : base(new ImgSvrOKCMDRequest())
        {
        }
    }

    [DataContract]
    public class ImgSvrOKCMDRequest
    {
        public ImgSvrOKCMDRequest()
        {
        }
    }
    #endregion
    #endregion
}
