//-----------------------------------------------------------------------
//  This file is part of the UC Merced Microsoft Robotics Developer Studio 
// Project Code Base.
//
//  Created by Benjamin Balaguer under the supervision of Stefano Carpin.
//
//  $File: GameBot.cs $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.Windows.Media.Imaging;

namespace Robotics.ComStation2009Controller
{
    #region GameBot Communication Interface
    class GameBot
    {
        #region Variables
        private static TcpClient gameBotClient;
        private static BufferedStream gameBotStream;
        private static Thread gameBotThread;
        private static TcpClient imgSvrClient;
        private static BufferedStream imgSvrStream;
        public GameBotParser Parser;
        public GameBotCmd Cmd;
        private ComStation2009ControllerOperations _mainPort;
        private WSS wss;

        public System.Object writeLock = new System.Object();
        #endregion

        #region Constructor
        public GameBot(ComStation2009ControllerOperations MainPort, WSS Wss) // Constructor
        {
            wss = Wss;
            _mainPort = MainPort;
            Parser = new GameBotParser(MainPort, wss);
            Cmd = new GameBotCmd();
        }
        #endregion

        #region Connect/Disconnect
        public void Connect(string IP, int Port)
        {
            // Create Game Bot Communication/Thread
            gameBotClient = new TcpClient();
            gameBotClient.Connect(new IPEndPoint(IPAddress.Parse(IP), Port));
            gameBotStream = new BufferedStream(gameBotClient.GetStream(), 10000);
            gameBotThread = new Thread(new ParameterizedThreadStart(HandleGameBotComm));
            gameBotThread.Start(gameBotClient);
        }

        public void ConnectImgSvr(string IP, int Port)
        {
            // Create Image Server Communication/Thread
            imgSvrClient = new TcpClient();
            imgSvrClient.Connect(new IPEndPoint(IPAddress.Parse(IP), Port));
            imgSvrStream = new BufferedStream(imgSvrClient.GetStream(), 10000);
            ReadImgSvrImage();
        }

        public void Disconnect()
        {
            Parser.ResetVariables();
            gameBotThread.Abort();
            gameBotStream.Close();
            gameBotClient.Close();
            //imgSvrThread.Abort();
            imgSvrStream.Close();
            imgSvrClient.Close();
        }
        #endregion

        #region GameBot and Image Server Threads
        private void HandleGameBotComm(object client)
        {
            while (true)
            {
                try
                {
                    Parser.Parse(ReadLine());
                }
                catch(Exception e)
                {
                    System.Console.WriteLine("Caught ReadLine Exception: " + e.ToString());
                }
            }
        }

        private void ReadImgSvrImage()
        {
            try
            {
                int imgType;   // Store the image type
                int imgSize;   // Store the image size 
                byte[] buffer;


                imgType = imgSvrStream.ReadByte();
                imgSize = (int)(imgSvrStream.ReadByte()) * 16777216 + (int)(imgSvrStream.ReadByte()) * 65536 + (int)(imgSvrStream.ReadByte()) * 256 + (int)(imgSvrStream.ReadByte());

                buffer = new byte[imgSize];

                // Read in the whole image
                int pos = 0;
                while (pos < imgSize)
                {
                    pos += imgSvrStream.Read(buffer, pos, imgSize - pos);
                }

                string msg = "IMG " + imgSize + " ";
                //for (int i = 0; i < imgSize; i++)
                //{
                //    msg += buffer[i].ToString() + " ";
                //}
                //msg += "test ";
                lock (writeLock)
                {
                    wss.WriteLine("GUI", msg + ".");
                    wss.WriteBinary("GUI", ref buffer);
                }
                //System.Console.WriteLine("Sent Image Buffer: " + msg);

                //System.Console.WriteLine("IMG IS: " + msg);
                /*
                Stream imageStream = new MemoryStream(buffer, 0, buffer.Length);
                JpegBitmapDecoder decoder = new JpegBitmapDecoder(imageStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
                //BitmapFrame test = BitmapFrame.Create(decoder.Frames[0]);

                int stride = decoder.Frames[0].PixelWidth * decoder.Frames[0].Format.BitsPerPixel / 8;
                stride = stride + (stride % 4) * 4;

                byte[] Pixels = new byte[stride * decoder.Frames[0].PixelHeight * decoder.Frames[0].Format.BitsPerPixel / 8];
                decoder.Frames[0].CopyPixels(Pixels, stride, 0);

                Image imgSvrImg;
                
                unsafe
                {
                    fixed (byte* pPixels = Pixels)
                    {
                        IntPtr ptr = new IntPtr(pPixels);
                        imgSvrImg = new Bitmap(decoder.Frames[0].PixelWidth, decoder.Frames[0].PixelHeight, stride, System.Drawing.Imaging.PixelFormat.Format24bppRgb, ptr);
                    }
                }

                //System.Console.WriteLine("R: " + ((Bitmap)imgSvrImg).GetPixel(0, ((Bitmap)imgSvrImg).Height - 2).R + " / G: " +
                //                     ((Bitmap)imgSvrImg).GetPixel(0, ((Bitmap)imgSvrImg).Height - 2).G + " / B: " +
                //                     ((Bitmap)imgSvrImg).GetPixel(0, ((Bitmap)imgSvrImg).Height - 2).B);

                _mainPort.Post(new IMAGE_MSG(imgSvrImg));
                */
                //imgSvrThread.Suspend();
                //}
            }
            catch (Exception e)
            {
                System.Console.WriteLine("Caught Exception in ReadImgSvrImage: " + e.ToString());
            }
        }
        #endregion

        #region Read/Write to GameBots
        public string ReadLine()
        {
            char currentChar;
            bool done = false;
            string result = "";

            for (int i = 0; !done; i++)
            {
                currentChar = (char)gameBotStream.ReadByte();

                // If we have found a carriage return
                if ((int)currentChar == 13)
                {
                    gameBotStream.ReadByte(); // Discard the next character (line feed)
                    done = true;              // Set the condition to drop out of the loop
                }
                else
                {
                    // Add the character to the resulting string
                    result = result + currentChar;
                }
            }

            return result;
        }

        public void WriteLine(string cmd)
        {
            ASCIIEncoding encoder = new ASCIIEncoding();
            byte[] buffer = encoder.GetBytes(cmd + (char)13 + (char)10);

            try
            {
                gameBotStream.Write(buffer, 0, buffer.Length);
                gameBotStream.Flush();
            }
            catch (Exception)
            {
                //System.Console.WriteLine("Caught an Exception while Writing: " + e.ToString());
            }
            
        }
        
        public void SendOKImgSvr()
        {
            ASCIIEncoding encoder = new ASCIIEncoding();
            byte[] buffer = encoder.GetBytes("OK"/* + (char)13 + (char)10*/);

            try
            {
                //System.Console.WriteLine("Writing OK Message");
                imgSvrStream.Write(buffer, 0, buffer.Length);
                //System.Console.WriteLine("Writing OK Message2");
                imgSvrStream.Flush();
                //System.Console.WriteLine("Writing OK Message3");
                ReadImgSvrImage();
                //System.Console.WriteLine("DONE! Writing OK Message");
            }
            catch (Exception/* e*/)
            {
                //System.Console.WriteLine("!!!!Caught an Exception while Writing: " + e.ToString());
            }
        }
        #endregion
    }
    #endregion

    #region GameBot String Parser
    class GameBotParser
    {
        #region Variables
        private ComStation2009ControllerOperations _mainPort;
        public ArrayList startPoses;
        private START_POSES_MSG StartPosesMsg;
        private MIS_STA_MSG MisPkgMsg;
        private GPS_MSG GPSMsg;
        private bool gotStartPoses;
        private bool gotMisPkgCONF;
        private WSS wss;
        #endregion

        #region Constructor
        public GameBotParser(ComStation2009ControllerOperations MainPort, WSS Wss)
        {
            wss = Wss;
            _mainPort = MainPort;
            startPoses = new ArrayList();
            StartPosesMsg = new START_POSES_MSG();
            MisPkgMsg = new MIS_STA_MSG();
            GPSMsg = new GPS_MSG();
            gotStartPoses = false;
            gotMisPkgCONF = false;
        }
        #endregion

        #region Reset Variable Section (in case we are disconnected)
        public void ResetVariables()
        {
            MisPkgMsg.Body.misPkg.Clear();
            gotStartPoses = false;
            gotMisPkgCONF = false;
        }
        #endregion

        #region Main Parsing Function
        public void Parse(string toParse)
        {
            try
            {
                if (toParse.IndexOf("NFO", 0) == 0)
                {
                    ParseNFO(DeleteTag(toParse, "NFO"));
                }
                else if (toParse.IndexOf("STA", 0) == 0)
                {
                    ParseSTA(DeleteTag(toParse, "STA"));
                }
                else if (toParse.IndexOf("MISSTA", 0) == 0)
                {
                    // Only process the MISSTA message if we have gotten its CONF information
                    if (gotMisPkgCONF)
                        ParseMISSTA(DeleteTag(toParse, "MISSTA"));
                }
                else if (toParse.IndexOf("CONF", 0) == 0)
                {
                    ParseCONF(DeleteTag(toParse, "CONF"));
                }
                else if (toParse.IndexOf("SEN", 0) == 0)
                {
                    ParseSEN(DeleteTag(toParse, "SEN"));
                }
            }
            catch(Exception e)
            {
                System.Console.WriteLine("Caught an Exception: " + e.ToString());
                System.Console.WriteLine("Error String: " + toParse);
            }
        }
        #endregion

        #region SEN Message Parsing
        private void ParseSEN(string SEN)
        {
            string tmp;

            // Get the time stamp
            SEN = DeleteTag(SEN, "{Time");
            tmp = GetParameter(SEN, "}");
            SEN = DeleteTag(SEN, tmp);

            if (SEN.IndexOf(" {Type GroundTruth}") == 0)
            {
                //ParseGroundTruth(DeleteTag(SEN, " {Type GroundTruth}"), Convert.ToSingle(tmp));
            }
            else if (SEN.IndexOf(" {Type GPS}") == 0)
            {
                ParseGPS(DeleteTag(SEN, " {Type GPS}"), Convert.ToSingle(tmp));
            }
            // TO DO: Other types of SENSOR messages

        }

        #region GPS Sensor Message Parsing
        private void ParseGPS(string GPS, float TimeStamp)
        {
            string tmp;
            string msg;
            //GPS gps = new GPS();

            msg = "GPS ";
            GPSMsg.Body.gps.TimeStamp = TimeStamp;
            msg += GPSMsg.Body.gps.TimeStamp + " ";

            // Get the name of the sensor
            GPS = DeleteTag(GPS, "{Name");
            GPSMsg.Body.gps.Name = GetParameter(GPS, "}");
            GPS = DeleteTag(GPS, GPSMsg.Body.gps.Name);
            msg += GPSMsg.Body.gps.Name + " ";

            // Check to see if we have a fix
            if (GPS.IndexOf(" {Latitude") == 0)
            {

                // Get the latitude degrees, minutes, and cardinal direction
                GPS = DeleteTag(GPS, " {Latitude");
                for (int i = 0; i < 3; i++)
                {
                    if (i == 0 || i == 1)
                        tmp = GetParameter(GPS, ",");
                    else
                        tmp = GetParameter(GPS, "}");

                    if (i == 0)
                        GPSMsg.Body.gps.LatitudeDegrees = Convert.ToInt32(tmp);
                    else if (i == 1)
                        GPSMsg.Body.gps.LatitudeMinutes = Convert.ToSingle(tmp);
                    else
                        GPSMsg.Body.gps.LatitudeCardinal = Convert.ToChar(tmp);

                    GPS = DeleteTag(GPS, tmp);
                }

                msg += GPSMsg.Body.gps.LatitudeDegrees + " " + GPSMsg.Body.gps.LatitudeMinutes + " " + GPSMsg.Body.gps.LatitudeCardinal + " ";

                // Get the longitude degrees, minutes, and cardinal direction
                GPS = DeleteTag(GPS, " {Longitude");
                for (int i = 0; i < 3; i++)
                {
                    if (i == 0 || i == 1)
                        tmp = GetParameter(GPS, ",");
                    else
                        tmp = GetParameter(GPS, "}");

                    if (i == 0)
                        GPSMsg.Body.gps.LongitudeDegrees = Convert.ToInt32(tmp);
                    else if (i == 1)
                        GPSMsg.Body.gps.LongitudeMinutes = Convert.ToSingle(tmp);
                    else
                        GPSMsg.Body.gps.LongitudeCardinal = Convert.ToChar(tmp);

                    GPS = DeleteTag(GPS, tmp);
                }

                msg += GPSMsg.Body.gps.LongitudeDegrees + " " + GPSMsg.Body.gps.LongitudeMinutes + " " + GPSMsg.Body.gps.LongitudeCardinal + " ";

                // Get the fix for the GPS sensor
                GPS = DeleteTag(GPS, " {Fix");
                tmp = GetParameter(GPS, "}");
                GPSMsg.Body.gps.Fix = Convert.ToInt32(tmp);
                GPS = DeleteTag(GPS, tmp);

                msg += GPSMsg.Body.gps.Fix + " ";

                // Get the satellites tracked by the GPS sensor
                GPS = DeleteTag(GPS, " {Satellites");
                tmp = GetParameter(GPS, "}");
                GPSMsg.Body.gps.Satellites = Convert.ToInt32(tmp);
                GPS = DeleteTag(GPS, tmp);

                msg += GPSMsg.Body.gps.Satellites + " ";
            }
            else // If we do not have a fix
            {
                // Wrong Values for Latitude and Longitude
                GPSMsg.Body.gps.LatitudeDegrees = 99;
                msg += GPSMsg.Body.gps.LatitudeDegrees + " ";
                GPSMsg.Body.gps.LatitudeMinutes = 99;
                msg += GPSMsg.Body.gps.LatitudeMinutes + " ";
                GPSMsg.Body.gps.LatitudeCardinal = 'X';
                msg += GPSMsg.Body.gps.LatitudeCardinal + " ";
                GPSMsg.Body.gps.LongitudeDegrees = 199;
                msg += GPSMsg.Body.gps.LongitudeDegrees + " ";
                GPSMsg.Body.gps.LongitudeMinutes = 99;
                msg += GPSMsg.Body.gps.LongitudeMinutes + " ";
                GPSMsg.Body.gps.LongitudeCardinal = 'X';
                msg += GPSMsg.Body.gps.LongitudeCardinal + " ";

                // Get the fix for the GPS sensor
                GPS = DeleteTag(GPS, " {Fix");
                tmp = GetParameter(GPS, "}");
                GPSMsg.Body.gps.Fix = Convert.ToInt32(tmp);
                GPS = DeleteTag(GPS, tmp);

                msg += GPSMsg.Body.gps.Fix + " ";

                // Get the satellites tracked by the GPS sensor
                GPS = DeleteTag(GPS, " {Satellites");
                tmp = GetParameter(GPS, "}");
                GPSMsg.Body.gps.Satellites = Convert.ToInt32(tmp);
                GPS = DeleteTag(GPS, tmp);

                msg += GPSMsg.Body.gps.Satellites + " ";
            }

            // Post GPS Message to GUI
            wss.WriteLine("GUI", msg);
            //_mainPort.Post(GPSMsg);
        }
        #endregion
        #endregion

        #region CONF Message Parsing
        private void ParseCONF(string CONF)
        {
            string tmp;

            CONF = DeleteTag(CONF, "{Type");
            tmp = GetParameter(CONF, "}");

            if (tmp.CompareTo("MisPkg") == 0)
                ParseCONFMisPkg(DeleteTag(CONF, tmp));
            // TO DO: parse other CONF messages
        }

        #region Mission Package CONF Message Parsing
        private void ParseCONFMisPkg(string CONF)
        {
            string tmp;

            while (CONF.CompareTo("") != 0)
            {
                MisStatus aMisPkg = new MisStatus();

                // Get the name of the mission package
                CONF = DeleteTag(CONF, " {Name");
                aMisPkg.Name = GetParameter(CONF, "}");
                CONF = DeleteTag(CONF, aMisPkg.Name);

                // Get information about all the links
                while (CONF.IndexOf(" {Link", 0) == 0)
                {
                    LinkProperty aLink = new LinkProperty();
                    
                    // Get the ID of the joint
                    CONF = DeleteTag(CONF, " {Link");
                    tmp = GetParameter(CONF, "}");
                    aLink.ID = Convert.ToInt32(tmp);

                    // Get the joint type of the joint
                    CONF = DeleteTag(CONF, tmp + "} {JointType");
                    aLink.Type = GetParameter(CONF, "}");

                    // Get the maximum speed of the joint
                    CONF = DeleteTag(CONF, aLink.Type + "} {MaxSpeed");
                    tmp = GetParameter(CONF, "}");
                    aLink.MaxSpeed = Convert.ToSingle(tmp);

                    // Get the maximum torque of the joint
                    CONF = DeleteTag(CONF, tmp + "} {MaxTorque");
                    tmp = GetParameter(CONF, "}");
                    aLink.MaxTorque = Convert.ToSingle(tmp);

                    // Get the minimum range of the joint
                    CONF = DeleteTag(CONF, tmp + "} {MinRange");
                    tmp = GetParameter(CONF, "}");
                    aLink.MinRange = Convert.ToSingle(tmp);

                    // Get the maximum range of the joint
                    CONF = DeleteTag(CONF, tmp + "} {MaxRange");
                    tmp = GetParameter(CONF, "}");
                    aLink.MaxRange = Convert.ToSingle(tmp);
                    CONF = DeleteTag(CONF, tmp);

                    // Add the link into the array of links
                    aMisPkg.Link.Add(aLink);
                }

                MisPkgMsg.Body.misPkg.Add(aMisPkg);

                //System.Console.WriteLine(CONF);
            }
            gotMisPkgCONF = true;
        }
        #endregion
        #endregion

        #region MISSTA Message Parsing
        private void ParseMISSTA(string MISSTA)
        {
            string tmp;
            float timeStamp;
            int MPIndex, LinkIndex;

            // Get the time stamp of the mission package
            MISSTA = DeleteTag(MISSTA, "{Time");
            tmp = GetParameter(MISSTA, "}");
            timeStamp = Convert.ToSingle(tmp);
            MISSTA = DeleteTag(MISSTA, tmp);

            // Get the name of the mission package
            MISSTA = DeleteTag(MISSTA, " {Name");
            tmp = GetParameter(MISSTA, "}");
            MISSTA = DeleteTag(MISSTA, tmp);

            MPIndex = FindMisPkg(tmp);

            // Could not find the misson package (should never happen if a mission package exists)
            if (MPIndex == -1)
            {
                //System.Console.WriteLine("Could not find the mission package!");
                return;
            }

            // Update the time stamp
            MisPkgMsg.Body.misPkg[MPIndex].TimeStamp = timeStamp;

            for (int i = 0; i < MisPkgMsg.Body.misPkg[MPIndex].Link.Count; i++)
            {
                // Get the link number
                MISSTA = DeleteTag(MISSTA, " {Link");
                tmp = GetParameter(MISSTA, "}");
                LinkIndex = FindLinkID(Convert.ToInt32(tmp), MPIndex);
                MISSTA = DeleteTag(MISSTA, tmp);

                // Could not find the misson package (should never happen!)
                if (LinkIndex == -1)
                {
                    System.Console.WriteLine("Could not find the link inside the mission package!");
                    return;
                }

                // Get the current value
                MISSTA = DeleteTag(MISSTA, " {Value");
                tmp = GetParameter(MISSTA, "}");
                MisPkgMsg.Body.misPkg[MPIndex].Link[LinkIndex].CurrVal = Convert.ToSingle(tmp);
                MISSTA = DeleteTag(MISSTA, tmp);

                // Get the current torque
                MISSTA = DeleteTag(MISSTA, " {Torque");
                tmp = GetParameter(MISSTA, "}");
                MisPkgMsg.Body.misPkg[MPIndex].Link[LinkIndex].CurrTorque = Convert.ToSingle(tmp);
                MISSTA = DeleteTag(MISSTA, tmp);
            }

            // Post MIS STA message to GUI
            _mainPort.Post(MisPkgMsg);
        }

        #region Finding Specific Mission Packages and Their Links
        private int FindMisPkg(string name)
        {
            for (int i = 0; i < MisPkgMsg.Body.misPkg.Count; i++)
                if (MisPkgMsg.Body.misPkg[i].Name.CompareTo(name) == 0)
                    return i;

            return -1;
        }

        private int FindLinkID(int ID, int index)
        {
            for (int i = 0; i < MisPkgMsg.Body.misPkg[index].Link.Count; i++)
                if (MisPkgMsg.Body.misPkg[index].Link[i].ID == ID)
                    return i;

            return -1;
        }
        #endregion
        #endregion

        #region STA Message Parsing
        private void ParseSTA(string STA)
        {
            string tmp;
            string msg;

            // Read the type parameter
            STA = DeleteTag(STA, "{Type");
            tmp = GetParameter(STA, "}");

            if (tmp.CompareTo("AerialVehicle") == 0)
            {
                msg = "STA Aerial ";

                // Read the time parameter
                STA = DeleteTag(STA, tmp + "} {Time");
                tmp = GetParameter(STA, "}");
                msg += tmp + " ";

                // Read the light parameter
                STA = DeleteTag(STA, tmp + "} {LightToggle");
                tmp = GetParameter(STA, "}");
                msg += tmp + " ";

                // Read the light intensity parameter
                STA = DeleteTag(STA, tmp + "} {LightIntensity");
                tmp = GetParameter(STA, "}");
                msg += tmp + " ";

                // Read the battery parameter
                STA = DeleteTag(STA, tmp + "} {Battery");
                tmp = GetParameter(STA, "}");
                msg += tmp + " ";

                // Read the view parameter
                STA = DeleteTag(STA, tmp + "} {View");
                tmp = GetParameter(STA, "}");
                msg += tmp + " ";

                // Post STA Message to GUI
                wss.WriteLine("GUI", msg); 
            }
            // TO DO: Get different types of vehicles (the STA message will be different)

        }
        #endregion

        #region NFO Message Parsing
        private void ParseNFO(string NFO)
        {
            if (NFO.IndexOf("{StartPoses", 0) == 0)
            {
                if (!gotStartPoses)
                    ParseStartPoses(DeleteTag(NFO, "{StartPoses"));
            }
            else
            {
                // TO DO: Read the NFO {Gametype string} {Level string} {TimeLimit int} message
            }
        }

        #region Start Poses Message Parsing
        private void ParseStartPoses(string Poses)
        {
            // Get the number of start poses and discard that section of the message
            string tmp = GetParameter(Poses, "}");
            int numStartPoses = Convert.ToInt32(tmp);
            Poses = DeleteTag(Poses, tmp + "} ");

            // Get each starting pose
            for (int i = 0; i < numStartPoses; i++)
            {
                StartPoses aPose = new StartPoses();

                aPose.Name = Poses.Substring(0, Poses.IndexOf(' '));
                Poses = DeleteTag(Poses, aPose.Name);

                // Get the X Position of the starting pose
                tmp = GetParameter(Poses, ",");
                aPose.xPos = Convert.ToSingle(tmp);
                Poses = DeleteTag(Poses, tmp);

                // Get the Y Position of the starting pose
                tmp = GetParameter(Poses, ",");
                aPose.yPos = Convert.ToSingle(tmp);
                Poses = DeleteTag(Poses, tmp);

                // Get the Z Position of the starting pose
                tmp = GetParameter(Poses, " ");
                aPose.zPos = Convert.ToSingle(tmp);
                Poses = DeleteTag(Poses, tmp);

                // Get the X Rotation of the starting pose
                tmp = GetParameter(Poses, ",");
                aPose.xRot = Convert.ToSingle(tmp);
                Poses = DeleteTag(Poses, tmp);

                // Get the Y Rotation of the starting pose
                tmp = GetParameter(Poses, ",");
                aPose.yRot = Convert.ToSingle(tmp);
                Poses = DeleteTag(Poses, tmp);

                // Get the Z Rotation of the starting pose
                if (i == numStartPoses - 1)
                    tmp = GetParameter(Poses, "}");
                else
                    tmp = GetParameter(Poses, " ");
                aPose.zRot = Convert.ToSingle(tmp);
                Poses = DeleteTag(Poses, tmp);

                // Add the pose to the array of starting poses
                StartPosesMsg.Body.startPoses.Add(aPose);
            }

            gotStartPoses = true;

            // Post Start Poses Message to GUI
            _mainPort.Post(StartPosesMsg);
        }
        #endregion
        #endregion

        #region String Operations
        // Deletes the front tag PLUS 1 (the delimiter)
        private string DeleteTag(string toParse, string Tag)
        {
            return toParse.Substring(Tag.Length + 1, toParse.Length - Tag.Length - 1);
        }

        // Gets the parameter from the front of the string to the delimeter "}"
        private string GetParameter(string toParse, string delimiter)
        {
            return toParse.Substring(0, toParse.IndexOf(delimiter));
        }
        #endregion
    }
    #endregion

    #region GameBot Command Generator
    class GameBotCmd
    {
        #region Constructor
        public GameBotCmd()
        {
        }
        #endregion

        #region INIT Command
        public string INIT(string robot, string name, string loc, string rot)
        {
            string result = "INIT {ClassName ";

            // If no Robot was given, we automatically use the AirRobot
            if (name.CompareTo("") == 0)
                result = result + "USARBot.AirRobot} ";
            else
                result = result + "USARBot." + robot + "} ";

            // If a robot name was given
            if (name.CompareTo("") != 0)
                result = result + "{Name " + name + "} ";

            result = result + "{Location " + loc + "} ";
            result = result + "{Rotation " + rot + "}";

            return result;
        }
        #endregion

        #region GETCONF Command
        public string GETCONF(string type, string name)
        {
            string result = "GETCONF {Type ";
            result = result + type + "}";

            // If no Robot was given, we automatically use the AirRobot
            if (name.CompareTo("") != 0)
                result = result + "{Name " + name + "}";

            return result;
        }
        #endregion

        #region DRIVE Commands
        #region Aerial Drive Command
        public string DRIVEAerial(string altitudeVelocity, string linearVelocity, string lateralVelocity, string rotationalVelocity, bool Normalized)
        {
            string result = "DRIVE {AltitudeVelocity " + altitudeVelocity + "} " +
                                  "{LinearVelocity " + linearVelocity + "} " +
                                  "{LateralVelocity " + lateralVelocity + "} " +
                                  "{RotationalVelocity " + rotationalVelocity + "}";
            if (Normalized)
                result = result + " {Normalized 1}";

            return result;
        }
        #endregion
        #endregion

        #region MISPKG Command
        public string MISPKG(string Name, List<LinkProperty>Link)
        {
            string result = "MISPKG {Name " + Name + "}";

            for (int i = 0; i < Link.Count; i++)
            {
                result = result + " {Link " + Link[i].ID + "}" +
                                  " {Value " + Link[i].DesiredVal + "}" +
                                  " {Order " + Link[i].DesiredOrder + "}";

            }

            return result;
        }
        #endregion

        #region SET {Type Camera} Command
        public string SET(string Robot, string Cam, string ip)
        {
            string result = "SET {Type Camera}";

            if (Robot.CompareTo("") != 0)
                result = result + " {Robot " + Robot + "}";
            if (Cam.CompareTo("") != 0)
                result = result + " {Name " + Cam + "}";

            result = result + " {Client " + ip + "}";

            return result;
        }
        #endregion
    }
    #endregion
}
