//-----------------------------------------------------------------------
//  This file is part of the UC Merced Microsoft Robotics Developer Studio 
// Project Code Base.
//
//  Created by Benjamin Balaguer under the supervision of Stefano Carpin.
//  Based on DriveControl.cs from UX\SimpleDashboard.
//
//  $File: DriveControl.cs $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Ccr.Core;
using game = Microsoft.Robotics.Services.GameController.Proxy;
using cs = Microsoft.Dss.Services.Constructor;
using Microsoft.Dss.ServiceModel.Dssp;
using System.Drawing.Drawing2D;
using System.IO;
using Microsoft.Dss.Core;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Media.Imaging;
using comStationController = Robotics.ComStation2009Controller.Proxy;
using System.Xml;
using System.Configuration;
using System.Net.Configuration;
using System.Reflection;
using Microsoft.Dss.Core.Attributes;
using System.Globalization;

namespace Microsoft.Robotics.Services.RoboCup2009GUI
{
    partial class DriveControl : Form
    {
        DriveControlEvents _eventsPort;
        RoboCup2009GUIOperations _mainPort;
        RoboCup2009GUIState _state = new RoboCup2009GUIState();

        // The robot we are currently controlling
        private int currentRobotNumber = 0; 

        // Battery-Related Variables
        private bool warning1 = false;
        private bool warning2 = false;
        private bool warning3 = false;

        private float oldCoverage = 0;

        // Map Variables
        private List<RobotMapInfo> RobotMap = new List<RobotMapInfo>();
        private int mapNum = 1;
        private bool saveMap = false;
        private System.Object saveMapLock = new System.Object();
        private float commStationX;
        private float commStationY;

        // Configuration Variables
        private float startPoseX;
        private float startPoseY;
        private string USARSimIP;
        private int USARSimPort;
        private int wssPort;
        private string ComStationName;
        private float WSSCutOff;
        private Color clearedColor;

        private string lastCommandSent = "";

        // AirRobot Controller Message Data Structure
        public List<comStationController.StartPoses> startPoses;
        private System.Object startPoseLock = new System.Object();
        public BotStatus botStatus;
        private System.Object botStatusLock = new System.Object();
        public List<comStationController.MisStatus> misStatus;
        private System.Object misStatusLock = new System.Object();
        //public airRobotController.GroundTruth groundTruth;
        private System.Object groundTruthLock = new System.Object();
        public GPS gps;
        private System.Object gpsLock = new System.Object();
        public GPS iniGPS = new GPS();
        public Image jpegImage;
        private System.Object jpegImageBufferLock = new System.Object();
        private System.Object mapLock = new System.Object();

        // Store Victim Information
        private List<VictimLoc> Victims = new List<VictimLoc>();
        private StreamWriter fVictims;// = new StreamWriter("RoboCup/Victims.txt");

        // Logging information
        private List<StreamWriter> fPath = new List<StreamWriter>();
        private List<int> numPoints = new List<int>();
        private StreamWriter fIniPose;
        private StreamWriter fMIFGrouping;
        private StreamWriter fMIDGrouping;
        private StreamWriter fMIFPathsToVictims;
        private StreamWriter fMIDPathsToVictims;
        private StreamWriter fMIFMeshNetwork;
        private StreamWriter fMIDMeshNetwork;

        // Store Points for Grouping
        private List<Point> GroupingPoints = new List<Point>();
        private int GroupedObjects = 0;

        // WSS Variables
        public WSS wss;
        private List<RobotsInformation> RobotInfo = new List<RobotsInformation>();
        private System.Object robotInfoLock = new System.Object();
        private int ownWSSPort;
        private string ownIP;
        private DataTable ssTable = new DataTable();
        private DataTable batteryTable = new DataTable();
        //private DataTable mapTable = new DataTable();

        // To Center the Map
        private bool mapGotInitialPosition = false;
        private float mapXPosInitial;
        private float mapYPosInitial;
        private double mapResolution;

        private VictimForm vForm;

        public DriveControl(DriveControlEvents EventsPort, RoboCup2009GUIOperations MainPort)
        {
            DriveControl.CheckForIllegalCrossThreadCalls = false;
            _eventsPort = EventsPort; // Initialize the events port
            _mainPort = MainPort;     // Initialize the main port

            // Get own IP address
            IPAddress[] aryLocalAddr = null;
            String strHostName = "";
            strHostName = Dns.GetHostName();
            IPHostEntry ipEntry = Dns.GetHostEntry(strHostName);
            aryLocalAddr = ipEntry.AddressList;
            ownIP = Convert.ToString(aryLocalAddr[0]);

            InitializeComponent();


            // Read Configuration File
            ReadConfigFile("samples/Config/RoboCup2009GUI.xml");
        }

        public void CloseForm()
        {
            //mapXPosInitial = 1.892435f;
            //mapYPosInitial = -8.270072f;
            //mapResolution = 0.1f;

            #region Variable Declaration
            string line;
            float XPos;
            float YPos;
            int row;
            int col;
            SolidBrush myBrush = new SolidBrush(Color.AliceBlue);
            StreamWriter Writer;
            StreamReader Reader;
            float MinX = (float)((1 - 500) * mapResolution) + mapYPosInitial;
            float MinY = (float)((1 - 500) * mapResolution) + mapXPosInitial;
            float MaxX = (float)((1001 - 500) * mapResolution) + mapYPosInitial;
            float MaxY = (float)((1001 - 500) * mapResolution) + mapXPosInitial;
            #endregion
            
            #region Close all the Files
            fVictims.WriteLine("END");
            fVictims.Close();
            fMIFGrouping.Close();
            fMIDGrouping.Close();
            fMIFPathsToVictims.Close();
            fMIDPathsToVictims.Close();
            for (int i = 0; i < fPath.Count; i++)
                fPath[i].Close();
            #endregion
            
            #region Generate the robots' path
            StreamWriter allRobotsPathsMIF = new StreamWriter("RoboCup/AllRobotsPath.mif");
            StreamWriter allRobotsPathsMID = new StreamWriter("RoboCup/AllRobotsPath.mid");
           
            #region MIF Header
            allRobotsPathsMIF.WriteLine("Version 450");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("Charset \"Neutral\"");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("Delimiter \",\"");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("CoordSys NonEarth Units \"m\" Bounds (" + MinX + ", " + MinY + ") (" + MaxX + ", " + MaxY + ")");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("Columns 5");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("  Robot_Name char(254)");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("  Robot_Type char(254)");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("  Distance_Covered char(254)");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("  Number_Of_Victims_Found integer");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("  Victims_Found char(254)");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine("Data");
            allRobotsPathsMIF.Flush();
            allRobotsPathsMIF.WriteLine();
            allRobotsPathsMIF.Flush();
            #endregion

            string[] lines;
            for(int i=0; i<RobotInfo.Count; i++)
            {
                lines = File.ReadAllLines("RoboCup/" + RobotInfo[i].Name + "Path.txt");

                #region MIF Headers
                allRobotsPathsMIF.WriteLine("PLINE");
                allRobotsPathsMIF.Flush();
                allRobotsPathsMIF.WriteLine("  " + lines.Length);
                allRobotsPathsMIF.Flush();
                Writer = new StreamWriter("RoboCup/" + RobotInfo[i].Name + "Path.mif");
                Writer.WriteLine("Version 450");
                Writer.Flush();
                Writer.WriteLine("Charset \"Neutral\"");
                Writer.Flush();
                Writer.WriteLine("Delimiter \",\"");
                Writer.Flush();
                Writer.WriteLine("CoordSys NonEarth Units \"m\" Bounds (" + MinX + ", " + MinY + ") (" + MaxX + ", " + MaxY + ")");
                Writer.Flush();
                Writer.WriteLine("Columns 5");
                Writer.Flush();
                Writer.WriteLine("  Robot_Name char(254)");
                Writer.Flush();
                Writer.WriteLine("  Robot_Type char(254)");
                Writer.Flush();
                Writer.WriteLine("  Distance_Covered char(254)");
                Writer.Flush();
                Writer.WriteLine("  Number_Of_Victims_Found integer");
                Writer.Flush();
                Writer.WriteLine("  Victims_Found char(254)");
                Writer.Flush();
                Writer.WriteLine("Data");
                Writer.Flush();
                Writer.WriteLine();
                Writer.Flush();
                Writer.WriteLine("PLINE");
                Writer.Flush();
                Writer.WriteLine("  " + lines.Length);
                Writer.Flush();
                #endregion

                float DistanceCovered = 0;
                bool gotFirstData = false;
                float oldXPos = 0;
                float oldYPose= 0;
                System.Console.WriteLine("Now Generating " + RobotInfo[i].Name + " path...");

                Reader = new StreamReader("RoboCup/" + RobotInfo[i].Name + "Path.txt");
                myBrush.Color = RobotInfo[i].Color;
                while (!Reader.EndOfStream)
                {
                    
                    #region Path Generation
                    line = Reader.ReadLine();
                    XPos = Convert.ToSingle(line.Substring(0, line.IndexOf(",")));
                    YPos = Convert.ToSingle(line.Substring(line.IndexOf(",") + 1, line.Length - line.IndexOf(",") - 1));

                    allRobotsPathsMIF.WriteLine(YPos + " " + XPos);
                    allRobotsPathsMIF.Flush();
                    Writer.WriteLine(YPos + " " + XPos);
                    Writer.Flush();
                    #endregion

                    // Calculate Distance Covered
                    if (!gotFirstData)
                    {
                        gotFirstData = true;
                    }
                    else
                    {
                        DistanceCovered += (float)Math.Sqrt(Math.Pow(XPos - oldXPos, 2) + Math.Pow(YPos - oldYPose, 2));
                    }
                    oldXPos = XPos;
                    oldYPose = YPos;
                }
                #region Color Setting
                int color = RobotInfo[i].Color.R * 65536 + RobotInfo[i].Color.G * 256 + RobotInfo[i].Color.B;
                Writer.WriteLine("PEN 3 65 " + color);
                Writer.Flush();
                Writer.WriteLine("SMOOTH");
                Writer.Flush();
                Writer.Close();
                allRobotsPathsMIF.WriteLine("PEN 3 65 " + color);
                allRobotsPathsMIF.Flush();
                allRobotsPathsMIF.WriteLine("SMOOTH");
                allRobotsPathsMIF.Flush();
                allRobotsPathsMIF.WriteLine();
                allRobotsPathsMIF.Flush();
                #endregion

                #region Attribute Generation
                Writer = new StreamWriter("RoboCup/" + RobotInfo[i].Name + "Path.mid");
                Writer.Write("\"" + RobotInfo[i].Name + "\",\"" + RobotInfo[i].Type + "\",\"" + DistanceCovered + " meters\"," + RobotInfo[i].Victims.Count + ",\"");
                allRobotsPathsMID.Write("\"" + RobotInfo[i].Name + "\",\"" + RobotInfo[i].Type + "\",\"" + DistanceCovered + " meters\"," + RobotInfo[i].Victims.Count + ",\"");
                for (int k = 0; k < RobotInfo[i].Victims.Count; k++)
                {
                    Writer.Write("Victim" + RobotInfo[i].Victims[k]);
                    allRobotsPathsMID.Write("Victim" + RobotInfo[i].Victims[k]);
                    if ((k + 1) != RobotInfo[i].Victims.Count)
                    {
                        Writer.Write(", ");
                        allRobotsPathsMID.Write(", ");
                    }
                }
                Writer.Write("\"\n");
                allRobotsPathsMID.Write("\"\n");
                Writer.Flush();
                allRobotsPathsMID.Flush();
                Writer.Close();
                #endregion

                System.Console.WriteLine("Done Generating " + RobotInfo[i].Name + " path...");
                lines = null;
            }

            allRobotsPathsMID.Close();
            allRobotsPathsMIF.Close();

            #endregion

            #region Generate the victims' images
            System.Console.Write("Generating TIFF image for Victims: ");

            Reader = new StreamReader("RoboCup/Victim_Locations.txt");
            for (int i = 1; !Reader.EndOfStream; i++)
            {
                line = Reader.ReadLine();

                if (line.CompareTo("END") != 0)
                {
                    System.Console.Write("#" + i);
                    Bitmap bmpPath = new Bitmap(1001, 1001);
                    Graphics gPath = Graphics.FromImage(bmpPath);
                    gPath.Clear(Color.Transparent);
                    Image image = Image.FromFile("RoboCup/Victim" + i + ".jpeg");
                    image.RotateFlip(RotateFlipType.Rotate90FlipNone);
                    gPath.Clear(Color.Transparent);

                    line = line.Remove(0, line.IndexOf(", ") + 2); // Remove Victim ID
                    XPos = Convert.ToSingle(line.Substring(0, line.IndexOf(",")));
                    line = line.Remove(0, line.IndexOf(", ") + 2); // Remove XPos
                    YPos = Convert.ToSingle(line.Substring(0, line.IndexOf(",")));
                    row = (int)Math.Round((XPos - mapXPosInitial) / mapResolution) + 500;
                    col = (int)Math.Round((YPos - mapYPosInitial) / mapResolution) + 500;

                    gPath.DrawImage(image, row - 60, col, 60, 80);

                    image = new Bitmap(bmpPath, bmpPath.Width, bmpPath.Height);
                    image.RotateFlip(RotateFlipType.Rotate270FlipNone);
                    image.Save("RoboCup/Victim" + i + ".tif", System.Drawing.Imaging.ImageFormat.Tiff);

                    // Write TWF file for GroundRobot Path
                    Writer = new StreamWriter("RoboCup/Victim" + i + ".tfw");
                    Writer.WriteLine(mapResolution);
                    Writer.WriteLine("0");
                    Writer.WriteLine("0");
                    Writer.WriteLine(-mapResolution);
                    Writer.WriteLine(Convert.ToString((float)((1 - 500) * mapResolution) + mapYPosInitial));
                    Writer.WriteLine(Convert.ToString((float)((1001 - 500) * mapResolution) + mapXPosInitial));
                    Writer.Flush();
                    Writer.Close();

                    // Deallocate memory
                    bmpPath = null;
                    gPath = null;
                    image = null;
                }
            }
            System.Console.Write("\n");
            System.Console.WriteLine("Done Generating TIFF image for Victims");
            #endregion
        }

        private void ReadConfigFile(string FileToRead)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(FileToRead);

            USARSimIP = ((XmlElement)doc.GetElementsByTagName("USARSimIP")[0]).InnerText;
            USARSimPort = Convert.ToInt32(((XmlElement)doc.GetElementsByTagName("USARSimPort")[0]).InnerText);
            //ImgSrvPort = Convert.ToInt32(((XmlElement)doc.GetElementsByTagName("ImgSrvPort")[0]).InnerText);
            wssPort = Convert.ToInt32(((XmlElement)doc.GetElementsByTagName("WSSPort")[0]).InnerText);
            ownWSSPort = Convert.ToInt32(((XmlElement)doc.GetElementsByTagName("GUIWSSPort")[0]).InnerText);
            //MOASTIP = ((XmlElement)doc.GetElementsByTagName("MOASTIP")[0]).InnerText;
            //MOASTPort = Convert.ToInt32(((XmlElement)doc.GetElementsByTagName("MOASTPort")[0]).InnerText);
            //AerialRobotName = ((XmlElement)doc.GetElementsByTagName("AerialRobotName")[0]).InnerText;
            //GroundRobotName = ((XmlElement)doc.GetElementsByTagName("GroundRobotName")[0]).InnerText;
            ComStationName = ((XmlElement)doc.GetElementsByTagName("ComStationName")[0]).InnerText;
            WSSCutOff = Convert.ToSingle(((XmlElement)doc.GetElementsByTagName("WSSCutOff")[0]).InnerText);

            string mapColor = ((XmlElement)doc.GetElementsByTagName("ClearedColor")[0]).InnerText;
            if (mapColor.CompareTo("White") == 0)
                clearedColor = Color.FromArgb(255, 255, 255);
            else
                clearedColor = Color.FromArgb(0, 255, 0);

            // Read the robots' names and set up our table of signal strenghts
            ssTable.Columns.Add(" ", typeof(string));
            ssTable.Columns.Add("CS", typeof(float));
            ssTable.Rows.Add("CS");
            //batteryTable.Columns.Add(" ", typeof(string));
            batteryTable.Columns.Add("Battery Life", typeof(string));
            batteryTable.Rows.Add("-1");

            mapGrid.Columns.Add("name", "test");

            for (int i = 1; i <= Convert.ToInt16(((XmlElement)doc.GetElementsByTagName("RobotNumber")[0]).InnerText); i++)
            {
                // Set Robot Information
                RobotsInformation someInfo = new RobotsInformation();
                someInfo.Name = ((XmlElement)doc.GetElementsByTagName("Name" + i)[0]).InnerText;
                someInfo.Type = "Unknown";
                someInfo.BatteryLife = -1;
                // Get robot's color
                if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Beige") == 0)
                    someInfo.Color = Color.Beige;
                else if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Green") == 0)
                    someInfo.Color = Color.DarkGreen;
                else if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Cyan") == 0)
                    someInfo.Color = Color.Cyan;
                else if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Yellow") == 0)
                    someInfo.Color = Color.Yellow;
                else if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Orange") == 0)
                    someInfo.Color = Color.DarkOrange;
                else if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Purple") == 0)
                    someInfo.Color = Color.Purple;
                else if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Pink") == 0)
                    someInfo.Color = Color.Pink;
                else if ((((XmlElement)doc.GetElementsByTagName("Color" + i)[0]).InnerText).CompareTo("Gray") == 0)
                    someInfo.Color = Color.DarkGray;


                RobotInfo.Insert(i - 1, someInfo);

                // Set Robot Map Information
                RobotMap.Insert(i - 1, new RobotMapInfo());

                // Set Map Grid
                mapGrid.Rows.Add();
                mapGrid.Rows[i - 1].Cells[1].Value = RobotInfo[i - 1].Name;
                mapGrid.Rows[i - 1].Cells[1].ReadOnly = true;
                mapGrid.Rows[i - 1].Visible = false;
                
                // Set Signal strength table
                ssTable.Columns.Add(("R" + i), typeof(float));
                ssTable.Rows.Add(("R" + i));
            }

            ssGrid.DataSource = ssTable;
            for (int i = 0; i < ssGrid.Columns.Count; i++)
            {
                ssGrid.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            }
            ssGrid.Columns[0].DefaultCellStyle = ssGrid.ColumnHeadersDefaultCellStyle;

            batteryGrid.DataSource = batteryTable;
            //for (int i = 0; i < batteryGrid.Columns.Count; i++)
            //{
            //    batteryGrid.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            //}
            //batteryGrid.Columns[0].DefaultCellStyle = ssGrid.ColumnHeadersDefaultCellStyle;

            // Initialize WSS
            wss = new WSS(_mainPort, RobotInfo, ownIP, ownWSSPort, WSSCutOff);
        }

        public void ProcessConnectedMSG()
        {
            // Send a GETSTARTPOSES Command Notification to the AirRobot Controller
            _mainPort.Post(new GET_POSES_CMD());
        }

        public void ProcessInitializedMSG()
        {
            // Update the GUI
            txtLocStation.Enabled = false;
            txtRotStation.Enabled = false;
            btnSpawn.Enabled = false;
            cBoxStartPosesStation.BeginUpdate();
            cBoxStartPosesStation.Items.Clear();
            cBoxStartPosesStation.EndUpdate();
            cBoxStartPosesStation.Enabled = false;
            btnWssConnect.Enabled = true;
        }

        public void ProcessStartPosesMSG(List<comStationController.StartPoses> sPoses)
        {
            lock (startPoseLock)
            {
                startPoses = sPoses;

                for (int i = 0; i < startPoses.Count; i++)
                {
                    cBoxStartPosesStation.BeginUpdate();
                    chkListPositions.BeginUpdate();

                    // Make sure that another thread is not modifying the start poses
                    lock (startPoseLock)
                    {
                        cBoxStartPosesStation.Items.Add((i + 1) + ": " + startPoses[i].Name);

                        if (startPoses[i].Name.CompareTo("CommStation")==0)
                        {
                            commStationX = startPoses[i].xPos;
                            commStationY = startPoses[i].yPos;
                        }

                        chkListPositions.Items.Add((i + 1) + ": " + startPoses[i].Name + " (Z = " + (Convert.ToDecimal(startPoses[i].zPos)).ToString("F2", CultureInfo.InvariantCulture) + ")");
                        chkListPositions.SetItemChecked(i, true);
                    }

                    cBoxStartPosesStation.SelectedIndex = 0;

                    cBoxStartPosesStation.EndUpdate();
                    chkListPositions.EndUpdate();
                }

                cBoxStartPosesStation.Enabled = true;

                // Enable the disconnect option
                btnConnect.Text = "Disconnect";
                btnConnect.Enabled = true;

                // Enable the spawn options
                //txtLoc.Enabled = true;
                //txtRot.Enabled = true;
                txtLocStation.Enabled = true;
                txtRotStation.Enabled = true;
                btnSpawn.Enabled = true;
                //cBoxComStation.Enabled = true;
                //cBoxRobot.Enabled = true;
            }
        }

        public void ProcessImageMSG(byte [] ImgBuffer)
        {
            try
            {
                lock(jpegImageBufferLock)
                {
                    Stream imageStream = new MemoryStream(ImgBuffer, 0, ImgBuffer.Length);
                    jpegImage = new Bitmap(imageStream);

                    // Show the image on the screen
                    pBoxWebcam.Image = jpegImage;

                    if (mapGotInitialPosition && vForm.Visible)
                    {
                        vForm.UpdateImage(imageStream);
                    }

                    // Send a new IMR command to the current robot
                    string cmd = "IMR";
                    ASCIIEncoding encoder = new ASCIIEncoding();
                    byte[] buffer = encoder.GetBytes(cmd);
                    wss.WriteLine(currentRobotNumber, buffer);

                    // Remove allocated emmory
                    imageStream.Dispose();
                    //jpegImage = null;
                    imageStream = null;
                    ImgBuffer = null;
                    buffer = null;
                    encoder = null;
                }
            }
            catch (Exception e)
            {
                System.Console.WriteLine("Caught an exception while processing the image msg:" + e.ToString());
                
                // Send a new IMR command to the current robot
                string cmd = "IMR";
                ASCIIEncoding encoder = new ASCIIEncoding();
                byte[] buffer = encoder.GetBytes(cmd);
                wss.WriteLine(currentRobotNumber, buffer);

                // Remove allocated emmory
                buffer = null;
                encoder = null;
            }
        }

        public void ProcessRTypeMSG(int RobotNumber, string RobotType)
        {
            lock (robotInfoLock)
            {
                RobotInfo[RobotNumber - 1].Type = RobotType;

                bool doneWithAllRobots = true;
                for (int i = 0; (doneWithAllRobots) && (i < RobotInfo.Count); i++)
                    if (RobotInfo[i].Type == "Unknown")
                        doneWithAllRobots = false;

                if (doneWithAllRobots)
                {
                    cBoxRobots.BeginUpdate();
                    for (int i = 0; i < RobotInfo.Count; i++)
                    {
                        cBoxRobots.Items.Add((i + 1) + ": " + RobotInfo[i].Name + " (" + RobotInfo[i].Type + ")");
                    }

                    cBoxRobots.EndUpdate();
                    cBoxRobots.Enabled = true;
                    cBoxRobots.SelectedIndex = 0;

                    //System.Console.WriteLine("Starting ssTimer");

                    // Enable the GETSS timer
                    //ssTimer.Enabled = true;

                    // Request First Image
                    currentRobotNumber = 1;
                    string cmd = "IMR";
                    ASCIIEncoding encoder = new ASCIIEncoding();
                    byte[] buffer = encoder.GetBytes(cmd);
                    wss.WriteLine(currentRobotNumber, buffer);

                    // Deallocate memory
                    encoder = null;
                    buffer = null;
                    //System.GC.Collect();
                }
            }
        }

        public void ProcessRssMSG(int RobotNumber, float [] rssValues, int batteryLife)
        {
            //System.Console.WriteLine("Processing RSS from Robot #" + RobotNumber);
            
            int offset = 0;            
            for (int i = 0; i < rssValues.GetLength(0)+1; i++)
            {   
                if (i == RobotNumber)
                {
                    if (!ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor.Equals(Color.Black))
                    {
                        ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor = Color.Black;
                    }
                    offset = 1;
                }
                else
                {
                    ssGrid.Rows[RobotNumber].Cells[i + 1].Value = rssValues[i-offset];

                    bool isDropped;
                    lock (wss.droppedRobotsLock)
                    {
                        isDropped = wss.DroppedRobots.Contains(i);
                    }
                    if (isDropped)
                    {
                        if (!ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor.Equals(Color.SlateGray))
                        {
                            ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor = Color.SlateGray;
                        }
                    }
                    else if ((float)(ssGrid.Rows[RobotNumber].Cells[i + 1].Value) < (WSSCutOff * 0.95))
                    {
                        if (!ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor.Equals(Color.Red))
                        {
                            ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor = Color.Red;
                        }
                    }
                    else if ((float)(ssGrid.Rows[RobotNumber].Cells[i + 1].Value) < (WSSCutOff * 0.85))
                    {
                        if (!ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor.Equals(Color.Orange))
                        {
                            ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor = Color.Orange;
                        }
                    }
                    else if ((float)(ssGrid.Rows[RobotNumber].Cells[i + 1].Value) < (WSSCutOff * 0.75))
                    {
                        if (!ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor.Equals(Color.Yellow))
                        {
                            ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor = Color.Yellow;
                        }
                    }
                    else
                    {
                        if (!ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor.Equals(Color.LightGreen))
                        {
                            ssGrid.Rows[RobotNumber].Cells[i + 1].Style.BackColor = Color.LightGreen;
                        }
                    }
                }
            }

            //System.Console.WriteLine("Battery Life (from RSS): " + batteryLife);
            RobotInfo[RobotNumber - 1].BatteryLife = batteryLife;
            //lBoxBattery.BeginUpdate();
            //lBoxBattery.Items[RobotNumber - 1] = Convert.ToString(RobotInfo[RobotNumber - 1].BatteryLife);
            //lBoxBattery.EndUpdate();
        }

        public void ProcessMapMSG(int RobotNumber, List<MOASTMapInfo> mapInfo, double xLow, double yLow, double resolution, float xPos, float yPos)
        {
            try
            {
                // Get the point in the map that will equal 500,500
                if (!mapGotInitialPosition && resolution>0)
                {
                    //System.Console.WriteLine("Initializing the position for the first time: " +xPos+","+yPos+","+resolution);
                    mapXPosInitial = xPos;
                    mapYPosInitial = yPos;
                    mapResolution = resolution;

                    // Write initial position into a file
                    fIniPose.WriteLine(mapXPosInitial + "," + mapYPosInitial + "," + mapResolution);
                    fIniPose.Close();

                    #region Write MIF file headers
                    float MinX = (float)((1 - 500) * mapResolution) + mapYPosInitial;
                    float MinY = (float)((1 - 500) * mapResolution) + mapXPosInitial;
                    float MaxX = (float)((1001 - 500) * mapResolution) + mapYPosInitial;
                    float MaxY = (float)((1001 - 500) * mapResolution) + mapXPosInitial;
                    fMIFGrouping.WriteLine("Version 450");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine("Charset \"Neutral\"");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine("Delimiter \",\"");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine("CoordSys NonEarth Units \"m\" Bounds (" + MinX + ", " + MinY + ") (" + MaxX + ", " + MaxY + ")");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine("Columns 2");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine("  ID integer");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine("  Feature_Type char(254)");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine("Data");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine();
                    fMIFGrouping.Flush();

                    fMIFPathsToVictims.WriteLine("Version 450");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("Charset \"Neutral\"");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("Delimiter \",\"");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("CoordSys NonEarth Units \"m\" Bounds (" + MinX + ", " + MinY + ") (" + MaxX + ", " + MaxY + ")");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("Columns 9");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_ID char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_Sex char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_State char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_Condition char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_Pose char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_Accessibility char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_Priority char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Victim_Features char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("  Path_Distance char(254)");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine("Data");
                    fMIFPathsToVictims.Flush();
                    fMIFPathsToVictims.WriteLine();
                    fMIFPathsToVictims.Flush();
                    #endregion

                    btnMap.Enabled = true;

                    vForm = new VictimForm(mapXPosInitial, mapYPosInitial, mapResolution);

                    mapGotInitialPosition = true;
                }

                // Update Robot Position and Map
                RobotMap[RobotNumber - 1].xPos = xPos;
                RobotMap[RobotNumber - 1].yPos = yPos;

                // Save the path to the files
                fPath[RobotNumber - 1].WriteLine(xPos + "," + yPos);
                fPath[RobotNumber - 1].Flush();
                numPoints[RobotNumber - 1] = numPoints[RobotNumber - 1] + 1;

                if (!RobotMap[RobotNumber - 1].Initialized)
                    RobotMap[RobotNumber - 1].Initialized = true;

                if (!chkListPositions.Enabled)
                    chkListPositions.Enabled = true;

                if (!chkBoxNetwork.Enabled)
                    chkBoxNetwork.Enabled = true;

                for (int i = 0; i < mapGrid.Rows.Count; i++)
                {
                    if (mapGrid.Rows[i].Cells[1].Value.ToString().CompareTo(RobotInfo[RobotNumber - 1].Name)==0)
                    {
                        if (!mapGrid.Rows[i].Visible)
                        {
                            mapGrid.Rows[i].Visible = true;
                            mapGrid.Rows[i].DefaultCellStyle.BackColor = RobotInfo[RobotNumber - 1].Color;
                            mapGrid.Rows[i].DefaultCellStyle.SelectionBackColor = RobotInfo[RobotNumber - 1].Color;

                            if (RobotInfo[RobotNumber - 1].Type == "AirRobot")
                            {
                                mapGrid.Rows[i].Cells[0].ReadOnly = true;
                                mapGrid.Rows[i].Cells[0].Value = false;
                            }
                            else
                            {
                                mapGrid.Rows[i].Cells[0].Value = true;
                            }
                        }
                        break;
                    }
                }

                if (mapInfo.Count > 0 && mapGotInitialPosition)
                {
                    RobotMap[RobotNumber - 1].UpdateMap(mapInfo, mapXPosInitial, mapYPosInitial, mapResolution, clearedColor);

                    //lock (mapLock)
                    //{
                        //ShowMap();
                    //}
                }

                // Deallocate memory
                mapInfo = null;
                //System.GC.Collect();
            }
            catch (Exception E)
            {
                System.Console.WriteLine("Caught Exception inside map update Timer: " + E.ToString());
            }
        }

        public void ShowMap()
        {
            int RobotNumber = -1;
            int row, col;
            bool undefined = true;
            SolidBrush myBrush = new SolidBrush(Color.FromArgb(0, 0, 0));
            Image finalImage = null;
            Graphics CombinedMapGraphics = null;

            #region Go through the list of maps and find the ones we want to display
            for (int i = mapGrid.Rows.Count - 1; i >= 0; i--)
            {
                // If we should display that map
                if (Convert.ToBoolean(mapGrid.Rows[i].Cells[0].Value))
                {
                    // Find the robot number, whose map we have to show
                    for (int j = 0; j < RobotInfo.Count; j++)
                    {
                        if (mapGrid.Rows[i].Cells[1].Value.ToString().CompareTo(RobotInfo[j].Name) == 0)
                        {
                            RobotNumber = j + 1;
                            break;
                        }
                    }

                    if (undefined)
                    {
                        finalImage = new Bitmap(RobotMap[RobotNumber - 1].BitmapMap.Width, RobotMap[RobotNumber - 1].BitmapMap.Height);
                        CombinedMapGraphics = Graphics.FromImage(finalImage);
                        CombinedMapGraphics.Clear(Color.FromArgb(0, 0, 255));
                        undefined = false;
                    }

                    //CombinedMapGraphics.DrawImageUnscaledAndClipped(RobotMap[RobotNumber - 1].BitmapMap, new Rectangle(500, 500, 1000, 1000));
                    //CombinedMapGraphics.DrawImageUnscaled(RobotMap[RobotNumber - 1].BitmapMap, 0, 0);

                    //CombinedMapGraphics.DrawImage(RobotMap[RobotNumber - 1].BitmapMap,,,,,

                    //CombinedMapGraphics.DrawImage(RobotMap[RobotNumber - 1].BitmapMap, new Rectangle(450, 450, 100, 100), 450, 450, 100, 100, GraphicsUnit.Pixel); 
                    
                    int xLow = RobotMap[RobotNumber - 1].MapRegionXlow;
                    int yLow = RobotMap[RobotNumber - 1].MapRegionYlow;
                    int width = RobotMap[RobotNumber - 1].MapRegionXhigh-RobotMap[RobotNumber - 1].MapRegionXlow+1;
                    int height = RobotMap[RobotNumber - 1].MapRegionYhigh-RobotMap[RobotNumber - 1].MapRegionYlow+1;

                    CombinedMapGraphics.DrawImage(RobotMap[RobotNumber - 1].BitmapMap, new Rectangle(xLow, yLow, width, height), xLow, yLow, width, height, GraphicsUnit.Pixel);
                    //Add this robot's map into our final map
                    //CombinedMapGraphics.DrawImage(RobotMap[RobotNumber - 1].BitmapMap, new Rectangle(0, 0, RobotMap[RobotNumber - 1].BitmapMap.Width, RobotMap[RobotNumber - 1].BitmapMap.Height));
                }
            }
            #endregion

            #region Put the Victims on the Map
            myBrush.Color = Color.FromArgb(255, 0, 0);
            for (int i = 0; i < Victims.Count; i++)
            {
                CombinedMapGraphics.FillRectangle(myBrush, Victims[i].Row - 1, Victims[i].Col - 1, 3, 3);
            }
            #endregion

            #region Save map
            bool saveFiles = false;
            lock (saveMapLock)
            {
                saveFiles = saveMap;
                saveMap = false;
            }

            if (saveFiles)
            {
                mapNum++;
                StreamWriter WorldFile = new StreamWriter("RoboCup/Map" + mapNum + ".tfw");

                // Make sure map is not zoomed
                Image map = new Bitmap(finalImage, finalImage.Width, finalImage.Height);
                map.RotateFlip(RotateFlipType.Rotate270FlipNone);

                // Generate tif file
                map.Save("RoboCup/Map" + mapNum + ".tif", System.Drawing.Imaging.ImageFormat.Tiff);

                // Write World File
                WorldFile.WriteLine(mapResolution);
                WorldFile.WriteLine("0");
                WorldFile.WriteLine("0");
                WorldFile.WriteLine(-mapResolution);
                WorldFile.WriteLine(Convert.ToString((float)((1 - 500) * mapResolution) + mapYPosInitial));
                WorldFile.WriteLine(Convert.ToString((float)((1001 - 500) * mapResolution) + mapXPosInitial));
                WorldFile.Flush();
                WorldFile.Close();
            }
            #endregion

            #region Add Interesting Points to the Map
            for (int i=0; i<chkListPositions.CheckedIndices.Count; i++)
            {
                // Get row and column from x and y position
                row = (int)Math.Round((startPoses[chkListPositions.CheckedIndices[i]].xPos - mapXPosInitial) / mapResolution) + 500;
                col = (int)Math.Round((startPoses[chkListPositions.CheckedIndices[i]].yPos - mapYPosInitial) / mapResolution) + 500;

                // Draw Position
                myBrush.Color = Color.Red;
                System.Drawing.Point [] pts = {new System.Drawing.Point(row, col-5),  new System.Drawing.Point(row+5, col), new System.Drawing.Point(row, col+5), new System.Drawing.Point(row-5, col)};
                CombinedMapGraphics.FillPolygon(myBrush, pts);
            }
            #endregion

            #region Add Robots to the Map
            for (int i = 0; i < RobotInfo.Count; i++)
            {
                if (RobotMap[i].Initialized)
                {
                    // Get row and column from robot's x and y position
                    row = (int)Math.Round((RobotMap[i].xPos - mapXPosInitial) / mapResolution) + 500;
                    col = (int)Math.Round((RobotMap[i].yPos - mapYPosInitial) / mapResolution) + 500;

                    // Draw Robot
                    myBrush.Color = RobotInfo[i].Color;
                    CombinedMapGraphics.FillRectangle(myBrush, row - 2, col - 2, 5, 5);
                }
            }
            #endregion

            #region Check for best communication network
            bool saveCoverage = false;
            if (saveFiles)
            {
                List<int> VisitedNodes = new List<int>();
                Queue<int> aQueue = new Queue<int>();

                float coverageMinX = commStationX;
                float coverageMaxX = commStationX;
                float coverageMinY = commStationY;
                float coverageMaxY = commStationY;
                for (int i = 0; i < RobotInfo.Count; i++)
                {
                    if ((float)(ssGrid.Rows[0].Cells[i + 2].Value) > (WSSCutOff * 0.99))
                    {
                        aQueue.Enqueue((i + 1));
                        VisitedNodes.Add((i + 1));

                        if (coverageMinX > RobotMap[i].xPos)
                            coverageMinX = RobotMap[i].xPos;

                        if (coverageMaxX < RobotMap[i].xPos)
                            coverageMaxX = RobotMap[i].xPos;

                        if (coverageMinY > RobotMap[i].yPos)
                            coverageMinY = RobotMap[i].yPos;

                        if (coverageMaxY < RobotMap[i].yPos)
                            coverageMaxY = RobotMap[i].yPos;
                    }
                }

                while (aQueue.Count != 0)
                {
                    int currRobot = aQueue.Dequeue();

                    for (int i = 0; i < RobotInfo.Count; i++)
                    {
                        if (!VisitedNodes.Contains((i + 1)))
                        {
                            if ((float)(ssGrid.Rows[currRobot].Cells[i + 2].Value) > (WSSCutOff * 0.99))
                            {
                                aQueue.Enqueue((i + 1));
                                VisitedNodes.Add((i + 1));

                                if (coverageMinX > RobotMap[i].xPos)
                                    coverageMinX = RobotMap[i].xPos;

                                if (coverageMaxX < RobotMap[i].xPos)
                                    coverageMaxX = RobotMap[i].xPos;

                                if (coverageMinY > RobotMap[i].yPos)
                                    coverageMinY = RobotMap[i].yPos;

                                if (coverageMaxY < RobotMap[i].yPos)
                                    coverageMaxY = RobotMap[i].yPos;
                            }
                        }
                    }
                }

                float currCoverage = (coverageMaxX - coverageMinX) * (coverageMaxY - coverageMinY);
                if (oldCoverage < currCoverage)
                {
                    oldCoverage = currCoverage;
                    saveCoverage = true;
                }
            }
            #endregion

            #region Write Temporary Mesh Network MIF/MID File
            StreamWriter tmpMIF = null;
            StreamWriter tmpMID = null;
            if (saveCoverage)
            {
                tmpMIF = new StreamWriter("RoboCup/RecommendedCommunicationRelay_" + oldCoverage + ".mif");
                tmpMID = new StreamWriter("RoboCup/RecommendedCommunicationRelay_" + oldCoverage + ".mid");
                float MinX = (float)((1 - 500) * mapResolution) + mapYPosInitial;
                float MinY = (float)((1 - 500) * mapResolution) + mapXPosInitial;
                float MaxX = (float)((1001 - 500) * mapResolution) + mapYPosInitial;
                float MaxY = (float)((1001 - 500) * mapResolution) + mapXPosInitial;

                tmpMIF.WriteLine("Version 450");
                tmpMIF.Flush();
                tmpMIF.WriteLine("Charset \"Neutral\"");
                tmpMIF.Flush();
                tmpMIF.WriteLine("Delimiter \",\"");
                tmpMIF.Flush();
                tmpMIF.WriteLine("CoordSys NonEarth Units \"m\" Bounds (" + MinX + ", " + MinY + ") (" + MaxX + ", " + MaxY + ")");
                tmpMIF.Flush();
                tmpMIF.WriteLine("Columns 3");
                tmpMIF.Flush();
                tmpMIF.WriteLine("  Connection char(254)");
                tmpMIF.Flush();
                tmpMIF.WriteLine("  Signal_Strength char(254)");
                tmpMIF.Flush();
                tmpMIF.WriteLine("  Coverage_Distance char(254)");
                tmpMIF.Flush();
                tmpMIF.WriteLine("Data");
                tmpMIF.Flush();
                tmpMIF.WriteLine();
                tmpMIF.Flush();
            }
            #endregion

            #region Show The Communication Network
            for (int i = 0; i <= RobotInfo.Count; i++)
            {
                if (saveCoverage)
                {
                    if (i == 0)
                    {
                        tmpMIF.WriteLine("POINT" + " " + commStationY + " " + commStationX);
                        tmpMIF.Flush();
                        tmpMIF.WriteLine("SYMBOL 46 0 7");
                        tmpMIF.Flush();
                        tmpMID.WriteLine("\"Communication Station\",\"Infinite\",\"N/A\"");
                        tmpMID.Flush();
                    }
                    else
                    {
                        tmpMIF.WriteLine("POINT" + " " + RobotMap[i - 1].yPos + " " + RobotMap[i - 1].xPos);
                        tmpMIF.Flush();
                        tmpMIF.WriteLine("SYMBOL 46 0 7");
                        tmpMIF.Flush();
                        tmpMID.WriteLine("\"" + RobotInfo[i - 1].Name + "\",\"" + Convert.ToString(ssGrid.Rows[0].Cells[i+1].Value) + "\",\"N/A\"");
                        tmpMID.Flush();
                    }
                }

                for (int j = i + 1; j <= RobotInfo.Count; j++)
                {
                    // Figure out the color
                    bool isDropped = false;
                    lock (wss.droppedRobotsLock)
                    {
                        if (i == 0)
                        {
                            isDropped = wss.DroppedRobots.Contains(j);
                        }
                    }
                    if ((float)(ssGrid.Rows[i].Cells[j + 1].Value) <= (WSSCutOff * 0.99))
                    {
                        if (i != 0)
                        {
                            isDropped = true;
                        }
                    }
                    else if ((float)(ssGrid.Rows[i].Cells[j + 1].Value) < (WSSCutOff * 0.95))
                    {
                        myBrush.Color = Color.Red;
                    }
                    else if ((float)(ssGrid.Rows[i].Cells[j + 1].Value) < (WSSCutOff * 0.85))
                    {
                        myBrush.Color = Color.Orange;
                    }
                    else if ((float)(ssGrid.Rows[i].Cells[j + 1].Value) < (WSSCutOff * 0.75))
                    {
                        myBrush.Color = Color.Yellow;
                    }
                    else
                    {
                        myBrush.Color = Color.White;
                    }

                    int LineX1 = 0;
                    int LineY1 = 0;
                    int LineX2 = (int)Math.Round((RobotMap[j - 1].xPos - mapXPosInitial) / mapResolution) + 500;
                    int LineY2 = (int)Math.Round((RobotMap[j - 1].yPos - mapYPosInitial) / mapResolution) + 500;

                    // Draw the network
                    if (i == 0) // If we have to draw the communication station
                    {
                        LineX1 = (int)Math.Round((commStationX - mapXPosInitial) / mapResolution) + 500;
                        LineY1 = (int)Math.Round((commStationY - mapYPosInitial) / mapResolution) + 500;   
                    }
                    else
                    {
                        LineX1 = (int)Math.Round((RobotMap[i - 1].xPos - mapXPosInitial) / mapResolution) + 500;
                        LineY1 = (int)Math.Round((RobotMap[i - 1].yPos - mapYPosInitial) / mapResolution) + 500;
                    }

                    if (chkBoxNetwork.Checked)
                    {
                        if (!isDropped)
                        {
                            CombinedMapGraphics.DrawLine(new Pen(myBrush, 3), LineX1, LineY1, LineX2, LineY2);
                        }
                    }

                    if (saveCoverage)
                    {
                        if (!isDropped)
                        {
                            float currentCoverage = 0;
                            if (i == 0) // If we have to draw the communication station
                            {
                                currentCoverage = (float)Math.Sqrt(Math.Pow(commStationX - RobotMap[j - 1].xPos, 2) + Math.Pow(commStationY - RobotMap[j - 1].yPos, 2));
                                tmpMIF.WriteLine("LINE " + commStationY + " " + commStationX + " " + RobotMap[j - 1].yPos + " " + RobotMap[j - 1].xPos);
                                tmpMIF.Flush();
                                tmpMIF.WriteLine("PEN 2 65 " + Convert.ToString(myBrush.Color.R * 65536 + myBrush.Color.G * 256 + myBrush.Color.B));
                                tmpMIF.Flush();
                                tmpMID.WriteLine("\"Communication<->" + RobotInfo[j - 1].Name + "\",\"" + Convert.ToString(ssGrid.Rows[i].Cells[j + 1].Value) + "\",\"" + currentCoverage + " meters\"");
                                tmpMID.Flush();
                            }
                            else
                            {
                                currentCoverage = (float)Math.Sqrt(Math.Pow(RobotMap[i - 1].xPos - RobotMap[j - 1].xPos, 2) + Math.Pow(RobotMap[i - 1].yPos - RobotMap[j - 1].yPos, 2));
                                tmpMIF.WriteLine("LINE " + RobotMap[i - 1].yPos + " " + RobotMap[i - 1].xPos + " " + RobotMap[j - 1].yPos + " " + RobotMap[j - 1].xPos);
                                tmpMIF.Flush();
                                tmpMIF.WriteLine("PEN 2 65 " + Convert.ToString(myBrush.Color.R * 65536 + myBrush.Color.G * 256 + myBrush.Color.B));
                                tmpMIF.Flush();
                                tmpMID.WriteLine("\"" + RobotInfo[i - 1].Name + "<->" + RobotInfo[j - 1].Name + "\",\"" + Convert.ToString(ssGrid.Rows[i].Cells[j + 1].Value) + "\",\"" + currentCoverage + " meters\"");
                                tmpMID.Flush();
                            }
                        }
                    }
                }
            }
            #endregion

            //pBoxMap.Image = finalImage;
            //if (!pBoxMap.Enabled)
                //pBoxMap.Enabled = true;

            int oldVSize = 0, oldHSize = 0, oldVVal = 0, oldHVal = 0;
            bool changeScroll;


            // Here, we Zoom the Map
            //Image img = new Bitmap(finalImage, finalImage.Width, finalImage.Height);
            //Bitmap bmMap = new Bitmap(img);
            //Graphics gMapWithRobs = Graphics.FromImage(bmMap);

            //if (cBoxZom.Enabled == false)
            //{
                // Turn on Map Zoom Option
                //cBoxZom.SelectedIndex = cBoxZom.FindString("100%");
                //cBoxZom.Enabled = true;
            //}

            // Create the image, which is possibly zoomed
            //string percent = cBoxZom.Items[cBoxZom.SelectedIndex].ToString();
            //float factor = Convert.ToInt32(percent.Substring(0, percent.Length - 1)) / 100;
            //Image image = new Bitmap(finalImage, (int)(finalImage.Width * factor), (int)(finalImage.Height * factor));

            // If we have zoomed in or out
            //if ((changeScroll = (pBoxMap.Enabled && pBoxMap.Image.Width != image.Width && pBoxMap.Image.Height != image.Height)))
            //{
                // Save some old values about the scrollable image
                //oldVSize = pBoxMap.Image.Height;
                //oldHSize = pBoxMap.Image.Width;
                //oldVVal = pBoxMap.vScrollBar1.Value;
                //oldHVal = pBoxMap.hScrollBar1.Value;
            //}

            // Set the image and enable the scrollable image, if needed
            lock (saveMapLock)
            {
                pBoxMap.Image = finalImage;
            
                if (!pBoxMap.Enabled)
                {
                    //mapTimer.Enabled = true;
                    pBoxMap.Enabled = true;
                }
            }

            // Deallocate memory
            myBrush = null;
            finalImage = null;
            CombinedMapGraphics = null;
            //System.GC.Collect();
            
            // If we have zoomed in or out
            //if (changeScroll)
            //{
                // Change the scroller values and image offset to keep image "centered"
                //pBoxMap.vScrollBar1.Value = (int)(oldVVal * (float)((float)(pBoxMap.Image.Height - 477) / (float)(oldVSize - 477)));
                //pBoxMap.hScrollBar1.Value = (int)(oldHVal * (float)((float)(pBoxMap.Image.Width - 477) / (float)(oldHSize - 477)));
                //pBoxMap.OffsetY = pBoxMap.vScrollBar1.Value;
                //pBoxMap.OffsetX = pBoxMap.hScrollBar1.Value;
            //}
        }

        public void ProcessDisconnect()
        {
            // Disable the spawn options
            //txtLoc.Text = "0,0,0";
            //txtRot.Text = "0,0,0";
            txtLocStation.Text = "0,0,0";
            txtRotStation.Text = "0,0,0";
            //cBoxStartPoses.BeginUpdate();
            //cBoxStartPoses.Items.Clear();
            //cBoxStartPoses.Text = "";
            //cBoxStartPoses.EndUpdate();
            cBoxStartPosesStation.BeginUpdate();
            cBoxStartPosesStation.Items.Clear();
            cBoxStartPosesStation.Text = "";
            cBoxStartPosesStation.EndUpdate();
            //txtLoc.Enabled = false;
            //txtRot.Enabled = false;
            txtLocStation.Enabled = false;
            txtRotStation.Enabled = false;
            btnSpawn.Enabled = false;
            //cBoxStartPoses.Enabled = false;
            cBoxStartPosesStation.Enabled = false;
            //cBoxRobot.Enabled = false;
            //cBoxComStation.Enabled = false;

            // Enable the Connect options
            btnConnect.Text = "Connect";
            txtMachine.Enabled = true;
            txtGBPort.Enabled = true;
            //txtISPort.Enabled = true;
            txtWSSPort.Enabled = true;
            btnConnect.Enabled = true;
        }

        private void cbJoystick_SelectedIndexChanged(object sender, EventArgs e)
        {
            IEnumerable<game.Controller> list = cbJoystick.Tag as IEnumerable<game.Controller>;

            if (list != null)
            {
                if (cbJoystick.SelectedIndex >= 0)
                {
                    int index = 0;
                    foreach (game.Controller controller in list)
                    {
                        if (index == cbJoystick.SelectedIndex)
                        {
                            OnChangeJoystick change = new OnChangeJoystick(this);
                            change.Joystick = controller;

                            _eventsPort.Post(change);

                            return;
                        }
                        index++;
                    }
                }
            }
        }

        private void DriveControl_Load(object sender, EventArgs e)
        {
            txtMachine.Text = USARSimIP;
            txtGBPort.Text = USARSimPort.ToString();
            //txtISPort.Text = ImgSrvPort.ToString();
            txtWSSPort.Text = wssPort.ToString();
            _eventsPort.Post(new OnLoad(this));
        }

        private void DriveControl_FormClosed(object sender, FormClosedEventArgs e)
        {
            _eventsPort.Post(new OnClosed(this));
        }

        public void ReplaceJoystickList(IEnumerable<game.Controller> controllers)
        {
            cbJoystick.BeginUpdate();
            try
            {
                cbJoystick.Items.Clear();
                foreach (game.Controller controller in controllers)
                {
                    int index = cbJoystick.Items.Add(controller.InstanceName);
                    if (controller.Current == true)
                    {
                        cbJoystick.SelectedIndex = index;
                    }
                }
                cbJoystick.Tag = controllers;
            }
            finally
            {
                cbJoystick.EndUpdate();
            }
        }

        public void UpdateJoystickAxes(game.Axes axes)
        {
            int x = axes.X;
            int y = -axes.Y;
            int z = axes.Z;
            int w = -axes.Rz;

            lblLJoy.Text = x.ToString() + "," + y.ToString();
            lblRJoy.Text = z.ToString() + "," + w.ToString();

            DrawJoystick(x, y, picLJoystick);
            DrawJoystick(z, w, picRJoystick);

            // If we have received all the required information
            if (cBoxRobots.Enabled)
            {
                if (btnConnect.Text.CompareTo("Disconnect") == 0)
                {
                    if (RobotInfo[currentRobotNumber - 1].Type == "AirRobot")
                    {
                        // Normalize the numbers for the AirRobot
                        float AltitudeVelocity = (Convert.ToSingle(y) * 5.0f) / 1000.0f;
                        float LinearVelocity = (Convert.ToSingle(w) * 5.0f) / 1000.0f;
                        float LateralVelocity = (Convert.ToSingle(z) * 5.0f) / 1000.0f;
                        float RotationalVelocity = (Convert.ToSingle(x) * 1.5708f) / 1000.0f;

                        // Throttle Control
                        AltitudeVelocity *= Convert.ToSingle(throttle.Value) / 100.0f;
                        LinearVelocity *= Convert.ToSingle(throttle.Value) / 100.0f;
                        LateralVelocity *= Convert.ToSingle(throttle.Value) / 100.0f;
                        RotationalVelocity *= Convert.ToSingle(throttle.Value) / 100.0f;

                        // Set up the 5 parameters to send an aerial drive command
                        byte[] buf1 = new byte[] { Convert.ToByte('A'), Convert.ToByte('R'), Convert.ToByte('D') }; // OpCode
                        byte[] buf2 = BitConverter.GetBytes(AltitudeVelocity); // Altitude Velocity
                        byte[] buf3 = BitConverter.GetBytes(LinearVelocity); // Linear Velocity
                        byte[] buf4 = BitConverter.GetBytes(LateralVelocity); // Lateral Velocity
                        byte[] buf5 = BitConverter.GetBytes(RotationalVelocity); // Rotational Velocity

                        // Combine the parameters into one command
                        byte[] buffer = new byte[buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0) + buf4.GetLength(0) + buf5.GetLength(0)];
                        Array.Copy(buf1, 0, buffer, 0, buf1.GetLength(0));
                        Array.Copy(buf2, 0, buffer, buf1.GetLength(0), buf2.GetLength(0));
                        Array.Copy(buf3, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0), buf3.GetLength(0));
                        Array.Copy(buf4, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0), buf4.GetLength(0));
                        Array.Copy(buf5, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0) + buf4.GetLength(0), buf5.GetLength(0));

                        // DEBUG
                        //byte[] tmp = new byte[4];
                        //System.Console.Write("DRIVE Command: " + Convert.ToChar(buffer[0]) + Convert.ToChar(buffer[1]) + Convert.ToChar(buffer[2]) + ",");
                        //Array.Copy(buffer, 3, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + ",");
                        //Array.Copy(buffer, 7, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + ",");
                        //Array.Copy(buffer, 11, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + ",");
                        //Array.Copy(buffer, 15, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + "\n");

                        // Send Command to MOAST
                        wss.WriteLine(currentRobotNumber, buffer);

                        lastCommandSent = "DRIVE";

                        // Deallocate memory
                        buf1 = null;
                        buf2 = null;
                        buf3 = null;
                        buf4 = null;
                        buf5 = null;
                        buffer = null;
                        //System.GC.Collect();
                    }
                    else if (RobotInfo[currentRobotNumber - 1].Type == "Matilda" || RobotInfo[currentRobotNumber - 1].Type == "P2AT" || RobotInfo[currentRobotNumber - 1].Type == "Talon")
                    {
                        float LeftVelocity = 0;
                        float RightVelocity = 0;

                        if (RobotInfo[currentRobotNumber - 1].Type == "Matilda")
                        {
                            // Normalize the numbers for the Matilda
                            LeftVelocity = (Convert.ToSingle(y) * 7.5f) / 1000.0f;
                            RightVelocity = (Convert.ToSingle(w) * 7.5f) / 1000.0f;
                        }
                        else if (RobotInfo[currentRobotNumber - 1].Type == "P2AT")
                        {
                            // Normalize the numbers for the Matilda
                            LeftVelocity = (Convert.ToSingle(y) * 5.385f) / 1000.0f;
                            RightVelocity = (Convert.ToSingle(w) * 5.385f) / 1000.0f;
                        }
                        else if (RobotInfo[currentRobotNumber - 1].Type == "Talon")
                        {
                            // Normalize the numbers for the Matilda
                            LeftVelocity = (Convert.ToSingle(y) * 6.43f) / 1000.0f;
                            RightVelocity = (Convert.ToSingle(w) * 6.43f) / 1000.0f;
                        }

                        //float LeftVelocity = (Convert.ToSingle(y) * 1f) / 1000.0f;
                        //float RightVelocity = (Convert.ToSingle(w) * 1f) / 1000.0f;

                        // Throttle Control
                        LeftVelocity *= Convert.ToSingle(throttle.Value) / 100.0f;
                        RightVelocity *= Convert.ToSingle(throttle.Value) / 100.0f;

                        // Set up the 5 parameters to send an aerial drive command
                        byte[] buf1 = new byte[] { Convert.ToByte('S'), Convert.ToByte('K'), Convert.ToByte('D') }; // OpCode
                        byte[] buf2 = BitConverter.GetBytes(LeftVelocity); // Left Track Velocity
                        byte[] buf3 = BitConverter.GetBytes(RightVelocity); // Right Track Velocity

                        // Combine the parameters into one command
                        byte[] buffer = new byte[buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0)];
                        Array.Copy(buf1, 0, buffer, 0, buf1.GetLength(0));
                        Array.Copy(buf2, 0, buffer, buf1.GetLength(0), buf2.GetLength(0));
                        Array.Copy(buf3, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0), buf3.GetLength(0));

                        // DEBUG
                        //byte[] tmp = new byte[4];
                        //System.Console.Write("DRIVE Command: " + Convert.ToChar(buffer[0]) + Convert.ToChar(buffer[1]) + Convert.ToChar(buffer[2]) + ",");
                        //Array.Copy(buffer, 3, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + ",");
                        //Array.Copy(buffer, 7, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + "\n");

                        // Send Command to MOAST
                        wss.WriteLine(currentRobotNumber, buffer);

                        lastCommandSent = "DRIVE";

                        // Deallocate memory
                        buf1 = null;
                        buf2 = null;
                        buf3 = null;
                        buffer = null;
                        //System.GC.Collect();
                    }
                }
            }
        }

        public void UpdateJoystickButtons(game.Buttons buttons)
        {
            if (buttons.Pressed != null && buttons.Pressed.Count > 0)
            {
                string[] buttonString = buttons.Pressed.ConvertAll<string>(
                    delegate(bool button)
                    {
                        return button ? "X" : "O";
                    }
                ).ToArray();

                lblButtons.Text = string.Join(" ", buttonString);

                // If button #1 is pressed: Toggle Map On/Off
                /*if (buttons.Pressed[0])
                {
                    if (btnMap.Text.CompareTo("Map Off") == 0)
                    {
                        btnMap.Text = "Map On";

                        // Send a new MOF command to the current robot
                        string cmd = "MOF";
                        ASCIIEncoding encoder = new ASCIIEncoding();
                        byte[] buffer = encoder.GetBytes(cmd);
                        wss.WriteLine(currentRobotNumber, buffer);
                        System.Console.WriteLine("Sent MAP OFF to Robot #" + currentRobotNumber);

                        // Remove allocated emmory
                        buffer = null;
                        encoder = null;
                    }
                    else
                    {
                        btnMap.Text = "Map Off";

                        // Send a new MON command to the current robot
                        string cmd = "MON";
                        ASCIIEncoding encoder = new ASCIIEncoding();
                        byte[] buffer = encoder.GetBytes(cmd);
                        wss.WriteLine(currentRobotNumber, buffer);
                        System.Console.WriteLine("Sent MAP ON to Robot #" + currentRobotNumber);

                        // Remove allocated emmory
                        buffer = null;
                        encoder = null;
                    }
                }
                // If button #6 is pressed: Decrease Throttle
                else */
                if (buttons.Pressed[5])
                {
                    if (throttle.Value >= 5)
                        throttle.Value -= 5;

                    if (throttle.Value == 100)
                        throttleLbl.Text = throttle.Value.ToString() + " %";
                    else if (throttle.Value < 100 && throttle.Value >= 10)
                        throttleLbl.Text = " " + throttle.Value.ToString() + " %";
                    else
                        throttleLbl.Text = "  " + throttle.Value.ToString() + " %";
                }
                // If button #8 is pressed: Increase Throttle
                else if (buttons.Pressed[7])
                {
                    if (throttle.Value <= 95)
                        throttle.Value += 5;

                    if (throttle.Value == 100)
                        throttleLbl.Text = throttle.Value.ToString() + " %";
                    else if(throttle.Value < 100 && throttle.Value >= 10)
                        throttleLbl.Text = " " + throttle.Value.ToString() + " %";
                    else
                        throttleLbl.Text = "  " + throttle.Value.ToString() + " %";
                }
                // If button #5 is pressed: Select Next Robot
                else if (buttons.Pressed[4])
                {
                    if (cBoxRobots.SelectedIndex < cBoxRobots.Items.Count-1)
                    {
                        cBoxRobots.SelectedIndex = cBoxRobots.SelectedIndex + 1;
                    }
                }
                // If button #7 is pressed
                else if (buttons.Pressed[6])
                {
                    if (cBoxRobots.SelectedIndex > 0)
                    {
                        cBoxRobots.SelectedIndex = cBoxRobots.SelectedIndex - 1;
                    }
                }
            }

        }

        private void DrawJoystick(int x, int y, PictureBox picJoystick)
        {
            Bitmap bmp = new Bitmap(picJoystick.Width, picJoystick.Height);
            Graphics g = Graphics.FromImage(bmp);

            int width = bmp.Width - 1;
            int height = bmp.Height - 1;

            g.Clear(Color.Transparent);
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            GraphicsPath path = new GraphicsPath();
            path.AddEllipse(0, 0, width, height);

            PathGradientBrush pathBrush = new PathGradientBrush(path);
            pathBrush.CenterPoint = new PointF(width / 3f, height / 3f);
            pathBrush.CenterColor = Color.White;
            pathBrush.SurroundColors = new Color[] { Color.LightGray };

            g.FillPath(pathBrush, path);
            g.DrawPath(Pens.Black, path);

            int partial = y * height / 2200;
            if (partial > 0)
            {
                g.DrawArc(Pens.Black,
                    0,
                    height / 2 - partial,
                    width,
                    2 * partial,
                    180,
                    180);
            }
            else if (partial == 0)
            {
                g.DrawLine(Pens.Black, 0, height / 2, width, height / 2);
            }
            else
            {
                g.DrawArc(Pens.Black,
                    0,
                    height / 2 + partial,
                    width,
                    -2 * partial,
                    0,
                    180);
            }

            partial = x * width / 2200;
            if (partial > 0)
            {
                g.DrawArc(Pens.Black,
                    width / 2 - partial,
                    0,
                    2 * partial,
                    height,
                    270,
                    180);
            }
            else if (partial == 0)
            {
                g.DrawLine(Pens.Black, width / 2, 0, width / 2, height);
            }
            else
            {
                g.DrawArc(Pens.Black,
                    width / 2 + partial,
                    0,
                    -2 * partial,
                    height,
                    90,
                    180);
            }

            picJoystick.Image = bmp;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            // If we are trying to connect
            if (btnConnect.Text.CompareTo("Connect") == 0)
            {
                // Send a CONNECT Command Notification to the ComStation2009Controller
                _mainPort.Post(new CONNECT_CMD(txtMachine.Text,
                                               Convert.ToInt32(txtGBPort.Text),
                                               Convert.ToInt32(txtWSSPort.Text)));

                // Disable the Connect Options
                btnConnect.Enabled = false;
                txtMachine.Enabled = false;
                txtGBPort.Enabled = false;
                txtWSSPort.Enabled = false;

                // Set up various files
                fVictims = new StreamWriter("RoboCup/Victim_Locations.txt"); // Store Victim Information
                fIniPose = new StreamWriter("RoboCup/INIPOSE.txt"); // Store initial pose
                fMIFGrouping = new StreamWriter("RoboCup/Grouping.mif"); // Store MID file for grouping
                fMIDGrouping = new StreamWriter("RoboCup/Grouping.mid"); // Store MIF file for grouping
                fMIFPathsToVictims = new StreamWriter("RoboCup/PathsToVictims.mif"); // Store MID file for the paths to the victims
                fMIDPathsToVictims = new StreamWriter("RoboCup/PathsToVictims.mid"); // Store MIF file for the paths to the victims

                for (int i = 0; i < RobotInfo.Count; i++)
                {
                    StreamWriter aPath = new StreamWriter("RoboCup/" + RobotInfo[i].Name + "Path.txt");
                    fPath.Insert(i, aPath);
                    numPoints.Insert(i, 0);
                }         
            }
            else if(btnConnect.Text.CompareTo("Disconnect") == 0)
            {
                // Send a DISCONNECT Command Notification to the AirRobot Controller
                //_mainPort.Post(new DISCONNECT_CMD()); // TO DO: Go Through the WSS
                btnConnect.Enabled = false;
                btnConnect.Text = "Exit";

                Thread.Sleep(2000);

                wss.Disconnect();

                btnConnect.Enabled = true;
            }
            else if (btnConnect.Text.CompareTo("Exit") == 0)
            {
                this.Close();
            }
        }

        #region Controls What Happens When the User Clicks on the Left Joypad Representation

        private void picLJoystick_MouseLeave(object sender, EventArgs e)
        {
            UpdateJoystickButtons(new game.Buttons());
            UpdateJoystickAxes(new game.Axes());
        }

        private void picLJoystick_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int x, y;
                x = Math.Min(picLJoystick.Width, Math.Max(e.X, 0));
                y = Math.Min(picLJoystick.Height, Math.Max(e.Y, 0));

                x = x * 2000 / picLJoystick.Width - 1000;
                y = y * 2000 / picLJoystick.Height - 1000;

                game.Axes axes = new game.Axes();
                axes.X = x;
                axes.Y = y;

                UpdateJoystickAxes(axes);
            }
        }

        private void picLJoystick_MouseUp(object sender, MouseEventArgs e)
        {
            picLJoystick_MouseLeave(sender, e);
        }

        #endregion

        #region Controls What Happens When the User Clicks on the Right Joypad Representation

        private void picRJoystick_MouseLeave(object sender, EventArgs e)
        {
            UpdateJoystickButtons(new game.Buttons());
            UpdateJoystickAxes(new game.Axes());
        }

        private void picRJoystick_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int z, w;
                z = Math.Min(picLJoystick.Width, Math.Max(e.X, 0));
                w = Math.Min(picLJoystick.Height, Math.Max(e.Y, 0));

                z = z * 2000 / picLJoystick.Width - 1000;
                w = w * 2000 / picLJoystick.Height - 1000;

                game.Axes axes = new game.Axes();
                axes.Z = z;
                axes.Rz = w;

                UpdateJoystickAxes(axes);
            }
        }

        private void picRJoystick_MouseUp(object sender, MouseEventArgs e)
        {
            picRJoystick_MouseLeave(sender, e);
        }

        #endregion

        public void PerformedRoundTrip(bool roundTrip)
        {
            string title = roundTrip ? "Remote Drive Control" : "Remote Drive Control - Connection Down";

            if (Text != title)
            {
                Text = title;
            }
        }

        private void throttle_Scroll(object sender, EventArgs e)
        {
            if (throttle.Value == 100)
                throttleLbl.Text = throttle.Value.ToString() + " %";
            else if (throttle.Value < 100 && throttle.Value >= 10)
                throttleLbl.Text = " " + throttle.Value.ToString() + " %";
            else
                throttleLbl.Text = "  " + throttle.Value.ToString() + " %";
        }

        private void btnSpawn_Click(object sender, EventArgs e)
        {
            _mainPort.Post(new INIT_CMD(ComStationName, txtLocStation.Text, txtRotStation.Text));

            // Start XML timer
            //xmlTimer.Enabled = true;

            string tmp = txtLocStation.Text;

            startPoseX = Convert.ToSingle(tmp.Substring(0, tmp.IndexOf(",")));
            startPoseY = Convert.ToSingle(tmp.Substring(tmp.IndexOf(",") + 1, tmp.IndexOf(",", tmp.IndexOf(",") + 1) - tmp.IndexOf(",") - 1));
        }

        private void pBoxMap_Click(object sender, EventArgs e)
        {
            // Take into account the zoom
            //string percent = cBoxZom.Items[cBoxZom.SelectedIndex].ToString();
            //float factor = Convert.ToInt32(percent.Substring(0, percent.Length - 1)) / 100;
            float factor = 1;

            // Find the row and column clicked (based on the zoom)
            int row = (int)((pBoxMap.OffsetX + ((MouseEventArgs)e).X) / factor);
            int col = (int)((pBoxMap.OffsetY + ((MouseEventArgs)e).Y) / factor);

            // Calculate the X and Y Position, in meters
            float XPos = ((row - 500) * Convert.ToSingle(mapResolution)) + mapXPosInitial;
            float YPos = ((col - 500) * Convert.ToSingle(mapResolution)) + mapYPosInitial;

            //System.Console.WriteLine("You have been clicked at: " + row + "," + col + "/" + XPos + "," + YPos);

            if (btnStartGrouping.Enabled)
            {
                // If the left mouse button was clicked, we send a waypoint to the ground robot
                if (((MouseEventArgs)e).Button.ToString().CompareTo("Left") == 0)
                {
                    if(btnConnect.Text.CompareTo("Disconnect")==0)
                    {
                        // Set up the 3 parameters to send an waypoint command
                        byte[] buf1 = new byte[] { Convert.ToByte('W'), Convert.ToByte('A'), Convert.ToByte('Y') }; // OpCode
                        byte[] buf2 = BitConverter.GetBytes(XPos); // Waypoint X Coordinate
                        byte[] buf3 = BitConverter.GetBytes(YPos); // Waypoint Y Coordinate

                        // Combine the parameters into one command
                        byte[] buffer = new byte[buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0)];
                        Array.Copy(buf1, 0, buffer, 0, buf1.GetLength(0));
                        Array.Copy(buf2, 0, buffer, buf1.GetLength(0), buf2.GetLength(0));
                        Array.Copy(buf3, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0), buf3.GetLength(0));

                        // DEBUG
                        //byte[] tmp = new byte[4];
                        //System.Console.Write("DRIVE Command: " + Convert.ToChar(buffer[0]) + Convert.ToChar(buffer[1]) + Convert.ToChar(buffer[2]) + ",");
                        //Array.Copy(buffer, 3, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + ",");
                        //Array.Copy(buffer, 7, tmp, 0, 4);
                        //System.Console.Write(BitConverter.ToSingle(tmp, 0) + "\n");

                        // Send Command to MOAST
                        wss.WriteLine(currentRobotNumber, buffer);

                        lastCommandSent = "WAYPOINT";
                    }
                }
                // If the right mouse button was clicked, we store the location as a victim location
                else if (((MouseEventArgs)e).Button.ToString().CompareTo("Right") == 0)
                {
                    VictimLoc newVicLoc = new VictimLoc();
                    newVicLoc.Row = row;
                    newVicLoc.Col = col;
                    newVicLoc.xPos = XPos;
                    newVicLoc.yPos = YPos;
                    Victims.Add(newVicLoc);

                    RobotInfo[currentRobotNumber - 1].Victims.Add(Victims.Count);

                    fVictims.WriteLine("Victim" + Victims.Count + ", " +  XPos + ", " + YPos + ", 0.0, VISUAL, 1.0");
                    fVictims.Flush();

                    vForm.newVictim(XPos, YPos);
                }
            }
            else
            {
                Point aPoint = new Point();
                aPoint.X = XPos;
                aPoint.Y = YPos;
                GroupingPoints.Add(aPoint);
                txtPointsClicked.Text = Convert.ToString(GroupingPoints.Count);
            }
        }

        private void mapTimer_Tick(object sender, EventArgs e)
        {
            if (pBoxMap.Enabled)
            {
                lock (saveMapLock)
                {
                    saveMap = true;
                }
            }
        }

        private void cBoxStartPosesStation_SelectedIndexChanged(object sender, EventArgs e)
        {
            string loc;
            string rot;
            string txtPose = cBoxStartPosesStation.Text;
            int index = Convert.ToInt32(txtPose.Substring(0, txtPose.IndexOf(":", 0))) - 1;

            // Make sure that another thread is not modifying the startPoses variable
            lock (startPoseLock)
            {
                loc = startPoses[index].xPos.ToString() + "," +
                      startPoses[index].yPos.ToString() + "," +
                      startPoses[index].zPos.ToString();

                rot = startPoses[index].xRot.ToString() + "," +
                      startPoses[index].yRot.ToString() + "," +
                      startPoses[index].zRot.ToString();
            }

            txtLocStation.Text = loc;
            txtRotStation.Text = rot;
        }

        private void btnStartGrouping_Click(object sender, EventArgs e)
        {
            txtPointsClicked.Text = "0";
            lBoxGrouping.SelectedIndex = 0;
            btnStartGrouping.Enabled = false;
            btnStopGrouping.Enabled = true;
            lBoxGrouping.Enabled = true;
            txtPointsClicked.Enabled = true;
        }

        private void btnStopGrouping_Click(object sender, EventArgs e)
        {
            if (lBoxGrouping.Items[lBoxGrouping.SelectedIndex].ToString().CompareTo("Victim Path") == 0)
            {
                float DistanceCovered = 0;
                bool gotFirstData = false;
                float oldXPose = 0;
                float oldYPose = 0;

                fMIFPathsToVictims.WriteLine("PLINE");
                fMIFPathsToVictims.Flush();
                fMIFPathsToVictims.WriteLine("  " + GroupingPoints.Count);
                fMIFPathsToVictims.Flush();
                for (int i = 0; i < GroupingPoints.Count; i++)
                {
                    fMIFPathsToVictims.WriteLine(GroupingPoints[i].Y + " " + GroupingPoints[i].X);
                    fMIFPathsToVictims.Flush();

                    // Calculate Distance Covered
                    if (!gotFirstData)
                        gotFirstData = true;
                    else
                        DistanceCovered += (float)Math.Sqrt(Math.Pow(GroupingPoints[i].X - oldXPose, 2) + Math.Pow(GroupingPoints[i].Y - oldYPose, 2));

                    oldXPose = GroupingPoints[i].X;
                    oldYPose = GroupingPoints[i].Y;
                    
                }

                fMIFPathsToVictims.WriteLine("PEN 3 65 16711680");
                fMIFPathsToVictims.Flush();
                fMIFPathsToVictims.WriteLine("SMOOTH");
                fMIFPathsToVictims.Flush();
                fMIFPathsToVictims.WriteLine();
                fMIFPathsToVictims.Flush();

                StreamReader VictimInfo = new StreamReader("RoboCup/Victims.mid");
                string Line = "";
                for (int i = 0; i < Victims.Count; i++) 
                {
                    Line = VictimInfo.ReadLine();
                }

                //fMIDPathsToVictims.WriteLine("\"Victim" + Victims.Count + "\",\"" + DistanceCovered + " meters\"");
                fMIDPathsToVictims.WriteLine(Line + ",\"" + DistanceCovered + " meters\"");
                fMIDPathsToVictims.Flush();
            }
            else
            {
                if (GroupingPoints.Count > 2)
                {
                    fMIFGrouping.WriteLine("REGION 1");
                    fMIFGrouping.Flush();
                    fMIFGrouping.WriteLine(GroupingPoints.Count);
                    fMIFGrouping.Flush();
                    for (int i = 0; i < GroupingPoints.Count; i++)
                    {
                        fMIFGrouping.WriteLine(GroupingPoints[i].Y + " " + GroupingPoints[i].X);
                        fMIFGrouping.Flush();
                    }
                }
                else if (GroupingPoints.Count == 2)
                {
                    fMIFGrouping.WriteLine("RECT " + GroupingPoints[0].Y + " " + GroupingPoints[0].X + " " + GroupingPoints[1].Y + " " + GroupingPoints[1].X);
                    fMIFGrouping.Flush();
                }

                if (lBoxGrouping.Items[lBoxGrouping.SelectedIndex].ToString().CompareTo("Corridor") == 0)
                    GroupFence();
                else if (lBoxGrouping.Items[lBoxGrouping.SelectedIndex].ToString().CompareTo("Hazard") == 0)
                    GroupWall();
                else if (lBoxGrouping.Items[lBoxGrouping.SelectedIndex].ToString().CompareTo("Office") == 0)
                    GroupVehicle();
                else if (lBoxGrouping.Items[lBoxGrouping.SelectedIndex].ToString().CompareTo("Fire") == 0)
                    GroupFire();
                else if (lBoxGrouping.Items[lBoxGrouping.SelectedIndex].ToString().CompareTo("Open Space") == 0)
                    GroupBuilding();
                else if (lBoxGrouping.Items[lBoxGrouping.SelectedIndex].ToString().CompareTo("Debris") == 0)
                    GroupDebris();

                fMIFGrouping.WriteLine();
                fMIFGrouping.Flush();
            }

            GroupingPoints.Clear();

            lBoxGrouping.Enabled = false;
            txtPointsClicked.Enabled = false;
            btnStopGrouping.Enabled = false;
            btnStartGrouping.Enabled = true;
        }

        public void GroupFence()
        {
            Color aColor = Color.DarkCyan;
            int color = aColor.R * 65536 + aColor.G * 256 + aColor.B;
            fMIFGrouping.WriteLine("  PEN (1, 66, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIFGrouping.WriteLine("  BRUSH (7, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIDGrouping.WriteLine((++GroupedObjects) + ",\"Corridor\"");
            fMIDGrouping.Flush();
        }

        public void GroupWall()
        {
            Color aColor = Color.LightBlue;
            int color = aColor.R * 65536 + aColor.G * 256 + aColor.B;
            fMIFGrouping.WriteLine("  PEN (1, 66, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIFGrouping.WriteLine("  BRUSH (4, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIDGrouping.WriteLine((++GroupedObjects) + ",\"Hazard\"");
            fMIDGrouping.Flush();
        }

        public void GroupVehicle()
        {
            Color aColor = Color.Green;
            int color = aColor.R * 65536 + aColor.G * 256 + aColor.B;
            fMIFGrouping.WriteLine("  PEN (1, 66, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIFGrouping.WriteLine("  BRUSH (5, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIDGrouping.WriteLine((++GroupedObjects) + ",\"Office\"");
            fMIDGrouping.Flush();
        }

        public void GroupFire()
        {
            Color aColor = Color.Red;
            int color = aColor.R * 65536 + aColor.G * 256 + aColor.B;
            fMIFGrouping.WriteLine("  PEN (1, 66, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIFGrouping.WriteLine("  BRUSH (8, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIDGrouping.WriteLine((++GroupedObjects) + ",\"Fire\"");
            fMIDGrouping.Flush();
        }

        public void GroupBuilding()
        {
            Color aColor = Color.DarkBlue;
            int color = aColor.R * 65536 + aColor.G * 256 + aColor.B;
            fMIFGrouping.WriteLine("  PEN (1, 66, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIFGrouping.WriteLine("  BRUSH (4, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIDGrouping.WriteLine((++GroupedObjects) + ",\"Open Space\"");
            fMIDGrouping.Flush();
        }

        public void GroupDebris()
        {
            Color aColor = Color.Orange;
            int color = aColor.R * 65536 + aColor.G * 256 + aColor.B;
            fMIFGrouping.WriteLine("  PEN (1, 66, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIFGrouping.WriteLine("  BRUSH (8, " + color.ToString() + ")");
            fMIFGrouping.Flush();
            fMIDGrouping.WriteLine((++GroupedObjects) + ",\"Debris\"");
            fMIDGrouping.Flush();
        }

        private void btnWssConnect_Click(object sender, EventArgs e)
        {
            wss.Connect(txtMachine.Text, Convert.ToInt32(txtWSSPort.Text));
            btnWssConnect.Enabled = false;
            btnInitAll.Enabled = true;
        }

        private void cBoxRobots_SelectedIndexChanged(object sender, EventArgs e)
        {
            string txtRobot = cBoxRobots.Text;
            int newRobotNumber = Convert.ToInt32(txtRobot.Substring(0, txtRobot.IndexOf(":", 0)));

            if (newRobotNumber != currentRobotNumber)
            {
                if(lastCommandSent.CompareTo("DRIVE")==0)
                {
                    SendStop(currentRobotNumber-1);
                }

                if (RobotInfo[newRobotNumber-1].Type.CompareTo("AirRobot") == 0)
                {
                    throttle.Value = 25;
                    throttleLbl.Text = " " + throttle.Value.ToString() + " %";
                }
                else if (RobotInfo[newRobotNumber-1].Type.CompareTo("P2AT") == 0)
                {
                    throttle.Value = 25;
                    throttleLbl.Text = " " + throttle.Value.ToString() + " %";
                }
                else if (RobotInfo[newRobotNumber-1].Type.CompareTo("Talon") == 0)
                {
                    throttle.Value = 35;
                    throttleLbl.Text = " " + throttle.Value.ToString() + " %";
                }
                else if (RobotInfo[newRobotNumber-1].Type.CompareTo("Matilda") == 0)
                {
                    throttle.Value = 35;
                    throttleLbl.Text = " " + throttle.Value.ToString() + " %";
                }

                currentRobotNumber = newRobotNumber;

                _mainPort.Post(new CHVIEW_CMD(RobotInfo[currentRobotNumber-1].Name, "Camera", USARSimIP));
            }
        }

        private void ssTimer_Tick(object sender, EventArgs e)
        {
            if (cBoxRobots.Enabled == true)
            {
                if (btnConnect.Text.CompareTo("Disconnect") == 0)
                {
                    if (!ssGrid.Rows[0].Cells[1].Style.BackColor.Equals(Color.Black))
                    {
                        ssGrid.Rows[0].Cells[1].Style.BackColor = Color.Black;
                    }

                    for (int i = 0; i < RobotInfo.Count; i++)
                    {
                        lock (wss.getSSLock)
                        {
                            ssGrid.Rows[0].Cells[i + 2].Value = wss.GetSignalStrength(RobotInfo[i].Name);
                        }

                        bool isDropped;
                        lock (wss.droppedRobotsLock)
                        {
                            isDropped = wss.DroppedRobots.Contains(i+1);
                        }
                        if (isDropped)
                        {
                            if (!ssGrid.Rows[0].Cells[i + 2].Style.BackColor.Equals(Color.SlateGray))
                            {
                                ssGrid.Rows[0].Cells[i + 2].Style.BackColor = Color.SlateGray;
                            }
                        }
                        else if ((float)(ssGrid.Rows[0].Cells[i + 2].Value) < (WSSCutOff * 0.95))
                        {
                            if (!ssGrid.Rows[0].Cells[i + 2].Style.BackColor.Equals(Color.Red))
                            {
                                ssGrid.Rows[0].Cells[i + 2].Style.BackColor = Color.Red;
                            }
                        }
                        else if ((float)(ssGrid.Rows[0].Cells[i + 2].Value) < (WSSCutOff * 0.85))
                        {
                            if (!ssGrid.Rows[0].Cells[i + 2].Style.BackColor.Equals(Color.Orange))
                            {
                                ssGrid.Rows[0].Cells[i + 2].Style.BackColor = Color.Orange;
                            }
                        }
                        else if ((float)(ssGrid.Rows[0].Cells[i + 2].Value) < (WSSCutOff * 0.75))
                        {
                            if (!ssGrid.Rows[0].Cells[i + 2].Style.BackColor.Equals(Color.Yellow))
                            {
                                ssGrid.Rows[0].Cells[i + 2].Style.BackColor = Color.Yellow;
                            }
                        }
                        else
                        {
                            if (!ssGrid.Rows[0].Cells[i + 2].Style.BackColor.Equals(Color.LightGreen))
                            {
                                ssGrid.Rows[0].Cells[i + 2].Style.BackColor = Color.LightGreen;
                            }
                        }

                        batteryGrid.Rows[0].Cells[0].Value = RobotInfo[currentRobotNumber-1].BatteryLife;

                        if (Convert.ToInt32(batteryGrid.Rows[0].Cells[0].Value) != -1)
                        {
                            if (Convert.ToInt32(batteryGrid.Rows[0].Cells[0].Value) <= (int)(30))
                            {
                                if (!warning3)
                                {
                                    warning3 = true;
                                    MessageBox.Show("Only 30 seconds left!");
                                }
                                if (!batteryGrid.Rows[0].Cells[0].Style.BackColor.Equals(Color.Red))
                                {
                                    batteryGrid.Rows[0].Cells[0].Style.BackColor = Color.Red;
                                }
                            }
                            else if (Convert.ToInt32(batteryGrid.Rows[0].Cells[0].Value) <= (int)(2 * 60))
                            {
                                if (!warning2)
                                {
                                    warning2 = true;
                                    MessageBox.Show("Only 2 minutes left!");
                                }
                                if (!batteryGrid.Rows[0].Cells[0].Style.BackColor.Equals(Color.Orange))
                                {
                                    batteryGrid.Rows[0].Cells[0].Style.BackColor = Color.Orange;
                                }
                            }
                            else if (Convert.ToInt32(batteryGrid.Rows[0].Cells[0].Value) <= (int)(5 * 60))
                            {
                                if (!warning1)
                                {
                                    warning1 = true;
                                    MessageBox.Show("5 Minutes Left...");
                                }
                                if (!batteryGrid.Rows[0].Cells[0].Style.BackColor.Equals(Color.Yellow))
                                {
                                    batteryGrid.Rows[0].Cells[0].Style.BackColor = Color.Yellow;
                                }
                            }
                            else
                            {
                                if (!batteryGrid.Rows[0].Cells[0].Style.BackColor.Equals(Color.LightGreen))
                                {
                                    batteryGrid.Rows[0].Cells[0].Style.BackColor = Color.LightGreen;
                                }
                            }
                        }
                    }
                }
            }
        }

        private void gcTimer_Tick(object sender, EventArgs e)
        {
            // Garbage Collect
            System.GC.Collect();
        }

        private void btnInitAll_Click(object sender, EventArgs e)
        {
            for (int i = 1; i <= RobotInfo.Count; i++)
            {
                // Send a new INI command to the current robot
                string cmd = "INI";
                ASCIIEncoding encoder = new ASCIIEncoding();
                byte[] buffer = encoder.GetBytes(cmd);
                wss.WriteLine(i, buffer);

                // Remove allocated emmory
                buffer = null;
                encoder = null;
            }

            btnInitAll.Enabled = false;
            btnSPAll.Enabled = true;
        }

        private void btnSPAll_Click(object sender, EventArgs e)
        {
            for (int i = 1; i <= RobotInfo.Count; i++)
            {

                // Send a new SPG command to the current robot
                string cmd = "SPG";
                ASCIIEncoding encoder = new ASCIIEncoding();
                byte[] buffer = encoder.GetBytes(cmd);
                wss.WriteLine(i, buffer);

                // Remove allocated emmory
                buffer = null;
                encoder = null;
            }

            btnSPAll.Enabled = false;
            btnInit.Enabled = true;
            btnSP.Enabled = true;
        }

        private void btnInit_Click(object sender, EventArgs e)
        {
            // Send a new INI command to the current robot
            string cmd = "INI";
            ASCIIEncoding encoder = new ASCIIEncoding();
            byte[] buffer = encoder.GetBytes(cmd);
            wss.WriteLine(currentRobotNumber, buffer);

            // Remove allocated emmory
            buffer = null;
            encoder = null;
        }

        private void btnSP_Click(object sender, EventArgs e)
        {
            // Send a new SPG command to the current robot
            string cmd = "SPG";
            ASCIIEncoding encoder = new ASCIIEncoding();
            byte[] buffer = encoder.GetBytes(cmd);
            wss.WriteLine(currentRobotNumber, buffer);

            // Remove allocated emmory
            buffer = null;
            encoder = null;
        }

        private void mapShowTimer_Tick(object sender, EventArgs e)
        {
            if (mapGotInitialPosition)
            {
                lock (mapLock)
                {
                    ShowMap();
                }
            }
        }

        private void btnMap_Click(object sender, EventArgs e)
        {
            if (btnMap.Text.CompareTo("Map Off") == 0)
            {
                btnMap.Text = "Map On";

                // Send a new MOF command to the current robot
                string cmd = "MOF";
                ASCIIEncoding encoder = new ASCIIEncoding();
                byte[] buffer = encoder.GetBytes(cmd);
                wss.WriteLine(currentRobotNumber, buffer);

                // Remove allocated emmory
                buffer = null;
                encoder = null;
            }
            else
            {
                btnMap.Text = "Map Off";

                // Send a new MON command to the current robot
                string cmd = "MON";
                ASCIIEncoding encoder = new ASCIIEncoding();
                byte[] buffer = encoder.GetBytes(cmd);
                wss.WriteLine(currentRobotNumber, buffer);

                // Remove allocated emmory
                buffer = null;
                encoder = null;
            }
        }

        private void btnStopAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 3; i++)
            {
                SendStops(); // Send a stop command to all the robots
            }
        }

        private void SendStops()
        {
            for (int i = 0; i < RobotInfo.Count; i++)
            {
                SendStop(i);
            }
        }

        private void SendStop(int i) // i is zero-indexed
        {
            // If we are dealing with an AirRobot
            if (RobotInfo[i].Type.CompareTo("AirRobot") == 0)
            {
                // Normalize the numbers for the AirRobot
                float AltitudeVelocity = 0.0f;
                float LinearVelocity = 0.0f;
                float LateralVelocity = 0.0f;
                float RotationalVelocity = 0.0f;

                // Set up the 5 parameters to send an aerial drive command
                byte[] buf1 = new byte[] { Convert.ToByte('A'), Convert.ToByte('R'), Convert.ToByte('D') }; // OpCode
                byte[] buf2 = BitConverter.GetBytes(AltitudeVelocity); // Altitude Velocity
                byte[] buf3 = BitConverter.GetBytes(LinearVelocity); // Linear Velocity
                byte[] buf4 = BitConverter.GetBytes(LateralVelocity); // Lateral Velocity
                byte[] buf5 = BitConverter.GetBytes(RotationalVelocity); // Rotational Velocity

                // Combine the parameters into one command
                byte[] buffer = new byte[buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0) + buf4.GetLength(0) + buf5.GetLength(0)];
                Array.Copy(buf1, 0, buffer, 0, buf1.GetLength(0));
                Array.Copy(buf2, 0, buffer, buf1.GetLength(0), buf2.GetLength(0));
                Array.Copy(buf3, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0), buf3.GetLength(0));
                Array.Copy(buf4, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0), buf4.GetLength(0));
                Array.Copy(buf5, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0) + buf4.GetLength(0), buf5.GetLength(0));

                // Send Command to MOAST
                wss.WriteLine(i + 1, buffer);

                // Deallocate memory
                buf1 = null;
                buf2 = null;
                buf3 = null;
                buf4 = null;
                buf5 = null;
                buffer = null;
            }
            else // If we are dealing with a ground robot
            {
                float LeftVelocity = 0.0f;
                float RightVelocity = 0.0f;

                // Set up the 5 parameters to send an aerial drive command
                byte[] buf1 = new byte[] { Convert.ToByte('S'), Convert.ToByte('K'), Convert.ToByte('D') }; // OpCode
                byte[] buf2 = BitConverter.GetBytes(LeftVelocity); // Left Track Velocity
                byte[] buf3 = BitConverter.GetBytes(RightVelocity); // Right Track Velocity

                // Combine the parameters into one command
                byte[] buffer = new byte[buf1.GetLength(0) + buf2.GetLength(0) + buf3.GetLength(0)];
                Array.Copy(buf1, 0, buffer, 0, buf1.GetLength(0));
                Array.Copy(buf2, 0, buffer, buf1.GetLength(0), buf2.GetLength(0));
                Array.Copy(buf3, 0, buffer, buf1.GetLength(0) + buf2.GetLength(0), buf3.GetLength(0));

                // Send Command to MOAST
                wss.WriteLine(i + 1, buffer);

                // Deallocate memory
                buf1 = null;
                buf2 = null;
                buf3 = null;
                buffer = null;
            }
        }

        private void btnIMR_Click(object sender, EventArgs e)
        {
            // Send a new IMR command to the current robot
            string cmd = "IMR";
            ASCIIEncoding encoder = new ASCIIEncoding();
            byte[] buffer = encoder.GetBytes(cmd);
            wss.WriteLine(currentRobotNumber, buffer);
        }
    }

    #region RobotsInformation data structure
    // To Talk to the Robots
    public class RobotsInformation
    {
        private string _rName;
        private string _type;
        private Color _color;
        private List<int> _victims = new List<int>();
        private int _batteryLife;

        [DataMember]
        public string Name
        {
            get { return _rName; }
            set { _rName = value; }
        }

        [DataMember]
        public string Type
        {
            get { return _type; }
            set { _type = value; }
        }

        [DataMember]
        public Color Color
        {
            get { return _color; }
            set { _color = value; }
        }

        [DataMember]
        public List<int> Victims
        {
            get { return _victims; }
            set { _victims = value; }
        }

        [DataMember]
        public int BatteryLife
        {
            get { return _batteryLife; }
            set { _batteryLife = value; }
        }
    }
    #endregion

    class DriveControlEvents 
        : PortSet<
            OnLoad,
            OnClosed,
            OnChangeJoystick
        >
    {
    }
    
    class DriveControlEvent
    {
        private DriveControl _driveControl;

        public DriveControl DriveControl
        {
            get { return _driveControl; }
            set { _driveControl = value; }
        }

        public DriveControlEvent(DriveControl driveControl)
        {
            _driveControl = driveControl;
        }
    }

    class OnLoad : DriveControlEvent
    {
        public OnLoad(DriveControl form)
            : base(form)
        {
        }
    }

    class OnClosed : DriveControlEvent
    {
        public OnClosed(DriveControl form)
            : base(form)
        {
        }
    }

    class OnChangeJoystick : DriveControlEvent
    {
        game.Controller _joystick;

        public game.Controller Joystick
        {
            get { return _joystick; }
            set { _joystick = value; }
        }

        public OnChangeJoystick(DriveControl form)
            : base(form)
        {
        }
    }
}
