//-----------------------------------------------------------------------
//  This file is part of the UC Merced Microsoft Robotics Developer Studio 
// Project Code Base.
//
//  Created by Benjamin Balaguer under the supervision of Stefano Carpin.
//  Based on SimpleDashboardTypes.cs from UX\SimpleDashboard.
//
//  $File: RoboCup2009GUITypes.cs $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;


using Microsoft.Ccr.Core;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.Core.Attributes;
using dssp = Microsoft.Dss.ServiceModel.Dssp;
using W3C.Soap;
using Microsoft.Dss.Core.DsspHttp;
using System.Drawing;

namespace Microsoft.Robotics.Services.RoboCup2009GUI
{

    /// <summary>
    /// DSS Contract for RoboCup2009GUI
    /// </summary>
    static class Contract
    {
        /// <summary>
        /// The DSS Namespace for RoboCup2009GUI
        /// </summary>
        public const string Identifier = "http://schemas.microsoft.com/robotics/2008/01/robocup2009gui.html";
    }

    /// <summary>
    /// The RoboCup2009GUI Operations Port
    /// </summary>
    class RoboCup2009GUIOperations : PortSet<
        DsspDefaultLookup, 
        DsspDefaultDrop, 
        Get, 
        HttpGet, 
        Subscribe, 
        CONNECT_CMD,
        DISCONNECT_CMD,
        GET_POSES_CMD,
        INIT_CMD,
        CHVIEW_CMD,
        //GET_CONF_CMD,
        //DRIVE_AERIAL_CMD,
        //MIS_PKG_CMD,
        //IMG_SVR_CAM_CMD,
        //IMG_SVR_OK_CMD,
        WAYPOINT_CMD,
        CONNECTED_MSG,
        START_POSES_MSG,
        INITIALIZED_MSG,
        //STA_MSG,
        //GPS_MSG,
        IMAGE_MSG,
        RTYPE_MSG,
        RSS_MSG,
        MAP_UPDATE_MSG>
    {
    }

    /// <summary>
    /// DSS Get Definition for RoboCup2009GUI 
    /// </summary>
    class Get : Get<dssp.GetRequestType, PortSet<RoboCup2009GUIState, W3C.Soap.Fault>>
    {
        /// <summary>
        /// Default DSS Get Constructor
        /// </summary>
        public Get()
        {
        }

        /// <summary>
        /// DSS GetRequestType Constructor
        /// </summary>
        /// <param name="body"></param>
        public Get(dssp.GetRequestType body)
            : base(body)
        {
        }

    }

    // To handle subscriptions
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }

    #region Commands Going to AirRobot Controller
    
    #region CONNECT COMMAND
    public class CONNECT_CMD : Update<ConnectCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public CONNECT_CMD()
            : base(new ConnectCMDRequest())
        {
        }

        public CONNECT_CMD(string IP, int gameBotPort, int wssPort)
            : base(new ConnectCMDRequest(IP, gameBotPort, wssPort))
        {
        }
    }
    
    [DataContract]
    public class ConnectCMDRequest
    {
        public ConnectCMDRequest()
        {
        }

        public ConnectCMDRequest(string ip, int gameBotPort, int wssPort)
        {
            _ip = ip;
            _gameBotPort = gameBotPort;
            _wssPort = wssPort;
        }

        private string _ip;
        private int _gameBotPort;
        private int _wssPort;

        [DataMember]
        public string IP
        {
            get { return _ip; }
            set { _ip = value; }
        }

        [DataMember]
        public int gameBotPort
        {
            get { return _gameBotPort; }
            set { _gameBotPort = value; }
        }

        [DataMember]
        public int wssPort
        {
            get { return _wssPort; }
            set { _wssPort = value; }
        }
    }
    #endregion

    #region DISCONNECT COMMAND
    public class DISCONNECT_CMD : Update<DisconnectCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public DISCONNECT_CMD() : base(new DisconnectCMDRequest())
        {
        }
    }

    [DataContract]
    public class DisconnectCMDRequest
    {
        public DisconnectCMDRequest()
        {
        }
    }
    #endregion
    
    #region GETSTARTPOSES COMMAND
    public class GET_POSES_CMD : Update<GetPosesCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public GET_POSES_CMD()
            : base(new GetPosesCMDRequest())
        {
        }
    }

    [DataContract]
    public class GetPosesCMDRequest
    {
        public GetPosesCMDRequest()
        {
        }
    }
    #endregion
    
    #region INIT COMMAND
    public class INIT_CMD : Update<InitCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public INIT_CMD() : base(new InitCMDRequest())
        {
        }

        public INIT_CMD(string Name, string Location, string Rotation) 
            : base(new InitCMDRequest(Name, Location, Rotation))
        {
        }
    }

    [DataContract]
    public class InitCMDRequest
    {
        public InitCMDRequest()
        {
        }

        public InitCMDRequest(string name, string location, string rotation)
        {
            _name = name;
            _location = location;
            _rotation = rotation;
        }

        private string _name;
        private string _location;
        private string _rotation;

        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public string Location
        {
            get { return _location; }
            set { _location = value; }
        }

        [DataMember]
        public string Rotation
        {
            get { return _rotation; }
            set { _rotation = value; }
        }
    }
    #endregion

    #region CHANGE VIEWPORT COMMAND
    public class CHVIEW_CMD : Update<ChViewCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public CHVIEW_CMD()
            : base(new ChViewCMDRequest())
        {
        }

        public CHVIEW_CMD(string RobotName, string CameraName, string IP)
            : base(new ChViewCMDRequest(RobotName, CameraName, IP))
        {
        }
    }

    [DataContract]
    public class ChViewCMDRequest
    {
        public ChViewCMDRequest()
        {
        }

        public ChViewCMDRequest(string robotName, string cameraName, string IP)
        {
            _robotName = robotName;
            _cameraName = cameraName;
            _IP = IP;
        }

        private string _robotName;
        private string _cameraName;
        private string _IP;

        [DataMember]
        public string RobotName
        {
            get { return _robotName; }
            set { _robotName = value; }
        }

        [DataMember]
        public string CameraName
        {
            get { return _cameraName; }
            set { _cameraName = value; }
        }

        [DataMember]
        public string IP
        {
            get { return _IP; }
            set { _IP = value; }
        }
    }
    #endregion

    /*
    #region GETCONF COMMAND
    public class GET_CONF_CMD : Update<GetConfCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public GET_CONF_CMD() : base(new GetConfCMDRequest())
        {
        }

        public GET_CONF_CMD(string Type, string Name) : base(new GetConfCMDRequest(Type, Name))
        {
        }
    }

    [DataContract]
    public class GetConfCMDRequest
    {
        public GetConfCMDRequest()
        {
        }

        public GetConfCMDRequest(string type, string name)
        {
            _type = type;
            _name = name;
        }

        private string _type;
        private string _name;
        
        [DataMember]
        public string Type
        {
            get { return _type; }
            set { _type = value; }
        }

        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }
    }
    #endregion
    
    #region AERIAL DRIVE COMMAND
    public class DRIVE_AERIAL_CMD : Update<DriveAerialCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public DRIVE_AERIAL_CMD() : base(new DriveAerialCMDRequest())
        {
        }

        public DRIVE_AERIAL_CMD(string AltitudeVelocity, string LinearVelocity, string LateralVelocity, string RotationalVelocity, bool Normalized)
            : base(new DriveAerialCMDRequest(AltitudeVelocity, LinearVelocity, LateralVelocity, RotationalVelocity, Normalized))
        {
        }
    }

    [DataContract]
    public class DriveAerialCMDRequest
    {
        public DriveAerialCMDRequest()
        {
        }

        public DriveAerialCMDRequest(string altitudeVelocity, string linearVelocity, string lateralVelocity, string rotationalVelocity, bool normalized)
        {
            _altitudeVelocity = altitudeVelocity;
            _linearVelocity = linearVelocity;
            _lateralVelocity = lateralVelocity;
            _rotationalVelocity = rotationalVelocity;
            _normalized = normalized;
        }

        private string _altitudeVelocity;
        private string _linearVelocity;
        private string _lateralVelocity;
        private string _rotationalVelocity;
        private bool _normalized;

        [DataMember]
        public string AltitudeVelocity
        {
            get { return _altitudeVelocity; }
            set { _altitudeVelocity = value; }
        }

        [DataMember]
        public string LinearVelocity
        {
            get { return _linearVelocity; }
            set { _linearVelocity = value; }
        }

        [DataMember]
        public string LateralVelocity
        {
            get { return _lateralVelocity; }
            set { _lateralVelocity = value; }
        }

        [DataMember]
        public string RotationalVelocity
        {
            get { return _rotationalVelocity; }
            set { _rotationalVelocity = value; }
        }

        [DataMember]
        public bool Normalized
        {
            get { return _normalized; }
            set { _normalized = value; }
        }
    }
    #endregion

    #region MISSION PACKAGE COMMAND
    public class MIS_PKG_CMD : Update<MisPkgCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public MIS_PKG_CMD()
            : base(new MisPkgCMDRequest())
        {
        }

        public MIS_PKG_CMD(string Name, List<LinkProperty> Link)
            : base(new MisPkgCMDRequest(Name, Link))
        {
        }
    }
    
    #region Mission Package Link Information
    [DataContract]
    public class LinkProperty
    {
        #region Variables
        private float _desiredVal;
        private int _desiredOrder;
        private int _id;
        #endregion

        #region Constructor
        public LinkProperty()
        {
            _id = 0;
            _desiredVal = 0;
            _desiredOrder = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public int ID
        {
            get { return _id; }
            set { _id = value; }
        }

        [DataMember]
        public float DesiredVal
        {
            get { return _desiredVal; }
            set { _desiredVal = value; }
        }

        [DataMember]
        public int DesiredOrder
        {
            get { return _desiredOrder; }
            set { _desiredOrder = value; }
        }
        #endregion
    }
    #endregion
    
    [DataContract]
    public class MisPkgCMDRequest
    {
        public MisPkgCMDRequest()
        {
        }

        public MisPkgCMDRequest(string name, List<LinkProperty> link)
        {
            _name = name;
            _link = link;
        }

        private string _name;
        private List<LinkProperty> _link;

        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public List<LinkProperty> Link
        {
            get { return _link; }
            set { _link = value; }
        }
    }
    #endregion

    #region IMAGE SERVER COMMAND
    public class IMG_SVR_CAM_CMD : Update<ImgSvrCamCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public IMG_SVR_CAM_CMD()
            : base(new ImgSvrCamCMDRequest())
        {
        }

        public IMG_SVR_CAM_CMD(string Robot, string Cam, string IP)
            : base(new ImgSvrCamCMDRequest(Robot, Cam, IP))
        {
        }
    }

    [DataContract]
    public class ImgSvrCamCMDRequest
    {
        public ImgSvrCamCMDRequest()
        {
        }

        public ImgSvrCamCMDRequest(string robot, string cam, string ip)
        {
            _robot = robot;
            _cam = cam;
            _ip = ip;
        }

        private string _robot;
        private string _cam;
        private string _ip;

        [DataMember]
        public string RobotName
        {
            get { return _robot; }
            set { _robot = value; }
        }
        
        [DataMember]
        public string CamName
        {
            get { return _cam; }
            set { _cam = value; }
        }

        [DataMember]
        public string IP
        {
            get { return _ip; }
            set { _ip = value; }
        }

    }
    #endregion

    #region IMAGE SERVER OK COMMAND
    public class IMG_SVR_OK_CMD : Update<ImgSvrOKCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public IMG_SVR_OK_CMD()
            : base(new ImgSvrOKCMDRequest())
        {
        }
    }

    [DataContract]
    public class ImgSvrOKCMDRequest
    {
        public ImgSvrOKCMDRequest()
        {
        }
    }
    #endregion
    */
    #region WAYPOINT COMMAND
    public class WAYPOINT_CMD : Update<WaypointCMDRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public WAYPOINT_CMD()
            : base(new WaypointCMDRequest())
        {
        }

        public WAYPOINT_CMD(RoboCup2009GUIState gps)
            : base(new WaypointCMDRequest(gps))
        {
        }
    }

    [DataContract]
    public class WaypointCMDRequest
    {
        public WaypointCMDRequest()
        {
        }

        public WaypointCMDRequest(RoboCup2009GUIState gps)
        {
            _gps = gps;
        }

        private RoboCup2009GUIState _gps;

        [DataMember]
        public RoboCup2009GUIState GPS
        {
            get { return _gps; }
            set { _gps = value; }
        }

    }
    #endregion

    #region CONNECTED MESSAGE
    public class CONNECTED_MSG : Update<ConnectedMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {

        public CONNECTED_MSG()
            : base(new ConnectedMSGRequest())
        {
        }
    }

    [DataContract]
    public class ConnectedMSGRequest
    {
        public ConnectedMSGRequest()
        {
        }
    }
        #endregion

    #region STARTPOSES MESSAGE
    public class START_POSES_MSG : Update<StartPosesMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {

        public START_POSES_MSG()
            : base(new StartPosesMSGRequest())
        {
        }
    }

    [DataContract]
    public class StartPosesMSGRequest
    {
        public StartPosesMSGRequest()
        {
            _startPoses = new List<StartPoses>();
        }

        private List<StartPoses> _startPoses;

        [DataMember]
        public List<StartPoses> startPoses
        {
            get { return _startPoses; }
            set { _startPoses = value; }
        }
    }
    #endregion

    #region INITIALIZED MESSAGE
    public class INITIALIZED_MSG : Update<InitializedMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {

        public INITIALIZED_MSG()
            : base(new InitializedMSGRequest())
        {
        }
    }

    [DataContract]
    public class InitializedMSGRequest
    {
        public InitializedMSGRequest()
        {
        }
    }
    #endregion

    #region IMAGE MESSAGE
    public class IMAGE_MSG : Update<ImageMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public IMAGE_MSG()
            : base(new ImageMSGRequest())
        {
        }

        public IMAGE_MSG(byte [] ImgBuffer)
            : base(new ImageMSGRequest(ImgBuffer))
        {
        }
    }

    [DataContract]
    public class ImageMSGRequest
    {
        public ImageMSGRequest()
        {
            _valid = false;
        }

        public ImageMSGRequest(byte [] imgBuffer)
        {
            _imgBuffer = imgBuffer;
            _valid = true;
        }

        private byte[] _imgBuffer;
        private bool _valid;

        [DataMember]
        public byte [] ImgBuffer
        {
            get { return _imgBuffer; }
            set { _imgBuffer = value; }
        }

        [DataMember]
        public bool Valid
        {
            get { return _valid; }
            set { _valid = value; }
        }
    }
    #endregion

    #region ROBOT TYPE MESSAGE
    public class RTYPE_MSG : Update<RTypeMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public RTYPE_MSG()
            : base(new RTypeMSGRequest())
        {
        }

        public RTYPE_MSG(int RobotNumber, string RobotType)
            : base(new RTypeMSGRequest(RobotNumber, RobotType))
        {
        }
    }

    [DataContract]
    public class RTypeMSGRequest
    {
        public RTypeMSGRequest()
        {
        }

        public RTypeMSGRequest(int robotNumber, string robotType)
        {
            _robotNumber = robotNumber;
            _robotType = robotType;
        }

        private int _robotNumber;
        private string _robotType;

        [DataMember]
        public int RobotNumber
        {
            get { return _robotNumber; }
            set { _robotNumber = value; }
        }

        [DataMember]
        public string RobotType
        {
            get { return _robotType; }
            set { _robotType = value; }
        }
    }
    #endregion

    #region ROBOTS SIGNAL STRENGTH MESSAGE
    public class RSS_MSG : Update<RSSMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public RSS_MSG()
            : base(new RSSMSGRequest())
        {
        }

        public RSS_MSG(int RobotNumber, float [] SS, int batteryLife)
            : base(new RSSMSGRequest(RobotNumber, SS, batteryLife))
        {
        }
    }

    [DataContract]
    public class RSSMSGRequest
    {
        public RSSMSGRequest()
        {
        }

        public RSSMSGRequest(int robotNumber, float [] SS, int batteryLife)
        {
            _robotNumber = robotNumber;
            _ss = SS;
            _batteryLife = batteryLife;
        }

        private int _robotNumber;
        private float [] _ss;
        private int _batteryLife;

        [DataMember]
        public int RobotNumber
        {
            get { return _robotNumber; }
            set { _robotNumber = value; }
        }

        [DataMember]
        public float [] SignalStrength
        {
            get { return _ss; }
            set { _ss = value; }
        }

        [DataMember]
        public int BatteryLife
        {
            get { return _batteryLife; }
            set { _batteryLife = value; }
        }
    }
    #endregion

    #region STARTPOSES MESSAGE
    public class MAP_UPDATE_MSG : Update<MapUpdateMSGRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {

        public MAP_UPDATE_MSG()
            : base(new MapUpdateMSGRequest())
        {
        }
    }

    [DataContract]
    public class MapUpdateMSGRequest
    {
        public MapUpdateMSGRequest()
        {
            _xLow=0;
            _yLow=0;
            _resolution=0;
            _xPos=0;
            _yPos=0;
            _mapInfo = new List<MOASTMapInfo>();
        }

        private int _robotNumber;
        private double _xLow;
        private double _yLow;
        private double _resolution;
        private float _xPos;
        private float _yPos;
        private List<MOASTMapInfo> _mapInfo;

        [DataMember]
        public int robotNumber
        {
            get { return _robotNumber; }
            set { _robotNumber = value; }
        }

        [DataMember]
        public double xLow
        {
            get { return _xLow; }
            set { _xLow = value; }
        }

        [DataMember]
        public double yLow
        {
            get { return _yLow; }
            set { _yLow = value; }
        }

        [DataMember]
        public double resolution
        {
            get { return _resolution; }
            set { _resolution = value; }
        }

        [DataMember]
        public float xPos
        {
            get { return _xPos; }
            set { _xPos = value; }
        }

        [DataMember]
        public float yPos
        {
            get { return _yPos; }
            set { _yPos = value; }
        }

        [DataMember]
        public List<MOASTMapInfo> mapInfo
        {
            get { return _mapInfo; }
            set { _mapInfo = value; }
        }
    }
    #endregion


    #endregion
}
