//-----------------------------------------------------------------------
//  This file is part of the UC Merced Microsoft Robotics Developer Studio 
// Project Code Base.
//
//  Created by Benjamin Balaguer under the supervision of Stefano Carpin.
//
//  $File: Sensors.cs $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Microsoft.Dss.Core.Attributes;
using System.Drawing;


namespace Microsoft.Robotics.Services.RoboCup2009GUI
{
    class Sensors
    {
    }

    #region MOAST Map Information
    [DataContract]
    public class MOASTMapInfo
    {
        #region Variables;
        private float _prob;
        private double _xPos;
        private double _yPos;
        #endregion

        #region Constructor
        public MOASTMapInfo()
        {
            _prob = 0;
            _xPos = 0;
            _yPos = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public float Prob
        {
            get { return _prob; }
            set { _prob = value; }
        }

        [DataMember]
        public double xPos
        {
            get { return _xPos; }
            set { _xPos = value; }
        }

        [DataMember]
        public double yPos
        {
            get { return _yPos; }
            set { _yPos = value; }
        }
        #endregion
    }
    #endregion


    #region Robot Map Information
    [DataContract]
    public class RobotMapInfo
    {
        #region Variables
        private bool _initialized;
        private float _xPos;
        private float _yPos;
        public Graphics GraphicsMap;
        public Bitmap BitmapMap;
        private int _mapRegionXlow;
        private int _mapRegionYlow;
        private int _mapRegionXhigh;
        private int _mapRegionYhigh;
        #endregion

        #region Constructor
        public RobotMapInfo()
        {
            _xPos = 0;
            _yPos = 0;

            _mapRegionXlow = 1001;
            _mapRegionYlow = 1001;
            _mapRegionXhigh = -1;
            _mapRegionYhigh = -1;

            BitmapMap = new Bitmap(1001, 1001);
            GraphicsMap = Graphics.FromImage(BitmapMap);
            GraphicsMap.Clear(Color.Transparent);
        }
        #endregion

        public void UpdateMap(List<MOASTMapInfo> mapInfo, float mapXPosInitial, float mapYPosInitial, double mapResolution, Color clearedColor)
        {
            int row, col;
            SolidBrush myBrush = new SolidBrush(Color.FromArgb(0, 0, 0));

            for (int i = 0; i < mapInfo.Count; i++)
            {
                // Get row and column from x and y position
                row = (int)Math.Round((mapInfo[i].xPos - mapXPosInitial) / mapResolution) + 500;
                col = (int)Math.Round((mapInfo[i].yPos - mapYPosInitial) / mapResolution) + 500;

                if ((row-1) < _mapRegionXlow)
                {
                    _mapRegionXlow = row-1;
                    if (_mapRegionXlow < 0)
                        _mapRegionXlow = 0;
                }
                if ((row+1) > _mapRegionXhigh)
                {
                    _mapRegionXhigh = (row+1);
                    if (_mapRegionXhigh > 1000)
                        _mapRegionXhigh = 1000;
                }
                if ((col-1) < _mapRegionYlow)
                {
                    _mapRegionYlow = (col-1);
                    if (_mapRegionYlow < 0)
                        _mapRegionYlow = 0;
                }
                if ((col+1) > _mapRegionYhigh)
                {
                    _mapRegionYhigh = (col+1);
                    if (_mapRegionYhigh > 1000)
                        _mapRegionYhigh = 1000;
                }

                //System.Console.WriteLine("Row: " + row + " / Col: " + col);

                // Update the map
                if (mapInfo[i].Prob < 0.5)
                    // Probabilistic
                    //myBrush.Color = Color.FromArgb(0, Convert.ToInt32((1 - val) * 255), 0);
                    // Deterministic
                    myBrush.Color = clearedColor;
                else
                    // Probabilistic
                    //myBrush.Color = Color.FromArgb(Convert.ToInt32((1 - val) * 255), Convert.ToInt32((1 - val) * 255), Convert.ToInt32((1 - val) * 255));
                    // Deterministic
                    myBrush.Color = Color.FromArgb(0, 0, 0);

                GraphicsMap.FillRectangle(myBrush, row, col, 1, 1); 
            }

            // Deallocate Memory
            myBrush = null;
            mapInfo = null;
            //System.GC.Collect();
        }

        #region Data Members
        
        [DataMember]
        public bool Initialized
        {
            get { return _initialized; }
            set { _initialized = value; }
        }
        
        [DataMember]
        public float xPos
        {
            get { return _xPos; }
            set { _xPos = value; }
        }

        [DataMember]
        public float yPos
        {
            get { return _yPos; }
            set { _yPos = value; }
        }

        [DataMember]
        public int MapRegionXlow
        {
            get { return _mapRegionXlow; }
            set { _mapRegionXlow = value; }
        }

        [DataMember]
        public int MapRegionYlow
        {
            get { return _mapRegionYlow; }
            set { _mapRegionYlow = value; }
        }

        [DataMember]
        public int MapRegionXhigh
        {
            get { return _mapRegionXhigh; }
            set { _mapRegionXhigh = value; }
        }

        [DataMember]
        public int MapRegionYhigh
        {
            get { return _mapRegionYhigh; }
            set { _mapRegionYhigh = value; }
        }
        #endregion
    }
    #endregion


    # region GPS Sensor
    [DataContract]
    public class GPS
    {
        #region Variables
        private float _timeStamp;
        private string _name;
        private int _latitudeDegrees;
        private float _latitudeMinutes;
        private char _latitudeCardinal;
        private int _longitudeDegrees;
        private float _longitudeMinutes;
        private char _longitudeCardinal;
        private int _fix;
        private int _satellites;
        #endregion

        # region Constructor
        public GPS()
        {
            _timeStamp = 0;
            _name = "";
            _latitudeDegrees = 0;
            _latitudeMinutes = 0;
            _latitudeCardinal = 'Q';
            _longitudeDegrees = 0;
            _longitudeMinutes = 0;
            _longitudeCardinal = 'Q';
            _fix = 0;
            _satellites = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public float TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public int LatitudeDegrees
        {
            get { return _latitudeDegrees; }
            set { _latitudeDegrees = value; }
        }

        [DataMember]
        public float LatitudeMinutes
        {
            get { return _latitudeMinutes; }
            set { _latitudeMinutes = value; }
        }

        [DataMember]
        public char LatitudeCardinal
        {
            get { return _latitudeCardinal; }
            set { _latitudeCardinal = value; }
        }

        [DataMember]
        public int LongitudeDegrees
        {
            get { return _longitudeDegrees; }
            set { _longitudeDegrees = value; }
        }

        [DataMember]
        public float LongitudeMinutes
        {
            get { return _longitudeMinutes; }
            set { _longitudeMinutes = value; }
        }

        [DataMember]
        public char LongitudeCardinal
        {
            get { return _longitudeCardinal; }
            set { _longitudeCardinal = value; }
        }

        [DataMember]
        public int Fix
        {
            get { return _fix; }
            set { _fix = value; }
        }

        [DataMember]
        public int Satellites
        {
            get { return _satellites; }
            set { _satellites = value; }
        }
        #endregion
    }
    #endregion

    #region GroundTruth Sensor
    [DataContract]
    public class GroundTruth
    {
        #region Variables
        private float _timeStamp;
        private string _name;
        private float _xPos;
        private float _yPos;
        private float _zPos;
        private float _xRot;
        private float _yRot;
        private float _zRot;
        #endregion

        #region Constructor
        public GroundTruth()
        {
            _timeStamp = 0;
            _name = "";
            _xPos = 0;
            _yPos = 0;
            _zPos = 0;
            _xRot = 0;
            _yRot = 0;
            _zRot = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public float TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }
        
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public float xPos
        {
            get { return _xPos; }
            set { _xPos = value; }
        }

        [DataMember]
        public float yPos
        {
            get { return _yPos; }
            set { _yPos = value; }
        }

        [DataMember]
        public float zPos
        {
            get { return _zPos; }
            set { _zPos = value; }
        }

        [DataMember]
        public float xRot
        {
            get { return _xRot; }
            set { _xRot = value; }
        }

        [DataMember]
        public float yRot
        {
            get { return _yRot; }
            set { _yRot = value; }
        }

        [DataMember]
        public float zRot
        {
            get { return _zRot; }
            set { _zRot = value; }
        }
        #endregion
    }
    #endregion

    #region Mission Package Link Information
    [DataContract]
    public class LinkProperty
    {
        #region Variables
        // Used for MISSTA Message
        private string _type;
        private float _maxSpeed;
        private float _maxTorque;
        private float _minRange;
        private float _maxRange;
        private float _currVal;
        private float _currTorque;

        // Used for MISPKG Command
        private float _desiredVal;
        private int _desiredOrder;

        // Used for both MISSTA Message and MISPKG Command
        private int _id;
        #endregion

        #region Constructor
        public LinkProperty()
        {
            _id = 0;
            _type = "";
            _maxSpeed = 0;
            _maxTorque = 0;
            _minRange = 0;
            _maxRange = 0;
            _currVal = 0;
            _currTorque = 0;
            _desiredVal = 0;
            _desiredOrder = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public int ID
        {
            get { return _id; }
            set { _id = value; }
        }

        [DataMember]
        public string Type
        {
            get { return _type; }
            set { _type = value; }
        }

        [DataMember]
        public float MaxSpeed
        {
            get { return _maxSpeed; }
            set { _maxSpeed = value; }
        }

        [DataMember]
        public float MaxTorque
        {
            get { return _maxTorque; }
            set { _maxTorque = value; }
        }

        [DataMember]
        public float MinRange
        {
            get { return _minRange; }
            set { _minRange = value; }
        }

        [DataMember]
        public float MaxRange
        {
            get { return _maxRange; }
            set { _maxRange = value; }
        }

        [DataMember]
        public float CurrVal
        {
            get { return _currVal; }
            set { _currVal = value; }
        }

        [DataMember]
        public float CurrTorque
        {
            get { return _currTorque; }
            set { _currTorque = value; }
        }

        [DataMember]
        public float DesiredVal
        {
            get { return _desiredVal; }
            set { _desiredVal = value; }
        }

        [DataMember]
        public int DesiredOrder
        {
            get { return _desiredOrder; }
            set { _desiredOrder = value; }
        }
        #endregion
    }
    #endregion

    #region Mission Package Sensor
    [DataContract]
    public class MisStatus
    {
        #region Variables
        private string _name;
        private float _timeStamp;
        private List<LinkProperty> _link;
        #endregion

        #region Constructor
        public MisStatus()
        {
            _name = "";
            _timeStamp = 0; ;
            _link = new List<LinkProperty>();
        }
        #endregion

        #region Data Members
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public float TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        [DataMember]
        public List<LinkProperty> Link
        {
            get { return _link; }
            set { _link = value; }
        }
        #endregion
    }
    #endregion

    #region Start Poses Information
    [DataContract]
    public class StartPoses
    {
        #region Variables
        private string _name;
        private float _xPos;
        private float _yPos;
        private float _zPos;
        private float _xRot;
        private float _yRot;
        private float _zRot;
        #endregion

        #region Constructor
        public StartPoses()
        {
            _xPos = 0;
            _yPos = 0;
            _zPos = 0;
            _xRot = 0;
            _yRot = 0;
            _zRot = 0;
            _name = "";
        }
        #endregion

        #region Data Members
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [DataMember]
        public float xPos
        {
            get { return _xPos; }
            set { _xPos = value; }
        }

        [DataMember]
        public float yPos
        {
            get { return _yPos; }
            set { _yPos = value; }
        }

        [DataMember]
        public float zPos
        {
            get { return _zPos; }
            set { _zPos = value; }
        }

        [DataMember]
        public float xRot
        {
            get { return _xRot; }
            set { _xRot = value; }
        }

        [DataMember]
        public float yRot
        {
            get { return _yRot; }
            set { _yRot = value; }
        }

        [DataMember]
        public float zRot
        {
            get { return _zRot; }
            set { _zRot = value; }
        }
        #endregion
    }
    #endregion

    #region Status Information
    [DataContract]
    public class BotStatus
    {
        #region Variables
        private string _type;
        private float _timeStamp;
        private bool _lightOn;
        private float _lightIntensity;
        private int _battery;
        private int _viewNum;
        #endregion

        #region Constructor
        public BotStatus()
        {
            _type = "";
            _timeStamp = 0;
            _lightOn = false;
            _lightIntensity = 0;
            _battery = 0;
            _viewNum = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public string Type
        {
            get { return _type; }
            set { _type = value; }
        }

        [DataMember]
        public float TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        [DataMember]
        public bool LightOn
        {
            get { return _lightOn; }
            set { _lightOn = value; }
        }

        [DataMember]
        public float LightIntensity
        {
            get { return _lightIntensity; }
            set { _lightIntensity = value; }
        }

        [DataMember]
        public int Battery
        {
            get { return _battery; }
            set { _battery = value; }
        }

        [DataMember]
        public int ViewNum
        {
            get { return _viewNum; }
            set { _viewNum = value; }
        }
        #endregion
    }
    #endregion

    # region Victim Location
    [DataContract]
    public class VictimLoc
    {
        #region Variables
        private int _row;
        private int _col;
        private float _xPos;
        private float _yPos;
        #endregion

        # region Constructor
        public VictimLoc()
        {
            _row = 0;
            _col = 0;
            _xPos = 0;
            _yPos = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public int Row
        {
            get { return _row; }
            set { _row = value; }
        }

        [DataMember]
        public int Col
        {
            get { return _col; }
            set { _col = value; }
        }

        [DataMember]
        public float xPos
        {
            get { return _xPos; }
            set { _xPos = value; }
        }

        [DataMember]
        public float yPos
        {
            get { return _yPos; }
            set { _yPos = value; }
        }
        #endregion
    }
    #endregion

    # region Victim Location
    [DataContract]
    public class Point
    {
        #region Variables
        private float _xPos;
        private float _yPos;
        #endregion

        # region Constructor
        public Point()
        {
            _xPos = 0;
            _yPos = 0;
        }
        #endregion

        #region Data Members
        [DataMember]
        public float X
        {
            get { return _xPos; }
            set { _xPos = value; }
        }

        [DataMember]
        public float Y
        {
            get { return _yPos; }
            set { _yPos = value; }
        }
        #endregion
    }
    #endregion
}
