﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Microsoft.Robotics.Services.RoboCup2009GUI
{
    public partial class VictimForm : Form
    {
        StreamWriter fMIFVictim;
        StreamWriter fMIDVictim;
        private float mapXPosInitial;
        private float mapYPosInitial;
        private double mapResolution;
        private int victimCount = 0;
        private float victimX = 0;
        private float victimY = 0;
        private static Object jpegImageLock = new Object();
        private Image jpegImage;

        public VictimForm(float mXPosInitial, float mYPosInitial, double mResolution)
        {
            InitializeComponent();

            mapXPosInitial = mXPosInitial;
            mapYPosInitial = mYPosInitial;
            mapResolution = mResolution;

            fMIFVictim = new StreamWriter("RoboCup/Victims.mif");

            cBoxAccessibility.SelectedIndex = 0;
            cBoxCondition.SelectedIndex = 0;
            cBoxPose.SelectedIndex = 0;
            cBoxPriority.SelectedIndex = 0;
            cBoxSex.SelectedIndex = 0;
            cBoxState.SelectedIndex = 0;

            for (int i = 0; i < chkListFeatures.Items.Count; i++)
                chkListFeatures.SetItemChecked(i, false);

            btnPicture.Enabled = true;
            btnDone.Enabled = false;

            #region Write MIF file header
            float MinX = (float)((1 - 500) * mapResolution) + mapYPosInitial;
            float MinY = (float)((1 - 500) * mapResolution) + mapXPosInitial;
            float MaxX = (float)((1001 - 500) * mapResolution) + mapYPosInitial;
            float MaxY = (float)((1001 - 500) * mapResolution) + mapXPosInitial;
            fMIFVictim.WriteLine("Version 450");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("Charset \"Neutral\"");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("Delimiter \",\"");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("CoordSys NonEarth Units \"m\" Bounds (" + MinX + ", " + MinY + ") (" + MaxX + ", " + MaxY + ")");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("Columns 8");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_ID char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_Sex char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_State char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_Condition char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_Pose char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_Accessibility char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_Priority char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("  Victim_Features char(254)");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("Data");
            fMIFVictim.Flush();
            fMIFVictim.WriteLine();
            fMIFVictim.Flush();
            #endregion
        }

        public void newVictim(float XPos, float YPos)
        {
            //Reset();
            victimX = XPos;
            victimY = YPos;
            victimCount++;
            this.Show();
            this.Visible = true;
        }
       
        public void Reset()
        {
            lock (jpegImageLock)
            {
                pBoxWebcam.Image.Dispose();
                pBoxWebcam.Image = null;
                cBoxAccessibility.SelectedIndex = 0;
                cBoxCondition.SelectedIndex = 0;
                cBoxPose.SelectedIndex = 0;
                cBoxPriority.SelectedIndex = 0;
                cBoxSex.SelectedIndex = 0;
                cBoxState.SelectedIndex = 0;

                for (int i = 0; i < chkListFeatures.Items.Count; i++)
                    chkListFeatures.SetItemChecked(i, false);

                btnPicture.Enabled = true;
                btnDone.Enabled = false;
            }
        }

        private void btnDone_Click(object sender, EventArgs e)
        {
            fMIFVictim.WriteLine("POINT" + " " + victimY + " " + victimX);
            fMIFVictim.Flush();
            fMIFVictim.WriteLine("SYMBOL 46 16711680 20");
            fMIFVictim.Flush();

            fMIDVictim = new StreamWriter("RoboCup/Victims.mid", true);
            fMIDVictim.Write("Victim" + victimCount + ",\"" + cBoxSex.Text + "\",\"" + cBoxState.Text + "\",\"" + cBoxCondition.Text + "\",\"" + cBoxPose.Text + "\",\"" + cBoxAccessibility.Text + "\",\"" + cBoxPriority.Text + "\",\"");

            for (int i = 0; i < chkListFeatures.CheckedIndices.Count; i++)
            {
                fMIDVictim.Write(chkListFeatures.Items[chkListFeatures.CheckedIndices[i]]);
                if (i != chkListFeatures.CheckedIndices.Count - 1)
                    fMIDVictim.Write(", ");
            }
            
            fMIDVictim.Write("\"\n");
            fMIDVictim.Flush();
            fMIDVictim.Close();

            Reset();
            this.Hide();
            this.Visible = false;
        }

        private void btnPicture_Click(object sender, EventArgs e)
        {
            lock (jpegImageLock)
            {
                Bitmap bmp = new Bitmap(jpegImage);
                Graphics g = Graphics.FromImage(bmp);
                Bitmap bmpNew = new Bitmap(bmp);
                g.DrawImage(bmpNew, new System.Drawing.Point(0, 0));

                bmpNew.Save("RoboCup/Victim" + victimCount + ".jpeg", System.Drawing.Imaging.ImageFormat.Jpeg);

                // Deallocate memory
                g.Dispose();
                bmp.Dispose();
                bmpNew.Dispose();
                bmp = null;
                g = null;
                bmpNew = null;

                btnPicture.Enabled = false;
                btnDone.Enabled = true;
            }
        }

        public void UpdateImage(Stream imageStream)
        {
            lock (jpegImageLock)
            {
                if (btnPicture.Enabled)
                {
                    jpegImage = new Bitmap(imageStream);
                    pBoxWebcam.Image = jpegImage;
                }
            }
        }
    }
}
