//-----------------------------------------------------------------------
//  This file is part of the UC Merced Microsoft Robotics Developer Studio 
// Project Code Base.
//
//  Created by Benjamin Balaguer under the supervision of Stefano Carpin.
//  Based on DriveControl.cs from UX\SimpleDashboard.
//
//  $File: DriveControl.cs $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Ccr.Core;
using cs = Microsoft.Dss.Services.Constructor;
using Microsoft.Dss.ServiceModel.Dssp;
using System.Drawing.Drawing2D;
using System.IO;
using Microsoft.Dss.Core;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Media.Imaging;
using System.Xml;
using System.Configuration;
using System.Net.Configuration;
using System.Reflection;

namespace Microsoft.Robotics.Services.GroupingGUI
{
    partial class DriveControl : Form
    {
        DriveControlEvents _eventsPort;
        GroupingGUIOperations _mainPort;
        GroupingGUIState _state = new GroupingGUIState();

        // Map Variables
        private Bitmap bmpMap;
        private Graphics gMap;
        private float resolution = 0.1f;

        public Image jpegImage;

        // Logging information
        private StreamWriter fMIF;
        private StreamWriter fMID;

        // Store Points for Grouping
        private List<Point> GroupingPoints = new List<Point>();
        private List<Point> allPoints = new List<Point>();
        private int GroupedObjects = 0;

        private Image finalImage = null;
        private Graphics CombinedMapGraphics = null;

        string imgFile;

        // To Center the Map
        private float XPosInitial;
        private float YPosInitial;

        public DriveControl(DriveControlEvents EventsPort, GroupingGUIOperations MainPort)
        {
            DriveControl.CheckForIllegalCrossThreadCalls = false;
            _eventsPort = EventsPort; // Initialize the events port
            _mainPort = MainPort;     // Initialize the main port

            // Read Configuration File
            ReadConfigFile("samples/Config/GroupingGUI.xml");

            // Store MID file for grouping
            fMIF = new StreamWriter("RoboCup/Skeleton.mif");

            // Store MIF file for grouping
            fMID = new StreamWriter("RoboCup/Skeleton.mid");

            StreamReader iniPose = new StreamReader("RoboCup/INIPOSE.txt");

            string line = iniPose.ReadLine();

            XPosInitial = Convert.ToSingle(line.Substring(0, line.IndexOf(",")));
            line = line.Remove(0, line.IndexOf(",") + 1);
            YPosInitial = Convert.ToSingle(line.Substring(0, line.IndexOf(",")));

            // Write MIF file header 
            float MinX = (float)((1 - 500) * 0.1) + YPosInitial;
            float MinY = (float)((1 - 500) * 0.1) + XPosInitial;
            float MaxX = (float)((1001 - 500) * 0.1) + YPosInitial;
            float MaxY = (float)((1001 - 500) * 0.1) + XPosInitial;
            fMIF.WriteLine("Version 450");
            fMIF.Flush();
            fMIF.WriteLine("Charset \"Neutral\"");
            fMIF.Flush();
            fMIF.WriteLine("Delimiter \",\"");
            fMIF.Flush();
            fMIF.WriteLine("CoordSys NonEarth Units \"m\" Bounds (" + MinX + ", " + MinY + ") (" + MaxX + ", " + MaxY + ")");
            fMIF.Flush();
            fMIF.WriteLine("Columns 2");
            fMIF.Flush();
            fMIF.WriteLine("  ID integer");
            fMIF.Flush();
            fMIF.WriteLine("  Feature_Type char(254)");
            fMIF.Flush();
            fMIF.WriteLine("Data");
            fMIF.Flush();
            fMIF.WriteLine();
            fMIF.Flush();

            bmpMap = new Bitmap(imgFile);
            bmpMap.RotateFlip(RotateFlipType.Rotate90FlipNone);
            gMap = Graphics.FromImage(bmpMap);

            InitializeComponent();
        }

        private void ReadConfigFile(string FileToRead)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(FileToRead);

            imgFile = ((XmlElement)doc.GetElementsByTagName("ImgFile")[0]).InnerText;
        }

        public void CloseForm()
        {
            System.Console.WriteLine("Form Closed.");

            fMIF.Close();
            fMID.Close();
        }

        private void DriveControl_Load(object sender, EventArgs e)
        {
            _eventsPort.Post(new OnLoad(this));
            cBoxZom.SelectedIndex = cBoxZom.FindString("100%");

            finalImage = new Bitmap(bmpMap, (int)(bmpMap.Width), (int)(bmpMap.Height));
            CombinedMapGraphics = Graphics.FromImage(finalImage);
            pBoxMap.Image = finalImage;
        }

        private void DriveControl_FormClosed(object sender, FormClosedEventArgs e)
        {
            _eventsPort.Post(new OnClosed(this));
        }

        private void pBoxMap_Click(object sender, EventArgs e)
        {
            // Take into account the zoom
            //string percent = cBoxZom.Items[cBoxZom.SelectedIndex].ToString();
            //float factor = Convert.ToInt32(percent.Substring(0, percent.Length - 1)) / 100;

            // Find the row and column clicked (based on the zoom)
            if (!btnStartGrouping.Enabled)
            {
                int row = (int)((pBoxMap.OffsetX + ((MouseEventArgs)e).X));
                int col = (int)((pBoxMap.OffsetY + ((MouseEventArgs)e).Y));

                // Calculate the X and Y Position, in meters
                float XPos = ((row - 500) * 0.1f) + XPosInitial;
                float YPos = ((col - 500) * 0.1f) + YPosInitial;

                // Look to see where the point is
                float minVal = 10000000;
                float newVal = 0;
                int index = -1;
                for (int i = 0; i < allPoints.Count; i++)
                {
                    newVal = Convert.ToSingle(Math.Sqrt(Convert.ToSingle(Math.Pow(allPoints[i].X - XPos, 2)) + Convert.ToSingle(Math.Pow(allPoints[i].Y - YPos, 2))));
                    if (newVal < minVal)
                    {
                        minVal = newVal;
                        index = i;
                    }
                }

                if (minVal < 0.5f)
                {
                    System.Console.WriteLine("Found a old point");
                    XPos = allPoints[index].X;
                    YPos = allPoints[index].Y;
                    index = -2;
                }

                Point aPoint = new Point();
                aPoint.X = XPos;
                aPoint.Y = YPos;
                GroupingPoints.Add(aPoint);

                if (index != -2)
                {
                    allPoints.Add(aPoint);
                    SolidBrush myBrush = new SolidBrush(Color.AntiqueWhite);
                    CombinedMapGraphics.FillRectangle(myBrush, row, col, 3, 3);
                }
                txtPointsClicked.Text = Convert.ToString(GroupingPoints.Count);

                pBoxMap.Image = finalImage;
            }
        }

        private void btnStartGrouping_Click(object sender, EventArgs e)
        {
            txtPointsClicked.Text = "0";
            lBoxGrouping.SelectedIndex = 0;
            btnStartGrouping.Enabled = false;
            btnStopGrouping.Enabled = true;
            lBoxGrouping.Enabled = true;
            txtPointsClicked.Enabled = true;
        }

        private void btnStopGrouping_Click(object sender, EventArgs e)
        {
            fMIF.WriteLine("PLINE");
            fMIF.Flush();
            fMIF.WriteLine("  " + GroupingPoints.Count);
            fMIF.Flush();
            for (int i = 0; i < GroupingPoints.Count; i++)
            {
                fMIF.WriteLine(GroupingPoints[i].Y + " " + GroupingPoints[i].X);
                fMIF.Flush();
            }

            fMIF.WriteLine("PEN 2 65 16777215");
            fMIF.Flush();
            fMIF.WriteLine();
            fMIF.Flush();
            fMID.WriteLine(1 + ",\"\"");
            fMID.Flush();

            for (int i = 0; i < GroupingPoints.Count; i++)
            {
                fMIF.WriteLine("POINT" + " " + GroupingPoints[i].Y + " " + GroupingPoints[i].X);
                fMIF.Flush();
                fMIF.WriteLine("SYMBOL 46 16711935 7");
                fMIF.Flush();

                fMID.WriteLine(1 + ",\"\"");
                fMID.Flush();
            }

            GroupingPoints.Clear();

            lBoxGrouping.Enabled = false;
            txtPointsClicked.Enabled = false;
            btnStopGrouping.Enabled = false;
            btnStartGrouping.Enabled = true;
        }

        private void cBoxZom_SelectedIndexChanged(object sender, EventArgs e)
        {
            int oldVSize = 0, oldHSize = 0, oldVVal = 0, oldHVal = 0;
            bool changeScroll;

            // Create the image, which is possibly zoomed
            string percent = cBoxZom.Items[cBoxZom.SelectedIndex].ToString();
            float factor = Convert.ToInt32(percent.Substring(0, percent.Length - 1)) / 100;
            Image image = new Bitmap(bmpMap, (int)(bmpMap.Width * factor), (int)(bmpMap.Height * factor));
            pBoxMap.Image = image;

            // If we have zoomed in or out
            if ((changeScroll = (pBoxMap.Enabled && pBoxMap.Image.Width != image.Width && pBoxMap.Image.Height != image.Height)))
            {
                // Save some old values about the scrollable image
                oldVSize = pBoxMap.Image.Height;
                oldHSize = pBoxMap.Image.Width;
                oldVVal = pBoxMap.vScrollBar1.Value;
                oldHVal = pBoxMap.hScrollBar1.Value;
            }

            // If we have zoomed in or out
            if (changeScroll)
            {
                // Change the scroller values and image offset to keep image "centered"
                pBoxMap.vScrollBar1.Value = (int)(oldVVal * (float)((float)(pBoxMap.Image.Height - 477) / (float)(oldVSize - 477)));
                pBoxMap.hScrollBar1.Value = (int)(oldHVal * (float)((float)(pBoxMap.Image.Width - 477) / (float)(oldHSize - 477)));
                pBoxMap.OffsetY = pBoxMap.vScrollBar1.Value;
                pBoxMap.OffsetX = pBoxMap.hScrollBar1.Value;
            }
        }
    }

    class DriveControlEvents 
        : PortSet<
            OnLoad,
            OnClosed
        >
    {
    }
    
    class DriveControlEvent
    {
        private DriveControl _driveControl;

        public DriveControl DriveControl
        {
            get { return _driveControl; }
            set { _driveControl = value; }
        }

        public DriveControlEvent(DriveControl driveControl)
        {
            _driveControl = driveControl;
        }
    }

    class OnLoad : DriveControlEvent
    {
        public OnLoad(DriveControl form)
            : base(form)
        {
        }
    }

    class OnConnect : DriveControlEvent
    {
        string _service;

        public string Service
        {
            get { return _service; }
            set { _service = value; }
        }

        public OnConnect(DriveControl form, string service)
            : base(form)
        {
            _service = service;
        }
    }
    
    class OnClosed : DriveControlEvent
    {
        public OnClosed(DriveControl form)
            : base(form)
        {
        }
    }
}
