AC_DEFUN([AC_HAVE_RCSLIB],
	[AC_MSG_CHECKING([for RCS library])]
	RCSLIB_DIR=`echo $RCSLIB_DIR`
	if test x$RCSLIB_DIR = x ; then
	for dir in /usr/local/rcslib /usr/rcslib /local/rcslib ; do
	if test -d $dir -o -L $dir ; then RCSLIB_DIR=$dir ; break ; fi
	done
	fi
	if test x$RCSLIB_DIR = x ; then
	[AC_MSG_ERROR([not found, set RCSLIB_DIR environment variable and try again])]
	else
	[AC_MSG_RESULT([$RCSLIB_DIR])]
	RCSLIB_CFLAGS="-I$RCSLIB_DIR/include"
	RCSLIB_LIBS="-L$RCSLIB_DIR/lib -lrcs -lposemath"
dnl put HAVE_RCSLIB in config.h
	[AC_DEFINE(HAVE_RCSLIB,
		1, [Define non-zero if you have the RCS Library.])]
dnl add needed libraries
	[AC_CHECK_LIB(socket, main)]
	[AC_CHECK_LIB(wsock32, main)]
	[AC_CHECK_LIB(rt, main)]
	[AC_CHECK_LIB(posix4, main)]
	[AC_CHECK_LIB(nsl, main)]
dnl put RCSLIB_DIR in Makefile
	[AC_SUBST(RCSLIB_DIR)]
	[AC_SUBST(RCSLIB_CFLAGS)]
	[AC_SUBST(RCSLIB_LIBS)]
dnl put RCSLIB_DIR into variable file for use by shell scripts
	echo RCSLIB_DIR=$RCSLIB_DIR > rcslib_dir
	fi
	)

AC_DEFUN([AC_HAVE_MYSQL],
# look for <mysql/mysql.h>, but define HAVE_MYSQL_H instead of
# HAVE_MYSQL_MYSQL_H, since code will include <mysql.h> and CFLAGS will
# specify something like -I/usr/include/mysql
	[AC_CHECK_HEADER([mysql/mysql.h], [AC_DEFINE(HAVE_MYSQL_H, 1, Define to 1 if you have the <mysql.h> header file.)] mysql=true, mysql=false)]
	[AM_CONDITIONAL(HAVE_MYSQL_H, test x$mysql = xtrue)]
# check for mysql_config, which if it exists will return CFLAGS and LIBS
	[AC_CHECK_PROGS(MYSQL_CONFIG, mysql_config)]
	[AM_CONDITIONAL(HAVE_MYSQL, test x$MYSQL_CONFIG != x)]
	if test x$MYSQL_CONFIG = x ; then
		MYSQL_CFLAGS=""
		MYSQL_LIBS=""
	else
		MYSQL_CFLAGS=`mysql_config --cflags`
		MYSQL_LIBS=`mysql_config --libs`
	fi
	[AC_SUBST(MYSQL_CFLAGS)]
	[AC_SUBST(MYSQL_LIBS)]
	)

AC_DEFUN([AC_HAVE_GTK],
	[AC_CHECK_PROGS(PKG_CONFIG, pkg-config)]
	if test x$PKG_CONFIG != x ; then
		[AC_MSG_CHECKING([for gtk+-2.0])]
		GTK_VERSION=`pkg-config --modversion gtk+-2.0`
		if test x$GTK_VERSION != x ; then
			GTK_CFLAGS=`pkg-config --cflags gtk+-2.0`
			GTK_LIBS=`pkg-config --libs gtk+-2.0`
			[AC_MSG_RESULT([$GTK_VERSION])]
		else
			[AC_MSG_RESULT([no])]
		fi
	else
		GTK_VERSION=""
		GTK_CFLAGS=""
		GTK_LIBS=""
	fi
	[AC_SUBST(GTK_VERSION)]
	[AC_SUBST(GTK_CFLAGS)]
	[AC_SUBST(GTK_LIBS)]
	[AM_CONDITIONAL(HAVE_GTK, test x$GTK_VERSION != x)]
	)

AC_DEFUN([AC_CHECK_MATH],
	[AC_CHECK_DECL(floorf,,
		AH_TEMPLATE([floorf], [Math function 'floor' for floats])
		AC_DEFINE([floorf(x)], [((float) floor((double) (x)))]),
		[#include <math.h>])]
	[AC_CHECK_DECL(sqrtf,,
		AH_TEMPLATE([sqrtf], [Math function 'sqrt' for floats])
		AC_DEFINE([sqrtf(x)], [((float) sqrt((double) (x)))]),
		[#include <math.h>])]
	)

AC_DEFUN([ACX_TK_INCLUDE],
    [AC_MSG_CHECKING([for Tk headers])]
    [AC_ARG_WITH(tkinclude,
	    [  --with-tkinclude=<path to Tk headers>  Specify path to Tk headers],
	    dirs=$withval,dirs="/usr/include /usr/include/tcl8.4 /usr/include/tcl8.3")]
    for dir in $dirs ; do
	if test -f $dir/tk.h ; then tkinclude_dir=$dir ; break; fi
    done
    if test x$tkinclude_dir = x ; then
	[AC_MSG_RESULT([not found in $dirs, try --with-tkinclude=<path to Tk headers>])]
    else
	TK_INCLUDE_DIR=$tkinclude_dir
dnl put HAVE_TK_INCLUDE in config.h
	[AC_DEFINE(HAVE_TK_INCLUDE,
		1, [Define non-zero if you have Tk headers.])]
dnl put TK_INCLUDE_DIR in Makefile
	[AC_SUBST(TK_INCLUDE_DIR)]
	[AC_MSG_RESULT([$TK_INCLUDE_DIR])]
    fi
    [AM_CONDITIONAL(HAVE_TK_INCLUDE, test x$tkinclude_dir != x)]
)

AC_DEFUN([ACX_TCL_LIB],
    [AC_MSG_CHECKING([for Tcl library])]
    [AC_ARG_WITH(tcllib,
	    [  --with-tcllib=<path to Tcl library>  Specify path to Tcl library],
	    dirs=$withval,dirs="/usr/lib")]
    [AC_ARG_WITH(tcllink,
	    [  --with-tcllink=<link name of Tcl library>  Specify link name of Tcl library],
	    names=$withval,names="tcl tcl8.4 tcl84 tcl8.3 tcl83")]
    for dir in $dirs ; do
	for name in $names ; do
		if test -f $dir/lib"$name".a ; then tcllink="-L$dir -l$name"; break; fi
		if test -f $dir/lib"$name".so ; then tcllink="-L$dir -l$name"; break; fi
	done
    done
    if test x"$tcllink" = x ; then
	[AC_MSG_RESULT([none of lib"$names".a,so found in $dirs, try --with-tcllib=<path to Tcl library, --with-tcllink=<link name of Tcl library>])]
    else
	TCL_LINK="$tcllink"
dnl put TCL_LINK in Makefile
	[AC_SUBST(TCL_LINK)]
	[AC_MSG_RESULT([$TCL_LINK])]
    fi
    [AM_CONDITIONAL(HAVE_TCL_LIB, test x"$tcllink" != x)]
)

AC_DEFUN([ACX_TK_LIB],
    [AC_MSG_CHECKING([for Tk library])]
    [AC_ARG_WITH(tklib,
	    [  --with-tklib=<path to Tk library>  Specify path to Tk library],
	    dirs=$withval,dirs="/usr/lib")]
    [AC_ARG_WITH(tklink,
	    [  --with-tklink=<link name of Tk library>  Specify link name of Tk library],
	    names=$withval,names="tk tk8.4 tk84 tk8.3 tk83")]
    for dir in $dirs ; do
	for name in $names ; do
		if test -f $dir/lib"$name".a ; then tklink="-L$dir -l$name"; break; fi
		if test -f $dir/lib"$name".so ; then tklink="-L$dir -l$name"; break; fi
	done
    done
    if test x"$tklink" = x ; then
	[AC_MSG_RESULT([none of lib"$names".a,so found in $dirs, try --with-tklib=<path to Tk library, --with-tklink=<link name of Tk library>])]
    else
	TK_LINK="$tklink"
dnl put TK_LINK in Makefile
	[AC_SUBST(TK_LINK)]
	[AC_MSG_RESULT([$TK_LINK])]
    fi
    [AM_CONDITIONAL(HAVE_TK_LIB, test x"$tklink" != x)]
)

AC_DEFUN([AC_HAVE_TCL_DEVEL],
	[AC_CHECK_HEADERS([tcl.h], tcl=true, tcl=false)]
	[AM_CONDITIONAL(HAVE_TCL_DEVEL, test x$tcl = xtrue)]
	)

AC_DEFUN([AC_HAVE_PERL_DEVEL],
	[AC_MSG_CHECKING([for perl devel])]
	dir=""
	if test x$LOCATE != x ; then
	if test x$GREP != x ; then
	for file in `locate perl.h | grep CORE` ; do
		dir=`dirname $file`
	done
	fi
	fi
	if test x$dir = x ; then
		PERL_FLAGS=""
		[AC_MSG_RESULT([no])]
	else
		PERL_FLAGS="-D_GNU_SOURCE -I$dir"
		[AC_MSG_RESULT([$dir])]
	fi
	[AC_SUBST(PERL_FLAGS)]
	[AM_CONDITIONAL(HAVE_PERL_DEVEL, test "x$PERL_FLAGS" != x)]
	)

AC_DEFUN([AC_HAVE_PYTHON_DEVEL],
	[AC_MSG_CHECKING([for python devel])]
	dir=""
	if test x$LOCATE != x ; then
	if test x$GREP != x ; then
	for file in `locate Python.h | grep python` ; do
		dir=`dirname $file`
	done
	fi
	fi
	if test x$dir = x ; then
		PYTHON_FLAGS=""
		[AC_MSG_RESULT([no])]
	else
		PYTHON_FLAGS="-I$dir"
		[AC_MSG_RESULT([$dir])]
	fi
	[AC_SUBST(PYTHON_FLAGS)]
	[AM_CONDITIONAL(HAVE_PYTHON_DEVEL, test "x$PYTHON_FLAGS" != x)]
	)

dnl @synopsis ACX_PTHREAD([ACTION-IF-FOUND[, ACTION-IF-NOT-FOUND]])
dnl
dnl This macro figures out how to build C programs using POSIX
dnl threads.  It sets the PTHREAD_LIBS output variable to the threads
dnl library and linker flags, and the PTHREAD_CFLAGS output variable
dnl to any special C compiler flags that are needed.  (The user can also
dnl force certain compiler flags/libs to be tested by setting these
dnl environment variables.)
dnl
dnl Also sets PTHREAD_CC to any special C compiler that is needed for
dnl multi-threaded programs (defaults to the value of CC otherwise).
dnl (This is necessary on AIX to use the special cc_r compiler alias.)
dnl
dnl NOTE: You are assumed to not only compile your program with these
dnl flags, but also link it with them as well.  e.g. you should link
dnl with $PTHREAD_CC $CFLAGS $PTHREAD_CFLAGS $LDFLAGS ... $PTHREAD_LIBS $LIBS
dnl
dnl If you are only building threads programs, you may wish to
dnl use these variables in your default LIBS, CFLAGS, and CC:
dnl
dnl        LIBS="$PTHREAD_LIBS $LIBS"
dnl        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"
dnl        CC="$PTHREAD_CC"
dnl
dnl In addition, if the PTHREAD_CREATE_JOINABLE thread-attribute
dnl constant has a nonstandard name, defines PTHREAD_CREATE_JOINABLE
dnl to that name (e.g. PTHREAD_CREATE_UNDETACHED on AIX).
dnl
dnl ACTION-IF-FOUND is a list of shell commands to run if a threads
dnl library is found, and ACTION-IF-NOT-FOUND is a list of commands
dnl to run it if it is not found.  If ACTION-IF-FOUND is not specified,
dnl the default action will define HAVE_PTHREAD.
dnl
dnl Please let the authors know if this macro fails on any platform,
dnl or if you have any other suggestions or comments.  This macro was
dnl based on work by SGJ on autoconf scripts for FFTW (www.fftw.org)
dnl (with help from M. Frigo), as well as ac_pthread and hb_pthread
dnl macros posted by Alejandro Forero Cuervo to the autoconf macro
dnl repository.  We are also grateful for the helpful feedback of
dnl numerous users.
dnl
dnl @version $Id: acinclude.m4,v 1.19 2009/05/07 16:35:52 proctor Exp $
dnl @author Steven G. Johnson <address@hidden>

AC_DEFUN([ACX_PTHREAD], [
AC_REQUIRE([AC_CANONICAL_HOST])
AC_LANG_SAVE
AC_LANG_C
acx_pthread_ok=no

# We used to check for pthread.h first, but this fails if pthread.h
# requires special compiler flags (e.g. on True64 or Sequent).
# It gets checked for in the link test anyway.

# First of all, check if the user has set any of the PTHREAD_LIBS,
# etcetera environment variables, and if threads linking works using
# them:
if test x"$PTHREAD_LIBS$PTHREAD_CFLAGS" != x; then
        save_CFLAGS="$CFLAGS"
        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"
        save_LIBS="$LIBS"
        LIBS="$PTHREAD_LIBS $LIBS"
        AC_MSG_CHECKING([for pthread_join in LIBS=$PTHREAD_LIBS with CFLAGS=$PTHREAD_CFLAGS])
        AC_TRY_LINK_FUNC(pthread_join, acx_pthread_ok=yes)
        AC_MSG_RESULT($acx_pthread_ok)
        if test x"$acx_pthread_ok" = xno; then
                PTHREAD_LIBS=""
                PTHREAD_CFLAGS=""
        fi
        LIBS="$save_LIBS"
        CFLAGS="$save_CFLAGS"
fi

# We must check for the threads library under a number of different
# names; the ordering is very important because some systems
# (e.g. DEC) have both -lpthread and -lpthreads, where one of the
# libraries is broken (non-POSIX).

# Create a list of thread flags to try.  Items starting with a "-" are
# C compiler flags, and other items are library names, except for "none"
# which indicates that we try without any flags at all, and "pthread-config"
# which is a program returning the flags for the Pth emulation library.

acx_pthread_flags="pthreads pthread none -Kthread -kthread lthread -pthread -pthreads -mthreads --thread-safe -mt pthread-config"

# The ordering *is* (sometimes) important.  Some notes on the
# individual items follow:

# pthreads: AIX (must check this before -lpthread)
# none: in case threads are in libc; should be tried before -Kthread and
#       other compiler flags to prevent continual compiler warnings
# -Kthread: Sequent (threads in libc, but -Kthread needed for pthread.h)
# -kthread: FreeBSD kernel threads (preferred to -pthread since SMP-able)
# lthread: LinuxThreads port on FreeBSD (also preferred to -pthread)
# -pthread: Linux/gcc (kernel threads), BSD/gcc (userland threads)
# -pthreads: Solaris/gcc
# -mthreads: Mingw32/gcc, Lynx/gcc
# -mt: Sun Workshop C (may only link SunOS threads [-lthread], but it
#      doesn't hurt to check since this sometimes defines pthreads too;
#      also defines -D_REENTRANT)
# pthread: Linux, etcetera
# --thread-safe: KAI C++
# pthread-config: use pthread-config program (for GNU Pth library)

case "${host_cpu}-${host_os}" in
        *solaris*)

        # On Solaris (at least, for some versions), libc contains stubbed
        # (non-functional) versions of the pthreads routines, so link-based
        # tests will erroneously succeed.  (We need to link with -pthread or
        # -lpthread.)  (The stubs are missing pthread_cleanup_push, or rather
        # a function called by this macro, so we could check for that, but
        # who knows whether they'll stub that too in a future libc.)  So,
        # we'll just look for -pthreads and -lpthread first:

        acx_pthread_flags="-pthread -pthreads pthread -mt $acx_pthread_flags"
        ;;
esac

if test x"$acx_pthread_ok" = xno; then
for flag in $acx_pthread_flags; do

        case $flag in
                none)
                AC_MSG_CHECKING([whether pthreads work without any flags])
                ;;

                -*)
                AC_MSG_CHECKING([whether pthreads work with $flag])
                PTHREAD_CFLAGS="$flag"
                ;;

		pthread-config)
		AC_CHECK_PROG(acx_pthread_config, pthread-config, yes, no)
		if test x"$acx_pthread_config" = xno; then continue; fi
		PTHREAD_CFLAGS="`pthread-config --cflags`"
		PTHREAD_LIBS="`pthread-config --ldflags` `pthread-config --libs`"
		;;

                *)
                AC_MSG_CHECKING([for the pthreads library -l$flag])
                PTHREAD_LIBS="-l$flag"
                ;;
        esac

        save_LIBS="$LIBS"
        save_CFLAGS="$CFLAGS"
        LIBS="$PTHREAD_LIBS $LIBS"
        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"

        # Check for various functions.  We must include pthread.h,
        # since some functions may be macros.  (On the Sequent, we
        # need a special flag -Kthread to make this header compile.)
        # We check for pthread_join because it is in -lpthread on IRIX
        # while pthread_create is in libc.  We check for pthread_attr_init
        # due to DEC craziness with -lpthreads.  We check for
        # pthread_cleanup_push because it is one of the few pthread
        # functions on Solaris that doesn't have a non-functional libc stub.
        # We try pthread_create on general principles.
        AC_TRY_LINK([#include <pthread.h>],
                    [pthread_t th; pthread_join(th, 0);
                     pthread_attr_init(0); pthread_cleanup_push(0, 0);
                     pthread_create(0,0,0,0); pthread_cleanup_pop(0); ],
                    [acx_pthread_ok=yes])

        LIBS="$save_LIBS"
        CFLAGS="$save_CFLAGS"

        AC_MSG_RESULT($acx_pthread_ok)
        if test "x$acx_pthread_ok" = xyes; then
                break;
        fi

        PTHREAD_LIBS=""
        PTHREAD_CFLAGS=""
done
fi

# Various other checks:
if test "x$acx_pthread_ok" = xyes; then
        save_LIBS="$LIBS"
        LIBS="$PTHREAD_LIBS $LIBS"
        save_CFLAGS="$CFLAGS"
        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"

        # Detect AIX lossage: threads are created detached by default
        # and the JOINABLE attribute has a nonstandard name (UNDETACHED).
        AC_MSG_CHECKING([for joinable pthread attribute])
        AC_TRY_LINK([#include <pthread.h>],
                    [int attr=PTHREAD_CREATE_JOINABLE;],
                    ok=PTHREAD_CREATE_JOINABLE, ok=unknown)
        if test x"$ok" = xunknown; then
                AC_TRY_LINK([#include <pthread.h>],
                            [int attr=PTHREAD_CREATE_UNDETACHED;],
                            ok=PTHREAD_CREATE_UNDETACHED, ok=unknown)
        fi
        if test x"$ok" != xPTHREAD_CREATE_JOINABLE; then
                AC_DEFINE(PTHREAD_CREATE_JOINABLE, $ok,
                          [Define to the necessary symbol if this constant
                           uses a non-standard name on your system.])
        fi
        AC_MSG_RESULT(${ok})
        if test x"$ok" = xunknown; then
                AC_MSG_WARN([we do not know how to create joinable pthreads])
        fi

        AC_MSG_CHECKING([if more special flags are required for pthreads])
        flag=no
        case "${host_cpu}-${host_os}" in
                *-aix* | *-freebsd* | *-darwin*) flag="-D_THREAD_SAFE";;
                *solaris* | *-osf* | *-hpux*) flag="-D_REENTRANT";;
        esac
        AC_MSG_RESULT(${flag})
        if test "x$flag" != xno; then
                PTHREAD_CFLAGS="$flag $PTHREAD_CFLAGS"
        fi

        LIBS="$save_LIBS"
        CFLAGS="$save_CFLAGS"

        # More AIX lossage: must compile with cc_r
        AC_CHECK_PROG(PTHREAD_CC, cc_r, cc_r, ${CC})
else
        PTHREAD_CC="$CC"
fi

AC_SUBST(PTHREAD_LIBS)
AC_SUBST(PTHREAD_CFLAGS)
AC_SUBST(PTHREAD_CC)

# Finally, execute ACTION-IF-FOUND/ACTION-IF-NOT-FOUND:
if test x"$acx_pthread_ok" = xyes; then
        ifelse([$1],,AC_DEFINE(HAVE_PTHREAD,1,[Define if you have POSIX threads libraries and header files.]),[$1])
        :
else
        acx_pthread_ok=no
        $2
fi
AC_LANG_RESTORE
])dnl ACX_PTHREAD

dnl @synopsis ACX_PTHREAD([ACTION-IF-FOUND[, ACTION-IF-NOT-FOUND]])
dnl
dnl This macro figures out how to build C programs using POSIX threads.
dnl It sets the PTHREAD_LIBS output variable to the threads library and
dnl linker flags, and the PTHREAD_CFLAGS output variable to any special
dnl C compiler flags that are needed. (The user can also force certain
dnl compiler flags/libs to be tested by setting these environment
dnl variables.)
dnl
dnl Also sets PTHREAD_CC to any special C compiler that is needed for
dnl multi-threaded programs (defaults to the value of CC otherwise).
dnl (This is necessary on AIX to use the special cc_r compiler alias.)
dnl
dnl NOTE: You are assumed to not only compile your program with these
dnl flags, but also link it with them as well. e.g. you should link
dnl with $PTHREAD_CC $CFLAGS $PTHREAD_CFLAGS $LDFLAGS ... $PTHREAD_LIBS
dnl $LIBS
dnl
dnl If you are only building threads programs, you may wish to use
dnl these variables in your default LIBS, CFLAGS, and CC:
dnl
dnl        LIBS="$PTHREAD_LIBS $LIBS"
dnl        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"
dnl        CC="$PTHREAD_CC"
dnl
dnl In addition, if the PTHREAD_CREATE_JOINABLE thread-attribute
dnl constant has a nonstandard name, defines PTHREAD_CREATE_JOINABLE to
dnl that name (e.g. PTHREAD_CREATE_UNDETACHED on AIX).
dnl
dnl ACTION-IF-FOUND is a list of shell commands to run if a threads
dnl library is found, and ACTION-IF-NOT-FOUND is a list of commands to
dnl run it if it is not found. If ACTION-IF-FOUND is not specified, the
dnl default action will define HAVE_PTHREAD.
dnl
dnl Please let the authors know if this macro fails on any platform, or
dnl if you have any other suggestions or comments. This macro was based
dnl on work by SGJ on autoconf scripts for FFTW (www.fftw.org) (with
dnl help from M. Frigo), as well as ac_pthread and hb_pthread macros
dnl posted by Alejandro Forero Cuervo to the autoconf macro repository.
dnl We are also grateful for the helpful feedback of numerous users.
dnl
dnl @category InstalledPackages
dnl @author Steven G. Johnson <stevenj@alum.mit.edu>
dnl @version 2005-01-14
dnl @license GPLWithACException

AC_DEFUN([ACX_PTHREAD_CXX], [
AC_REQUIRE([AC_CANONICAL_HOST])
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
acx_pthread_ok=no

# We used to check for pthread.h first, but this fails if pthread.h
# requires special compiler flags (e.g. on True64 or Sequent).
# It gets checked for in the link test anyway.

# First of all, check if the user has set any of the PTHREAD_LIBS,
# etcetera environment variables, and if threads linking works using
# them:
if test x"$PTHREAD_LIBS$PTHREAD_CXXFLAGS" != x; then
        save_CXXFLAGS="$CXXFLAGS"
        CXXFLAGS="$CXXFLAGS $PTHREAD_CXXFLAGS"
        save_LIBS="$LIBS"
        LIBS="$PTHREAD_LIBS $LIBS"
        AC_MSG_CHECKING([for pthread_join in LIBS=$PTHREAD_LIBS with CXXFLAGS=$PTHREAD_CXXFLAGS])
        AC_TRY_LINK_FUNC(pthread_join, acx_pthread_ok=yes)
        AC_MSG_RESULT($acx_pthread_ok)
        if test x"$acx_pthread_ok" = xno; then
                PTHREAD_LIBS=""
                PTHREAD_CXXFLAGS=""
        fi
        LIBS="$save_LIBS"
        CXXFLAGS="$save_CXXFLAGS"
fi

# We must check for the threads library under a number of different
# names; the ordering is very important because some systems
# (e.g. DEC) have both -lpthread and -lpthreads, where one of the
# libraries is broken (non-POSIX).

# Create a list of thread flags to try.  Items starting with a "-" are
# C compiler flags, and other items are library names, except for "none"
# which indicates that we try without any flags at all, and "pthread-config"
# which is a program returning the flags for the Pth emulation library.

acx_pthread_flags="pthreads none -Kthread -kthread lthread -pthread -pthreads -mthreads pthread --thread-safe -mt pthread-config"

# The ordering *is* (sometimes) important.  Some notes on the
# individual items follow:

# pthreads: AIX (must check this before -lpthread)
# none: in case threads are in libc; should be tried before -Kthread and
#       other compiler flags to prevent continual compiler warnings
# -Kthread: Sequent (threads in libc, but -Kthread needed for pthread.h)
# -kthread: FreeBSD kernel threads (preferred to -pthread since SMP-able)
# lthread: LinuxThreads port on FreeBSD (also preferred to -pthread)
# -pthread: Linux/gcc (kernel threads), BSD/gcc (userland threads)
# -pthreads: Solaris/gcc
# -mthreads: Mingw32/gcc, Lynx/gcc
# -mt: Sun Workshop C (may only link SunOS threads [-lthread], but it
#      doesn't hurt to check since this sometimes defines pthreads too;
#      also defines -D_REENTRANT)
# pthread: Linux, etcetera
# --thread-safe: KAI C++
# pthread-config: use pthread-config program (for GNU Pth library)

case "${host_cpu}-${host_os}" in
        *solaris*)

        # On Solaris (at least, for some versions), libc contains stubbed
        # (non-functional) versions of the pthreads routines, so link-based
        # tests will erroneously succeed.  (We need to link with -pthread or
        # -lpthread.)  (The stubs are missing pthread_cleanup_push, or rather
        # a function called by this macro, so we could check for that, but
        # who knows whether they'll stub that too in a future libc.)  So,
        # we'll just look for -pthreads and -lpthread first:

        acx_pthread_flags="-pthread -pthreads pthread -mt $acx_pthread_flags"
        ;;
esac

if test x"$acx_pthread_ok" = xno; then
for flag in $acx_pthread_flags; do

        case $flag in
                none)
                AC_MSG_CHECKING([whether pthreads work without any flags])
                ;;

                -*)
                AC_MSG_CHECKING([whether pthreads work with $flag])
                PTHREAD_CXXFLAGS="$flag"
                ;;

		pthread-config)
		AC_CHECK_PROG(acx_pthread_config, pthread-config, yes, no)
		if test x"$acx_pthread_config" = xno; then continue; fi
		PTHREAD_CXXFLAGS="`pthread-config --cflags`"
		PTHREAD_LIBS="`pthread-config --ldflags` `pthread-config --libs`"
		;;

                *)
                AC_MSG_CHECKING([for the pthreads library -l$flag])
                PTHREAD_LIBS="-l$flag"
                ;;
        esac

        save_LIBS="$LIBS"
        save_CXXFLAGS="$CXXFLAGS"
        LIBS="$PTHREAD_LIBS $LIBS"
        CXXFLAGS="$CXXFLAGS $PTHREAD_CXXFLAGS"

        # Check for various functions.  We must include pthread.h,
        # since some functions may be macros.  (On the Sequent, we
        # need a special flag -Kthread to make this header compile.)
        # We check for pthread_join because it is in -lpthread on IRIX
        # while pthread_create is in libc.  We check for pthread_attr_init
        # due to DEC craziness with -lpthreads.  We check for
        # pthread_cleanup_push because it is one of the few pthread
        # functions on Solaris that doesn't have a non-functional libc stub.
        # We try pthread_create on general principles.
        AC_TRY_LINK([#include <pthread.h>],
                    [pthread_t th; pthread_join(th, 0);
                     pthread_attr_init(0); pthread_cleanup_push(0, 0);
                     pthread_create(0,0,0,0); pthread_cleanup_pop(0); ],
                    [acx_pthread_ok=yes])

        LIBS="$save_LIBS"
        CXXFLAGS="$save_CXXFLAGS"

        AC_MSG_RESULT($acx_pthread_ok)
        if test "x$acx_pthread_ok" = xyes; then
                break;
        fi

        PTHREAD_LIBS=""
        PTHREAD_CXXFLAGS=""
done
fi

# Various other checks:
if test "x$acx_pthread_ok" = xyes; then
        save_LIBS="$LIBS"
        LIBS="$PTHREAD_LIBS $LIBS"
        save_CXXFLAGS="$CXXFLAGS"
        CXXFLAGS="$CXXFLAGS $PTHREAD_CXXFLAGS"

        # Detect AIX lossage: JOINABLE attribute is called UNDETACHED.
	AC_MSG_CHECKING([for joinable pthread attribute])
	attr_name=unknown
	for attr in PTHREAD_CREATE_JOINABLE PTHREAD_CREATE_UNDETACHED; do
	    AC_TRY_LINK([#include <pthread.h>], [int attr=$attr;],
                        [attr_name=$attr; break])
	done
        AC_MSG_RESULT($attr_name)
        if test "$attr_name" != PTHREAD_CREATE_JOINABLE; then
            AC_DEFINE_UNQUOTED(PTHREAD_CREATE_JOINABLE, $attr_name,
                               [Define to necessary symbol if this constant
                                uses a non-standard name on your system.])
        fi

        AC_MSG_CHECKING([if more special flags are required for pthreads])
        flag=no
        case "${host_cpu}-${host_os}" in
            *-aix* | *-freebsd* | *-darwin*) flag="-D_THREAD_SAFE";;
            *solaris* | *-osf* | *-hpux*) flag="-D_REENTRANT";;
        esac
        AC_MSG_RESULT(${flag})
        if test "x$flag" != xno; then
            PTHREAD_CXXFLAGS="$flag $PTHREAD_CXXFLAGS"
        fi

        LIBS="$save_LIBS"
        CXXFLAGS="$save_CXXFLAGS"

        # More AIX lossage: must compile with cc_r
        AC_CHECK_PROG(PTHREAD_CXX, cc_r, cc_r, ${CXX})
else
        PTHREAD_CXX="$CXX"
fi

AC_SUBST(PTHREAD_LIBS)
AC_SUBST(PTHREAD_CXXFLAGS)
AC_SUBST(PTHREAD_CXX)

# Finally, execute ACTION-IF-FOUND/ACTION-IF-NOT-FOUND:
if test x"$acx_pthread_ok" = xyes; then
        ifelse([$1],,AC_DEFINE(HAVE_PTHREAD,1,[Define if you have POSIX threads libraries and header files.]),[$1])
        :
else
        acx_pthread_ok=no
        $2
fi
AC_LANG_RESTORE
])dnl ACX_PTHREAD

# Configure paths for SDL
# Sam Lantinga 9/21/99
# stolen from Manish Singh
# stolen back from Frank Belew
# stolen from Manish Singh
# Shamelessly stolen from Owen Taylor

dnl AM_PATH_SDL([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for SDL, and define SDL_CFLAGS and SDL_LIBS
dnl
AC_DEFUN([AM_PATH_SDL],
[dnl 
dnl Get the cflags and libraries from the sdl-config script
dnl
AC_ARG_WITH(sdl-prefix,[  --with-sdl-prefix=PFX   Prefix where SDL is installed (optional)],
            sdl_prefix="$withval", sdl_prefix="")
AC_ARG_WITH(sdl-exec-prefix,[  --with-sdl-exec-prefix=PFX Exec prefix where SDL is installed (optional)],
            sdl_exec_prefix="$withval", sdl_exec_prefix="")
AC_ARG_ENABLE(sdltest, [  --disable-sdltest       Do not try to compile and run a test SDL program],
		    , enable_sdltest=yes)

  if test x$sdl_exec_prefix != x ; then
    sdl_args="$sdl_args --exec-prefix=$sdl_exec_prefix"
    if test x${SDL_CONFIG+set} != xset ; then
       SDL_CONFIG=$sdl_exec_prefix/bin/sdl-config
    fi
  fi
  if test x$sdl_prefix != x ; then
    sdl_args="$sdl_args --prefix=$sdl_prefix"
    if test x${SDL_CONFIG+set} != xset ; then
       SDL_CONFIG=$sdl_prefix/bin/sdl-config
    fi
  fi

  if test "x$prefix" != xNONE; then
    PATH="$prefix/bin:$prefix/usr/bin:$PATH"
  fi
  AC_PATH_PROG(SDL_CONFIG, sdl-config, no, [$PATH])
  min_sdl_version=ifelse([$1], ,0.11.0,$1)
  AC_MSG_CHECKING(for SDL - version >= $min_sdl_version)
  no_sdl=""
  if test "$SDL_CONFIG" = "no" ; then
    no_sdl=yes
  else
    SDL_CFLAGS=`$SDL_CONFIG $sdlconf_args --cflags`
    SDL_LIBS=`$SDL_CONFIG $sdlconf_args --libs`

    sdl_major_version=`$SDL_CONFIG $sdl_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    sdl_minor_version=`$SDL_CONFIG $sdl_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    sdl_micro_version=`$SDL_CONFIG $sdl_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_sdltest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $SDL_CFLAGS"
      CXXFLAGS="$CXXFLAGS $SDL_CFLAGS"
      LIBS="$LIBS $SDL_LIBS"
dnl
dnl Now check if the installed SDL is sufficiently new. (Also sanity
dnl checks the results of sdl-config to some extent
dnl
      rm -f conf.sdltest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDL.h"

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.sdltest");
  */
  { FILE *fp = fopen("conf.sdltest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_sdl_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_sdl_version");
     exit(1);
   }

   if (($sdl_major_version > major) ||
      (($sdl_major_version == major) && ($sdl_minor_version > minor)) ||
      (($sdl_major_version == major) && ($sdl_minor_version == minor) && ($sdl_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'sdl-config --version' returned %d.%d.%d, but the minimum version\n", $sdl_major_version, $sdl_minor_version, $sdl_micro_version);
      printf("*** of SDL required is %d.%d.%d. If sdl-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If sdl-config was wrong, set the environment variable SDL_CONFIG\n");
      printf("*** to point to the correct copy of sdl-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_sdl=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_sdl" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$SDL_CONFIG" = "no" ; then
       echo "*** The sdl-config script installed by SDL could not be found"
       echo "*** If SDL was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the SDL_CONFIG environment variable to the"
       echo "*** full path to sdl-config."
     else
       if test -f conf.sdltest ; then
        :
       else
          echo "*** Could not run SDL test program, checking why..."
          CFLAGS="$CFLAGS $SDL_CFLAGS"
          CXXFLAGS="$CXXFLAGS $SDL_CFLAGS"
          LIBS="$LIBS $SDL_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
#include "SDL.h"

int main(int argc, char *argv[])
{ return 0; }
#undef  main
#define main K_and_R_C_main
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding SDL or finding the wrong"
          echo "*** version of SDL. If it is not finding SDL, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means SDL was incorrectly installed"
          echo "*** or that you have moved SDL since it was installed. In the latter case, you"
          echo "*** may want to edit the sdl-config script: $SDL_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          CXXFLAGS="$ac_save_CXXFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     SDL_CFLAGS=""
     SDL_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(SDL_CFLAGS)
  AC_SUBST(SDL_LIBS)
  rm -f conf.sdltest
])

AC_DEFUN([ACX_HAVE_SDL],
	[AM_PATH_SDL([1.2.8], have_sdl=true)]
	[AM_CONDITIONAL(HAVE_SDL, test x$have_sdl != x)]
)

AC_DEFUN([ACX_HAVE_PLAYER],
	[AC_CHECK_PROGS(PKG_CONFIG, pkg-config)]
	if test x$PKG_CONFIG != x ; then
		[AC_MSG_CHECKING([for Player 2.0])]
		if `pkg-config --exists playerc++` ; then
			PLAYER_VERSION=`pkg-config --modversion playerc++`
			PLAYER_CFLAGS=`pkg-config --cflags playerc++`
			PLAYER_LIBS=`pkg-config --libs playerc++`
			[AC_MSG_RESULT([$PLAYER_VERSION])]
dnl put HAVE_PLAYER in config.h
			[AC_DEFINE(HAVE_PLAYER,	1, [Define non-zero if you have Player 2.0.])]
dnl put PLAYER_VERSION,CFLAGS,LIBS in Makefile
			[AC_SUBST(PLAYER_VERSION)]
			[AC_SUBST(PLAYER_CFLAGS)]
			[AC_SUBST(PLAYER_LIBS)]
		else
			[AC_MSG_RESULT(no)]
		fi
	fi
dnl enable HAVE_PLAYER test in Makefile
	[AM_CONDITIONAL(HAVE_PLAYER, test x$PLAYER_VERSION != x)]
	)

AC_DEFUN([AC_HAVE_GOMOTION],
	[AC_MSG_CHECKING([for Go Motion])]
	[AC_ARG_WITH(gomotion,
		[ --with-gomotion=<path to Go Motion>  Specify path to Go Motion directory],
		dirs=$withval,dirs="/usr/local/gomotion /usr/local/src/gomotion $HOME/gomotion")]
	for dir in $dirs ; do
		if test -f $dir/include/go.h ; then GOMOTION_DIR=$dir ; break ; fi
	done
	if test x$GOMOTION_DIR = x ; then
	[AC_MSG_WARN([not found, specify using --with-gomotion=<path to Go Motion>])]
	else
	[AC_MSG_RESULT([$GOMOTION_DIR])]
	GOMOTION_CFLAGS="-I$GOMOTION_DIR/include"
	GOMOTION_LIBS="-L$GOMOTION_DIR/lib -lunix_ulapi -lgo"
dnl put HAVE_GOMOTION in config.h
	[AC_DEFINE(HAVE_GOMOTION,
		1, [Define non-zero if you have Go Motion.])]
dnl put GOMOTION_DIR in Makefile
	[AC_SUBST(GOMOTION_DIR)]
	[AC_SUBST(GOMOTION_CFLAGS)]
	[AC_SUBST(GOMOTION_LIBS)]
	fi
dnl enable HAVE_GOMOTION test in Makefile
	[AM_CONDITIONAL(HAVE_GOMOTION, test x$GOMOTION_DIR != x)]
	)

AC_DEFUN([ACX_HAVE_DL],
dnl put HAVE_DLFCN_H in config/config.h
	[AC_CHECK_HEADERS([dlfcn.h])]
dnl put -ldl in default link line, and then set a custom variable
	[AC_SEARCH_LIBS([dlopen], [dl], have_dl_libs=yes)]
dnl check the custom variable, and add some CFLAGS for compiling shared objs
	if test x$have_dl_libs = xyes ; then
	DL_CFLAGS="-shared -fPIC"
	else
	DL_CFLAGS=""
	fi
	[AC_SUBST(DL_CFLAGS)]
	[AM_CONDITIONAL(HAVE_DL, test x$have_dl_libs = xyes)]
)
