#!/bin/sh

if test "x${DEBUG}" != "x" ; then
    set -x;
fi

echo "Running $0 $* in " `pwd`;

if test "x${VEH_ID}" = "x" ; then
    VEH_ID=1;
fi

if test "x${RCAW_PID_LIST_FILE}" = "x" ; then
    RCAW_PID_LIST_FILE=`pwd`/rcaw_pid_${VEH_ID}.list;
fi

echo "VEH_ID=${VEH_ID}"
echo "RCAW_PID_LIST_FILE=${RCAW_PID_LIST_FILE}";

function find_current_pids() {

    echo "" 
    echo "Processes still left to kill with SIGINT :"
    CURRENT_PIDS=
    for pid in  ${PIDS_FROM_FILE} ; do 
	if ps --no-header -o pid,command  -p $pid ; then 
	    ps --ppid $pid --no-header -o pid,command | grep -v 'tee rcaw_'  ; 
	    CHILD_PIDS=`ps --ppid $pid --no-header -o pid`; 
	    if test "x${CHILD_PIDS}" != "x" ; then 
		for cpid in ${CHILD_PIDS}; do 
		    	    ps --ppid $cpid --no-header -o pid,command | grep -v 'tee rcaw_' ; 
			    GRAND_CHILD_PIDS=`ps --ppid $cpid --no-header -o pid`; 
			    if test "x${GRAND_CHILD_PIDS}" != "x" ; then 
				kill -INT ${GRAND_CHILD_PIDS}
			    fi
		done
		kill -INT ${CHILD_PIDS}; 
	    fi; 
	    kill -INT $pid; 
	    CURRENT_PIDS="${CURRENT_PIDS} ${pid}"; 
	fi ;  
    done

    sleep 1;

    CURRENT_PIDS=
    for pid in  ${PIDS_FROM_FILE} ; do 
	if ps --no-header -o pid,command  -p $pid >/dev/null ; then 
	    CURRENT_PIDS="${CURRENT_PIDS} ${pid}"; 
	fi
    done
    echo ""
}

if test -f "${RCAW_PID_LIST_FILE}" ; then

    PIDS_FROM_FILE=`cat ${RCAW_PID_LIST_FILE}`;
    
    echo "PIDS_FROM_FILE=${PIDS_FROM_FILE}";

    
    find_current_pids
    if test "x${CURRENT_PIDS}" != "x" ; then
	echo "Sending SIGINT to ${CURRENT_PIDS}"
	kill -INT ${CURRENT_PIDS}
	sleep 3
    fi


    find_current_pids
    quit=n
    SCREEN=`screen -ls | grep Detached | head -n 1 | awk '{print $1}'`
    while test "x${CURRENT_PIDS}" != "x" -a "x${SCREEN}" != "x" -a "x${quit}" != "xy" ; do
	screen -wipe
	SCREEN=`screen -ls | grep Detached | head -n 1 | awk '{print $1}'`
	screen -r ${SCREEN}

	find_current_pids
	if test "x${CURRENT_PIDS}" != "x" ; then
	    echo "Processes still alive : Quit by sending SIGKILL(y/n)?"
	    read quit
	else
	    quit=y;
	fi
    done

    find_current_pids
    if test "x${CURRENT_PIDS}" != "x" ; then
	echo "Sending SIGKILL to ${CURRENT_PIDS}"
	kill -KILL ${CURRENT_PIDS}
    fi

fi

if test "x${RCSLIB_DIR}" = "x" ; then
    RCSLIB_DIR=`grep RCSLIB_DIR ../Makefile | awk '{print $3}'`
fi

if test "x${CONFIG_NML}" = "x" ; then
    CONFIG_NML=moast_wps_${VEH_ID}.nml;
fi

echo "Running ${RCSLIB_DIR}/bin/nmlclean" "${CONFIG_NML}" " . . . "
env NMLCLEAN_LOCAL_ONLY=1 "${RCSLIB_DIR}/bin/nmlclean" "${CONFIG_NML}"

screen -wipe

echo "IPCS:"
ipcs | awk '{ if ( $1 != "0x00000000" && $3 != "root" ) print $0 ;}'

echo "NETSTAT:"
netstat -naptuee | grep $UID

"${RCSLIB_DIR}/bin/check_core_files.sh"

echo "Finished $0 $* in " `pwd`;

 
