#!/bin/sh

echo

if test "x${DEBUG}" = "x1" ; then
    echo "Running $0 $* from : "  `pwd`;
    set -x;
fi


echo "Deleting old log files."
\rm -f nmlclean.log rcaw_*.log killWpsAtrv.log 


# Run this as "./runWps.sh <id-#> <platform name> <startloc name>", e.g.,
# ./playRunWps.sh 1 P2AT RobotStart1

#
# USERS EDIT THIS AREA
#
# set GNDTRUTHONLY=no to have navData filled in with encoder values
# set GNDTRUTHONLY=yes to have ground truth in all fields of navData
GNDTRUTHONLY=yes

if ps -o cmd -C player | grep playerstage.cfg >/dev/null ; then
    GNDTRUTHONLY=no
fi

#set SECT=yes to run section, VEH, AM, prim, and servo, else SECT=no
SECT=no

# if SECT=no, set VEH=yes to run vehicle, AM, prim, and servo, else VEH=no
VEH=no

# if VEH=no, set AM=yes to run AM, prim, and servo, else AM=no
#export AM=yes

# if AM=no, set PRIM=yes to run prim, and servo, else PRIM=no
export PRIM=yes

# set USARSIM=yes to run usarSim, else USARSIM=no to run simpleSim
USARSIM=yes
export SERVO=$USARSIM
#
# END OF USER EDIT AREA
#


# get RCS directory and add to shared library load path
if test "x${RCSLIB_DIR}" = "x" ; then
    RCSLIB_DIR=`grep RCSLIB_DIR ../Makefile | awk '{print $3}'`
fi

if test x$RCSLIB_DIR != x ; then
    LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$RCSLIB_DIR/lib
    export LD_LIBRARY_PATH
fi

echo "RCSLIB_DIR=${RCSLIB_DIR}";

RCS_VERSION=`grep RCS_VERSION ${RCSLIB_DIR}/include/rcsvers.hh | sed 'y/"/ /' | sed 'y/./ /' | awk '{printf("%s%s%s",$3,$4,$5);}' | awk '{printf("%d",$1);}'`;
export RCS_VERSION

echo "RCS_VERSION=${RCS_VERSION}";

if test "x${RCS_VERSION}" = "x" ; then
    echo "Could not determine RCS_VERSION";
    exit 1;
fi

if test ${RCS_VERSION} -lt 20090212 ; then
    echo "You must upgrade the RCS library to at least 2009.02.12";
    exit 1;
fi

#echo `dirname $0`/..
if [ $# = 0 ]; then
 VEH_ID=1
else
 VEH_ID=$1
fi
export VEH_ID;

if [ $# -le 1 ]; then
 PLATFORM=P2AT
else
 PLATFORM=$2
fi
if [ $# -le 2 ]; then
 STARTLOC=default
else
 STARTLOC=$3
fi
#
#are we on cygwin?
CYGWIN=no
case `uname -s` in 
CYGWIN*) CYGWIN=yes
;;
esac

MISSION_PACKAGES=`./inifind ../etc/moast.ini MISSION_PACKAGES PLATFORM.${PLATFORM}`;

if test "x${MISSION_PACKAGES}" != "x" ; then
    if test ${MISSION_PACKAGES} -gt 0 ; then
	export MIS=yes;
    fi
fi

if test "x${CONFIG_INI}" = "x" ; then
    CONFIG_INI=../etc/moast.ini;
fi

if test "x${CONFIG_NML}" = "x" ; then
    CONFIG_NML=../etc/moast_wps_${VEH_ID}.nml;
fi

if test ! -f "${CONFIG_NML}" ; then
    NEED_TO_RUN_NMLCFG=1;
elif test ./runWps.sh -nt "${CONFIG_NML}" -o ../etc/moastWPS.cfg -nt "${CONFIG_NML}" -o  ../etc/moastBuffersWPS.cfg -nt "${CONFIG_NML}" ; then 
    NEED_TO_RUN_NMLCFG=1;
elif test  "x${NMLCFG_EXTRA_CFGFILES}" != "x" ; then
    for nec in ${NMLCFG_EXTRA_CFGFILES} ; do 
	if test $nec -nt "${CONFIG_NML}" ; then
	        NEED_TO_RUN_NMLCFG=1;
	fi
    done
fi

if test "x${NEED_TO_RUN_NMLCFG}" != "x" ; then
    "${RCSLIB_DIR}/bin/nmlcfg" ${NMLCFG_EXTRA_CFGFILES} ../etc/moastWPS.cfg -o "${CONFIG_NML}"
fi

if test ! -f "${CONFIG_NML}" ; then
    echo "${CONFIG_NML} does not exist.";
    exit 1
fi

export RCAW_PID_LIST_FILE=`pwd`/rcaw_pid_${VEH_ID}.list;
\rm -f ${RCAW_PID_LIST_FILE}

export CONFIG_NML;
export CONFIG_INI

TOP=servoShell
if [ x$PRIM = xyes ] ; then TOP=primShell ; fi
if [ x$AM = xyes ] ; then PRIM=yes; TOP=amShell ; fi
if [ x$VEH = xyes ] ; then AM=yes; PRIM=yes; TOP=vehShell ; fi
if [ x$SECT = xyes ] ; then VEH=yes; AM=yes; PRIM=yes; TOP=sectShell ; fi

killme() 
{
    ./killWpsAll.sh
    exit 1
}

trap killme SIGINT


echo "Starting server..."
\rm -f moastNmlWpsSvr.running;
"${RCSLIB_DIR}/bin/screen_x.sh" -n moastNmlWpsSvr$VEH_ID -e ./moastNmlWpsSvr 
"${RCSLIB_DIR}/bin/wait_for.sh" 15 ./moastNmlWpsSvr.running
if test ! -f ./moastNmlWpsSvr.running ; then
    echo "moastNmlWpsSvr would not start. giving up"
    screen -r moastNmlWpsSvr$VEH_ID
    ./killWpsAll.sh
    exit 1
fi

if test "x${EXTRA_SCRIPT}" != "x" ; then
    . ${EXTRA_SCRIPT};
fi

if test "x${INF_SKIN}" = "x" ; then
    if ps -C ut2004-bin >/dev/null ; then 
	INF_SKIN=usarsim
    else
	INF_SKIN=player
    fi
fi

echo "INF_SKIN=${INF_SKIN}";

echo "Starting simWare ..."
\rm -f simWare.running;
"${RCSLIB_DIR}/bin/screen_x.sh" -n simWare$VEH_ID -e ./simWare --infSkin ${INF_SKIN} -i$VEH_ID -p$PLATFORM -s$STARTLOC
"${RCSLIB_DIR}/bin/wait_for.sh" 15 ./simWare.running
if test ! -f ./simWare.running ; then
    echo "simWare would not start. giving up"
    screen -r simWare$VEH_ID
    ./killWpsAll.sh
    exit 1
fi

echo "Starting slamStub"
if [ x$GNDTRUTHONLY = xno ] ; then
    "${RCSLIB_DIR}/bin/screen_x.sh" -n slamStub$VEH_ID -e ./slamStub -t -i$VEH_ID&
else
    "${RCSLIB_DIR}/bin/screen_x.sh" -n slamStub$VEH_ID -e ./slamStub -i$VEH_ID&
fi

env NO_RCAW_LOG=1 "${RCSLIB_DIR}/bin/screen_x.sh" -e ./servoShell -n${CONFIG_NML} -d$VEH_ID

if [ x$PRIM = xyes ] ; then
  echo "Starting prim..."
  sleep 2
  "${RCSLIB_DIR}/bin/screen_x.sh" -n primMob$VEH_ID  -e ./primMobMain -d$VEH_ID
  "${RCSLIB_DIR}/bin/screen_x.sh" -n primSP$VEH_ID -e ./primSPMain -d$VEH_ID
  if test "x${MIS}" = "xyes" ; then 
      "${RCSLIB_DIR}/bin/screen_x.sh" -primMis$VEH_ID -e ./primMisMain -d$VEH_ID
  fi
  env NO_RCAW_LOG=1 "${RCSLIB_DIR}/bin/screen_x.sh" -n primShell$VEH_ID -e ./primShell -n${CONFIG_NML} -d$VEH_ID
fi

if [ x$AM = xyes ] ; then
    echo "Starting am..."
   "${RCSLIB_DIR}/bin/screen_x.sh" -n amMob$VEH_ID -e ./amMobMain -d$VEH_ID
   "${RCSLIB_DIR}/bin/screen_x.sh" -n amSP$VEH_ID -e ./amSPMain -z.1 -d$VEH_ID
   if test "x${MIS}" = "xyes" ; then 
       "${RCSLIB_DIR}/bin/screen_x.sh" -n amMis$VEH_ID -e ./amMisMain -d$VEH_ID
   fi
   env NO_RCAW_LOG=1 "${RCSLIB_DIR}/bin/screen_x.sh" -e ./amShell -n${CONFIG_NML} -d$VEH_ID
fi

if [ x$VEH = xyes ] ; then
  echo "Starting vehicle..."
#  ./vehMobPLMain -i -g1 -r.2 -d$VEH_ID &
  "${RCSLIB_DIR}/bin/screen_x.sh" -n vehMobPL$VEH_ID -e ./vehMobPLMain -g1 -r.5 -d$VEH_ID
  "${RCSLIB_DIR}/bin/screen_x.sh" -n vehMobVis$VEH_ID -e ./vehMobVis -d$VEH_ID
  env NO_RCAW_LOG=1 "${RCSLIB_DIR}/bin/screen_x.sh" -e ./vehShell -n${CONFIG_NML} -d$VEH_ID
fi

if [ x$SECT = xyes ] ; then
  echo "Starting section..."
  "${RCSLIB_DIR}/bin/screen_x.sh" -e ./sectManfact
fi

if test "x${DISPLAY}" = "x:0.0" ; then
    ./runDiag.sh
fi

echo "Enter a screen from below to see text output from processes:"
echo "     Use <CNTRL>A then <CNTRL>D to return to this script from within a screen."
screen -ls

echo "OR"
echo "Enter quit to KILL the moast processes started by this script."
echo "Running $0 $* from " `pwd`;
echo " VEH_ID=${VEH_ID} ; CONFIG_NML=${CONFIG_NML}"
read cmd
while test "x${cmd}" != "xquit" ; do
    screen -r ${cmd}

    echo "Enter a screen from below to see text output from processes:"
    echo "     Use <CNTRL>A then <CNTRL>D to return to this script from within a screen."
    screen -ls

    echo "OR"
    echo "Enter quit to KILL the moast processes started by this script."
    echo "Running $0 $* from " `pwd`;
    echo " VEH_ID=${VEH_ID} ; CONFIG_NML=${CONFIG_NML}"
    read cmd
done


./killWpsAll.sh


exit 0
