#ifndef COMMUNICATIONS_BASE_H
#define COMMUNICATIONS_BASE_H

#define ROBOT_COUNT 9  // Includes CommStation

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string>
#include <strings.h>
#include <cstring>
#include <iostream>
#include <cstdlib>
#include <queue>
#include <pthread.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>

using namespace std;

class Communications
{
  public:
  static const int ROBOTCOUNT = ROBOT_COUNT;
  typedef struct commArgs{
          int sockfd;
          int robID;
          Communications* obj;
  }CommArgs;
  // Constructors
  Communications();
  Communications(int robID, char* wssIP);

  // WSS Communications
  bool SendInit(string wssIP, int port);
  float GetSignalStrength(string RobotName);
  string INITResponse();
  float SignalStrengthResponse(string msg);
  int DNSResponse(string msg);
  int ReverseDNSResponse(string msg);
  int getRobotDNS(string RobotName);
  int getRobotReverseDNS(int);
  char* WSSIP;
  // Thread starting points
  static void* startManageIncomingMessages(void *args){
   (((CommArgs*)(args))->obj)->manageIncomingMessages(args);
   return NULL;
  }
  static void* startManageOutgoingMessages(void *args){
   (((CommArgs*)(args))->obj)->manageOutgoingMessages(args);
    return NULL;
  }
  static void* startListenForConnections(void *args){
   (((CommArgs*)(args))->obj)->listenForConnections(args);
    return NULL;
  }
  void* manageIncomingMessages(void* arg);
  void* manageOutgoingMessages(void* arg);
  void* listenForConnections(void* arg);
  
  // Functions for the actual sending/recieving of messages.
  bool readIncomingMessage(int robID,string& msg);
  void sendMessage(int robID,string msg);

  //attempt to close all the sockets - this could use some testing to see side effects
  void closeAllSockets();

  // Communications Queues and socket descriptor for listen port.
  int socket_descriptor;
  int listen_sockfd;
  int robotcomm_sockfd[ROBOT_COUNT];
  int myID;
  queue<string> msgsOut[ROBOT_COUNT];
  queue<string> msgsIn[ROBOT_COUNT];
  
  // THREAD: Condition Declarations
  pthread_cond_t msgOutCond[ROBOT_COUNT];


  // THREAD: Mutex Declarations
  pthread_mutex_t msgInBlock[ROBOT_COUNT];
  pthread_mutex_t msgOutBlock[ROBOT_COUNT];
  pthread_mutex_t msgOutCondBlock[ROBOT_COUNT];
  pthread_mutex_t wssCommBlock;

  

};

#endif
