#include <iostream>
#include <list>
#include <cstdlib> 
#include <FreeImagePlus.h>

using namespace std;

class SimplePoint{
  public:
  int x;
  int y;
  float weight;
  SimplePoint(int a,int b){x=a;y=b;weight=1.0f;};
  static bool compare_weights(SimplePoint *first,SimplePoint *second)
  {
    return(first->weight < second->weight);
  }
};

class Score
{
public:
	// Class variables
	int** map;	
	int mapHeight;
	int mapWidth;

	// Constructors
	Score();
	Score(int** newMap, int height, int width);

	// Class functions
	float computeScore(list<SimplePoint*> *points, int range);
	bool worthPoint(SimplePoint* point);
};

class DeploymentPlan
{
public:
  int** voronoiMap;
  int voronoiWidth;
  int voronoiHeight;
  char* fileName;
  int loadVoronoiFromBMP(char* filePath);
  

  void printVoronoiToConsole();

  Score scoring;

  list<SimplePoint*> *searchResults;
  list<SimplePoint*> *bestPoints;
  list<SimplePoint*> *waypoints;
  int writeSolutionToBMP(list<SimplePoint*>  *points,char* path);
  int writeBestSolution();
  int maxSearchNodes;
  int searchCallCount;
  int solutionNumber;
  int solutionCount;
  int searchDistance;
  float bestScore;
  int pixelRange;
  void runSearch(SimplePoint *p,int depth,int robotCount);
  bool generateWaypointsToGoal(SimplePoint *start, SimplePoint *goal);
  void setPixel(int x,int y);
  bool voronoiPointIntersect(int x,int y,bool removePointsNearRobots = true);

  ///SOME TESTING SEARCH CODE
  int searchRadius;
  int communicationRadius;
  int currentDepth;
  int robCount;
  list<SimplePoint*> * getChildren(SimplePoint *parent, int childCount);
  list<SimplePoint*> * calculateDeployment(SimplePoint *start, int totalRobots, int iterations);
  void testAndAddIntersectionPoint(list<SimplePoint*> * l,int x,int y);
  list<SimplePoint*> * getIntersectionPoints(SimplePoint *parent);
  list<SimplePoint*> * mergePointList(list<SimplePoint*> *l1,list<SimplePoint*> *l2);
  void deletePointList(list<SimplePoint*> *l);

  DeploymentPlan(int commRangeInPixels){
    solutionNumber=0;
    maxSearchNodes = -1;
    pixelRange = commRangeInPixels;
    searchResults = new list<SimplePoint *>();
    bestPoints = new list<SimplePoint *>();
    waypoints = new list<SimplePoint *>();
    srand((unsigned)time(0)); 
  };
  ~DeploymentPlan(){
    searchResults->clear();
    bestPoints->clear();
    waypoints->clear();
    delete searchResults;
    delete bestPoints;
    delete waypoints;
  };
};