/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file factCtrl.hh

  \brief NML declarations for factory controller command, status, config
  and settings.


  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.4 $
  $Date: 2009/03/30 18:44:46 $
  \endcode

  \author Stephen Balakirsky
*/

#include <stdio.h>
#include <rcs.hh>
#include "moastNmlOffsets.hh"
#include "moastTypes.hh"

#ifndef FACT_CTRL_HH
#define FACT_CTRL_HH

#define FACT_CTRL_CMD_NAME "factoryCtrlCmd"
#define FACT_CTRL_STAT_NAME "factoryCtrlStat"
#define FACT_CTRL_CFG_NAME "factoryCtrlCfg"
#define FACT_CTRL_SET_NAME "factoryCtrlSet"


#define FACT_CTRL_CMD_BASE        (FACT_CTRL_BASE)
#define FACT_CTRL_STAT_BASE       (FACT_CTRL_BASE + 100)
#define FACT_CTRL_CFG_BASE        (FACT_CTRL_BASE + 200)
#define FACT_CTRL_SET_BASE        (FACT_CTRL_BASE + 300)

#define FACT_CTRL_CMD_INIT_TYPE          (FACT_CTRL_CMD_BASE + 1)
#define FACT_CTRL_CMD_ABORT_TYPE         (FACT_CTRL_CMD_BASE + 2)
#define FACT_CTRL_CMD_HALT_TYPE          (FACT_CTRL_CMD_BASE + 3)
#define FACT_CTRL_CMD_SHUTDOWN_TYPE      (FACT_CTRL_CMD_BASE + 4)
#define FACT_CTRL_CMD_NOP_TYPE           (FACT_CTRL_CMD_BASE + 5)

#define FACT_CTRL_CMD_DROP_PACKAGE_TYPE  (FACT_CTRL_CMD_BASE + 6)
#define FACT_CTRL_CMD_SET_CONVEYOR_TYPE  (FACT_CTRL_CMD_BASE + 7)
#define FACT_CTRL_CMD_CLEAR_CARGO_TYPE   (FACT_CTRL_CMD_BASE + 8)

#define FACT_CTRL_STAT_TYPE              (FACT_CTRL_STAT_BASE + 1)

#define FACT_CTRL_CFG_CYCLE_TIME_TYPE    (FACT_CTRL_CFG_BASE + 1)
#define FACT_CTRL_CFG_DEBUG_TYPE         (FACT_CTRL_CFG_BASE + 2)
#define FACT_CTRL_CFG_NOP_TYPE           (FACT_CTRL_CFG_BASE + 3)
#define FACT_CTRL_CFG_SET_PACK_FILE_TYPE (FACT_CTRL_CFG_BASE + 4)

#define FACT_CTRL_SET_TYPE               (FACT_CTRL_SET_BASE + 1)


#define MAX_FACTORY_NAME 255
#define MAX_FACTORY_CONVEYORS 5

/////////////////////////////////////////////////////////////////
// structurs
typedef struct {
  char name[MAX_FACTORY_NAME];
  float speed;
}ConveyorStat;

typedef struct {
  char name[MAX_FACTORY_NAME];
  float minSpeed;
  float maxSpeed;
  PM_CARTESIAN location;
  PM_RPY orientation;
}ConveyorSet;

/////////////////////////////////////////////////////////////////
// commands
class FactCtrlCmdInit:public RCS_CMD_MSG {
public:
  FactCtrlCmdInit():RCS_CMD_MSG
    (FACT_CTRL_CMD_INIT_TYPE,
     sizeof(FactCtrlCmdInit)) {};
  void update(CMS *);
};

class FactCtrlCmdHalt:public RCS_CMD_MSG {
public:
  FactCtrlCmdHalt():RCS_CMD_MSG
    (FACT_CTRL_CMD_HALT_TYPE,
     sizeof(FactCtrlCmdHalt)) {};
  void update(CMS *);
};

class FactCtrlCmdAbort:public RCS_CMD_MSG {
public:
  FactCtrlCmdAbort():RCS_CMD_MSG
    (FACT_CTRL_CMD_ABORT_TYPE,
     sizeof(FactCtrlCmdAbort)) {};
  void update(CMS *);
};

class FactCtrlCmdShutdown:public RCS_CMD_MSG {
public:
  FactCtrlCmdShutdown():RCS_CMD_MSG
    (FACT_CTRL_CMD_SHUTDOWN_TYPE,
     sizeof(FactCtrlCmdShutdown)) {};
  void update(CMS *);
};

class FactCtrlCmdNop:public RCS_CMD_MSG {
public:
  FactCtrlCmdNop():RCS_CMD_MSG
    (FACT_CTRL_CMD_NOP_TYPE,
     sizeof(FactCtrlCmdNop)) {};
  void update(CMS *);
};

//! Drop a package from the package chute
class FactCtrlCmdDropPackage:public RCS_CMD_MSG {
public:
  FactCtrlCmdDropPackage():RCS_CMD_MSG
     (FACT_CTRL_CMD_DROP_PACKAGE_TYPE,
      sizeof(FactCtrlCmdDropPackage)) {};
  char chuteName[MAX_FACTORY_NAME];
  void update(CMS *);
};

//! sets the speed of the given conveyor
class FactCtrlCmdSetConveyor:public RCS_CMD_MSG {
public:
  FactCtrlCmdSetConveyor():RCS_CMD_MSG
    (FACT_CTRL_CMD_SET_CONVEYOR_TYPE,
     sizeof(FactCtrlCmdSetConveyor)) {};
  char conveyorName[MAX_FACTORY_NAME];
  float speed;
  void update(CMS *);

};

//! clears all of the cargo from the world
class FactCtrlCmdClearCargo:public RCS_CMD_MSG {
public:
  FactCtrlCmdClearCargo():RCS_CMD_MSG
    (FACT_CTRL_CMD_CLEAR_CARGO_TYPE,
     sizeof(FactCtrlCmdClearCargo)) {};
  void update(CMS *);
};

/*!
  Status message
*/

class FactCtrlStat:public RCS_STAT_MSG_V2 {
public:
  FactCtrlStat():RCS_STAT_MSG_V2(FACT_CTRL_STAT_TYPE, sizeof(FactCtrlStat)) {};
  void update(CMS *);

  DECLARE_NML_DYNAMIC_LENGTH_ARRAY(ConveyorStat, conveyorStat, 
				   MAX_FACTORY_CONVEYORS);
};


/*!
  Commands that will come into the configuration channel
*/
class FactCtrlCfgCycleTime:public RCS_CMD_MSG {
public:
  FactCtrlCfgCycleTime():RCS_CMD_MSG
    (FACT_CTRL_CFG_CYCLE_TIME_TYPE,
     sizeof(FactCtrlCfgCycleTime)) {};
  void update(CMS *);

  double cycleTime;
};

class FactCtrlCfgNop:public RCS_CMD_MSG {
public:
  FactCtrlCfgNop():RCS_CMD_MSG
    (FACT_CTRL_CFG_NOP_TYPE,
     sizeof(FactCtrlCfgNop)) {};
  void update(CMS *);
};

/*! sets the name of a file for the factory controller to read rfid values from
    these values are set in the tag of a package before it is spawned
 */
class FactCtrlCfgSetPackFile:public RCS_CMD_MSG {
public:
  FactCtrlCfgSetPackFile():RCS_CMD_MSG
    (FACT_CTRL_CFG_SET_PACK_FILE_TYPE,
     sizeof(FactCtrlCfgSetPackFile)) {};
  void update(CMS *);

  char fileName[MOAST_FILE_NAME_LEN];
};

#define FACT_CTRL_DEBUG_ALL 0xFFFFFFFF

class FactCtrlCfgDebug:public RCS_CMD_MSG {
public:
  FactCtrlCfgDebug():RCS_CMD_MSG
    (FACT_CTRL_CFG_DEBUG_TYPE,
     sizeof(FactCtrlCfgDebug)) {};
  void update(CMS *);

  int debug;			// debug mask
};


/*!
  FactCtrlSet is the class that holds the quasi-static vehicle settings,
  those that change infrequently and in response to vehicle configuration
  messages.
 */
class FactCtrlSet:public RCS_STAT_MSG_V2 {
public:
  FactCtrlSet():RCS_STAT_MSG_V2(FACT_CTRL_SET_TYPE, sizeof(FactCtrlSet)) {};
  void update(CMS *);
  
  //! the desired cycleTime of the planner
  double cycleTime;
  //! debug mask
  int debug;			
  DECLARE_NML_DYNAMIC_LENGTH_ARRAY(ConveyorSet, conveyorSet, 
				   MAX_FACTORY_CONVEYORS);
};

extern int factCtrl_format(NMLTYPE type, void *buf, CMS * cms);
extern const char * factCtrl_symbol_lookup(long type);

#endif // FACT_CTRL_HH

/*
  Modification history:

  $Log: factCtrl.hh,v $
  Revision 1.4  2009/03/30 18:44:46  dr_steveb
  added more commands

  Revision 1.3  2009/03/25 13:11:51  dr_steveb
  Added clear cargo command

  Revision 1.2  2009/02/19 15:27:10  dr_steveb
  Moved status information to proper place

  Revision 1.1  2009/02/17 21:09:48  dr_steveb
  Initial add of comm file


*/
