/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   moastGeom.hh
  \brief  Header file for geometry routines that are commonly used by various MOAST modules.

  A number of commonly used routines have been gathered together in the MoastGeom
  class. This class acts as a convient container to store these routines.

  \code CVS Status:
  $Author: proctor $
  $Revision: 1.5 $
  $Date: 2008/12/02 19:49:40 $
  \endcode

  \author Stephen Balakirsky
  \date   07/05/05
*/
#ifndef MOAST_GEOM_HH
#define MOAST_GEOM_HH

#include <stdio.h>		// FILE, stdout
#include "moastTypes.hh"

//! A directed line segment may be represented by its two endpoints
class LineSegment {
public:
  LineSegment(){};
  LineSegment( double n1, double e1, double n2, double e2 );
  LineSegment( Location2D startIn, Location2D endIn );
  double distToPoint( const Location2D &pointIn );
  Location2D intersect( const LineSegment &line2 );
  LineSegment moveLineParallel( double distance, int direction );
  void setStart( double north, double east );
  void setEnd( double north, double east );
  void print(FILE *output=stdout);
  //! is point c between points a & b?
  bool between( Location2D &a, Location2D &b, Location2D &c );
  //! compute if three points are collinear
  bool collinear( Location2D &a, Location2D &b, 
		  Location2D &c, double threshold=0.0);
  //! compute if two line segments intersect
  bool testIntersect( LineSegment &l2 );
  //! compute if two line segments intersect in their interiors
  bool testIntersectProp( LineSegment &l2 );
  //! is the point to the left of the line
  bool left( Location2D &point );

  Location2D start;
  Location2D end;
  long properties; //!< a pointer that points to a properties structure for this line
};

#endif
/*! CVS Log Information
 * $Log: moastGeom.hh,v $
 * Revision 1.5  2008/12/02 19:49:40  proctor
 * Fixed lack of M_PI on Windows
 *
 * Revision 1.4  2007/12/13 13:17:26  dr_steveb
 * Changes for vehicle level visibility graph planning.
 *
 * Revision 1.3  2007/11/08 13:20:59  dr_steveb
 * Checked in to allow for Tom to compile
 *
 * Revision 1.2  2007/11/07 19:56:30  dr_steveb
 * Moved the LineSegment class from moastTypes into moastGeom
 *
 * Revision 1.1.1.1  2005/10/12 20:50:32  root
 * initial import
 *
 * Revision 1.1  2005/07/21 18:59:11  stephen
 * Fixed merged conflicts and added visibility graph stuff.
 *
 *
 */
