/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   moastMySQL.hh
  \brief  Header file for moast to mySql interface.

          This is the data base API for accessing the 
          global world model in a MYSQL database. This API will populate 
	  the WM with state and a priori knowledge.

	  Querying database to populate WM
	  1). setSqlQueryCmd(): hold SQL queries and sets them in commandBuffer
	  2). submitQuery()
	  3). processQuery()

	  user of this header must include
	  #include <mysql.h> 
	  
  \code CVS Status:
  $Author: root $
  $Revision: 1.1.1.1 $
  $Date: 2005/10/12 20:50:32 $
  \endcode

  \author Stephen Balakirsky (based on Chris Scrapper code)
  \date   06-14-2005
*/ 
#ifndef __moastMySQL__
#define __moastMySQL__

#define MOAST_MAX_SQL_STRING_LEN 255
#define INIT_FAILURE           0x01
#define CONNECT_FAILURE        0x02
#define QUERY_FAILURE          0x04
#define STORE_RESULT_FAILURE   0x08

#define STR_QUALIFIED_SELECT_QUERY "SELECT * FROM %s \"%s\" "
#define INT_QUALIFIED_SELECT_QUERY "SELECT * FROM %s %d"
#define SELECT_QUERY "SELECT * FROM %s"
#define UPDATE_QUERY "UPDATE %s SET %s"

class MoastMySQL
{
public:
  //! verbose flag
  int myVerbose;
  //!  server host 
  char *hostName;
  //!   username      
  char *userName;
  //!   password 
  char *password; 
  //! database name
  char *dbName;    
  
  //!  port number  
  unsigned int portNum; 
  //!  socket name
  char *socketName;
  //!   conn flags   
  unsigned int flags;   
  //!  conn handler
  MYSQL *conn;       
  //!  stores results 
  MYSQL_RES *resSet;
  //!  stores row
  MYSQL_ROW row;

  //! Buffer containing query command
  char *commandBuffer;
  ////////////////////////////////////////////////////////////////////////
  //! constructor
  MoastMySQL(int myVerboseIn=1);
  //! destructor
  ~MoastMySQL();
  //! opens the connection to the database,
  int init(char *hostIn, char *userIn, char *passwordIn, 
	   char *dbIn, char *socketIn=NULL, 
	   unsigned int portIn=0,
	   unsigned int flagsIn=0 );
  //! Dumps the query results in MYSQL_RES *res_set
  void showQuery();
  //! Submits the SQL query that was set by setSqlQueryCmd
  int submitQuery();
  //! Submits insert query that was set by setSqlQuertCmd
  int updateQuery();

  //! Processes results of query submited by submitQuery and stores results res_set  
  void processQuery();

  //! Need to set sql command into buffer for processing
  /*  Developer must set command into character array named commandBuffer*/
  void setSqlQueryCmd();
 
  //! Frees the results stored in MYSQL_RES * res_set
  void freeResults();
  //! Closes the database connection and frees memory
  void closeDB();
};

#endif

/*
  $Log: moastMySQL.hh,v $
  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.2  2005/06/29 21:08:06  proctor
  Took out \defgroup comment for CVS history, since that's not what \defgroup
  is used for.

  Revision 1.1  2005/06/15 13:38:53  stephen
  Added mysql support.
*/
