#include "communications.h"

using namespace std;

// Constructors

Communications::Communications()
{
}

Communications::Communications(int robID, char* wssIP)
{
	myID = robID;
	WSSIP = wssIP;

	//init mutexes + condition variables
	pthread_mutex_init(&wssCommBlock,NULL);
	for(int i=0;i<ROBOT_COUNT;i++){
		pthread_mutex_init(&(msgInBlock[i]),NULL);
		pthread_mutex_init(&(msgOutBlock[i]),NULL);
		pthread_cond_init (&(msgOutCond[i]), NULL);
		pthread_mutex_init(&(msgOutCondBlock[i]), NULL);
	}
	// Set up connection with WSS
	struct sockaddr_in pin;
	struct hostent *server_host_name;
	server_host_name = gethostbyname(wssIP);
	bzero(&pin, sizeof(pin));
	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = htonl(INADDR_ANY);
	pin.sin_addr.s_addr = ((struct in_addr *)(server_host_name->h_addr))->s_addr;
	pin.sin_port = htons(50000);
	socket_descriptor = socket(AF_INET, SOCK_STREAM, 0);
	connect(socket_descriptor, (sockaddr *)&pin, sizeof(pin));

	// Initialize the WSS
	char wssName[50];
	if(myID==0) sprintf(wssName,"CommStation");
	else sprintf(wssName,"Robot%d",myID);
	if(!SendInit(wssName, 40001+myID))
		printf("Problem  registering with WSS when sending INIT command.\n");

	printf("Done with INIT.\n");

	pthread_t threadID;
	CommArgs* a = new CommArgs();
	a->obj=this;
	int threadStatus = pthread_create(&threadID, NULL, Communications::startListenForConnections, a);
	//printf("after thread started\n");
	for(int i=myID+1;i<ROBOT_COUNT;i++){
		CommArgs* args2 = new CommArgs();
		args2->sockfd=-1;
		args2->robID=i;
		args2->obj=this;
		pthread_t threadIDConnect2;
		pthread_create(&threadIDConnect2,NULL,Communications::startManageIncomingMessages,args2);
	}
}

/******************************
WSS Functions
******************************/
bool Communications::SendInit(string RobotName, int Port)
{
	char response[8192];
	char sPort[33];
	bool result = false;

	sprintf(sPort, "%d", Port);
	string cmd = "INIT {Robot ";
	cmd.append(RobotName);
	cmd.append("} {Port ");
	cmd.append(sPort);
	cmd.append("}\r\n");

	pthread_mutex_lock(&(wssCommBlock));
	// Send Command
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);

	// Get Response
	//recv(socket_descriptor, response, 8192, 0);  
	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}
	pthread_mutex_unlock(&(wssCommBlock));
	// Evaluate Response
	if(string(response).compare(INITResponse())==0)
		result = true;

	return result;
}

float Communications::GetSignalStrength(string RobotName)
{

	char response[8192];
	string cmd = "GETSS {Robot ";//100;
	cmd.append(RobotName);
	cmd.append("}\r\n");

	//printf("Sending Command: %s\n", cmd.c_str());
	// Send Command
	pthread_mutex_lock(&(wssCommBlock));
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);
	//printf("Done Sending...\n");

	//printf("Receiving Response\n");

	//while(!done)
	// Get Response
	//recv(socket_descriptor, response, 8192, 0);  


	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}

	//printf("Done Receiving Response (Got: %s\n",response);
	pthread_mutex_unlock(&(wssCommBlock));
	return SignalStrengthResponse(string(response));
}

int Communications::getRobotDNS(string RobotName){
	pthread_mutex_lock(&(wssCommBlock));
	char response[8192];
	string cmd = "DNS {Robot ";//100
	cmd.append(RobotName);
	cmd.append("}\r\n");

	//printf("Sending Command: %s\n", cmd.c_str());
	// Send Command
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);
	//printf("Done Sending...\n");

	//printf("Receiving Response\n");

	//while(!done)
	// Get Response
	//recv(socket_descriptor, response, 8192, 0);  


	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}

	//printf("Done Receiving Response ( Got: %s )\n",response);
	pthread_mutex_unlock(&(wssCommBlock));
	return DNSResponse(response);//SignalStrengthResponse(string(response));	
}
int Communications::getRobotReverseDNS(int port){
	pthread_mutex_lock(&wssCommBlock);
	char response[8192];
	char c[255];
	sprintf(c,"REVERSEDNS {Port %d}\r\n",port);
	string cmd(c);

	//printf("Sending Command: %s\n", cmd.c_str());
	// Send Command
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);
	//printf("Done Sending...\n");

	//printf("Receiving Response\n");

	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}

	printf("Done Receiving Response Got: %s",response);
	pthread_mutex_unlock(&wssCommBlock);
	return ReverseDNSResponse(response);//SignalStrengthResponse(string(response));	
}
string Communications::INITResponse()
{
	return "INITREPLY {Status OK}\r\n";
}

float Communications::SignalStrengthResponse(string msg)
{
	float result;
	string tmp = msg.substr(msg.find("{Strength ") + 10, msg.length() - 1 - msg.find("{Strength ") - 10);

	if(tmp.compare("NaN")==0)
	{
		result = 100;
	}
	else if(tmp.compare("INF")==0)
	{
		result = 101;
	}
	else
	{
		sscanf(tmp.c_str(), "%f", &result);
	}

	return result;
}

int Communications::DNSResponse(string msg)
{
	int result;

	string tmp = msg.substr(msg.find("{Port ") + 6, msg.length() - 1 - msg.find("{Port ") - 6);
	sscanf(tmp.c_str(), "%d", &result);

	return result;
}

int Communications::ReverseDNSResponse(string msg)
{
	int result;
	if(msg.find("CommStation")!=string::npos) return 0;
	if(msg.find("UnknownOrIllegalPort")!=string::npos) return -1;
	string tmp = msg.substr(msg.find("{Robot Robot") + 12, msg.length() - 1 - msg.find("{Robot Robot") - 12);
	//printf("I got %s from %s\n",tmp.c_str(),msg.c_str());
	sscanf(tmp.c_str(), "%d", &result);

	return result;
}

void* Communications::listenForConnections(void* arg){
	delete (CommArgs*)arg;
	int sockfd, newsockfd, portno, clilen;
	struct sockaddr_in serv_addr, cli_addr;
	portno = 40001+myID;
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0) 
		printf("ERROR opening socket\n");
	int optval = 1;
	setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *) &optval, sizeof (optval));
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = INADDR_ANY;
	serv_addr.sin_port = htons(portno);
	if (bind(sockfd, (struct sockaddr *) &serv_addr,
		sizeof(serv_addr)) < 0) 
		printf("ERROR on binding when trying to Listen\n");
	listen(sockfd,5);
	clilen = sizeof(cli_addr);
	listen_sockfd = sockfd;
	while(true){
		printf("Waiting for connection on port %d\n",portno);

		newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, 
			(socklen_t*)&clilen);
		if (newsockfd < 0) 
			printf("ERROR on accept\n");
		else{
			CommArgs* args1 = new CommArgs();
			CommArgs* args2 = new CommArgs();
			args1->sockfd=newsockfd;
			args1->robID=getRobotReverseDNS(htons(((struct sockaddr_in *)&cli_addr)->sin_port));;
			args1->obj=this;
			args2->sockfd=newsockfd;
			args2->robID=args1->robID;
			args2->obj=this;
			if(args1->robID>=0){
				printf("Got connection from Robot %u\n",args1->robID);
				robotcomm_sockfd[args1->robID] = newsockfd;
				pthread_t threadID;
				int threadStatus=pthread_create(&threadID,NULL, Communications::startManageIncomingMessages,args1);
				pthread_t threadID2;
				threadStatus=pthread_create(&threadID2,NULL, Communications::startManageOutgoingMessages,args2);
			}else{
				printf("Connection Accept Error (invalid robID)\n");
			}
		}
	}
}
void* Communications::manageOutgoingMessages(void* arg){
	int sockfd = ((CommArgs*)arg)->sockfd;
	int robID = ((CommArgs*)arg)->robID;
	delete (CommArgs*)arg;
	while(true){
		pthread_cond_wait(&(msgOutCond[robID]),&(msgOutCondBlock[robID]));
		pthread_mutex_lock(&(msgOutBlock[robID]));
		while(!msgsOut[robID].empty()){
			int s = send(sockfd, msgsOut[robID].front().c_str(), strlen(msgsOut[robID].front().c_str()), 0);
			if(s<0){
				printf("error sending to %d\n",robID);
				sockfd=-1;
				break;
			}else{
				msgsOut[robID].pop();
			}
		}
		pthread_mutex_unlock(&(msgOutBlock[robID]));
		//sleep(1);
	}
	pthread_exit(0);
}
void* Communications::manageIncomingMessages(void* arg){
	int sockfd = ((CommArgs*)arg)->sockfd;
	int robID = ((CommArgs*)arg)->robID;
	delete (CommArgs*)arg;
	char buffer[256];
	while(true){
		if(sockfd==-1){
			char rName[33];
			struct sockaddr_in pin;
			struct hostent *server_host_name;
			if(robID==0) sprintf(rName,"CommStation");
			else sprintf(rName,"Robot%d",robID);
			string rNameString(rName);
			int rob2Port = getRobotDNS(rNameString);
			if(rob2Port < 0){
				printf("failed to get port for robot %d. Waiting 10 seconds before retry...\n",robID);
				sockfd = -1 ;
			}else{
				printf("Connecting to Robot %d on Port %d\n",robID,rob2Port);

				//connect to robot 2
				sockfd=-1;
				//struct sockaddr_in pin;
				//struct hostent *server_host_name;
				server_host_name = gethostbyname(WSSIP);
				bzero(&pin, sizeof(pin));
				pin.sin_family = AF_INET;
				pin.sin_addr.s_addr = htonl(INADDR_ANY);
				pin.sin_addr.s_addr = ((struct in_addr *)(server_host_name->h_addr))->s_addr;
				pin.sin_port = htons(rob2Port);
				sockfd = socket(AF_INET, SOCK_STREAM, 0);
				connect(sockfd, (sockaddr *)&pin, sizeof(pin));
				CommArgs* args2 = new CommArgs();
				args2->sockfd=sockfd;
				args2->robID=robID;
				args2->obj=this;
				robotcomm_sockfd[robID] = sockfd;
				pthread_t threadID2;
				int threadStatus=pthread_create(&threadID2,NULL, Communications::startManageOutgoingMessages,args2);
			}
		}
		while(sockfd !=-1){
			bzero(buffer,256);
			//printf("reading\n");
			int n = read(sockfd,buffer,255);
			if (n <= 0){
				printf("ERROR reading from socket for rob %d\n",robID);
				sockfd=-1;
				break;
			}else{
				buffer[n]='\0';
				string s(buffer);
				pthread_mutex_lock(&(msgInBlock[robID]));
				msgsIn[robID].push(s);
				pthread_mutex_unlock(&(msgInBlock[robID]));
			}
			//printf("Message From %d: %s\n",robID,buffer);
		}
		if(myID < robID) sleep(10);
		else break;
	}
	pthread_exit(0);
}

bool Communications::readIncomingMessage(int robID,string& msg){
	bool gotMsg = true;
	pthread_mutex_lock(&(msgInBlock[robID]));
	if(!msgsIn[robID].empty()){
		msg=msgsIn[robID].front();
		//printf("Got message from %d: %s\n",robID,msgsIn[robID].front().c_str());
		msgsIn[robID].pop();
	}else gotMsg = false;
	pthread_mutex_unlock(&(msgInBlock[robID]));
	return gotMsg;
}
void Communications::sendMessage(int robID,string msg){
	pthread_mutex_lock(&(msgOutBlock[robID]));
	msgsOut[robID].push(msg);
	pthread_mutex_unlock(&(msgOutBlock[robID]));
	pthread_cond_signal(&(msgOutCond[robID]));
}

void Communications::closeAllSockets()
{
	//shutdown the connections
	shutdown(socket_descriptor,2);
	shutdown(listen_sockfd,2);
	for(int i=0;i<ROBOT_COUNT;i++){
		shutdown(robotcomm_sockfd[i],2);
	}

	//close all the connections
	close(socket_descriptor);
	close(listen_sockfd);
	for(int i=0;i<ROBOT_COUNT;i++){
		close(robotcomm_sockfd[i]);
	}
}