// commStationShell interfaces with WSS using the wssIP found in the /etc/moast.ini file.
// It is used to send commands to each robot in the deployment challenge of Robocup 2009.

// User defines
#define COMM_STATION_PORT 39000
#define COMM_STATION_NAME "CommStation"
#define MOAST_INI_LOC "../etc/moast.ini"
#define DEPLOYMENT_COMMRANGE 7.0  //in meters

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stddef.h>		// sizeof(), NULL
#include <stdlib.h>		// getenv()
#include <fcntl.h>	        // open()
#include <sys/stat.h>           // S_IWUSR, etc.
#include <unistd.h>	        // close()
#include <string>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
#include "getopt.h"
#endif
#include <rcs.hh>               // INIFILE, etc.
#include <queue>
#include <iostream>
#include <fstream>
#include <limits.h>
// Communications
#include "communications.h"

//deployment plan
#include "deploymentplan.h"

//robocup map
#include "robocupmap.h"

#define doDebug false

using namespace std;

int main(int argc, char * argv[]);
static bool readConfigFile();

static char wssIP[33];

string robotStartPoses[9] =  {"CommStation","robot1","robot2","Robot3","Robot4","Robot5","Robot6","Robot7","Robot8"};

int robotStartPointX[9];
int robotStartPointY[9];

int main(int argc, char * argv[])
{
	int* dummy;
	// Read in the configuration file
	if(!readConfigFile()) 
	{
		printf("Failed to read configuration file.\n");
		return 0;
	}

	printf("Config file read successfully.\n");

	// Set up Communications

	Communications comm(0, wssIP);

	int opt = 0;
	if(doDebug){
		cout << "input option" << endl;
		cout << "0 - send messages to robots" << endl;
		cout << "1 - start deployment (WITH moast path planning only)" << endl;
		cout << "2 - start deployment (WITH multiwaypoint generation)" << endl;
		cout << "3 - send messages to a specific robot" << endl;
		cin >> opt;
	}else{
		sleep(10);
		int confirmation = 3;
		while(confirmation>0){
			cout << endl << "*******Press Enter " <<confirmation << " more time(s) to start deployment**********" << endl << endl;
			cin.get();
			confirmation--;
		}
		opt = 2;
	}
	if(opt == 0){
		while(true){
			string msg;
			cin >> msg;
			for(int i=0;i<Communications::ROBOTCOUNT;i++){
				comm.sendMessage(i,msg);
			}
		}
	}else if(opt == 1 || opt == 2){
		RobocupMap m;
		m.readMapFromTIFF("CommOverhead.tif","CommOverhead.tfw");

		cout << "robocup map read" << endl;

		int dpRange = int(DEPLOYMENT_COMMRANGE / m.xScale);
		cout << "Deployment range set to " << DEPLOYMENT_COMMRANGE << " meters (" << dpRange << " map cells)" << endl;
		DeploymentPlan dp(dpRange);
		cout << "loading voronoi" << endl;
		dp.loadVoronoiFromBMP("voronoi_IMPRINT.BMP");

		//find the comm station start location
		cout << "finding start location" << endl;
		SimplePoint *startLocation = new SimplePoint(0,0);
		string line;
		char startName[255];
		float startX;
		float startY;
		fstream startlocs("startlocations.txt"); // input
		while (! startlocs.eof() ){
			getline (startlocs,line);
			sscanf(line.c_str(),"%s %f %f",startName,&startX,&startY);
			for(int i=0;i<9;i++){
				if(strcmp(startName,robotStartPoses[i].c_str()) == 0){
					robotStartPointX[i] = (int)((startY - m.xShift)/m.xScale);
					robotStartPointY[i] = (int)((startX - m.yShift)/m.yScale);		
				}
			}
			if(strcmp(startName,"CommStation") == 0){
				startLocation->x = (int)((startY - m.xShift)/m.xScale);
				startLocation->y = (int)((startX - m.yShift)/m.yScale);
				cout << "CommStation start location set to:(" << startX << "," << startY << ")" << endl;
			}
		}
		startlocs.close();
		int searchType = 1;
		float *destX = new float[Communications::ROBOTCOUNT];
		float *destY = new float[Communications::ROBOTCOUNT];

		int deployOrderR[8];
		int deployOrderP[8];

		if(doDebug){
			cout << "Select a search Type" << endl;
			cout << " 0: Chain Deployment" << endl;
			cout << " 1: Random Search Deployment" << endl;
			cout << "choice:";
			cin >> searchType;
		}
		if(searchType == 0){
			cout << "begining  search for " << (Communications::ROBOTCOUNT - 1) << " robots" << endl;
			dp.runSearch(startLocation,0,Communications::ROBOTCOUNT - 1);
			while(dp.bestPoints->size() < Communications::ROBOTCOUNT - 1 && dp.pixelRange > 0){
				dp.pixelRange--;
				dp.runSearch(startLocation,0,Communications::ROBOTCOUNT - 1);
			}
			cout << "search complete: reading in points" << endl;
			list<SimplePoint*>::iterator it;
			int rNum = 0;
			for ( it=dp.bestPoints->begin() ; it != dp.bestPoints->end(); it++ ){
				if(rNum>Communications::ROBOTCOUNT-1) break;
				destX[rNum] = (*it)->x;
				destY[rNum] = (*it)->y;
				rNum++;
			}
		}else{
			dp.searchRadius = dp.pixelRange;
			dp.communicationRadius = dp.pixelRange;
			dp.bestPoints = dp.calculateDeployment(startLocation,Communications::ROBOTCOUNT,20000);
			while(dp.bestPoints->size() < Communications::ROBOTCOUNT - 1 && dp.searchRadius > 0){
				dp.searchRadius--;
				dp.bestPoints = dp.calculateDeployment(startLocation,Communications::ROBOTCOUNT,2000);
			}
			delete startLocation;
			cout << "search complete: reading in points" << endl;
			list<SimplePoint*>::reverse_iterator it;
			int rNum = 0;
			for ( it = dp.bestPoints->rbegin() ; it != dp.bestPoints->rend(); it++ ){
				if(rNum>Communications::ROBOTCOUNT-1) break;
				destX[rNum] = (*it)->x;
				destY[rNum] = (*it)->y;
				rNum++;
			}
			//attempt to get a search distance....FUN
			int**distanceMatrix = new int*[Communications::ROBOTCOUNT-1];
			for(int r = 0; r < Communications::ROBOTCOUNT-1; r++){
				distanceMatrix[r] = new int[Communications::ROBOTCOUNT-1];
				for(int p = 0; p < Communications::ROBOTCOUNT-1; p++){
					SimplePoint * s = new SimplePoint(robotStartPointX[r+1],robotStartPointY[r+1]);
					SimplePoint * g = new SimplePoint(int(destX[p+1]),int(destY[p+1]));
					printf("searching from (%d,%d) to (%d,%d)\n",s->x,s->y,g->x,g->y);
					if(!dp.generateWaypointsToGoal(s,g))printf("failed\n");
					distanceMatrix[r][p] = dp.searchDistance;
				}
			}
			for(int r = 0; r < Communications::ROBOTCOUNT-1; r++){
				printf("%d:",r);
				for(int p = 0; p < Communications::ROBOTCOUNT-1; p++){
					printf("%d\t",distanceMatrix[r][p]);
				}
				printf("\n");
			}
			for(int i = 0; i < Communications::ROBOTCOUNT-1; i++){
				int minValue[Communications::ROBOTCOUNT-1];
				int minRobot[Communications::ROBOTCOUNT-1];
				//find the min paths for each point
				for(int p = 0; p < Communications::ROBOTCOUNT-1; p++){
					int m = INT_MAX;
					int mr = 0;
					//check each robot
					for(int r = 0; r < Communications::ROBOTCOUNT-1; r++){
						if(distanceMatrix[r][p] != -1 && distanceMatrix[r][p] < m){
							m = distanceMatrix[r][p];
							mr = r;
						}
					}
					if(m == INT_MAX) m = -1;
					minValue[p] = m;
					minRobot[p] = mr;
					printf("Robot %d has the min value of %d for point %d\n",mr,m,p);
				}
				int maxDist = -1;
				int maxPoint = 0;
				for(int p = 0; p < Communications::ROBOTCOUNT-1; p++){
					int m = -1;
					int mp = 0;
					if(minValue[p] > maxDist){
						maxDist = minValue[p];
						maxPoint = p;
					}
				}
				printf("#%d set to point %d and Robot %d\n",i,maxPoint, minRobot[maxPoint] + 1);
				deployOrderP[i] = maxPoint + 1;
				deployOrderR[i] = minRobot[maxPoint] + 1;
				for(int r = 0; r < Communications::ROBOTCOUNT-1; r++){
					distanceMatrix[r][maxPoint] = -1;
				}
				for(int p = 0; p < Communications::ROBOTCOUNT-1; p++){
					distanceMatrix[minRobot[maxPoint]][p] = -1;
				}
				printf("*****************************\n");
				for(int r = 0; r < Communications::ROBOTCOUNT-1; r++){
					printf("%d:",r);
					for(int p = 0; p < Communications::ROBOTCOUNT-1; p++){
						printf("%d\t",distanceMatrix[r][p]);
					}
					printf("\n");
				}
			}
			printf("Deployment Ordering!\n");
			for(int i = 0; i < Communications::ROBOTCOUNT-1; i++){
				printf("Sending robot %d to Point %d (%d,%d)\n",deployOrderR[i],deployOrderP[i],int(destX[deployOrderP[i]]),int(destY[deployOrderP[i]]));
			}
		}
		dp.writeSolutionToBMP(dp.bestPoints,"planimages/CommPlan.bmp");


		if(opt == 1){
			//convert coords to map coordinates
			for(int r=1;r<Communications::ROBOTCOUNT;r++){
				float tx = destX[r];
				float ty = destY[r];
				destY[r] = m.xShift + tx * m.xScale;
				destX[r] = m.yShift + ty * m.yScale;
			}
		}	
		//Tell all the robots to init themselves
		for(int r=Communications::ROBOTCOUNT-1;r>0;r--){
			cout << "sending init command to robot " << r << endl;
			string cmd("init");
			comm.sendMessage(r,cmd);
		}
		sleep(30);

		//move everyone
		if(searchType == 0){
			for(int r=Communications::ROBOTCOUNT-1;r>0;r--){
				string cmd("spgo");
				comm.sendMessage(r,cmd);
				cout << "sending spgo command to robot " << r << endl;
				sleep(10);
				char ccmd[200];
				if(opt == 1){
					cout << "sending move command to robot " << r << ": (" << destX[r] << "," << destY[r] << ")" << endl;
					sprintf(ccmd,"move_%f_%f",destX[r],destY[r]);
				}
				else{
					cout << "sending movepov command to robot " << r << ": (" << destX[r] << "," << destY[r] << ")" << endl;
					sprintf(ccmd,"movepov_%f_%f",destX[r],destY[r]);
				}
				string cmd2(ccmd);
				comm.sendMessage(r,cmd2);
				sleep(50);
			}
		}else{
			printf("Sending with Calculated Order\n");
			for(int i=0;i<Communications::ROBOTCOUNT-1;i++){
				int tx = int(destX[deployOrderP[i]]);
				int ty = int(destY[deployOrderP[i]]);
				int r = deployOrderR[i];
				string cmd("spgo");
				comm.sendMessage(r,cmd);
				cout << "sending spgo command to robot " << r << endl;
				sleep(10);
				char ccmd[200];
				cout << "sending movepov command to robot " << r << ": (" << tx << "," << ty << ")" << endl;
				sprintf(ccmd,"movepov_%d_%d",tx,ty);
				string cmd2(ccmd);
				comm.sendMessage(r,cmd2);
				sleep(50);
			}
		}
		delete destX;
		delete destY;
	}
	else{
		while(true){
			int rID;
			cin >> rID;
			if(rID ==0 ) break;
			string msg;
			cin >> msg;
			comm.sendMessage(rID,msg);
		}
	}
	cout << "Press 1 then enter to quit the program" << endl;
	int q;
	cin >> q;
	comm.closeAllSockets();
	return 0;
}



static bool readConfigFile()
{
	INIFILE if_d;
	const char *strPtr;

	if (if_d.open (MOAST_INI_LOC))
	{
		printf
			("** ERROR ** <commStationShell::readConfigFile> Inifile %s not Found \n\t",
			MOAST_INI_LOC);
		return false;
	}

	if (if_d.isSection ("USARSIM.INFSKIN"))
	{
		printf
			("** ERROR ** <commStationShell::readConfigFile> : Section USARSIM.INFSKIN not found\n");
		if_d.close();
		return false;
	}

	if (NULL == (strPtr = if_d.find ("HOST_NAME", "USARSIM.INFSKIN")))
	{
		printf("** WARNING ** <commStationShell::readConfigFile> : Parameter HOST_NAME not found\n");
	}
	else
	{
		strcpy(wssIP, strPtr);
	}

	printf("Read wssIP from config file: %s\n", wssIP);

	return true;
}
