#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include "ulapi.h"
#include "simware.h"
#include "skin.h"

static int socket_fd = -1;

static int waiting_for_conf = 0;
static int waiting_for_geo = 0;

#define SOCKET_MUTEX_KEY 1
static void *socket_mutex = NULL;

#define BUFFERLEN 8
static int buildlen = BUFFERLEN;
static char *build;
static char *build_ptr;
static char *build_end;
static char cargoBeingMonitored = 0;

#define DELIMITER 10
#define MAX_MSG_LEN 256
#define MAX_TOKEN_LEN 256

#define PORT 3000
#define HOST "rogerrabbit.mel.nist.gov"

#define SHOPS_HOST "porky"

typedef struct {
  int socket_fd;
}CargoMonitorThreadArgs;

static char *get_head(char *msg, char *head)
{
  char *ptr = msg;

  while (isspace(*ptr)) ptr++;	/* skip over space  */
  if (*ptr == 0) return msg;	/* didn't find {  */

  /* copy everything until we hit space or {  */
  while (1) {
    if ((*ptr == 0) ||
	(*ptr == ',') ||
	(*ptr == '}')) {
      /* unexpected delimiter -- return the original pointer */
      return msg;
    }
    if (isspace(*ptr) ||
	(*ptr == '{')) {
      /* expected delimiter -- break to return current pointer */
      break;
    }
    /* else copy it */
    *head++ = *ptr++;
  }
  *head = 0;			/* null terminate  */

  return ptr;
}

static char *get_key(char *msg, char *key)
{
  char *ptr = msg;

  while ((*ptr != 0) && (*ptr != '{')) ptr++; /* find the {  */
  if (*ptr == 0) return msg;	/* didn't find the {  */
  ptr++;			/* skip over the {  */
  while (isspace(*ptr)) ptr++;	/* skip over any space  */

  /* copy everything until we hit space */
  while (1) {
    if ((*ptr == 0) ||
	(*ptr == ',') ||
	(*ptr == '{') ||
	(*ptr == '}')) {
      /* unexpected delimiter -- return the original pointer */
      return msg;
    }
    if (isspace(*ptr)) {
      /* expected delimiter -- break to return current pointer */
      break;
    }
    /* else copy it */
    *key++ = *ptr++;
  }
  *key = 0;			/* null terminate  */

  return ptr;
}

static char *get_value(char *msg, char *value)
{
  char *ptr = msg;

  while ((isspace(*ptr)) || (*ptr == ',')) ptr++; /* skip over delimiters  */

  /* copy everything until we hit a delimiter  */
  while (! isspace(*ptr)) {
    if (*ptr == 0) return msg;	/* didn't finish cleanly */
    if ((*ptr == ',') ||
	(*ptr == '}')) {
      break;
    }
    /* ok to copy  */
    *value++ = *ptr++;
  }
  *value = 0;			/* null terminate  */

  return ptr;
}

typedef struct _usarsim_class_struct {
  int did_conf;
  int did_geo;
  char *name;
  struct _usarsim_class_struct *next;
} usarsim_class_struct, *usarsim_class_list;

#define usarsim_class_init(CS) (CS) = malloc(sizeof(*(CS))); (CS)->did_conf = 0; (CS)->did_geo = 0; (CS)->name = NULL

static int usarsim_class_find(usarsim_class_list ptr, char *name, usarsim_class_list *where)
{
  if (NULL == ptr) return -1;

  while (NULL != ptr->name) {
    if (! strcmp(ptr->name, name)) {
      /* found it */
      *where = ptr;
      return 0;
    }
    ptr = ptr->next;
  }

  /* a new one */
  ptr->name = malloc(strlen(name) + 1);
  if (NULL == ptr->name) return -1;
  strcpy(ptr->name, name);
  ptr->did_conf = 0;
  ptr->did_geo = 0;

  ptr->next = malloc(sizeof(*ptr));
  if (NULL == ptr->next) return -1;
  ptr->next->name = NULL;

  *where = ptr;
  return 0;
}

static void usarsim_class_print(usarsim_class_list cs)
{
  usarsim_class_struct *ptr = cs;

  if (NULL == ptr) return;

  while (NULL != ptr->name) {
    printf("%s %d %d\n", ptr->name, ptr->did_conf, ptr->did_geo);
    ptr = ptr->next;
  }

  return;
}

static usarsim_class_list encoders;
static usarsim_class_list sonars;
static usarsim_class_list rangescanners;
static usarsim_class_list machines;

/*
  NFO {Gametype BotDeathMatch} {Level DM-Factory_250} {TimeLimit 0}
*/

static int handle_nfo(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  SEN {Time 5608.8500} {Type Sonar}

  {Name F1 Range 2.5098} {Name F2 Range 3.3153} {Name F3 Range 4.9950} {Name F4 Range 2.0774} {Name F5 Range 2.0780} {Name F6 Range 5.0000} {Name F7 Range 5.0000} {Name F8 Range 4.9978} {Name R1 Range 4.9978} {Name R2 Range 4.0775} {Name R3 Range 2.9782} {Name R4 Range 2.6137} {Name R5 Range 2.6148} {Name R6 Range 2.9942} {Name R7 Range 3.3052} {Name R8 Range 2.5098}
*/

static int handle_sen_sonar(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_SENSOR_SONAR;
  sw.data.sonar.number = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Sonar")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Time")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.time = d;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (0 != usarsim_class_find(sonars, token, &where)) {
	return -1;
      }
      /* set the simware name to be the name of the first in this set */
      if (simware_no_name(&sw)) simware_set_name(&sw, where->name);
      ptr = nextptr;
      /* expecting "Range" */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (strcmp(token, "Range")) return -1;
      ptr = nextptr;
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      if (sw.data.sonar.number >= SW_SENSOR_SONAR_MAX) {
	/* ran over the limit, so drop it */
      } else {
	sw.data.sonar.range[sw.data.sonar.number] = d;
	sw.data.sonar.number++;
      }
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

/*
  SEN {Time 5608.85} {Type RangeScanner} 

  {Name Scanner1} {Resolution 0.0174} {FOV 3.1415} {Range 11.3059,11.3014,11.3002,...}
*/

static int handle_sen_rangescanner(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_SENSOR_RANGESCANNER;
  sw.data.rangescanner.number = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "RangeScanner")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Time")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.time = d;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (0 != usarsim_class_find(rangescanners, token, &where)) {
	return -1;
      }
      simware_set_name(&sw, where->name);
      ptr = nextptr;
    } else if (! strcmp(token, "Resolution")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.resolution = d;
      ptr = nextptr;
      count++;
    } else if (! strcmp(token, "FOV")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.fov = d;
      ptr = nextptr;
      count++;
    } else if (! strcmp(token, "Range")) {
      /* look for one or more range values */
      while (1) {
	nextptr = get_value(ptr, token);
	if (nextptr == ptr) {
	  if (0 == sw.data.rangescanner.number) return -1; /* need at least one range value */
	  else break;
	}
	if (1 != sscanf(token, "%lf", &d)) return -1;
	if (sw.data.rangescanner.number >= SW_SENSOR_RANGESCANNER_MAX) {
	  /* drop it */
	} else {
	  sw.data.rangescanner.range[sw.data.rangescanner.number] = d;
	  sw.data.rangescanner.number++;
	}
	ptr = nextptr;
      }
      /* all ok, so credit the count */
      count++;
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

/*
  SEN {Type Encoder} 

  {Name ECLeft Tick 0} {Name ECRight Tick 0} {Name ECLeftBack Tick 0} {Name ECRightBack Tick 0}
*/
static int handle_sen_encoder(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr = ptr;
  int count = 0;
  double d;
  int i;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_SENSOR_ENCODER;
  sw.data.encoder.number = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Encoder")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Time")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.time = d;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (0 != usarsim_class_find(encoders, token, &where)) {
	return -1;
      }
      if (simware_no_name(&sw)) simware_set_name(&sw, where->name);
      ptr = nextptr;
      /* expecting Tick */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (strcmp(token, "Tick")) return -1;
      ptr = nextptr;
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%i", &i)) return -1;
      if (sw.data.encoder.number >= SW_SENSOR_ENCODER_MAX) {
	/* drop it */
      } else {
	sw.data.encoder.tick[sw.data.encoder.number] = i;
	sw.data.encoder.number++;
      }
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

/*
  SEN {Type Touch}

  {Name Touch Touch False}
*/
static int handle_sen_touch(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  SEN {Type CO2Sensor}

  {Name CO2} {Gas CO2} {Density 0.00}
*/
static int handle_sen_co2sensor(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  SEN {Time 5608.8500} {Type GroundTruth}

  {Name GroundTruth} {Location 12.61,-2.68,1.64} {Orientation 0.00,6.23,0.00}
*/

static int handle_sen_groundtruth(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  SEN {Type GPS}

  {Name GPS} {Latitude 39,8.0341,N} {Longitude 77,13.0001,W} {Fix 1} {Satellites 11}
*/

static int handle_sen_gps(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  SEN {Type INS}

  {Name INS} {Location 12.61,-2.68,1.84} {Orientation 0.00,0.00,0.00}
*/

static int handle_sen_ins(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  SEN {Type Odometry}

  {Name Odometry} {Pose 0.0000,0.0000,0.0000}
*/

static int handle_sen_odometry(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  SEN {Time 5609.4467} {Type VictSensor}

  {Status NoVictims}
*/

static int handle_sen_victsensor(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_sen(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    /* look for {Type <name>}, and pass the whole msg to the sensor */
    if (! strcmp(token, "Type")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (! strcmp(token, "Sonar")) {
	return handle_sen_sonar(msg, tell);
      } else if (! strcmp(token, "RangeScanner")) {
	return handle_sen_rangescanner(msg, tell);
      } else if (! strcmp(token, "Encoder")) {
	return handle_sen_encoder(msg, tell);
      } else if (! strcmp(token, "Touch")) {
	return handle_sen_touch(msg);
      } else if (! strcmp(token, "CO2Sensor")) {
	return handle_sen_co2sensor(msg);
      } else if (! strcmp(token, "GroundTruth")) {
	return handle_sen_groundtruth(msg);
      } else if (! strcmp(token, "GPS")) {
	return handle_sen_gps(msg);
      } else if (! strcmp(token, "INS")) {
	return handle_sen_ins(msg);
      } else if (! strcmp(token, "Odometry")) {
	return handle_sen_odometry(msg);
      } else if (! strcmp(token, "VictSensor")) {
	return handle_sen_victsensor(msg);
      } else {
	printf("Unknown sensor type %s\n", token);
	/* skip it and keep going */
      }
    }
    /* else something else to be handled by sensor, probably {Time #} */
  }

  return count;
}

/*
  STA {Type GroundVehicle}

  {Time 5609.45} {FrontSteer 0.0000} {RearSteer 0.0000} {LightToggle False} {LightIntensity 0} {Battery 1199} {View -1}
*/

static int handle_sta_groundvehicle(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  STA {Type BaseMachine}

  {Time 5609.45}
*/
static int handle_sta_basemachine(char *msg)
{
  // nothing to really do for this since it only reports the time
  return 1;
}

static int handle_sta(char *msg)
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    /* look for {Type <name>}, and pass the whole msg to the STA */
    if (! strcmp(token, "Type")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (! strcmp(token, "GroundVehicle")) {
	return handle_sta_groundvehicle(msg);
      } else if (! strcmp(token, "BaseMachine")) {
	return handle_sta_basemachine(msg);
      } else {
	printf("Unknown STA type %s\n", token);
	/* skip it and keep going */
      }
    }
    /* else something else, probably {Time #} */
  }

  return count;
}

/*
  MISSTA {Time 5609.45} {Name CameraPanTilt} {Link 1} {Value 0.0000} {Torque -20.00} {Link 2} {Value 0.0000} {Torque -20.00}
*/

static int handle_missta(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

/*
  RES {Type Effecter} {Name Loading} {Status OK}
*/
static int handle_res(char *msg, int (*tell)(simware_struct *))
{
  simware_struct sw;
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count = 0;

  simware_init(&sw);
  sw.type = SW_COMMAND_RESPONSE;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* expecting type */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      strncpy( sw.data.response.type, token, SW_NAME_MAX);
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      strncpy( sw.name, token, SW_NAME_MAX);
      ptr = nextptr;
    } else if (! strcmp(token, "CmdType")) {
      /* expecting type */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      strncpy( sw.data.response.type, token, SW_NAME_MAX);
      ptr = nextptr;
    } else if (! strcmp(token, "Status")) {
      /* expecting status */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (! strcmp(token, "Fail:" )){
	sw.data.response.status = -1;
	ptr = nextptr;
	/* expecting reason */
	while( *ptr != '}' ) {
	  sw.data.response.errorString[count++] = *ptr++;
	  if( count == SW_NAME_MAX ) return -1;
	}
	printf( "Falied due to: %s\n", sw.data.response.errorString );
      } else if (! strcmp(token, "OK" ))
	sw.data.response.status = 0;
      else {
	printf( "Res: unrecognized status %s\n", token );
	return -1;
      }
      ptr = nextptr;
    } else{
      printf("not handling %s\n", token);
    }
  }

    tell(&sw);
  return 0;
}

/*
  CONF {Type Sonar} {Name R8} {MaxRange 5.0000} {MinRange 0.1000} {BeamAngle 0.3491}
*/

static int handle_conf_sonar(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int sawname = 0;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_CONF_SONAR;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Sonar")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      if (sawname) tell(&sw);
      sawname = 1;
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (usarsim_class_find(sonars, token, &where) == 0) {
	simware_set_name(&sw, where->name);
	where->did_conf = 1;
      } else {
	return -1;
      }
      ptr = nextptr;
    } else if (! strcmp(token, "MaxRange")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.maxrange = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else if (! strcmp(token, "MinRange")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.minrange = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else if (! strcmp(token, "BeamAngle")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.beamangle = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

/*
  CONF {Type RangeScanner} {Name Scanner1} {MaxRange 20.0000} {MinRange 0.1000} {Resolution 0.0174} {Fov 3.1415} {Paning True} {Tilting False}
*/
static int handle_conf_rangescanner(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int sawname = 0;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_CONF_RANGESCANNER;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "RangeScanner")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      if (sawname) tell(&sw);
      sawname = 1;
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (usarsim_class_find(rangescanners, token, &where) == 0) {
	simware_set_name(&sw, where->name);
	where->did_conf = 1;
      } else {
	return -1;
      }
      ptr = nextptr;
    } else if (! strcmp(token, "MaxRange")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.maxrange = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else if (! strcmp(token, "MinRange")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.minrange = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else if (! strcmp(token, "Resolution")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.resolution = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else if (! strcmp(token, "Fov")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.fov = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

/*
  CONF {Type Encoder} {Name ECLeft} {Resolution 0.0174}
*/
static int handle_conf_encoder(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int sawname = 0;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_CONF_ENCODER;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Encoder")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      if (sawname) tell(&sw);
      sawname = 1;
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (usarsim_class_find(encoders, token, &where) == 0) {
	simware_set_name(&sw, where->name);
	where->did_conf = 1;
      } else {
	return -1;
      }
      ptr = nextptr;
    } else if (! strcmp(token, "Resolution")) {
      /* expecting number */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.encoder.resolution = d;
      ptr = nextptr;
      /* all ok, so credit the count */
      count++;
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

static int handle_conf_touch(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_conf_co2sensor(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_conf_groundtruth(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_conf_gps(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_conf_ins(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_conf_odometry(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_conf_victsensor(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_conf(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count = 0;

  waiting_for_conf = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    /* look for {Type <name>}, and pass the whole msg to the sensor */
    if (! strcmp(token, "Type")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (! strcmp(token, "Sonar")) {
	return handle_conf_sonar(msg, tell);
      } else if (! strcmp(token, "RangeScanner")) {
	return handle_conf_rangescanner(msg, tell);
      } else if (! strcmp(token, "Encoder")) {
	return handle_conf_encoder(msg, tell);
      } else if (! strcmp(token, "Touch")) {
	return handle_conf_touch(msg);
      } else if (! strcmp(token, "CO2Sensor")) {
	return handle_conf_co2sensor(msg);
      } else if (! strcmp(token, "GroundTruth")) {
	return handle_conf_groundtruth(msg);
      } else if (! strcmp(token, "GPS")) {
	return handle_conf_gps(msg);
      } else if (! strcmp(token, "INS")) {
	return handle_conf_ins(msg);
      } else if (! strcmp(token, "Odometry")) {
	return handle_conf_odometry(msg);
      } else if (! strcmp(token, "VictSensor")) {
	return handle_conf_victsensor(msg);
      } else {
	printf("Unknown conf type %s\n", token);
	/* skip it and keep going */
      }
    }
  }

  return count;
}

/*
  GEO {Type Sonar} {Name F2 Location 0.1850,-0.1150,0.0000 Orientation 0.0000,0.0000,-0.8727 Mount HARD}

  Why didn't the original USARSim designers use the same tag convention
  as for the CONF message?
*/

static int handle_geo_sonar(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int sawname = 0;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_GEO_SONAR;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Sonar")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      if (sawname) tell(&sw);
      sawname = 1;
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (usarsim_class_find(sonars, token, &where) == 0) {
	simware_set_name(&sw, where->name);
	where->did_geo = 1;
      } else {
	return -1;
      }
      ptr = nextptr;
      /* expecting "Location" */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Location")) return -1;
      ptr = nextptr;
      /* expecting three values */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.x = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.y = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.z = d;
      count++;
      ptr = nextptr;
      /* expecting "Orientation" */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Orientation")) return -1;
      ptr = nextptr;
      /* expecting three values */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.roll = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.pitch = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.sonar.yaw = d;
      count++;
      ptr = nextptr;
      /* ignore MOUNT HARD */
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

/*
  GEO {Type RangeScanner} {Name Scanner1 Location 0.1439,0.0000,-0.0920 Orientation 0.0000,0.0000,0.0000 Mount HARD}
*/

static int handle_geo_rangescanner(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int sawname = 0;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_GEO_RANGESCANNER;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "RangeScanner")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      if (sawname) tell(&sw);
      sawname = 1;
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (usarsim_class_find(rangescanners, token, &where) == 0) {
	simware_set_name(&sw, where->name);
	where->did_geo = 1;
      } else {
	return -1;
      }
      ptr = nextptr;
      /* expecting "Location" */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Location")) return -1;
      ptr = nextptr;
      /* expecting three values */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.x = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.y = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.z = d;
      count++;
      ptr = nextptr;
      /* expecting "Orientation" */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Orientation")) return -1;
      ptr = nextptr;
      /* expecting three values */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.roll = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.pitch = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.rangescanner.yaw = d;
      count++;
      ptr = nextptr;
      /* ignore MOUNT HARD */
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

/*
  GEO {Type Encoder} {Name ECRight Location 0.0000,0.0000,0.0000 Orientation 0.0000,0.0000,1.5707 Mount RightFWheel} < {Name ...} >
*/
static int handle_geo_encoder(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int sawname = 0;
  int count = 0;
  double d;
  usarsim_class_list where;
  simware_struct sw;

  simware_init(&sw);
  sw.type = SW_GEO_ENCODER;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "Type")) {
      /* double check our type */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Encoder")) return -1;
      ptr = nextptr;
    } else if (! strcmp(token, "Name")) {
      if (sawname) tell(&sw);
      sawname = 1;
      /* expecting name */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (usarsim_class_find(encoders, token, &where) == 0) {
	simware_set_name(&sw, where->name);
	where->did_geo = 1;
      } else {
	return -1;
      }
      ptr = nextptr;
      /* expecting "Location" */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Location")) return -1;
      ptr = nextptr;
      /* expecting three values */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.encoder.x = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.encoder.y = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.encoder.z = d;
      count++;
      ptr = nextptr;
      /* expecting "Orientation" */
      nextptr = get_value(ptr, token);
      if (strcmp(token, "Orientation")) return -1;
      ptr = nextptr;
      /* expecting three values */
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.encoder.roll = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.encoder.pitch = d;
      count++;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.encoder.yaw = d;
      count++;
      ptr = nextptr;
      /* ignore MOUNT RightFWheel */
    } else {
      /* skip unknown entry  */
      nextptr = get_value(ptr, token);
    }
  }

  tell(&sw);

  return count;
}

static int handle_geo_touch(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_geo_co2sensor(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_geo_groundtruth(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_geo_gps(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_geo_ins(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_geo_odometry(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_geo_victsensor(char *msg)
{
  static int didit = 0;

  if (! didit) {
    printf("not handling %s\n", msg);
    didit = 1;
  }
  return 0;
}

static int handle_geo(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count = 0;

  waiting_for_geo = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    /* look for {Type <name>}, and pass the whole msg to the sensor */
    if (! strcmp(token, "Type")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (! strcmp(token, "Sonar")) {
	return handle_geo_sonar(msg, tell);
      } else if (! strcmp(token, "RangeScanner")) {
	return handle_geo_rangescanner(msg, tell);
      } else if (! strcmp(token, "Encoder")) {
	return handle_geo_encoder(msg, tell);
      } else if (! strcmp(token, "Touch")) {
	return handle_geo_touch(msg);
      } else if (! strcmp(token, "CO2Sensor")) {
	return handle_geo_co2sensor(msg);
      } else if (! strcmp(token, "GroundTruth")) {
	return handle_geo_groundtruth(msg);
      } else if (! strcmp(token, "GPS")) {
	return handle_geo_gps(msg);
      } else if (! strcmp(token, "INS")) {
	return handle_geo_ins(msg);
      } else if (! strcmp(token, "Odometry")) {
	return handle_geo_odometry(msg);
      } else if (! strcmp(token, "VictSensor")) {
	return handle_geo_victsensor(msg);
      } else {
	printf("Unknown geo type %s\n", token);
	/* skip it and keep going */
      }
    }
  }

  return count;
}

static int handle_cargo(char *msg, int (*tell)(simware_struct *))
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  double d;
  simware_struct sw;

  sw.type = SW_SENSOR_CARGO;
  sw.data.cargo.numCargo = -1;
  //  printf( "%s", msg );
  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    ptr = nextptr;

    if (! strcmp(token, "ID")) {
      sw.data.cargo.numCargo++;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) break;
      sw.data.cargo.id[sw.data.cargo.numCargo] = atoi(token);
      ptr = nextptr;
    } else if (! strcmp(token, "Memory")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) break;
      sw.data.cargo.memory[sw.data.cargo.numCargo] = atoi(token);
      ptr = nextptr;
    } else if (! strcmp(token, "Location")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.cargo.x[sw.data.cargo.numCargo] = d;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.cargo.y[sw.data.cargo.numCargo] = d;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.cargo.z[sw.data.cargo.numCargo] = d;
      ptr = nextptr;
    } else if (! strcmp(token, "Orientation")) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.cargo.roll[sw.data.cargo.numCargo] = d;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.cargo.pitch[sw.data.cargo.numCargo] = d;
      ptr = nextptr;
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) return -1;
      if (1 != sscanf(token, "%lf", &d)) return -1;
      sw.data.cargo.yaw[sw.data.cargo.numCargo] = d;
      ptr = nextptr;
    } else {
      printf("Unknown cargo type %s\n", token);
      /* skip it and keep going */
      ptr = nextptr;
    }
  }
  sw.data.cargo.numCargo++;
  tell(&sw);
  return sw.data.cargo.numCargo;
}

static int handle_em(char *msg)
{
  char token[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr = ptr;
  int count = 0;

  while (1) {
    nextptr = get_key(ptr, token);
    if (nextptr == ptr) break;
    printf("key %s\n", token);
    ptr = nextptr;
    nextptr = get_value(ptr, token);
    if (nextptr == ptr) break;
    printf("value %s\n", token);
    ptr = nextptr;
    count++;
    while (1) {
      nextptr = get_value(ptr, token);
      if (nextptr == ptr) break;
      printf("value %s\n", token);
      ptr = nextptr;
    }
  }

  return count;
}

static int do_sen_confs(usarsim_class_list where, char *type)
{
  char str[MAX_MSG_LEN];

  if (NULL == where) return -1;

  while (NULL != where->name) {					
    if (! where->did_conf && ! waiting_for_conf) {
      sprintf(str, "GETCONF {Type %s} {Name %s}\r\n", type, where->name);
      ulapi_mutex_take(socket_mutex);
      ulapi_socket_write(socket_fd, str, strlen(str));
      ulapi_mutex_give(socket_mutex);		
      waiting_for_conf = 1;				
    } else if (! where->did_geo && ! waiting_for_geo) {
      sprintf(str, "GETGEO {Type %s} {Name %s}\r\n", type, where->name);
      ulapi_mutex_take(socket_mutex);
      ulapi_socket_write(socket_fd, str, strlen(str));
      ulapi_mutex_give(socket_mutex);		
      waiting_for_geo = 1;				
    }									
    where = where->next;						
  }									

  return 0;
}

static int do_eff_confs(usarsim_class_list where, char *type)
{
  char str[MAX_MSG_LEN];

  if (NULL == where) return -1;

  while (NULL != where->name) {					
    if (! where->did_conf && ! waiting_for_conf) {			
      sprintf(str, "Factory {CmdType GetConf} {ItemType %s}\r\n", type);
      ulapi_mutex_take(socket_mutex);					
      ulapi_socket_write(socket_fd, str, strlen(str));		
      ulapi_mutex_give(socket_mutex);					
      waiting_for_conf = 1;						
    } else if (! where->did_geo && ! waiting_for_geo) {		
      sprintf(str, "Factory {CmdType GetGeo} {ItemType %s}\r\n", type);
      ulapi_mutex_take(socket_mutex);					
      ulapi_socket_write(socket_fd, str, strlen(str));		
      ulapi_mutex_give(socket_mutex);					
      waiting_for_geo = 1;						
    }									
    where = where->next;						
  }

  return 0;
}

/*!
  \return Returns -1 on error, otherwise returns a number indicating how
  many elements were handled.
*/
static int handle_msg(char *msg, int (*tell)(simware_struct *))
{
  char head[MAX_TOKEN_LEN];
  char *ptr = msg;
  char *nextptr;
  int count;

  nextptr = get_head(ptr, head);
  if (nextptr == ptr) {
    printf("no head on %s\n", msg);
    return -1;
  }

  if (! strcmp(head, "NFO")) {
    count = handle_nfo(msg);
  } else if (! strcmp(head, "SEN")) {
    count = handle_sen(msg, tell);
  } else if (! strcmp(head, "STA")) {
    count = handle_sta(msg);
  } else if (! strcmp(head, "MISSTA")) {
    count = handle_missta(msg);
  } else if (! strcmp(head, "RES")) {
    count = handle_res(msg, tell);
  } else if (! strcmp(head, "CONF")) {
    count = handle_conf(msg, tell);
  } else if (! strcmp(head, "GEO")) {
    count = handle_geo(msg, tell);
  } else if (! strcmp(head, "CARGO")) {
    count = handle_cargo(msg, tell);
  } else {
    printf("%s?\n", msg);
    count = handle_em(msg);
  }

  do_sen_confs(encoders, "Encoder");
  do_sen_confs(sonars, "Sonar");
  do_sen_confs(rangescanners, "RangeScanner");

  do_eff_confs(machines, "Effectors");

  return count;
}

int init(simware_struct *initRobot)
{
  char default_init_string[] = "localhost 3000";
  int port = PORT;
  char *host = HOST;
  char str[255];
  char *hostname = NULL;

  if( initRobot->type != SW_COMMAND_INIT ){
    printf("usarsiminf: bad message to init\n" );
    return -1;
  }

  /* fix me!
  if (NULL == init_string || 0 == init_string[0]) init_string = default_init_string;

  printf("usarsiminf init: %s\n", init_string);
  */

  hostname = getenv("HOSTNAME");

  if (ULAPI_OK != ulapi_init(UL_USE_DEFAULT)) {
    return -1;
  }

  socket_mutex = ulapi_mutex_new(SOCKET_MUTEX_KEY);
  if (NULL == socket_mutex) {
    ulapi_socket_close(socket_fd);
    return -1;
  }

  if (NULL == hostname || strncmp(hostname, SHOPS_HOST, strlen(SHOPS_HOST)))
    {


  /* fix me 2
  host = malloc(strlen(init_string) + 1);
  if (2 != sscanf(init_string, "%s %i", host, &port)) {
    free(host);
    return -1;
  }
  */

  socket_fd = ulapi_socket_client(port, host);
  /* fix me 3
  free(host);
  */
  if (socket_fd < 0) {
    return -1;
  }

  sprintf( str, "INIT {ClassName %s} {Name %s} {Location %f %f %f} {Rotation %f %f %f}\r\n", 
	   initRobot->data.initRobot.robot_type, initRobot->data.initRobot.name, 
	   initRobot->data.initRobot.north, initRobot->data.initRobot.east, 
	   initRobot->data.initRobot.elevation, initRobot->data.initRobot.roll, 
	   initRobot->data.initRobot.pitch, initRobot->data.initRobot.yaw );
  ulapi_mutex_take(socket_mutex);
  ulapi_socket_write(socket_fd, str, strlen(str));
  ulapi_mutex_give(socket_mutex);

  build = (char *) malloc(buildlen * sizeof(char));
  build_ptr = build;
  build_end = build + buildlen;

  usarsim_class_init(encoders);
  usarsim_class_init(sonars);
  usarsim_class_init(rangescanners);
  usarsim_class_init(machines);

    }

  else
    {
      system( "sudo /sbin/parput 0x00" );
    }

  return 0;
}

int ask(int (*tell)(simware_struct *))
{
  char buffer[BUFFERLEN];
  char *buffer_ptr;
  char *buffer_end;
  ptrdiff_t offset;
  int nchars;

  nchars = ulapi_socket_read(socket_fd, buffer, BUFFERLEN);
  if (-1 == nchars) {		/* bad read */
    return -1;
  }
  if (0 == nchars) {		/* end of file */
    return -1;
  }
  buffer_ptr = buffer;
  buffer_end = buffer + nchars;

  while (buffer_ptr != buffer_end) {
    if (build_ptr == build_end) {
      offset = build_ptr - build;
      buildlen *= 2;
      build = (char *) realloc(build, buildlen * sizeof(char));
      build_ptr = build + offset;
      build_end = build + buildlen;
    }
    *build_ptr++ = *buffer_ptr;
    if (*buffer_ptr++ == DELIMITER) {
      offset = build_ptr - build;
      build_ptr = build;
      build[offset] = 0;
      if (handle_msg(build, tell) < 0) {
	fprintf(stderr, "error handling %s\n", build);
      }
    }
  }

  return 0;
}

void cargoMonitorThread( void *argIn )
{
  CargoMonitorThreadArgs *args = (CargoMonitorThreadArgs*)argIn;
  char str[MAX_MSG_LEN];


  free( args );
  while( 1 )
    {
      sprintf( str, "Factory {CmdType GetCargo}\r\n");
      ulapi_mutex_take(socket_mutex);
      ulapi_socket_write(socket_fd, str, strlen(str));
      ulapi_mutex_give(socket_mutex);
      //  printf( "Wrote %s to socket %d\n", str, socket_fd );
      sleep(1);
    }
}

int tell(simware_struct *sw)
{
  char str[MAX_MSG_LEN];
  void *cargoMonitorTask;
  CargoMonitorThreadArgs *args;
  static unsigned char parput = 0x0;
  char *hostname;

  hostname = getenv("HOSTNAME");

  switch (sw->type) {
  case SW_COMMAND_BOXCHUTE_RELEASE:

  if (NULL == hostname || strncmp(hostname, SHOPS_HOST, strlen(SHOPS_HOST)))
    {

    sprintf( str, "Factory {CmdType SET} {Name %s} {Opcode Release} {Params %f}\r\n",
	     sw->name, sw->data.boxchute.RFID_value);
    ulapi_mutex_take(socket_mutex);
    ulapi_socket_write(socket_fd, str, strlen(str));
    ulapi_mutex_give(socket_mutex);
    //    printf( "Writing %s to usarsim\n", str );
    /* since we added cargo, must now spawn a thread to monitor it */
    if( !cargoBeingMonitored ){
      args = (CargoMonitorThreadArgs*)malloc(sizeof(CargoMonitorThreadArgs));
      cargoMonitorTask = ulapi_task_new();
      ulapi_task_start( cargoMonitorTask,
			cargoMonitorThread,
			args,
			ulapi_prio_lowest(),
			1);
      cargoBeingMonitored = 1;
    }
    }
    break;
  case SW_COMMAND_CONVEYOR_SPEED:

  if (NULL == hostname || strncmp(hostname, SHOPS_HOST, strlen(SHOPS_HOST)))
    {

    sprintf( str, "Factory {CmdType SET} {Name %s} {Opcode Animate} {Params %f}\r\n", 
	       sw->name, sw->data.conveyor.speed );
    ulapi_mutex_take(socket_mutex);
    ulapi_socket_write(socket_fd, str, strlen(str));
    ulapi_mutex_give(socket_mutex);
    } else {
    if( !strncasecmp( sw->name, "ALPHA", 5 ) )
      {
	if( sw->data.conveyor.speed > 0 )
	  {
	    parput |= 0x4;
	    parput &= ~0x8;
	  }
	else if( sw->data.conveyor.speed < 0 )
	  {
	    parput &= ~0x4;
	    parput |= 0x8;
	  }
	else if( sw->data.conveyor.speed == 0 )
	  {
	    parput &= ~0xc;
	  }
      }
    else if( !strncasecmp( sw->name, "BETA", 4 ) )
      {
	if( sw->data.conveyor.speed > 0 )
	  {
	    parput |= 0x1;
	    parput &= ~0x2;
	  }
	else if( sw->data.conveyor.speed < 0 )
	  {
	    parput &= ~0x1;
	    parput |= 0x2;
	  }
	else if( sw->data.conveyor.speed == 0 )
	  {
	    parput &= ~0x3;
	  }
      }

      sprintf( str, "sudo /sbin/parput %d\n", parput );
      system( str );

  }
    break;
  case SW_COMMAND_INIT:
    init( sw );
    break;
  case SW_COMMAND_SKID:
    sprintf(str, "Drive {Left %f} {Right %f}\r\n", sw->data.skid.left, sw->data.skid.right);
    ulapi_mutex_take(socket_mutex);
    ulapi_socket_write(socket_fd, str, strlen(str));
    ulapi_mutex_give(socket_mutex);
    break;
  case SW_COMMAND_CLEAR_CARGO:
    sprintf(str, "Factory {CmdType ClearCargo}\r\n");
    ulapi_mutex_take(socket_mutex);
    ulapi_socket_write(socket_fd, str, strlen(str));
    ulapi_mutex_give(socket_mutex);
    break;
  default:
    printf("not handling %d\n", sw->type);
    break;
  }

  return 0;
}

int fini(void)
{
  printf("usarsiminf fini");

  return 0;
}
