#include "robocupmap.h"

using namespace std;

// Read in the file at pathname, storing it pixel-by-pixel in mapGrid[][].

bool RobocupMap::readMapFromTIFF(string pathname, string tfwPath)
{
	// Init the FreeImage library
	FreeImage_Initialise();

	// Instantiate a FreeImagePlus Wrapper
	fipImage mapImage;

	// Get the file format of the provided file.
	FREE_IMAGE_FORMAT fif = FreeImage_GetFileType(pathname.c_str());

	// Failed to identify the provided file; fail out.
	if(fif == FIF_UNKNOWN)
	{
		printf("File not found or invalid file type.\n");
		return false;
	}

	// Load the image into memory.
	bool retVal = mapImage.load(pathname.c_str());

	if(!retVal)
	{
		printf("Failed to load image %s\n", pathname.c_str());
		FreeImage_DeInitialise();
		return 0;
	}

	RGBQUAD color;  

	int dummy;
	mapWidth = mapImage.getWidth();
	mapHeight = mapImage.getHeight();
	mapGrid = new int*[mapImage.getWidth()];
	for(int i=0;i<mapImage.getWidth();i++) mapGrid[i] = new int[mapImage.getHeight()];
	// Read the map image into the array.
	for(unsigned int i=mapImage.getHeight(); i>0; i--)
	{
		for(unsigned int j=0; j<mapImage.getWidth(); j++)
		{
			mapImage.getPixelColor(j, i-1, &color);
			if(color.rgbRed==0 && color.rgbBlue==0 && color.rgbGreen==0)
			{
				dummy = 0;
			}
			else
				dummy = WALL;

			mapGrid[j][mapImage.getHeight()-i] = dummy;
		}
	}


	FreeImage_DeInitialise();

	//Now to read the Tfw

	fstream in(tfwPath.c_str()); // input

	if(!in) {
		cout << "Cannot open the tfw file.\n";
		return 0;
	}
	in >> xScale;
	in >> rot1;
	in >> rot2;
	in >> yScale;
	in >> xShift;
	in >> yShift;

	in.close();

	return 0;
}
float RobocupMap::px2m(int x)
{
	return xShift + x * xScale;
}
float RobocupMap::py2m(int y)
{
	return yShift + y * yScale - mapHeight*yScale;
}
int RobocupMap::mx2p(float x)
{
	return (int)((x - yShift+mapHeight*yScale)/yScale);
}
int RobocupMap::my2p(float y)
{
	return (int)((y - xShift)/xScale);
}
