/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file swKBEff.hh

  \brief 
  \code CVS Status:
  $Author: cj_scrapper $
  $Revision: 1.3 $
  $Date: 2007/11/20 15:41:11 $
  \endcode

  \author Chris Scrapper
*/

#ifndef SIMWARE_KB_EFFECTOR_HH
#define SIMWARE_KB_EFFECTOR_HH

#include <string>
#include <map>
using namespace std;

#include <posemath.h>
#include "swKBTypes.hh"
#include "swKBUtils.hh"
#include "swKBMis.hh"

class SIMwareEffOpcodeConfig
{
private:
  double maxVal;
  double minVal;
  SIMwareEffOpcodeType type;
public:
  SIMwareEffOpcodeConfig(SIMwareEffOpcodeType);
  ~SIMwareEffOpcodeConfig();
  double getMaxValue();
  void setMaxValue(double);
  double getMinValue();
  void setMinValue(double);
  SIMwareEffOpcodeType getOpcodeType();
  void dump();
};

class  SIMwareKBEff {
private: 
  static int effCount;
  //! Effectors name
  string effName;
  //! Effectors id
  int effID;
  //! Handle to mount, if NULL then mounted to vehicle
  SIMwareKBMount * mount;
  //! Transform from effecter to link mounted, or vehicle
  PM_POSE effToMount;


 

public:
  //! opcodes implemented by effecter and configuration information
  map <SIMwareEffOpcodeType,SIMwareEffOpcodeConfig *> opConfMap; 

 //! Constructor
  SIMwareKBEff();
  SIMwareKBEff(string);
  //! Destructor
  ~SIMwareKBEff();
  //! Sets the name for effector
  void setName(string);
  //! Gets the name for effector
  string getName();
  //! Sets the id of effector
  void setID(int);
  //! Sets the id of effector
  int getID();


  void setEff2MountXYZ(double, double, double);
  void setEff2MountRPY(double, double, double);
  PM_CARTESIAN getEff2MountXYZ();
  PM_RPY getEff2MountRPY();
  PM_POSE getEff2Mount ();
  void setMount(SIMwareKBMount *);
  SIMwareKBMount * getMount();

  SIMwareEffOpcodeConfig * getOpcodeConfig(SIMwareEffOpcodeType, bool=false);
};


#endif
