/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file swKBMis.hh

  \brief 
  \code CVS Status:
  $Author: cj_scrapper $
  $Revision: 1.4 $
  $Date: 2007/03/13 20:01:17 $
  \endcode

  \author Chris Scrapper
*/

#ifndef SIMWARE_KB_MISPKG_HH
#define SIMWARE_KB_MISPKG_HH

#include <string>
#include <map>
using namespace std;
#include <posemath.h>
#include "swKBTypes.hh"


class SIMwareKBLink;
class SIMwareKBMisPkg;

 
class SIMwareKBMount
{
public:
  SIMwareKBLink * link;
  SIMwareKBMisPkg * misPkg;
  SIMwareKBMount () : link(NULL), misPkg(NULL) {};
};

class SIMwareKBLink
{
private:
  int linkID;
  SIMwareKBLink * parentLink;
  SIMwareJointType jType;
  double maxSpeed;
  double maxTorque;
  double maxRange;
  double minRange;
  PM_CARTESIAN loc;
  PM_RPY rot;
  double jVal;
  double torque;

  
public:
  SIMwareKBLink();
  SIMwareKBLink(int);
  ~SIMwareKBLink();
  void setLinkID (int);
  int getLinkID ();
  void setJointType (SIMwareJointType);
  SIMwareJointType getJointType ();
  void setMaxSpeed(double);
  double getMaxSpeed ();
  void setMaxTorque(double);
  double getMaxTorque ();
  void setMaxRange(double);
  double getMaxRange ();
  void setMinRange(double);
  double getMinRange ();
  void setParentLink(SIMwareKBLink *);
  SIMwareKBLink * getParentLink();
  void setLocation (double,double,double);
  void setLocation (PM_CARTESIAN);
  PM_CARTESIAN getLocation ();

  void setOrientation (double,double,double);
  void setOrientation (PM_RPY);
  PM_RPY getOrientation ();
  void setJointVal (double);
  double getJointVal();
  void setTorque (double);
  double getTorque();
 
};


class  SIMwareKBMisPkg {
private: 
  static int misPkgCount;
  //! Mission packate name
  string mName;
  double timeStamp;
  int misPkgID;
public:
  // FIXME accessor
  map <int, SIMwareKBLink *> linkMap;
  
  //! Constructor
  SIMwareKBMisPkg();
  SIMwareKBMisPkg(string);
  //! Destructor
  ~SIMwareKBMisPkg();
  void setName (string);
  string getName();
  int getMisPkgID ();
  void setMisPkgID();

  
  SIMwareKBLink * getLink(int, bool=false);
  int getLinkCount();  
  void setTimeStamp (double);
  double getTimeStamp(); 
};

#endif
