#include "swKBMsg.hh"
////////////// BASE CMD MSG ///////////////////////////
int SIMwareCmdMsg::count=0;
SIMwareCmdMsg::SIMwareCmdMsg (SIMwareCmdType t) : cmdType(t) {msgID=count++;}
SIMwareCmdMsg::~SIMwareCmdMsg() {}
int SIMwareCmdMsg::getCmdType() {return cmdType;}
int SIMwareCmdMsg::getCmdID() {return msgID;}

////////////// MOBILITY CMD ///////////////////////////
SIMwareMobCmd::SIMwareMobCmd(SIMwareMobCmdType cmdType) :  SIMwareCmdMsg(SW_MOB_CMD_TYPE), mobCmdType(cmdType) {}
SIMwareMobCmdType SIMwareMobCmd::getMobCmdType() {return mobCmdType;}




////////////// FLIPPER STEERED ///////////////////////////
SIMwareFlipperCmd::SIMwareFlipperCmd() : SIMwareMobCmd(SW_MOB_CMD_DRV_FLIPPER_TYPE)
{
  fCmd.clear();
}

////////////// ACKERMAN STEERED ///////////////////////////
SIMwareAckermanCmd::SIMwareAckermanCmd() : SIMwareMobCmd(SW_MOB_CMD_DRV_ACKERMAN_TYPE)
{
  vLinear=0;
  steerAngle=0;
  crabAngle=0;
}

SIMwareAckermanCmd::SIMwareAckermanCmd(double lVel, double steerAng, double crabAng) : SIMwareMobCmd(SW_MOB_CMD_DRV_ACKERMAN_TYPE)
{
  vLinear=lVel;
  steerAngle=steerAng;
  crabAngle=crabAng;
}

////////////// SKID STEERED ///////////////////////////
SIMwareSkidCmd::SIMwareSkidCmd() : SIMwareMobCmd(SW_MOB_CMD_DRV_SKID_TYPE)
{
  wRight=0;
  wLeft=0;
}

SIMwareSkidCmd::SIMwareSkidCmd(double left,double right) : SIMwareMobCmd(SW_MOB_CMD_DRV_SKID_TYPE)
{
  wRight=right;
  wLeft=left;
}

////////////// OMNI DRIVE ///////////////////////////
SIMwareOmniCmd::SIMwareOmniCmd() :SIMwareMobCmd(SW_MOB_CMD_DRV_OMNI_TYPE)
{
  vLateral=0;
  vLinear=0;
  vRot=0;
}

SIMwareOmniCmd::SIMwareOmniCmd( double linear, double lateral, double rot) :SIMwareMobCmd(SW_MOB_CMD_DRV_OMNI_TYPE)
{
  vLateral= lateral;
  vLinear=linear;
  vRot=rot;
}

////////////// NATUIC DRIVE CMDS ///////////////////////////
SIMwareSubmarineCmd::SIMwareSubmarineCmd() :SIMwareMobCmd(SW_MOB_CMD_DRV_SUBMARINE_TYPE) 
{
  propeller=0;
  rudder=0;
  sternPlane=0;
}

SIMwareSubmarineCmd::SIMwareSubmarineCmd(double prop, double rud, double stern) :SIMwareMobCmd(SW_MOB_CMD_DRV_SUBMARINE_TYPE) 
{
  propeller=prop;
  rudder=rud;
  sternPlane=stern;
}

////////////// AERIAL DRIVE CMDS ///////////////////////////
SIMwareRotaryWingCmd::SIMwareRotaryWingCmd() :SIMwareMobCmd(SW_MOB_CMD_DRV_ROTARY_WING_TYPE) 
{
  vLinear=0.00;   
  vLateral=0.00;  
  vVertical=0.00; 
  vRotation=0.00; 
}
SIMwareRotaryWingCmd::SIMwareRotaryWingCmd(double vLin, double vLat, double vVert, double vRot) :SIMwareMobCmd(SW_MOB_CMD_DRV_ROTARY_WING_TYPE) 
{
  vLinear=vLin;   
  vLateral=vLat;  
  vVertical=vVert; 
  vRotation=vRot; 
}


////////////// MISPKG CMD   ///////////////////////////
SIMwareMisLinkCmd::SIMwareMisLinkCmd() 
{
  linkID=0;
  value=0.00;
  type=SW_MIS_LINK_CMD_INVALID_TYPE;
}
SIMwareMisLinkCmd::SIMwareMisLinkCmd(int id, double val, SIMwareMisLinkCmdType cmdType) 
{
  linkID = id;
  value = val;
  type = cmdType;
}

SIMwareMisCmd::SIMwareMisCmd(SIMwareMisCmdType cmdType) :  SIMwareCmdMsg(SW_MIS_CMD_TYPE), misCmdType(cmdType) {};
SIMwareMisCmdType SIMwareMisCmd::getMisCmdType() {return misCmdType;};

SIMwareMisMoveCmd::SIMwareMisMoveCmd() :SIMwareMisCmd(SW_MIS_CMD_MOVE_TYPE)  {}



////////////// EFFECTER CMD   ///////////////////////////

SIMwareEffCmd::SIMwareEffCmd(SIMwareEffCmdType cmdType) :  SIMwareCmdMsg(SW_EFF_CMD_TYPE), effCmdType(cmdType) {};
SIMwareEffCmdType SIMwareEffCmd::getEffCmdType() {return effCmdType;};

SIMwareEffOpcodeCmd::SIMwareEffOpcodeCmd() :SIMwareEffCmd(SW_EFF_CMD_OPCODE_TYPE)  
{
  param =0.00;
  opcode=SW_EFF_OPCODE_INVALID_TYPE;
  effName.clear();
  
}

SIMwareEffOpcodeCmd::SIMwareEffOpcodeCmd(string name, SIMwareEffOpcodeType type, double value) :SIMwareEffCmd(SW_EFF_CMD_OPCODE_TYPE)  
{
  effName=name;
  opcode=type;
  param=value;
}

////////////// SENSOR PROCESSING CMD ///////////////////////////
SIMwareSenCmd::SIMwareSenCmd(SIMwareSenCmdType cmdType) :  SIMwareCmdMsg(SW_SEN_CMD_TYPE), senCmdType(cmdType) {};
SIMwareSenCmdType SIMwareSenCmd::getSenCmdType() {return senCmdType;};

///// DATA MESSAGE
SIMwareDataMsg::SIMwareDataMsg() {}
double 
SIMwareDataMsg::getMsgTime ()
{
  return time;
}

void 
SIMwareDataMsg::setMsgTime (double t)
{
  time=t;
}

///// RFID MESSAGE
SIMwareRfidMsg::SIMwareRfidMsg ()
{
  rfidID = -1;
}

SIMwareRfidMsg::SIMwareRfidMsg (int id, string data)
{
  rfidID = id;
  rfidData = data;
}

///// Victim RFId msg
SIMwareVictimMsg::SIMwareVictimMsg ()
{
}

SIMwareVictimMsg::SIMwareVictimMsg (SIMwareVictimPartType pType, 
				    PM_CARTESIAN ploc)
{
  partType = pType;
  partLoc = ploc;

}

