#include "swKBTypes.hh"
#include "swServoMob.hh"
#include "moastSupSkin.hh"

SIMwareServoMob::SIMwareServoMob(MoastSupSkin * m) : 
  SIMwareServoNodeBase(m)
{
  nodeName="ServoMob";
  switch(mSkin->kb->bot->getBotType())
    {
    case SW_KB_GROUND_VEHICLE_TYPE:
      setMsg=new ServoMobJASetGrdVeh;
      statMsg=new ServoMobJAStatGrdVeh;
      break;
    case SW_KB_AERIAL_ROBOT_TYPE:    
      setMsg=new ServoMobJASetAirBot;
      statMsg=new ServoMobJAStatAirBot;
      break;
    case SW_KB_LEGGED_ROBOT_TYPE:
    case SW_KB_NAUTIC_ROBOT_TYPE: 
    default:
      setMsg=new ServoMobJASet;
      statMsg=new ServoMobJAStat;
      break;
    }
  statMsg->admin_state=ADMIN_UNINITIALIZED;
  CLASS_CAST(ServoMobJAStat *, statMsg)->heartbeat=0;
  CLASS_CAST(ServoMobJASet *, setMsg)->cycleTime=SERVO_NODE_CYCLE_TIME_DEFAULT;
  CLASS_CAST(ServoMobJASet *, setMsg)->debug=0;
}

SIMwareServoMob::~SIMwareServoMob() 
{
  if(0)
    printf("<SIMwareServoMob::~SIMwareServoMob>\n");
  if(statMsg)
    {
      delete statMsg;
      statMsg=0;
    }
  if(setMsg)
    {
      delete setMsg;
      setMsg=0;
    }
}

void SIMwareServoMob::doSingleStep() 
{
  if(0)
    printf("<SIMwareServoMob::doSingleStep>\n");
  int serialNumber;
  int cfgType = 0;
  int cmdType = 0;
  // Read Configuration Message
  cfgType = cfgBuf->read();
  serialNumber = cfgBuf->get_address()->serial_number;
  switch (cfgType) {
    
  case 0:
    // no new cmd
    break;
    
  case -1:
    rcs_print_error("NML cmd read error\n");
    break;
    
  case SERVO_MOB_JA_CFG_CYCLE_TIME_TYPE:
  case SERVO_MOB_JA_CFG_TELEPORT_TYPE:
    if (serialNumber != setMsg->echo_serial_number) 
      {
	setMsg->command_type = cfgType;
	setMsg->echo_serial_number = serialNumber;
	setMsg->state = NEW_COMMAND;
	setMsg->status = RCS_EXEC;
      }
    break;
    
  default:
    rcs_print_error("unknown cmd: %d\n", (int) cfgType);
    break;
  }	
  
  
  // Read Command Message 
  cmdType = cmdBuf->read();
  serialNumber = cmdBuf->get_address()->serial_number;
  switch (cmdType) {
    
  case 0:
    // no new cmd
    break;
    
  case -1:
    // comm error
    rcs_print_error("NML cmd read error\n");
    break;
    
  case SERVO_MOB_JA_CMD_INIT_TYPE:
  case SERVO_MOB_JA_CMD_HALT_TYPE:
  case SERVO_MOB_JA_CMD_ABORT_TYPE:
  case SERVO_MOB_JA_CMD_SHUTDOWN_TYPE:
  case SERVO_MOB_JA_CMD_ACKERMAN_TYPE:
  case SERVO_MOB_JA_CMD_SUBMARINE_TYPE:
  case SERVO_MOB_JA_CMD_SKID_TYPE:  
  case SERVO_MOB_JA_CMD_FLIP_SKID_TYPE:  
  case SERVO_MOB_JA_CMD_OMNI_TYPE:  
  case SERVO_MOB_JA_CMD_ROTARY_WING_TYPE:  
    if (serialNumber != statMsg->echo_serial_number) 
      {
	statMsg->command_type = cmdType;
	statMsg->echo_serial_number = serialNumber;
	statMsg->state = NEW_COMMAND;
	statMsg->status = RCS_EXEC;
	// error stuff
	CLASS_CAST(ServoMobJAStat *, statMsg)->error=0;
	statMsg->message_length=0;
      }
    break;
  
  default:
    rcs_print_error("unknown cmd: %d\n", (int) cmdType);
    break;
  }
  
  
  // Handle Configuration Message
  switch( setMsg->command_type) {
  
  case SERVO_MOB_JA_CFG_CYCLE_TIME_TYPE:
    doMobCfgCycleTime(CLASS_CAST(ServoMobJACfgCycleTime *, cfgBuf->get_address()));
    break;
    
  case SERVO_MOB_JA_CFG_DEBUG_TYPE:
    doMobCfgDebug(CLASS_CAST(ServoMobJACfgDebug *, cfgBuf->get_address()));
    break;
    
  case SERVO_MOB_JA_CFG_TELEPORT_TYPE:
    doMobCfgTeleport(CLASS_CAST(ServoMobJACfgTeleport *, cfgBuf->get_address()));
    break;
  
  default:
    break;
  }
  
  // Handle Command Message
  switch (statMsg->command_type) {
    
  case SERVO_MOB_JA_CMD_INIT_TYPE:
    doMobCmdInit();
    break;
    
  case SERVO_MOB_JA_CMD_HALT_TYPE:
    doMobCmdHalt();
    break;
    
  case SERVO_MOB_JA_CMD_ABORT_TYPE:
    doMobCmdAbort();
    break;
    
  case SERVO_MOB_JA_CMD_SHUTDOWN_TYPE:
    doMobCmdShutdown();
    break;
    
  case SERVO_MOB_JA_CMD_ACKERMAN_TYPE:
    doMobCmdDrvAckerman(CLASS_CAST(ServoMobJACmdAckerman *, cmdBuf->get_address()));
    break;

  case SERVO_MOB_JA_CMD_SUBMARINE_TYPE:
    doMobCmdDrvSubmarine(CLASS_CAST(ServoMobJACmdSubmarine *, cmdBuf->get_address()));
    break;
    
  case SERVO_MOB_JA_CMD_SKID_TYPE:
    doMobCmdDrvSkid(CLASS_CAST(ServoMobJACmdSkid *, cmdBuf->get_address()));
    break;

  case SERVO_MOB_JA_CMD_FLIP_SKID_TYPE:
    doMobCmdDrvFlipSkid(CLASS_CAST(ServoMobJACmdFlipSkid *, cmdBuf->get_address()));
    break;

  case SERVO_MOB_JA_CMD_OMNI_TYPE:
    doMobCmdDrvOmni(CLASS_CAST(ServoMobJACmdOmni *, cmdBuf->get_address()));
    break;

  case SERVO_MOB_JA_CMD_ROTARY_WING_TYPE:
    doMobCmdDrvRotaryWing(CLASS_CAST(ServoMobJACmdRotaryWing *, cmdBuf->get_address()));
    break;

  default:
    // no cmd to handle
    break;
  }	
  
  if( cfgType > 0 )
    publishSettings();
  sendStatus();
}

void SIMwareServoMob::publishSettings()
{
  SIMwareKBFlipper * flipper;
  snprintf(CLASS_CAST(ServoMobJASet *, setMsg)->platformName,MOAST_PLATFORM_NAME_LEN,"%s",mSkin->kb->bot->getPlatformName().c_str());
  snprintf(CLASS_CAST(ServoMobJASet *, setMsg)->worldName,MOAST_WORLD_NAME_LEN,"%s",mSkin->kb->world.getWorldName().c_str());
  CLASS_CAST(ServoMobJASet *, setMsg)->length=(mSkin->kb->bot->getDimensions()).x;
  CLASS_CAST(ServoMobJASet *, setMsg)->width=(mSkin->kb->bot->getDimensions()).y;
  CLASS_CAST(ServoMobJASet *, setMsg)->height=(mSkin->kb->bot->getDimensions()).z;
  CLASS_CAST(ServoMobJASet *, setMsg)->mass=mSkin->kb->bot->getMass();
  CLASS_CAST(ServoMobJASet *, setMsg)->cg.x=(mSkin->kb->bot->getCentroid()).x;
  CLASS_CAST(ServoMobJASet *, setMsg)->cg.y=(mSkin->kb->bot->getCentroid()).y;
  CLASS_CAST(ServoMobJASet *, setMsg)->cg.z=(mSkin->kb->bot->getCentroid()).z;
  CLASS_CAST(ServoMobJASet *, setMsg)->steerType=mSkin->kb->bot->getSteerType();
  CLASS_CAST(ServoMobJASet *, setMsg)->botType=mSkin->kb->bot->getBotType();
  CLASS_CAST(ServoMobJASet *, setMsg)->effCount=mSkin->kb->effMap.size();
  CLASS_CAST(ServoMobJASet *, setMsg)->senCount=mSkin->kb->senMap.size();
  CLASS_CAST(ServoMobJASet *, setMsg)->misCount=mSkin->kb->misMap.size();
  
  // Get robot type
  switch(mSkin->kb->bot->getBotType())
    {
    case SW_KB_GROUND_VEHICLE_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->botType=MOAST_GROUND_VEHICLE_TYPE;
      break;
    case SW_KB_NAUTIC_ROBOT_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->botType=MOAST_NAUTIC_ROBOT_TYPE;
      break;
    case SW_KB_AERIAL_ROBOT_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->botType=MOAST_AERIAL_ROBOT_TYPE;
      break;
    case SW_KB_UNKNOWN_ROBOT_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_UNKNOWN_ROBOT_TYPE;
      break;
    default:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_INVALID_ROBOT_TYPE;
      break;
    }
  

  // Get steering type
  switch(mSkin->kb->bot->getSteerType())
    {
    case SW_KB_ACKERMAN_STEER_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_ACKERMAN_STEER_TYPE;
      break;
    case SW_KB_SKID_STEER_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_SKID_STEER_TYPE;
      break;
    case SW_KB_OMNI_STEER_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_OMNI_STEER_TYPE;
      break;
    case SW_KB_SUBMARINE_STEER_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_SUBMARINE_STEER_TYPE;
      break;
    case SW_KB_ROTARY_WING_STEER_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_ROTARY_WING_STEER_TYPE;
      break;
    case SW_KB_UNKNOWN_STEER_TYPE:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_UNKNOWN_STEER_TYPE;
      break;
    default:
      CLASS_CAST(ServoMobJASet *, setMsg)->steerType=MOAST_INVALID_STEER_TYPE;
      break;
    }
 
  switch(setMsg->type)
    {
    case SERVO_MOB_JA_SET_GRD_VEH_TYPE:
      
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->maxWheelRot=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getMaxWheelRot();
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->maxTorque=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getMaxTorque();
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->wheelSeparation=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getWheelSeparation();
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->wheelRadius=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getWheelRadius();
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->wheelBase=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getWheelBase();
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->maxSteerAngle=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getMaxSteerAngle();
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->maxCrabAngle=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getMaxCrabAngle();
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->minTurningRadius=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getMinTurnRadius();
      
      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length=0;
      for(int i=0; i<SERVO_MOB_JA_FLIPPER_TYPE_MAX; i++)
	{
	  if(NULL != (flipper=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getFlipper((SIMwareFlipperType) i)))
	    { 
	      switch((SIMwareFlipperType) i)
		{
		case SW_KB_FR_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].fType=MOAST_FR_FLIPPER_TYPE;
		  break;
		case SW_KB_FL_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].fType=MOAST_FL_FLIPPER_TYPE;
		  break;
		case SW_KB_RR_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].fType=MOAST_RR_FLIPPER_TYPE;
		  break;
		case SW_KB_RL_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].fType=MOAST_RL_FLIPPER_TYPE;
		  break;
		default:
		  continue;
		}
	     
	      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].minAngle=flipper->getMinAng();
	      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].maxAngle=flipper->getMaxAng();
	      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].length=flipper->getLength();
	      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].width=flipper->getWidth();
	      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].location=flipper->getLocation();
	      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet[CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length].orientation=flipper->getOrientation();
	      CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length++;
	    }
	}

      setBuf->write(CLASS_CAST(ServoMobJASetGrdVeh *, setMsg));
      break;

   case SERVO_MOB_JA_SET_AIR_BOT_TYPE:
     CLASS_CAST(ServoMobJASetAirBot *, setMsg)->maxLatVel=CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getMaxLatVel();
     CLASS_CAST(ServoMobJASetAirBot *, setMsg)->maxLinVel=CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getMaxLinVel();
     CLASS_CAST(ServoMobJASetAirBot *, setMsg)->maxRotVel=CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getMaxRotVel();
     CLASS_CAST(ServoMobJASetAirBot *, setMsg)->maxVertVel=CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getMaxVertVel();
     setBuf->write(CLASS_CAST(ServoMobJASetAirBot *, setMsg));
     break;
    default:
      setBuf->write(CLASS_CAST(ServoMobJASet *, setMsg));
      break;
    }
 
}
 
void SIMwareServoMob::sendStatus()
{ 
  SIMwareKBFlipper * flipper;  
  
  //cycle_time_tracker(&statMsg->tt);
  mSkin->kb->lockBotMutex();
  CLASS_CAST(ServoMobJAStat *, statMsg)->heartbeat++;
  CLASS_CAST(ServoMobJAStat *, statMsg)->time=mSkin->kb->bot->getTimeStamp();
  CLASS_CAST(ServoMobJAStat *, statMsg)->batteryVolt=mSkin->kb->bot->getBatteryLevel();
  CLASS_CAST(ServoMobJAStat *, statMsg)->fuelLevel=mSkin->kb->bot->getFuelLevel();
  CLASS_CAST(ServoMobJAStat *, statMsg)->headLights=mSkin->kb->bot->getLightsOn();
  
  switch(statMsg->type)
    {

    case SERVO_MOB_JA_STAT_GRD_VEH_TYPE:
      CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->steerAngle=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getSteerAngle();
      CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->crabAngle=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getCrabAngle();
      CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat_length=0;      
      for(int i=0; i<SERVO_MOB_JA_FLIPPER_TYPE_MAX; i++)
	{
	  if(NULL != (flipper=CLASS_CAST(SIMwareKBGroundVehicle *, mSkin->kb->bot)->getFlipper((SIMwareFlipperType) i)))
	    { 
	      switch((SIMwareFlipperType) i)
		{
		case SW_KB_FR_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat[CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat_length].fType=MOAST_FR_FLIPPER_TYPE;
		  break;
		case SW_KB_FL_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat[CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat_length].fType=MOAST_FL_FLIPPER_TYPE;
		  break;
		case SW_KB_RR_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat[CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat_length].fType=MOAST_RR_FLIPPER_TYPE;
		  break;
		case SW_KB_RL_FLIPPER_TYPE:
		  CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat[CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat_length].fType=MOAST_RL_FLIPPER_TYPE;
		  break;
		default:
		  continue;
		}
	      CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat[CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat_length].flipperAngle=flipper->getFlipperAngle();
	      CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg)->fStat_length++;
	    }
	}
      statBuf->write(CLASS_CAST(ServoMobJAStatGrdVeh *, statMsg));
      break;
      
    case SERVO_MOB_JA_STAT_AIR_BOT_TYPE:
      CLASS_CAST(ServoMobJAStatAirBot *, statMsg)->vLateral  = CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getLatVel();
      CLASS_CAST(ServoMobJAStatAirBot *, statMsg)->vLinear   = CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getLinVel();
      CLASS_CAST(ServoMobJAStatAirBot *, statMsg)->vRotation = CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getRotVel();
      CLASS_CAST(ServoMobJAStatAirBot *, statMsg)->vVertical = CLASS_CAST(SIMwareKBAerialRobot *, mSkin->kb->bot)->getVertVel();
      statBuf->write(CLASS_CAST(ServoMobJAStatAirBot *, statMsg));
      break;
      
    default:
      statBuf->write(CLASS_CAST(ServoMobJAStat *, statMsg));
      break;
    }
  mSkin->kb->unlockBotMutex();
}

void SIMwareServoMob::errorHandler(int error) 
{
  switch(error)
    {
    case SE0:
      CLASS_CAST(ServoMobJAStat *, statMsg)->error=SE0;
      snprintf(statMsg->message,80,"Robot is not initialized%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;
    case SE1:
      CLASS_CAST(ServoMobJAStat *, statMsg)->error=SE1;
      snprintf(statMsg->message,80,"Invalid command for %s steer type%c",
	       servoMobJA_enum_moastSteerType_symbol_lookup(CLASS_CAST(ServoMobJASet *, setMsg)->steerType),
	       (char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;  
    case SE2:
      CLASS_CAST(ServoMobJAStat *, statMsg)->error=SE2;
      snprintf(statMsg->message,80,"Invalid command : robot has no flipper%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;
    default:
      break;
    }

}
void SIMwareServoMob::doMobCmdInit()
{
  if (state_match(statMsg, NEW_COMMAND)) 
    { 
      statMsg->admin_state=ADMIN_INITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_INIT_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);      
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMob::doMobCmdHalt() 
{
   if (state_match(statMsg, NEW_COMMAND)) 
     {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);
  
}

void SIMwareServoMob::doMobCmdAbort() 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMob::doMobCmdShutdown() 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);  
}



void SIMwareServoMob::doMobCmdDrvSkid(ServoMobJACmdSkid * skidDrvCmd) 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else if( CLASS_CAST(ServoMobJASet *, setMsg)->steerType!= MOAST_SKID_STEER_TYPE) 
	state_next(statMsg, SE1);
      
      else 
	state_next(statMsg, S0);
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      mSkin->kb->addCmdToQ(new SIMwareSkidCmd(skidDrvCmd->wLeft,skidDrvCmd->wRight));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}



void SIMwareServoMob::doMobCmdDrvFlipSkid(ServoMobJACmdFlipSkid * fSkidDrvCmd) 
{
  SIMwareFlipperCmd * flipperCmds=new SIMwareFlipperCmd;
  SIMwareFlipperPair fPair;
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else if( CLASS_CAST(ServoMobJASet *, setMsg)->steerType!= MOAST_SKID_STEER_TYPE) 
	state_next(statMsg, SE1);

      else if( CLASS_CAST(ServoMobJASetGrdVeh *, setMsg)->fSet_length==0)
	state_next(statMsg, SE2);
      
      else 
	state_next(statMsg, S0);
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      mSkin->kb->addCmdToQ(new SIMwareSkidCmd(fSkidDrvCmd->wLeft,fSkidDrvCmd->wRight));
      for(int i=0;i<fSkidDrvCmd->fCmd_length;i++)
	{
	  switch(fSkidDrvCmd->fCmd[i].fType)
	    {
	    case MOAST_FR_FLIPPER_TYPE:
	      fPair.fType=SW_KB_FR_FLIPPER_TYPE;
	      break;

	    case MOAST_FL_FLIPPER_TYPE:
	      fPair.fType=SW_KB_FL_FLIPPER_TYPE;
	      break;

	    case MOAST_RR_FLIPPER_TYPE:
	      fPair.fType=SW_KB_RR_FLIPPER_TYPE;
	      break;

	    case MOAST_RL_FLIPPER_TYPE:
	      fPair.fType=SW_KB_RL_FLIPPER_TYPE;
	      break;
		
	    default:
	      continue;
	    }
	  fPair.flipperAngle=fSkidDrvCmd->fCmd[i].flipperAngle;
	  flipperCmds->fCmd.push_back(fPair);
	}
      mSkin->kb->addCmdToQ(flipperCmds);
      
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    } 
  else if (state_match(statMsg, SE2))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE2);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMob::doMobCmdDrvAckerman(ServoMobJACmdAckerman * ackDrvCmd) 
{
 if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else if( CLASS_CAST(ServoMobJASet *, setMsg)->steerType!= MOAST_ACKERMAN_STEER_TYPE) 
	state_next(statMsg, SE1);
      
      else 
	state_next(statMsg, S0);
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      mSkin->kb->addCmdToQ(new SIMwareAckermanCmd(ackDrvCmd->velocity,
						  ackDrvCmd->steerAngle,
						  ackDrvCmd->crabAngle));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMob::doMobCmdDrvSubmarine(ServoMobJACmdSubmarine * sDrvCmd) 
{
if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else if( CLASS_CAST(ServoMobJASet *, setMsg)->steerType!= MOAST_SUBMARINE_STEER_TYPE) 
	state_next(statMsg, SE1);
      
      else 
	state_next(statMsg, S0);
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      mSkin->kb->addCmdToQ(new SIMwareSubmarineCmd(sDrvCmd->propeller,
						   sDrvCmd->rudder,
						   sDrvCmd->sternPlane));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMob::doMobCmdDrvOmni(ServoMobJACmdOmni * omniDrvCmd) 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else if( CLASS_CAST(ServoMobJASet *, setMsg)->steerType!= MOAST_OMNI_STEER_TYPE) 
	state_next(statMsg, SE1);
      
      else 
	state_next(statMsg, S0);
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      mSkin->kb->addCmdToQ(new SIMwareOmniCmd(omniDrvCmd->vLinear,
					      omniDrvCmd->vLateral,
					      omniDrvCmd->rot));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMob::doMobCmdDrvRotaryWing(ServoMobJACmdRotaryWing * rwDrvCmd) 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else if( CLASS_CAST(ServoMobJASet *, setMsg)->steerType!= MOAST_ROTARY_WING_STEER_TYPE) 
	state_next(statMsg, SE1);
      
      else 
	state_next(statMsg, S0);
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);

      
      mSkin->kb->addCmdToQ(new SIMwareRotaryWingCmd(rwDrvCmd->vLinear,
						    rwDrvCmd->vLateral,
						    rwDrvCmd->vVertical,
						    rwDrvCmd->vRotation));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMob::doMobCfgCycleTime( ServoMobJACfgCycleTime * cTCmd ) 
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      state_new(setMsg);
      CLASS_CAST(ServoMobJASet *, setMsg)->cycleTime = cTCmd->cycleTime;
      setTimer(CLASS_CAST(ServoMobJASet *, setMsg)->cycleTime);
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);   
    }
  else
    state_default(setMsg);  
  
}

void SIMwareServoMob::doMobCfgDebug( ServoMobJACfgDebug * cTCmd ) 
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      state_new(setMsg);
      CLASS_CAST(ServoMobJASet *, setMsg)->debug = cTCmd->debug;
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);
    }
  else
    state_default(setMsg); 
}

void SIMwareServoMob::doMobCfgTeleport( ServoMobJACfgTeleport * cTCmd ) 
{
 if (state_match(setMsg, NEW_COMMAND)) 
   {
     state_new(setMsg);
     status_next(setMsg, RCS_ERROR);
     state_next(setMsg, NOP_STATE);
     printf("** ERROR ** <SIMwareServoMob::doMobCfgTeleport> : Not implemented\n");
   }
 else
   state_default(setMsg); 
}
