#include "swKBTypes.hh"
#include "swServoSP.hh"
#include "moastSupSkin.hh"

SIMwareServoSP::SIMwareServoSP(MoastSupSkin * m) : SIMwareServoNodeBase(m)
{
  nodeName="ServoSP";
  statMsg=new ServoSPStat;
  setMsg=new ServoSPSet;

  //initialize nml buffers
  linescanBuf.clear();
  sonarBuf=NULL;
  rfidBuf=NULL;
  victimBuf=NULL;
  navBuf=NULL;  
  
  // initialize kb sensor containers
  linescan.clear();
  sonar.clear();
  rfid.clear();
  victim.clear();
  navSen.clear();

  // initialize stat msg
  CLASS_CAST(ServoSPStat *, statMsg)->transmitData=false;
  CLASS_CAST(ServoSPStat *, statMsg)->heartbeat=0;
  CLASS_CAST(ServoSPStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;

  CLASS_CAST(ServoSPSet *, setMsg)->cycleTime=SERVO_NODE_CYCLE_TIME_DEFAULT;
  CLASS_CAST(ServoSPSet *, setMsg)->debug=0;
}

SIMwareServoSP::~SIMwareServoSP() 
{
  if(0)
    printf("<SIMwareServoSP::~SIMwareServoSP>\n");
  cleanUpNMLBuf();
  linescan.clear();
  sonar.clear();
  rfid.clear();
  victim.clear();
  navSen.clear();
  if(statMsg)
    {
      delete statMsg;
      statMsg=0;
    }
  if(setMsg)
    {
      delete setMsg;
      setMsg=0;
    }
  for(unsigned int i=0;i<linescanData.size();i++)
    {
      if(linescanData[i])
	{
	  delete linescanData[i];
	}
    }
  linescanData.clear();
}

bool SIMwareServoSP::openNML(string cmdName, string statName, 
			     string cfgName,string setName,
			     NML_FORMAT_PTR fPtr, int compID) 
  
{
  stringstream str; 
  if(! SIMwareServoNodeBase::openNML(cmdName, statName, 
				     cfgName,setName,
				     fPtr,compID) )
    {
      printf("<SIMwareServoSP::openNML> Error Opening Servo SP Buffers\n");
      return false;
    }
  if(1)
    printf("<SIMwareServoSP::openNML> Opening NML data Buffers\n");
  
  // Get handles on sensors in the knowledge base
  map <const char *, SIMwareKBSen *, strCmp >::iterator itr = mSkin->kb->senMap.begin();
  while ( itr != mSkin->kb->senMap.end() ) 
    {
      switch( itr->second->getSensorType())
	{
	case SW_KB_LINESCAN_SENSOR_TYPE:
	  // ** need to add check for max sensor allowed
	  linescan.push_back(CLASS_CAST(SIMwareKBSenLinescan *, itr->second));
	  str << SERVO_SP_LINESCAN_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << linescan.size();
	  printf("\t%s\n",str.str().c_str());
	  linescanBuf.push_back( new NML(sensorData_format, 
					 str.str().c_str(), 
					 mSkin->nmlProcName.c_str(), 
					 mSkin->nmlCfgFile.c_str()));
	  if (!linescanBuf[linescan.size()-1]->valid()) 
	    {
	      delete linescanBuf[linescan.size()-1];
	      linescanBuf[linescan.size()-1] = 0;
	      return false;
	    }
	  linescanBuf[linescan.size()-1]->clear();
	  linescanData.push_back(new SensorData1D);
	  str.str("");
	  break;
	  
	case SW_KB_SONAR_SENSOR_TYPE:
	  sonar.push_back(CLASS_CAST(SIMwareKBSenSonar *, itr->second));
	  if(sonarBuf==NULL) 
	    {
	      str << SERVO_SP_SONAR_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1 ;
	      printf("\t%s\n",str.str().c_str());
	      sonarBuf = new NML(sensorData_format, 
				 str.str().c_str(), 
				 mSkin->nmlProcName.c_str(), 
				 mSkin->nmlCfgFile.c_str());
	      if (!sonarBuf->valid()) 
		{
		  delete sonarBuf;
		  sonarBuf = 0;
		  return false;
		}
	      sonarBuf->clear();
	      str.str("");
	    }
	  break;
	
	case SW_KB_INU_SENSOR_TYPE:
	case SW_KB_INS_SENSOR_TYPE:
	case SW_KB_GPS_SENSOR_TYPE:
	case SW_KB_ODOMETRY_SENSOR_TYPE:
	case SW_KB_GRD_TRUTH_SENSOR_TYPE:
	  navSen.push_back(itr->second);
	  if( navBuf==NULL) 
	    {
	      str << SERVO_SP_NAV_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1 ;
	      printf("\t%s\n",str.str().c_str());
	      navBuf = new NML(sensorData_format, 
			       str.str().c_str(), 
			       mSkin->nmlProcName.c_str(), 
			       mSkin->nmlCfgFile.c_str());
	      if (!navBuf->valid()) 
		{
		  delete navBuf;
		  navBuf = 0;
		  return false;
		}
	      navBuf->clear();
	      str.str("");
	    }
	  break;
	  
	case SW_KB_RFID_SENSOR_TYPE:
	  rfid.push_back(CLASS_CAST(SIMwareKBSenRFID *, itr->second));
	  if( rfidBuf==NULL) 
	    {
	      str << SERVO_SP_RFID_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1 ;
	      printf("\t%s\n",str.str().c_str());
	      rfidBuf = new NML(sensorData_format, 
				str.str().c_str(), 
				mSkin->nmlProcName.c_str(), 
				mSkin->nmlCfgFile.c_str());
	      if (!rfidBuf->valid()) 
		{
		  delete rfidBuf;
		  rfidBuf = 0;
		  return false;
		}
	      rfidBuf->clear();
	      str.str("");
	    }
	  break;
	  
	case SW_KB_VICTIM_SENSOR_TYPE:
	  victim.push_back(CLASS_CAST(SIMwareKBSenVictim *, itr->second));
	  if( victimBuf==NULL) 
	    {
	      str << SERVO_SP_VICTIM_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1 ;
	      printf("\t%s\n",str.str().c_str()); 
	      victimBuf = new NML(sensorData_format, 
				    str.str().c_str(), 
				    mSkin->nmlProcName.c_str(), 
				    mSkin->nmlCfgFile.c_str());
	      if (!victimBuf->valid()) 
		{
		  delete victimBuf;
		  victimBuf = 0;
		  return false;
		}
	      victimBuf->clear();
	      str.str("");
	    }
	  break;
	
	case SW_KB_CAMERA_SENSOR_TYPE:
	  xSen.push_back(itr->second);
	  break;
	
	case SW_KB_ENCODER_SENSOR_TYPE:
	case SW_KB_TOUCH_SENSOR_TYPE:
	  break;

	default:
	  printf
	    ("** WARNING ** <SIMwareServoSP::openSkin> Sensor %s not added, sensor type %d not valid sensor\n",
	     itr->first, itr->second->getSensorType());
	  break;
	}      
      itr++;
    }
  publishSettings();
  return true;
}


void SIMwareServoSP::cleanUpNMLBuf()
{
  //printf("<SIMwareServoSP::cleanUpNMLBuf> : clean up data buffers\n");  
  
  linescanBuf.clear();
  
  if(sonarBuf==NULL)
    delete sonarBuf;
  sonarBuf=NULL;
  
  if(rfidBuf==NULL)
    delete rfidBuf;
  rfidBuf=NULL;
  
  if(victimBuf==NULL)
    delete victimBuf;
  victimBuf=NULL;
      
  if(navBuf==NULL)
    delete navBuf;
  navBuf=NULL;
}



void SIMwareServoSP::doSingleStep() 
{
  if(0)
    printf("<SIMwareServoSP::doSingleStep>\n");
  
  int serialNumber;
  int type = 0;
  
  // Read Configuration Message
  type = cfgBuf->read();
  serialNumber = cfgBuf->get_address()->serial_number;
  switch (type) 
    {
      
    case 0:
      // no new cmd
      break;
      
    case -1:
      // comm error
      rcs_print_error("<SIMwareServoSP::doSingleStep> : NML cmd read error\n");
      break;
      
    case SERVO_SP_CFG_CYCLE_TIME_TYPE:
    case SERVO_SP_CFG_DEBUG_TYPE:
      
      if (serialNumber != CLASS_CAST(ServoSPSet *, setMsg)->echo_serial_number) 
	{
	  CLASS_CAST(ServoSPSet *, setMsg)->command_type = type;
	  CLASS_CAST(ServoSPSet *, setMsg)->echo_serial_number = serialNumber;
	  CLASS_CAST(ServoSPSet *, setMsg)->state = NEW_COMMAND;
	  CLASS_CAST(ServoSPSet *, setMsg)->status = RCS_EXEC;
	}    
      break;
      
    default:
      rcs_print_error("<SIMwareServoSP::doSingleStep> : SPCfg msg type %d not recoginzed\n", (int) type);
      break;
    }	
  
  //Read Command Message
  type = cmdBuf->read();
  serialNumber = cmdBuf->get_address()->serial_number;
  switch (type) 
    {
      
    case 0:
      // no new cmd
      break;
      
    case -1:
      // comm error
      rcs_print_error("<SIMwareServoSP::doSingleStep> : NML cmd read error\n");
      break;
    case SERVO_SP_CMD_INIT_TYPE:
    case SERVO_SP_CMD_HALT_TYPE:
    case SERVO_SP_CMD_ABORT_TYPE:
    case SERVO_SP_CMD_SHUTDOWN_TYPE:
    case SERVO_SP_CMD_GO_TYPE:
      if (serialNumber != CLASS_CAST(ServoSPStat *, statMsg)->echo_serial_number) 
	{
	  CLASS_CAST(ServoSPStat *, statMsg)->command_type = type;
	  CLASS_CAST(ServoSPStat *, statMsg)->echo_serial_number = serialNumber;
	  CLASS_CAST(ServoSPStat *, statMsg)->state = NEW_COMMAND;
	  CLASS_CAST(ServoSPStat *, statMsg)->status = RCS_EXEC;
	  CLASS_CAST(ServoSPStat *, statMsg)->error = 0;
	}    
      break;
    default:
      rcs_print_error("<SIMwareServoSP::doSingleStep> : SPCmd msg type %d not recoginzed\n", (int) type);
      break;
    }				// switch (command_type)
  
  
  
  // Handle Configuration Messages 
  switch( CLASS_CAST(ServoSPSet *, setMsg)->command_type ) 
    {
      
    case SERVO_SP_CFG_CYCLE_TIME_TYPE:
      doSPCfgCycleTime(CLASS_CAST(ServoSPCfgCycleTime *, cfgBuf->get_address()));
      break;
      
    case SERVO_SP_CFG_DEBUG_TYPE:
      doSPCfgDebug(CLASS_CAST(ServoSPCfgDebug *, cfgBuf->get_address()));
      break;
    default:
      
      break;
    }
  
  // Handle Command Message 
  switch (CLASS_CAST(ServoSPStat *, statMsg)->command_type) 
    {
      
    case SERVO_SP_CMD_INIT_TYPE:
      doSPCmdInit();
      break;
      
    case SERVO_SP_CMD_HALT_TYPE:
      doSPCmdHalt();
      break;
      
    case SERVO_SP_CMD_ABORT_TYPE:
      doSPCmdAbort();
      break;
      
    case SERVO_SP_CMD_SHUTDOWN_TYPE:
      doSPCmdShutdown();
      break;
      
    case SERVO_SP_CMD_GO_TYPE:
      doSPCmdGo();
      break;
      
    default:
      // no cmd to handle
      break;
    }				
  if( mSkin->kb->settingsFresh() )
    publishSettings();
  sendStatus();
  sendSensorData();
}

void SIMwareServoSP::doSPCmdInit() 
{
  if (state_match(CLASS_CAST(ServoSPStat *, statMsg), NEW_COMMAND)) 
    { 
      CLASS_CAST(ServoSPStat *, statMsg)->admin_state=ADMIN_INITIALIZED;
      state_new(CLASS_CAST(ServoSPStat *, statMsg));
      mSkin->kb->addCmdToQ(new SIMwareSenCmd(SW_SEN_CMD_INIT_TYPE));
      status_next(CLASS_CAST(ServoSPStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoSPStat *, statMsg), NOP_STATE);      
    }
  else
    state_default(CLASS_CAST(ServoSPStat *, statMsg));  
}

void SIMwareServoSP::doSPCmdHalt() 
{
  if (state_match(CLASS_CAST(ServoSPStat *, statMsg), NEW_COMMAND)) 
    {
      CLASS_CAST(ServoSPStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;
      CLASS_CAST(ServoSPStat *, statMsg)->transmitData=false;
      mSkin->kb->addCmdToQ(new SIMwareSenCmd(SW_SEN_CMD_POWERDOWN_TYPE));
      state_new(CLASS_CAST(ServoSPStat *, statMsg));
      status_next(CLASS_CAST(ServoSPStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoSPStat *, statMsg), NOP_STATE);
    }
  else
    state_default(CLASS_CAST(ServoSPStat *, statMsg));  
}

void SIMwareServoSP::doSPCmdAbort() {
  if (state_match(CLASS_CAST(ServoSPStat *, statMsg), NEW_COMMAND)) 
    {
      CLASS_CAST(ServoSPStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;
      CLASS_CAST(ServoSPStat *, statMsg)->transmitData=false;
      mSkin->kb->addCmdToQ(new SIMwareSenCmd(SW_SEN_CMD_POWERDOWN_TYPE));
      state_new(CLASS_CAST(ServoSPStat *, statMsg));
      status_next(CLASS_CAST(ServoSPStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoSPStat *, statMsg), NOP_STATE);
    }
  else
    state_default(CLASS_CAST(ServoSPStat *, statMsg));  
}

void SIMwareServoSP::doSPCmdShutdown() 
{
  if (state_match(CLASS_CAST(ServoSPStat *, statMsg), NEW_COMMAND)) 
    {
      CLASS_CAST(ServoSPStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;
      CLASS_CAST(ServoSPStat *, statMsg)->transmitData=false;
      mSkin->kb->addCmdToQ(new SIMwareSenCmd(SW_SEN_CMD_POWERDOWN_TYPE));
      state_new(CLASS_CAST(ServoSPStat *, statMsg));
      status_next(CLASS_CAST(ServoSPStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoSPStat *, statMsg), NOP_STATE);
    }
  else
    state_default(CLASS_CAST(ServoSPStat *, statMsg));  
}

void SIMwareServoSP::doSPCmdGo() 
{
  if (state_match(CLASS_CAST(ServoSPStat *, statMsg), NEW_COMMAND)) 
    {
      state_new(CLASS_CAST(ServoSPStat *, statMsg));
      status_next(CLASS_CAST(ServoSPStat *, statMsg), RCS_EXEC);    
      if(CLASS_CAST(ServoSPStat *, statMsg)->admin_state!=ADMIN_INITIALIZED)
	state_next(CLASS_CAST(ServoSPStat *, statMsg), SE0);
      else
	state_next(CLASS_CAST(ServoSPStat *, statMsg), S0);
    }
  
  if(state_match(CLASS_CAST(ServoSPStat *, statMsg), S0))
    {
      CLASS_CAST(ServoSPStat *, statMsg)->transmitData=true;
      status_next(CLASS_CAST(ServoSPStat *, statMsg), RCS_EXEC);  
      state_next(CLASS_CAST(ServoSPStat *, statMsg), NOP_STATE);
    }
  else if ( state_match(CLASS_CAST(ServoSPStat *, statMsg), SE0))
    {
      status_next(CLASS_CAST(ServoSPStat *, statMsg), RCS_ERROR);  
      state_next(CLASS_CAST(ServoSPStat *, statMsg), NOP_STATE);
      CLASS_CAST(ServoSPStat *, statMsg)->error=SE0;
      snprintf(CLASS_CAST(ServoSPStat *, statMsg)->message,80,"Robot is not initialized%c",(char)NULL);
      CLASS_CAST(ServoSPStat *, statMsg)->message_length=strlen(CLASS_CAST(ServoSPStat *, statMsg)->message);
    }
  else
    state_default(CLASS_CAST(ServoSPStat *, statMsg));  
}


void SIMwareServoSP::doSPCfgCycleTime( ServoSPCfgCycleTime * cTCmd ) 
{
  if (state_match(CLASS_CAST(ServoSPSet *, setMsg), NEW_COMMAND)) 
    {
      state_new(CLASS_CAST(ServoSPSet *, setMsg));
      CLASS_CAST(ServoSPSet *, setMsg)->cycleTime = cTCmd->cycleTime;
      setTimer(CLASS_CAST(ServoSPSet *, setMsg)->cycleTime);
      status_next(CLASS_CAST(ServoSPSet *, setMsg), RCS_DONE);
      state_next(CLASS_CAST(ServoSPSet *, setMsg), NOP_STATE);    
      setBuf->write(CLASS_CAST(ServoSPSet *, setMsg));	
    }
  else
    state_default(CLASS_CAST(ServoSPSet *, setMsg));  
}

void SIMwareServoSP::doSPCfgDebug( ServoSPCfgDebug * cTCmd ) 
{
  if (state_match(CLASS_CAST(ServoSPSet *, setMsg), NEW_COMMAND)) 
    {
      state_new(CLASS_CAST(ServoSPSet *, setMsg));
      CLASS_CAST(ServoSPSet *, setMsg)->debug = cTCmd->debug;
      status_next(CLASS_CAST(ServoSPSet *, setMsg), RCS_DONE);
      state_next(CLASS_CAST(ServoSPSet *, setMsg), NOP_STATE);
      setBuf->write(CLASS_CAST(ServoSPSet *, setMsg));	 
    }
  else
    state_default(CLASS_CAST(ServoSPSet *, setMsg)); 
}


void SIMwareServoSP::sendSensorData()
{
  if(!CLASS_CAST(ServoSPStat *, statMsg)->transmitData)
    return;

  mSkin->kb->lockSenMutex();
  
  sendNavData();
  sendLinescanData();
  sendSonarData();
  sendVictimData();
  sendRfidData();

  mSkin->kb->unlockSenMutex();
}

void SIMwareServoSP::sendLinescanData()
{  
  int end;
  for(unsigned int i=0;i<linescanBuf.size();i++)
    if(linescan[i]->hasNewData())
      {
	linescanData[i]->isGlobal = false;  
	linescanData[i]->collectTimeStart=linescan[i]->getTimeStamp();
	linescanData[i]->collectTimeDuration=0;
	linescanData[i]->maxRange = linescan[i]->getMaxRange();
	linescanData[i]->minRange = linescan[i]->getMinRange();
	linescanData[i]->start = -0.5 * linescan[i]->getFov();
	linescanData[i]->increment = linescan[i]->getBeamSpacing();
	linescanData[i]->featArray_length = linescan[i]->getReturnCount();
	if( linescanData[i]->featArray_length > MAX_SENSOR_DATA_1D )
	  //	    NML_ARRAY_LEN(linescanData[i]->featArray))
	  {
	    fprintf(stderr, "%s:%d linescan %d has return count of %d but max length of the featArray is %d\n",
		    __FILE__,
		    __LINE__,
		    i,
		    linescanData[i]->featArray_length,
		    MAX_SENSOR_DATA_1D );
		    //		    NML_ARRAY_LEN(linescanData[i]->featArray));
	    linescanData[i]->featArray_length = MAX_SENSOR_DATA_1D;
	    //	      NML_ARRAY_LEN(linescanData[i]->featArray);
	  }
	end = linescan[i]->getReturnCount() - 1;
	for(int j=0;j<linescanData[i]->featArray_length;j++)
	  linescanData[i]->featArray[j]=linescan[i]->getRangeReturn(end-j);
	linescan[i]->resetNewDataFlag();
	linescanBuf[i]->write(linescanData[i]);
      }
}
   
void SIMwareServoSP::sendSonarData()
{
  int count=0;
  for(unsigned int i=0;i<sonar.size();i++)
    if(sonar[i]->hasNewData())
      {
	sonarData.arrayElem[i].type = SERVO_SP_SONAR_SENSOR;
	sonarData.arrayElem[i].id = sonar[i]->getSenID();
	sonarData.arrayElem[i].dataTuple[0]=sonar[i]->getRangeReturn();
	sonarData.arrayElem[i].collectionTime=sonar[i]->getTimeStamp();
	sonar[i]->resetNewDataFlag();
	count++;
      }
  sonarData.arrayElem_length=count;
  if(sonarData.arrayElem_length)
    sonarBuf->write(sonarData);
  
}

void SIMwareServoSP::sendNavData() 
{
  int count=0;
  unsigned int i=0;  
  
  for(i=0;i<navSen.size();i++)
    {
 	
      if( navSen[i]->hasNewData())
	{
	
	  switch(navSen[i]->getSensorType())
	    {
	    case SW_KB_INU_SENSOR_TYPE:
	      navData.arrayElem[count].type = SERVO_SP_INU_SENSOR;
	      navData.arrayElem[count].dataTuple[0]=CLASS_CAST(SIMwareKBSenINU *, navSen[i])->getRoll();
	      navData.arrayElem[count].dataTuple[1]=CLASS_CAST(SIMwareKBSenINU *, navSen[i])->getPitch();
	      navData.arrayElem[count].dataTuple[2]=CLASS_CAST(SIMwareKBSenINU *, navSen[i])->getYaw();
	      navData.arrayElem[count].dataTuple[3]=0.00;
	      navData.arrayElem[count].dataTuple[4]=0.00;
	      navData.arrayElem[count].dataTuple[5]=0.00;
	      break;
	    case SW_KB_INS_SENSOR_TYPE:
	
	      navData.arrayElem[count].type = SERVO_SP_INS_SENSOR;
	      navData.arrayElem[count].dataTuple[0]=CLASS_CAST(SIMwareKBSenINS *, navSen[i])->getX();
	      navData.arrayElem[count].dataTuple[1]=CLASS_CAST(SIMwareKBSenINS *, navSen[i])->getY();
	      navData.arrayElem[count].dataTuple[2]=CLASS_CAST(SIMwareKBSenINS *, navSen[i])->getZ();
	      navData.arrayElem[count].dataTuple[3]=CLASS_CAST(SIMwareKBSenINS *, navSen[i])->getRoll();
	      navData.arrayElem[count].dataTuple[4]=CLASS_CAST(SIMwareKBSenINS *, navSen[i])->getPitch();
	      navData.arrayElem[count].dataTuple[5]=CLASS_CAST(SIMwareKBSenINS *, navSen[i])->getYaw();
	      break;
	    case SW_KB_ODOMETRY_SENSOR_TYPE:
	      navData.arrayElem[count].type = SERVO_SP_ODOMETRY_SENSOR;
	      navData.arrayElem[count].dataTuple[0]=CLASS_CAST(SIMwareKBSenOdometry *,  navSen[i])->getX();
	      navData.arrayElem[count].dataTuple[1]=CLASS_CAST(SIMwareKBSenOdometry *,  navSen[i])->getY();
	      navData.arrayElem[count].dataTuple[2]=CLASS_CAST(SIMwareKBSenOdometry *,  navSen[i])->getOrient();
	      navData.arrayElem[count].dataTuple[3]=0.00;
	      navData.arrayElem[count].dataTuple[4]=0.00;
	      navData.arrayElem[count].dataTuple[5]=0.00;
	      break;
	    case SW_KB_GPS_SENSOR_TYPE:
	      navData.arrayElem[count].type = SERVO_SP_GPS_SENSOR;
	      // North is assigned positive values
              if(((SIMwareKBSenGPS *)navSen[i])->getLatitudeCardinality()=='N')
              {
	        navData.arrayElem[count].dataTuple[0] = (double)((SIMwareKBSenGPS *)navSen[i])->getLatitudeDegrees();
	        navData.arrayElem[count].dataTuple[1] = (double)((SIMwareKBSenGPS *)navSen[i])->getLatitudeMinutes();
              }
              // South is assigned negative values
              else if(((SIMwareKBSenGPS *)navSen[i])->getLatitudeCardinality()=='S')
              {
                navData.arrayElem[count].dataTuple[0] = -(double)((SIMwareKBSenGPS *)navSen[i])->getLatitudeDegrees();
                navData.arrayElem[count].dataTuple[1] = -(double)((SIMwareKBSenGPS *)navSen[i])->getLatitudeMinutes();
              }

              // East is assigned positive values
              if(((SIMwareKBSenGPS *)navSen[i])->getLongitudeCardinality()=='E')
              {
	        navData.arrayElem[count].dataTuple[2] = (double)((SIMwareKBSenGPS *)navSen[i])->getLongitudeDegrees();
	        navData.arrayElem[count].dataTuple[3] = (double)((SIMwareKBSenGPS *)navSen[i])->getLongitudeMinutes();
              }
              // West is assigned negative values
              else if(((SIMwareKBSenGPS *)navSen[i])->getLongitudeCardinality()=='W')
              {
                navData.arrayElem[count].dataTuple[2] = -(double)((SIMwareKBSenGPS *)navSen[i])->getLongitudeDegrees();
                navData.arrayElem[count].dataTuple[3] = -(double)((SIMwareKBSenGPS *)navSen[i])->getLongitudeMinutes();
              }
	      navData.arrayElem[count].dataTuple[4] = (double)((SIMwareKBSenGPS *)navSen[i])->getFix();
	      navData.arrayElem[count].dataTuple[5] = (double)((SIMwareKBSenGPS *)navSen[i])->getSatellites();

	      break;
	    case SW_KB_GRD_TRUTH_SENSOR_TYPE:
              //printf("INSIDE GROUNDTRUTH SENSOR=======================");
	      navData.arrayElem[count].type = SERVO_SP_GRD_TRUTH_SENSOR;
	      navData.arrayElem[count].dataTuple[0]=CLASS_CAST(SIMwareKBSenGrdTruth *,  navSen[i])->getLocation().x;
	      navData.arrayElem[count].dataTuple[1]=CLASS_CAST(SIMwareKBSenGrdTruth *,  navSen[i])->getLocation().y;
	      navData.arrayElem[count].dataTuple[2]=CLASS_CAST(SIMwareKBSenGrdTruth *,  navSen[i])->getLocation().z;
	      navData.arrayElem[count].dataTuple[3]=CLASS_CAST(SIMwareKBSenGrdTruth *,  navSen[i])->getOrientation().r;
	      navData.arrayElem[count].dataTuple[4]=CLASS_CAST(SIMwareKBSenGrdTruth *,  navSen[i])->getOrientation().p;
	      navData.arrayElem[count].dataTuple[5]=CLASS_CAST(SIMwareKBSenGrdTruth *,  navSen[i])->getOrientation().y;    
	      break;
	    default:
	      continue;
	      break;
	    }
	  navData.arrayElem[count].id = navSen[i]->getSenID();
	  navData.arrayElem[count].collectionTime=navSen[i]->getTimeStamp();
	  navSen[i]->resetNewDataFlag();
	  count++;
	}
    }
  
  navData.arrayElem_length=count;
  if(navData.arrayElem_length)
    navBuf->write(navData);
}


void SIMwareServoSP::sendVictimData()
{
  SIMwareVictimMsg vMsg;
  int count=0;
  
  for(unsigned int i=0;i<victim.size();i++)
    {
      if(!victim[i]->hasNewData())
	continue;
      
      while( victim[i]->hasVictMsg() 
	     && (count < MAX_SENSOR_DATA_ARRAY) )
	    {
	      vMsg = victim[i]->popVictMsg();
	      victimData.arrayElem[count].collectionTime=vMsg.getMsgTime();
	      victimData.arrayElem[count].id=victim[i]->getSenID();
	      victimData.arrayElem[count].type=SERVO_SP_VICTIM_SENSOR;
	      switch(vMsg.partType)
		{
		case SW_KB_VICTIM_HEAD_PART_TYPE:
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_HEAD_TYPE;  
		  break;
		case SW_KB_VICTIM_CHEST_PART_TYPE:
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_CHEST_TYPE;  
		  break;
		case SW_KB_VICTIM_PELVIS_PART_TYPE:
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_PELVIS_TYPE;  
		  break;
		case SW_KB_VICTIM_LEG_PART_TYPE:
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_LEG_TYPE;  
		  break;
		case SW_KB_VICTIM_FOOT_PART_TYPE: 
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_FOOT_TYPE;  
		  break;
		case SW_KB_VICTIM_ARM_PART_TYPE:
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_ARM_TYPE;  
		  break;
		case SW_KB_VICTIM_HAND_PART_TYPE:
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_HAND_TYPE;  
		  break;
		default:
		  victimData.arrayElem[count].dataTuple[0]=BODY_PART_UNKNOWN_TYPE;  
		  break;
		}
	      victimData.arrayElem[count].dataTuple[1]=vMsg.partLoc.x;
	      victimData.arrayElem[count].dataTuple[2]=vMsg.partLoc.y;
	      victimData.arrayElem[count].dataTuple[3]=vMsg.partLoc.z;
	      count++;
	    }
    }
  if( count )
    {
      victimData.arrayElem_length=count;
      victimBuf->write(victimData);
    }
}

void SIMwareServoSP::sendRfidData()  
{
  SIMwareRfidMsg rMsg;
  int count=0;
  
  for(unsigned int i=0;i<rfid.size();i++)
    {
      if(!rfid[i]->hasNewData())
 	continue;
      
      while( rfid[i]->hasRfidMsg() 
	     && (count <  MAX_SENSOR_DATA_TEXT_ARRAY) )
 	  {
	    rMsg = rfid[i]->popRfidMsg();
	    rfidData.dataElem[count].collectionTime=rMsg.getMsgTime();
	    rfidData.dataElem[count].sensorID=rfid[i]->getSenID();
	    rfidData.dataElem[count].sensorType=SERVO_SP_RFID_SENSOR;

	    
	    rfidData.dataElem[count].dataTuple_length=1;
	    rfidData.dataElem[count].dataTuple[0]=rMsg.rfidID;
	    
	    rfidData.dataElem[count].dataStr_length
	      =snprintf(  rfidData.dataElem[count].dataStr, 
			  MAX_SENSOR_DATA_ARRAY_STRLEN,
			  "%s", 
			  rMsg.rfidData.c_str());
 	    count++;
 	  }
 	rfid[i]->resetNewDataFlag();
     }

   if(count)
     {
       rfidData.dataElem_length=count;
       rfidBuf->write(rfidData);
     } 
  
}

void SIMwareServoSP::sendStatus()
{
  CLASS_CAST(ServoSPStat *, statMsg)->heartbeat++;
  cycle_time_tracker(&CLASS_CAST(ServoSPStat *, statMsg)->tt);
  statBuf->write(CLASS_CAST(ServoSPStat *, statMsg));
}

void SIMwareServoSP::publishSettings()
{
  stringstream str; 
  int count;
  unsigned int i;
  ServoSPSet * setMsgPtr = CLASS_CAST(ServoSPSet *, setMsg);
  SIMwareKBMount * mount;

  mSkin->kb->setSettingsFresh(false);
  /*
  sleep(1);
  printf( "swServoSP just woke up!\n" );
  */
  setMsgPtr->vehicleID= mSkin->kb->bot->getBotID();
  sprintf(setMsgPtr->worldName,"%s",mSkin->kb->world.getWorldName().c_str());
  setMsgPtr->startPose= mSkin->kb->world.getStartPose();

  // Check sensor limitation of NML messages
  if(mSkin->kb->senMap.size() > SERVO_SP_SENSOR_MAX )
    {
      printf("** WARNING ** <SIMwareServoSP::publishSettings> :\n   Sensor count exceeds size constraints of ServoSPSet buffers\n");
      printf("\t* Increase the size of servoSPSet buffers\n\t* Increase size of SERVO_SP_SENSOR_MAX\n");
      raise(SIGTERM);
    }

  count = 0;
 
  for(i=0;i<linescanBuf.size();i++) 
    {
      setMsgPtr->sSetElem[count].sType=SERVO_SP_LINESCANNER_SENSOR;
      str << SERVO_SP_LINESCAN_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << i+1;
      sprintf(setMsgPtr->sSetElem[count].senDataBufName,"%s",str.str().c_str());
      setMsgPtr->sSetElem[count].sensorID=linescan[i]->getSenID();
      if(NULL == (mount=linescan[i]->getMount()))
	{
	  setMsgPtr->sSetElem[count].mountID=-1;
	  setMsgPtr->sSetElem[count].linkID=0;
	}
      else
	{
	  setMsgPtr->sSetElem[count].mountID=mount->misPkg->getMisPkgID();
	  setMsgPtr->sSetElem[count].linkID=mount->link->getLinkID();
	}
      
      setMsgPtr->sSetElem[count].sensorToMount=linescan[i]->getSen2Mount();
      setMsgPtr->sSetElem[count].desiredPeriod=0;
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_LINESCAN_MAX_RANGE]=linescan[i]->getMaxRange();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_LINESCAN_MIN_RANGE]=linescan[i]->getMinRange();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_LINESCAN_RESOLUTION]=linescan[i]->getBeamSpacing();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_LINESCAN_FOV]=linescan[i]->getFov();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_LINESCAN_BEAM_DISPERSION]=linescan[i]->getBeamDispersion();
      setMsgPtr->sSetElem[count].parameters_length=5;
      count++;
      str.str("");    
    }
  

  str << SERVO_SP_SONAR_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1;
  for(i=0;i<sonar.size();i++)
    {
      setMsgPtr->sSetElem[count].sType=SERVO_SP_SONAR_SENSOR;
      sprintf(setMsgPtr->sSetElem[count].senDataBufName,"%s",str.str().c_str());
      setMsgPtr->sSetElem[count].sensorID=sonar[i]->getSenID();
      if(NULL == (mount=sonar[i]->getMount()))
	{
	  setMsgPtr->sSetElem[count].mountID=-1;
	  setMsgPtr->sSetElem[count].linkID=0;
	}
      else
	{
	  setMsgPtr->sSetElem[count].mountID=mount->misPkg->getMisPkgID();
	  setMsgPtr->sSetElem[count].linkID=mount->link->getLinkID();
	}
      setMsgPtr->sSetElem[count].sensorToMount=sonar[i]->getSen2Mount();
      setMsgPtr->sSetElem[count].desiredPeriod=0;
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_SONAR_MAX_RANGE]=sonar[i]->getMaxRange();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_SONAR_MIN_RANGE]=sonar[i]->getMinRange();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_SONAR_BEAM_DISPERSION]=sonar[i]->getBeamDispersion();
      setMsgPtr->sSetElem[count].parameters_length=3;
      count++;
    }
  str.str("");
  
  str << SERVO_SP_NAV_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1;

  for(i=0;i<navSen.size();i++)
    {
      sprintf(setMsgPtr->sSetElem[count].senDataBufName,"%s",str.str().c_str());
      setMsgPtr->sSetElem[count].sensorID=navSen[i]->getSenID();
      if(NULL == (mount=navSen[i]->getMount()))
	{
	  setMsgPtr->sSetElem[count].mountID=-1;
	  setMsgPtr->sSetElem[count].linkID=0;
	}
      else
	{
	  setMsgPtr->sSetElem[count].mountID=mount->misPkg->getMisPkgID();
	  setMsgPtr->sSetElem[count].linkID=mount->link->getLinkID();
	}
      setMsgPtr->sSetElem[count].sensorToMount=navSen[i]->getSen2Mount();
      setMsgPtr->sSetElem[count].desiredPeriod=0;
      
      switch(navSen[i]->getSensorType())
	{
	case SW_KB_INU_SENSOR_TYPE:
	  setMsgPtr->sSetElem[count].sType=SERVO_SP_INU_SENSOR;
	  setMsgPtr->sSetElem[count].parameters_length=0;
	  break;
	case SW_KB_INS_SENSOR_TYPE:
	  setMsgPtr->sSetElem[count].sType=SERVO_SP_INS_SENSOR;
	  setMsgPtr->sSetElem[count].parameters_length=0;
	  break;
	  //case SW_KB_INS_SENSOR:
	  // break;
	case SW_KB_ODOMETRY_SENSOR_TYPE:
	  setMsgPtr->sSetElem[count].sType=SERVO_SP_ODOMETRY_SENSOR;
	  setMsgPtr->sSetElem[count].parameters_length=0;
	  break;
	case SW_KB_GPS_SENSOR_TYPE:
	  setMsgPtr->sSetElem[count].sType=SERVO_SP_GPS_SENSOR;
	  setMsgPtr->sSetElem[count].parameters_length=0;
	  break;
	case SW_KB_GRD_TRUTH_SENSOR_TYPE:
	  setMsgPtr->sSetElem[count].sType=SERVO_SP_GRD_TRUTH_SENSOR;
	  // FIXME
	  //  setMsgPtr->sSetElem[count].parameters[SERVO_SP_GRD_TRUTH_UTM_OFFSET_NORTHING]=(mSkin->kb->world.getUtmOffset()).x;
	  // 	  setMsgPtr->sSetElem[count].parameters[SERVO_SP_GRD_TRUTH_UTM_OFFSET_EASTING]=(mSkin->kb->world.getUtmOffset()).y;
	  // 	  setMsgPtr->sSetElem[count].parameters[SERVO_SP_GRD_TRUTH_UTM_OFFSET_Z_VALUE]=(mSkin->kb->world.getUtmOffset()).z;
	  // 	  setMsgPtr->sSetElem[count].parameters[SERVO_SP_GRD_TRUTH_UTM_LETTER]=mSkin->kb->world.getUtmLetter();
	  // 	  setMsgPtr->sSetElem[count].parameters[SERVO_SP_GRD_TRUTH_UTM_ZONE]=mSkin->kb->world.getUtmZone();
	  setMsgPtr->sSetElem[count].parameters_length=5;
	  break;
	default:
	  continue;
	  break;
	}
      count++;
    }


  str.str("");  
  str << SERVO_SP_RFID_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1;
  for(i=0;i<rfid.size();i++)
    {
      setMsgPtr->sSetElem[count].sType=SERVO_SP_RFID_SENSOR;
      sprintf(setMsgPtr->sSetElem[count].senDataBufName,"%s",str.str().c_str());
      setMsgPtr->sSetElem[count].sensorID=rfid[i]->getSenID();
      if(NULL == (mount=rfid[i]->getMount()))
	{
	  setMsgPtr->sSetElem[count].mountID=-1;
	  setMsgPtr->sSetElem[count].linkID=0;
	}
      else
	{
	  setMsgPtr->sSetElem[count].mountID=mount->misPkg->getMisPkgID();
	  setMsgPtr->sSetElem[count].linkID=mount->link->getLinkID();
	}
      setMsgPtr->sSetElem[count].sensorToMount=rfid[i]->getSen2Mount();
      setMsgPtr->sSetElem[count].desiredPeriod=0;
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_RFID_MAX_RANGE]=rfid[i]->getMaxRange();
      setMsgPtr->sSetElem[count].parameters_length=1;
      count++;
    }

  str.str("");    
  str << SERVO_SP_VICTIM_BUF_NAME << mSkin->kb->bot->getBotID() << "_" << 1;
  for(i=0;i<victim.size();i++)
    {
      setMsgPtr->sSetElem[count].sType=SERVO_SP_VICTIM_SENSOR;
      sprintf(setMsgPtr->sSetElem[count].senDataBufName,"%s",str.str().c_str());
      setMsgPtr->sSetElem[count].sensorID=victim[i]->getSenID();
      if(NULL == (mount=victim[i]->getMount()))
	{
	  setMsgPtr->sSetElem[count].mountID=-1;
	  setMsgPtr->sSetElem[count].linkID=0;
	}
      else
	{
	  setMsgPtr->sSetElem[count].mountID=mount->misPkg->getMisPkgID();
	  setMsgPtr->sSetElem[count].linkID=mount->link->getLinkID();
	}
      setMsgPtr->sSetElem[count].sensorToMount=victim[i]->getSen2Mount();
      setMsgPtr->sSetElem[count].desiredPeriod=0;
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_VICTIM_MAX_RANGE]=victim[i]->getMaxRange();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_VICTIM_HORZ_FOV]=victim[i]->getHorizontalFOV();
      setMsgPtr->sSetElem[count].parameters[SERVO_SP_VICTIM_VERT_FOV]=victim[i]->getVerticalFOV();
      setMsgPtr->sSetElem[count].parameters_length=3;
      count++;
    }

  str.str("");
  for(i=0;i<xSen.size();i++)
    {
      switch(xSen[i]->getSensorType())
	{
	case SW_KB_CAMERA_SENSOR_TYPE:
	  setMsgPtr->sSetElem[count].senDataBufName[0]=(char)NULL;
	  setMsgPtr->sSetElem[count].sensorID=xSen[i]->getSenID();
	  setMsgPtr->sSetElem[count].sensorToMount=xSen[i]->getSen2Mount();
	  setMsgPtr->sSetElem[count].desiredPeriod=0;
	  setMsgPtr->sSetElem[count].sType=SERVO_SP_CAMERA_SENSOR;
	  if(NULL == (mount=xSen[i]->getMount()))
	    {
	      setMsgPtr->sSetElem[count].mountID=-1;
	      setMsgPtr->sSetElem[count].linkID=0;
	    }
	  else
	    {
	      setMsgPtr->sSetElem[count].mountID=mount->misPkg->getMisPkgID();
	      setMsgPtr->sSetElem[count].linkID=mount->link->getLinkID();
	    }
	  setMsgPtr->sSetElem[count].parameters_length=3;
	  setMsgPtr->sSetElem[count].parameters[SERVO_SP_CAMERA_MIN_FOV]=CLASS_CAST(SIMwareKBSenCamera *, xSen[i])->getMinFov();
	  setMsgPtr->sSetElem[count].parameters[SERVO_SP_CAMERA_MAX_FOV]=CLASS_CAST(SIMwareKBSenCamera *, xSen[i])->getMaxFov();
	  setMsgPtr->sSetElem[count].parameters[SERVO_SP_CAMERA_DEFAULT_FOV]=CLASS_CAST(SIMwareKBSenCamera *, xSen[i])->getDefFov();
	  count++;
	  break;
	  
	default:
	  break;
	}
    }
  setMsgPtr->sSetElem_length=count;
  setBuf->write(setMsgPtr);
}



