#include "trafficSupSkin.hh"

void *
spawnTrafficThr (void *tArgs)
{
  SIMwareTraffic *node = (SIMwareTraffic *) tArgs;
  node->run ();
  pthread_exit ((void *) 0);
  return (void *) 0;
}

TrafficSupSkin::TrafficSupSkin()  
{
  if(1)
    printf( "<TrafficSupSkin:: constructor>\n" );
  traffic=NULL;
}

TrafficSupSkin::~TrafficSupSkin() 
{
  if(1)
    printf("<TrafficSupSkin::~TrafficSupSkin>\n");
  closeSkin();
}



void TrafficSupSkin::configSkin(SIMwareKB ** kBase, SIMwareInfSkin ** iSkin, int numVehIn) 
{
  kb_list= kBase;
  infSkin_list=iSkin;
  configSupSkin(kBase[0],iSkin[0]);
  numVeh = numVehIn;
}

void TrafficSupSkin::configSupSkin(SIMwareKB * kBase, SIMwareInfSkin * iSkin) 
{
  if(0)
    printf("<TrafficSupSkin::configSupSkin>\n");
  kb=kBase;
  infSkin = iSkin;
  state = SW_KB_STATE_UNINITIALIZED;
}

bool TrafficSupSkin::openSkin() 
{
  extern char *SIMWARE_INIFILE_PATH;
  double vmax, amax, wmax, alphamax, vCutoffAngle, wCutoffAngle;
  double d1, d2, d3, d4, d5, d6;
  PM_POSE controlPoint;
  PM_RPY rpy;
  INIFILE if_d;
  const char *strPtr;
  map <const char *, SIMwareKBSen *, strCmp>::iterator itr;

  if (1)
    printf ("<TrafficSupSkin::openSkin> : Opening inifile %s\n",
	    SIMWARE_INIFILE_PATH);

  if (if_d.open (SIMWARE_INIFILE_PATH))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> : Invalid inifile path %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }

  if (NULL == (strPtr = if_d.find ("NML_CFG_FILE", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'NML_CFG_FILE' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  nmlCfgFile = strPtr;
  
  if (NULL == (strPtr = if_d.find ("NML_PROC_NAME", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'NML_PROC_NAME' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  nmlProcName = strPtr;

  if (NULL == (strPtr = if_d.find ("MAX_TRAN_VEL", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'MAX_TRAN_VEL' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  vmax = atof(strPtr);
  if (NULL == (strPtr = if_d.find ("MAX_TRAN_ACC", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'MAX_TRAN_ACC' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  amax = atof(strPtr);
  if (NULL == (strPtr = if_d.find ("MAX_ROT_VEL", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'MAX_ROT_VEL' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  wmax = RAD(atof(strPtr));
  if (NULL == (strPtr = if_d.find ("MAX_ROT_ACC", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'MAX_ROT_ACC' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  alphamax = RAD(atof(strPtr));
  if (NULL == (strPtr = if_d.find ("V_CUTOFF_ANGLE", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'V_CUTOFF_ANGLE' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  vCutoffAngle = RAD(atof(strPtr));
  if (NULL == (strPtr = if_d.find ("W_CUTOFF_ANGLE", "TRAFFIC.SUPSKIN")))
    {
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> 'W_CUTOFF_ANGLE' not defined in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  wCutoffAngle = RAD(atof(strPtr));

  if (NULL == (strPtr = if_d.find ("CONTROL_POINT", "TRAFFIC.SUPSKIN"))) {
    // ignore missing CONTROL_POINT and set to identity pose
    controlPoint = PM_POSE(PM_CARTESIAN(0,0,0),PM_QUATERNION(1,0,0,0));
  } else {
    if (6 != sscanf(strPtr, "%lf %lf %lf %lf %lf %lf", &d1, &d2, &d3, &d4, &d5, &d6)) {
      // found, but invalid
      printf
	("** ERROR ** <TrafficSupSkin::openSkin> invalid 'CONTROL_POINT' in section 'TRAFFIC.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    } else {
      controlPoint.tran.x = d1;
      controlPoint.tran.y = d2;
      controlPoint.tran.z = d3;
      rpy.r = RAD(d4);
      rpy.p = RAD(d5);
      rpy.y = RAD(d6);
      controlPoint.rot = rpy;
    }
  }

  // done with ini reads
  if_d.close ();
  if( 1 )
    printf("<TrafficSupSkin::openSkin>\n\tNML Config file  : %s\n\tNML Process Name : %s \n", 
	   nmlCfgFile.c_str(), nmlProcName.c_str());
  

  traffic = new SIMwareTraffic(this, vmax, amax, wmax, alphamax, vCutoffAngle, wCutoffAngle, controlPoint);
  if( ! traffic->openNML(TRAFFIC_CTRL_CMD_NAME,TRAFFIC_CTRL_STAT_NAME,
		     TRAFFIC_CTRL_CFG_NAME,TRAFFIC_CTRL_SET_NAME,  
		     trafficCtrl_format))
    {
      printf("** ERROR ** <TrafficSupSkin::openSkin> : Error Initializing Trafficility Object\n");  
      return false;
    }
  

  //** First get handle to kb
  // figure out handles for ground truth sensor
  grdTruth = new SIMwareKBSenGrdTruth* [numVeh];
  vector<SIMwareKBSen *> navSen;

  for( int count=0; count<numVeh; count++ )
    {
      itr = kb_list[count]->senMap.begin();
      while ( itr != kb_list[count]->senMap.end() ) 
	{
	  switch( itr->second->getSensorType())
	    {
	    case SW_KB_GRD_TRUTH_SENSOR_TYPE:
	      grdTruth[count] = (SIMwareKBSenGrdTruth*)itr->second;
	      break;
	    default:
	      printf
		("** WARNING ** <SIMwareTraffic::openSkin> Sensor %s not added, sensor type %d not needed for traffic\n",
		 itr->first, itr->second->getSensorType());
	      break;
	    }      
	  itr++;
	}
    }
  //** Then start thread that uses kb
  thr.push_back(new pthread_t);
  if ( 0 != pthread_create (thr[thr.size()-1], NULL, (void *(*)(void *)) spawnTrafficThr,
			    (void *) traffic) )
    {
      printf("** ERROR ** <TrafficSupSkin::openSkin> : Error Spawning Trafficility thread\n");  
      return false;
    }

  state=SW_KB_STATE_INITIALIZED;
  
  nml_start();
  
  return true;
}

void TrafficSupSkin::closeSkin() 
{

  printf("<TrafficSupSkin::closeSkin>\n");
  state = SW_KB_STATE_SHUTDOWN;

  if (1)
    printf ("<TrafficSupSkin::closeSkin> : Cleaning up NML buffers \n");
  nml_cleanup();

  if (1)
    printf ("<TrafficSupSkin::closeSkin> : Cancelling Parser Threads\n");
  
  for(unsigned int i=0;i<thr.size();i++)
     pthread_cancel(*thr[i]);

  if(traffic!=NULL)
    delete traffic;
  traffic=NULL;
  if(grdTruth != NULL)
    delete grdTruth;
  grdTruth = NULL;
}
