/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file trafficSupSkin.hh

  \brief 
  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.3 $
  $Date: 2007/05/03 16:34:51 $
  \endcode

  \author Chris Scrapper
*/

#ifndef TRAFFIC_SUP_SKIN_HH
#define TRAFFIC_SUP_SKIN_HH

#include <string>
#include <map>
#include <sstream>
using namespace std;

//! SIMware includes 
#include "swBaseSkin.hh"
#include "simWareKB.hh"
#include "swKBTypes.hh"
#include "swTraffic.hh"
#include <inifile.h>
#include <pthread.h>


class SIMwareKB;

void * spawnTrafficThr (void *tArgs);

class TrafficSupSkin : public SIMwareSupSkin {
  //  friend class SIMwareServoNodeBase;
  friend class SIMwareTraffic;

private:
  pthread_t mobThr; 
  pthread_t spThr; 
  vector <pthread_t *> thr;
  vector <pthread_attr_t *> thrAttr;
  SIMwareTraffic * traffic;
  
  string nmlProcName;
  string nmlCfgFile;
  
  SIMwareKB ** kb_list;
  SIMwareInfSkin ** infSkin_list;
  int numVeh;

public:
  SIMwareKBSenGrdTruth **grdTruth;

  TrafficSupSkin();
  ~TrafficSupSkin();
  void configSkin(SIMwareKB **, SIMwareInfSkin **, int);
  void configSupSkin(SIMwareKB *, SIMwareInfSkin *);
  bool openSkin();
  void closeSkin();
  int getNumVeh(){return numVeh;};
};
#endif
