/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   nmlRead.cc
  \brief example of reading an NML buffer

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.3 $
  $Date: 2007/10/16 17:13:08 $
  \endcode

  \author Stephen Balakisky
  \date   10/15/2007
*/

#include <stddef.h>		// NULL
#include <stdlib.h>		// getenv()
#include "rcs.hh"		// RCS_CMD,STAT_CHANNEL, rcs_print()
#include "skelCommon.hh"
#include "skel.hh"

static RCS_CMD_CHANNEL * skel_cmd_buf= NULL;

#define DEFAULT_NML_FILE "./skel.nml"
#define SKEL_CMD_NAME "skelCmd" // NML cmd buffer name
#define THIS_PROCESS "nmlRead" // NML process name for the server

static int init_nml_buffers(char * config_file)
{
  if (NULL != skel_cmd_buf) delete skel_cmd_buf;
  skel_cmd_buf = new RCS_CMD_CHANNEL(skel_format, SKEL_CMD_NAME, THIS_PROCESS, config_file);
  if (! skel_cmd_buf->valid()) {
    delete skel_cmd_buf;
    skel_cmd_buf = 0;
    return 1;
  }
  skel_cmd_buf->clear();
  return 0;
}

#define max(a,b) ((a) > (b) ? (a) : (b))

int main(int argc, char *argv[])
{
  char config_file_default[] = DEFAULT_NML_FILE;
  char * config_file_env;
  char * NML_FILE = config_file_default;
  int retval;
  NMLTYPE cmdType;
  RCS_TIMER timer(.1);

  // get the config file name from arg, if provided, next environment,
  // if there, else default
  if (argc > 1) {
    NML_FILE = argv[1];
    rcs_print("%s: using specified config file %s\n", argv[0], NML_FILE);
  } else {
    config_file_env = getenv("CONFIG_NML");
    if (NULL != config_file_env) {
      NML_FILE = config_file_env;
      rcs_print("%s: using environment config file %s\n", argv[0], NML_FILE);
    } else {
      rcs_print("%s: using default config file %s\n", argv[0], NML_FILE);
    }
  }

  // initial allocate all the buffers
  retval = init_nml_buffers(NML_FILE);

  if (retval) {
    rcs_print_error("%s: can't allocate NML buffers\n", argv[0]);
    return 1;
  }

  // read a value from the buffer
  /****************************************************************** 
   * READ COMMAND - Command information is received, and the system
   *                is prepared to execute the command. When a new
   *		    command is received, the old command is immediately
   *		    replaced by this command. If the command is not 
   *		    able to be interrupted, error flags are set and
   *		    will be reported during the writng of status
   */

    // read command
  for(;;)
    {
      cmdType = skel_cmd_buf->read();
      switch (cmdType) {
      case 0:
	// no new command
	break;
      case -1:
	// comm error
	rcs_print_error("%s: NML command read error\n", argv[0]);
	break;
      case SKEL_CMD_GO_TYPE:
	rcs_print( "new Go command received with value: %f\n", ((SkelCmdGo *) skel_cmd_buf->get_address())->where );
      break;
      case SKEL_CMD_SHUTDOWN_TYPE:
	nml_cleanup();
	rcs_print( "Exiting...\n" );
	return 0;
      }
      timer.wait();
    }
  // will never get here
  return 0;
}

/*
  Modification history:

  $Log: nmlRead.cc,v $
  Revision 1.3  2007/10/16 17:13:08  dr_steveb
  Added timer to loop

  Revision 1.2  2007/10/16 12:53:48  dr_steveb
  Added the ability to have programs exit and some comments.

  Revision 1.1  2007/10/15 21:09:55  dr_steveb
  Changes made for IEEE workshop presentation

*/
