/*!
  \file   skelApp.java
  \brief Skeleton RCS application.

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.5 $
  $Date: 2009/02/24 19:40:32 $
  \endcode

  \author Stephen Balakirsky (based on skelApp.cc)
  \date   03/2/2005
*/

import java.io.*;

// Import all NML, CMS, and RCS classes and interfaces
//import rcs.*;
import rcs.nml.*;

// Import MOAST
import moast.*;

// Import the state definitions
//import rcs_states.*;


public class skelApp
{
    static rcs_states rcsState = new rcs_states();
    static NMLConnection primCmdBuf;
    static NMLConnection primStatBuf;
    static NMLConnection primCfgBuf;
    static NMLConnection primSetBuf;

    static NMLConnection navDataBuf;

    static NMLConnection servoMobCmdBuf;
    static NMLConnection servoMobStatBuf;
    static NMLConnection servoMobSetBuf;
    static NMLConnection servoMobCfgBuf;

    static PrimMobJAStat primStat = new PrimMobJAStat();
    static PrimMobJASet primSet = new PrimMobJASet();

    static String DEFAULT_NML_FILE = "../../etc/moast.nml";
    static String PRIM_CMD_NAME = "primMobJACmd1"; // NML cmd buffer name
    static String PRIM_STAT_NAME = "primMobJAStat1"; // NLM stat buffer name
    static String PRIM_CFG_NAME = "primMobJACfg1"; // NML stat buffer name
    static String PRIM_SET_NAME = "primMobJASet1"; // NML stat buffer name
    static String THIS_PROCESS = "primMobJA"; // NML process name for the server

    static int direction =1;
	
    public static int initNmlBuffers(String config_file)
    {
	try
	    {
		primCmdBuf = new NMLConnection(new primMobJA_MsgDict(), PRIM_CMD_NAME, THIS_PROCESS, config_file);
		primStatBuf = new NMLConnection(new primMobJA_MsgDict(), PRIM_STAT_NAME, THIS_PROCESS, config_file);
		primCfgBuf = new NMLConnection(new primMobJA_MsgDict(), PRIM_CFG_NAME, THIS_PROCESS, config_file);
		primSetBuf = new NMLConnection(new primMobJA_MsgDict(), PRIM_SET_NAME, THIS_PROCESS, config_file);
		servoMobCmdBuf = new NMLConnection(new servoMobJA_MsgDict(), "servoMobJACmd1", THIS_PROCESS, config_file);
		servoMobStatBuf = new NMLConnection(new servoMobJA_MsgDict(), "servoMobJAStat1", THIS_PROCESS, config_file);
		servoMobCfgBuf = new NMLConnection(new servoMobJA_MsgDict(), "servoMobJACfg1", THIS_PROCESS, config_file);
		servoMobSetBuf = new NMLConnection(new servoMobJA_MsgDict(), "servoMobJASet1", THIS_PROCESS, config_file);
		navDataBuf = new NMLConnection(new navDataExt_MsgDict(), "navDataExt1", THIS_PROCESS, config_file);
	    }
	catch(NMLException nml_except)
	    {
		System.out.println("Cannot open nml buffer.");
		return 1;
	    }
	return 0;
    }
  
    static void doCmdNop(PrimMobJAStat stat)
    {
      if (rcsState.state_match(stat, rcsState.NEW_COMMAND)){
	    rcsState.state_new(stat);
	    rcsState.state_next(stat, rcsState.S0);
	    rcsState.status_next(stat, RCS_STAT_MSG.RCS_DONE);
	}
    }

    static void doCmdInit(PrimMobJAStat stat)
    {
	if (rcsState.state_match(stat, rcsState.NEW_COMMAND)) {
	    rcsState.state_new(stat);
	    if( primSet.debug > 1 )
		System.out.println( "In Init command" );
	    stat.admin_state = rcsState.ADMIN_INITIALIZED;
	    rcsState.status_next(stat, RCS_STAT_MSG.RCS_DONE);
	    rcsState.state_next(stat, rcsState.S0);
	} else {
	    rcsState.state_default(stat);
	}
    }

    static void	doCmdMoveWaypoint(PrimMobJAStat stat, PrimMobJACmdMoveWaypoint cmdIn)
    {
	if (rcsState.state_match(stat, rcsState.NEW_COMMAND)) {
	    rcsState.state_new(stat);
	    if( primSet.debug > 1 )
		System.out.println( "In MoveWaypoint command" );
	    for( int i=0; i<cmdIn.waypoint_length; i++ )
		{
		    System.out.println( "waypoint < " + 
					cmdIn.waypoint[i].p.x +
					" " +
					cmdIn.waypoint[i].p.y +
					" > speed: " +
					cmdIn.waypoint[i].speed +
					" error distance: " +
					cmdIn.waypoint[i].neighborhood );
		}
	    rcsState.status_next(stat, RCS_STAT_MSG.RCS_DONE);
	    rcsState.state_next(stat, rcsState.S0);
	} else {
	    rcsState.state_default(stat);
	}
    }

    static void doCmdAbort(PrimMobJAStat stat)
    {
	if (rcsState.state_match(stat, rcsState.NEW_COMMAND)) {
	    rcsState.state_new(stat);
	    if( primSet.debug > 1 )
		System.out.println( "In Abort command" );
	    stat.admin_state = rcsState.ADMIN_UNINITIALIZED;
	    rcsState.status_next(stat, RCS_STAT_MSG.RCS_DONE);
	    rcsState.state_next(stat, rcsState.S0);
	} else {
	    rcsState.state_default(stat);
	}
    }
    static void doCmdHalt(PrimMobJAStat stat)
    {
	if (rcsState.state_match(stat, rcsState.NEW_COMMAND)) {
	    rcsState.state_new(stat);
	    if( primSet.debug > 1 )
		System.out.println( "In Halt command" );
	    stat.admin_state = rcsState.ADMIN_UNINITIALIZED;
	    rcsState.status_next(stat, RCS_STAT_MSG.RCS_DONE);
	    rcsState.state_next(stat, rcsState.S0);
	} else {
	    rcsState.state_default(stat);
	}
    }
    static void doCmdShutdown(PrimMobJAStat stat)
    {
	if (rcsState.state_match(stat, rcsState.NEW_COMMAND)) {
	    rcsState.state_new(stat);
	    if( primSet.debug > 1 )
		System.out.println( "In Shutdown command" );
	    stat.admin_state = rcsState.ADMIN_SHUT_DOWN;
	    rcsState.status_next(stat, RCS_STAT_MSG.RCS_DONE);
	    rcsState.state_next(stat, rcsState.S0);
	} else {
	    return;
	}
    }


    static int done = 0;
    static void quit(int sig)
    {
	done = 1;
    }

    public static void main(String[] args) throws IOException, InterruptedException
    {
	String NMLFile;
	String configFileDefault = DEFAULT_NML_FILE;
	int retval;
	int serialNumber;
	PrimMobJACmdNop primCmdNop = new PrimMobJACmdNop();
	PrimMobJACmdMoveWaypoint primCmdMoveWaypoint = new PrimMobJACmdMoveWaypoint();
	PrimMobJACfgNop primCfgNop = new PrimMobJACfgNop();
	NavDataExt navDataExtMsg;

	RCS_CMD_MSG cmdMsgIn, configMsgIn;
	NMLmsg navDataMsgIn;
  
	System.out.println("primApp Java Version!");

	/****************************************************************** 
	 * INITIALIZE - The initialization opens any communication buffers, 
	 *              places the syste in a safe known state, and 
	 *		  initializes any control parameters.
	 */
	
	// get the config file name from arg, if provided, next environment,
	// if there, else default
	if (args.length > 1) 
	    {
		NMLFile = args[0];
		System.out.println("using specified config file " + NMLFile);
	    }
	else 
	    {
		NMLFile = configFileDefault;
		System.out.println("using default config file " + NMLFile);
	    }

	// allocate all the buffers
	retval = initNmlBuffers(NMLFile);

	if (retval == 1) {
	    System.err.println("can't allocate NML buffers" );
	    return;
	}

	// stuff a nop command
	primCmdNop.serial_number = 1;
	primCmdBuf.write(primCmdNop);

	// set the status up so that init command goes through
	primStat.echo_serial_number = 0;
	
	// stuff a nop config
	primCfgNop.serial_number = 1;
	primCfgBuf.write(primCfgNop);

	// set the settings up so that the length goes through
	primSet.cycleTime = 0.1;
	primSet.debug=2;
	primSet.echo_serial_number = 0;

	// initialize the time tracker
	primStat.tt.count = 0;

	done = 0;

	while (done==0) {
	    /******************************
	     * House keeping: set time tracker to increment
	     */
	    //	    cycle_time_tracker( primStat.tt );

	    /****************************************************************** 
	     * READ COMMAND - Command information is received, and the system
	     *                is prepared to execute the command. When a new
	     *		    command is received, the old command is immediately
	     *		    replaced by this command. If the command is not 
	     *		    able to be interrupted, error flags are set and
	     *		    will be reported during the writng of status
	     */
	    
	    // read data if there is any to read
	    
	    // read command
		try
		    {
			cmdMsgIn = (RCS_CMD_MSG)primCmdBuf.read();
		    }
		catch(NMLException nml_except)
		    {
			System.out.println("Cannot read from nml buffer.");
			return;
		    }

		if (cmdMsgIn != null){
		  serialNumber = cmdMsgIn.serial_number;
		  switch (cmdMsgIn.type) {
		  case 0:
		    // no new command
		    break;
		  case -1:
		    // comm error
		    System.err.println("NML command read error\n");
		    break;
		  case primMobJA_MsgDict.PrimMobJACmdNop_TYPE:
		  case primMobJA_MsgDict.PrimMobJACmdInit_TYPE:
		  case primMobJA_MsgDict.PrimMobJACmdHalt_TYPE:
		  case primMobJA_MsgDict.PrimMobJACmdShutdown_TYPE:
		  case primMobJA_MsgDict.PrimMobJACmdAbort_TYPE:
		  case primMobJA_MsgDict.PrimMobJACmdMoveWaypoint_TYPE:
		    break;
		  default:

		    System.err.println("unknown command: " + (int) cmdMsgIn.type);
		    break;
		  } // switch (cmdType)
		  if (serialNumber != primStat.echo_serial_number) {
		      if( primSet.debug > 0 )
			  System.out.println( "New command received " + cmdMsgIn.type ); // skel_symbol_lookup(cmdType) );
		      primStat.command_type = cmdMsgIn.type;
		      primStat.echo_serial_number = serialNumber;
		      primStat.state = rcsState.NEW_COMMAND;
		      primStat.status = RCS_STAT_MSG.RCS_EXEC;
		  }

		}

	    /****************************************************************** 
	     * READ CONFIG - Configuration information is received, and the 
	     *               system is prepared to change its setting. A separate 
	     *               configuration channel is provided to allow for control 
	     *               parameters to be changed without interrupting the 
	     *               current controller. For example, a user may want to 
	     *               change a system's cycle time without interrupting a 
	     *               complex control function. The configuration channel is 
	     *               used to change the cycle time parameters without 
	     *               aborting the control algorithm.
	     */
	    
	    // read config
		try
		    {
			configMsgIn = (RCS_CMD_MSG)primCfgBuf.read();
		    }
		catch(NMLException nml_except)
		    {
			System.out.println("Cannot read from  nml cfg buffer.");
			return;
		    }
		if (configMsgIn != null)
		  {

		    serialNumber = configMsgIn.serial_number;
		    switch (configMsgIn.type) {
		    case 0:
		      // no new command
		      break;
		    case -1:
		      // comm error
		      System.err.println("NML config read error" );
		      break;
		    case primMobJA_MsgDict.PrimMobJACfgNop_TYPE:
		    case primMobJA_MsgDict.PrimMobJACfgDebug_TYPE:
		    case primMobJA_MsgDict.PrimMobJACfgCycleTime_TYPE:
		      if (serialNumber != primSet.echo_serial_number) {
			if( primSet.debug > 0 )
			  System.out.println( "New config received" + configMsgIn.type ); //prim_symbol_lookup(cfgType) );
			primSet.command_type = configMsgIn.type;
			primSet.echo_serial_number = serialNumber;
			primSet.state = rcsState.NEW_COMMAND;
			primSet.status = RCS_STAT_MSG.RCS_EXEC;
		      }
		      break;

		    default:
		      System.err.println("unknown config: " + (int) configMsgIn.type);
		      break;
		    } // switch (cfgType)
		  }

		///////////////////////// 
		/* read data */
		/////////////////////////
		try
		    {
			navDataMsgIn = (NMLmsg)navDataBuf.read();
		    }
		catch(NMLException nml_except)
		    {
			System.out.println("Cannot read from  nml navData buffer.");
			return;
		    }
		if (navDataMsgIn != null){

		    switch (navDataMsgIn.type) {
		    case 0:
		      // no new command
		      break;
		    case -1:
		      // comm error
			System.err.println("NML config read error" );
			break;
		    case navDataExt_MsgDict.NavDataExt_TYPE:
			navDataExtMsg = (NavDataExt)navDataMsgIn;
			break;
		    }
		}
	    
	    /****************************************************************** 
	     * COMMAND EXECUTION - Command execution is carried out. 
	     *                     All of the modules run on a fixed cycle time. 
	     *                     Therefore, command functions must either 
	     *                     guarantee that they finish in under the cycle time, 
	     *                     or provide for being reentrant. Although this 
	     *                     example refers to the command execution as being 
	     *                     finite state machine (FSM) based, search based 
	     *                     planning systems have also been implemented under 
	     *                     this framework. The searcher simply stores its 
	     *                     current location in the search when its time 
	     *                     expires and then resumes searching on the next cycle.
	     */
	    
	    // handle command state tables
	    switch ((int)primStat.command_type) {
	    case primMobJA_MsgDict.PrimMobJACmdNop_TYPE:
		doCmdNop(primStat);
		break;
	    case primMobJA_MsgDict.PrimMobJACmdInit_TYPE:
		doCmdInit(primStat);
		break;
	    case primMobJA_MsgDict.PrimMobJACmdHalt_TYPE:
		doCmdHalt(primStat);
		break;
	    case primMobJA_MsgDict.PrimMobJACmdAbort_TYPE:
		doCmdAbort(primStat);
		break;
	    case primMobJA_MsgDict.PrimMobJACmdShutdown_TYPE:
		doCmdShutdown(primStat);
		break;
	    case primMobJA_MsgDict.PrimMobJACmdMoveWaypoint_TYPE:
		primCmdMoveWaypoint = (PrimMobJACmdMoveWaypoint)cmdMsgIn;
		doCmdMoveWaypoint(primStat, primCmdMoveWaypoint);
		break;
	    default:
		// no command to handle
		break;
	    } // switch (cmd)


	    /****************************************************************** 
	     * UPDATE STATUS AND SETTING - The current module status and the 
	     *                             configuration's settings are sent
	     *                             out over communications buffers.
	     */
	    // update status and settings
	    primStatBuf.write(primStat);
	    primSetBuf.write(primSet);
	    
	    Thread.sleep((int)primSet.cycleTime);

	}
	
	//	return cleanupNmlBuffers();
    }
}
/*
  Modification history:

  $Log: skelApp.java,v $
  Revision 1.5  2009/02/24 19:40:32  dr_steveb
  Java code should now work!

  Revision 1.1  2009/01/18 04:06:43  dr_steveb
  Another try at the java code

*/
