/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/

/*!
  \file inifind.cc
  \brief Prints out ini file entries.
  Syntax: inifind <file> <variable> <section>
  Example: For this ini file:
  ---moast.ini---
  [MOAST]
  VERSION = 1
  ---end---
  inifind moast.ini VERSION MOAST would print "1", and return 0.
  Returns 1 if no entry is found, and prints nothing.

  \code CVS Status:
  $Author: root $
  $Revision: 1.1.1.1 $
  $Date: 2005/10/12 20:50:32 $
  \endcode

  \author Fred Proctor
*/

#include <stdio.h>              /* printf(), fprintf(), FILE, fopen() */
#include "inifile.h"            /* iniFind() */

/*
  Syntax: inifind <file> <variable> <section>
*/

int main(int argc, char *argv[])
{
  FILE *fp;
  const char *inistring;

  if (argc != 4) {
    fprintf(stderr, "%s: need <file> <variable> <section>\n", argv[0]);
    return 1;
  }

  if (NULL == (fp = fopen(argv[1], "r"))) {
    fprintf(stderr, "%s: can't open %s\n", argv[0], argv[1]);
    return 1;
  }

  inistring = iniFind(fp, argv[2], argv[3]);
  if (NULL != inistring) {
    printf("%s\n", inistring);
  }

  fclose(fp);

  return 0;
}

/*
  Modification history:

  $Log: inifind.cc,v $
  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.1  2005/06/30 19:57:46  proctor
  Moved inifind.c to inifind.cc, to get a C++ compile and link with rcslib

  Revision 1.1  2005/06/28 13:21:34  proctor
  Initial checkin

*/
