/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   nmlConvert.cc
  \brief  convert autogenerated nml file for use on the PC with autoSim

  This program changes some keywords in the pbc.nml file and creates a new
  file named autoSim.nml that will be copied to the PC for use with autoSim

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.3 $
  $Date: 2006/03/26 15:37:05 $
  \endcode

  \author Stephen Balakirsky
  \date   2003-12-10
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

int main(int argc, char **argv)
{
  FILE *inFile, *outFile;
  char *lineptr = NULL;
  size_t n = 500;
  char *wrptr, *wrptr2, needle[20], needle2[20], needle3[20];
  char *linuxHost;
  char *configNml = getenv("CONFIG_NML");

  lineptr = new char[n]; /* needs to be long enough to fit entire line
			    from configuraiton file */
  if(!configNml)
    {
      fprintf(stderr,"Please set CONFIG_NML environment variable.\n");
      return(false);
    }

  strcpy( needle, "localhost" );
  strcpy( needle2, "safTest" );
  strcpy( needle3, "LOCAL" );
  wrptr2 = (char*)malloc(400);

  if( argc!=2 )
    {
      printf( "Usage: %s <IP address or host name>\n", argv[0] );
      exit(1);
    }

  linuxHost = (char*)malloc(80);
  strncpy( linuxHost, argv[1], 80 );
  /*
  if( strlen(linuxHost) < 9 )
    {
      printf( "Makeing linuxHost longer\n" );
      sprintf( linuxHost, "%s        \0", linuxHost );
    }
  */
  inFile = fopen( configNml, "r" );
  if( inFile == NULL )
    {
      printf( "Unable to open input file, %s\n", configNml );
      exit(1);
    }

  outFile = fopen( "remote.nml", "w" );
  if( outFile == NULL )
    {
      printf( "Unable to open output file, remote.nml\n" );
      exit(1);
    }
  //  while( getline( &lineptr, &n, inFile ) != -1 )
  while( fgets( lineptr, n, inFile ) != NULL )
    {
      if( lineptr[0] == 'B' )
	{
	  wrptr = strstr( lineptr, needle );
	  if( wrptr == NULL )
	    {
	      printf( "Unable to find 'localhost' token in line\n" );
	      //	      exit(1);
	    }
	  else
	    {
	      strncpy( wrptr2, lineptr, (wrptr-lineptr) );
	      strcpy( wrptr2+(wrptr-lineptr), "\0");
	      strcat( wrptr2, linuxHost );
	      strcat( wrptr2, wrptr+9 );
	      //	  strncpy( wrptr, linuxHost, strlen(linuxHost) );
	      strcpy( lineptr, wrptr2 );
	    }
	}
      /*
      if( lineptr[0] == 'P' )
	{
	  wrptr = strstr( lineptr, needle2 );
	  if( wrptr != NULL )
	    {
	      wrptr = strstr( lineptr, needle3 );
	      if( wrptr == NULL )
		{
		  printf( "Unable to find 'LOCAL' token in line\n" );
		  //		  exit(1);
		}
	      else
		strncpy( wrptr, "REMOTE", 6 );
	    }
	}
      */
      fprintf( outFile, "%s", lineptr );
    }
  fclose( outFile );
  fclose( inFile );
  return(1);
}

/*! 
 * $Log: nmlConvert.cc,v $
 * Revision 1.3  2006/03/26 15:37:05  dr_steveb
 * replaced getline with fgets for portablity
 *
 * Revision 1.2  2006/03/09 02:44:15  dr_steveb
 * Got rid of int casts that were causing problems on some versions of OS
 *
 * Revision 1.1  2006/02/01 18:59:49  dr_steveb
 * This utility converts the buffer locations in the nml file to be that of a specifed host. Allows the creation of a NML file for remote hosts.
 *
 */
