/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file moastNmlGlobal.hh

  \brief Declarations for NML global convenience buffers, connect/disconnect
  functions.

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.12 $
  $Date: 2009/02/25 21:06:52 $
  \endcode

  \author Fred Proctor
*/

#ifndef MOAST_NML_GLOBAL_HH
#define MOAST_NML_GLOBAL_HH

#define MAX_MOAST_VEHICLES 4
#define USE_SPQR
//! The number of symbolicData buffers to open
#define NUM_SYMBOLIC_DATA_BUFFERS MAX_MOAST_VEHICLES

//! The number of navDataExt buffers to open
#define NUM_NAV_DATA_EXT_BUFFERS 0

//! The number of SP data buffers of various kinds
#define NUM_SERVO_SP_LINESCAN_BUFFERS 0
#define NUM_PRIM_SP_LINESCAN_BUFFERS MAX_MOAST_VEHICLES
#define NUM_AM_SP_MAP_BUFFERS MAX_MOAST_VEHICLES

//! The number of servoMobJA_XX buffers to open
#define NUM_SERVO_MOB_JA_BUFFERS 0

//! The number of servoSP_XX buffers to open
#define NUM_SERVO_SP_BUFFERS 0

//! The number of servoMisJA_XX buffers to open
#define NUM_SERVO_MIS_JA_BUFFERS 0

//! The number of primMobJA_XX buffers to open
#define NUM_PRIM_MOB_JA_BUFFERS MAX_MOAST_VEHICLES

//! The number of primMisJA_XX buffers to open
#define NUM_PRIM_MIS_JA_BUFFERS MAX_MOAST_VEHICLES

//! The number of primSP_XX buffers to open
#define NUM_PRIM_SP_BUFFERS MAX_MOAST_VEHICLES

//! The number of amMobJA_XX buffers to open
#define NUM_AM_MOB_JA_BUFFERS MAX_MOAST_VEHICLES

//! The number of amMisJA_XX buffers to open
#define NUM_AM_MIS_JA_BUFFERS MAX_MOAST_VEHICLES

//! The number of amSP_XX buffers to open
#define NUM_AM_SP_BUFFERS MAX_MOAST_VEHICLES

//! The number of vehJA_XX buffers to open
#define NUM_VEH_MOB_PL_BUFFERS MAX_MOAST_VEHICLES

//! The number of sectJA_XX buffers to open
#define NUM_SECT_MOB_PL_BUFFERS 1

extern NML *moastErrorBuf;

extern NML *symbolicDataBuf[NUM_SYMBOLIC_DATA_BUFFERS];

#if NUM_NAV_DATA_EXT_BUFFERS
extern NML *navDataExtBuf[NUM_NAV_DATA_EXT_BUFFERS];
#else
extern NML *navDataExtBuf;
#endif

#if NUM_SERVO_SP_LINESCAN_BUFFERS
extern NML *servoSPLinescanBuf[NUM_SERVO_SP_LINESCAN_BUFFERS];
#else
extern NML *servoSPLinescanBuf;
#endif

extern NML *primSPLinescanBuf[NUM_PRIM_SP_LINESCAN_BUFFERS];
extern NML *amSPMapBuf[NUM_AM_SP_MAP_BUFFERS];
extern NML *vehMapBuf[NUM_VEH_MOB_PL_BUFFERS];

#if NUM_SERVO_MOB_JA_BUFFERS
extern RCS_CMD_CHANNEL *servoMobJACmdBuf[NUM_SERVO_MOB_JA_BUFFERS];
extern RCS_CMD_CHANNEL *servoMobJACfgBuf[NUM_SERVO_MOB_JA_BUFFERS];
extern RCS_STAT_CHANNEL *servoMobJAStatBuf[NUM_SERVO_MOB_JA_BUFFERS];
extern RCS_STAT_CHANNEL *servoMobJASetBuf[NUM_SERVO_MOB_JA_BUFFERS];
#else
extern RCS_CMD_CHANNEL *servoMobJACmdBuf;
extern RCS_CMD_CHANNEL *servoMobJACfgBuf;
extern RCS_STAT_CHANNEL *servoMobJAStatBuf;
extern RCS_STAT_CHANNEL *servoMobJASetBuf;
#endif

#if NUM_SERVO_SP_BUFFERS
extern RCS_CMD_CHANNEL *servoSPCmdBuf[NUM_SERVO_SP_BUFFERS];
extern RCS_STAT_CHANNEL *servoSPStatBuf[NUM_SERVO_SP_BUFFERS];
extern RCS_STAT_CHANNEL *servoSPSetBuf[NUM_SERVO_SP_BUFFERS];
extern RCS_CMD_CHANNEL *servoSPCfgBuf[NUM_SERVO_SP_BUFFERS];
#else
extern RCS_CMD_CHANNEL *servoSPCmdBuf;
extern RCS_STAT_CHANNEL *servoSPStatBuf;
extern RCS_STAT_CHANNEL *servoSPSetBuf;
extern RCS_CMD_CHANNEL *servoSPCfgBuf;
#endif

#if NUM_SERVO_MIS_JA_BUFFERS
extern RCS_CMD_CHANNEL *servoMisJACmdBuf[NUM_SERVO_MIS_JA_BUFFERS];
extern RCS_STAT_CHANNEL *servoMisJAStatBuf[NUM_SERVO_MIS_JA_BUFFERS];
extern RCS_STAT_CHANNEL *servoMisJASetBuf[NUM_SERVO_MIS_JA_BUFFERS];
extern RCS_CMD_CHANNEL *servoMisJACfgBuf[NUM_SERVO_MIS_JA_BUFFERS];
#else
extern RCS_CMD_CHANNEL *servoMisJACmdBuf;
extern RCS_STAT_CHANNEL *servoMisJAStatBuf;
extern RCS_STAT_CHANNEL *servoMisJASetBuf;
extern RCS_CMD_CHANNEL *servoMisJACfgBuf;
#endif

extern RCS_CMD_CHANNEL *primMobJACmdBuf[NUM_PRIM_MOB_JA_BUFFERS];
extern RCS_STAT_CHANNEL *primMobJAStatBuf[NUM_PRIM_MOB_JA_BUFFERS];
extern RCS_CMD_CHANNEL *primMobJACfgBuf[NUM_PRIM_MOB_JA_BUFFERS];
extern RCS_STAT_CHANNEL *primMobJASetBuf[NUM_PRIM_MOB_JA_BUFFERS];

extern RCS_CMD_CHANNEL *primMisJACmdBuf[NUM_PRIM_MIS_JA_BUFFERS];
extern RCS_STAT_CHANNEL *primMisJAStatBuf[NUM_PRIM_MIS_JA_BUFFERS];
extern RCS_CMD_CHANNEL *primMisJACfgBuf[NUM_PRIM_MIS_JA_BUFFERS];
extern RCS_STAT_CHANNEL *primMisJASetBuf[NUM_PRIM_MIS_JA_BUFFERS];

extern RCS_CMD_CHANNEL *primSPCmdBuf[NUM_PRIM_SP_BUFFERS];
extern RCS_STAT_CHANNEL *primSPStatBuf[NUM_PRIM_SP_BUFFERS];
extern RCS_STAT_CHANNEL *primSPSetBuf[NUM_PRIM_SP_BUFFERS];
extern RCS_CMD_CHANNEL *primSPCfgBuf[NUM_PRIM_SP_BUFFERS];

extern RCS_CMD_CHANNEL *amMobJACmdBuf[NUM_AM_MOB_JA_BUFFERS];
extern RCS_STAT_CHANNEL *amMobJAStatBuf[NUM_AM_MOB_JA_BUFFERS];
extern RCS_CMD_CHANNEL *amMobJACfgBuf[NUM_AM_MOB_JA_BUFFERS];
extern RCS_STAT_CHANNEL *amMobJASetBuf[NUM_AM_MOB_JA_BUFFERS];

extern RCS_CMD_CHANNEL *amMisJACmdBuf[NUM_AM_MIS_JA_BUFFERS];
extern RCS_STAT_CHANNEL *amMisJAStatBuf[NUM_AM_MIS_JA_BUFFERS];
extern RCS_CMD_CHANNEL *amMisJACfgBuf[NUM_AM_MIS_JA_BUFFERS];
extern RCS_STAT_CHANNEL *amMisJASetBuf[NUM_AM_MIS_JA_BUFFERS];

extern RCS_CMD_CHANNEL *amSPCmdBuf[NUM_AM_SP_BUFFERS];
extern RCS_STAT_CHANNEL *amSPStatBuf[NUM_AM_SP_BUFFERS];
extern RCS_STAT_CHANNEL *amSPSetBuf[NUM_AM_SP_BUFFERS];
extern RCS_CMD_CHANNEL *amSPCfgBuf[NUM_AM_SP_BUFFERS];

extern RCS_CMD_CHANNEL *vehMobPLCmdBuf[NUM_VEH_MOB_PL_BUFFERS];
extern RCS_STAT_CHANNEL *vehMobPLStatBuf[NUM_VEH_MOB_PL_BUFFERS];
extern RCS_CMD_CHANNEL *vehMobPLCfgBuf[NUM_VEH_MOB_PL_BUFFERS];
extern RCS_STAT_CHANNEL *vehMobPLSetBuf[NUM_VEH_MOB_PL_BUFFERS];

extern RCS_CMD_CHANNEL *sectMobPLCmdBuf[NUM_SECT_MOB_PL_BUFFERS];
extern RCS_STAT_CHANNEL *sectMobPLStatBuf[NUM_SECT_MOB_PL_BUFFERS];
extern RCS_CMD_CHANNEL *sectMobPLCfgBuf[NUM_SECT_MOB_PL_BUFFERS];
extern RCS_STAT_CHANNEL *sectMobPLSetBuf[NUM_SECT_MOB_PL_BUFFERS];

#ifdef USE_SPQR
extern NML *spqrMapBufP2DX;
extern NML *spqrMapBufZerg;
extern NML *spqrPosBufMapper;
extern NML *spqrPosBufVictim;
#endif

extern NML * joystickDataBuf;

extern int moastZeroNmlBuffers(void);

extern int moastInitNmlBuffers(char *config_file, char *this_process);

extern int moastCleanupNmlBuffers(void);

#endif // MOAST_NML_GLOBAL_HH

/*
  Modification history:

  $Log: moastNmlGlobal.hh,v $
  Revision 1.12  2009/02/25 21:06:52  dr_steveb
  Added VMAC commands to section echelon and made the maximum number of vehicles a #define

  Revision 1.11  2007/07/24 18:38:22  proctor
  Added JoystickData

  Revision 1.10  2007/06/28 12:14:12  dr_steveb
  Removed 0 length arrays that Microsoft compiliers complain about

  Revision 1.9  2006/10/31 09:30:41  dr_steveb
  Changes for SPQR integration

  Revision 1.8  2006/10/30 12:59:34  dr_steveb
  Files added for connection to SPQR

  Revision 1.7  2006/07/18 19:05:15  cj_scrapper
  Made SIM-ware the master of all Servo Echelon buffers that spawns an internal server to server all buffers.  This change is reflected in the moast.cfg file.  Added #if statement in moastNmlGlobl to exclude servo buffers if the number of buffers is 0.  Therefore, moastNmlSvr is not reponsible for servo Echelon buffers.

  Revision 1.6  2006/04/24 21:00:02  proctor
  Added MOAST error NML buffer

  Revision 1.5  2006/04/06 21:37:47  dr_steveb
  Changed number of vehicles to 2. Three ran out of resources

  Revision 1.4  2006/04/06 21:24:52  dr_steveb
  There are now four vehicles supported

  Revision 1.3  2006/02/07 17:46:51  dr_steveb
  Added symbolicData to server and changed its default size.

  Revision 1.2  2005/10/21 13:50:42  dr_steveb
  Added section level buffers

  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.11  2005/07/28 20:54:58  stephen
  Added multiple vehicles (3)

  Revision 1.10  2005/07/18 21:16:28  proctor
  Replaced sensorData with servoSPLinescan, ...

  Revision 1.9  2005/06/23 17:56:30  proctor
  Added servoSPData buffers

  Revision 1.8  2005/06/13 18:56:49  beytin
  Added functionality for amSP NML buffers.

  Revision 1.7  2005/06/13 16:27:54  stephen
  Added primSP

  Revision 1.6  2005/06/10 21:50:11  kramer
  added AM buffers

  Revision 1.5  2005/06/08 20:49:35  stephen
  Changed occupancyData to sensorData

  Revision 1.4  2005/06/08 14:29:54  proctor
  Added vehMobPLMain name change

  Revision 1.3  2005/06/07 21:45:11  proctor
  Changed servoPL to servoMobJA

  Revision 1.2  2005/06/07 15:45:22  proctor
  Added servo,primMis buffers

  Revision 1.1  2005/06/02 18:32:43  proctor
  Initial checkin

*/
