/* nmlMap.hh */
/* This file  created for the RoboCamp 06 practical integration with the SPQR robot
 */
#ifndef NML_MAP_HH
#define NML_MAP_HH
#include "rcs.hh"

// we will use "MapBufP2DX" for the buffer that the P2DX writes
// and "MapBufZerg" for the buffer that the zerg writes
#define MAX_MAP_SIZE 200*200

/* Give the new structure a unique id number */
#define MAP_MSG_TYPE 102
/* The id number must be unique within a CMS buffer, i.e. the number
must be different than the id of any other type that might be written to
a particular buffer. For simplicity it is recommended that the id number
also be unique within an application. */

enum C8Color {
  C8Black =   1, // obstacle
  C8Blue =    2, // unknown
  C8Green =   4,
  C8Cyan =    8,
  C8Red =     16,
  C8Magenta = 32,
  C8White =   64, // free space
  C8Gray =    128};


/* Define the new message structure */
struct MapMsg: public NMLmsg 
{
  /* The constructor needs to store the id number  */
  /* and the size of the new structure */
  /* by passing them as arguments to the base class constructor. */
  MapMsg(): 
    NMLmsg(MAP_MSG_TYPE, sizeof(MapMsg)){};

  /* Each new type needs to overload the update function. */
  void update(CMS *cms); 

  /* number of rows and columns. */
  unsigned int nRows;
  unsigned int nCols;
  
  /* coordinate in the shared frame of the lower left corner of the map*/
  double mapX;
  double mapY;
  
  /* map meter width*/
  double mapRealWidth;
  
  /* the following will create an array of doubles called "da" with a maximum 
     length of 20. Only the number of elements that contain data will be sent 
     over the network. This is controlled by setting the autogenerated variable
     map_length to the size required
  */
  //! NOTE: The map corresponds to x-north y-west! This is not the USARSim standard.
  DECLARE_NML_DYNAMIC_LENGTH_ARRAY(char, map, MAX_MAP_SIZE);
};

/* Declare the NML Format function. */
int nmlMap_format(NMLTYPE type, void *buf, CMS *cms);
//! standard NML command name lookup
extern const char *nmlMap_symbol_lookup(long type);


#endif /* End of NML_EMC_HH */
