#include "swKBTypes.hh"
#include "moastSupSkin.hh"

void
spawnMoastThr (void *tArgs)
{
  SIMwareServoNodeBase *node = VOID_CAST(SIMwareServoNodeBase *, tArgs);
  node->run ();
  ulapi_task_exit();
}

MoastSupSkin::MoastSupSkin()
{
  state = SW_KB_STATE_UNINITIALIZED;
  mob=NULL;
  sp=NULL;
  eff=NULL;
}

MoastSupSkin::~MoastSupSkin() 
{
  for(unsigned int i=0;i<thr.size();i++)
    ulapi_task_delete(thr[i]);
  thr.clear();

}


bool MoastSupSkin::configSupSkin(SIMwareKB * kBase, SIMwareInfSkin * iSkin) 
{
  if(0)
    printf("<MoastSupSkin::configSupSkin>\n");
  kb=kBase;
  infSkin = iSkin;
  return true;
}

bool MoastSupSkin::openSkin() 
{
  INIFILE if_d;
  const char *strPtr;
  
  if (0)
    printf ("<MoastSupSkin::openSkin> : Opening inifile %s\n",
	    SIMWARE_INIFILE_PATH);

  if (if_d.open (SIMWARE_INIFILE_PATH))
    {
      printf
	("** ERROR ** <MoastSupSkin::openSkin> : Invalid inifile path %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }

//   if (NULL == (strPtr = if_d.find ("NML_CFG_FILE", "MOAST.SUPSKIN")))
//     {
//       printf
// 	("** ERROR ** <MoastSupSkin::openSkin> 'NML_CFG_FILE' not defined in section 'MOAST.SUPSKIN' in %s\n",
// 	 SIMWARE_INIFILE_PATH);
//       if_d.close ();
//       return false;
//     }
  // It looks to me as though all other processes use getenv("CONFIG_NML")
  // so do not be different and therefore potentially incompatible.
  strPtr = getenv("CONFIG_NML");
  if(!strPtr)
    {
      fprintf(stderr,"Please set the CONFIG_NML environment variable.\n");
      if_d.close();
      return false;
    }

  nmlCfgFile = strPtr;
  
  if (NULL == (strPtr = if_d.find ("NML_PROC_NAME", "MOAST.SUPSKIN")))
    {
      printf
	("** ERROR ** <MoastSupSkin::openSkin> 'NML_PROC_NAME' not defined in section 'MOAST.SUPSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  nmlProcName = strPtr;

  if_d.close ();
    
  if( 1 )
    {
      printf("<MoastSupSkin::openSkin>\n");
      printf("\tNML Config file    : %s\n",nmlCfgFile.c_str()); 
      printf("\tNML Process Name   : %s \n",  nmlProcName.c_str());
    }
	   
  mob = new SIMwareServoMob(this);
  if( ! mob->openNML(SERVO_MOB_JA_CMD_NAME,SERVO_MOB_JA_STAT_NAME,
		     SERVO_MOB_JA_CFG_NAME,SERVO_MOB_JA_SET_NAME,  
		     servoMobJA_format))
    {
      printf("** ERROR ** <MoastSupSkin::openSkin> : Error Initializing Mobility Object\n");  
      return false;
    }
  
  thr.push_back(ulapi_task_new());
  if ( 0 != ulapi_task_start(thr[thr.size()-1], spawnMoastThr,
			     VOID_CAST(void *, mob), ulapi_prio_highest(), 1))
    {
      printf("** ERROR ** <MoastSupSkin::openSkin> : Error Spawning Mobility thread\n");  
      return false;
    }

  sp = new SIMwareServoSP(this);
  if( ! sp->openNML(SERVO_SP_CMD_NAME,SERVO_SP_STAT_NAME,
		    SERVO_SP_CFG_NAME,SERVO_SP_SET_NAME,  
		    servoSP_format))
    {
      printf("** ERROR ** <MoastSupSkin::openSkin> : Error Initializing Sensor Object\n");  
      return false;
    }
  
  
  thr.push_back(ulapi_task_new());
  if ( 0 != ulapi_task_start(thr[thr.size()-1], spawnMoastThr,
			     VOID_CAST(void *, sp), ulapi_prio_highest(), 1))
    {
      printf("** ERROR ** <MoastSupSkin::openSkin> : Error Spawning Sensor thread\n");  
      return false;
    }

  if( kb->effMap.size() > 0 )
    {
      eff = new SIMwareServoEff(this);
      if( ! eff->openNML(SERVO_EFF_JA_CMD_NAME,SERVO_EFF_JA_STAT_NAME,
			 SERVO_EFF_JA_CFG_NAME,SERVO_EFF_JA_SET_NAME,  
			 servoEffJA_format,
			 /*
			   Since we've changed the effector NML
			   buffers so that we can have more than
			   one per vehicle, we need to use the
			   1_1 suffix conventions. However, we'll
			   still only use one of them in simWare.
			  */
			 1))
	{
	  printf("** ERROR ** <MoastSupSkin::openSkin> : Error Initializing Effecter Channels\n");  
	  return false;
	}
      
      thr.push_back(ulapi_task_new());
      if ( 0 != ulapi_task_start(thr[thr.size()-1], spawnMoastThr,
				 VOID_CAST(void *, eff), ulapi_prio_highest(), 1))
	{
	  printf("** ERROR ** <MoastSupSkin::openSkin> : Error Spawning Effeceter Thread\n");  
	  return false;
	}
    }

  // INITIALIZE MISSION PACKAGES
  // Are the to many mission packages for NML configuration
  if( kb->misMap.size() > SERVO_MIS_JA_MIS_PKG_MAX )
    {
      printf("** ERROR ** <MoastSupSkin::openSkin> : Mission Package Max Exceeded\n");
      return false;
    }
  
  map <const char *, SIMwareKBMisPkg *, strCmp >::iterator itr = kb->misMap.begin();
  while ( itr != kb->misMap.end() ) 
    { 
            
      // Does MisPkg have too many links for NML message
      if( itr->second->getLinkCount() > SERVO_MIS_JA_LINK_MAX)
	{
	  printf("** ERROR ** <MoastSupSkin::openSkin> : Mission Package Link Max Exceeded\n");
	  return false;
	}

      // create new mission package object
      mis.push_back(new SIMwareServoMis(this, itr->second));
      // open NML channels for mission package object
      if( ! mis[mis.size()-1]->openNML(SERVO_MIS_JA_CMD_NAME,SERVO_MIS_JA_STAT_NAME,
				       SERVO_MIS_JA_CFG_NAME,SERVO_MIS_JA_SET_NAME,  
				       servoMisJA_format, 
				       mis.size()))
	{
	  printf("** ERROR ** <MoastSupSkin::openSkin> : Error Initializing MisPkg Object\n");  
	  return false;
	}
     

      thr.push_back(ulapi_task_new());
      if ( 0 != ulapi_task_start(thr[thr.size()-1], spawnMoastThr,
				 VOID_CAST(void *, mis[mis.size()-1]), ulapi_prio_highest(), 1))
 	{
 	  printf("** ERROR ** <MoastSupSkin::openSkin> : Error Spawning MisPkg thread\n");  
 	  return false;
 	}
      itr++;
    }
  
  nml_start();
  state=SW_KB_STATE_INITIALIZED;
  return true;
}

void MoastSupSkin::closeSkin() 
{

  printf("<MoastSupSkin::closeSkin>   : closing MOAST superior skin\n");
  state = SW_KB_STATE_SHUTDOWN;
  
  printf ("<MoastSupSkin::closeSkin>   : waiting for threads to terminate\n");
  for(unsigned int i=0;i<thr.size();i++)
    ulapi_task_join(thr[i]);
  
  printf ("<MoastSupSkin::closeSkin>   : cleaning up NML buffers \n");
  // nml_cleanup();

  if(mob!=NULL)
    delete mob;
  mob=NULL;

  if(eff!=NULL)
    delete eff;
  eff=NULL;
  
  if(sp!=NULL)
    delete sp;
  sp=NULL;

  for(unsigned int i=0;i<mis.size();i++)
    {
      if(mis[i]!=NULL)
	delete mis[i];
      mis[i]=NULL;
    }
  mis.clear();
}
