#include "swKBBot.hh"

SIMwareKBFlipper::SIMwareKBFlipper(SIMwareFlipperType type)
{
  fType  = type;
  width  = 0.00;
  length = 0.00;
  minAng = 0.00;
  maxAng = 0.00;
  loc    = PM_CARTESIAN(0,0,0);
  rpy    = PM_RPY(0,0,0);
  fAng   = 0.00;
}
SIMwareKBFlipper::~SIMwareKBFlipper() {}

SIMwareFlipperType SIMwareKBFlipper::getFlipperType()
{
  return fType;
}

double SIMwareKBFlipper::getWidth()
{
  return width;
}
double SIMwareKBFlipper::getLength()
{
  return length;
}
double SIMwareKBFlipper::getMinAng()
{
  return minAng;
}
double SIMwareKBFlipper::getMaxAng()
{
  return maxAng;
}
PM_CARTESIAN SIMwareKBFlipper::getLocation()
{
  return loc;
}
PM_RPY SIMwareKBFlipper::getOrientation()
{
  return rpy;
}
double SIMwareKBFlipper::getFlipperAngle()
{
return fAng;
}
void SIMwareKBFlipper::setWidth(double w)
{
  width = w;
}
void SIMwareKBFlipper::setLength(double l)
{
  length=l;
}
void SIMwareKBFlipper::setMinAng(double min)
{
  minAng=min;
}
void SIMwareKBFlipper::setMaxAng(double max)
{
  maxAng=max;
}
void SIMwareKBFlipper::setLocation(double x, double y, double z)
{
  loc.x=x;
  loc.y=y;
  loc.z=z;

}
void SIMwareKBFlipper::setLocation(PM_CARTESIAN location)
{
  loc = location;
}
void SIMwareKBFlipper::setOrientation(double r, double p, double y)
{
  rpy.r=r;
  rpy.p=p;
  rpy.y=y;
}
void SIMwareKBFlipper::setOrientation(PM_RPY orient)
{ 
  rpy = orient;
}void SIMwareKBFlipper::setFlipperAngle(double fAngle)
{
  fAng = fAngle;
}

///////** SIMWARE GENERAL ROBOT **///////
SIMwareKBBot::SIMwareKBBot ()
{
  bID = 1;
  sType = SW_KB_UNKNOWN_STEER_TYPE;
  rType = SW_KB_UNKNOWN_ROBOT_TYPE;
  time=0;
}

SIMwareKBBot::~SIMwareKBBot () {}

void
SIMwareKBBot::setPlatformName (string n)
{
  pName = n;
}
string
SIMwareKBBot::getPlatformName ()
{
  return pName;
}
void
SIMwareKBBot::setName (string n)
{
  bName = n;
}
string
SIMwareKBBot::getName ()
{
  return bName;
}
void
SIMwareKBBot::setBotID (int i)
{
  bID = i;
}
int
SIMwareKBBot::getBotID ()
{
  return bID;
}

bool
SIMwareKBBot::setBotType(SIMwareRobotType rt)
{
  bool retval;

  switch (rt) {
  case SW_KB_GROUND_VEHICLE_TYPE:
  case SW_KB_LEGGED_ROBOT_TYPE:
  case SW_KB_NAUTIC_ROBOT_TYPE:
  case SW_KB_AERIAL_ROBOT_TYPE:
    rType = rt;
    retval = true;
    break;
  default:
    retval = false;
    break;
  }

  return retval;
}

SIMwareRobotType
SIMwareKBBot::getBotType ()
{
  return rType;
}

void
SIMwareKBBot::setCentroid (double x, double y, double z)
{
  cg.x = x;
  cg.y = y;
  cg.z = z;
}
void
SIMwareKBBot::setCentroid (PM_CARTESIAN c)
{
  cg = c;
}
PM_CARTESIAN
SIMwareKBBot::getCentroid ()
{
  return cg;
}
void
SIMwareKBBot::setDimensions (PM_CARTESIAN xyz)
{
  dim = xyz;
}
void
SIMwareKBBot::setDimensions (double length, double width, double height)
{
  dim.x = length;
  dim.y = width;
  dim.z = height;
}
PM_CARTESIAN
SIMwareKBBot::getDimensions ()
{
  return dim;
}
void
SIMwareKBBot::setMass (double m)
{
  mass = m;
}
double
SIMwareKBBot::getMass ()
{
  return mass;
}
void
SIMwareKBBot::setLightsOn (bool isOn)
{
  lightsOn = isOn;
}
bool
SIMwareKBBot::getLightsOn ()
{
  return lightsOn;
}
void
SIMwareKBBot::setBatteryLevel (double bl)
{
  bLvl = bl;
}
double
SIMwareKBBot::getBatteryLevel ()
{
  return bLvl;
}
void
SIMwareKBBot::setFuelLevel (double fl)
{
  fLevel = fl;
}
double
SIMwareKBBot::getFuelLevel ()
{
  return fLevel;
}
void
SIMwareKBBot::setSteerType (SIMwareSteerType s)
{
  sType = s;
}
SIMwareSteerType
SIMwareKBBot::getSteerType ()
{
  return sType;
}
void
SIMwareKBBot::setTimeStamp (double time)
{
  this->time = time;
}
double
SIMwareKBBot::getTimeStamp ()
{
  return time;
}
void
SIMwareKBBot::setLightLevel (double lLevel)
{
  lLvl = lLevel;
}
double
SIMwareKBBot::getLightLevel ()
{
  return lLvl;
}


///////** SIMWARE GROUND VEHICLE **///////
SIMwareKBGroundVehicle::SIMwareKBGroundVehicle ()
{
  rType = SW_KB_GROUND_VEHICLE_TYPE;
  maxWheelRot = 0.00;
  maxSteerAng = 0.00;
  maxCrabAng = 0.00;
  wRadius = 0.00;
  wSep = 0.00;
  wBase = 0.00;
  maxTorque = 0.00;
  steerAng = 0.00;
  crabAng = 0.00;
  for(int i=0;i<SIMWARE_FLIPPER_MAX;i++)
    flippers[i]=NULL;
}

SIMwareKBGroundVehicle::~SIMwareKBGroundVehicle ()
{
  for(int i=0;i<SIMWARE_FLIPPER_MAX;i++)
    {
      if(flippers[i]!=NULL)
	delete flippers[i];
      flippers[i]=NULL;
    }
}

void
SIMwareKBGroundVehicle::setMaxWheelRot (double s)
{
  maxWheelRot = s;
}

double
SIMwareKBGroundVehicle::getMaxWheelRot ()
{
  return maxWheelRot;
}

void
SIMwareKBGroundVehicle::setMaxTorque (double t)
{
  maxTorque = t;
}

double
SIMwareKBGroundVehicle::getMaxTorque ()
{
  return maxTorque;
}

void
SIMwareKBGroundVehicle::setMaxSteerAngle (double fsa)
{
  maxSteerAng = fsa;
}

double
SIMwareKBGroundVehicle::getMaxSteerAngle ()
{
  return maxSteerAng;
}

void
SIMwareKBGroundVehicle::setMaxCrabAngle (double rsa)
{
  maxCrabAng = rsa;
}

double
SIMwareKBGroundVehicle::getMaxCrabAngle ()
{
  return maxCrabAng;
}

void
SIMwareKBGroundVehicle::setWheelSeparation (double ws)
{
  wSep = ws;
}

double
SIMwareKBGroundVehicle::getWheelSeparation ()
{
  return wSep;
}

void
SIMwareKBGroundVehicle::setWheelBase (double wb)
{
  wBase = wb;
}

double
SIMwareKBGroundVehicle::getWheelBase ()
{
  return wBase;
}

void
SIMwareKBGroundVehicle::setWheelRadius (double wr)
{
  wRadius = wr;
}

double
SIMwareKBGroundVehicle::getWheelRadius ()
{
  return wRadius;
}


double
SIMwareKBGroundVehicle::getMinTurnRadius ()
{
  if( sType ==  SW_KB_ACKERMAN_STEER_TYPE)
    {
      if( maxSteerAng == 0.00)
	{
	  printf("Max Crab %lf %lf\n", maxCrabAng, maxSteerAng);
	  return wBase / tan(maxCrabAng);
	}
      else
	{
	  printf("Max Steer %lf %lf\n", maxCrabAng, maxSteerAng);
	  return wBase / tan(maxSteerAng);
	}
    }
  else 
    return 0;
}

void SIMwareKBGroundVehicle::setSteerAngle(double ang)
{
  steerAng = ang;
}
void SIMwareKBGroundVehicle::setCrabAngle(double ang)
{
  crabAng= ang;
}
double SIMwareKBGroundVehicle::getSteerAngle()
{
  return steerAng;
}
double SIMwareKBGroundVehicle::getCrabAngle()
{
  return crabAng;
}
SIMwareKBFlipper * SIMwareKBGroundVehicle::getFlipper(SIMwareFlipperType type)
{
  return flippers[type];
}

bool SIMwareKBGroundVehicle::addFlipper(SIMwareKBFlipper * flipper)
{
  if(flippers[flipper->getFlipperType()]!=NULL)
    return false;
  flippers[flipper->getFlipperType()] = flipper;
  return true;
}

///////** SIMWARE NAUTIC ROBOT **///////
SIMwareKBNauticRobot::SIMwareKBNauticRobot ()
{
  rType = SW_KB_NAUTIC_ROBOT_TYPE;
  sternPlaneAngle = 0.00;;
  rudderAngle = 0.00;

}

double SIMwareKBNauticRobot::getSternPlaneAngle()
{
  return sternPlaneAngle;
}
double SIMwareKBNauticRobot::getRudderAngle()
{
  return rudderAngle;
}
void SIMwareKBNauticRobot::setSternPlaneAngle(double sAng)
{
  sternPlaneAngle = sAng;
}
void SIMwareKBNauticRobot::setRudderAngle(double rAng)
{
  rudderAngle = rAng;
}
///////** SIMWARE NAUTIC ROBOT **///////
SIMwareKBAerialRobot::SIMwareKBAerialRobot ()
{
  rType = SW_KB_AERIAL_ROBOT_TYPE;
  maxRotVel = 0.00;
  maxLatVel = 0.00;
  maxLinVel = 0.00;
  maxVertVel = 0.00;
  vRot = 0.00;
  vLat = 0.00;
  vLin = 0.00;
  vVert = 0.00;
}

double SIMwareKBAerialRobot::getMaxRotVel()
{
  return maxRotVel;
}
double SIMwareKBAerialRobot::getMaxLatVel()
{
return maxLatVel;
}
double SIMwareKBAerialRobot::getMaxLinVel()
{
return maxLinVel;
}
double SIMwareKBAerialRobot::getMaxVertVel()
{
return maxVertVel;
}

void SIMwareKBAerialRobot::setMaxRotVel(double mrv)
{
  maxRotVel = mrv;
}
void SIMwareKBAerialRobot::setMaxLatVel(double mlv)
{
  maxLatVel = mlv;
}
void SIMwareKBAerialRobot::setMaxLinVel(double mlv)
{
  maxLinVel = mlv;
}
void SIMwareKBAerialRobot::setMaxVertVel(double mvv)
{
  maxVertVel = mvv;
}

double SIMwareKBAerialRobot::getRotVel()
{
  return vRot;
}
double SIMwareKBAerialRobot::getLatVel()
{
  return vLat;
}
double SIMwareKBAerialRobot::getLinVel()
{
  return vLin;
}
double SIMwareKBAerialRobot::getVertVel()
{
  return vVert;
}

void SIMwareKBAerialRobot::setRotVel(double rv)
{
  vRot = rv;
}
void SIMwareKBAerialRobot::setLatVel(double lv)
{
  vLat = lv;
}
void SIMwareKBAerialRobot::setLinVel(double lv)
{  
  vLin = lv;
}
void SIMwareKBAerialRobot::setVertVel(double vv)
{
  vVert = vv;
}
