#include "swKBTypes.hh"
#include "swServoEff.hh"
#include "moastSupSkin.hh"

SIMwareServoEff::SIMwareServoEff(MoastSupSkin * m) : SIMwareServoNodeBase(m)
{
  nodeName="ServoEff";
  statMsg=new ServoEffJAStat;
  setMsg=new ServoEffJASet;
  // initialize stat msg
  CLASS_CAST(ServoEffJAStat *, statMsg)->heartbeat=0;
  CLASS_CAST(ServoEffJAStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;

  CLASS_CAST(ServoEffJASet *, setMsg)->cycleTime=SERVO_NODE_CYCLE_TIME_DEFAULT;
  CLASS_CAST(ServoEffJASet *, setMsg)->debug=0;


  map<const char *, SIMwareKBEff*, strCmp>::iterator effItr;
  for( effItr = mSkin->kb->effMap.begin(); effItr !=  mSkin->kb->effMap.end(); effItr++) 
    effList.push_back(effItr->second);
  
}

SIMwareServoEff::~SIMwareServoEff() 
{
  if(0)
    printf("<SIMwareServoEff::~SIMwareServoEff>\n");
}




void SIMwareServoEff::doSingleStep() 
{
  if(0)
    printf("<SIMwareServoEff::doSingleStep>\n");
  
   int serialNumber;
   int type = 0;
  
   // Read Configuration Message
   type = cfgBuf->read();
   serialNumber = cfgBuf->get_address()->serial_number;
   switch (type) 
     {
       
     case 0:
       // no new cmd
       break;
       
     case -1:
       // comm error
       rcs_print_error("<SIMwareServoEffJA::doSingleStep> : NML cmd read error\n");
       break;
       
     case SERVO_EFF_JA_CFG_CYCLE_TIME_TYPE:
     case SERVO_EFF_JA_CFG_DEBUG_TYPE:
       
       if (serialNumber != CLASS_CAST(ServoEffJASet *, setMsg)->echo_serial_number) 
	 {
	   CLASS_CAST(ServoEffJASet *, setMsg)->command_type = type;
	   CLASS_CAST(ServoEffJASet *, setMsg)->echo_serial_number = serialNumber;
	   CLASS_CAST(ServoEffJASet *, setMsg)->state = NEW_COMMAND;
	   CLASS_CAST(ServoEffJASet *, setMsg)->status = RCS_EXEC;
	 }    
       break;
       
     default:
       rcs_print_error("<SIMwareServoEff::doSingleStep> : EffJA Cfg msg type %d not recoginzed\n", (int) type);
       break;
     }	
  
   //Read Command Message
   type = cmdBuf->read();
   serialNumber = cmdBuf->get_address()->serial_number;
   switch (type) 
     {
      
     case 0:
       // no new cmd
       break;
       
     case -1:
       // comm error
       rcs_print_error("<SIMwareServoEff::doSingleStep> : NML cmd read error\n");
       break;
     case SERVO_EFF_JA_CMD_INIT_TYPE:
     case SERVO_EFF_JA_CMD_HALT_TYPE:
     case SERVO_EFF_JA_CMD_ABORT_TYPE:
     case SERVO_EFF_JA_CMD_SHUTDOWN_TYPE:
     case SERVO_EFF_JA_CMD_OPCODE_TYPE:
       if (serialNumber != CLASS_CAST(ServoEffJAStat *, statMsg)->echo_serial_number) 
	 {
	   CLASS_CAST(ServoEffJAStat *, statMsg)->command_type = type;
	   CLASS_CAST(ServoEffJAStat *, statMsg)->echo_serial_number = serialNumber;
	   CLASS_CAST(ServoEffJAStat *, statMsg)->state = NEW_COMMAND;
	   CLASS_CAST(ServoEffJAStat *, statMsg)->status = RCS_EXEC;
	   CLASS_CAST(ServoEffJAStat *, statMsg)->error = 0;
	 }    
       break;
     default:
       rcs_print_error("<SIMwareServoEff::doSingleStep> : EFF JA Cmd msg type %d not recoginzed\n", (int) type);
       break;
     }				// switch (command_type)
   
   
   
   // Handle Configuration Messages 
   switch( CLASS_CAST(ServoEffJASet *, setMsg)->command_type ) 
     {
       
     case SERVO_EFF_JA_CFG_CYCLE_TIME_TYPE:
       doEffCfgCycleTime(CLASS_CAST(ServoEffJACfgCycleTime *, cfgBuf->get_address()));
       break;
       
     case SERVO_EFF_JA_CFG_DEBUG_TYPE:
       doEffCfgDebug(CLASS_CAST(ServoEffJACfgDebug *, cfgBuf->get_address()));
       break;
     default:   
       break;
     }
   
   // Handle Command Message 
   switch (CLASS_CAST(ServoEffJAStat *, statMsg)->command_type) 
    {
      
    case SERVO_EFF_JA_CMD_INIT_TYPE:
      doEffCmdInit();
      break;
      
    case SERVO_EFF_JA_CMD_HALT_TYPE:
      doEffCmdHalt();
      break;
      
    case SERVO_EFF_JA_CMD_ABORT_TYPE:
      doEffCmdAbort();
      break;
      
    case SERVO_EFF_JA_CMD_SHUTDOWN_TYPE:
      doEffCmdShutdown();
      break;
      
    case SERVO_EFF_JA_CMD_OPCODE_TYPE:
      doEffCmdOpcode(CLASS_CAST(ServoEffJACmdOpcode *, cmdBuf->get_address()));
      break;
      
    default:
      // no cmd to handle
      break;
    }	
  sendStatus();
}

void SIMwareServoEff::doEffCmdInit() 
{
  if (state_match(CLASS_CAST(ServoEffJAStat *, statMsg), NEW_COMMAND)) 
    { 
      CLASS_CAST(ServoEffJAStat *, statMsg)->admin_state=ADMIN_INITIALIZED;
      state_new(CLASS_CAST(ServoEffJAStat *, statMsg));
      mSkin->kb->addCmdToQ(new SIMwareEffCmd(SW_EFF_CMD_INIT_TYPE));
      status_next(CLASS_CAST(ServoEffJAStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoEffJAStat *, statMsg), NOP_STATE);      
    }
  else
    state_default(CLASS_CAST(ServoEffJAStat *, statMsg));  
}

void SIMwareServoEff::doEffCmdHalt() 
{
  if (state_match(CLASS_CAST(ServoEffJAStat *, statMsg), NEW_COMMAND)) 
    {
      CLASS_CAST(ServoEffJAStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;
      mSkin->kb->addCmdToQ(new SIMwareEffCmd(SW_EFF_CMD_POWERDOWN_TYPE));
      state_new(CLASS_CAST(ServoEffJAStat *, statMsg));
      status_next(CLASS_CAST(ServoEffJAStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoEffJAStat *, statMsg), NOP_STATE);
    }
  else
    state_default(CLASS_CAST(ServoEffJAStat *, statMsg));  
}

void SIMwareServoEff::doEffCmdAbort() 
{
  if (state_match(CLASS_CAST(ServoEffJAStat *, statMsg), NEW_COMMAND)) 
    {
      CLASS_CAST(ServoEffJAStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;
      mSkin->kb->addCmdToQ(new SIMwareEffCmd(SW_EFF_CMD_POWERDOWN_TYPE));
      state_new(CLASS_CAST(ServoEffJAStat *, statMsg));
      status_next(CLASS_CAST(ServoEffJAStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoEffJAStat *, statMsg), NOP_STATE);
    }
  else
    state_default(CLASS_CAST(ServoEffJAStat *, statMsg));  
}

void SIMwareServoEff::doEffCmdShutdown() 
{
  if (state_match(CLASS_CAST(ServoEffJAStat *, statMsg), NEW_COMMAND)) 
    {
      CLASS_CAST(ServoEffJAStat *, statMsg)->admin_state=ADMIN_UNINITIALIZED;
      mSkin->kb->addCmdToQ(new SIMwareEffCmd(SW_EFF_CMD_POWERDOWN_TYPE));
      state_new(CLASS_CAST(ServoEffJAStat *, statMsg));
      status_next(CLASS_CAST(ServoEffJAStat *, statMsg), RCS_DONE);  
      state_next(CLASS_CAST(ServoEffJAStat *, statMsg), NOP_STATE);
    }
  else
    state_default(CLASS_CAST(ServoEffJAStat *, statMsg));  
}


void SIMwareServoEff::doEffCmdOpcode(ServoEffJACmdOpcode * msg) 
{

 

  SIMwareEffOpcodeConfig * swOpConf;
  SIMwareEffOpcodeType type;
  if (state_match(CLASS_CAST(ServoEffJAStat *, statMsg), NEW_COMMAND)) 
    {
      status_next(statMsg, RCS_EXEC);
      state_new(statMsg);
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      else 
	state_next(statMsg, S0);
    }
  
  if (state_match(statMsg, S0))
    {
      if( msg->effID > (int)effList.size() )
	state_next(statMsg, SE1);
      else 	
	state_next(statMsg, S1)	;
    }
  
  if (state_match(statMsg, S1))
    {
      
      switch(msg->opcode)
	{
	case SERVO_EFF_JA_ACTIVATE_OPCODE_TYPE: 
	  type = SW_EFF_OPCODE_ACTIVATE_TYPE;
	  break;
	case SERVO_EFF_JA_ANIMATE_OPCODE_TYPE:
	  type = SW_EFF_OPCODE_ANIMATE_TYPE;
	  break;
	case SERVO_EFF_JA_FIRE_OPCODE_TYPE:   
	  type = SW_EFF_OPCODE_FIRE_TYPE;
	  break;
	case SERVO_EFF_JA_RELEASE_OPCODE_TYPE:
	  type = SW_EFF_OPCODE_RELEASE_TYPE; 
	  break;
	case SERVO_EFF_JA_RESET_OPCODE_TYPE: 
	  type = SW_EFF_OPCODE_RESET_TYPE;  
	  break;
	case SERVO_EFF_JA_NOP_OPCODE_TYPE: 
	  type = SW_EFF_OPCODE_NOP_TYPE;  
	  break;
	default:
	  type = SW_EFF_OPCODE_INVALID_TYPE;
	  break;
	}
      swOpConf = effList[msg->effID-1]->getOpcodeConfig(type);
      
      if( type == SW_EFF_OPCODE_INVALID_TYPE)
	state_next(statMsg, SE2);
      if(swOpConf == NULL )
	state_next(statMsg, SE2);
      else 	
	state_next(statMsg, S2);
    }
  
  if (state_match(statMsg, S2))
    {
      if ( msg->param > swOpConf->getMaxValue() || msg->param < swOpConf->getMinValue())
	state_next(statMsg, SE3);
      else 
	state_next(statMsg, S3);
    }
  
  
  if (state_match(statMsg, S3))
    {
      mSkin->kb->addCmdToQ(new SIMwareEffOpcodeCmd(effList[msg->effID-1]->getName(),type,msg->param)); 
      status_next(statMsg, RCS_DONE);
      state_next(statMsg, NOP_STATE);
    }
  
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else if (state_match(statMsg, SE2))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE2);
    } 
  else if (state_match(statMsg, SE3))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE3);
    }
  else
    state_default(statMsg);  
  
}

void SIMwareServoEff::doEffCfgCycleTime( ServoEffJACfgCycleTime * cTCmd ) 
{
  if (state_match(CLASS_CAST(ServoEffJASet *, setMsg), NEW_COMMAND)) 
    {
      state_new(CLASS_CAST(ServoEffJASet *, setMsg));
      CLASS_CAST(ServoEffJASet *, setMsg)->cycleTime = cTCmd->cycleTime;
      setTimer(CLASS_CAST(ServoEffJASet *, setMsg)->cycleTime);
      status_next(CLASS_CAST(ServoEffJASet *, setMsg), RCS_DONE);
      state_next(CLASS_CAST(ServoEffJASet *, setMsg), NOP_STATE);    
      setBuf->write(CLASS_CAST(ServoEffJASet *, setMsg));	
    }
  else
    state_default(CLASS_CAST(ServoEffJASet *, setMsg));  
}

void SIMwareServoEff::doEffCfgDebug( ServoEffJACfgDebug * cTCmd ) 
{
  if (state_match(CLASS_CAST(ServoEffJASet *, setMsg), NEW_COMMAND)) 
    {
      state_new(CLASS_CAST(ServoEffJASet *, setMsg));
      CLASS_CAST(ServoEffJASet *, setMsg)->debug = cTCmd->debug;
      status_next(CLASS_CAST(ServoEffJASet *, setMsg), RCS_DONE);
      state_next(CLASS_CAST(ServoEffJASet *, setMsg), NOP_STATE);
      setBuf->write(CLASS_CAST(ServoEffJASet *, setMsg));	 
    }
  else
    state_default(CLASS_CAST(ServoEffJASet *, setMsg)); 
}


void SIMwareServoEff::sendStatus()
{
  CLASS_CAST(ServoEffJAStat *, statMsg)->heartbeat++;
  statBuf->write(CLASS_CAST(ServoEffJAStat *, statMsg));
}

void SIMwareServoEff::publishSettings()
{
  ServoEffJASet * setMsgPtr = CLASS_CAST(ServoEffJASet *, setMsg);
  map <SIMwareEffOpcodeType,SIMwareEffOpcodeConfig *>::iterator itr;
  SIMwareKBMount * mount;
  SIMwareEffOpcodeConfig * swOpConf;
  int opCount;
  
  setMsgPtr->vehicleID= mSkin->kb->bot->getBotID();

  for(int i = 0; i < (int)effList.size(); i++)
    {
      
      setMsgPtr->effector[setMsgPtr->effector_length].effID=i+1;//effList[i]->getID();
      if(NULL == (mount=effList[i]->getMount()))
 	{
 	  setMsgPtr->effector[setMsgPtr->effector_length].mountID=-1;
 	  setMsgPtr->effector[setMsgPtr->effector_length].linkID=0;
 	}
      else
 	{
 	  setMsgPtr->effector[setMsgPtr->effector_length].mountID=mount->misPkg->getMisPkgID();
 	  setMsgPtr->effector[setMsgPtr->effector_length].linkID=mount->link->getLinkID();
 	}
     
       setMsgPtr->effector[setMsgPtr->effector_length].effToMount=effList[i]->getEff2Mount();
       
       opCount = 0;
       setMsgPtr->effector[setMsgPtr->effector_length].opConf_length=(int)effList[i]->opConfMap.size();
       for( itr = effList[i]->opConfMap.begin(); itr !=  effList[i]->opConfMap.end(); itr++)
	 {
	   swOpConf = itr->second;
	   switch(swOpConf->getOpcodeType())
	     {
	     case SW_EFF_OPCODE_ACTIVATE_TYPE: 
	       setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].opcode= SERVO_EFF_JA_ACTIVATE_OPCODE_TYPE;
	       break;
	     case SW_EFF_OPCODE_ANIMATE_TYPE:
	       setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].opcode= SERVO_EFF_JA_ANIMATE_OPCODE_TYPE;
	       break;
	     case SW_EFF_OPCODE_FIRE_TYPE:   
	       setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].opcode= SERVO_EFF_JA_FIRE_OPCODE_TYPE;
	       break;
	     case SW_EFF_OPCODE_RELEASE_TYPE: 
	       setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].opcode= SERVO_EFF_JA_RELEASE_OPCODE_TYPE;
	       break;
	     case SW_EFF_OPCODE_RESET_TYPE:   
	       setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].opcode= SERVO_EFF_JA_RESET_OPCODE_TYPE;
	       break;
	     case SW_EFF_OPCODE_NOP_TYPE:   
	       setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].opcode= SERVO_EFF_JA_NOP_OPCODE_TYPE;
	       break;
	     default:
	       continue;
	       break;
	     }
	   setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].max=swOpConf->getMaxValue();
	   setMsgPtr->effector[setMsgPtr->effector_length].opConf[opCount].min=swOpConf->getMinValue();
	   opCount++;
	 }
       setMsgPtr->effector_length++;
       setMsgPtr->effector[setMsgPtr->effector_length].opConf_length=opCount;
    }
  setBuf->write(setMsgPtr);
}



void SIMwareServoEff::errorHandler(int error) 
{
  switch(error)
    {
    case SE0:
      CLASS_CAST(ServoEffJAStat *, statMsg)->error=SE0;
      snprintf(statMsg->message,80,"Robot is not initialized%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;

    case SE1:
      CLASS_CAST(ServoEffJAStat *, statMsg)->error=SE1;
      snprintf(statMsg->message,80,"Invalid effector ID%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;
      
    case SE2:
      CLASS_CAST(ServoEffJAStat *, statMsg)->error=SE2;
      snprintf(statMsg->message,80,"Effector does not implement this Opcode%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;  
    case SE3:
      CLASS_CAST(ServoEffJAStat *, statMsg)->error=SE3;
      snprintf(statMsg->message,80,"Parameter value out of bounds%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;
    default:
      break;
    }

}
