#ifndef SW_SERVO_SP_HH
#define SW_SERVO_SP_HH

#include <signal.h>

#include <vector>
#include <string>
#include <map>
#include <sstream>
using namespace std;

//! SIMware Headers
#include "swServoNodeBase.hh"
#include "simWareKB.hh"

// MOAST Headers
#include "servoSP.hh"
#include "sensorData.hh"

class SIMwareServoSP : public SIMwareServoNodeBase {
protected:

  vector<SIMwareKBSenLinescan *> linescan;
  vector<SensorData1D *> linescanData;
  vector<NML *> linescanBuf;

  vector<SIMwareKBSenSonar *> sonar;
  NML * sonarBuf;
  SensorDataArray sonarData;

  vector<SIMwareKBSenRFID *> rfid;
  NML * rfidBuf;
  SensorDataTextArray rfidData;

  vector<SIMwareKBSenVictim *> victim;
  NML * victimBuf;
  SensorDataArray victimData;
  
  vector<SIMwareKBSen *> navSen;
  NML * navBuf;
  SensorDataArray navData;

  vector<SIMwareKBSen *> xSen;
  
  void doSPCmdInit();
  void doSPCmdHalt();
  void doSPCmdAbort();
  void doSPCmdShutdown();
  void doSPCmdGo();
  void doSPCmdReset();
  
  void doSPCfgCycleTime(ServoSPCfgCycleTime *);
  void doSPCfgDebug(ServoSPCfgDebug *);
  					      
  void sendSensorData();
  void sendNavData();
  void sendLinescanData();
  void sendSonarData();
  void sendVictimData();
  void sendRfidData();
  
  void publishSettings();
  void sendStatus();
public:
  SIMwareServoSP(MoastSupSkin *);
  ~SIMwareServoSP();
  bool openNML(string,        // cmd channel name
	       string,        // stat channel name
	       string,        // cfg channel name
	       string,        // set channel name
	       NML_FORMAT_PTR, // format function pointer
	       int = 0           // component id
	       );
  void cleanUpNMLBuf();
  void doSingleStep();


};
#endif
