#ifndef SW_TRAFFIC_HH
#define SW_TRAFFIC_HH

//! C++ STL Headers
#include<string>
#include<sstream>
using namespace std;

//! SIMware Headers
#include "swKBTypes.hh"
#include "simWareKB.hh"
#include "swKBUtils.hh"
#include "trafficCtrl.hh"

//! Prim Headers
#include "primMobEngine.hh"

class TrafficSupSkin;

//! RCS Headers
#include "rcs.hh" 
#include "trafficData.hh"

#define TRAFFIC_CYCLE_TIME_DEFAULT 0.1

class SIMwareTraffic {
protected:
  string nodeName;

  RCS_CMD_CHANNEL  * cmdBuf;
  RCS_STAT_CHANNEL * statBuf;
  RCS_CMD_CHANNEL  * cfgBuf;
  RCS_STAT_CHANNEL * setBuf;
  NML              * dataBuf;

  RCS_STAT_MSG_V2 * statMsg;
  RCS_STAT_MSG * setMsg;
  
  RCS_TIMER * timer;
  
  TrafficSupSkin * mSkin;
  TrafficData *trafficData;

  PrimMobEngine *primEngine;

  void doTrafficCtrlCmdNop();
  void doTrafficCtrlCmdInit();
  void doTrafficCtrlCmdAbort();
  void doTrafficCtrlCmdHalt();
  void doTrafficCtrlCmdShutdown();
  
  void doMobCmdDrvAckerman(TrafficCtrlCmdAckerman *);
  void doMobCmdDrvSubmarine(TrafficCtrlCmdSubmarine *);
  void doMobCmdDrvSkid(TrafficCtrlCmdSkid *);
  void doMobCmdDrvFlipSkid(TrafficCtrlCmdFlipSkid *);
  void doMobCmdDrvOmni(TrafficCtrlCmdOmni *);
  void doMobCmdDrvRotaryWing(TrafficCtrlCmdRotaryWing *);


  void doTrafficCtrlCmdStop();
  void doTrafficCtrlCmdMoveWaypoint(); //TrafficCtrlCmdMoveWaypoint *);

  void doTrafficCtrlCfgNop();
  void doTrafficCtrlCfgCycleTime(TrafficCtrlCfgCycleTime *);
  void doTrafficCtrlCfgDebug(TrafficCtrlCfgDebug *);
  void doTrafficCtrlCfgMove(TrafficCtrlCfgMove *);
  
  void errorHandler(int);


public:
  SIMwareTraffic(TrafficSupSkin *, double, double, double, double, double, double, PM_POSE);
  ~SIMwareTraffic();
  bool openNML(string,        // cmd channel name
	       string,        // stat channel name
	       string,        // cfg channel name
	       string,        // set channel name
	       NML_FORMAT_PTR, // format function pointer
	       int = 0           // component id
	       );
  void cleanUpNMLBuf();
  void doSingleStep();
  void publishSettings();
  void sendStatus();
  void sendData();
  void run();
  SIMwareStateType getNodeState();
  void setTimer(double);
};
#endif
