/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   vehMobPlWm.hh
  \brief  Include file for the vehicle mobility world model classes


  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.2 $
  $Date: 2007/11/08 13:27:51 $
  \endcode

  \author Stephen Balakirsky
  \date   05/25/2005
*/
#ifndef VEH_MOB_PL_WM_HH
#define VEH_MOB_PL_WM_HH

#include "moastTypes.hh"
#include "moastGeom.hh"

////////////////////////////////
class VehMobWmNode
{
public:
  // functions
  VehMobWmNode(int numVerts, Location2D *verticesIn, int idIn);
  ~VehMobWmNode();
  int getNumVertices(){
    return(numVertices);
  };
  VehMobWmNode *getNext(){
    return(next);
  };
  void setNext(VehMobWmNode *nextIn){
    next = nextIn;
  };
  int getId(){
    return(id);
  };
  LineSegment getLine(int id);

private:
  // functions

  // variables
  int           numVertices;
  Location2D   *vertices;
  VehMobWmNode *next;
  int           id;
};

////////////////////////////////
class VehMobPlWm
{
public:
  //functions
  VehMobPlWm(bool myVerboseIn=true);
  ~VehMobPlWm();
  VehMobWmNode *getTail(){
    return(tail);
  };
  bool add(int numVerts, Location2D *verticesIn, int idIn);
  void dump();
  //variables
  
private:
  //functions

  // variables
  VehMobWmNode *wmNodes;
  VehMobWmNode *tail;
  
  //! should verbose output be turned on?
  bool myVerbose;
};

#endif // VEH_MOB_PL_WM_HH

/*
  Modification history:

  $Log: vehMobPlWm.hh,v $
  Revision 1.2  2007/11/08 13:27:51  dr_steveb
  Changed location of LineSegment

  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.3  2005/07/27 12:34:59  stephen
  Fixes to get code to compile.

  Revision 1.2  2005/07/26 20:30:37  stephen
  Debug in progress.

  Revision 1.1  2005/07/26 13:25:12  stephen
  Added visibility graph planner (not yet working)

*/
