#ifndef COMMUNICATIONS_BASE_H
#define COMMUNICATIONS_BASE_H

#define ROBOT_COUNT 5   // Includes CommStation
#define IMGSVR_PORTNUM 5003 // Define of the portnum of ImgSvr (need to add to CFG).
#define WSS_CUTOFF -93

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string>
#include <strings.h>
#include <cstring>
#include <iostream>
#include <cstdlib>
#include <queue>
#include <pthread.h>
#include <errno.h>
#include <signal.h>
#include <map>

using namespace std;

class CommMsg
{
  public:
  char type[4];
  int numParams;
  float* params;
  
  CommMsg()
  {
    numParams = 0;
    params = NULL;
  }
  
  ~CommMsg()
  {
    if(params != NULL)
      delete [] params;
    params = NULL;
  }
  
  CommMsg(int nParams, char* Type)
  {
    numParams = nParams;
    if(numParams>0)
      params = new float[numParams];
    else
      params = NULL;
    setType(Type);
  }
  
  void setType(char* newType)
  {
    memcpy(type, newType, 4);
    return;
  }
};

class ImgSvr
{
  public:
    int imgType, imgSize;
    unsigned char* buffer;

    ImgSvr();
    ~ImgSvr();
    void ConnectImgSvr(string IP, int Port);
    void Disconnect();
    void ReadImgSvrImage(); 
    void SendOKImgSvr();
    void ClearBuffer();

  private:
    int sock_fd;
};

class Communications
{
  public:
  static const int ROBOTCOUNT = ROBOT_COUNT;
  unsigned char robotType;
  float robSS [ROBOT_COUNT];

  int batteryLife;

  map<string, int> lastSeenMsgID[ROBOTCOUNT];
  
  typedef struct commArgs{
          int sockfd;
          int robID;
          Communications* obj;
  }CommArgs;

  typedef struct message{
          int msgLen;
          unsigned char* msg;
  } Message;

  // Constructors
  Communications();
  Communications(int robID, char* wssIP);
  Communications(int robID, char* wssIP, char robType);
  void initCommunications(int robID, char* wssIP);

  // WSS Communications
  bool SendInit(string wssIP, int port);
  float GetSignalStrength(string RobotName);
  string INITResponse();
  float SignalStrengthResponse(string msg);
  int DNSResponse(string msg);
  int ReverseDNSResponse(string msg);
  int getRobotDNS(string RobotName);
  int getRobotReverseDNS(int);
  int updateRobSS();
  char* WSSIP;
  ImgSvr* imgsvr;

  // Thread starting points
  static void* startManageIncomingMessages(void *args){
   (((CommArgs*)(args))->obj)->manageIncomingMessages(args);
   return NULL;
  }
  static void* startManageOutgoingMessages(void *args){
   (((CommArgs*)(args))->obj)->manageOutgoingMessages(args);
    return NULL;
  }
  static void* startListenForConnections(void *args){
   (((CommArgs*)(args))->obj)->listenForConnections(args);
    return NULL;
  }
  static void* startRSSThread(void* args){
    (((CommArgs*)(args))->obj)->RSSReportThread(args);
    return NULL;
  }
  void* manageIncomingMessages(void* arg);
  void* manageOutgoingMessages(void* arg);
  void* listenForConnections(void* arg);
  
  // Functions for the actual sending/recieving of messages.
  bool readIncomingMessage(int robID, CommMsg** msg);
  //void sendMessage(int robID,string msg);

  void sendMessage(int robID, unsigned char* msg, int msgLen);

  //attempt to close all the sockets - this could use some testing to see side effects
  void closeAllSockets();
  
  // Robot Signal Strength Report Thread
  void* RSSReportThread(void* arg);

  // Communications Queues and socket descriptor for listen port.
  int socket_descriptor; /// WSS Socket Descriptor
  int listen_sockfd;
  int robotcomm_sockfd[ROBOT_COUNT];
  int myID;
  queue<Message> msgsOut[ROBOT_COUNT];
  queue<CommMsg*> msgsIn[ROBOT_COUNT];
  
  // THREAD: Condition Declarations
  pthread_cond_t msgOutCond[ROBOT_COUNT];


  // THREAD: Mutex Declarations
  pthread_mutex_t msgInBlock[ROBOT_COUNT];
  pthread_mutex_t msgOutBlock[ROBOT_COUNT];
  pthread_mutex_t msgOutCondBlock[ROBOT_COUNT];
  pthread_mutex_t wssCommBlock;
  pthread_mutex_t lastSeenMsgIDBlock[ROBOT_COUNT];

  

};

#endif
