/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file joystickData.hh

  \brief NML declarations for joystick data channel. 

  This channel information about the state of joystick control,
  such as which mode is active (joint, mobility, world, tool).

  \code CVS Status:
  $Author: proctor $
  $Revision: 1.1 $
  $Date: 2007/07/24 20:30:36 $
  \endcode

  \author Fred Proctor
*/

#ifndef JOYSTICK_DATA_HH
#define JOYSTICK_DATA_HH

#include "moastNmlOffsets.hh"	// JOYSTICK_DATA_BASE

#define JOYSTICK_DATA_TYPE (JOYSTICK_DATA_BASE + 1)

#define JOYSTICK_MODE_MOBILITY 1
#define JOYSTICK_MODE_JOINT    2
#define JOYSTICK_MODE_TOOL     3
#define JOYSTICK_MODE_WORLD    4

class JoystickData : public NMLmsg {
public:
  JoystickData() : NMLmsg(JOYSTICK_DATA_TYPE, sizeof(JoystickData)) {
    mode = JOYSTICK_MODE_MOBILITY;
  };
  void update(CMS *);

  int mode;			//!< one of JOYSTICK_MODE_MOBILITY, ...
};

extern int joystickData_format(NMLTYPE type, void * buf, CMS * cms);
extern const char *joystickData_symbol_lookup(long type);

#endif	// JOYSTICK_DATA_HH

/*
  Modification history:

  $Log: joystickData.hh,v $
  Revision 1.1  2007/07/24 20:30:36  proctor
  Initial checkin

*/
