/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/

/*! \file nodeInfo.hh
    \brief defines NodeInfo class for use in all search routines

This is used by dijkAVL.cc, dijkTabs.cc.

*/

#ifndef NODE_INFO_HH
#define NODE_INFO_HH

/***********************************************************************/

/*! \class NodeInfo
    \brief parent class for additional attributes of a node

Planners that need additional information about graph nodes than is
available in the classes (such as dijkNode and AVLNode) that use
NodeInfo can create a child class of NodeInfo and use instances of the
child class as the value of moreInfo in each node.

As defined, the only information provided by NodeInfo is the index (in
some array) of a node.

*/

class NodeInfo
{
public:
  NodeInfo(){} /*!< constructor taking no arguments, setting nothing */
  NodeInfo(int index_in){index = index_in;} /*!<onstructor setting index */
  ~NodeInfo(){}
  
  int index; /*!< index in an array of nodes of the node with this info */
};

/***********************************************************************/

#endif  // NODE_INFO_HH
