/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file vehSP.hh

  \brief NML declaration for the veh sensor processing level.  Contains
         veh level sensor command, status, set, and data messages.  

  \code CVS Status:
  $Author: proctor $
  $Revision: 1.2 $
  $Date: 2006/06/21 19:38:13 $
  \endcode

  \author Fred Proctor
  \date 8 June 2005
*/

#ifndef VEH_SP_HH
#define VEH_SP_HH

#include "moastNmlOffsets.hh"

#define VEH_SP_CMD_NAME          "vehSPCmd"
#define VEH_SP_STAT_NAME         "vehSPStat"
#define VEH_SP_CFG_NAME          "vehSPCfg"
#define VEH_SP_SET_NAME          "vehSPSet"
#define VEH_SP_DATA_NAME         "vehSPData"

#define VEH_SP_CMD_BASE          (VEH_SP_BASE)
#define VEH_SP_STAT_BASE         (VEH_SP_BASE + 100)
#define VEH_SP_CFG_BASE          (VEH_SP_BASE + 200)
#define VEH_SP_SET_BASE          (VEH_SP_BASE + 300)
#define VEH_SP_DATA_BASE         (VEH_SP_BASE + 400)

#define VEH_SP_CMD_INIT_TYPE     (VEH_SP_CMD_BASE + 1)
#define VEH_SP_CMD_ABORT_TYPE    (VEH_SP_CMD_BASE + 2)
#define VEH_SP_CMD_HALT_TYPE     (VEH_SP_CMD_BASE + 3)
#define VEH_SP_CMD_SHUTDOWN_TYPE (VEH_SP_CMD_BASE + 4)

#define VEH_SP_STAT_TYPE         (VEH_SP_STAT_BASE + 1)

#define VEH_SP_CFG_CYCLE_TIME_TYPE (VEH_SP_CFG_BASE + 1)
#define VEH_SP_CFG_DEBUG_TYPE      (VEH_SP_CFG_BASE + 2)

#define VEH_SP_SET_TYPE          (VEH_SP_SET_BASE + 1)
         
#define VEH_SP_DATA_1D_TYPE      (VEH_SP_DATA_BASE + 1)
#define VEH_SP_DATA_2D_TYPE      (VEH_SP_DATA_BASE + 2)

#define VEH_SP_DATA_1D_MAXSIZE   128
#define VEH_SP_DATA_2D_MAXSIZE   256

 
// VEH_SP_CMD_XXX
/////////////////////////////////////////////////
class VehSPCmdInit:public RCS_CMD_MSG {
public:
  VehSPCmdInit():RCS_CMD_MSG(VEH_SP_CMD_INIT_TYPE,
				   sizeof(VehSPCmdInit)) {};
  void update(CMS *);
};

class VehSPCmdAbort:public RCS_CMD_MSG {
public:
  VehSPCmdAbort():RCS_CMD_MSG(VEH_SP_CMD_ABORT_TYPE,
				    sizeof(VehSPCmdAbort)) {};
  void update(CMS *);
};

class VehSPCmdHalt:public RCS_CMD_MSG {
public:
  VehSPCmdHalt():RCS_CMD_MSG(VEH_SP_CMD_HALT_TYPE,
				   sizeof(VehSPCmdHalt)) {};
  void update(CMS *);
};

class VehSPCmdShutdown:public RCS_CMD_MSG {
public:
  VehSPCmdShutdown():RCS_CMD_MSG(VEH_SP_CMD_SHUTDOWN_TYPE,
				   sizeof(VehSPCmdShutdown)) {};
  void update(CMS *);
};

// VEH_SP_STAT
//////////////////////////////////////////////
class VehSPStat : public RCS_STAT_MSG {
public:
  VehSPStat() : RCS_STAT_MSG(VEH_SP_STAT_TYPE, sizeof(VehSPStat)) {};
  void update(CMS *);

  unsigned int heartbeat;
};

// VEH_SP_CFG_XXX
//////////////////////////////////////////////////////

class VehSPCfgCycleTime:public RCS_CMD_MSG {
public:
  VehSPCfgCycleTime():RCS_CMD_MSG(VEH_SP_CFG_CYCLE_TIME_TYPE,
			       sizeof(VehSPCfgCycleTime)) {};
  void update(CMS *);

  //! period in seconds
  double cycleTime;
};

class VehSPCfgDebug:public RCS_CMD_MSG {
public:
  VehSPCfgDebug():RCS_CMD_MSG(VEH_SP_CFG_DEBUG_TYPE,
			       sizeof(VehSPCfgDebug)) {};
  void update(CMS *);

  //! debug level
  int debug;
};

// VEH_SP_SET
//////////////////////////////////////////////////////
class VehSPSet : public RCS_STAT_MSG {
public:
  VehSPSet():RCS_STAT_MSG(VEH_SP_SET_TYPE, sizeof(VehSPSet)) {};
  void update(CMS *);

  double cycleTime;
  int debug;
};

extern int vehSP_format(NMLTYPE type, void *buf, CMS * cms);
extern const char * vehSP_symbol_lookup(long type);

#endif // VEH_SP_HH

/*
  Modification history:

  $Log: vehSP.hh,v $
  Revision 1.2  2006/06/21 19:38:13  proctor
  Added support for Java, which involved including some left-out headers
  like moastNmlOffsets.hh and moastTypes.hh

  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.1  2005/06/08 20:06:36  proctor
  initial checkin

*/
