#include "communications.h"
#include <stdlib.h>
#include <stdio.h>
#include <fstream>

using namespace std;

// Constructors

Communications::Communications()
{
	imgsvr = new ImgSvr();
}

Communications::Communications(int robID, char* wssIP)
{
	imgsvr = new ImgSvr();
	myID = robID;
	WSSIP = wssIP;

	//init mutexes + condition variables
	pthread_mutex_init(&wssCommBlock,NULL);
	for(int i=0;i<ROBOT_COUNT;i++){
		pthread_mutex_init(&(msgInBlock[i]),NULL);
		pthread_mutex_init(&(msgOutBlock[i]),NULL);
		pthread_cond_init (&(msgOutCond[i]), NULL);
		pthread_mutex_init(&(msgOutCondBlock[i]), NULL);
	}

	// Set up connection with WSS
	struct sockaddr_in pin;
	struct hostent *server_host_name;
	server_host_name = gethostbyname(wssIP);
	bzero(&pin, sizeof(pin));
	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = htonl(INADDR_ANY);
	pin.sin_addr.s_addr = ((struct in_addr *)(server_host_name->h_addr))->s_addr;
	pin.sin_port = htons(50000);
	socket_descriptor = socket(AF_INET, SOCK_STREAM, 0);
	connect(socket_descriptor, (sockaddr *)&pin, sizeof(pin));

	// Set up the connection with ImgSvr
	printf("Connecting to ImgSvr in Communications constructor\n");
	string sIP = "";
	sIP.insert(0, wssIP);
	imgsvr->ConnectImgSvr(sIP, IMGSVR_PORTNUM);

	// Initialize the WSS
	printf("Initializing WSS connection... \n");
	char wssName[50];
	if(myID==0) sprintf(wssName,"CommStation");
	else sprintf(wssName,"Robot%d",myID);
	if(!SendInit(wssName, 40001+myID))
		printf("Problem  registering with WSS when sending INIT command.\n");

	printf("Done with INIT.\n");

	pthread_t threadID;
	CommArgs* a = new CommArgs();
	a->obj=this;
	int threadStatus = pthread_create(&threadID, NULL, Communications::startListenForConnections, a);
	//printf("after thread started\n");
	for(int i=myID+1;i<ROBOT_COUNT;i++){
		CommArgs* args2 = new CommArgs();
		args2->sockfd=-1;
		args2->robID=i;
		args2->obj=this;
		pthread_t threadIDConnect2;
		pthread_create(&threadIDConnect2,NULL,Communications::startManageIncomingMessages,args2);
	}

	// Set robotType to NULL.
	robotType = '\0';
}

Communications::Communications(int robID, char* wssIP, char robType)
{
	initCommunications(robID, wssIP);  
	robotType = (unsigned char) robType;
}

void Communications::initCommunications(int robID, char* wssIP)
{
	imgsvr = new ImgSvr();
	myID = robID;
	WSSIP = wssIP;

	printf("Inside Comm constructor\n");

	//init mutexes + condition variables
	pthread_mutex_init(&wssCommBlock,NULL);
	for(int i=0;i<ROBOT_COUNT;i++){
		pthread_mutex_init(&(msgInBlock[i]),NULL);
		pthread_mutex_init(&(msgOutBlock[i]),NULL);
		pthread_cond_init (&(msgOutCond[i]), NULL);
		pthread_mutex_init(&(msgOutCondBlock[i]), NULL);
		pthread_mutex_init(&(lastSeenMsgIDBlock[i]), NULL);
		printf("INITTED lsmidBlock[%d]\n", i);
		pthread_mutex_lock(&(lastSeenMsgIDBlock[i]));
		pthread_mutex_unlock(&(lastSeenMsgIDBlock[i]));
	}

	// Set up connection with WSS
	struct sockaddr_in pin;
	struct hostent *server_host_name;
	server_host_name = gethostbyname(wssIP);
	bzero(&pin, sizeof(pin));
	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = htonl(INADDR_ANY);
	pin.sin_addr.s_addr = ((struct in_addr *)(server_host_name->h_addr))->s_addr;
	pin.sin_port = htons(50000);
	socket_descriptor = socket(AF_INET, SOCK_STREAM, 0);
	connect(socket_descriptor, (sockaddr *)&pin, sizeof(pin));

	// Set up the connection with ImgSvr
	printf("Connecting to ImgSvr in Communications constructor\n");
	string sIP = "";
	sIP.insert(0, wssIP);
	imgsvr->ConnectImgSvr(sIP, IMGSVR_PORTNUM);

	// Initialize the WSS
	printf("Initializing WSS connection... \n");
	char wssName[50];
	if(myID==0) sprintf(wssName,"CommStation");
	else sprintf(wssName,"Robot%d",myID);
	if(!SendInit(wssName, 40001+myID))
		printf("Problem  registering with WSS when sending INIT command.\n");

	printf("Done with INIT.\n");

	pthread_t threadID;
	CommArgs* a = new CommArgs();
	a->obj=this;
	int threadStatus = pthread_create(&threadID, NULL, Communications::startListenForConnections, a);
	//printf("after thread started\n");
	for(int i=myID+1;i<ROBOT_COUNT;i++){
		CommArgs* args2 = new CommArgs();
		args2->sockfd=-1;
		args2->robID=i;
		args2->obj=this;
		pthread_t threadIDConnect2;
		pthread_create(&threadIDConnect2,NULL,Communications::startManageIncomingMessages,args2);
	}

	// Set robotType to NULL.
	robotType = '\0';

	// Set batteryLife to an uninitialized value.
	batteryLife = -1;
}
/******************************
WSS Functions
******************************/
bool Communications::SendInit(string RobotName, int Port)
{
	char response[8192];
	char sPort[33];
	bool result = false;

	sprintf(sPort, "%d", Port);
	string cmd = "INIT {Robot ";
	cmd.append(RobotName);
	cmd.append("} {Port ");
	cmd.append(sPort);
	cmd.append("}\r\n");

	pthread_mutex_lock(&(wssCommBlock));
	// Send Command
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);

	// Get Response
	//recv(socket_descriptor, response, 8192, 0);  
	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}
	pthread_mutex_unlock(&(wssCommBlock));
	// Evaluate Response
	if(string(response).compare(INITResponse())==0)
		result = true;

	return result;
}

float Communications::GetSignalStrength(string RobotName)
{

	char response[8192];
	string cmd = "GETSS {Robot ";//100;
	cmd.append(RobotName);
	cmd.append("}\r\n");

	// Send Command
	pthread_mutex_lock(&(wssCommBlock));
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);


	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}

	pthread_mutex_unlock(&(wssCommBlock));
	return SignalStrengthResponse(string(response));
}

int Communications::getRobotDNS(string RobotName){
	pthread_mutex_lock(&(wssCommBlock));
	char response[8192];
	string cmd = "DNS {Robot ";//100
	cmd.append(RobotName);
	cmd.append("}\r\n");

	// Send Command
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);
	
	// Get Response

	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}

	pthread_mutex_unlock(&(wssCommBlock));
	return DNSResponse(response);//SignalStrengthResponse(string(response));	
}
int Communications::getRobotReverseDNS(int port){
	pthread_mutex_lock(&wssCommBlock);
	char response[8192];
	char c[255];
	sprintf(c,"REVERSEDNS {Port %d}\r\n",port);
	string cmd(c);

	// Send Command
	send(socket_descriptor, cmd.c_str(), strlen(cmd.c_str()), 0);

	bool done = false;
	for(int i=0; !done; i++)
	{
		read(socket_descriptor, response+i, 1);
		if((int)response[i] == 13)
		{
			read(socket_descriptor, response+i+1, 1);
			done = true;
		}
	}

	pthread_mutex_unlock(&wssCommBlock);
	return ReverseDNSResponse(response);
}
string Communications::INITResponse()
{
	return "INITREPLY {Status OK}\r\n";
}

float Communications::SignalStrengthResponse(string msg)
{
	float result;
	string tmp = msg.substr(msg.find("{Strength ") + 10, msg.find("}", msg.find("{Strength ") + 10)-msg.find("{Strength ")-10);

	if(tmp.compare("NaN")==0)
	{
		result = 100;
	}
	else if(tmp.compare("INF")==0)
	{
		result = 101;
	}
	else
	{
		sscanf(tmp.c_str(), "%f", &result);
	}

	return result;
}

int Communications::DNSResponse(string msg)
{
	int result;

	string tmp = msg.substr(msg.find("{Port ") + 6, msg.length() - 1 - msg.find("{Port ") - 6);
	sscanf(tmp.c_str(), "%d", &result);

	return result;
}

int Communications::ReverseDNSResponse(string msg)
{
	int result;
	if(msg.find("CommStation")!=string::npos) return 0;
	if(msg.find("UnknownOrIllegalPort")!=string::npos) return -1;
	string tmp = msg.substr(msg.find("{Robot Robot") + 12, msg.length() - 1 - msg.find("{Robot Robot") - 12);
	printf("I got %s from %s\n",tmp.c_str(),msg.c_str());
	sscanf(tmp.c_str(), "%d", &result);

	return result;
}

void* Communications::listenForConnections(void* arg){
	delete (CommArgs*)arg;
	int sockfd, newsockfd, portno, clilen;
	struct sockaddr_in serv_addr, cli_addr;
	portno = 40001+myID;
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0) 
		printf("ERROR opening socket\n");

	int flags = 1;
	setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,(char*)&flags,sizeof(flags));

	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = INADDR_ANY;
	serv_addr.sin_port = htons(portno);
	if (bind(sockfd, (struct sockaddr *) &serv_addr,
		sizeof(serv_addr)) < 0) 
		printf("ERROR on binding when trying to Listen\n");
	listen(sockfd,5);
	clilen = sizeof(cli_addr);
	listen_sockfd = sockfd;
	while(true){
		printf("Waiting for connection on port %d\n",portno);

		newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, 
			(socklen_t*)&clilen);
		if (newsockfd < 0) 
			printf("ERROR on accept\n");
		else{
			CommArgs* args1 = new CommArgs();
			CommArgs* args2 = new CommArgs();
			args1->sockfd=newsockfd;
			args1->robID=getRobotReverseDNS(htons(((struct sockaddr_in *)&cli_addr)->sin_port));;
			args1->obj=this;
			args2->sockfd=newsockfd;
			args2->robID=args1->robID;
			args2->obj=this;
			if(args1->robID>=0){
				printf("Got connection from Robot %u\n",args1->robID);
				robotcomm_sockfd[args1->robID] = newsockfd;
				pthread_t threadID;
				int threadStatus=pthread_create(&threadID,NULL, Communications::startManageIncomingMessages,args1);
				pthread_t threadID2;
				threadStatus=pthread_create(&threadID2,NULL, Communications::startManageOutgoingMessages,args2);
			}else{
				printf("Connection Accept Error (invalid robID)\n");
			}
		}
	}
}
void* Communications::manageOutgoingMessages(void* arg){
	int sockfd = ((CommArgs*)arg)->sockfd;
	int robID = ((CommArgs*)arg)->robID;
	pthread_t RSSThreadID;

	//printf("Spawned new outgoing messages thread for robot # %d\n",robID);

	// If this is the connection to base station, send a robotType message.
	if(robID == 0)
	{
		unsigned char* RTPMsg = new unsigned char[4];
		RTPMsg[0] = 'R';
		RTPMsg[1] = 'T';
		RTPMsg[2] = 'P';
		RTPMsg[3] = robotType;
		int success = send(sockfd, RTPMsg, 4, 0);
		if(success < 0)
		{
			printf("Failed to send RTP Message: %s\n", (char*)RTPMsg);
		}
		else
		{
			printf("Sent RTP Message to Base Station (%d bytes)\n",success);
		}
		delete RTPMsg;

		// In addition, spawn the heartbeat thread for reporting Signal Strength
		// to the base station.
		if(1)
		{
			CommArgs* args1 = new CommArgs();
			args1->sockfd=sockfd;
			args1->robID=robID;
			args1->obj=this;
			pthread_create(&RSSThreadID, NULL, Communications::startRSSThread,args1);
		}
	}

	delete (CommArgs*)arg;
	while(true){
		pthread_cond_wait(&(msgOutCond[robID]),&(msgOutCondBlock[robID]));
		pthread_mutex_lock(&(msgOutBlock[robID]));
		while(!msgsOut[robID].empty()){
			//printf("Sending outgoing message over socket.\n");
			//printf("Message.length = %d\n",msgsOut[robID].front().msgLen);
			//printf("OPCODE: %c%c%c\n", (char)msgsOut[robID].front().msg[0], (char)msgsOut[robID].front().msg[1], (char)msgsOut[robID].front().msg[2]);
			//printf("manageOutgoing# %d: Sending Message w/ %d bytes.\n",robID,msgsOut[robID].front().msgLen);
			int s = send(sockfd, msgsOut[robID].front().msg, msgsOut[robID].front().msgLen, 0);
			//printf("Outgoing Message was successfully sent!\n");
			if(s<0){
				printf("error sending to %d\n",robID);
				sockfd=-1;
				break;
			}else{
				// Free allocated memory for the messages
				//printf("Now deleting message. %d elements left on queue...", msgsOut[robID].size());
				delete [] msgsOut[robID].front().msg;
				msgsOut[robID].pop();
				//printf("Now %d\n",msgsOut[robID].size());
			}
		}
		pthread_mutex_unlock(&(msgOutBlock[robID]));
	}
	pthread_exit(0);
}
void* Communications::RSSReportThread(void* arg)
{
	//printf("Inside RSSReportThread\n");
	float robSS;
	char RobotName[33];
	int robID = ((CommArgs*)arg)->robID;
	int byteNum;

	// 3 bytes for opcode, 4 bytes per float.
	int RSSMsgLen = 3+(4*(ROBOTCOUNT-1))+4; //+4 for batteryLife.
	unsigned char RSSMsg [RSSMsgLen];

	// Set the opcode.
	RSSMsg[0] = 'R';
	RSSMsg[1] = 'S';
	RSSMsg[2] = 'S';

	while(true)
	{
		//printf("Sending RSS:");
		byteNum = 3;
		// Query WSS for signal strength to robot i, appending it to RSSMsg.
		for(int i=0; i<ROBOTCOUNT; i++)
		{
			if(i!=myID)
			{
				if(i==0) sprintf(RobotName, "CommStation");
				else sprintf(RobotName, "Robot%d", i);
				//else if(i>myID) sprintf(RobotName, "Robot%d", i);

				robSS = GetSignalStrength(string(RobotName));

				//printf(" %f",robSS);

				memcpy(&(RSSMsg[byteNum]), &robSS, 4);

				bzero(RobotName, 33);
				byteNum+=4;
			}
		}

		memcpy(&(RSSMsg[byteNum]), &batteryLife, 4);

		//printf("sent RSS Message with %d battery life starting at byte index %d\n",batteryLife,byteNum);

		//printf("\n");

		sendMessage(0, RSSMsg, RSSMsgLen);

		sleep(3);
	}

	pthread_exit(0);
}
void* Communications::manageIncomingMessages(void* arg){
	int sockfd = ((CommArgs*)arg)->sockfd;
	int robID = ((CommArgs*)arg)->robID;

	//printf("Inside manageIncomingMessages for robot # %d\n",robID);

	delete (CommArgs*)arg;
	unsigned char buffer[1024000];
	unsigned char* pch;
	char opcode [3];
	while(true){
		if(sockfd==-1){
			char rName[33];
			struct sockaddr_in pin;
			struct hostent *server_host_name;
			if(robID==0) sprintf(rName,"CommStation");
			else sprintf(rName,"Robot%d",robID);
			string rNameString(rName);
			int rob2Port = getRobotDNS(rNameString);
			if(rob2Port < 0){
				printf("failed to get port for robot %d. Waiting 10 seconds before retry...\n",robID);
				sockfd = -1 ;
			}else{
				printf("Connecting to Robot %d on Port %d\n",robID,rob2Port);

				sockfd=-1;
				server_host_name = gethostbyname(WSSIP);
				bzero(&pin, sizeof(pin));
				pin.sin_family = AF_INET;
				pin.sin_addr.s_addr = htonl(INADDR_ANY);
				pin.sin_addr.s_addr = ((struct in_addr *)(server_host_name->h_addr))->s_addr;
				pin.sin_port = htons(rob2Port);
				sockfd = socket(AF_INET, SOCK_STREAM, 0);
				connect(sockfd, (sockaddr *)&pin, sizeof(pin));
				CommArgs* args2 = new CommArgs();
				args2->sockfd=sockfd;
				args2->robID=robID;
				args2->obj=this;
				robotcomm_sockfd[robID] = sockfd;
				pthread_t threadID2;
				int threadStatus=pthread_create(&threadID2,NULL, Communications::startManageOutgoingMessages,args2);
			}
		}
		while(sockfd !=-1)
		{
			// Zero out the buffer
			bzero(buffer,1024000);

			// read the OPCODE
			int n = read(sockfd,&buffer[0],3);
			if(n==0)
				continue;

			///printf("Got read length of %d\n",n);

			//printf("\n\nI got the following n value: %d\n\n", n);
			if (n != 3)
			{
				printf("ERROR reading from socket for rob %d\n",robID);
				printf("Got %d bytes\n", n);
				sockfd=-1;
				break;
			}
			else
			{
				///printf("\nGot %c%c%c message.\n", buffer[0],buffer[1],buffer[2]);
				bool isRelayGoal = false;
				unsigned char *msgBuf = NULL;
				int newMsgSize = 0;
				queue<int> fwdRecipients;
				int originator = -1;

				//printf("\n\nI got the following: %c%c%c\n\n", ((char)buffer[0]),((char)buffer[1]),((char)buffer[2]));

				/// REL Command (RELAY)
				if(((char)buffer[0])=='R' && ((char)buffer[1])=='E' && ((char)buffer[2])=='L')
				{
					int msgSize = 0;
					int curMsgID = 0;
					unsigned char byteBuf;
					unsigned char* msgPath = new unsigned char[ROBOTCOUNT];
					char RobotName[33];
					int byteNum = 0;

					n += read(sockfd, msgPath, ROBOTCOUNT); // Read off the message route header
					n += read(sockfd, &curMsgID, 4); // Read the ID of the message.
					n += read(sockfd, &msgSize, 4); // Read the size in bytes of the packaged message.

					// Read the opcode of the payload message into char opcode []. The message sitting
					// on the socket is now MISSING its opcode.
					bzero(opcode, 3);
					n += read(sockfd, opcode, 3);

					///printf("Recieved REL: %c%c%c %d %d %c%c%c\n", msgPath[0],msgPath[1],msgPath[2],curMsgID,msgSize,opcode[0],opcode[1],opcode[2]);

					// Set up the list of recipients of the forward. Also, determine the originator
					// in order to update the proper lastSeenMsg counter.
					// - First though, get the latest signal strength values from WSS.
					if(updateRobSS() != 0)
						printf("COMM: updateRobSS() failed!\n");

					for(int robNum=0; robNum<ROBOT_COUNT; robNum++)
					{
						// robNum = this robot. Do nothing.
						if(robNum == myID) continue;

						// robNum is the originating robot. Set originator and increment the proper message ID counter.
						if(msgPath[robNum] == (unsigned char)'I')
						{
							///printf("Robot %d was the originator.\n",robNum);
							// Set the originator to this robot ID.
							originator = robNum;

							// Lock the memory we're going to access.
							pthread_mutex_lock(&(lastSeenMsgIDBlock[robNum]));

							string str = "   ";
							str[0] = opcode[0];
							str[1] = opcode[1];
							str[2] = opcode[2];

							///printf("String Opcode: %s\n", str.c_str());
							// Check whether the originating robot has sent this type of message before.
							map<string,int>::iterator iter = lastSeenMsgID[robNum].find(str);
							// - If it has:
							if(iter != lastSeenMsgID[robNum].end())
							{
								///printf("The message type has been seen from this robot before.\n");
								// Seen this message before. Set originator to -1 so we know after the loop exits.
								if(curMsgID <= lastSeenMsgID[robNum][str])
								{
									//printf("Current message has been seen before (Have %d). Ignoring.\n",lastSeenMsgID[robNum][str]);
									originator = -1;
								}
								else
								{
									iter->second = curMsgID;
								}
							}
							// - If this is the first instance of this message type:
							else
							{
								///printf("Never seen this message before. Creating...\n");
								lastSeenMsgID[robNum][str] = 1;
							}

							// Unlock the Msg counter
							pthread_mutex_unlock(&(lastSeenMsgIDBlock[robNum]));
						}
						else if(msgPath[robNum] == (unsigned char)'N' || msgPath[robNum] == (unsigned char)'G')
						{
							if(robSS[robNum] > .98*WSS_CUTOFF)
							{
								fwdRecipients.push(robNum);
								if(msgPath[robNum] != (unsigned char)'G')
								{
									msgPath[robNum] = (unsigned char)'S';
								}
								///else
								///printf("In range of Goal!\n");
							}
						}
					}

					// If the originator is -1, this means that we've seen this message
					// already.  Read the rest of the message we've already seen off of
					// the socket and discard it.
					if(originator==-1)
					{
						// Read off the remaining useless bytes of the payload.
						int readBytes = 0;
						while(readBytes != msgSize-3)
						{
							readBytes += read(sockfd, buffer, msgSize-3-readBytes);
						}
						continue;
					}

					// Not the target robot.  The code in this if statement deals with
					// reconstructing and propagating the relayed message to each robot
					// in the forward queue.
					if(msgPath[myID] != (unsigned char)'G')
					{
						int robSS;
						buffer[0] = (unsigned char)'R';
						buffer[1] = (unsigned char)'E';
						buffer[2] = (unsigned char)'L';
						byteNum = 3;

						// Put the outgoing message together.
						// - Message Path (already modified above)
						memcpy(&buffer[byteNum], msgPath, ROBOT_COUNT);
						byteNum += ROBOT_COUNT;
						// - Message ID
						memcpy(&buffer[byteNum], &curMsgID, 4);
						byteNum += 4;
						// - Payload Size
						memcpy(&buffer[byteNum], &msgSize, 4);
						byteNum += 4;
						// - Payload
						// -- First, the opcode needs to be put back.
						memcpy(&buffer[byteNum], opcode, 3);
						byteNum += 3;
						// -- Read the payload into the buffer.
						int readSize = 0;
						while(readSize != msgSize-3)
						{
							readSize += read(sockfd, &buffer[byteNum+readSize], msgSize-3-readSize);
						}
						byteNum += msgSize-3;

						// Send the message out to the robots in the forward list.
						if(fwdRecipients.empty())
							printf("manageIncMsgs: No downstream robots in signal range.\n");
						while(!fwdRecipients.empty())
						{
							///printf("Sending message to Robot %d\n",fwdRecipients.front());
							sendMessage(fwdRecipients.front(), buffer, byteNum);
							fwdRecipients.pop();
						}

						// Clean up any dynamic memory allocations
						delete [] msgPath;

						// Go back to the start of the loop and wait for the next message.
						continue;
					}
					else // This robot was the target.
					{
						isRelayGoal = true;
						delete [] msgPath;
					}
				}

				if(isRelayGoal)
				{
					// Zero out the buffer
					bzero(buffer,1024000);

					// read the OPCODE
					memcpy(buffer, opcode, 3);
				}

				// Check for message types
				if(((char)buffer[0])=='I' && ((char)buffer[1])=='M' && ((char)buffer[2])=='R') // IMR message type
				{
					///printf("\nProcessing IMR Command\n");
					///if(originator != -1)
					///printf("Recieved IMR from robot %d\n", originator);
					//Process Image Request (grab image from ImgSvr)
					imgsvr->SendOKImgSvr();
					imgsvr->ReadImgSvrImage();
					//printf("Got IMR Request\n.");

					//printf("Creating OPCODE\n");
					// Build message to send to the requestor.
					newMsgSize = 3 + 4 + imgsvr->imgSize;
					msgBuf = new unsigned char[3 + 4 + imgsvr->imgSize];
					msgBuf[0] = (unsigned char)('I');
					msgBuf[1] = (unsigned char)('M');
					msgBuf[2] = (unsigned char)('G');

					//printf("adding image size\n");

					memcpy(&msgBuf[3], &(imgsvr->imgSize), 4);

					//printf("adding image\n");
					memcpy(&msgBuf[7], imgsvr->buffer, imgsvr->imgSize);

					//printf("sending message\n");

					// Send the message to the requestor if it wasn't a relay message.
					if(1 || !isRelayGoal)
					{
						sendMessage(0, msgBuf, 3 + 4 + imgsvr->imgSize);
						delete [] msgBuf;
					}
					//printf("sent message\n");

					// Clean everything up
					imgsvr->ClearBuffer();

					//printf("cleared buffer\n");
				}
				else if(((char)buffer[0])=='T' && ((char)buffer[1])=='S' && ((char)buffer[2])=='T') // TST message type
				{
					n += read(sockfd,&buffer[3],6);
					buffer[n]='\0';
					string s((char*)buffer);
					printf("\n\nI got the following TST Message: %s\n\n", s.c_str());
				}
				// Unneeded for robots... used by CommStationShell.
				else if(((char)buffer[0])=='I' && ((char)buffer[1])=='M' && ((char)buffer[2])=='G')
				{
					printf("Got IMG back.\n");
					int size;
					read(sockfd,&size,4);

					ofstream file("bufferContents.jpg", ios::binary | ios::trunc);
					for(int count=0; count<size; count++)
					{
						read(sockfd,&buffer[3],1);
						file << buffer[3];
					}
					file.close();


				}
				else if(((char)buffer[0])=='M' && ((char)buffer[1])=='O' && ((char)buffer[2])=='F')
				{
					printf("Command identified as: MOF\n");
					// Create a new CommMsg to send to the robot's controller.
					CommMsg* msgStruct = new CommMsg(1, "MOF");

					msgStruct->params[0] = 1.1;

					// Push it onto the queue to be read.
					pthread_mutex_lock(&(msgInBlock[robID]));
					msgsIn[robID].push(msgStruct);
					pthread_mutex_unlock(&(msgInBlock[robID]));

					//printf("Init command pushed onto incoming message queue.\n");
				}
				else if (((char)buffer[0])=='M' && ((char)buffer[1])=='O' && ((char)buffer[2])=='N')
				{
					//printf("Command identified as: MON\n");
					// Create a new CommMsg to send to the robot's controller.
					CommMsg* msgStruct = new CommMsg(1, "MON");

					msgStruct->params[0] = 1.1;

					// Push it onto the queue to be read.
					pthread_mutex_lock(&(msgInBlock[robID]));
					msgsIn[robID].push(msgStruct);
					pthread_mutex_unlock(&(msgInBlock[robID]));

					//printf("Init command pushed onto incoming message queue.\n");
				}
				// MAP Response... used by CommStationShell.
				else if(((char)buffer[0])=='M' && ((char)buffer[1])=='A' && ((char)buffer[2])=='P')
				{
					double xLow;
					double yLow;
					double resolution;
					float xPos;
					float yPos;
					int numUpdates;
					int size = 3;

					size += read(sockfd,&xLow,8);
					size += read(sockfd,&yLow,8);
					size += read(sockfd,&resolution,8);
					size += read(sockfd,&xPos,4);
					size += read(sockfd,&yPos,4);
					size += read(sockfd,&numUpdates,4);

					int singleSize = 0;
					float prob=0;
					double cellxPos=0;
					double cellyPos=0;
					unsigned char* readBuf;

					for(int i=0; i<numUpdates; i++)
					{
						readBuf = (unsigned char*)&prob;
						singleSize = read(sockfd, readBuf ,4);
						while(singleSize != 4)
						{
							printf("MAP-ERROR on update #%d: Read %d bytes for val\n",i,singleSize);
							singleSize += read(sockfd, &readBuf[singleSize], 4-singleSize);
						}
						size += singleSize;

						readBuf = (unsigned char*)&cellxPos;
						singleSize = read(sockfd, readBuf,8);
						while(singleSize != 8)
						{
							printf("MAP-ERROR on update #%d: Read %d bytes for xPos.\n",i, singleSize);
							singleSize += read(sockfd, &readBuf[singleSize], 8-singleSize);
						}
						size += singleSize;

						readBuf = (unsigned char*)&cellyPos;
						singleSize = read(sockfd, readBuf,8);
						while(singleSize != 8)
						{
							printf("MAP-ERROR on update #%d: Read %d bytes for yPos.\n",i, singleSize);
							singleSize += read(sockfd, &readBuf[singleSize], 8-singleSize);
						}
						//if(singleSize != 20) printf("MAP: Error reading update#: %d %d bytes\n",i, singleSize);
						size += singleSize;

					}
					printf("MAP: %lf,%lf,%lf,%f,%f,%d\n", xLow, yLow, resolution, xPos, yPos, numUpdates);

					printf("MAP: Got %d bytes\n\n",size);

				}
				/// SKD command (Skid steer)
				else if (((char)buffer[0])=='S' && ((char)buffer[1])=='K' && ((char)buffer[2])=='D')
				{
					// Create a new CommMsg to send to the controller.
					CommMsg* msgStruct = new CommMsg(2, "SKD");

					//printf("Got SKD Command.\n");

					// Read in the parameters.

					if(read(sockfd, msgStruct->params, 8) != 8)
						printf("Failed to read parameters from SKD command.\n");

					//printf("COMMAND: SKD %f %f\n", msgStruct->params[0], msgStruct->params[1]);

					// Push it onto the queue to be read.
					pthread_mutex_lock(&(msgInBlock[robID]));
					msgsIn[robID].push(msgStruct);
					pthread_mutex_unlock(&(msgInBlock[robID]));
				}
				/// ARD Command (Aerial Drive)
				else if(((char)buffer[0])=='A' && ((char)buffer[1])=='R' && ((char)buffer[2])=='D')
				{
					// Create the CommMsg that will be sent to the controller.  Has 4 params.
					CommMsg* msgStruct = new CommMsg(4, "ARD");

					if(read(sockfd, msgStruct->params, 16) != 16)
						printf("Failed to read parameters from ARD command.\n");
					else
					{

						//printf("COMMAND: ARD %f %f %f %f\n", msgStruct->params[0],
						//     msgStruct->params[1], msgStruct->params[2], msgStruct->params[3]);

						// Push it onto the queue to be read.
						pthread_mutex_lock(&(msgInBlock[robID]));
						msgsIn[robID].push(msgStruct);
						pthread_mutex_unlock(&(msgInBlock[robID]));
					}
				}
				/// WAY Command (Waypoint)
				else if(((char)buffer[0])=='W' && ((char)buffer[1])=='A' && ((char)buffer[2])=='Y')
				{
					printf("Communications: Command identified as: WAYPOINT\n");
					// Create the CommMsg to send to the controller.
					CommMsg* msgStruct = new CommMsg(2, "WAY");

					if(read(sockfd, msgStruct->params, 8) != 8)
						printf("Failed to read parameters from WAY command.\n");
					else
					{
						pthread_mutex_lock(&(msgInBlock[robID]));
						msgsIn[robID].push(msgStruct);
						pthread_mutex_unlock(&(msgInBlock[robID]));
					}
				}
				/// INI Command (INIT)
				else if(((char)buffer[0])=='I' && ((char)buffer[1])=='N' && ((char)buffer[2])=='I')
				{
					printf("Command identified as: INITIALIZE\n");
					// Create a new CommMsg to send to the robot's controller.
					CommMsg* msgStruct = new CommMsg(1, "INI");

					msgStruct->params[0] = 1.1;

					// Push it onto the queue to be read.
					pthread_mutex_lock(&(msgInBlock[robID]));
					msgsIn[robID].push(msgStruct);
					pthread_mutex_unlock(&(msgInBlock[robID]));

					//printf("Init command pushed onto incoming message queue.\n");
				}
				/// SPG Command (SPGo)
				else if (((char)buffer[0])=='S' && ((char)buffer[1])=='P' && ((char)buffer[2])=='G')
				{
					printf("Command identified as: SPGO\n");
					CommMsg* msgStruct = new CommMsg(1, "SPG");

					msgStruct->params[0] = 1.1;

					pthread_mutex_lock(&(msgInBlock[robID]));
					msgsIn[robID].push(msgStruct);
					pthread_mutex_unlock(&(msgInBlock[robID]));
				}
				// RTP Reply (RobotType)... used by CommStationShell.
				else if(((char)buffer[0])=='R' && ((char)buffer[1])=='T' && ((char)buffer[2])=='P')
				{
					CommMsg* msgStruct = new CommMsg(1, "RTP");
					unsigned char readType;
					if(1!=read(sockfd, &readType, 1))
						printf("Failed to read last byte of RTP Message.\n");
					else
					{
						msgStruct->params[0] = -1; /// No valid robotType
						if(readType==(unsigned char)'A') /// Aerial Robot
							msgStruct->params[0] = 1;
						else if(readType==(unsigned char)'S') /// Skid Steer Robot
							msgStruct->params[0] = 2;
					}

					pthread_mutex_lock(&(msgInBlock[robID]));
					msgsIn[robID].push(msgStruct);
					pthread_mutex_unlock(&(msgInBlock[robID]));
				}
				// RSS Reply (Robot Signal Strength)... used by CommStationShell.
				else if(((char)buffer[0])=='R' && ((char)buffer[1])=='S' && ((char)buffer[2])=='S')
				{
					float recSS=0;
					//printf("RSS:");
					for(int i=0; i<ROBOTCOUNT-1; i++)
					{
						read(sockfd, &recSS, 4);
						printf(" %f", recSS);
						recSS = 0;
					}

					read(sockfd, &recSS, 4);
					printf("\n");
				}
			}
			//printf("Message From %d: %s\n",robID,buffer);
		}
		if(myID < robID) sleep(6);
		else break;
	}
	pthread_exit(0);
}

bool Communications::readIncomingMessage(int robID, CommMsg** msg){
	bool gotMsg = true;
	//printf("Readinc called. ");
	pthread_mutex_lock(&(msgInBlock[robID]));
	if(!msgsIn[robID].empty())
	{
		*(msg)=msgsIn[robID].front();
		//printf("readIncomingMessage():Got message from %d: %s\n",robID,(char*)msgsIn[robID].front()->type);
		msgsIn[robID].pop();
	}
	else gotMsg = false;
	pthread_mutex_unlock(&(msgInBlock[robID]));

	return gotMsg;
}


void Communications::closeAllSockets()
{
	//shutdown the connections
	shutdown(socket_descriptor,2);
	shutdown(listen_sockfd,2);
	for(int i=0;i<ROBOT_COUNT;i++){
		shutdown(robotcomm_sockfd[i],2);
	}

	//close all the connections
	close(socket_descriptor);
	close(listen_sockfd);
	for(int i=0;i<ROBOT_COUNT;i++){
		close(robotcomm_sockfd[i]);
	}
}

void Communications::sendMessage(int robID, unsigned char* msg, int msgLen)
{

	updateRobSS();
	//printf("sendMessage: got RobSS[%d]= %f\n",robID,robSS[robID]);
	if(robSS[robID] > .98*WSS_CUTOFF)
	{
		///printf("In Range -> Sending normal message\n");
		Message message;
		message.msgLen = msgLen;

		// Copy message
		unsigned char * msgCopy = new unsigned char[msgLen];
		memcpy(msgCopy,msg,msgLen);
		message.msg = msgCopy;

		pthread_mutex_lock(&(msgOutBlock[robID]));
		msgsOut[robID].push(message);
		pthread_mutex_unlock(&(msgOutBlock[robID]));
		pthread_cond_signal(&(msgOutCond[robID]));
	}
	// Otherwise, if the message has to be relayed. This conditional is
	// probably unnecessary, but good to have all the same so we don't
	// end up with layered REL messages.
	else// if(msg[0]!='R' && msg[1]!='E' && msg[2]!='L')
	{
		///printf("Out of Range & not REL packaged. Generating relay message.\n");
		Message message;
		// message.msg = 'REL' + Messageheader[ROBOTCOUNT] + int msgID + int payloadSize + payload
		message.msgLen = 3 + ROBOTCOUNT + 4 + 4 + msgLen;

		unsigned char * buffer = new unsigned char[message.msgLen];

		// Set up a queue to determine which robots to send to.
		queue<int> fwdRecipients;

		// Set up a char [] to hold the opcode of the message.
		char opcode [3];

		// Set up the message.
		buffer[0] = 'R';
		buffer[1] = 'E';
		buffer[2] = 'L';

		int byteNum = 3;

		// Generate the message path header.
		// - First, update the signal strength array Communications::robSS[]
		updateRobSS();
		// - Also, clear out the queue of message recipients.
		while(!fwdRecipients.empty())
			fwdRecipients.pop();

		///printf("Before setting message header.\n");

		// - Now, run through and for each robot, set a value in the message header.
		for(int i=0; i<ROBOTCOUNT; i++)
		{
			if(myID == i) // This robot => set to originator.
			{
				buffer[byteNum] = 'I';
				byteNum++;
			}
			else if(robID == i) // Goal robot => set as goal.
			{
				buffer[byteNum] = 'G';
				byteNum++;
			}
			else if(robSS[i] > .98*WSS_CUTOFF) // Robot is in range.
			{
				buffer[byteNum] = 'S';
				byteNum++;
				fwdRecipients.push(i);
			}
			else // Robot is not the goal and is not in range.
			{
				buffer[byteNum] = 'N';
				byteNum++;
			}
		}

		///printf("Set message header: %c%c%c\n",buffer[3],buffer[4],buffer[5]);

		// Now, append the proper message serial onto it.
		bzero(opcode, 3);
		memcpy(opcode, msg, 3);

		string str = "   ";
		str[0] = opcode[0];
		str[1] = opcode[1];
		str[2] = opcode[2];

		///printf("Sending string opcode: %s\n", str.c_str());

		// - If this robot has never sent this type of message before:
		// -- Lock the memory we're going to access.
		//printf("%d, %d\n",myID,ROBOT_COUNT);
		pthread_mutex_lock(&(lastSeenMsgIDBlock[myID]));
		//printf("Locked mutex.\n");
		if(lastSeenMsgID[myID].find(str) == lastSeenMsgID[myID].end())
		{
			lastSeenMsgID[myID][str] = 1;
			memcpy(&buffer[byteNum], &(lastSeenMsgID[myID][str]), 4);
			byteNum += 4;
		}
		// - If the robot has, increment the counter and append it to the message.
		else
		{
			lastSeenMsgID[myID][str] += 1;
			memcpy(&buffer[byteNum], &(lastSeenMsgID[myID][str]), 4);
			byteNum += 4;
		}

		// -- Unlock the memory now
		pthread_mutex_unlock(&(lastSeenMsgIDBlock[myID]));

		//printf("Made it past the mutexes\n");

		// Append the size of the payload:
		memcpy(&buffer[byteNum], &msgLen, 4);
		byteNum += 4;

		int serial;
		memcpy(&serial, &buffer[6], 4);
		int msgSIZE;
		memcpy(&msgSIZE, &buffer[10], 4);

		// Finally, copy the message into the buffer.
		memcpy(&buffer[byteNum], msg, msgLen);
		byteNum += msgLen;

		///printf("Sending REL: %c%c%c %d %d %c%c%c\n", buffer[3],buffer[4],buffer[5],serial,msgSIZE,buffer[14],buffer[15],buffer[16]);

		// Point message.msg at buffer
		message.msg = buffer;

		// We can now push the message onto the outgoing queue for each robot in
		// range.
		Message* newMsg;
		if(fwdRecipients.empty())
			printf("sendMessage: No downstream robots in signal range\n");
		while(!fwdRecipients.empty())
		{
			newMsg = new Message;
			newMsg->msgLen = byteNum;
			newMsg->msg = new unsigned char[byteNum];
			memcpy(newMsg->msg, buffer, byteNum);
			///printf("Sending message to robot # %d\n",fwdRecipients.front());
			pthread_mutex_lock(&(msgOutBlock[(fwdRecipients.front())]));
			msgsOut[fwdRecipients.front()].push(*newMsg);
			pthread_mutex_unlock(&(msgOutBlock[(fwdRecipients.front())]));
			pthread_cond_signal(&(msgOutCond[(fwdRecipients.front())]));
			fwdRecipients.pop();
		}

		delete [] buffer;

		// Clean up memory.
		//delete [] buffer;
	}
}


// Image Server Code
ImgSvr::ImgSvr()
{
	imgType = 0;
	imgSize = 0;
	buffer = NULL;
}

ImgSvr::~ImgSvr()
{
	ClearBuffer(); 
}

void ImgSvr::ConnectImgSvr(string IP, int Port)
{
	struct sockaddr_in pin;
	struct hostent *server_host_name;
	server_host_name = gethostbyname(IP.c_str());
	bzero(&pin, sizeof(pin));
	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = htonl(INADDR_ANY);
	pin.sin_addr.s_addr = ((struct in_addr *)(server_host_name->h_addr))->s_addr;
	pin.sin_port = htons(Port);
	sock_fd = socket(AF_INET, SOCK_STREAM, 0);
	printf("Performing actual IMGSVR connection... ");
	connect(sock_fd, (sockaddr *)&pin, sizeof(pin));
	printf("Success!\n");

	printf("Now returning to calling function.\n");
	return;
}

void ImgSvr::Disconnect()
{
	close(sock_fd);
}

// BIG TODO: FLUSH the buffer on a fail.
void ImgSvr::ReadImgSvrImage()
{
	ClearBuffer();
	int temp = 0;	   // Temp storage for the image size.
	unsigned char byteTemp; // Temp storage for read bytes.
	bool readSuccess = true;
	//unsigned char * buffer;

	// Read in the image type
	if(read(sock_fd, &imgType, 1) == -1)
	{
		printf("Failed to read image type (1st byte).\n");
		return;
	}

	// Read in the image size, stored in the next 4 bytes of the array.
	for(int shiftAmt=24; shiftAmt>=0; shiftAmt-=8)
	{
		if(read(sock_fd, &temp, 1) == -1)	//If the read fails
		{
			readSuccess = false;
			break;
		}
		else	// Otherwise, temp now holds the value of the last-read byte.
		{
			imgSize += temp<<shiftAmt;
			temp = 0;
		}
	}

	//cout << "Image Type: " << imgType << "\n";
	//cout << "Image Size: " << imgSize << " bytes\n";

	if(!readSuccess)	// The read failed on the imgSize.
	{
		printf("Failed to read image size (bytes 2-5).\n");
		return;
	}

	//printf("Allocating buffer...\n");
	buffer = new unsigned char[imgSize];

	// Read in the whole image.
	int byteNum = 0;
	//printf("Reading image into buffer...\n");

	while(byteNum < imgSize)
	{
		read(sock_fd, &byteTemp, 1);
		buffer[byteNum] = byteTemp;
		byteNum++;

	}

	return;
}

void ImgSvr::SendOKImgSvr()
{
	// Send OK message to the image server

	char buf[3] = "OK";
	write(sock_fd, buf, 3);
}

void ImgSvr::ClearBuffer()
{
	if(buffer!=NULL)
		delete [] buffer;
	buffer = NULL;

	imgType = 0;
	imgSize = 0;

}

// Updates the robSS[] class variable with the current signal strength values
// reported by WSS.
int Communications::updateRobSS()
{
	//printf("Inside updateRobSS()\n");
	char RobotName[33];
	for(int robNum=0; robNum<ROBOTCOUNT; robNum++)
	{
		bzero(RobotName, 33);
		if(robNum==0) sprintf(RobotName, "CommStation");
		else sprintf(RobotName, "Robot%d", robNum);

		robSS[robNum] = GetSignalStrength(string(RobotName));
	}
	return 0;
}



