/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file mfdServoTest.hh

  \brief MOAST For Dummies example application. Designed to ease the learning curve for using MOAST.

  Shows how you can control the servo echelon of a vehicle with minimal code.

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.2 $
  $Date: 2009/04/27 16:09:39 $
  \endcode

  \author Stephen Balakirsky
  \date April 7, 2009
*/

#include <stdio.h>
#include <mfd.hh>
#include <unistd.h>

int main()
{
  Mfd *mfd = new Mfd();
  Waypoint waypoint;

  // initialize everything
  if( mfd->init(MFD_AM) != true )
    printf( "Error from init %s\n", mfd->mfdErrorStr());
  else
    printf( "Initialized am\n" );
  /*
  if( mfd->init(MFD_PRIM) != true )
    printf( "Error from init %s\n", mfd->mfdErrorStr());
  else
    printf( "Initialized prim\n" );
  if( mfd->init(MFD_SERVO) != true )
    printf( "Error from init %s\n", mfd->mfdErrorStr());
  else
    printf( "Initialized servo\n" );
  */
  if( mfd->init(MFD_FACTORY) != true )
    printf( "Error from init %s\n", mfd->mfdErrorStr());
  else
    printf( "Initialized factory\n" );

  // go to loading station
  waypoint.p.x = 13.87;
  waypoint.p.y = 27.32;
  waypoint.neighborhood = .1;
  waypoint.speed = .5;
  if( mfd->amMoveWaypoint(&waypoint, 1) != true )
    printf( "Error from am movement %s\n", mfd->mfdErrorStr());
  else
    printf( "Reached initial point\n" );

  return 1;
  if( mfd->primRotate( 178.76/180*3.1415 ) != true )
    printf( "Error from rotate %s\n", mfd->mfdErrorStr());
  else
    printf( "Rotated\n" );
  waypoint.p.x = 11.17;
  waypoint.p.y = 27.32;
  waypoint.neighborhood = .1;
  waypoint.speed = .5;
  if( mfd->primMoveWaypoint(&waypoint, 1) != true )
    printf( "Error from prim movement %s\n", mfd->mfdErrorStr());
  else
    printf( "Reached dock point\n" );
  
  // drop box
  if( mfd->factoryDropPackage("BoxChute1") != true )
    printf( "Error from init %s\n", mfd->mfdErrorStr());
  else
    printf( "Spawned box\n" );


  // load it
  if( mfd->factorySetConveyor("Loading", -6) != true )
    printf( "Error from init %s\n", mfd->mfdErrorStr());
  else
    printf( "set speed\n" );
  if( mfd->servoEffOpcode( 1, SERVO_EFF_JA_ANIMATE_OPCODE_TYPE, 3 ) != true )
    printf( "Error from effOpcode %s\n", mfd->mfdErrorStr());




  /*
  if( mfd->servoSkidSteer(1, 1) != true )
    printf( "Error from skid %s\n", mfd->mfdErrorStr());
  sleep(5);
  if( mfd->servoSkidSteer(0, 0) != true )
    printf( "Error from skid %s\n", mfd->mfdErrorStr());
  */
}
/*
  Modification history:

  $Log: mfdServoTest.cc,v $
  Revision 1.2  2009/04/27 16:09:39  dr_steveb
  Getting ready for demo...

  Revision 1.1  2009/04/24 18:45:17  dr_steveb
  Initial check-in of moast for dummies class

*/
