/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/

/*!
  \file moastNmlGlobal.cc
  \brief Global variables for all Moast NML buffers, and connect/disconnect
  functions

  \code CVS Status:
  $Author: proctor $
  $Revision: 1.8 $
  $Date: 2007/07/24 18:34:12 $
  \endcode

  \author Fred Proctor
*/

#include <stdio.h>		// sprintf(), FILE
#include "rcs.hh"		// NML
#include "moastTypes.hh"
#include "moastNmlOffsets.hh"
#include "moastError.hh"	// moastError_format()
#include "navDataExt.hh"	// navDataExt_format()
#include "servoMobJA.hh"	// servoMobJA_format()
#include "servoSP.hh"		// servoSP_format()
#include "servoMisJA.hh"	// servoMisJA_format()
#include "sensorData.hh"	// sensorData_format()
#include "primMobJA.hh"		// primMobJA_format()
#include "primMisJA.hh"		// primMisJA_format()
#include "primSP.hh"            // primSP_format()
#include "vehMobPL.hh"          // vehMobPL_format()
#include "sectMobPL.hh"         // sectMobPL_format()
#include "amMobJA.hh"		// amMobJA_format()
#include "amMisJA.hh"		// amMisJA_format()
#include "amSP.hh"		// amSP_format()
#include "moastNmlGlobal.hh"	// these decls
#ifdef USE_SPQR
#include "nmlMap.hh"
#include "nmlPosition.hh"
#include "joystickData.hh"

NML *spqrMapBufZerg = NULL;
NML *spqrMapBufP2DX = NULL;
NML *spqrPosBufMapper = NULL;
NML *spqrPosBufVictim = NULL;
#endif

NML *moastErrorBuf = NULL;

NML *symbolicDataBuf[NUM_SYMBOLIC_DATA_BUFFERS] = {NULL};

#if NUM_NAV_DATA_EXT_BUFFERS 
  NML *navDataExtBuf[NUM_NAV_DATA_EXT_BUFFERS] = {NULL};
#endif

#if NUM_SERVO_SP_LINESCAN_BUFFERS
  NML *servoSPLinescanBuf[NUM_SERVO_SP_LINESCAN_BUFFERS] = {NULL};
#endif

NML *primSPLinescanBuf[NUM_PRIM_SP_LINESCAN_BUFFERS] = {NULL};
NML *amSPMapBuf[NUM_AM_SP_MAP_BUFFERS] = {NULL};

#if NUM_SERVO_MOB_JA_BUFFERS 
  RCS_CMD_CHANNEL *servoMobJACmdBuf[NUM_SERVO_MOB_JA_BUFFERS] = {NULL};
  RCS_CMD_CHANNEL *servoMobJACfgBuf[NUM_SERVO_MOB_JA_BUFFERS] = {NULL};
  RCS_STAT_CHANNEL *servoMobJAStatBuf[NUM_SERVO_MOB_JA_BUFFERS] = {NULL};
  RCS_STAT_CHANNEL *servoMobJASetBuf[NUM_SERVO_MOB_JA_BUFFERS] = {NULL};
#endif

#if NUM_SERVO_SP_BUFFERS
  RCS_CMD_CHANNEL *servoSPCmdBuf[NUM_SERVO_SP_BUFFERS] = {NULL};
  RCS_STAT_CHANNEL *servoSPStatBuf[NUM_SERVO_SP_BUFFERS] = {NULL};
  RCS_STAT_CHANNEL *servoSPSetBuf[NUM_SERVO_SP_BUFFERS] = {NULL};
  RCS_CMD_CHANNEL *servoSPCfgBuf[NUM_SERVO_SP_BUFFERS] = {NULL};
#endif

#if NUM_SERVO_MIS_JA_BUFFERS
  RCS_CMD_CHANNEL *servoMisJACmdBuf[NUM_SERVO_MIS_JA_BUFFERS] = {NULL};
  RCS_STAT_CHANNEL *servoMisJAStatBuf[NUM_SERVO_MIS_JA_BUFFERS] = {NULL};
  RCS_STAT_CHANNEL *servoMisJASetBuf[NUM_SERVO_MIS_JA_BUFFERS] = {NULL};
  RCS_CMD_CHANNEL *servoMisJACfgBuf[NUM_SERVO_MIS_JA_BUFFERS] = {NULL};
#endif

RCS_CMD_CHANNEL *primMobJACmdBuf[NUM_PRIM_MOB_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *primMobJAStatBuf[NUM_PRIM_MOB_JA_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *primMobJACfgBuf[NUM_PRIM_MOB_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *primMobJASetBuf[NUM_PRIM_MOB_JA_BUFFERS] = {NULL};

RCS_CMD_CHANNEL *primMisJACmdBuf[NUM_PRIM_MIS_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *primMisJAStatBuf[NUM_PRIM_MIS_JA_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *primMisJACfgBuf[NUM_PRIM_MIS_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *primMisJASetBuf[NUM_PRIM_MIS_JA_BUFFERS] = {NULL};

RCS_CMD_CHANNEL *primSPCmdBuf[NUM_PRIM_SP_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *primSPStatBuf[NUM_PRIM_SP_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *primSPSetBuf[NUM_PRIM_SP_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *primSPCfgBuf[NUM_PRIM_SP_BUFFERS] = {NULL};

RCS_CMD_CHANNEL *amMobJACmdBuf[NUM_AM_MOB_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *amMobJAStatBuf[NUM_AM_MOB_JA_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *amMobJACfgBuf[NUM_AM_MOB_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *amMobJASetBuf[NUM_AM_MOB_JA_BUFFERS] = {NULL};

RCS_CMD_CHANNEL *amMisJACmdBuf[NUM_AM_MIS_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *amMisJAStatBuf[NUM_AM_MIS_JA_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *amMisJACfgBuf[NUM_AM_MIS_JA_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *amMisJASetBuf[NUM_AM_MIS_JA_BUFFERS] = {NULL};

RCS_CMD_CHANNEL *amSPCmdBuf[NUM_AM_SP_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *amSPStatBuf[NUM_AM_SP_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *amSPSetBuf[NUM_AM_SP_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *amSPCfgBuf[NUM_AM_SP_BUFFERS] = {NULL};

RCS_CMD_CHANNEL *vehMobPLCmdBuf[NUM_VEH_MOB_PL_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *vehMobPLStatBuf[NUM_VEH_MOB_PL_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *vehMobPLCfgBuf[NUM_VEH_MOB_PL_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *vehMobPLSetBuf[NUM_VEH_MOB_PL_BUFFERS] = {NULL};
NML *vehMapBuf[NUM_VEH_MOB_PL_BUFFERS] = {NULL};


RCS_CMD_CHANNEL *sectMobPLCmdBuf[NUM_SECT_MOB_PL_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *sectMobPLStatBuf[NUM_SECT_MOB_PL_BUFFERS] = {NULL};
RCS_CMD_CHANNEL *sectMobPLCfgBuf[NUM_SECT_MOB_PL_BUFFERS] = {NULL};
RCS_STAT_CHANNEL *sectMobPLSetBuf[NUM_SECT_MOB_PL_BUFFERS] = {NULL};

NML * joystickDataBuf = NULL;

int moastZeroNmlBuffers(void)
{
  int bufnum;

#ifdef USE_SPQR
  spqrMapBufP2DX = NULL;
  spqrMapBufZerg = NULL;
  spqrPosBufMapper = NULL;
  spqrPosBufVictim = NULL;
#endif

  moastErrorBuf = NULL;

  // initialize symbolicDaa buffers to NULL
  for (bufnum = 0; bufnum < NUM_SYMBOLIC_DATA_BUFFERS; bufnum++)
    symbolicDataBuf[bufnum] = NULL;

#if NUM_NAV_DATA_EXT_BUFFER
  // initialize navDataExtBuf buffers to NULL
  for (bufnum = 0; bufnum < NUM_NAV_DATA_EXT_BUFFERS; bufnum++)
    navDataExtBuf[bufnum] = NULL;
#endif

#if NUM_SERVO_SP_LINESCAN_BUFFERS
  // initialize SP data buffers to NULL
  for (bufnum = 0; bufnum < NUM_SERVO_SP_LINESCAN_BUFFERS; bufnum++)
    servoSPLinescanBuf[bufnum] = NULL;
#endif

  for (bufnum = 0; bufnum < NUM_PRIM_SP_LINESCAN_BUFFERS; bufnum++)
    primSPLinescanBuf[bufnum] = NULL;
  for (bufnum = 0; bufnum < NUM_AM_SP_MAP_BUFFERS; bufnum++)
    amSPMapBuf[bufnum] = NULL;

#if NUM_SERVO_MOB_JA_BUFFERS
  // initialize servoMobJA buffers to NULL
  for (bufnum = 0; bufnum < NUM_SERVO_MOB_JA_BUFFERS; bufnum++) {
    servoMobJAStatBuf[bufnum] = NULL;
    servoMobJACmdBuf[bufnum] = NULL;
    servoMobJASetBuf[bufnum] = NULL;
    servoMobJACfgBuf[bufnum] = NULL;
  }
#endif

#if NUM_SERVO_SP_BUFFERS
  // initialize servoSP buffers to NULL
  for (bufnum = 0; bufnum < NUM_SERVO_SP_BUFFERS; bufnum++) {
    servoSPStatBuf[bufnum] = NULL;
    servoSPCmdBuf[bufnum] = NULL;
    servoSPSetBuf[bufnum] = NULL;
    servoSPCfgBuf[bufnum] = NULL;
  }
#endif

#if NUM_SERVO_MIS_JA_BUFFERS
  // initialize servoMisJA buffers to NULL
  for (bufnum = 0; bufnum < NUM_SERVO_MIS_JA_BUFFERS; bufnum++) {
    servoMisJAStatBuf[bufnum] = NULL;
    servoMisJACmdBuf[bufnum] = NULL;
    servoMisJASetBuf[bufnum] = NULL;
    servoMisJACfgBuf[bufnum] = NULL;
  }
#endif

  // initialize primMobJA buffers to NULL
  for (bufnum = 0; bufnum < NUM_PRIM_MOB_JA_BUFFERS; bufnum++) {
    primMobJAStatBuf[bufnum] = NULL;
    primMobJACmdBuf[bufnum] = NULL;
    primMobJASetBuf[bufnum] = NULL;
    primMobJACfgBuf[bufnum] = NULL; // TK added 6/10/5
  }

  // initialize primMisJA buffers to NULL
  for (bufnum = 0; bufnum < NUM_PRIM_MIS_JA_BUFFERS; bufnum++) {
    primMisJAStatBuf[bufnum] = NULL;
    primMisJACmdBuf[bufnum] = NULL;
    primMisJASetBuf[bufnum] = NULL;
    primMisJACfgBuf[bufnum] = NULL; // TK added 6/10/5
  }

  // initialize amMobJA buffers to NULL
  for (bufnum = 0; bufnum < NUM_AM_MOB_JA_BUFFERS; bufnum++) {
    amMobJAStatBuf[bufnum] = NULL;
    amMobJACmdBuf[bufnum] = NULL;
    amMobJASetBuf[bufnum] = NULL;
    amMobJACfgBuf[bufnum] = NULL;
  }

  // initialize amMisJA buffers to NULL
  for (bufnum = 0; bufnum < NUM_AM_MIS_JA_BUFFERS; bufnum++) {
    amMisJAStatBuf[bufnum] = NULL;
    amMisJACmdBuf[bufnum] = NULL;
    amMisJASetBuf[bufnum] = NULL;
    amMisJACfgBuf[bufnum] = NULL;
  }

  // initialize amSP buffers to NULL
  for (bufnum = 0; bufnum < NUM_AM_SP_BUFFERS; bufnum++) {
    amSPStatBuf[bufnum] = NULL;
    amSPCmdBuf[bufnum] = NULL;
    amSPSetBuf[bufnum] = NULL;
    amSPCfgBuf[bufnum] = NULL;
  }

  // initialize primSP buffers to NULL
  for (bufnum = 0; bufnum < NUM_PRIM_SP_BUFFERS; bufnum++) {
    primSPStatBuf[bufnum] = NULL;
    primSPCmdBuf[bufnum] = NULL;
    primSPSetBuf[bufnum] = NULL;
    primSPCfgBuf[bufnum] = NULL;
  }

  // initialize vehMobPL buffers to NULL
  for (bufnum = 0; bufnum < NUM_VEH_MOB_PL_BUFFERS; bufnum++) {
    vehMobPLStatBuf[bufnum] = NULL;
    vehMobPLCmdBuf[bufnum] = NULL;
    vehMobPLSetBuf[bufnum] = NULL;
    vehMobPLCfgBuf[bufnum] = NULL; // TK added 6/10/5
    vehMapBuf[bufnum] = NULL;
  }

  // initialize sectMobPL buffers to NULL
  for (bufnum = 0; bufnum < NUM_SECT_MOB_PL_BUFFERS; bufnum++) {
    sectMobPLStatBuf[bufnum] = NULL;
    sectMobPLCmdBuf[bufnum] = NULL;
    sectMobPLSetBuf[bufnum] = NULL;
    sectMobPLCfgBuf[bufnum] = NULL; // TK added 6/10/5
  }

  joystickDataBuf = NULL;

  return 0;
}

int moastCleanupNmlBuffers(void)
{
  int count;

#ifdef USE_SPQR
NML *spqrMapBufP2DX = NULL;
NML *spqrMapBufZerg = NULL;
NML *spqrPosBufMapper = NULL;
NML *spqrPosBufVictim = NULL;

  if (NULL != spqrMapBufP2DX) {
    delete spqrMapBufP2DX;
    spqrMapBufP2DX = 0;
  }
  if (NULL != spqrMapBufZerg) {
    delete spqrMapBufZerg;
    spqrMapBufZerg = 0;
  }
  if (NULL != spqrPosBufMapper) {
    delete spqrPosBufMapper;
    spqrPosBufMapper = 0;
  }
  if (NULL != spqrPosBufVictim) {
    delete spqrPosBufVictim;
    spqrPosBufVictim = 0;
  }
#endif

  if (NULL != moastErrorBuf) {
    delete moastErrorBuf;
    moastErrorBuf = 0;
  }

  for (count = 0; count < NUM_SYMBOLIC_DATA_BUFFERS; count++) {
    if (NULL != symbolicDataBuf[count])
      delete symbolicDataBuf[count];
    symbolicDataBuf[count] = NULL;
  }

#if NUM_NAV_DATA_EXT_BUFFERS
  for (count = 0; count < NUM_NAV_DATA_EXT_BUFFERS; count++) {
    if (NULL != navDataExtBuf[count])
      delete navDataExtBuf[count];
    navDataExtBuf[count] = NULL;
  }
#endif

#if  NUM_SERVO_SP_LINESCAN_BUFFERS
  for (count = 0; count < NUM_SERVO_SP_LINESCAN_BUFFERS; count++) {
    if (NULL != servoSPLinescanBuf[count])
      delete servoSPLinescanBuf[count];
    servoSPLinescanBuf[count] = NULL;
  }
#endif

  for (count = 0; count < NUM_PRIM_SP_LINESCAN_BUFFERS; count++) {
    if (NULL != primSPLinescanBuf[count])
      delete primSPLinescanBuf[count];
    primSPLinescanBuf[count] = NULL;
  }

  for (count = 0; count < NUM_AM_SP_MAP_BUFFERS; count++) {
    if (NULL != amSPMapBuf[count])
      delete amSPMapBuf[count];
    amSPMapBuf[count] = NULL;
  }

#if NUM_SERVO_MOB_JA_BUFFERS
  for (count = 0; count < NUM_SERVO_MOB_JA_BUFFERS; count++) {
    if (NULL != servoMobJAStatBuf[count])
      delete servoMobJAStatBuf[count];
    servoMobJAStatBuf[count] = 0;
    if (NULL != servoMobJACmdBuf[count])
      delete servoMobJACmdBuf[count];
    servoMobJACmdBuf[count] = 0;
    if (NULL != servoMobJASetBuf[count])
      delete servoMobJASetBuf[count];
    servoMobJASetBuf[count] = 0;
    if (NULL != servoMobJACfgBuf[count])
      delete servoMobJACfgBuf[count];
    servoMobJACfgBuf[count] = 0;
  }
#endif

#if NUM_SERVO_SP_BUFFERS
  for (count = 0; count < NUM_SERVO_SP_BUFFERS; count++) {
    if (NULL != servoSPStatBuf[count])
      delete servoSPStatBuf[count];
    servoSPStatBuf[count] = 0;
    if (NULL != servoSPCmdBuf[count])
      delete servoSPCmdBuf[count];
    servoSPCmdBuf[count] = 0;
    if (NULL != servoSPSetBuf[count])
      delete servoSPSetBuf[count];
    servoSPSetBuf[count] = 0;
    if (NULL != servoSPCfgBuf[count])
      delete servoSPCfgBuf[count];
    servoSPCfgBuf[count] = 0;
  }
#endif

#if NUM_SERVO_MIS_JA_BUFFERS
  for (count = 0; count < NUM_SERVO_MIS_JA_BUFFERS; count++) {
    if (NULL != servoMisJAStatBuf[count])
      delete servoMisJAStatBuf[count];
    servoMisJAStatBuf[count] = 0;
    if (NULL != servoMisJACmdBuf[count])
      delete servoMisJACmdBuf[count];
    servoMisJACmdBuf[count] = 0;
    if (NULL != servoMisJASetBuf[count])
      delete servoMisJASetBuf[count];
    servoMisJASetBuf[count] = 0;
    if (NULL != servoMisJACfgBuf[count])
      delete servoMisJACfgBuf[count];
    servoMisJACfgBuf[count] = 0;
  }
#endif

  for (count = 0; count < NUM_PRIM_MOB_JA_BUFFERS; count++) {
    if (NULL != primMobJAStatBuf[count])
      delete primMobJAStatBuf[count];
    primMobJAStatBuf[count] = 0;
    if (NULL != primMobJACmdBuf[count])
      delete primMobJACmdBuf[count];
    primMobJACmdBuf[count] = 0;
    if (NULL != primMobJACfgBuf[count])
      delete primMobJACfgBuf[count];
    primMobJACfgBuf[count] = 0;
    if (NULL != primMobJASetBuf[count])
      delete primMobJASetBuf[count];
    primMobJASetBuf[count] = 0;
  }

  for (count = 0; count < NUM_PRIM_MIS_JA_BUFFERS; count++) {
    if (NULL != primMisJAStatBuf[count])
      delete primMisJAStatBuf[count];
    primMisJAStatBuf[count] = 0;
    if (NULL != primMisJACmdBuf[count])
      delete primMisJACmdBuf[count];
    primMisJACmdBuf[count] = 0;
    if (NULL != primMisJACfgBuf[count])
      delete primMisJACfgBuf[count];
    primMisJACfgBuf[count] = 0;
    if (NULL != primMisJASetBuf[count])
      delete primMisJASetBuf[count];
    primMisJASetBuf[count] = 0;
  }

  for (count = 0; count < NUM_PRIM_SP_BUFFERS; count++) {
    if (NULL != primSPStatBuf[count])
      delete primSPStatBuf[count];
    primSPStatBuf[count] = 0;
    if (NULL != primSPCmdBuf[count])
      delete primSPCmdBuf[count];
    primSPCmdBuf[count] = 0;
    if (NULL != primSPSetBuf[count])
      delete primSPSetBuf[count];
    primSPSetBuf[count] = 0;
    if (NULL != primSPCfgBuf[count])
      delete primSPCfgBuf[count];
    primSPCfgBuf[count] = 0;
  }


  for (count = 0; count < NUM_AM_MOB_JA_BUFFERS; count++) {
    if (NULL != amMobJAStatBuf[count])
      delete amMobJAStatBuf[count];
    amMobJAStatBuf[count] = 0;
    if (NULL != amMobJACmdBuf[count])
      delete amMobJACmdBuf[count];
    amMobJACmdBuf[count] = 0;
    if (NULL != amMobJACfgBuf[count])
      delete amMobJACfgBuf[count];
    amMobJACfgBuf[count] = 0;
    if (NULL != amMobJASetBuf[count])
      delete amMobJASetBuf[count];
    amMobJASetBuf[count] = 0;
  }

  for (count = 0; count < NUM_AM_MIS_JA_BUFFERS; count++) {
    if (NULL != amMisJAStatBuf[count])
      delete amMisJAStatBuf[count];
    amMisJAStatBuf[count] = 0;
    if (NULL != amMisJACmdBuf[count])
      delete amMisJACmdBuf[count];
    amMisJACmdBuf[count] = 0;
    if (NULL != amMisJACfgBuf[count])
      delete amMisJACfgBuf[count];
    amMisJACfgBuf[count] = 0;
    if (NULL != amMisJASetBuf[count])
      delete amMisJASetBuf[count];
    amMisJASetBuf[count] = 0;
  }

  for (count = 0; count < NUM_AM_SP_BUFFERS; count++) {
    if (NULL != amSPStatBuf[count])
      delete amSPStatBuf[count];
    amSPStatBuf[count] = 0;
    if (NULL != amSPCmdBuf[count])
      delete amSPCmdBuf[count];
    amSPCmdBuf[count] = 0;
    if (NULL != amSPCfgBuf[count])
      delete amSPCfgBuf[count];
    amSPCfgBuf[count] = 0;
    if (NULL != amSPSetBuf[count])
      delete amSPSetBuf[count];
    amSPSetBuf[count] = 0;
  }

  for (count = 0; count < NUM_VEH_MOB_PL_BUFFERS; count++) {
    if (NULL != vehMobPLStatBuf[count])
      delete vehMobPLStatBuf[count];
    vehMobPLStatBuf[count] = 0;
    if (NULL != vehMobPLCmdBuf[count])
      delete vehMobPLCmdBuf[count];
    vehMobPLCmdBuf[count] = 0;
    if (NULL != vehMobPLCfgBuf[count])
      delete vehMobPLCfgBuf[count];
    vehMobPLCfgBuf[count] = 0;
    if (NULL != vehMobPLSetBuf[count])
      delete vehMobPLSetBuf[count];
    vehMobPLSetBuf[count] = 0;
    if (NULL != vehMapBuf[count])
      delete vehMapBuf[count];
    vehMapBuf[count] = 0;
  }

  for (count = 0; count < NUM_SECT_MOB_PL_BUFFERS; count++) {
    if (NULL != sectMobPLStatBuf[count])
      delete sectMobPLStatBuf[count];
    sectMobPLStatBuf[count] = 0;
    if (NULL != sectMobPLCmdBuf[count])
      delete sectMobPLCmdBuf[count];
    sectMobPLCmdBuf[count] = 0;
    if (NULL != sectMobPLCfgBuf[count])
      delete sectMobPLCfgBuf[count];
    sectMobPLCfgBuf[count] = 0;
    if (NULL != sectMobPLSetBuf[count])
      delete sectMobPLSetBuf[count];
    sectMobPLSetBuf[count] = 0;
  }

  if (NULL != joystickDataBuf) {
    delete joystickDataBuf;
    joystickDataBuf = NULL;
  }

  return 0;
}

int moastInitNmlBuffers(char *config_file, char *this_process)
{
  int count;
  char chanName[MOAST_NML_BUFFER_NAME_LEN];

  (void) moastCleanupNmlBuffers();

#ifdef USE_SPQR
  spqrMapBufZerg =
    new NML(nmlMap_format, "MapBufZerg", this_process, config_file);
  if (!spqrMapBufZerg->valid() &&
      spqrMapBufZerg->error_type != NML_INVALID_CONFIGURATION) {
    delete spqrMapBufZerg;
    spqrMapBufZerg = 0;
    return 1;
  }
  spqrMapBufP2DX =
    new NML(nmlMap_format, "MapBufP2DX", this_process, config_file);
  if (!spqrMapBufP2DX->valid() &&
      spqrMapBufP2DX->error_type != NML_INVALID_CONFIGURATION) {
    delete spqrMapBufP2DX;
    spqrMapBufP2DX = 0;
    return 1;
  }
  spqrPosBufVictim =
    new NML(nmlPosition_format, NML_POS_VICTIM_NAME, this_process, config_file);
  if (!spqrPosBufVictim->valid() &&
      spqrPosBufVictim->error_type != NML_INVALID_CONFIGURATION) {
    delete spqrPosBufVictim;
    spqrPosBufVictim = 0;
    return 1;
  }
  spqrPosBufMapper =
    new NML(nmlPosition_format, NML_POS_MAPPER_NAME, this_process, config_file);
  if (!spqrPosBufMapper->valid() &&
      spqrPosBufMapper->error_type != NML_INVALID_CONFIGURATION) {
    delete spqrPosBufMapper;
    spqrPosBufMapper = 0;
    return 1;
  }
#endif
  moastErrorBuf =
    new NML(moastError_format, "moastError", this_process, config_file);
  if (!moastErrorBuf->valid() &&
      moastErrorBuf->error_type != NML_INVALID_CONFIGURATION) {
    delete moastErrorBuf;
    moastErrorBuf = 0;
    return 1;
  }

  for (count = 0; count < NUM_SYMBOLIC_DATA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SYMBOLIC_DATA_NAME, count+1);
    symbolicDataBuf[count] =
      new NML(sensorData_format, chanName, this_process, config_file);
    if (!symbolicDataBuf[count]->valid() &&
	symbolicDataBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete symbolicDataBuf[count];
      symbolicDataBuf[count] = 0;
      return 1;
    }
  }

#if NUM_NAV_DATA_EXT_BUFFERS
  for (count = 0; count < NUM_NAV_DATA_EXT_BUFFERS; count++) {
    sprintf(chanName, "%s%d", NAV_DATA_EXT_NAME, count+1);
    navDataExtBuf[count] =
      new NML(navDataExt_format, chanName, this_process, config_file);
    if (!navDataExtBuf[count]->valid() &&
	navDataExtBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete navDataExtBuf[count];
      navDataExtBuf[count] = 0;
      return 1;
    }
  }
#endif

#if NUM_SERVO_SP_LINESCAN_BUFFERS
  for (count = 0; count <  NUM_SERVO_SP_LINESCAN_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_SP_LINESCAN_NAME, count+1);
    servoSPLinescanBuf[count] =
      new NML(sensorData_format, chanName, this_process, config_file);
    if (!servoSPLinescanBuf[count]->valid() &&
	servoSPLinescanBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoSPLinescanBuf[count];
      servoSPLinescanBuf[count] = 0;
      return 1;
    }
  }
#endif

  for (count = 0; count < NUM_PRIM_SP_LINESCAN_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_SP_LINESCAN_NAME, count+1);
    primSPLinescanBuf[count] =
      new NML(sensorData_format, chanName, this_process, config_file);
    if (!primSPLinescanBuf[count]->valid() &&
	primSPLinescanBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primSPLinescanBuf[count];
      primSPLinescanBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_SP_MAP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_SP_MAP_NAME, count+1);
    amSPMapBuf[count] =
      new NML(sensorData_format, chanName, this_process, config_file);
    if (!amSPMapBuf[count]->valid() &&
	amSPMapBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amSPMapBuf[count];
      amSPMapBuf[count] = 0;
      return 1;
    }
  }

  /*
    Create Servo JA buffers for cmd, stat, cfg and set.
  */
#if NUM_SERVO_MOB_JA_BUFFERS
  for (count = 0; count < NUM_SERVO_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MOB_JA_CMD_NAME, count+1);
    servoMobJACmdBuf[count] =
      new RCS_CMD_CHANNEL(servoMobJA_format, chanName,
			  this_process, config_file);
    if (!servoMobJACmdBuf[count]->valid() &&
	servoMobJACmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMobJACmdBuf[count];
      servoMobJACmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MOB_JA_STAT_NAME, count+1);
    servoMobJAStatBuf[count] =
      new RCS_STAT_CHANNEL(servoMobJA_format, chanName,
			  this_process, config_file);
    if (!servoMobJAStatBuf[count]->valid() &&
	servoMobJAStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMobJAStatBuf[count];
      servoMobJAStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MOB_JA_CFG_NAME, count+1);
    servoMobJACfgBuf[count] =
      new RCS_CMD_CHANNEL(servoMobJA_format, chanName,
			  this_process, config_file);
    if (!servoMobJACfgBuf[count]->valid() &&
	servoMobJACfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMobJACfgBuf[count];
      servoMobJACfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MOB_JA_SET_NAME, count+1);
    servoMobJASetBuf[count] =
      new RCS_STAT_CHANNEL(servoMobJA_format, chanName,
			  this_process, config_file);
    if (!servoMobJASetBuf[count]->valid() &&
	servoMobJASetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMobJASetBuf[count];
      servoMobJASetBuf[count] = 0;
      return 1;
    }
  }

#endif


  /*
    Create Servo SP buffers for cmd, stat, cfg and set.
  */
#if NUM_SERVO_SP_BUFFERS
  for (count = 0; count < NUM_SERVO_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_SP_CMD_NAME, count+1);
    servoSPCmdBuf[count] =
      new RCS_CMD_CHANNEL(servoSP_format, chanName,
			  this_process, config_file);
    if (!servoSPCmdBuf[count]->valid() &&
	servoSPCmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoSPCmdBuf[count];
      servoSPCmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_SP_STAT_NAME, count+1);
    servoSPStatBuf[count] =
      new RCS_STAT_CHANNEL(servoSP_format, chanName,
			  this_process, config_file);
    if (!servoSPStatBuf[count]->valid() &&
	servoSPStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoSPStatBuf[count];
      servoSPStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_SP_CFG_NAME, count+1);
    servoSPCfgBuf[count] =
      new RCS_CMD_CHANNEL(servoSP_format, chanName,
			  this_process, config_file);
    if (!servoSPCfgBuf[count]->valid() &&
	servoSPCfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoSPCfgBuf[count];
      servoSPCfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_SP_SET_NAME, count+1);
    servoSPSetBuf[count] =
      new RCS_STAT_CHANNEL(servoSP_format, chanName,
			  this_process, config_file);
    if (!servoSPSetBuf[count]->valid() &&
	servoSPSetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoSPSetBuf[count];
      servoSPSetBuf[count] = 0;
      return 1;
    }
  }
#endif


  /*
    Create Servo Mission buffers for cmd, stat, cfg and set.
  */
#if NUM_SERVO_MIS_JA_BUFFERS
  for (count = 0; count < NUM_SERVO_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MIS_JA_CMD_NAME, count+1);
    servoMisJACmdBuf[count] =
      new RCS_CMD_CHANNEL(servoMisJA_format, chanName,
			  this_process, config_file);
    if (!servoMisJACmdBuf[count]->valid() &&
	servoMisJACmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMisJACmdBuf[count];
      servoMisJACmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MIS_JA_STAT_NAME, count+1);
    servoMisJAStatBuf[count] =
      new RCS_STAT_CHANNEL(servoMisJA_format, chanName,
			  this_process, config_file);
    if (!servoMisJAStatBuf[count]->valid() &&
	servoMisJAStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMisJAStatBuf[count];
      servoMisJAStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MIS_JA_CFG_NAME, count+1);
    servoMisJACfgBuf[count] =
      new RCS_CMD_CHANNEL(servoMisJA_format, chanName,
			  this_process, config_file);
    if (!servoMisJACfgBuf[count]->valid() &&
	servoMisJACfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMisJACfgBuf[count];
      servoMisJACfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SERVO_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SERVO_MIS_JA_SET_NAME, count+1);
    servoMisJASetBuf[count] =
      new RCS_STAT_CHANNEL(servoMisJA_format, chanName,
			  this_process, config_file);
    if (!servoMisJASetBuf[count]->valid() &&
	servoMisJASetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete servoMisJASetBuf[count];
      servoMisJASetBuf[count] = 0;
      return 1;
    }
  }
#endif
  /*
    Create Prim PL buffers for cmd, stat, cfg and set.
  */

  for (count = 0; count < NUM_PRIM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MOB_JA_CMD_NAME, count+1);
    primMobJACmdBuf[count] =
      new RCS_CMD_CHANNEL(primMobJA_format, chanName,
			  this_process, config_file);
    if (!primMobJACmdBuf[count]->valid() &&
	primMobJACmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMobJACmdBuf[count];
      primMobJACmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MOB_JA_STAT_NAME, count+1);
    primMobJAStatBuf[count] =
      new RCS_STAT_CHANNEL(primMobJA_format, chanName,
			  this_process, config_file);
    if (!primMobJAStatBuf[count]->valid() &&
	primMobJAStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMobJAStatBuf[count];
      primMobJAStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MOB_JA_CFG_NAME, count+1);
    primMobJACfgBuf[count] =
      new RCS_CMD_CHANNEL(primMobJA_format, chanName,
			  this_process, config_file);
    if (!primMobJACfgBuf[count]->valid() &&
	primMobJACfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMobJACfgBuf[count];
      primMobJACfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MOB_JA_SET_NAME, count+1);
    primMobJASetBuf[count] =
      new RCS_STAT_CHANNEL(primMobJA_format, chanName,
			  this_process, config_file);
    if (!primMobJASetBuf[count]->valid() &&
	primMobJASetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMobJASetBuf[count];
      primMobJASetBuf[count] = 0;
      return 1;
    }
  }

  /*
    Create Prim Mission buffers for cmd, stat, cfg and set.
  */

  for (count = 0; count < NUM_PRIM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MIS_JA_CMD_NAME, count+1);
    primMisJACmdBuf[count] =
      new RCS_CMD_CHANNEL(primMisJA_format, chanName,
			  this_process, config_file);
    if (!primMisJACmdBuf[count]->valid() &&
	primMisJACmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMisJACmdBuf[count];
      primMisJACmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MIS_JA_STAT_NAME, count+1);
    primMisJAStatBuf[count] =
      new RCS_STAT_CHANNEL(primMisJA_format, chanName,
			  this_process, config_file);
    if (!primMisJAStatBuf[count]->valid() &&
	primMisJAStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMisJAStatBuf[count];
      primMisJAStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MIS_JA_CFG_NAME, count+1);
    primMisJACfgBuf[count] =
      new RCS_CMD_CHANNEL(primMisJA_format, chanName,
			  this_process, config_file);
    if (!primMisJACfgBuf[count]->valid() &&
	primMisJACfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMisJACfgBuf[count];
      primMisJACfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_MIS_JA_SET_NAME, count+1);
    primMisJASetBuf[count] =
      new RCS_STAT_CHANNEL(primMisJA_format, chanName,
			  this_process, config_file);
    if (!primMisJASetBuf[count]->valid() &&
	primMisJASetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primMisJASetBuf[count];
      primMisJASetBuf[count] = 0;
      return 1;
    }
  }

   /*
    Create Am PL buffers for cmd, stat, cfg and set.
  */

  for (count = 0; count < NUM_AM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MOB_JA_CMD_NAME, count+1);
    amMobJACmdBuf[count] =
      new RCS_CMD_CHANNEL(amMobJA_format, chanName,
			  this_process, config_file);
    if (!amMobJACmdBuf[count]->valid() &&
	amMobJACmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMobJACmdBuf[count];
      amMobJACmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MOB_JA_STAT_NAME, count+1);
    amMobJAStatBuf[count] =
      new RCS_STAT_CHANNEL(amMobJA_format, chanName,
			  this_process, config_file);
    if (!amMobJAStatBuf[count]->valid() &&
	amMobJAStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMobJAStatBuf[count];
      amMobJAStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MOB_JA_CFG_NAME, count+1);
    amMobJACfgBuf[count] =
      new RCS_CMD_CHANNEL(amMobJA_format, chanName,
			  this_process, config_file);
    if (!amMobJACfgBuf[count]->valid() &&
	amMobJACfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMobJACfgBuf[count];
      amMobJACfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_MOB_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MOB_JA_SET_NAME, count+1);
    amMobJASetBuf[count] =
      new RCS_STAT_CHANNEL(amMobJA_format, chanName,
			  this_process, config_file);
    if (!amMobJASetBuf[count]->valid() &&
	amMobJASetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMobJASetBuf[count];
      amMobJASetBuf[count] = 0;
      return 1;
    }
  }

  /*
    Create Prim SP buffers for cmd, stat, cfg and set.
  */

  for (count = 0; count < NUM_PRIM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_SP_CMD_NAME, count+1);
    primSPCmdBuf[count] =
      new RCS_CMD_CHANNEL(primSP_format, chanName,
			  this_process, config_file);
    if (!primSPCmdBuf[count]->valid() &&
	primSPCmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primSPCmdBuf[count];
      primSPCmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_SP_STAT_NAME, count+1);
    primSPStatBuf[count] =
      new RCS_STAT_CHANNEL(primSP_format, chanName,
			  this_process, config_file);
    if (!primSPStatBuf[count]->valid() &&
	primSPStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primSPStatBuf[count];
      primSPStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_SP_CFG_NAME, count+1);
    primSPCfgBuf[count] =
      new RCS_CMD_CHANNEL(primSP_format, chanName,
			  this_process, config_file);
    if (!primSPCfgBuf[count]->valid() &&
	primSPCfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primSPCfgBuf[count];
      primSPCfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_PRIM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", PRIM_SP_SET_NAME, count+1);
    primSPSetBuf[count] =
      new RCS_STAT_CHANNEL(primSP_format, chanName,
			  this_process, config_file);
    if (!primSPSetBuf[count]->valid() &&
	primSPSetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete primSPSetBuf[count];
      primSPSetBuf[count] = 0;
      return 1;
    }
  }



  /*
    Create Am Mission buffers for cmd, stat, cfg and set.
  */

  for (count = 0; count < NUM_AM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MIS_JA_CMD_NAME, count+1);
    amMisJACmdBuf[count] =
      new RCS_CMD_CHANNEL(amMisJA_format, chanName,
			  this_process, config_file);
    if (!amMisJACmdBuf[count]->valid() &&
	amMisJACmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMisJACmdBuf[count];
      amMisJACmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MIS_JA_STAT_NAME, count+1);
    amMisJAStatBuf[count] =
      new RCS_STAT_CHANNEL(amMisJA_format, chanName,
			  this_process, config_file);
    if (!amMisJAStatBuf[count]->valid() &&
	amMisJAStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMisJAStatBuf[count];
      amMisJAStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MIS_JA_CFG_NAME, count+1);
    amMisJACfgBuf[count] =
      new RCS_CMD_CHANNEL(amMisJA_format, chanName,
			  this_process, config_file);
    if (!amMisJACfgBuf[count]->valid() &&
	amMisJACfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMisJACfgBuf[count];
      amMisJACfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_MIS_JA_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_MIS_JA_SET_NAME, count+1);
    amMisJASetBuf[count] =
      new RCS_STAT_CHANNEL(amMisJA_format, chanName,
			  this_process, config_file);
    if (!amMisJASetBuf[count]->valid() &&
	amMisJASetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amMisJASetBuf[count];
      amMisJASetBuf[count] = 0;
      return 1;
    }
  }

  /*
    Create Am SP buffers for cmd, stat, cfg and set.
  */

  for (count = 0; count < NUM_AM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_SP_CMD_NAME, count+1);
    amSPCmdBuf[count] =
      new RCS_CMD_CHANNEL(amSP_format, chanName,
			  this_process, config_file);
    if (!amSPCmdBuf[count]->valid() &&
	amSPCmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amSPCmdBuf[count];
      amSPCmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_SP_STAT_NAME, count+1);
    amSPStatBuf[count] =
      new RCS_STAT_CHANNEL(amSP_format, chanName,
			  this_process, config_file);
    if (!amSPStatBuf[count]->valid() &&
	amSPStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amSPStatBuf[count];
      amSPStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_SP_CFG_NAME, count+1);
    amSPCfgBuf[count] =
      new RCS_CMD_CHANNEL(amSP_format, chanName,
			  this_process, config_file);
    if (!amSPCfgBuf[count]->valid() &&
	amSPCfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amSPCfgBuf[count];
      amSPCfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_AM_SP_BUFFERS; count++) {
    sprintf(chanName, "%s%d", AM_SP_SET_NAME, count+1);
    amSPSetBuf[count] =
      new RCS_STAT_CHANNEL(amSP_format, chanName,
			  this_process, config_file);
    if (!amSPSetBuf[count]->valid() &&
	amSPSetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete amSPSetBuf[count];
      amSPSetBuf[count] = 0;
      return 1;
    }
  }

 /*
    Create Veh PL buffers for cmd, stat, cfg and set. 
  */
  for (count = 0; count < NUM_VEH_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", VEH_MOB_MAP_NAME, count+1);
    vehMapBuf[count] =
      new NML(sensorData_format, chanName,
			  this_process, config_file);
    if (!vehMapBuf[count]->valid() &&
	vehMapBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete vehMapBuf[count];
      vehMapBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_VEH_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", VEH_MOB_PL_CMD_NAME, count+1);
    vehMobPLCmdBuf[count] =
      new RCS_CMD_CHANNEL(vehMobPL_format, chanName,
			  this_process, config_file);
    if (!vehMobPLCmdBuf[count]->valid() &&
	vehMobPLCmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete vehMobPLCmdBuf[count];
      vehMobPLCmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_VEH_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", VEH_MOB_PL_STAT_NAME, count+1);
    vehMobPLStatBuf[count] =
      new RCS_STAT_CHANNEL(vehMobPL_format, chanName,
			  this_process, config_file);
    if (!vehMobPLStatBuf[count]->valid() &&
	vehMobPLStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete vehMobPLStatBuf[count];
      vehMobPLStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_VEH_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", VEH_MOB_PL_CFG_NAME, count+1);
    vehMobPLCfgBuf[count] =
      new RCS_CMD_CHANNEL(vehMobPL_format, chanName,
			  this_process, config_file);
    if (!vehMobPLCfgBuf[count]->valid() &&
	vehMobPLCfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete vehMobPLCfgBuf[count];
      vehMobPLCfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_VEH_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", VEH_MOB_PL_SET_NAME, count+1);
    vehMobPLSetBuf[count] =
      new RCS_STAT_CHANNEL(vehMobPL_format, chanName,
			  this_process, config_file);
    if (!vehMobPLSetBuf[count]->valid() &&
	vehMobPLSetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete vehMobPLSetBuf[count];
      vehMobPLSetBuf[count] = 0;
      return 1;
    }
  }

 /*
    Create Sect PL buffers for cmd, stat, cfg and set. 
  */

  for (count = 0; count < NUM_SECT_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SECT_MOB_PL_CMD_NAME, count+1);
    sectMobPLCmdBuf[count] =
      new RCS_CMD_CHANNEL(sectMobPL_format, chanName,
			  this_process, config_file);
    if (!sectMobPLCmdBuf[count]->valid() &&
	sectMobPLCmdBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete sectMobPLCmdBuf[count];
      sectMobPLCmdBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SECT_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SECT_MOB_PL_STAT_NAME, count+1);
    sectMobPLStatBuf[count] =
      new RCS_STAT_CHANNEL(sectMobPL_format, chanName,
			  this_process, config_file);
    if (!sectMobPLStatBuf[count]->valid() &&
	sectMobPLStatBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete sectMobPLStatBuf[count];
      sectMobPLStatBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SECT_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SECT_MOB_PL_CFG_NAME, count+1);
    sectMobPLCfgBuf[count] =
      new RCS_CMD_CHANNEL(sectMobPL_format, chanName,
			  this_process, config_file);
    if (!sectMobPLCfgBuf[count]->valid() &&
	sectMobPLCfgBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete sectMobPLCfgBuf[count];
      sectMobPLCfgBuf[count] = 0;
      return 1;
    }
  }

  for (count = 0; count < NUM_SECT_MOB_PL_BUFFERS; count++) {
    sprintf(chanName, "%s%d", SECT_MOB_PL_SET_NAME, count+1);
    sectMobPLSetBuf[count] =
      new RCS_STAT_CHANNEL(sectMobPL_format, chanName,
			  this_process, config_file);
    if (!sectMobPLSetBuf[count]->valid() &&
	sectMobPLSetBuf[count]->error_type != NML_INVALID_CONFIGURATION) {
      delete sectMobPLSetBuf[count];
      sectMobPLSetBuf[count] = 0;
      return 1;
    }
  }

  joystickDataBuf =  new NML(joystickData_format, "joystickData",
			     this_process, config_file);
  if (! joystickDataBuf->valid() &&
      joystickDataBuf->error_type != NML_INVALID_CONFIGURATION) {
    delete joystickDataBuf;
    joystickDataBuf = NULL;
    return 1;
  }
  
  return 0;
}

/*
  Modification history:

  $Log: moastNmlGlobal.cc,v $
  Revision 1.8  2007/07/24 18:34:12  proctor
  Added joystickData

  Revision 1.7  2006/10/31 09:30:42  dr_steveb
  Changes for SPQR integration

  Revision 1.6  2006/07/18 19:05:15  cj_scrapper
  Made SIM-ware the master of all Servo Echelon buffers that spawns an internal server to server all buffers.  This change is reflected in the moast.cfg file.  Added #if statement in moastNmlGlobl to exclude servo buffers if the number of buffers is 0.  Therefore, moastNmlSvr is not reponsible for servo Echelon buffers.

  Revision 1.5  2006/04/25 20:30:18  proctor
  Added support for moastError to nmlPrint, and added error printing for
  bad configurations to primMobJA

  Revision 1.4  2006/04/24 21:00:03  proctor
  Added MOAST error NML buffer

  Revision 1.3  2006/02/06 22:05:31  dr_steveb
  Added symbolicData buffer to server.

  Revision 1.2  2005/10/21 13:54:49  dr_steveb
  Added sectMobPL buffers.

  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.14  2005/10/05 17:38:48  proctor
  Referenced NML::error_type when deciding to fail; if it's a configuration
  error then continue, so we can step around commented-out buffers.

  Revision 1.13  2005/10/05 13:24:24  proctor
  moastNmlGlobal.cc, moastNmlSvr.cc: didn't fail when an NML buffer couldn't
  be created.

  Revision 1.12  2005/07/28 20:55:09  stephen
  Added multiple vehicles (3)

  Revision 1.11  2005/07/18 21:16:36  proctor
  Replaced sensorData with servoSPLinescan, ...

  Revision 1.10  2005/06/23 17:56:34  proctor
  Added servoSPData buffers

  Revision 1.9  2005/06/13 18:55:23  beytin
  Added creation of amSP NML buffers.

  Revision 1.8  2005/06/13 16:31:33  stephen
  Added primSP

  Revision 1.7  2005/06/10 21:50:39  kramer
  added AM buffers

  Revision 1.6  2005/06/08 20:49:36  stephen
  Changed occupancyData to sensorData

  Revision 1.5  2005/06/08 14:30:52  proctor
  Added VehMobJA name changes

  Revision 1.4  2005/06/08 13:18:02  proctor
  Did Mob,MisJA name change

  Revision 1.3  2005/06/07 21:45:32  proctor
  autogen

  Revision 1.2  2005/06/07 15:46:29  proctor
  Added primMisJA.cc to autogen list in 'makefile'; added handling of
  servo,primMisJA buffers in moastNmlGlobal.cc and moved out of moastNmlSvr.cc.

  Revision 1.1  2005/06/02 18:33:20  proctor
  Initial checkin

*/
