/*

Software Copywrite/Warranty Disclaimer

   This software was developed at the National Institute of Standards and
Technology by employees of the Federal Government in the course of their
official duties. Pursuant to title 17 Section 105 of the United States
Code this software is not subject to copyright protection and is in the
public domain. NIST Real-Time Control System software is an experimental
system. NIST assumes no responsibility whatsoever for its use by other
parties, and makes no guarantees, expressed or implied, about its
quality, reliability, or any other characteristic. We would appreciate
acknowledgement if the software is used. This software can be
redistributed and/or modified freely provided that any derivative works
bear some notice that they are derived from it, and any modified
versions bear some notice that they have been modified.

*/

#include <stddef.h>		// NULL
#include <stdlib.h>		// getenv()
#include <errno.h>
#include <string.h>
#include <stdio.h>

#include <rcs.hh>

#include "moastNmlOffsets.hh"
#include "moastNmlGlobal.hh"	// NUM_NAV_DATA_BUFFERS, servoMobJACmdBuf, etc.
#include "primMisJA.hh"		// primMisJA_format()

#include <stdio.h>		// sprintf(), FILE
#include "rcs.hh"		// NML
#include "moastTypes.hh"
#include "moastNmlOffsets.hh"
#include "moastError.hh"	// moastError_format()
#include "navDataExt.hh"	// navDataExt_format()
#include "servoMobJA.hh"	// servoMobJA_format()
#include "servoSP.hh"		// servoSP_format()
#include "servoMisJA.hh"	// servoMisJA_format()
#include "servoEffJA.hh"	// servoEffJA_format()
#include "sensorData.hh"	// sensorData_format()
#include "primMobJA.hh"		// primMobJA_format()
#include "primMisJA.hh"		// primMisJA_format()
#include "primSP.hh"            // primSP_format()
#include "vehMobPL.hh"          // vehMobPL_format()
#include "sectMobPL.hh"         // sectMobPL_format()
#include "amMobJA.hh"		// amMobJA_format()
#include "amMisJA.hh"		// amMisJA_format()
#include "amSP.hh"		// amSP_format()

#ifdef USE_SPQR
#include "nmlMap.hh"
#include "nmlPosition.hh"
#include "joystickData.hh"
#endif

static bool debug=false;
static char *CONFIG_NML=0;
static char *VEH_ID=0;

static int ignored_count=0;
static int err_count=0;
static int ok_count = 0;

static const char *PROG_SOURCE_CVS_ID="$Id: moastNmlWpsSvr.cc,v 1.5 2009/02/27 13:52:48 wshackle Exp $";

static void inline
check_channel(NML *nmlChan, 
	      const char *buffer_name)
{
  if(nmlChan && 
     ( nmlChan->error_type == NML_IGNORED_REMOTE 
#if RCS_MAJOR_VERSION >= 2009
       || nmlChan->error_type == NML_IGNORED_NO_BUFLINE
#endif
     ))
    {
      delete nmlChan;
      nmlChan=0;
      if(debug) printf("%s ignored.\n",buffer_name);
      ignored_count++;
    }
  else if(!nmlChan || !nmlChan->valid())
    {
      delete nmlChan;
      nmlChan=0;
      if(debug) printf("%s failed.\n",buffer_name);
      err_count++;
    }
  else
    {
      if(debug) printf("%s ok.\n",buffer_name);
      ok_count++;
    }      
}

static void inline 
OPEN_NML(NML_FORMAT_PTR _format_function_ptr, 
         const char  *buffer_name)
{
  if(debug) printf("Opening NML %s.\n", buffer_name);
  NML *nmlChan = 
    new NML(_format_function_ptr,buffer_name,
	    "moastNmlWpsSvr",
	    CONFIG_NML);
  check_channel(nmlChan,buffer_name);
}


static void inline 
OPEN_NML_WITH_ID(NML_FORMAT_PTR _format_function_ptr, 
		 const char *buffer_name_prefix)
{
  static char buffer_name[CMS_CONFIG_LINELEN];
  strncpy(buffer_name,buffer_name_prefix,sizeof(buffer_name));
  if(VEH_ID) 
    {
      strncat(buffer_name,VEH_ID, sizeof(buffer_name));
    }
  OPEN_NML(_format_function_ptr,buffer_name);
}


static void inline 
OPEN_NML_WITH_ID_AND_COUNT(NML_FORMAT_PTR _format_function_ptr, 
			   const char *buffer_name_prefix,
			   int count)
{
  static char buffer_name[CMS_CONFIG_LINELEN];
  for(int i=0; i < count; i++)
    {
      snprintf(buffer_name,
	       sizeof(buffer_name),
	       "%s%s_%d",buffer_name_prefix,(VEH_ID?VEH_ID:""),(i+1));
      OPEN_NML(_format_function_ptr,buffer_name);
    }
}


static void inline 
OPEN_RCS_STAT_CHANNEL(NML_FORMAT_PTR _format_function_ptr, 
         const char  *buffer_name)
{
  if(debug) printf("Opening RCS_STAT_CHANNEL %s.\n", buffer_name);
  RCS_STAT_CHANNEL *nmlChan = 
    new RCS_STAT_CHANNEL(_format_function_ptr,buffer_name,
	    "moastNmlWpsSvr",
	    CONFIG_NML);
  check_channel(nmlChan,buffer_name);
}


static void inline 
OPEN_RCS_STAT_CHANNEL_WITH_ID(NML_FORMAT_PTR _format_function_ptr, 
		 const char *buffer_name_prefix)
{
  static char buffer_name[CMS_CONFIG_LINELEN];
  strncpy(buffer_name,buffer_name_prefix,sizeof(buffer_name));
  if(VEH_ID) 
    {
      strncat(buffer_name,VEH_ID, sizeof(buffer_name));
    }
  OPEN_RCS_STAT_CHANNEL(_format_function_ptr,buffer_name);
}

static void inline 
OPEN_RCS_STAT_CHANNEL_WITH_ID_AND_COUNT(NML_FORMAT_PTR _format_function_ptr, 
					const char *buffer_name_prefix,
					int count)
{
  static char buffer_name[CMS_CONFIG_LINELEN];
  for(int i=0; i < count; i++)
    {
      snprintf(buffer_name,
	       sizeof(buffer_name),
	       "%s%s_%d",buffer_name_prefix,(VEH_ID?VEH_ID:""),(i+1));
      OPEN_RCS_STAT_CHANNEL(_format_function_ptr,buffer_name);
    }
}

static void inline 
OPEN_RCS_CMD_CHANNEL(NML_FORMAT_PTR _format_function_ptr, 
         const char  *buffer_name)
{
  if(debug) printf("Opening RCS_CMD_CHANNEL %s.\n", buffer_name);
  RCS_CMD_CHANNEL *nmlChan = 
    new RCS_CMD_CHANNEL(_format_function_ptr,buffer_name,
	    "moastNmlWpsSvr",
	    CONFIG_NML);
  check_channel(nmlChan,buffer_name);
}


static void inline 
OPEN_RCS_CMD_CHANNEL_WITH_ID(NML_FORMAT_PTR _format_function_ptr, 
		 const char *buffer_name_prefix)
{
  static char buffer_name[CMS_CONFIG_LINELEN];
  strncpy(buffer_name,buffer_name_prefix,sizeof(buffer_name));
  if(VEH_ID) 
    {
      strncat(buffer_name,VEH_ID, sizeof(buffer_name));
    }
  OPEN_RCS_CMD_CHANNEL(_format_function_ptr,buffer_name);
}

static void inline 
OPEN_RCS_CMD_CHANNEL_WITH_ID_AND_COUNT(NML_FORMAT_PTR _format_function_ptr, 
				       const char *buffer_name_prefix,
				       int count)
{
  static char buffer_name[CMS_CONFIG_LINELEN];
  for(int i=0; i < count; i++)
    {
      snprintf(buffer_name,
	       sizeof(buffer_name),
	       "%s%s_%d",
	       buffer_name_prefix,(VEH_ID?VEH_ID:""),(i+1));
      OPEN_RCS_CMD_CHANNEL(_format_function_ptr,buffer_name);
    }
}


static void
touch_running_file(int argc, const char **argv)
{
  const char *RUNNING_FILE="moastNmlWpsSvr.running";
  char *RUNNING_FILE_ENV=getenv("RUNNING_FILE");
  if(RUNNING_FILE_ENV)
    {
      RUNNING_FILE=RUNNING_FILE_ENV;
    }
  else
    {
      for(int i = 0 ; i  < argc-1; i++)
	{
	  if(!strcmp(argv[i],"--running_file"))
	    {
	      RUNNING_FILE=argv[i+1];
	      break;
	    }
	}
    }
  FILE *f = fopen(RUNNING_FILE,"w");
  if(!f)
    {
      fprintf(stderr,"Can't open %s -- %s \n",
	      RUNNING_FILE,strerror(errno));
      return;
    }
  else
    {
      fprintf(f,"prog=\t%s\n", argv[0]);
      fprintf(f,"cmd_line=\t%s", argv[0]);
      for(int i=1; i < argc; i++)
	{
	  fprintf(f," %s", argv[i]);
	}
      fprintf(f,"\n");
      fprintf(f,"$Id: moastNmlWpsSvr.cc,v 1.5 2009/02/27 13:52:48 wshackle Exp $ compiled on " __DATE__ " at " __TIME__ "\n");
      fprintf(f,"%s compiled from %s at %s on %s with id %s starting ...\n",
	      argv[0],__FILE__, __DATE__,__TIME__, PROG_SOURCE_CVS_ID);
      fprintf(f,"\n%d ignored, %d failed, %d ok\n",
	      ignored_count, err_count, ok_count);
      fflush(f);
      fclose(f);
    }
}

static void
moastInitNmlBuffers()
{

#ifdef USE_SPQR
  OPEN_NML(nmlMap_format, "MapBufZerg");
  OPEN_NML(nmlMap_format, "MapBufP2DX");
  OPEN_NML(nmlPosition_format, NML_POS_VICTIM_NAME);
  OPEN_NML(nmlPosition_format, NML_POS_MAPPER_NAME);
#endif
  OPEN_NML(moastError_format, "moastError");

  //  OPEN_NML_WITH_ID(sensorData_format,SYMBOLIC_DATA_NAME);
  OPEN_NML_WITH_ID(navDataExt_format,NAV_DATA_EXT_NAME);
  OPEN_NML_WITH_ID_AND_COUNT(sensorData_format,SERVO_SP_LINESCAN_BUF_NAME,4);
  OPEN_NML_WITH_ID_AND_COUNT(sensorData_format,SERVO_SP_SONAR_BUF_NAME,1);
  OPEN_NML_WITH_ID_AND_COUNT(sensorData_format,SERVO_SP_NAV_BUF_NAME,1);
  OPEN_NML_WITH_ID_AND_COUNT(sensorData_format,SERVO_SP_RFID_BUF_NAME,1);
  OPEN_NML_WITH_ID_AND_COUNT(sensorData_format,SERVO_SP_VICTIM_BUF_NAME,1);
  OPEN_NML_WITH_ID(sensorData_format,PRIM_SP_LINESCAN_NAME);
  OPEN_NML_WITH_ID(sensorData_format,AM_SP_MAP_NAME);

  static const bool mis = true;

  char *servo_env = getenv("SERVO");
  if(servo_env && !strcmp(servo_env,"yes"))
    {

      OPEN_RCS_CMD_CHANNEL_WITH_ID(servoMobJA_format,SERVO_MOB_JA_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(servoMobJA_format,SERVO_MOB_JA_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(servoMobJA_format,SERVO_MOB_JA_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(servoMobJA_format,SERVO_MOB_JA_SET_NAME);

      if(mis)
	{
	  OPEN_RCS_CMD_CHANNEL_WITH_ID_AND_COUNT(servoMisJA_format,SERVO_MIS_JA_CMD_NAME,2);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID_AND_COUNT(servoMisJA_format,SERVO_MIS_JA_STAT_NAME,2);
	  OPEN_RCS_CMD_CHANNEL_WITH_ID_AND_COUNT(servoMisJA_format,SERVO_MIS_JA_CFG_NAME,2);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID_AND_COUNT(servoMisJA_format,SERVO_MIS_JA_SET_NAME,2);

	  OPEN_RCS_CMD_CHANNEL_WITH_ID(servoEffJA_format,SERVO_EFF_JA_CMD_NAME);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID(servoEffJA_format,SERVO_EFF_JA_STAT_NAME);
	  OPEN_RCS_CMD_CHANNEL_WITH_ID(servoEffJA_format,SERVO_EFF_JA_CFG_NAME);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID(servoEffJA_format,SERVO_EFF_JA_SET_NAME);
	}

      OPEN_RCS_CMD_CHANNEL_WITH_ID(servoSP_format,SERVO_SP_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(servoSP_format,SERVO_SP_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(servoSP_format,SERVO_SP_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(servoSP_format,SERVO_SP_SET_NAME);

    }

  char *prim_env = getenv("PRIM");
  if(prim_env && !strcmp(prim_env,"yes"))
    {
      OPEN_RCS_CMD_CHANNEL_WITH_ID(primMobJA_format,PRIM_MOB_JA_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(primMobJA_format,PRIM_MOB_JA_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(primMobJA_format,PRIM_MOB_JA_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(primMobJA_format,PRIM_MOB_JA_SET_NAME);

      if(mis)
	{
	  OPEN_RCS_CMD_CHANNEL_WITH_ID(primMisJA_format,PRIM_MIS_JA_CMD_NAME);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID(primMisJA_format,PRIM_MIS_JA_STAT_NAME);
	  OPEN_RCS_CMD_CHANNEL_WITH_ID(primMisJA_format,PRIM_MIS_JA_CFG_NAME);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID(primMisJA_format,PRIM_MIS_JA_SET_NAME);
	}

      OPEN_RCS_CMD_CHANNEL_WITH_ID(primSP_format,PRIM_SP_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(primSP_format,PRIM_SP_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(primSP_format,PRIM_SP_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(primSP_format,PRIM_SP_SET_NAME);
    }

  char *am_env = getenv("AM");
  if(am_env && !strcmp(am_env,"yes"))
    {
      OPEN_RCS_CMD_CHANNEL_WITH_ID(amMobJA_format,AM_MOB_JA_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(amMobJA_format,AM_MOB_JA_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(amMobJA_format,AM_MOB_JA_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(amMobJA_format,AM_MOB_JA_SET_NAME);

      
      if(mis)
	{
	  OPEN_RCS_CMD_CHANNEL_WITH_ID(amMisJA_format,AM_MIS_JA_CMD_NAME);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID(amMisJA_format,AM_MIS_JA_STAT_NAME);
	  OPEN_RCS_CMD_CHANNEL_WITH_ID(amMisJA_format,AM_MIS_JA_CFG_NAME);
	  OPEN_RCS_STAT_CHANNEL_WITH_ID(amMisJA_format,AM_MIS_JA_SET_NAME);
	}

      OPEN_RCS_CMD_CHANNEL_WITH_ID(amSP_format,AM_SP_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(amSP_format,AM_SP_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(amSP_format,AM_SP_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(amSP_format,AM_SP_SET_NAME);
    }


  char *veh_env = getenv("VEH");
  if(veh_env && !strcmp(veh_env,"yes"))
    {
      OPEN_NML_WITH_ID(sensorData_format,VEH_MOB_MAP_NAME);

      OPEN_RCS_CMD_CHANNEL_WITH_ID(vehMobPL_format,VEH_MOB_PL_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(vehMobPL_format,VEH_MOB_PL_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(vehMobPL_format,VEH_MOB_PL_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(vehMobPL_format,VEH_MOB_PL_SET_NAME);
    }

  char *sect_env = getenv("SECT");
  if(sect_env && !strcmp(sect_env,"yes"))
    {
      OPEN_RCS_CMD_CHANNEL_WITH_ID(sectMobPL_format,SECT_MOB_PL_CMD_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(sectMobPL_format,SECT_MOB_PL_STAT_NAME);
      OPEN_RCS_CMD_CHANNEL_WITH_ID(sectMobPL_format,SECT_MOB_PL_CFG_NAME);
      OPEN_RCS_STAT_CHANNEL_WITH_ID(sectMobPL_format,SECT_MOB_PL_SET_NAME);
    }

  OPEN_NML(joystickData_format, "joystickData");
}


int 
main(int argc, const char *argv[])
{
  CONFIG_NML = getenv("CONFIG_NML");
  if(!CONFIG_NML)
    {
      fprintf(stderr,"Please set CONFIG_NML environment variable.\n");
      exit(1);
    }
  if(getenv("DEBUG"))
    {
      debug = true;
    }
  VEH_ID = getenv("VEH_ID");
  if(!VEH_ID)
    {
      fprintf(stderr,"Please set VEH_ID environment variable.\n");
      exit(1);
    }

  printf("%s compiled from %s at %s on %s with id %s starting ...\n",
	 argv[0],__FILE__, __DATE__,__TIME__, PROG_SOURCE_CVS_ID);

  printf("To see details:\n");
  printf("export DEBUG=1; # or setenv DEBUG 1\n");
  nmlSetToServer();
  nmlSetIgnoreRemote();
#if RCS_MAJOR_VERSION >= 2009
  nmlSetIgnoreNoBufline();
#endif

  load_nml_config_file(CONFIG_NML);

  moastInitNmlBuffers();

  printf("\n%d ignored, %d failed, %d ok\n",
	 ignored_count, err_count, ok_count);


  if(ok_count > 0)
    {
      touch_running_file(argc,argv);
      run_nml_servers();
    }
  else
    {
      nml_cleanup();
    }

  return 0;
}
