/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file laserTest.cc

  \brief Provides a test interface for the Hocuu sensor

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.1 $
  $Date: 2008/08/18 18:13:53 $
  \endcode

  \author Stephen Balakirsky
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>		/* f,printf, stderr, perror */
//#include <stddef.h>		/* sizeof, ptrdiff_t */
//#include <stdlib.h>		/* malloc, free, atoi */
//#include <string.h>		/* memset, strerror */
#include <stdarg.h>		/* va_list */
//#include <math.h>		/* M_PI */
//#include <errno.h>		/* errno */
//#include <unistd.h>		/* read, write */
//#include <time.h>               /* struct timespec, nanosleep */
//#include <termios.h>		/* struct termios */
//#include <fcntl.h>		/* open, O_RDWR */
#include <sys/time.h>           /* gettimeofday, struct timeval */
//#include <pthread.h>		/* pthread_attr_int, ... */
//#include "sockutils.h"		/* get_socket,client_fd */
//#include "ttycfg.h"		/* tty_raw */
//#include "matilda.h"		/* USARSIM,SMART_MOTOR_MSG_LEN */
//#include "inifile.h"

#include "../sim/simWare/swKBUtils.hh"

#include "../tools/sockutils.c"
#include "../tools/ttycfg.c"
#include "urg_laser.h"

/*************************
globals
************************/
const char *portName = "/dev/ttyACM0";
//const char *dataRequest = "\nG00076801\n";
//const char *compareTo =  "G00076801\n";
const char *dataRequest = "\nG38438401\n";
const char *compareTo =  "G38438401\n";

/*
  'dbprintf' is debug printf. Set 'debug' and you get some diagnostics
  on stderr that show what's going on. Fatal errors are printed
  regardless.
*/
static int debug = 1;
static void dbprintf(const char * fmt, ...)
{
  va_list ap;

  if (debug) {
    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
  }
}

/* get the time in seconds */
static double etime(void)
{
  struct timeval tv;

  gettimeofday(&tv, NULL);

  return ((double) tv.tv_sec) + ((double) tv.tv_usec) * 1.0e-6;
}

static void esleep(double secs)
{
  struct timespec ts;

  if( secs < 0 )return;
  ts.tv_sec = (int) secs;
  ts.tv_nsec = (int) ((secs - ts.tv_sec) * 1.0e9);

  (void) nanosleep(&ts, NULL);

  return;
}

#define ROUND(x) (x) < 0.0 ? (int) ((x) - 0.5) : (int) ((x) + 0.5)


/*
  Syntax: matildaserver <TCP port number> {<serial ports ...>}
*/
int main(int argc, char * argv[])
{
  int count;
  urg_laser hokuyo;
  urg_laser_readings_t hokuyoReadings;
  FILE *fd;

  hokuyo.Open( portName, 0, 0 );
  for(;;)
    {
      fd = fopen( "hokuyo.dat", "w" );
      hokuyo.GetReadings( &hokuyoReadings, 0, 6 );
      //  printf( "read %d readings\n", hokuyoReadings.number);
      for( count=0; count<hokuyoReadings.number; count++ )
	fprintf( fd, "%d\n", hokuyoReadings.Readings[count] );
      fclose(fd);
      sleep(1);
    }
}

/*
  Modification history:

  $Log: laserTest.cc,v $
  Revision 1.1  2008/08/18 18:13:53  dr_steveb
  Commit before new version release. Includes new code for matilda real robot

*/

