#include <unistd.h>
#include <fstream>
#include <iostream>
#include <gridfastslam/gridslamprocessor.h>
#include <sensor/sensor_range/rangesensor.h>
#include <sys/time.h>
#include <vector>

#include "gmapper.hh" 

using namespace std;
using namespace GMapping;

GMapper::GMapper(GrisettiParameters grisPar, OrientedPoint initPos) : GridSlamProcessor(cerr)
{

    //	cout<<"GMAP:"<<res<<endl;
    bestmap= NULL; // Important!!!! Else you get a segfault on resetting the map.
    mapWriteBuffer=NULL;

    double xmin=grisPar.minX;//-(float)(GRID_XRES*GRID_X_SCALE)/2000.0;
    double ymin=grisPar.minY;//-(float)(GRID_YRES*GRID_Y_SCALE)/2000.0;
    double xmax=grisPar.maxX;//(float)(GRID_XRES*GRID_X_SCALE)/2000.0;
    double ymax=grisPar.maxY;//(float)(GRID_YRES*GRID_Y_SCALE)/2000.0;
    //
    double delta= grisPar.delta;//(float)(GRID_X_SCALE)/1000.0;

    resolution = grisPar.angResRad;
    orig_beams_num = grisPar.numberOfBeams;
    max_range = grisPar.maxRangeM;

	skipBeams = grisPar.skipBeams;

    //	bool autoParams=false;
    //scan matching parameters
    double sigma=0.05;
    //	double maxrange=80.;
    double maxUrange=80.;
    //  regscore=1e4;
    double lstep=0.05;
    double astep=0.05;
    int kernelSize=1;
    int iterations=5;
    //  critscore=0.;
    //  maxMove=1.;
    double lsigma=0.075;
    double ogain=3;
    unsigned int lskip=0;
    //	bool autosize=false;
    bool skipMatching=false;

    //motion model parameters
    double srr=0.1, srt=0.1, str=0.1, stt=0.1;
    //particle parameters
    int particles=grisPar.numberOfParticles;
    //	double randseed=0;

    //gfs parameters
    double angularUpdate=0.01; 
    double linearUpdate=0.05; 
    double resampleThreshold=0.2;

    /*double input=0;

    bool running=false;
    int eventBufferLength=0;
    double mapUpdateTime=5;
    double mapTimer=0;
    bool readFromStdin=false;
    bool onLine=false;*/
    bool generateMap=false;//changed false to true

    // This  are the dafault settings for a grid map of 5 cm
    double llsamplerange=0.01;
    double llsamplestep=0.01;
    double lasamplerange=0.005;
    double lasamplestep=0.005;
    double linearOdometryReliability=0.4;
    double angularOdometryReliability=0.5;//this was 0.2 before i changed it

    bool considerOdometryCovariance=false;

    SensorMap smap;

    odometry=new OdometrySensor("ODOM");
    OdometrySensor* truepos=new OdometrySensor("TRUEPOS", true);
    smap.insert(make_pair(odometry->getName(), odometry));
    smap.insert(make_pair(truepos->getName(), truepos));
    OrientedPoint p(0,0,0);

    laser=new RangeSensor("FLASER",grisPar.numberOfBeams,grisPar.angResRad,p,0,grisPar.maxRangeM);
    smap.insert(make_pair(laser->getName(), laser));
    setSensorMap(smap);

    setMatchingParameters(maxUrange, grisPar.maxRangeM, sigma, kernelSize, lstep, astep, iterations, lsigma, ogain, lskip);
    setMotionModelParameters(srr, srt,str,stt);
    setUpdateDistances(linearUpdate, angularUpdate,resampleThreshold);
    setgenerateMap(generateMap);
    //setperformScanmatching(!skipMatching);
    GridSlamProcessor::init(particles,xmin, ymin, xmax,ymax, delta, initPos);//, p,m_covScanmatch);//changed 100 to 25

    setllsamplerange(llsamplerange);
    setllsamplestep(llsamplestep);
    setlasamplerange(lasamplerange);
    setlasamplestep(lasamplestep);
    //setlinearOdometryReliability(linearOdometryReliability);
    //setangularOdometryReliability(angularOdometryReliability);
    //setconsiderOdometryCovariance(considerOdometryCovariance);

    pthread_mutex_init(&data_mutex, NULL);

    bestParticleIndex=-1;
    newParticle=false;
}


GMapper::~GMapper(){

    delete laser;
    laser=0;
    delete odometry;
    odometry=0;
    pthread_mutex_destroy(&data_mutex);
}


void GMapper::UpdateMap(double Xpos,double Ypos,double Theta, vector<double> &laserData, double Xsen, double Ysen, double Tsen)
{
    if(laserData.size()>0)
    {
        timeval agstime;
        gettimeofday(&agstime,NULL);
        double gstime=agstime.tv_sec+agstime.tv_usec/1000000.0;

        SensorMap smap;
        smap.insert(make_pair(odometry->getName(), odometry));
        vector<RangeSensor::Beam> &beams = laser->beams();
        beams.clear();
        RangeReading r(laser,gstime);
        //int beamCounter=0;

        //vector<float>::const_iterator angleItr = laserdata->getAnglesRadBegin();
        vector<double>::const_iterator dataItr = laserData.begin();
        //Jacobotics::Geometry geo;
        //laserdata->lrf->getSensorGeometry(geo);

        for(int i = 0; i < laserData.size(); i++)
        {
                RangeSensor::Beam b;
                b.span=0;
                b.pose.x = Xsen;
                b.pose.y = Ysen;
                b.pose.theta=Tsen + ((-90 + i)*M_PI/180);
                b.maxRange=max_range;
                beams.push_back(b);
                r.push_back(*dataItr);
                dataItr++;
                //beamCounter ++;
        }
        //if(beamCounter>0){
            //	cout<<"GMAP: READING HAD "<<r.size()<<" BEAMS "<<endl<<"GMAP: BEAMS HAD "<<beams.size()<<endl;
            //	cout<<"GMAP: "<<laser->beams().size()<<endl;
            smap.insert(make_pair(laser->getName(), laser));
            setSensorMap(smap);	
            OdometryReading o(odometry,gstime);
            o.setPose(OrientedPoint(Xpos,Ypos,Theta));
            o.setSpeed(OrientedPoint(0,0,0));
            o.setAcceleration(OrientedPoint(0,0,0));
            OrientedPoint pos(Xpos,Ypos,Theta);
            r.setPose(pos);
            processScan(r);
            //processTruePos(o);	
        //}	
    }

}

void GMapper::UpdateMap(double Xpos,double Ypos,double Theta)
{
  timeval agstime;
  gettimeofday(&agstime,NULL);
  double gstime=agstime.tv_sec+agstime.tv_usec/1000000.0;

  SensorMap smap;
  smap.insert(make_pair(odometry->getName(), odometry));
  vector<RangeSensor::Beam> &beams = laser->beams();
  beams.clear();
  RangeReading r(laser,gstime);

  smap.insert(make_pair(laser->getName(), laser));
  setSensorMap(smap);	
  OdometryReading o(odometry,gstime);
  o.setPose(OrientedPoint(Xpos,Ypos,Theta));
  o.setSpeed(OrientedPoint(0,0,0));
  o.setAcceleration(OrientedPoint(0,0,0));
  OrientedPoint pos(Xpos,Ypos,Theta);
  r.setPose(pos);
  processScan(r);
            //processTruePos(o);
             
}
/*
void drawMap(const ScanMatcherMap & map){
    char filename[10];
    static int count = 0;
    ofstream fout;

    sprintf(filename,"pic%d",count);
    count++;

    fout.open(filename);

    cout<<"GMAP:Dumping grisetti map to "<<filename<<endl;

    //map = getParticles()[getBestParticleIndex()].map;

    Point min=map.map2world(0,0);
    Point max=map.map2world(map.getMapSizeX()-1,map.getMapSizeY()-1);
    //       	bool found=false;
    double delta=map.getDelta();
    // 	double top,bottom,left,right;
    double v;
    int grayValue;

    for(double j=max.y;j > min.y;j-=delta){
        for(double i=min.x;i<max.x;i+=delta){
            v=map.cell(i,j);
            if(v>=0){
                grayValue=255-(int)(255.*v);
                fout<<grayValue<<" ";
            }else
                fout<<128<<" ";
        }
        fout<<endl;
    }
    fout.close();
}
*/
bool GMapper::getPose(double &x, double &y, double &yaw){
    x= bestpose.x;
    y = bestpose.y;
    yaw = bestpose.theta;
    return true;
}
/*
bool GMapper::getPosemm(int &x, int &y, int &yaw){
    x= (int)(bestpose.x*1000);
    y =(int) (bestpose.y*1000);
    yaw = (int)(bestpose.theta*(18000.0/M_PI));
    return true;
}

bool GMapper::getPosemm(double &x, double &y, double &yaw){
    x= (bestpose.x*1000);
    y = (bestpose.y*1000);
    yaw = (bestpose.theta*(18000.0/M_PI));
    return true;
}


bool GMapper::getPath(vector<OrientedPoint> &path){
    path=bestpath;
    return true;
}

bool GMapper::getVictims(vector<Landmark>& vics){
    vics=bestvictims;
    return true;
}
*/

void GMapper::onOdometryUpdate()
{
    if(getParticles().size()==0)
	return;

    //MutexLocker ml(data_mutex);
    unsigned int bestParticleIndex = getBestParticleIndex();
    bestpose=getParticles()[bestParticleIndex].pose;
    //bestpath=getParticles()[bestParticleIndex].path;
    //bestvictims=getParticles()[bestParticleIndex].landmarks;
}

void GMapper::onResampleUpdate(){
    //DBG("GMAP::I just did a resample");
}

// void GMapper::onScanmatchUpdate()
// {
//     if(getParticles().size()==0)
// 	return;
//     delete mapWriteBuffer;
//     mapWriteBuffer=NULL;
//     //  mapWriteBuffer=&(getParticles()[getBestParticleIndex()].map);
//     mapWriteBuffer=new ScanMatcherMap(getParticles()[getBestParticleIndex()].map);
// 
//     //MutexLocker ml(data_mutex);
//     ScanMatcherMap *tempBuffer=bestmap;
//     bestmap=mapWriteBuffer; //swap buffers
//     mapWriteBuffer=tempBuffer;
//     bestpose=getParticles()[getBestParticleIndex()].pose;
//     //bestpath=getParticles()[getBestParticleIndex()].path;
//     if(getBestParticleIndex()!=bestParticleIndex){
//         newParticle=true;
//         bestParticleIndex=getBestParticleIndex();
//     }else{
//         newParticle=false;
//     }
//     //bestvictims=getParticles()[getBestParticleIndex()].landmarks;
// }

void GMapper::onScanmatchUpdate()
{
    if(getParticles().size()==0)
	return;
    //delete mapWriteBuffer;
    //mapWriteBuffer=NULL;
    //  mapWriteBuffer=&(getParticles()[getBestParticleIndex()].map);
    //mapWriteBuffer=new ScanMatcherMap(getParticles()[getBestParticleIndex()].map);

    //MutexLocker ml(data_mutex);
    //ScanMatcherMap *tempBuffer=bestmap;
    //bestmap=mapWriteBuffer; //swap buffers
    //mapWriteBuffer=tempBuffer;
    bestpose=getParticles()[getBestParticleIndex()].pose;
    //bestpath=getParticles()[getBestParticleIndex()].path;
    //if(getBestParticleIndex()!=bestParticleIndex){
    //    newParticle=true;
    //    bestParticleIndex=getBestParticleIndex();
    //}else{
    //    newParticle=false;
    //}
    //bestvictims=getParticles()[getBestParticleIndex()].landmarks;
}

/*
GridType GMapper::getGridCell(int x,int y){
    if (bestmap != NULL){
        double value=bestmap->cell(bestmap->map2world(x,y));
        if(value>=0){
            return GridType(GRID_OCCUPIED*value);
        }
        return GRID_UNEXPLORED;
    }
    return GRID_UNEXPLORED;
}

void GMapper::addVictim(int _id,Point3Dint _relpos){

    Landmark v(NULL);
    v.setLocation(_relpos);
    v.id.setLocalID(_id);
    v.setType( Landmark::LM_VICTIM );
    ParticleVector *particles=getParticlesUnconst();
//   cout << "particles: "<<particles->size()<<endl;
//   cout << "add victim pos: "<<(string)_pos<<endl;

    for(ParticleVector::iterator it=particles->begin(); it!=particles->end(); it++){
        it->addLandmark(v);	
    }
}

void GMapper::addLandmark(int _id, Point3Dint _relpos,  Landmark::LandmarkTypes _class){

    Landmark v(NULL);
    v.setLocation(_relpos);
    v.id.setLocalID(_id);
    v.setType( _class );
    ParticleVector *particles=getParticlesUnconst();

    for(ParticleVector::iterator it=particles->begin(); it!=particles->end(); it++){
        it->addLandmark(v);	
    }
}
*/

bool GMapper::newBestParticle(){
    bool tmp = newParticle;
    newParticle=false;
    return tmp;
}
