#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stddef.h>		// sizeof(), NULL
#include <stdlib.h>		// getenv()
#include <ctype.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
#include "getopt.h"
#endif
#ifdef HAVE_READLINE_READLINE_H
#include <readline/readline.h>
#include <readline/history.h>
#endif

#include <signal.h>

#include <vector>
#include <string>
#include <iostream>
using namespace std;

#include "playerInfSkin.hh"
#include "moastSupSkin.hh"
#include "simWareKB.hh"

SIMwareInfSkin * infSkin;
SIMwareSupSkin * supSkin;
SIMwareKB *kb;
int vehID = 1;
int platTypeID = 1;
int stPoseID = 0;
int verbosity = 0;
char *SIMWARE_INIFILE_PATH;

void swShutdown() 
{
  infSkin->closeSkin();
  supSkin->closeSkin();
  exit(1);
}

void
sigHandler (int signum)
{ 
  switch (signum)
    {
    case SIGINT:
      fprintf (stderr, "<<simWare::sigHandler> Recv SIGINT  : Shutting down simWare program\n");
      swShutdown();
      exit (1);
    case SIGQUIT:
      fprintf (stderr, "<simWare::sigHandler> Recv SIGQUIT : Shutting down simWare program\n");
      swShutdown();
      exit (1);
    default:
      break;
    }
}


#define SW_STATE_2_STRING(s) (s) == SW_KB_STATE_UNINITIALIZED ? "UNINITIALIZED" : (s) == SW_KB_STATE_INITIALIZED ? "INITIALIZED" : (s) == SW_KB_STATE_SHUTDOWN ? "SHUTDOWN" : "?"

#if 0
static void adminView(void)
{
  printf ("Admin State Menu\n");
  printf ("Vehicle ID             : %d\n", vehID);
  printf ("Platform Type ID       : %d\n", platTypeID);
  printf ("Starting Pose ID       : %d\n", stPoseID);
  printf ("Verbosity Level        : %d\n", verbosity);
  printf ("KB State               : %s\n", SW_STATE_2_STRING(kb->getKBState()));
  printf ("Infskin State          : %s\n", SW_STATE_2_STRING(infSkin->getSkinState()));
  printf ("SupSkin State          : %s\n", SW_STATE_2_STRING(supSkin->getSkinState()));
}
#endif

#if 0
static void helpView(void)
{
  printf("Help Menu\n");
  printf("q | quit                : quit\n");
  printf("? | help                : print this help\n");
  printf("start                   : starts SIMware\n");
  printf("verbose <int>           : not implemented\n");
  
}
#endif

void run()
{
  if(!infSkin->openSkin())
    swShutdown();
  
  if( ! infSkin->startBot(vehID,platTypeID,stPoseID) )
    swShutdown();
  
  if(!supSkin->openSkin())
    swShutdown();

  while(1)
    {
      esleep(1);
    }
}



int
main (int argc, char *argv[])
{
  char option;
 
  SIMWARE_INIFILE_PATH = DEFINED_SIMWARE_INIFILE_PATH;
  // set up signals
  if( signal (SIGINT, sigHandler) == SIG_ERR )
    {
      perror (" Could not set signal SIGINT");
      exit(1);
    }
  if( signal (SIGQUIT, sigHandler)  == SIG_ERR )
    {
      perror (" Could not set signal SIGQUIT");
      exit(1);
    }
  
  // set options  
  while (1)
    {
      option = getopt (argc, argv, "p:i:s:v:");
      if (option == -1)
	break;

      switch (option)
	{
	case 'p':
	  if (1 != sscanf (optarg, "%i", &platTypeID))
	    {
	      fprintf (stderr, "%s: bad value for vehicle ID: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	case 'i':
	  if (1 != sscanf (optarg, "%i", &vehID))
	    {
	      fprintf (stderr, "%s: bad value for vehicle ID: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	case 's':
	  if (1 != sscanf (optarg, "%i", &stPoseID))
	    {
	      fprintf (stderr, "%s: bad value for starting pose ID: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	case 'v':
	  if (1 != sscanf (optarg, "%i", &verbosity))
	    {
	      fprintf (stderr, "%s: bad value for verbosity level: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	default:
	  printf
	    ("\nUsage: -p<platform id> -i<vehID> -s<starting pose id> -v<verbosity level>\n");
	  return -1;
	  break;
	}
    }

  printf("\033[2J");
  printf("=================================\n");
  printf("*****      SIMware          *****\n");
  printf("=================================\n");

  kb      = new SIMwareKB;
  infSkin = new playerInfSkin;
  supSkin = new MoastSupSkin;  
  
  infSkin->configInfSkin(kb,supSkin);
  supSkin->configSupSkin(kb,infSkin);

  
  run();
 
  return 0;
}



