#define PLAYER2_0
/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   playerInfSkin.cc
  \brief  Provides an interface to player for simWare.

  Based on the simWare USARSim inferior skin.

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.7 $
  $Date: 2009/02/17 14:57:35 $
  \endcode

  \author Stephen Balakirsky
  \date   01/18/2008
*/
#include "playerInfSkin.hh"

#define PLAYER_INF_SKIN_CYCLE_TIME 0.005

/////////////// globals
vector <ClientConnection> clientProxies;
bool inPull; // are we we in pull or push mode of player

static bool using_stage=false;


/////////////// routines
void *
startPlayerClientThr (void *tArgs)
{
  vector<ClientConnection>::iterator it;
  RCS_TIMER timer(PLAYER_INF_SKIN_CYCLE_TIME);
  playerc_client_t *myClient = (playerc_client_t *) tArgs;
  int count;
  void *proxy;

  for(;;)
    {
    if(!inPull)  // if we're in push mode
      {
	// see if there's data
	count = playerc_client_peek(myClient, 50);
	if (count < 0)
	  {
	    printf("**Error** <playerInfSkin>:startPlayerClientThr %s", 
		   playerc_error_str());
	    break;
	  }
	if (count > 0)
	  {
	    proxy = playerc_client_read(myClient);
	    // NULL return from playerc_client_read() means an error in the
	    // connection to the server (I think)
	    if(!proxy)
	      break;
	  }
      }
    else // we're in pull mode
      {
	// we only want to request new data at roughly 20 Hz
        // this requests a round of data from the server to be read
        proxy = playerc_client_read(myClient);
        // NULL return from playerc_client_read() means an error in the
        // connection to the server (I think)
        if(!proxy)
          break;
	timer.wait();
      }
    // Update the devices
    for( it=clientProxies.begin(); it!=clientProxies.end(); it++ )
      {
	if(it->device.proxy)
	  (*(it->device.fnupdate)) (&(*it));
	//	(*(it->device.fnupdate)) (it->device.proxy);
      }
    }
  /*
      playerc_client_requestdata(myClient);
      playerc_client_read(myClient);
      // Update the devices
      for( it=clientProxies.begin(); it!=clientProxies.end(); it++ )
	{
	  if(it->device.proxy)
	    (*(it->device.fnupdate)) (&(*it));
	  //	    (*(it->device.fnupdate)) (it->device.proxy);
	}
      timer.wait();
    }
  */
}

void *
startPlayerParserThr (void *tArgs)
{
  // this should never get called since we use signals
  fprintf( stderr, "Hey bonehead, why are you starting a parser thread in playerInfSkin?\n" );
  return (void *) 0;
}

void *
startPlayerCmdThr (void *tArgs)
{
  PlayerInfSkin *skin = (PlayerInfSkin *) tArgs;
  skin->runCmdHandler ();
  pthread_exit ((void *) 0);
  return (void *) 0;
}

/////////////// Other routines

/////////////// PlayerInfSkin Class routines
PlayerInfSkin::PlayerInfSkin()  {}

PlayerInfSkin::~PlayerInfSkin() 
{
  closeSkin ();
}

bool PlayerInfSkin::loadWorldInfo()
{
  kb->world.insertStartPose(string("Zero"), 
			    PM_POSE( PM_CARTESIAN(0,0,0),
				     PM_RPY(0,0,0) ));
  return true;
}

bool PlayerInfSkin::configInfSkin(SIMwareKB * kBase, SIMwareSupSkin * sSkin) 
{
  kb=kBase;
  supSkin = sSkin;
  state = SW_KB_STATE_UNINITIALIZED;
  return true;
}

bool PlayerInfSkin::openSkin() 
{ 
  INIFILE if_d;
  const char *strPtr;
  int retval;

  if (state != SW_KB_STATE_UNINITIALIZED)
    return false;

  if (0)
    printf ("<PlayerInfSkin::openSkin>: Open inifile %s\n",
	    SIMWARE_INIFILE_PATH);
  if (if_d.open (SIMWARE_INIFILE_PATH))
    {
      printf
	("** ERROR ** <PlayerInfSkin::openSkin> invalid inifile path %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }

  // get the host name so that we can connect to the robot
  strPtr = if_d.find ("HOST_NAME", "PLAYER.INFSKIN");
  if ( strPtr == 0 )
    {
      strPtr = getenv("PlayerHost");
    }
  if (NULL == strPtr)
    {
      printf
	("** ERROR ** <PlayerInfSkin::openSkin> 'HOST_NAME' not defined in section 'PLAYER.INFSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  /* connect to the server and set the data modes*/
  if (0)
    printf ("<PlayerInfSkin::openSkin> : Connecting to [%s:%d]\n",
	    strPtr, 6665);

  client = playerc_client_create(NULL, strPtr, 6665);
  if( playerc_client_connect(client) != 0 )
    {
      printf( "%s", playerc_error_str());
      return false;
    }

  // get the world information and configure the world
  if (NULL == (strPtr = if_d.find ("WORLD", "PLAYER.INFSKIN")))
    {
      printf
	("** ERROR ** <USARSimInfSkin::openSkin> 'WORLD' not defined in section 'PLAYER.INFSKIN' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  kb->world.setWorldName(strPtr);
  state = SW_KB_STATE_INITIALIZED;

  if_d.close ();

  // start cmd thread - yes to handle superior skin commands
  retval = pthread_create (&cmdThr, NULL, (void *(*)(void *)) startPlayerCmdThr,
			   (void *) this);
  if (retval != 0)
    {
      printf ("**Error ** <playerInfSkin:openSkin>: pthread_create");
      return false;
    }

  retval = playerc_client_set_replace_rule(client, -1, -1, -1, -1, 1);
  if( retval != 0 )
    {
      printf("**Error **<playerInfSkin:openSkin>: %s", playerc_error_str());
      return false;
    }

    // Change the server's data delivery mode.
    // PLAYERC_DATAMODE_PUSH, PLAYERC_DATAMODE_PULL
  inPull = false;
  if( inPull )
    retval = playerc_client_datamode(client, PLAYERC_DATAMODE_PULL);
  else
    retval = playerc_client_datamode(client, PLAYERC_DATAMODE_PUSH);
  if( retval != 0 )
    {
      printf( "**Error **<playerInfSkin:openSkin>: %s", playerc_error_str());
      return false;
    }
  if( state != SW_KB_STATE_INITIALIZED)
    return false;
  else
    return true;
}

void PlayerInfSkin::closeSkin() 
{
  vector<ClientConnection>::iterator it;
  printf ("<PlayerInfSkin::closeSkin> :\n");
  
  state = SW_KB_STATE_SHUTDOWN;

  /*
  for( it=clientProxies.begin(); it!=clientProxies.end(); it++ )
    {
      cout << "Disconnecting driver: " << it->proxy->GetDriverName()
	   << ":" << it->proxy->GetInterfaceStr()  << "\n";
      it->proxy->DisconnectReadSignal(it->connection);
      //      delete it->proxy;
    }
  client->StopThread();
  */
}

int
PlayerInfSkin::writeMsg (string msg)
{
  int outChars = 0;
  return outChars;
}


void PlayerInfSkin::runCmdHandler()
{
  RCS_TIMER cTimer(PLAYER_INF_SKIN_CYCLE_TIME);
  static SIMwareCmdMsg * cmdMsg;
  while (state != SW_KB_STATE_SHUTDOWN)
    {
      cmdMsg=kb->popCmdFromQ();
       
      if( cmdMsg != NULL )
	{
	  switch(cmdMsg->getCmdType())
	    {
	    case SW_MOB_CMD_TYPE:
	      doMobCmd((SIMwareMobCmd *)cmdMsg);
	      break;
	    case SW_MIS_CMD_TYPE:
	      doMisCmd((SIMwareMisCmd *)cmdMsg);
	      break;
	    case SW_EFF_CMD_TYPE:
	      doEffCmd((SIMwareEffCmd *)cmdMsg);
	      break;
	    case SW_SEN_CMD_TYPE:
	      doSenCmd((SIMwareSenCmd *)cmdMsg);
	      break;
	    case SW_SET_CMD_TYPE:
	      break;
	    default:
	      break;
	    }
	  delete cmdMsg;
	}
      cTimer.wait();
    }
}


void PlayerInfSkin::doEffCmd(SIMwareEffCmd * effCmd)
{

  fprintf( stderr, "Effector commands not yet implemented with playerSkin\n" );
  /*
    stringstream cmdStr;
    string opName;
    map<const char *, int, strCmp>::iterator itr;
    switch(effCmd->getEffCmdType())
    {
    case SW_EFF_CMD_INIT_TYPE: 
    break;
    case SW_EFF_CMD_POWERDOWN_TYPE: 
    break;
    case SW_EFF_CMD_OPCODE_TYPE:
    opName = "NOP";
    for(itr=effOpcodeNameMap.begin();itr != effOpcodeNameMap.end();itr++)
    if( ((SIMwareEffOpcodeCmd *)effCmd)->opcode == itr->second)
    opName = itr->first;
      
    cmdStr << "SET {Type Effecter}{Name " << ((SIMwareEffOpcodeCmd *)effCmd)->effName 
    << "}{Opcode " << opName << "}{Params " << ((SIMwareEffOpcodeCmd *)effCmd)->param << "}";
    writeMsg(cmdStr.str());
      
    break;
    default:
    break;
    }
  */
}


void PlayerInfSkin::doMisCmd(SIMwareMisCmd * misCmd)
{

  fprintf( stderr, "Mission commands not yet implemented with playerSkin\n" );
  /*
    stringstream cmdStr;
  
    switch(misCmd->getMisCmdType())
    {
    case SW_MIS_CMD_INIT_TYPE: 
    break;
    case SW_MIS_CMD_POWERDOWN_TYPE: 
    break;
    case SW_MIS_CMD_MOVE_TYPE:
    cmdStr << "MISPKG {Name " << ((SIMwareMisMoveCmd *)misCmd)->misPkgName << "} ";
    for(unsigned int i=0;i<((SIMwareMisMoveCmd *)misCmd)->linkCmd.size();i++)
    {
    cmdStr << "{Link " << ((SIMwareMisMoveCmd *)misCmd)->linkCmd[i].linkID
    << "} {Value  " << ((SIMwareMisMoveCmd *)misCmd)->linkCmd[i].value 
    << "} {Order  ";
    switch(((SIMwareMisMoveCmd *)misCmd)->linkCmd[i].type)
    {
    case SW_MIS_LINK_CMD_VELOCITY_TYPE:
    cmdStr << "1}";
    break;
    case SW_MIS_LINK_CMD_TORQUE_TYPE:
    cmdStr << "2}";
    break;
    case SW_MIS_LINK_CMD_ABS_VAL_TYPE:
    default:
    cmdStr << "0}";
    break;
    }
    writeMsg(cmdStr.str());
    }
    break;
    default:
    break;
    }
  */
}

// returns false on error
bool PlayerInfSkin::doMobCmd(SIMwareMobCmd * mobCmd)
{
  double v, sideV, dTheta; // the forward, sideways, and angular speeds in m/sec,
                           // m/sec, and radians/sec to be sent to player
  static double oldv=0, oldSideV=0, oldDTheta=0;
  bool sendToMotor = false;
  static double cycleCounter = 1;

  if( motorProxy == NULL )
    return false;
  switch(mobCmd->getMobCmdType())
    {
    case SW_MOB_CMD_INIT_TYPE: 
      if( !motorProxy->info.subscribed )
	{
	  printf( "<playerInfSkin> Error, not subscribed to motor proxy for init\n" );
	  return false;
	}
      break; //debug
      for( int i=0; i<clientProxies.size(); i++ )
	if( !strcmp(clientProxies[i].device.drivername, "rflex") ||
	    !strcmp(clientProxies[i].device.drivername, "p2os") )
	  {
	    if( playerc_position2d_enable((playerc_position2d_t*)clientProxies[i].device.proxy, 1)
		!= 0 )
	      {
		printf( "<playerInfSkin> Error from init: %s\n", playerc_error_str());
		return false;
	      }
	    printf( "playerInfSkin: Found motor connected to %s\n",
		    clientProxies[i].device.drivername);
	    break;
	  }
      break;
    case SW_MOB_CMD_POWERDOWN_TYPE: 
      for( int i=0; i<clientProxies.size(); i++ )
	if( !strcmp(clientProxies[i].device.drivername, "rflex") ||
	    !strcmp(clientProxies[i].device.drivername, "p2os") )
	  {
	    /*
	    if( playerc_position2d_enable((playerc_position2d_t*)clientProxies[i].device.proxy, 0)
		!= 0 )
	      {
		printf( "<playerInfSkin> Error from init: %s\n", playerc_error_str());
		return false;
	      }
	    */
	    printf( "playerInfSkin: Found motor connected to %s\n",
		    clientProxies[i].device.drivername);
	    break;
	  }
      break;
    case SW_MOB_CMD_DRV_ACKERMAN_TYPE:
      v = ((SIMwareAckermanCmd *)mobCmd)->vLinear;
      dTheta = ((SIMwareAckermanCmd *)mobCmd)->steerAngle;
      sideV = 0;
      sendToMotor = true;
      break;
    case SW_MOB_CMD_DRV_SKID_TYPE: 
      fwdJacobian (((SIMwareSkidCmd *)mobCmd)->wRight,
		   ((SIMwareSkidCmd *)mobCmd)->wLeft,
		   ((SIMwareKBGroundVehicle *)kb->bot)->getWheelRadius (),
		   1/((SIMwareKBGroundVehicle *)kb->bot)->getWheelBase (),
		   &v,
		   &dTheta);
      sideV = 0;
      sendToMotor = true;
      break;
    case SW_MOB_CMD_DRV_FLIPPER_TYPE:
      printf("** WARNING ** <playerInfSkin::doMobCmd> mobility command not implemented yet\n");
      break;
    case SW_MOB_CMD_DRV_OMNI_TYPE: 
      sideV = ((SIMwareOmniCmd *)mobCmd)->vLateral;
      v =((SIMwareOmniCmd *)mobCmd)->vLinear;
      dTheta = ((SIMwareOmniCmd *)mobCmd)->vRot;
      sendToMotor = true;
      break;
    case SW_MOB_CMD_DRV_SUBMARINE_TYPE: 
      printf("** WARNING ** <playerInfSkin::doMobCmd> mobility command not implemented yet\n");
      break;
    case SW_MOB_CMD_DRV_ROTARY_WING_TYPE: 
      printf("** WARNING ** <playerInfSkin::doMobCmd> mobility command not implemented yet\n");
      break;
    default:
      printf("** ERROR ** <PlayerInfSkin::doMobCmd> mobility command not recognized\n");
      break;
    }
  if(using_stage)
    {
      // This seems to make the yaw direction prim expects to match the way
      // the stage simulation works.
      // Notice in primMobEngine.cc 
      //  retV1 = wheel_radius_inverse * (wheel_separation_half * w);
      //  retV2 = wheel_radius_inverse * (-wheel_separation_half * w);
      //  retV1 is a reference to skidMsg.wLeft and 
      //  retV2 is a reference to skidMsg.wRight
      // fwdJacobian above has the reverse association.
      //  w1 is passed ((SIMwareSkidCmd *)mobCmd)->wRight
      //  w2 is passed ((SIMwareSkidCmd *)mobCmd)->wLeft
      //    *w = L_inv * R * (w1 - w2);
      // FIXME : get rid of the number references both places so it is 
      // easy to see what is left what is right and whether you have flipped a
      // sign or not.
      dTheta = -1.0*dTheta;
    }
  if( sendToMotor )
    {
      if( fabs(v-oldv)>.1 || fabs(sideV-oldSideV)>.1 || fabs(dTheta-oldDTheta)>.1)
	{
	  //	  printf( "Sending motor proxy: v:%f sv:%f dTheta:%f\n", v, sideV, dTheta );
	  fflush(stdout);
	  oldv = v;
	  oldSideV = sideV;
	  oldDTheta = dTheta;
	}

      //      if( fmod(cycleCounter, 60 ) == 0 )
      //	{
	  playerc_position2d_set_cmd_vel( motorProxy, v, sideV, dTheta, 1 );
	  //	printf( "Sending motor proxy: v:%f sv:%f dTheta:%f\n", v, sideV, dTheta );
	  //	}
      cycleCounter++;
    }
  return true;
}


void PlayerInfSkin::doSenCmd(SIMwareSenCmd * senCmd)
{  

  switch(senCmd->getSenCmdType())
    {
    case SW_SEN_CMD_INIT_TYPE:
      break;
    case SW_SEN_CMD_POWERDOWN_TYPE:
      break;
    default:
      if(1)
	printf("** ERROR ** <PlayerInfSkin::doSenCmd> sensor command not recognized\n");
      break;

    }
}
      
bool PlayerInfSkin::startBot ( int vehID, string platformName, string platformUniqueID, PM_POSE stPose )
{
  ClientConnection *newConnection;
  PM_RPY rpy = stPose.rot;
  int count;
  char devID[10];
  char section[256];
  vector <ClientConnection>::iterator iter;
  bool foundSensor = false;

  printf("<PlayerInfSkin.startBot> : Initializing Robot\n");
  printf("\tRobot Platform : %s\n", platformName.c_str());
  printf("\tRobot ID       : %d\n", vehID);
  printf("\t[x,y,z]        : [%f,%f,%f]\n", stPose.tran.x, stPose.tran.y, stPose.tran.z);
  printf("\t[r,p,y]        : [%f,%f,%f]\n", rpy.r, rpy.p, rpy.y);

  // Get the available devices.
  if (playerc_client_get_devlist(client) != 0)
  {
    printf("%s", playerc_error_str());
    return -1;
  }

  // print list of possible clients
  printf( "Found %d possible devices\n", client->devinfo_count );
  for( count=0; count<client->devinfo_count; count++)
    {
      /*snprintf(section, sizeof(section), "%s:%d",
#ifdef PLAYER2_0
	       playerc_lookup_name(client->devinfos[count].addr.interf), 
#else// use this for player 2.1.1 
	       interf_to_str(client->devinfos[count].addr.interf),
#endif
	       client->devinfos[count].addr.index);*/
      printf("%-16s %-40s\n", section, client->devinfos[count].drivername);
    }

  // Get list of devices,set up proxies for them, 
  // add them to knowledgebase
  for( count=0; count<client->devinfo_count; count++)
    {
      newConnection = create_proxy(client, count);

      switch( newConnection->device.addr.interf )
	{
	case PLAYER_POSITION2D_CODE: // 2D positioning sensor
	  // set up proxy
	  clientProxies.push_back(*newConnection);
	  newConnection = &clientProxies.back();
	  // set up knowledge base
	  if( !strcmp(newConnection->device.drivername, "rflex") )
	    {
	      // the rflex is the main driver for the ATRV, so set up the kb robot parameters here
	      if( configGndRobot( vehID, 
				  (playerc_position2d_t*)(newConnection->device.proxy), 
				  platformName.c_str(),
				  newConnection->uniqueName.c_str() ) )
		printf( "<PlayerInfSkin:: Opened rflex p2d sensor and attached motorproxy to %s\n",
			newConnection->uniqueName.c_str());
	      motorProxy = (playerc_position2d_t*)newConnection->device.proxy;


	      printf( "Trying motor proxy on\n" );
	      if( playerc_position2d_enable(motorProxy, 1) != 0 )
		{
		  printf( "<playerInfSkin> Error from init: %s\n", playerc_error_str());
		}


	      // use this sensor for the KB odometry
	    }
	  else if( !strcmp(newConnection->device.drivername, "stage") )
	    {
	      using_stage=true;

	      // the rflex is the main driver for the ATRV, so set up the kb robot parameters here
	      if( configGndRobot( vehID, 
				  (playerc_position2d_t*)(newConnection->device.proxy), 
				  platformName.c_str(), 
				  newConnection->uniqueName.c_str() ) )
		printf( "<PlayerInfSkin:: Opened rflex p2d sensor and attached motorproxy to %s\n",
			newConnection->uniqueName.c_str());
	      motorProxy = (playerc_position2d_t*)newConnection->device.proxy;

	      // preset sensor to the preset location from the .ini file
	      // Get the global start pose
	      //	      p2d->SetOdometry( stPose.tran.x, stPose.tran.y, rpy.y );

	      // use this sensor for the KB odometry
	    }
	  else if( !strcmp(newConnection->device.drivername, "sicknav200") )
	    {
	      printf( "Creating sensor %s for sicknav200\n", 
		      newConnection->uniqueName.c_str() );
	      if( kb->getSensor( newConnection->uniqueName, true, 
				 SW_KB_GRD_TRUTH_SENSOR_TYPE ) == NULL )
		printf( "<PlayerInfSkin:: could not create gndTruth sensor %s in KB\n",
			newConnection->uniqueName.c_str() );
	      //      printf( "Memory location of sick: 0x%x \n", newConnection );
	    }
	  else if( !strcmp(newConnection->device.drivername, "p2os") )
	    {
	      printf( "Found pioneer robot\n" );
	      if( configGndRobot( vehID, 
				  (playerc_position2d_t*)(newConnection->device.proxy), 
				  platformName.c_str(), 
				  newConnection->uniqueName.c_str() ) )
		printf( "<PlayerInfSkin:: Opened p2os p2d sensor and attached motorproxy to %s\n",
			newConnection->uniqueName.c_str());
	      motorProxy = (playerc_position2d_t*)newConnection->device.proxy;



	      /*
	      if( playerc_position2d_enable(motorProxy, 1) != 0 )
		{
		  printf( "<playerInfSkin> Error from init: %s\n", playerc_error_str());
		  exit(1);
		}
	      printf( "Motor proxy 0x%x enabled\n", motorProxy );
	      */
	    }
	  else
	    printf( "<PlayerInfSkin::**WARNING** Unknown interface type %s to KB Position sensor creation\n",
		    newConnection->device.drivername );
	  break;
	case PLAYER_POWER_CODE: // there is no special entry in the KB for a power sensor
	  clientProxies.push_back(*newConnection);
	  newConnection = &clientProxies.back();
	  break;
	case PLAYER_LASER_CODE:
	  clientProxies.push_back(*newConnection);
	  newConnection = &clientProxies.back();
	  foundSensor = true;
	  break;
	case PLAYER_IR_CODE:
	  break;
	  /*
	case PLAYER_VECTORMAP_CODE:
	  printf( "<PlayerInfSkin::**Warning** ignoring PLAYER_VECTORMAP_CODE\n" );
	  break;
	  */
	case PLAYER_OPAQUE_CODE:
	  printf( "<PlayerInfSkin::**Warning** ignoring PLAYER_OPAQUE_CODE\n" );
	  break;
	case PLAYER_BUMPER_CODE:
	  printf( "<PlayerInfSkin::**Warning** ignoring PLAYER_BUMPER_CODE\n" );
	  break;
	case PLAYER_SONAR_CODE:
	  clientProxies.push_back(*newConnection);
	  newConnection = &clientProxies.back();
	  foundSensor = true;
	  break;
	case PLAYER_AIO_CODE:
	  printf( "<PlayerInfSkin::**Warning** ignoring PLAYER_AIO_CODE\n" );
	  break;
	case PLAYER_DIO_CODE:
	  printf( "<PlayerInfSkin::**Warning** ignoring PLAYER_DIO_CODE\n" );
	  break;
	default:
	  printf( "** ERROR ** interface %d (%s) not yet supported\n", newConnection->device.addr.interf,
		  newConnection->device.drivername);
	  //playerc_lookup_name(dlistIt->addr.interf) );
	  break;
	}
    }
  if( !foundSensor )
    {
      printf( "SimWare requires some sensor to work correctly, so adding a fake laser\n" );
      kb->getSensor("FakeLaser", true, SW_KB_LINESCAN_SENSOR_TYPE);
    }
  printf( "Connected devices:\n" );
  for( iter=clientProxies.begin(); iter!=clientProxies.end(); iter++ )
    {
/*
      printf( "%s:%d %s named: %s status: ",
#ifdef PLAYER2_0
	      playerc_lookup_name(iter->device.addr.interf), 
#else // use for player 2.1.1
	      interf_to_str(iter->device.addr.interf), 
#endif
	      iter->device.addr.index,
	      iter->device.drivername,
	      iter->uniqueName.c_str());*/
      if( iter->device.proxy)
	{
	  if( iter->device.subscribe)
	    printf("subscribed");
	  else printf("ready");
	}
      else
	printf("unsupported");
      printf("\n");
    }

  // if we are a disembodied sensor (i.e. no robot platform) we still need to 
  // set something to avoid core dumps
  if( kb->bot == NULL )
    {
      printf( "No robot body found\n" );
      configGndRobot( vehID, NULL, "NoPlatform", 
		      "NoNavSensor");
      motorProxy = NULL;
    }


  kb->setKBState(SW_KB_STATE_INITIALIZED);
  int retval =
    pthread_create (&playerThr, NULL, (void *(*)(void *)) startPlayerClientThr,
		    (void *) client);
  if (retval != 0)
    {
      printf ("**Error ** <playerInfSkin:startBot>: pthread_create");
      return false;
    }
  return true;
}

void PlayerInfSkin::runParser() { 
}

bool PlayerInfSkin::configGndRobot( int vehID, 
				    playerc_position2d_t *position2dProxy, 
				    const char *platformName, 
				    const char *position2DName )
{
  PM_CARTESIAN dimensions;
  // HARD CODED ITEMS SHOULD BE SET SOMEWHERE ELSE!
  player_bbox3d_t vehSize;

  kb->bot = new SIMwareKBGroundVehicle; 
  kb->bot->setBotID(vehID);
  ((SIMwareKBGroundVehicle *)kb->bot)->setPlatformName(platformName);
  ((SIMwareKBGroundVehicle *)kb->bot)->setSteerType(SW_KB_SKID_STEER_TYPE);
  ((SIMwareKBGroundVehicle *)kb->bot)->setMass (0.0);
  ((SIMwareKBGroundVehicle *)kb->bot)->setMaxWheelRot (5.2); // for demo was 2.6, before was 5.2
  ((SIMwareKBGroundVehicle *)kb->bot)->setMaxTorque (150.0);
  ((SIMwareKBGroundVehicle *)kb->bot)->setMaxSteerAngle (0.0);
  ((SIMwareKBGroundVehicle *)kb->bot)->setMaxCrabAngle (0.0);
  ((SIMwareKBGroundVehicle *)kb->bot)->setWheelSeparation (.673);
  ((SIMwareKBGroundVehicle *)kb->bot)->setWheelRadius (.203);
  ((SIMwareKBGroundVehicle *)kb->bot)->setWheelBase (.457);
  ((SIMwareKBGroundVehicle *)kb->bot)->setCentroid( 0, 0, 0 );
  playerc_position2d_get_geom(position2dProxy);
  ((SIMwareKBGroundVehicle *)kb->bot)->setDimensions(PM_CARTESIAN(position2dProxy->size[0], position2dProxy->size[1], 0.55));
  dimensions = ((SIMwareKBGroundVehicle *)kb->bot)->getDimensions();
  printf( "Robot dimenstions set to: %f %f %f\n",
	  dimensions.x, dimensions.y, dimensions.z );

  printf( "Creating sensor %s in configGndRobot\n", position2DName );
  if( kb->getSensor( position2DName, true, SW_KB_ODOMETRY_SENSOR_TYPE ) == NULL)
    {
      printf( "<PlayerInfSkin::**ERROR** could not create Odometry sensor %s in KB\n",
	      position2DName );
      return false;
    }
  return true;
}

/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<  PARSING FUNCTIONS FOR GEO MESSAGES  >>>>>> */
/*
void
PlayerInfSkin::pSenGeo (const vector < string > &toks)
{
  SIMwareKBSen *sensor;
  unsigned int i=0;
  SIMwareKBMount * mount=NULL;
  
  if (0)
    printf ("<PlayerInfSkin::pSenGeo>\n");
  if (0)
    dumpTokens (toks);

  while( i < toks.size () )
    {
      if (!nocase_cmp (toks[i], "Name"))
	{
	  sensor = kb->getSensor (toks[++i]);
	  if (sensor == NULL)
	    {
	      printf
		("** Warning ** <PlayerInfSkin::pSenGeo> Sensor %s not found\n",
		 toks[i].c_str ());
	      continue;
	    }
	  while( ++i < toks.size() )
	    {
	      if (!nocase_cmp (toks[i], "Location"))
		{
		  sensor->setSen2MountXYZ (atof (toks[i + 1].c_str ()),
					   atof (toks[i + 2].c_str ()),
					   atof (toks[i + 3].c_str ()));
		  i += 3;
		  
		}
	      else if (!nocase_cmp (toks[i], "Orientation"))
		{
		  sensor->setSen2MountRPY (atof (toks[i + 1].c_str ()),
					   atof (toks[i + 2].c_str ()),
					   atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      else if (!nocase_cmp (toks[i], "Mount"))
		{
		  i++;
		  if (nocase_cmp (toks[i], "HARD"))
		    {
		      mount = new SIMwareKBMount;
		      mount->misPkg=kb->getMisPkg(toks[i].substr(0,toks[i].find("_")));
		     
		      if( mount->misPkg == NULL ) 
			{
			  printf("**ERROR** <PlayerInfSkin::pSenGeo> Mount %s not recognized\n", toks[i].c_str());
			  delete mount;
			}
		      else
			{
			  mount->link=
			    mount->misPkg->getLink(atoi(toks[i].substr(toks[i].find_last_of("Link")+1).c_str()));
			  sensor->setMount(mount);
			  mount = NULL;
			}
		    }
		  else 
		    sensor->setMount(NULL);
		}
	      
	      else if (!nocase_cmp (toks[i], "Name"))
		break;
	      
	      else
		printf
		  ("** WARNING ** <PlayerInfSkin::pSenGeo> Token %s not recognized\n",
		   toks[i].c_str ());
	      
	    }
	  sensor = NULL;
	}
      else 
	i++;
    }
}
*/
/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<  PARSING FUNCTIONS FOR SEN MESSAGES  >>>>>> */
 /*
void
PlayerInfSkin::pSenLinescan (const vector < string > &toks)
{
  SIMwareKBSenLinescan *lsPtr = NULL;
  unsigned int i=0;
  double t = 0.00;
  static vector < double >returns;

  if (0)
    printf ("<PlayerInfSkin::pSenLinescan>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
	{
	  t = atof (toks[++i].c_str ());
	  i++;
	}
      else if (!nocase_cmp (toks[i], "Name")) 
	{
	  lsPtr = (SIMwareKBSenLinescan *) kb->getSensor (toks[++i].c_str ());
	  if (lsPtr == NULL)
	    {
	      if(1) 
		printf
		  ("** WARNING ** <PlayerInfSkin::pSenLinescan> Linescanner %s not found\n",
		   toks[i].c_str ());
	      continue;
	    }

	  // Set Data
	  lsPtr->setTimeStamp (t);
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Resolution"))
		{
		  ++i;		  
		  //lsPtr->setBeamSpacing (atof (toks[++i].c_str ()));
		}
		  
	      else if (!nocase_cmp (toks[i], "FOV"))
		lsPtr->setFov (atof (toks[++i].c_str ()));
	      
	      else if (!nocase_cmp (toks[i], "Range"))
		{
		  while(++i< toks.size())
		    {
		      if (!nocase_cmp (toks[i], "Name"))
			break;
		      returns.push_back(atof (toks[i].c_str ()));
 		    }
		  lsPtr->setRangeReturn (returns);
 		  lsPtr->setBeamSpacing(lsPtr->getFov()/(lsPtr->getReturnCount()-1));
		  returns.clear();
		}
	      else if (!nocase_cmp (toks[i], "Name")) 
		break;
	      
	      else
		printf
		  ("** WARNING ** <PlayerInfSkin::pSenLinescan> Token <%s> is not recognized\n",
		   toks[i].c_str ());
	    }
	  lsPtr=NULL;
	}
      else 
	i++;
    }
}



void
PlayerInfSkin::pSenSonar (const vector < string > &toks)
{
  SIMwareKBSenSonar *sPtr;
  unsigned int i=0;
  double t = 0.00;
  
  if (0)
    printf ("<PlayerInfSkin::pSenSonar>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = (SIMwareKBSenSonar *) kb->getSensor (toks[++i].c_str ());
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <PlayerInfSkin::pSenSonar> Sonar %s not found\n",
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
      
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Range"))
		sPtr->setRangeReturn (atof (toks[++i].c_str ()));
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <PlayerInfSkin::pSenSonar> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}

void
PlayerInfSkin::pSenINS (const vector < string > &toks)
{
  unsigned int i=0;
  SIMwareKBSenINS *sPtr;
  double t = 0.00;
  if (0)
    printf ("<PlayerInfSkin::pSenINS>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = (SIMwareKBSenINS *) kb->getSensor (toks[++i].c_str ());
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <PlayerInfSkin::pSenINU> INS sensor %s not found\n",  
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
	  while (++i < toks.size()) 
	    {
	      
	      if (!nocase_cmp (toks[i], "Orientation"))
		{
		  sPtr->setOrientation (atof (toks[i + 1].c_str ()),
					atof (toks[i + 2].c_str ()),
					atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      else if (!nocase_cmp (toks[i], "Location"))
		{
		  sPtr->setLocation (atof (toks[i + 1].c_str ()),
				     atof (toks[i + 2].c_str ()),
				     atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <PlayerInfSkin::pSenINS> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}
void
PlayerInfSkin::pSenINU (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenINU *sPtr;
  double t = 0.00;

  if (0)
    printf ("<PlayerInfSkin::pSenINU>\n");
  if (0)
    dumpTokens (toks);
 
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = (SIMwareKBSenINU *) kb->getSensor (toks[++i].c_str ());
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <PlayerInfSkin::pSenINU> INU sensor %s not found\n",  
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
      
	  while (++i < toks.size()) 
	    {
	      
	      if (!nocase_cmp (toks[i], "Orientation"))
		{
		  sPtr->setOrientation (atof (toks[i + 1].c_str ()),
					atof (toks[i + 2].c_str ()),
					atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <PlayerInfSkin::pSenINU> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}

void
PlayerInfSkin::pSenOdometry (const vector < string > &toks)
{
  unsigned int i = 0;
  double t = 0.00;
  SIMwareKBSenOdometry *sPtr = NULL;

  if (0)
    printf ("<PlayerInfSkin::pSenOdometry>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = (SIMwareKBSenOdometry *) kb->getSensor (toks[++i].c_str ());
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <PlayerInfSkin::pSenOdometry> Odometry sensor %s not found\n",
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Pose"))
		{
		  sPtr->setOdometry (atof (toks[i + 1].c_str ()),
				     atof (toks[i + 2].c_str ()),
				     atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <PlayerInfSkin::pSenOdometry> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}

void
PlayerInfSkin::pSenGrdTruth (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenGrdTruth *sPtr = NULL;
  double time=0.00;
  if (0)
    printf ("<PlayerInfSkin::pSenGrdTruth>\n");
  if (0)
    dumpTokens (toks);
  
  while(++i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
	time = atof (toks[++i].c_str ());
	
      else if (!nocase_cmp (toks[i], "Type"))
	i++;
 	
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = (SIMwareKBSenGrdTruth *) kb->getSensor (toks[++i].c_str ());
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <PlayerInfSkin::pSenGrdTruth> Touch sensor %s not found\n",
       		   toks[i].c_str ());
       	      return;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (time);
	}
      else if (!nocase_cmp (toks[i], "Location"))
	{
	  sPtr->setLocation (atof (toks[i+1].c_str ()),
			     atof (toks[i+2].c_str ()),
			     atof (toks[i+3].c_str ()));
	  i+=3;
	}
      else if (!nocase_cmp (toks[i], "Orientation"))
	{
	  sPtr->setOrientation (atof (toks[i+1].c_str ()),
				atof (toks[i+2].c_str ()),
				atof (toks[i+3].c_str ()));
	  i+=3;
	}

      else 
	i++;
    }
}


/ <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> 
/ <<<<<  PARSING FUNCTIONS FOR STA MESSAGES  >>>>>> 
void
PlayerInfSkin::parseMsg_STA (const vector < string > &toks)
{
  
  map < const char *, int, strCmp >::iterator iter;
  int type;
  int i=0;
  double t;
  SIMwareKBFlipper * flipper=NULL;

  if (0)
    printf ("PlayerInfSkin::parseMsg_STA\n");
  if (0)
    dumpTokens (toks);
  
  if (kb->getKBState () != SW_KB_STATE_INITIALIZED ) {
    if(0) printf ("<PlayerInfSkin::parseMsg_STA> discarding message\n");
    return;
  }
  
  // find 'Type' in tokens
  if(-1 == (i=findToken(toks,"Type")))
    if(1)
      {
 	printf("**Warning ** <PlayerInfSkin::parseMsg_STA> No Type found in message, discard message\n");
 	return;
      }
  
  if (botTypeMap.end () == (iter = botTypeMap.find (toks[++i].c_str ())))
    {
      if(1)
	printf ("** WARNING ** <PlayerInfSkin::parseMsg_STA>  Robot %s is not in the bot type map, discard message\n",
		toks[i].c_str ());
      return;
    }
  
  type=iter->second;
  
  //  kb->lockBotMutex();
  
  while ( ++i < (int)toks.size() )
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  kb->bot->setTimeStamp (t);
	}
      else if (!nocase_cmp (toks[i], "Location"))
	{
	  i += 3;
	}
      else if (!nocase_cmp (toks[i], "Orientation"))
	{ 
	  i += 3;
	}
      else if (!nocase_cmp (toks[i], "Velocity"))
	{
	  i += 3;
	}
      else if (!nocase_cmp (toks[i], "LightToggle"))
	{
	  kb->bot->setLightsOn( (!nocase_cmp (toks[++i], "True")) ? true : false );
	}
      else if (!nocase_cmp (toks[i], "LightIntensity"))
	{
	  kb->bot->setLightLevel (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "Battery"))
	{
	  kb->bot->setBatteryLevel (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "View"))
	{
	  //printf("> %s %s \n", toks[i].c_str(),toks[i+1].c_str());
	  ++i;
	}
      // Ground Vehicle specific parameters
      else if (!nocase_cmp (toks[i], "FrontSteer") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  ((SIMwareKBGroundVehicle *)kb->bot)->setSteerAngle (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "RearSteer") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  ((SIMwareKBGroundVehicle *)kb->bot)->setCrabAngle(atof (toks[++i].c_str ()));
	}

      else if (!nocase_cmp (toks[i], "FRFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = ((SIMwareKBGroundVehicle *)kb->bot)->getFlipper(SW_KB_FR_FLIPPER_TYPE)))
	    flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      else if (!nocase_cmp (toks[i], "FLFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = ((SIMwareKBGroundVehicle *)kb->bot)->getFlipper(SW_KB_FL_FLIPPER_TYPE)))
	    flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      else if (!nocase_cmp (toks[i], "RRFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = ((SIMwareKBGroundVehicle *)kb->bot)->getFlipper(SW_KB_RR_FLIPPER_TYPE)))
	    flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      else if (!nocase_cmp (toks[i], "RLFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = ((SIMwareKBGroundVehicle *)kb->bot)->getFlipper(SW_KB_RL_FLIPPER_TYPE)))
	    flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      // Nautical Robot specific parameters
      else if (!nocase_cmp (toks[i], "SternPlaneAngle") && type==SW_KB_NAUTIC_ROBOT_TYPE)
	{
	  ((SIMwareKBNauticRobot *)kb->bot)->setSternPlaneAngle (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "RudderAngle") && type==SW_KB_NAUTIC_ROBOT_TYPE)
	{
	  ((SIMwareKBNauticRobot *)kb->bot)->setRudderAngle(atof (toks[++i].c_str ()));
	}


      else 
	if(1)
	  printf
	    ("** WARNING ** <PlayerInfSkin::parseMsg_STA> Token <%s> is not a recognized token \n",
	     toks[i].c_str ());     
    }
    //  kb->unlockBotMutex();
}
*/

////////////////////////////////////////////////////////////////////////////////
// Player specific routines
///////////////////////////////////////////////////////////////////////////////

// Translates wheel rotation into speed, yaw speed.
/*
  This function translates left and right wheel velocities into a velocity and turn radius
  \param w1 Right wheel velocity
  \param w2 Left wheel velocity
  \param R wheel radius
  \param L_inv wheel base
  \param v Returns the velocity
  \param w Returns the theta
*/
void PlayerInfSkin::fwdJacobian(double w1, double w2, double R, double L_inv,
				       double *v, double *w)
{
  *v = 0.5 * R * (w1 + w2);
  *w = L_inv * R * (w1 - w2);
}


/*
  Modification history:

  $Log: playerInfSkin.cc,v $
  Revision 1.7  2009/02/17 14:57:35  dr_steveb
  Fixes so that player will work with both 2.0 and 2.1. Also added fresh tag to database for settings.

  Revision 1.6  2009/02/13 19:34:56  wshackle
  two changes needed to work with stage, just set vels to zero rather than setting enable to 0, reverse the direction of dTheta, plus the same change as usarInfSkin = work around for bad timer settings but probably a good idea anyway: wait until the cmdQ is empty before having the cmdQ reading thread sleep

  Revision 1.5  2009/02/04 15:22:21  wshackle
  allow moast to run on top of player/stage for testing-debug purposes.
  if player is not started or playerc_client_connect()  fails, shutdown more
  gracefully without the core dump.
  eliminate several compiler warnings about unused variables.

  Revision 1.4  2008/12/18 15:48:53  dr_steveb
  Added checking for null robot (the case of a disebodied sensor)

  Revision 1.3  2008/11/20 16:01:57  dr_steveb
  Fixed player skin to work with c interface of player 2.1.1

  Revision 1.2  2008/09/17 18:09:33  dr_steveb
  Changed to work with new simWare prototype

  Revision 1.1  2008/02/13 17:49:50  dr_steveb
  Initial check-in. Not quite ready for prime-time.

*/
