#include "simWareKB.hh"

enum {
  SEN_MUTEX_KEY = 1,
  MIS_MUTEX_KEY,
  EFF_MUTEX_KEY,
  BOT_MUTEX_KEY,
  CMDQ_MUTEX_KEY
};

SIMwareKB::SIMwareKB()
{
  state = SW_KB_STATE_UNINITIALIZED;

  senMutex = ulapi_mutex_new(SEN_MUTEX_KEY);
  misMutex = ulapi_mutex_new(MIS_MUTEX_KEY);
  effMutex = ulapi_mutex_new(EFF_MUTEX_KEY);
  botMutex = ulapi_mutex_new(BOT_MUTEX_KEY);
  cmdQMutex = ulapi_mutex_new(CMDQ_MUTEX_KEY);

  bot = NULL;
}

SIMwareKB::~SIMwareKB()
{
  map <const char *, SIMwareKBSen *, strCmp >::iterator sen_itr = senMap.begin();
  while ( sen_itr != senMap.end() ) {
    if( sen_itr->second != NULL )
      delete sen_itr->second;
    sen_itr++;
  }
  senMap.clear();

  map < const char *, SIMwareKBMisPkg *, strCmp >::iterator mis_itr = misMap.begin();
  while ( mis_itr != misMap.end() ) {
    if( mis_itr->second != NULL )
      delete mis_itr->second;
    mis_itr++;
  }
  misMap.clear();
  
  while(!cmdQ.empty()) {
    SIMwareCmdMsg * cMsg;
    cMsg = cmdQ.front();
    if(cMsg)
      {
	delete cMsg;
      }
    cmdQ.pop();
  }

  ulapi_mutex_delete(cmdQMutex);
  ulapi_mutex_delete(senMutex);
  ulapi_mutex_delete(misMutex);
  ulapi_mutex_delete(effMutex);
  ulapi_mutex_delete(botMutex);

  if (bot != NULL) {
    delete bot;
    bot = NULL;
  }
}

bool SIMwareKB::settingsFresh()
{
  return settingsAreFresh;
}
void SIMwareKB::setSettingsFresh( bool in )
{
  settingsAreFresh = in;
}


int SIMwareKB::lockSenMutex() 
{
  return ulapi_mutex_take(senMutex);
}
int SIMwareKB::lockMisMutex() 
{
  return ulapi_mutex_take(misMutex);
}
int SIMwareKB::lockEffMutex() 
{
  return ulapi_mutex_take(effMutex);
}
int SIMwareKB::lockBotMutex() 
{
  return ulapi_mutex_take(botMutex);
}


int SIMwareKB::unlockMisMutex() 
{
  return ulapi_mutex_give(misMutex);
}
int SIMwareKB::unlockEffMutex() 
{
  return ulapi_mutex_give(effMutex);
}
int SIMwareKB::unlockSenMutex() 
{
  return ulapi_mutex_give(senMutex);
}
int SIMwareKB::unlockBotMutex() 
{
  return ulapi_mutex_give(botMutex);
}

void SIMwareKB::addCmdToQ(SIMwareCmdMsg *cMsg)
{
  ulapi_mutex_take(cmdQMutex);
  cmdQ.push(cMsg);
  ulapi_mutex_give(cmdQMutex);
//   static int max_length = 0;
//   int len = cmdQ.size();
//   if(len > max_length)
//     {
//       printf("cmdQ.size() = %d\n",max_length);
//       max_length=len;
//     }
}
SIMwareCmdMsg * SIMwareKB::popCmdFromQ()
{ 
  SIMwareCmdMsg * cMsg;
  ulapi_mutex_take(cmdQMutex);
  if(cmdQ.empty())
    cMsg=NULL;
  else
    {
      cMsg=cmdQ.front();
      cmdQ.pop();
    }
  ulapi_mutex_give(cmdQMutex);
  return cMsg;
}

SIMwareKBSen *
SIMwareKB::getSensor (const string & name, bool insert,
		      SIMwareSensorType sType)
{
  map < const char *, SIMwareKBSen *, strCmp >::iterator itr;

  itr = senMap.find (name.c_str ());

  if (itr == senMap.end () && insert)
    {
      switch (sType)
	{
	case SW_KB_LINESCAN_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenLinescan (name);
	  break;
	case SW_KB_SONAR_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenSonar (name);
	  break;
	case SW_KB_INU_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenINU (name);
	  break;
	case SW_KB_INS_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenINS (name);
	  break;
	case SW_KB_GPS_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenGPS (name);
	  break;
	case SW_KB_ENCODER_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenEncoder (name);
	  break;
	case SW_KB_ODOMETRY_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenOdometry (name);
	  break;
	case SW_KB_TOUCH_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenTouch (name);
	  break;
	case SW_KB_RFID_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenRFID (name);
	  break;
	case SW_KB_VICTIM_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenVictim (name);
	  break;
	case SW_KB_CAMERA_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenCamera (name);
	  break;
	case SW_KB_GRD_TRUTH_SENSOR_TYPE:
	  senMap[name.c_str ()] = new SIMwareKBSenGrdTruth (name);
	  break;
	default:
	  printf
	    ("** WARNING ** <SIMwareKB::getSensor> Sensor %s not added, sensor type %d not valid sensor\n",
	     name.c_str (), sType);
	  return NULL;
	  break;

	}
      itr = senMap.find (name.c_str ());
    }
  
  if(itr == senMap.end())
    return NULL;
  else
    return itr->second;
}

SIMwareKBMisPkg *
SIMwareKB::getMisPkg (const string & name, bool insert)
{
  map < const char *, SIMwareKBMisPkg *, strCmp >::iterator itr;

  itr = misMap.find (name.c_str ());

  if (itr == misMap.end () && insert)
    {
      misMap[name.c_str ()] = new SIMwareKBMisPkg (name);
      itr = misMap.find (name.c_str ());   
    }
  
  if( itr == misMap.end ())
    return NULL;
  else
    return itr->second;
}


SIMwareKBEff *
SIMwareKB::getEffecter (const string & name, bool insert)
{
  map < const char *, SIMwareKBEff *, strCmp >::iterator itr;

  itr = effMap.find (name.c_str ());

  if (itr == effMap.end () && insert)
    {
      effMap[name.c_str ()] = new SIMwareKBEff (name);
      itr = effMap.find (name.c_str ());   
    }
  
  if( itr == effMap.end ())
    return NULL;
  else
    return itr->second;
}

SIMwareStateType SIMwareKB::getKBState()
{
  return state;
}

void SIMwareKB::setKBState(SIMwareStateType sType)
{
  state=sType;
}





