/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file simWareKB.hh

  \brief 
  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.16 $
  $Date: 2009/02/17 14:57:35 $
  \endcode

  \author Chris Scrapper
*/

#ifndef SIMWARE_KB_HH
#define SIMWARE_KB_HH

//! C++ STL Headers
#include <string>
#include <vector>
#include <map>
#include <queue>
using namespace std;
#include "ulapi.h"
#include "swKBBot.hh"
#include "swKBWorld.hh"
#include "swKBSen.hh"
#include "swKBMis.hh"
#include "swKBEff.hh"

#define SIMWARE_INIFILE_PATH "../etc/moast.ini"

class SIMwareKB {
private: 
  SIMwareStateType state;
  void * senMutex;
  void * misMutex;
  void * effMutex;
  void * botMutex;
  void * cmdQMutex;
  queue<SIMwareCmdMsg *> cmdQ;
  bool settingsAreFresh;
public:

  //! Gets the fresh flag
  bool settingsFresh();
  //! Sets the fresh flag
  void setSettingsFresh(bool in );

  map<const char *, SIMwareKBEff*, strCmp>effMap;
  int lockEffMutex();
  int unlockEffMutex();
  
  map<const char *, SIMwareKBMisPkg*, strCmp>misMap;
  int lockMisMutex();
  int unlockMisMutex();

  map<const char *, SIMwareKBSen *, strCmp>senMap;
  int lockSenMutex();
  int unlockSenMutex();
  
  SIMwareKBBot * bot;
  int lockBotMutex();
  int unlockBotMutex();

  SIMwareKBWorld world; 
 
  void addCmdToQ(SIMwareCmdMsg *);
  SIMwareCmdMsg * popCmdFromQ();
  
  //! Constructor
  SIMwareKB();
  //! Destructor
  ~SIMwareKB();

  //! Read the state of the knowledge base
  SIMwareStateType getKBState();
  //! Set the state of the knowledge base
  void setKBState(SIMwareStateType);
  //! Find and/or insert sensors into knowledge base
  SIMwareKBSen * getSensor(const string &, bool=false,
			   SIMwareSensorType=SW_KB_INVALID_SENSOR_TYPE); 
  //! Find and/or insert mission packages into knowledge base
  SIMwareKBMisPkg * getMisPkg(const string &, bool=false);
  //! Find and/or insert mission packages into knowledge base
  SIMwareKBEff * getEffecter(const string &, bool=false);
};


#endif
