#include "swKBSen.hh"

int SIMwareKBSen::senCount=0;

SIMwareKBSen::SIMwareKBSen (){
  sType = SW_KB_INVALID_SENSOR_TYPE;
  tStamp = 0.00;
  cTime = 0.00;
  sID=++senCount;
  mount = NULL;
}

SIMwareKBSen::~SIMwareKBSen ()
{
  if(mount!=NULL)
    delete mount;
  mount=NULL;
}

void
SIMwareKBSen::setName (string n)
{
  sName = n;
}

string SIMwareKBSen::getName ()
{
  return sName;
}

void
SIMwareKBSen::setSenID (int senID)
{
  sID = senID;
}

int
SIMwareKBSen::getSenID ()
{
  return sID;
}

void
SIMwareKBSen::setSensorType (SIMwareSensorType s)
{
  sType = s;
}

SIMwareSensorType SIMwareKBSen::getSensorType ()
{
  return sType;
}

PM_POSE SIMwareKBSen::getSen2Mount ()
{
  return senToMount;
}

void
SIMwareKBSen::setSen2MountXYZ (double x, double y, double z)
{
  senToMount.tran.x = x;
  senToMount.tran.y = y;
  senToMount.tran.z = z;
}

void
SIMwareKBSen::setSen2MountRPY (double r, double p, double y)
{
  senToMount.rot = PM_RPY(r,p,y);
}

void
SIMwareKBSen::setMount (SIMwareKBMount * m)
{
  mount = m;
}



SIMwareKBMount * 
SIMwareKBSen::getMount()
{
  return mount;
}

double
SIMwareKBSen::getCycleTime ()
{
  return cTime;
}

void
SIMwareKBSen::setCycleTime (double t)
{
  cTime = t;
}

double
SIMwareKBSen::getTimeStamp ()
{
  return tStamp;
}

void
SIMwareKBSen::setTimeStamp (double t)
{
  tStamp = t;
}


bool
SIMwareKBSen::hasNewData ()
{
  return dFlag;
}

void
SIMwareKBSen::resetNewDataFlag ()
{
  dFlag = false;
}

void
SIMwareKBSen::clearData ()
{
  dFlag = false;
  tStamp = 0.00;
}

/////  Odometry
SIMwareKBSenOdometry::SIMwareKBSenOdometry ()
{
  sType = SW_KB_ODOMETRY_SENSOR_TYPE;
  clearData ();
}

SIMwareKBSenOdometry::SIMwareKBSenOdometry (string name)
{
  setName (name);
  sType = SW_KB_ODOMETRY_SENSOR_TYPE;
  clearData ();
}

void
SIMwareKBSenOdometry::setX (double xVal)
{
  dFlag = true;
  x = xVal;
}

double
SIMwareKBSenOdometry::getX ()
{
  return x;
}

void
SIMwareKBSenOdometry::setY (double yVal)
{
  dFlag = true;
  y = yVal;
}

double
SIMwareKBSenOdometry::getY ()
{
  return y;
}

void
SIMwareKBSenOdometry::setOrient (double rot)
{
  dFlag = true;
  orient = ZeroTo2PI(rot);
}

double
SIMwareKBSenOdometry::getOrient ()
{
  return orient;
}

void
SIMwareKBSenOdometry::setOdometry (double xVal, double yVal, double rot)
{
  dFlag = true;
  x = xVal;
  y = yVal;
  orient = rot;
}

void
SIMwareKBSenOdometry::clearData ()
{
  SIMwareKBSen::clearData ();
  x = 0.00;
  y = 0.00;
  orient = 0.00;
}

//////   INU
SIMwareKBSenINU::SIMwareKBSenINU ()
{
  sType = SW_KB_INU_SENSOR_TYPE;
  clearData ();
}

SIMwareKBSenINU::SIMwareKBSenINU (string name)
{
  setName (name);
  sType = SW_KB_INU_SENSOR_TYPE;
  clearData ();
}

void
SIMwareKBSenINU::setOrientation (PM_RPY rpy)
{
  dFlag = true;
  orient = rpy;
}

void
SIMwareKBSenINU::setOrientation (double r, double p, double y)
{
  dFlag = true;
  orient.r = r;
  orient.p = p;
  orient.y = y;
}

PM_RPY SIMwareKBSenINU::getOrientation ()
{
  return orient;
}

void
SIMwareKBSenINU::clearData ()
{
  SIMwareKBSen::clearData ();
  orient.r = 0.00;
  orient.p = 0.00;
  orient.y = 0.00;
}
double SIMwareKBSenINU::getRoll()
{
  return orient.r;
}
double SIMwareKBSenINU::getPitch()
{
  return orient.p;
}
double SIMwareKBSenINU::getYaw()
{
  return orient.y;
}



//////   INS
SIMwareKBSenINS::SIMwareKBSenINS ()
{
  sType = SW_KB_INS_SENSOR_TYPE;
  clearData ();
}

SIMwareKBSenINS::SIMwareKBSenINS (string name)
{
  setName (name);
  sType = SW_KB_INS_SENSOR_TYPE;
  clearData ();
}

void
SIMwareKBSenINS::setOrientation (PM_RPY rpy)
{
  dFlag = true;
  this->rpy=rpy;
}

void
SIMwareKBSenINS::setOrientation (double r, double p, double y)
{
  dFlag = true;
  rpy.r=r;
  rpy.p=p;
  rpy.y=y;
}

void
SIMwareKBSenINS::setLocation (PM_CARTESIAN cart)
{
  dFlag = true;
  xyz=cart;
}
void
SIMwareKBSenINS::setLocation (double x, double y, double z)
{
  dFlag = true;
  xyz.x=x;
  xyz.y=y;
  xyz.z=z;
}

PM_RPY SIMwareKBSenINS::getOrientation ()
{
  return rpy;
}

PM_CARTESIAN SIMwareKBSenINS::getLocation ()
{
  return xyz;
}

double SIMwareKBSenINS::getRoll()
{
  return rpy.r;
}
double SIMwareKBSenINS::getPitch()
{
  return rpy.p;
}
double SIMwareKBSenINS::getYaw()
{
  return rpy.y;
}
double SIMwareKBSenINS::getX()
{
  return xyz.x;
}
double SIMwareKBSenINS::getY()
{
  return xyz.y;
}
double SIMwareKBSenINS::getZ()
{
  return xyz.z;
}
void
SIMwareKBSenINS::clearData ()
{
  SIMwareKBSen::clearData ();
  setLocation(0,0,0);
  setOrientation(0,0,0);
}


// GPS

SIMwareKBSenGPS::SIMwareKBSenGPS(void)
{
  sType = SW_KB_GPS_SENSOR_TYPE;
  clearData();
}

SIMwareKBSenGPS::SIMwareKBSenGPS(string name)
{
  setName(name);
  sType = SW_KB_GPS_SENSOR_TYPE;
  clearData();
}

void
SIMwareKBSenGPS::setLatitudeDegrees(int d)
{
  dFlag = true;
  LatitudeDegrees = d;
}

void
SIMwareKBSenGPS::setLatitudeMinutes(float m)
{
  dFlag = true;
  LatitudeMinutes = m;
}

void
SIMwareKBSenGPS::setLatitudeCardinality(char c)
{
  dFlag = true;
  LatitudeCardinality = c;
}

void
SIMwareKBSenGPS::setLongitudeDegrees(int d)
{
  dFlag = true;
  LongitudeDegrees = d;
}

void
SIMwareKBSenGPS::setLongitudeMinutes(float m)
{
  dFlag = true;
  LongitudeMinutes = m;
}

void
SIMwareKBSenGPS::setLongitudeCardinality(char c)
{
  dFlag = true;
  LongitudeCardinality = c;
}

void
SIMwareKBSenGPS::setFix(int f)
{
  dFlag = true;
  Fix = f;
}

void
SIMwareKBSenGPS::setSatellites(int s)
{
  dFlag = true;
  Satellites = s;
}

int
SIMwareKBSenGPS::getLatitudeDegrees(void)
{
  return LatitudeDegrees;
}

float
SIMwareKBSenGPS::getLatitudeMinutes(void)
{
  return LatitudeMinutes;
}

char
SIMwareKBSenGPS::getLatitudeCardinality(void)
{
  return LatitudeCardinality;
}

int
SIMwareKBSenGPS::getLongitudeDegrees(void)
{
  return LongitudeDegrees;
}

float
SIMwareKBSenGPS::getLongitudeMinutes(void)
{
  return LongitudeMinutes;
}

char
SIMwareKBSenGPS::getLongitudeCardinality(void)
{
  return LongitudeCardinality;
}

int
SIMwareKBSenGPS::getFix(void)
{
  return Fix;
}

int
SIMwareKBSenGPS::getSatellites(void)
{
  return Satellites;
}


void
SIMwareKBSenGPS::clearData(void)
{
  SIMwareKBSen::clearData();

  LatitudeDegrees = 0;
  LatitudeMinutes = 0;
  LatitudeCardinality = ' ';
  LongitudeDegrees = 0;
  LongitudeMinutes = 0;
  LongitudeCardinality = ' ';
  Fix = 0;
  Satellites = 0;
}

/////  Encoder
SIMwareKBSenEncoder::SIMwareKBSenEncoder ()
{
  sType = SW_KB_ENCODER_SENSOR_TYPE;
  clearData ();
  res = 0.00;
}

SIMwareKBSenEncoder::SIMwareKBSenEncoder (string name)
{
  sType = SW_KB_ENCODER_SENSOR_TYPE;
  setName (name);
  clearData ();
  res = 0.00;
}

void
SIMwareKBSenEncoder::setResolution (double r)
{
  res = r;
}

double
SIMwareKBSenEncoder::getResolution ()
{
  return res;
}

void
SIMwareKBSenEncoder::setTick (int t)
{
  tick = t;
}

int
SIMwareKBSenEncoder::getTick ()
{
  return tick;
}

void
SIMwareKBSenEncoder::clearData ()
{
  SIMwareKBSen::clearData ();
  tick = 0;
}

/////  Sonar
SIMwareKBSenSonar::SIMwareKBSenSonar ()
{
  sType = SW_KB_SONAR_SENSOR_TYPE;
  clearData ();
  maxRange = 0.00;
  minRange = 0.00;
  bDispersion = 0.00;

}

SIMwareKBSenSonar::SIMwareKBSenSonar (string name)
{
  sType = SW_KB_SONAR_SENSOR_TYPE;
  setName (name);
  clearData ();
  maxRange = 0.00;
  minRange = 0.00;
  bDispersion = 0.00;

}

void
SIMwareKBSenSonar::setMaxRange (double max)
{
  maxRange = max;
}

double
SIMwareKBSenSonar::getMaxRange ()
{
  return maxRange;
}

void
SIMwareKBSenSonar::setMinRange (double min)
{
  minRange = min;
}

double
SIMwareKBSenSonar::getMinRange ()
{
  return minRange;
}

void
SIMwareKBSenSonar::setBeamDispersion (double b)
{
  bDispersion = b;
}

double
SIMwareKBSenSonar::getBeamDispersion ()
{
  return bDispersion;
}

void
SIMwareKBSenSonar::setRangeReturn (double r)
{
  dFlag = true;
  rVal = r;
}

double
SIMwareKBSenSonar::getRangeReturn ()
{
  return rVal;
}

void
SIMwareKBSenSonar::clearData ()
{
  SIMwareKBSen::clearData ();
  rVal = 0.00;
}


/////  Linescanner
SIMwareKBSenLinescan::SIMwareKBSenLinescan ()
{
  sType = SW_KB_LINESCAN_SENSOR_TYPE;
  clearData ();
  maxRange = 0.00;
  minRange = 0.00;
  fov = 0.00;
  bSep = 0.00;
  bDispersion = 0.01;
}

SIMwareKBSenLinescan::SIMwareKBSenLinescan (string name)
{
  sType = SW_KB_LINESCAN_SENSOR_TYPE;
  setName (name);
  clearData ();
  maxRange = 0.00;
  minRange = 0.00;
  fov = 0.00;
  bSep = 0.00;
  bDispersion = 0.01;
}

void
SIMwareKBSenLinescan::setMaxRange (double max)
{
  maxRange = max;
}

double
SIMwareKBSenLinescan::getMaxRange ()
{
  return maxRange;
}

void
SIMwareKBSenLinescan::setMinRange (double min)
{
  minRange = min;
}

double
SIMwareKBSenLinescan::getMinRange ()
{
  return minRange;
}

void
SIMwareKBSenLinescan::setBeamSpacing (double b)
{
  bSep = b;
}

double
SIMwareKBSenLinescan::getBeamSpacing ()
{
  return bSep;
}
void
SIMwareKBSenLinescan::setBeamDispersion (double b)
{
  bDispersion = b;
}

double
SIMwareKBSenLinescan::getBeamDispersion ()
{
  return bDispersion;
}

void
SIMwareKBSenLinescan::setFov (double f)
{
  fov = f;
}

double
SIMwareKBSenLinescan::getFov ()
{
  return fov;
}

void
SIMwareKBSenLinescan::appendRangeReturn (double val)
{
  dFlag = true;
  retVals.push_back (val);
}

void
SIMwareKBSenLinescan::setRangeReturn (int i, double val)
{
  dFlag = true;
  retVals[i] = val;
}

void
SIMwareKBSenLinescan::setRangeReturn (const vector < double >&rVals)
{
  dFlag = true;
  retVals.assign (rVals.begin (), rVals.end ());
}

int
SIMwareKBSenLinescan::getReturnCount ()
{
  return retVals.size ();
}

double 
SIMwareKBSenLinescan::getRangeReturn (int index)
{
  if( index >= (int) retVals.size() )
    return -1;
  return retVals[index];
}
vector < double >
SIMwareKBSenLinescan::getReturns ()
{
  dFlag = false;
  return retVals;
}

void
SIMwareKBSenLinescan::clearData ()
{
  SIMwareKBSen::clearData ();
  retVals.clear ();
}

/////  Camera
SIMwareKBSenCamera::SIMwareKBSenCamera ()
{
  sType = SW_KB_CAMERA_SENSOR_TYPE;
  clearData ();
  fov = 0.00;
  maxFov = 0.00;
  minFov = 0.00;
}

SIMwareKBSenCamera::SIMwareKBSenCamera (string name)
{
  sType = SW_KB_CAMERA_SENSOR_TYPE;
  setName (name);
  clearData ();
  fov = 0.00;
  maxFov = 0.00;
  minFov = 0.00;
}

void
SIMwareKBSenCamera::setMaxFov (double m)
{
  maxFov = m;
}

double
SIMwareKBSenCamera::getMaxFov ()
{
  return maxFov;
}

void
SIMwareKBSenCamera::setMinFov (double m)
{
  minFov = m;
}

double
SIMwareKBSenCamera::getMinFov ()
{
  return minFov;
}

void
SIMwareKBSenCamera::setFov (double f)
{
  fov = f;
}

double
SIMwareKBSenCamera::getFov ()
{
  return fov;
}

void
SIMwareKBSenCamera::setDefFov (double f)
{
  defFov = f;
}

double
SIMwareKBSenCamera::getDefFov ()
{
  return defFov;
}

void
SIMwareKBSenCamera::clearData ()
{
  SIMwareKBSen::clearData ();
}

/////  RFID
SIMwareKBSenRFID::SIMwareKBSenRFID ()
{
  clearData ();
  sType = SW_KB_RFID_SENSOR_TYPE;
  maxRange = 0.00;
}

SIMwareKBSenRFID::SIMwareKBSenRFID (string name)
{
  sType = SW_KB_RFID_SENSOR_TYPE;
  setName (name);
  clearData ();
  maxRange = 0.00;
}

void
SIMwareKBSenRFID::setMaxRange (double m)
{
  maxRange = m;
}

double
SIMwareKBSenRFID::getMaxRange ()
{
  return maxRange;
}

int
SIMwareKBSenRFID::getReturnCount ()
{
  return rfidMsgs.size ();
}

bool SIMwareKBSenRFID::hasRfidMsg()
{
  bool isEmpty=rfidMsgs.empty();
  if( isEmpty )
    dFlag=false;
  return !isEmpty;
}

void SIMwareKBSenRFID::addRfidMsg (SIMwareRfidMsg vMsg)
{
  dFlag=true;
  rfidMsgs.push_back(vMsg);
}

SIMwareRfidMsg SIMwareKBSenRFID::popRfidMsg()
{
  SIMwareRfidMsg msg = rfidMsgs.front();
  rfidMsgs.pop_front();
  return msg;
}


void
SIMwareKBSenRFID::clearData ()
{
  SIMwareKBSen::clearData ();
  rfidMsgs.clear ();
}

/////  VICTIM SENSOR
SIMwareKBSenVictim::SIMwareKBSenVictim ()
{
  sType = SW_KB_VICTIM_SENSOR_TYPE;
  clearData ();
  maxRange = 0.00;
}

SIMwareKBSenVictim::SIMwareKBSenVictim (string name)
{
  sType = SW_KB_VICTIM_SENSOR_TYPE;
  setName (name);
  clearData ();
  maxRange = 0.00;
}

void
SIMwareKBSenVictim::setMaxRange (double mRange)
{
  maxRange = mRange;
}

double
SIMwareKBSenVictim::getMaxRange ()
{
  return maxRange;
}

void SIMwareKBSenVictim::setHorizontalFOV(double horizontalFOV)
{
  hFov= horizontalFOV;
}
double SIMwareKBSenVictim::getHorizontalFOV()
{
  return hFov;
}
void SIMwareKBSenVictim::setVerticalFOV(double verticalFOV)
{
  vFov = verticalFOV;
}
double SIMwareKBSenVictim::getVerticalFOV()
{
  return vFov;
}
bool SIMwareKBSenVictim::hasVictMsg()
{
  bool isEmpty=victMsgs.empty();
  if( isEmpty )
    dFlag=false;
  return !isEmpty;
}

void SIMwareKBSenVictim::addVictMsg (SIMwareVictimMsg vMsg)
{
  dFlag=true;
  victMsgs.push_back(vMsg);
}

SIMwareVictimMsg SIMwareKBSenVictim::popVictMsg()
{
  SIMwareVictimMsg msg = victMsgs.front();
  victMsgs.pop_front();
  return msg;
}

void
SIMwareKBSenVictim::clearData ()
{
  SIMwareKBSen::clearData ();
  victMsgs.clear ();
}

//////   Touch 
SIMwareKBSenTouch::SIMwareKBSenTouch ()
{
  dia = 0.00;
  clearData ();
}

SIMwareKBSenTouch::SIMwareKBSenTouch (string name)
{
  setName (name);
  dia = 0.00;
  clearData ();
}

double
SIMwareKBSenTouch::getDiameter ()
{
  return dia;
}

void
SIMwareKBSenTouch::setDiameter (double d)
{
  dia = d;
}

void
SIMwareKBSenTouch::clearData ()
{
  SIMwareKBSen::clearData ();
}

void
SIMwareKBSenTouch::setTouch (bool isTouching)
{
  dFlag=true;			
  touch=isTouching;
}

bool
SIMwareKBSenTouch::getTouch ()
{
  return touch;
}

//////   Ground Truth
SIMwareKBSenGrdTruth::SIMwareKBSenGrdTruth ()
{
  sType = SW_KB_GRD_TRUTH_SENSOR_TYPE;
  clearData ();
}

SIMwareKBSenGrdTruth::SIMwareKBSenGrdTruth (string name)
{
  sType = SW_KB_GRD_TRUTH_SENSOR_TYPE;
  setName (name);
  clearData ();
}

void SIMwareKBSenGrdTruth::clearData()
{
  SIMwareKBSen::clearData ();
  setLocation(0.00,0.00,0.00);
  setOrientation(0.00,0.00,0.00);
}

void SIMwareKBSenGrdTruth::setPose(PM_POSE pose)
{
  setOrientation(pose.rot);
  setLocation(pose.tran);
}
void SIMwareKBSenGrdTruth::setLocation(PM_CARTESIAN loc)
{
  dFlag = true;
  xyz = loc;
}
void SIMwareKBSenGrdTruth::setLocation(double x, double y, double z)
{
  dFlag = true;
  xyz.x=x;
  xyz.y=y;
  xyz.z=z;
}
void SIMwareKBSenGrdTruth::setOrientation(PM_RPY rpy)
{
  dFlag = true;
  this->rpy= rpy;
}
void SIMwareKBSenGrdTruth::setOrientation(double r, double p, double y)
{
  dFlag = true;
  rpy.r=r;
  rpy.p=p;
  rpy.y=y;
}
PM_CARTESIAN SIMwareKBSenGrdTruth::getLocation()
{
  return xyz;
}
PM_RPY SIMwareKBSenGrdTruth::getOrientation()
{
  return rpy;
}
double  SIMwareKBSenGrdTruth::getX()
{
  return xyz.x;
}
double  SIMwareKBSenGrdTruth::getY()
{
  return xyz.y;
}
double  SIMwareKBSenGrdTruth::getZ()
{
  return xyz.z;
}
double  SIMwareKBSenGrdTruth::getRoll()
{
  return rpy.r;
}
double  SIMwareKBSenGrdTruth::getPitch()
{
  return rpy.p;
}
double  SIMwareKBSenGrdTruth::getYaw()
{
  return rpy.y;
}
