#ifndef SIMWARE_KB_TYPES_HH
#define SIMWARE_KB_TYPES_HH

#define SW_KB_RFID_MSG_MAX 20

#define VOID_CAST(T,i) (reinterpret_cast<T>(i))
#define CLASS_CAST(T,i) (reinterpret_cast<T>(i))

typedef enum SIMWARE_EFF_OPCODE_TYPE
{
  SW_EFF_OPCODE_ACTIVATE_TYPE, //turn on
  SW_EFF_OPCODE_ANIMATE_TYPE,  // move 
  SW_EFF_OPCODE_FIRE_TYPE,     // trigger single action
  SW_EFF_OPCODE_RELEASE_TYPE,  // detach or drop
  SW_EFF_OPCODE_RESET_TYPE,    // refresh reload or zero out
  SW_EFF_OPCODE_NOP_TYPE,
  SW_EFF_OPCODE_INVALID_TYPE
} SIMwareEffOpcodeType;

typedef enum SIMWARE_VICTIM_PART_TYPE 
{
  SW_KB_VICTIM_HEAD_PART_TYPE,
  SW_KB_VICTIM_CHEST_PART_TYPE,
  SW_KB_VICTIM_PELVIS_PART_TYPE,
  SW_KB_VICTIM_LEG_PART_TYPE,
  SW_KB_VICTIM_FOOT_PART_TYPE, 
  SW_KB_VICTIM_ARM_PART_TYPE,
  SW_KB_VICTIM_HAND_PART_TYPE,
  SW_KB_VICTIM_UNKNOWN_PART_TYPE,
} SIMwareVictimPartType;

typedef enum SIMWARE_VERBOSITY_TYPES {
  SW_KB_DEBUG_OFF = 0x01,
  SW_KB_DEBUG_ON  = 0x02,
  SW_KB_DEBUG_INF_SKIN = 0x04,
  SW_KB_DEBUG_SUP_SKIN = 0x08,
  SW_KB_DEBUG_KB = 0x10,
  SW_KB_DEBUG_WARNING=0x20,
} SIMwareVerbosityType;

typedef enum SIMWARE_STATE_TYPES {
  SW_KB_STATE_NULL,
  SW_KB_STATE_UNINITIALIZED,
  SW_KB_STATE_INITIALIZED,
  SW_KB_STATE_SHUTDOWN
} SIMwareStateType;

#define SW_STATE_2_STRING(s) (s) == SW_KB_STATE_UNINITIALIZED ? "UNINITIALIZED" : (s) == SW_KB_STATE_INITIALIZED ? "INITIALIZED" : (s) == SW_KB_STATE_SHUTDOWN ? "SHUTDOWN" : "?"


typedef enum SIMWARE_JOINT_TYPE {
  SW_KB_PRISMATIC_JOINT_TYPE,
  SW_KB_SCISSOR_JOINT_TYPE,
  SW_KB_REVOLUTE_JOINT_TYPE,
  SW_KB_UNKNOWN_JOINT_TYPE
}SIMwareJointType;


typedef enum SIMWARE_ROBOT_TYPE {
  SW_KB_GROUND_VEHICLE_TYPE,
  SW_KB_LEGGED_ROBOT_TYPE,
  SW_KB_NAUTIC_ROBOT_TYPE,
  SW_KB_AERIAL_ROBOT_TYPE,
  SW_KB_UNKNOWN_ROBOT_TYPE
}SIMwareRobotType;

typedef enum SIMWARE_STEER_TYPES {
  SW_KB_INVALID_STEER_TYPE=-1,
  SW_KB_ACKERMAN_STEER_TYPE,
  SW_KB_SKID_STEER_TYPE,
  SW_KB_OMNI_STEER_TYPE,
  SW_KB_ROTARY_WING_STEER_TYPE,
  SW_KB_FIXED_WING_STEER_TYPE,
  SW_KB_SUBMARINE_STEER_TYPE,
  SW_KB_UNKNOWN_STEER_TYPE
} SIMwareSteerType;

#define SIMWARE_FLIPPER_MAX 4
typedef enum SIMWARE_FLIPPER_TYPES {
  SW_KB_FR_FLIPPER_TYPE,
  SW_KB_FL_FLIPPER_TYPE,
  SW_KB_RR_FLIPPER_TYPE,
  SW_KB_RL_FLIPPER_TYPE,
}SIMwareFlipperType;

typedef enum SIMWARE_SENSOR_TYPE {
  // start at 0 for _COUNT to be correct
  SW_KB_LINESCAN_SENSOR_TYPE = 0,
  SW_KB_SONAR_SENSOR_TYPE,
  SW_KB_INU_SENSOR_TYPE,
  SW_KB_INS_SENSOR_TYPE,
  SW_KB_GPS_SENSOR_TYPE,
  SW_KB_ENCODER_SENSOR_TYPE,
  SW_KB_ODOMETRY_SENSOR_TYPE,
  SW_KB_TOUCH_SENSOR_TYPE,
  SW_KB_RFID_SENSOR_TYPE,
  SW_KB_VICTIM_SENSOR_TYPE,
  SW_KB_CAMERA_SENSOR_TYPE,
  SW_KB_GRD_TRUTH_SENSOR_TYPE,
  // make this last for _COUNT to be correct
  SW_KB_INVALID_SENSOR_TYPE
} SIMwareSensorType;
#define SW_KB_SENSOR_TYPE_COUNT SW_KB_INVALID_SENSOR_TYPE

typedef enum SIMWARE_EFFECTOR_TYPE {
  // start at 0 for _COUNT to be correct
  SW_KB_GRIPPER_EFFECTOR_TYPE = 0,
  SW_KB_RFID_RELEASER_EFFECTOR_TYPE,
  // make this last for _COUNT to be correct
  SW_KB_INVALID_EFFECTOR_TYPE
} SIMwareEffectorType;
#define SW_KB_EFFECTOR_TYPE_COUNT SW_KB_INVALID_EFFECTOR_TYPE


////////////// SIMWARE COMMAND TYPES //////////////
typedef enum SIMWARE_CMD_TYPE {
  SW_MOB_CMD_TYPE,
  SW_MIS_CMD_TYPE,
  SW_EFF_CMD_TYPE,
  SW_SEN_CMD_TYPE,
  SW_SET_CMD_TYPE, 
}SIMwareCmdType;

////////////// SIMWARE MOB CMD TYPES  //////////////
typedef enum SIMWARE_MOB_CMD_TYPE {
  SW_MOB_CMD_INIT_TYPE,
  SW_MOB_CMD_POWERDOWN_TYPE,
  SW_MOB_CMD_DRV_ACKERMAN_TYPE,
  SW_MOB_CMD_DRV_SKID_TYPE,
  SW_MOB_CMD_DRV_OMNI_TYPE,
  SW_MOB_CMD_DRV_SUBMARINE_TYPE,
  SW_MOB_CMD_DRV_ROTARY_WING_TYPE,
  SW_MOB_CMD_DRV_FLIPPER_TYPE,
}SIMwareMobCmdType;

typedef enum SIMWARE_SEN_CMD_TYPE {
  SW_SEN_CMD_INIT_TYPE,
  SW_SEN_CMD_RESET_TYPE,
  SW_SEN_CMD_POWERDOWN_TYPE
}SIMwareSenCmdType;


typedef enum SIMWARE_EFF_CMD_TYPE {
  SW_EFF_CMD_INIT_TYPE,
  SW_EFF_CMD_POWERDOWN_TYPE,
  SW_EFF_CMD_OPCODE_TYPE
}SIMwareEffCmdType;

typedef enum SIMWARE_MIS_CMD_TYPE {
  SW_MIS_CMD_INIT_TYPE,
  SW_MIS_CMD_POWERDOWN_TYPE,
  SW_MIS_CMD_MOVE_TYPE
}SIMwareMisCmdType;

typedef enum SIMWARE_MIS_LINK_CMD_TYPE {
  SW_MIS_LINK_CMD_INVALID_TYPE = -1,
  SW_MIS_LINK_CMD_ABS_VAL_TYPE,
  SW_MIS_LINK_CMD_VELOCITY_TYPE,
  SW_MIS_LINK_CMD_TORQUE_TYPE
}SIMwareMisLinkCmdType;


#endif
