#include "swKBWorld.hh"
#include "simWareKB.hh"

SIMwareKBWorld::SIMwareKBWorld ()
{
  wName.clear ();
  sPoseMap.clear();
  sPoseName.clear();
  startPose = PM_POSE();
}

SIMwareKBWorld::~SIMwareKBWorld () {}

string 
SIMwareKBWorld::getWorldName () 
{   
  return wName; 
}

void
SIMwareKBWorld::setWorldName (string worldName)
{
  wName = worldName;
}

int 
SIMwareKBWorld::insertStartPose(string spName, PM_POSE pose)
{
  int retval = 0;
  map <string, PM_POSE>::iterator itr;
  itr = sPoseMap.find(spName);
  if( itr == sPoseMap.end() )
    {
      sPoseMap[spName]=pose;
      retval = 1;
    }
  
  return retval;
}

void 
SIMwareKBWorld::dumpWorldInfo()
{
  PM_RPY rpy;
  map <string, PM_POSE>::iterator itr;
  printf("Starting poses for world %s\n", wName.c_str());
  for(itr=sPoseMap.begin();itr!=sPoseMap.end();itr++)
    {
      rpy = itr->second.rot;
      printf("%s %f %f %f %f %f %f\n", 
	     itr->first.c_str(),
	     itr->second.tran.x,itr->second.tran.y,itr->second.tran.z,
	     rpy.r, rpy.p, rpy.y);
    }
}

void SIMwareKBWorld::setStartPoseOffset(PM_POSE pOffset)
{
  sPoseOffset = pOffset;
}

int SIMwareKBWorld::setStartPose(string poseName)
{
  int rval;
  map <string, PM_POSE>::iterator itr;
  
  if( sPoseMap.end() != (itr = sPoseMap.find(poseName)))
    {
      sPoseName = poseName;
      startPose = itr->second;
      startPose.rot = startPose.rot + sPoseOffset.rot;
      startPose.tran = startPose.tran + sPoseOffset.tran;
      rval = 1;
    }
  else if (!nocase_cmp (poseName, "default"))
    {
      itr = sPoseMap.begin();
      sPoseName = itr->first;
      startPose = itr->second;
      startPose.rot = startPose.rot + sPoseOffset.rot;
      startPose.tran = startPose.tran + sPoseOffset.tran;
      rval=0;
    }
  else if (!nocase_cmp (poseName, "offset"))
    {
      sPoseName = poseName;
      startPose.rot = sPoseOffset.rot;
      startPose.tran = sPoseOffset.tran;
      rval=0;
    }
  else 
    rval=-1;
  
  return rval;
}


string SIMwareKBWorld::getStartPoseName(int pID)
{
  string name = "";
  printf("<SIMwareKBWorld::getStartPoseName>\n");
  map <string, PM_POSE>::iterator itr; 
  if( pID <= (int)sPoseMap.size())
    {  
      itr=sPoseMap.begin();
      for(int i=0;i<pID;i++)
	itr++;
      name = itr->first;
    }
  return name;
}

PM_POSE SIMwareKBWorld::getStartPose()
{
  return startPose;
}
