#include <stdio.h>		// printf
#include <float.h>		// DBL_MIN,MAX
#include <rcs.hh>		// etime
#include "swServoNodeBase.hh"
#include "moastSupSkin.hh"

SIMwareServoNodeBase::SIMwareServoNodeBase(MoastSupSkin * m)
{
  cmdBuf=NULL;
  statBuf=NULL;
  setBuf=NULL;
  cfgBuf=NULL;
  statMsg = NULL;
  setMsg=NULL;
  mSkin=m;
  timer = new RCS_TIMER(SERVO_NODE_CYCLE_TIME_DEFAULT);
}

SIMwareServoNodeBase::~SIMwareServoNodeBase() 
{
  if(0)
    printf("<SIMwareServoNodeBase::~SIMwareServoNodeBase> : %s\n",nodeName.c_str());
 
  cleanUpNMLBuf();
  if(timer)
    {
      delete timer;
      timer=0;
    }
  if(statMsg)
    {
      delete statMsg;
      statMsg=0;
      //statMsg->admin_state=ADMIN_SHUT_DOWN;
    }
  if(setMsg)
    {
      delete setMsg;
      setMsg=0;
    }

}

void SIMwareServoNodeBase::run() 
{
  if(0)
    printf("<SIMwareServoNodeBase::run> :  %s\n",nodeName.c_str());
  this->publishSettings();
  while( mSkin->state !=  SW_KB_STATE_SHUTDOWN) 
    {
      doSingleStep();
      timer->wait();
    }
}

SIMwareStateType SIMwareServoNodeBase::getNodeState() {
  switch(statMsg->admin_state) {
  case ADMIN_UNINITIALIZED:
    return SW_KB_STATE_UNINITIALIZED;
  case ADMIN_INITIALIZED:
    return SW_KB_STATE_INITIALIZED;
  case ADMIN_SHUT_DOWN:
    return SW_KB_STATE_SHUTDOWN;
  default:
    return SW_KB_STATE_NULL;
  }
}

void SIMwareServoNodeBase::setTimer(double t) 
{
  delete timer;
  timer=new RCS_TIMER(t);
}


bool SIMwareServoNodeBase::openNML(string cmdName, 
				   string statName, 
				   string cfgName,
				   string setName,
				   NML_FORMAT_PTR fPtr,
				   int compID) {
  stringstream str; 
  char suffix[10];
  
  if(1)
    printf("<SIMwareServoNodeBase::openNML> Opening NML Buffers\n");
  
  // Check to see if there is a component id, this changes naming convention
  if(compID==0)
    sprintf(suffix,"%d",mSkin->kb->bot->getBotID());
  else
    sprintf(suffix,"%d_%d",mSkin->kb->bot->getBotID(),compID);
  
  str << cmdName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  cmdBuf = new RCS_CMD_CHANNEL(fPtr, 
			       str.str().c_str(), 
			       mSkin->nmlProcName.c_str(), 
			       mSkin->nmlCfgFile.c_str());
  if (!cmdBuf->valid()) 
    {
      delete cmdBuf;
      cmdBuf = NULL;
      return false;
    }
  cmdBuf->clear();
  str.str("");


  str << cfgName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  cfgBuf = new RCS_CMD_CHANNEL(fPtr, 
			       str.str().c_str(), 
			       mSkin->nmlProcName.c_str(), 
			       mSkin->nmlCfgFile.c_str());
  if (!cfgBuf->valid()) 
    {
      delete cfgBuf;
      cfgBuf = NULL;
      return false;
    }
  cfgBuf->clear();
  str.str("");
  
  str << statName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  statBuf = new RCS_STAT_CHANNEL(fPtr, 
				 str.str().c_str(), 
				 mSkin->nmlProcName.c_str(), 
				 mSkin->nmlCfgFile.c_str());
  if (!statBuf->valid()) 
    {
      delete statBuf;
      statBuf = NULL;
      return false;
    }
  statBuf->clear();
  str.str("");
  
  
  str << setName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  setBuf = new RCS_STAT_CHANNEL(fPtr, 
				str.str().c_str(), 
				mSkin->nmlProcName.c_str(), 
				mSkin->nmlCfgFile.c_str());
  if (!setBuf->valid()) 
    {
      delete setBuf;
      setBuf = NULL;
      return false;
    }
  setBuf->clear();
  str.str("");
  return true;
}

void SIMwareServoNodeBase::cleanUpNMLBuf()
{
  //printf("<SIMwareServoNodeBase::cleanUpNMLBuf> : %s\n",nodeName.c_str());  
  if (NULL != cmdBuf)
      delete cmdBuf;
    cmdBuf=NULL;
  
    if (NULL != statBuf)
      delete statBuf;
    statBuf=NULL;
  
    if (NULL != setBuf)
      delete setBuf;
    setBuf=NULL;
  
    if (NULL != cfgBuf)
      delete cfgBuf;
    cfgBuf=NULL;
} 
