#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stddef.h>		// sizeof(), NULL
#include <stdlib.h>		// getenv()
#include <ctype.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
#include "getopt.h"
#endif
#ifdef HAVE_READLINE_READLINE_H
#include <readline/readline.h>
#include <readline/history.h>
#endif

#include <signal.h>

#include <vector>
#include <string>
#include <iostream>
using namespace std;


#include "usarInfSkin.hh"
#include "trafficSupSkin.hh"
#include "simWareKB.hh"

////////////////////////////////////////////////////////
// defines
///////////////////////////////////////////////////////
#define DEFAULT_INI_FILE "../etc/traffic.ini"
char *SIMWARE_INIFILE_PATH;

////////////////////////////////////////////////////////
// structures
///////////////////////////////////////////////////////
typedef struct{
  char platform[80];
  PM_CARTESIAN location;
  PM_RPY rotation;
}TrafficVehicleSpawn;

SIMwareInfSkin ** infSkin;
SIMwareSupSkin * supSkin;
SIMwareKB ** kb;

int vehNum = 2;
int platTypeID = 1;
int verbosity = 0;

void swShutdown() 
{
  supSkin->closeSkin();
  for(int i=0;i<vehNum;i++)
    infSkin[i]->closeSkin();
  exit(1);
}

void
sigHandler (int signum)
{ 
  switch (signum)
    {
    case SIGINT:
      fprintf (stderr, "<<simWare::sigHandler> Recv SIGINT  : Shutting down simWare program\n");
      swShutdown();
      exit (1);
    case SIGQUIT:
      fprintf (stderr, "<simWare::sigHandler> Recv SIGQUIT : Shutting down simWare program\n");
      swShutdown();
      exit (1);
    default:
      break;
    }
}

#if ! defined(INI_OK)
#define INI_OK      0
#define INI_DEFAULT 1
#define INI_INVALID 2
#endif
static int iniLoad(const char * prog, const char * filename, TrafficVehicleSpawn *spawnPoint, 
		   int vehID, const char *section)
{
  INIFILE inifile;
  char  key[80];
  const char * inistring;
  int retval = INI_OK;

  if (0 != inifile.open(filename)) {
    rcs_print_error("%s: can't open INI file %s, exiting\n", prog, filename);
    return INI_INVALID;
  }

  sprintf( key, "PLATFORM_TYPE_%d", vehID);
  if (NULL != (inistring = inifile.find(key, section))) 
    {
      if (1 != sscanf(inistring, "%s", spawnPoint->platform) )
	{
	  // found, but invalid
	  rcs_print_error("%s: invalid value '%s' for [%s] %s in '%s'\n", prog,
			  inistring, section, key, filename);
	  return INI_INVALID;
	}
    }
  else 
    {
      // not found at all
      rcs_print_error("%s: can't find [%s] %s in '%s'\n", prog, section, key, filename);
      return INI_INVALID;
    }
  sprintf( key, "UTM_START_POSE_%d", vehID);
  if (NULL != (inistring = inifile.find(key, section))) 
    {
      if (6 != sscanf(inistring, "%lf %lf %lf %lf %lf %lf", 
		      &spawnPoint->location.x,
		      &spawnPoint->location.y,
		      &spawnPoint->location.z,
		      &spawnPoint->rotation.r,
		      &spawnPoint->rotation.p,
		      &spawnPoint->rotation.y) )
	{
	  // found, but invalid
	  rcs_print_error("%s: invalid value '%s' for [%s] %s in '%s'\n", prog,
			  inistring, section, key, filename);
	  return INI_INVALID;
	}
    }
  else 
    {
      // not found at all
      rcs_print_error("%s: can't find [%s] %s in '%s'\n", prog, section, key, filename);
      return INI_INVALID;
    }
  inifile.close();
  return retval;
}


#define SW_STATE_2_STRING(s) (s) == SW_KB_STATE_UNINITIALIZED ? "UNINITIALIZED" : (s) == SW_KB_STATE_INITIALIZED ? "INITIALIZED" : (s) == SW_KB_STATE_SHUTDOWN ? "SHUTDOWN" : "?"



int
main (int argc, char *argv[])
{
  char option;
  char *INI_FILE = NULL;
  char *ini_file_env;
  char ini_file_default[] = DEFAULT_INI_FILE;
  TrafficVehicleSpawn spawnPoint;
  //  string botName;
  char botName[MOAST_WORLD_NAME_LEN];
  stringstream intOut;
  string trashString;

  SIMWARE_INIFILE_PATH = DEFINED_SIMWARE_INIFILE_PATH;
  // set up signals
  if( signal (SIGINT, sigHandler) == SIG_ERR )
    {
      perror (" Could not set signal SIGINT");
      exit(1);
    }
  if( signal (SIGQUIT, sigHandler)  == SIG_ERR )
    {
      perror (" Could not set signal SIGQUIT");
      exit(1);
    }
  
  // set options  
  while (1)
    {
      option = getopt (argc, argv, "p:n:s:v:i:");
      if (option == -1)
	break;

      switch (option)
	{
	case 'i':
	  INI_FILE = optarg;
	  break;
	case 'p':
	  if (1 != sscanf (optarg, "%i", &platTypeID))
	    {
	      fprintf (stderr, "%s: bad value for vehicle ID: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	case 'n':
	  if (1 != sscanf (optarg, "%i", &vehNum))
	    {
	      fprintf (stderr, "%s: bad value for vehicle number: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	  /*
	case 's':
	  if (1 != sscanf (optarg, "%i", &stPoseID))
	    {
	      fprintf (stderr, "%s: bad value for starting pose ID: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	  */
	case 'v':
	  if (1 != sscanf (optarg, "%i", &verbosity))
	    {
	      fprintf (stderr, "%s: bad value for verbosity level: %s\n",
		       argv[0], optarg);
	      return 1;
	    }
	  break;
	default:
	  printf
	    ("\nUsage: -p<platform id> -i<vehID> -n<num vehicles> -v<verbosity level>\n");
	  return -1;
	  break;
	}
    }

  printf("TrafficSimWare robot count: %d\n", vehNum);
  // read in the INI file params
  if (NULL == INI_FILE) {
    ini_file_env = getenv("TRAFFIC_INI");
    if (NULL != ini_file_env) {
      INI_FILE = ini_file_env;
    } else {
      INI_FILE = ini_file_default;
    }
  }
  //  spawnPoints = new TrafficVehicleSpawn[vehNum];
  kb = new SIMwareKB * [vehNum];
  infSkin = new SIMwareInfSkin * [vehNum];
  supSkin = new TrafficSupSkin;

  for(int i=0;i<vehNum;i++)
    {
      kb[i]=new SIMwareKB;
      infSkin[i]=new USARSimInfSkin;
      infSkin[i]->configInfSkin(kb[i],supSkin);
      if(!infSkin[i]->openSkin())
	swShutdown();
      printf( "%s\n", kb[0]->world.getWorldName().c_str());
      if (INI_INVALID == iniLoad(argv[0], INI_FILE, &spawnPoint, i+1, 
				 kb[i]->world.getWorldName().c_str())) 
	{
	  rcs_print_error("%s: error loading INI file '%s'\n", argv[0], INI_FILE);
	  return 1;
	}
      intOut << i;
      sprintf( botName, "%s%d", spawnPoint.platform, i ); //+ intOut.str();
      printf( "botName: %s\n", botName);
      if( ! infSkin[i]->startBot(i+1,spawnPoint.platform,
				 botName,
				 spawnPoint.location,
				 spawnPoint.rotation) )
	swShutdown();
      kb[i]->bot->setName( botName );
    }
  
  ((TrafficSupSkin *)supSkin)->configSkin(kb,infSkin, vehNum);
  if(!supSkin->openSkin())
    swShutdown();
  
   while(1)
    {
      esleep(1);
    }

  return 0;
}



