#include <stdlib.h>		// getenv
#include <errno.h>		// errno
#include <signal.h>		// raise, SIGINT

#include <string>
#include <sstream>
#include <vector>
#include <map>

#include "rcs.hh"

#include "ulapi.h"
#include "swBaseSkin.hh"
#include "swKBTypes.hh"
#include "swKBUtils.hh"
#include "usarInfSkin.hh"

#undef VERBOSE

using namespace std;

void
startParserThr (void *tArgs)
{
  USARSimInfSkin *skin = VOID_CAST(USARSimInfSkin *, tArgs);
  skin->runParser ();
  //printf("<USARSimInfSkin::startParserThr> exiting\n");
  ulapi_task_exit();
}

void
startCmdThr (void *tArgs)
{
  USARSimInfSkin *skin = VOID_CAST(USARSimInfSkin *, tArgs);
  skin->runCmdHandler ();
  //printf("<USARSimInfSkin::startCmdThr> exiting\n");
  ulapi_task_exit();
}

USARSimInfSkin::USARSimInfSkin():
  socket_fd(-1),
  portNumber(3000),
  hostname("localhost"),
  traceInterval(-1),
  parThr(0),
  cmdThr(0),
  syncCond(0),
  syncMutex(0)
{
  state = SW_KB_STATE_UNINITIALIZED;
  portNumber = 3000;
  hostname = "localhost";
  traceInterval = -1;
  socket_fd = -1;
}

USARSimInfSkin::~USARSimInfSkin() 
{
  if(parThr)
    {
      ulapi_task_delete(parThr);
      parThr=0;
    }
  if(cmdThr)
    {
      ulapi_task_delete(cmdThr);
      cmdThr=0;
    }
}
 
enum {
  SYNC_COND_KEY = 1
};

enum {
  SYNC_MUTEX_KEY = 1
};

bool USARSimInfSkin::configInfSkin(SIMwareKB * kBase, SIMwareSupSkin * sSkin) 
{
  INIFILE if_d;
  const char *strPtr;
  
  kb=kBase;
  supSkin = sSkin;
    
  // initialize mutexes
  syncCond = ulapi_cond_new(SYNC_COND_KEY);
  syncMutex = ulapi_mutex_new(SYNC_MUTEX_KEY);

  // FIXME use environmental variables for parameters
  
  //printf ("<USARSimInfSkin::openSkin>: Open inifile %s\n", SIMWARE_INIFILE_PATH);
  if (if_d.open (SIMWARE_INIFILE_PATH))
    {
      printf
	("** ERROR ** <USARSimInfSkin::configInfSkin> invalid inifile path %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }

  //printf("<USARSimInfSkin::openSkin>: Get USARSim hostname\n");
  strPtr = getenv("USARSimHost");
  if(strPtr == NULL)
    {
      strPtr = if_d.find ("HOST_NAME", "USARSIM.INFSKIN");
      if(strPtr == NULL)
	{
	  printf
	    ("** ERROR ** <USARSimInfSkin::configInfSkin> 'HOST_NAME' not defined in section 'USARSIM.INFSKIN' in %s\n",
	     SIMWARE_INIFILE_PATH);
	  if_d.close ();
	  return false;    
	}
    }
  hostname = strPtr;  
  
  
  //printf("<USARSimInfSkin::openSkin>: Get USARSim port number\n");
  strPtr = getenv("USARSimPortNumber");
  if(strPtr == NULL)
    {
      strPtr = if_d.find ("PORT_NUMBER", "USARSIM.INFSKIN");
      if(strPtr == NULL)
	{
	  printf("<USARSimInfSkin::configInfSkin> ** ERROR ** 'PORT_NUMBER' not defined\n");
	  if_d.close ();
	  return false;    
	}
    }
  portNumber = atoi (strPtr);
  
  
  //printf("<USARSimInfSkin::openSkin>: Get trace interval from inifile\n");
  if (NULL == (strPtr = if_d.find ("TRACE_FUNCTION", "USARSIM.INFSKIN")))
    {
      printf
	("** ERROR ** <USARSimInfSkin::configInfSkin> 'TRACE_FUNCTION' not defined in section 'USARSIM.INIFILE' in %s\n",
	 SIMWARE_INIFILE_PATH);
      if_d.close ();
      return false;
    }
  traceInterval = atof (strPtr);

  if_d.close ();
  
  if(1)
    {
      printf ("<USARSimInfSkin:configInfSkin>\n");
      printf("\tUSARSim hostname   : %s\n", hostname.c_str());
      printf("\tUSARSim portnumber : %d\n", portNumber);
      printf("\tTrace function     : %lf\n", traceInterval);
    }


  // Robot Types
  botTypeMap["GroundVehicle"] = SW_KB_GROUND_VEHICLE_TYPE;
  botTypeMap["NauticVehicle"] = SW_KB_NAUTIC_ROBOT_TYPE;
  botTypeMap["AerialVehicle"] = SW_KB_AERIAL_ROBOT_TYPE;
  
  // Steering types
  steerNameMap["AckermanSteered"] = SW_KB_ACKERMAN_STEER_TYPE;
  steerNameMap["SkidSteered"] = SW_KB_SKID_STEER_TYPE;
  steerNameMap["Omni"] = SW_KB_OMNI_STEER_TYPE;
  steerNameMap["VariableDepth"] = SW_KB_SUBMARINE_STEER_TYPE;
  steerNameMap["RotaryWing"] = SW_KB_ROTARY_WING_STEER_TYPE;
  steerNameMap["Unknown"] = SW_KB_UNKNOWN_STEER_TYPE;

  //Sensor Types
  senNameMap["RangeScanner"] = SW_KB_LINESCAN_SENSOR_TYPE;
  senNameMap["Sonar"] = SW_KB_SONAR_SENSOR_TYPE;
  senNameMap["INU"] = SW_KB_INU_SENSOR_TYPE;
  senNameMap["INS"] = SW_KB_INS_SENSOR_TYPE;
  senNameMap["Encoder"] = SW_KB_ENCODER_SENSOR_TYPE;
  senNameMap["Odometry"] = SW_KB_ODOMETRY_SENSOR_TYPE;
  senNameMap["Touch"] = SW_KB_TOUCH_SENSOR_TYPE;
  senNameMap["RFID"] = SW_KB_RFID_SENSOR_TYPE;
  senNameMap["VictSensor"] = SW_KB_VICTIM_SENSOR_TYPE;
  senNameMap["Camera"] = SW_KB_CAMERA_SENSOR_TYPE;
  senNameMap["GroundTruth"] = SW_KB_GRD_TRUTH_SENSOR_TYPE;
  senNameMap["GPS"] = SW_KB_GPS_SENSOR_TYPE; // Added by Ben for GPS sensor support

  effOpcodeNameMap["Activate"]=SW_EFF_OPCODE_ACTIVATE_TYPE;
  effOpcodeNameMap["Animate"]=SW_EFF_OPCODE_ANIMATE_TYPE;
  effOpcodeNameMap["Fire"]=SW_EFF_OPCODE_FIRE_TYPE;
  effOpcodeNameMap["Release"]=SW_EFF_OPCODE_RELEASE_TYPE;
  effOpcodeNameMap["Reset"]=SW_EFF_OPCODE_RESET_TYPE;
  effOpcodeNameMap["NOP"]=SW_EFF_OPCODE_NOP_TYPE;


  return true;
}

bool USARSimInfSkin::openSkin() 
{ 
  int retval;
  
  
  if (state != SW_KB_STATE_UNINITIALIZED)
    return false;

  socket_fd = ulapi_socket_client(portNumber, const_cast<char *>(hostname.c_str()));
 
  // printf ("<USARSimInfSkin:openSkin> : locking sync mutex\n");
  // Wait for NFO message to be fully initialized
  ulapi_mutex_take(syncMutex);
  
  // printf ("<USARSimInfSkin:openSkin>: Spawning Parser Thread\n");
  parThr = ulapi_task_new();
  retval =
    ulapi_task_start(parThr, startParserThr, VOID_CAST(void *, this), ulapi_prio_highest(), 1);
  if (retval != 0)
    {
      printf ("**Error ** <USARSimInfSkin:openSkin>: spawning parser thread\n");
      return false;
    }
 
  // printf ("<USARSimInfSkin:openSkin>: conditional wait\n");
  // Wait til NFO msg recieved
  ulapi_cond_wait(syncCond, syncMutex);

  // printf ("<USARSimInfSkin:openSkin> : unlocking mutex\n");
  ulapi_mutex_give(syncMutex);
  
  //printf ("<USARSimInfSkin:openSkin>: Spawning Cmd Thread\n");
  cmdThr = ulapi_task_new();
  retval =
    ulapi_task_start(cmdThr, startCmdThr, VOID_CAST(void *, this), ulapi_prio_highest(), 1);
  if (retval != 0)
    {
      printf ("**Error ** <USARSimInfSkin:openSkin>: spawning cmd thread\n");
      return false;
    }
  
  if( state != SW_KB_STATE_INITIALIZED)
    return false;
  else
    return true;
}

void USARSimInfSkin::closeSkin() 
{
  printf ("<USARSimInfSkin::closeSkin> : closing USARSim inferior skin\n");
  state = SW_KB_STATE_SHUTDOWN;
  
  printf ("<USARSimInfSkin::closeSkin> : waiting for cmd thread to terminate\n");
  ulapi_task_join(cmdThr);
  
  printf ("<USARSimInfSkin::closeSkin> : waiting for parser thread to terminate\n");
  ulapi_task_join(parThr);
  
  printf ("<USARSimInfSkin::closeSkin> : closing Socket\n");
  if (socket_fd != -1)
    ulapi_socket_close(socket_fd);
  socket_fd = -1;
  
  if(syncCond) {
    void *vp=syncCond;
    syncCond=0;
    ulapi_cond_delete(vp);
  }
  if(syncMutex) {
    void *vp=syncMutex;
    syncMutex=0;		
    ulapi_mutex_delete(vp);
  }
}

int
USARSimInfSkin::writeMsg (string msg)
{
  int outChars = 0;

#ifdef VERBOSE
    printf ("<USARSimInfSkin::writeMsg> : Writing Outgoing Message: \n\t%s\n", msg.c_str ());
#endif

  msg += "\r\n";
  outChars = ulapi_socket_write(socket_fd, const_cast<char *>(msg.c_str ()), msg.size ());
  return outChars;
}


void USARSimInfSkin::runCmdHandler()
{
  RCS_TIMER cTimer(0.05);
  static SIMwareCmdMsg * cmdMsg;
  while (state != SW_KB_STATE_SHUTDOWN)
    {
      cmdMsg=kb->popCmdFromQ();
       
      // Process messages and don't sleep until the cmdQ is empty.
      while(state != SW_KB_STATE_SHUTDOWN && cmdMsg != NULL)
	{
	  switch(cmdMsg->getCmdType())
	    {
	    case SW_MOB_CMD_TYPE:
	      doMobCmd(CLASS_CAST(SIMwareMobCmd *, cmdMsg));
	      break;
	    case SW_MIS_CMD_TYPE:
	      doMisCmd(CLASS_CAST(SIMwareMisCmd *, cmdMsg));
	      break;
	    case SW_EFF_CMD_TYPE:
	      doEffCmd(CLASS_CAST(SIMwareEffCmd *, cmdMsg));
	      break;
	    case SW_SEN_CMD_TYPE:
	      doSenCmd(CLASS_CAST(SIMwareSenCmd *, cmdMsg));
	      break;
	    case SW_SET_CMD_TYPE:
	      break;
	    default:
	      break;
	    }
	  delete cmdMsg;
	  cmdMsg=kb->popCmdFromQ();
	}
      if(cmdMsg)
	{
	  delete cmdMsg;
	  cmdMsg=0;
	}
      cTimer.wait();
    }
}


void USARSimInfSkin::doEffCmd(SIMwareEffCmd * effCmd)
{
  stringstream cmdStr;
  string opName;
  map<const char *, int, strCmp>::iterator itr;
  switch(effCmd->getEffCmdType())
    {
    case SW_EFF_CMD_INIT_TYPE: 
      break;
    case SW_EFF_CMD_POWERDOWN_TYPE: 
      break;
    case SW_EFF_CMD_OPCODE_TYPE:
      opName = "NOP";
      for(itr=effOpcodeNameMap.begin();itr != effOpcodeNameMap.end();itr++)
	if( CLASS_CAST(SIMwareEffOpcodeCmd *, effCmd)->opcode == itr->second)
	  opName = itr->first;
      
      cmdStr << "SET {Type Effecter}{Name " << CLASS_CAST(SIMwareEffOpcodeCmd *, effCmd)->effName 
	     << "}{Opcode " << opName << "}{Params " << CLASS_CAST(SIMwareEffOpcodeCmd *, effCmd)->param << "}";
      writeMsg(cmdStr.str());
      
      break;
    default:
      break;
    }
 
}


void USARSimInfSkin::doMisCmd(SIMwareMisCmd * misCmd)
{
  stringstream cmdStr;
  
  switch(misCmd->getMisCmdType())
    {
    case SW_MIS_CMD_INIT_TYPE: 
      break;
    case SW_MIS_CMD_POWERDOWN_TYPE: 
      break;
    case SW_MIS_CMD_MOVE_TYPE:
      cmdStr << "MISPKG {Name " << CLASS_CAST(SIMwareMisMoveCmd *, misCmd)->misPkgName << "} ";
      for(unsigned int i=0;i<CLASS_CAST(SIMwareMisMoveCmd *, misCmd)->linkCmd.size();i++)
	{
	  cmdStr << "{Link " << CLASS_CAST(SIMwareMisMoveCmd *, misCmd)->linkCmd[i].linkID
		 << "} {Value  " << CLASS_CAST(SIMwareMisMoveCmd *, misCmd)->linkCmd[i].value 
		 << "} {Order  ";
	  switch(CLASS_CAST(SIMwareMisMoveCmd *, misCmd)->linkCmd[i].type)
	    {
	    case SW_MIS_LINK_CMD_VELOCITY_TYPE:
	      cmdStr << "1}";
	      break;
	    case SW_MIS_LINK_CMD_TORQUE_TYPE:
	      cmdStr << "2}";
	      break;
	    case SW_MIS_LINK_CMD_ABS_VAL_TYPE:
	    default:
	      cmdStr << "0}";
	      break;
	    }
	  writeMsg(cmdStr.str());
	}
      break;
    default:
      break;
    }
 
}

void USARSimInfSkin::doMobCmd(SIMwareMobCmd * mobCmd)
{
  stringstream cmdStr;
  switch(mobCmd->getMobCmdType())
    {
    case SW_MOB_CMD_INIT_TYPE: 
      cmdStr << "DRIVE {Light " << "true" << "}";
      break;
    
    case SW_MOB_CMD_POWERDOWN_TYPE: 
      switch(kb->bot->getSteerType())
	{
	case SW_KB_ACKERMAN_STEER_TYPE: 
	  cmdStr << "DRIVE {Speed 0} {FrontSteer 0} {RearSteer 0}";
	  break;
	case SW_KB_SKID_STEER_TYPE:
	  cmdStr << "DRIVE {Left 0} {Right 0}";
	  break;
	case SW_KB_OMNI_STEER_TYPE:
	  cmdStr << "DRIVE {STRAIGHT 0} {SIDE 0} {ROT 0}";
	  break;
	case SW_KB_SUBMARINE_STEER_TYPE: 
	  cmdStr << "DRIVE {Propeller 0} {Rudder 0} {SternPlane 0}";
	  break;
	case SW_KB_ROTARY_WING_STEER_TYPE: 
	  cmdStr << "DRIVE {AltitudeVelocity 0} {LinearVelocity 0} {LateralVelocity 0} {RotationalVelocity 0}";
	default:
	  cmdStr.str("");
	  break;
	}
      if(!cmdStr.str().empty())
	{
	  writeMsg(cmdStr.str());
	  cmdStr.str("");
	  esleep(0.01);
	}
      cmdStr << "DRIVE {Light " << "false" << "}";
      break;
    case SW_MOB_CMD_DRV_ACKERMAN_TYPE:
      cmdStr << "DRIVE {Speed " << CLASS_CAST(SIMwareAckermanCmd *, mobCmd)->vLinear 
	     << "} {FrontSteer " << CLASS_CAST(SIMwareAckermanCmd *, mobCmd)->steerAngle
	     << "} {RearSteer " << CLASS_CAST(SIMwareAckermanCmd *, mobCmd)->crabAngle
	  << "} {Normalized false}";
      break;
    case SW_MOB_CMD_DRV_SKID_TYPE: 
      cmdStr << "DRIVE {Left " << CLASS_CAST(SIMwareSkidCmd *, mobCmd)->wLeft
	     << "} {Right " << CLASS_CAST(SIMwareSkidCmd *, mobCmd)->wRight
	  << "} {Normalized false}";
      break;
    case SW_MOB_CMD_DRV_FLIPPER_TYPE:
      cmdStr << "MULTIDRIVE";
      for(unsigned int i=0;i<CLASS_CAST(SIMwareFlipperCmd *, mobCmd)->fCmd.size();i++)
	{
	  switch (CLASS_CAST(SIMwareFlipperCmd *, mobCmd)->fCmd[i].fType)
	    {
	    case SW_KB_FR_FLIPPER_TYPE:
	      cmdStr << " {FRFlipper";
	      break;

	    case SW_KB_FL_FLIPPER_TYPE:
	      cmdStr << " {FLFlipper";
	      break;

	    case SW_KB_RR_FLIPPER_TYPE:
	      cmdStr << " {RRFlipper";
	      break;

	    case SW_KB_RL_FLIPPER_TYPE:
	      cmdStr << " {RLFlipper";
	      break;
	     
	    default:
	      continue;
	    }
	  cmdStr << " " << CLASS_CAST(SIMwareFlipperCmd *, mobCmd)->fCmd[i].flipperAngle << "}";
	}
      break;

    case SW_MOB_CMD_DRV_OMNI_TYPE: 
      cmdStr << "DRIVE {STRAIGHT " << CLASS_CAST(SIMwareOmniCmd *, mobCmd)->vLateral
	     << "} {SIDE " << CLASS_CAST(SIMwareOmniCmd *, mobCmd)->vLinear
	     << "} {ROT " << CLASS_CAST(SIMwareOmniCmd *, mobCmd)->vRot
	  << "} {Normalized false}";
      break;
    case SW_MOB_CMD_DRV_SUBMARINE_TYPE: 
      cmdStr << "DRIVE {Propeller " << CLASS_CAST(SIMwareSubmarineCmd *, mobCmd)->propeller 
	     << "} {Rudder " <<  CLASS_CAST(SIMwareSubmarineCmd *, mobCmd)->rudder
	     << "} {SternPlane "<< CLASS_CAST(SIMwareSubmarineCmd *, mobCmd)->sternPlane
	  << "} {Normalized false}";
      break;
    case SW_MOB_CMD_DRV_ROTARY_WING_TYPE: 
      //! FIXME HACK , need aerial robot status 
      CLASS_CAST(SIMwareKBAerialRobot *, kb->bot)->setLatVel( CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vLateral );
      CLASS_CAST(SIMwareKBAerialRobot *, kb->bot)->setLinVel(CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vLinear);
      CLASS_CAST(SIMwareKBAerialRobot *, kb->bot)->setRotVel(CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vRotation);
      CLASS_CAST(SIMwareKBAerialRobot *, kb->bot)->setVertVel(CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vVertical);
      
      cmdStr << "DRIVE {AltitudeVelocity " << CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vVertical
	     << "} {RotationalVelocity " <<  CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vRotation
	     << "} {LinearVelocity " << CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vLinear
	     << "} {LateralVelocity " << CLASS_CAST(SIMwareRotaryWingCmd *, mobCmd)->vLateral
	     << "} {Normalized false}";
      break;
    default:
      if(1)
	printf("** ERROR ** <USARSimInfSkin::doMobCmd> mobility command not recognized\n");
      return;
      break;
    }
  writeMsg(cmdStr.str());
}


void USARSimInfSkin::doSenCmd(SIMwareSenCmd * senCmd)
{  
  //fprintf(stderr, "\n\nHELLO WORLD\n\n");
  stringstream cmdStr;
  map <const char *, SIMwareKBSen *, strCmp >::iterator itr = kb->senMap.begin();
  map <const char *, int, strCmp >::iterator nameItr;
  switch(senCmd->getSenCmdType())
    {
    case SW_SEN_CMD_INIT_TYPE:
      while ( itr != kb->senMap.end() ) 
	{
	  switch( itr->second->getSensorType())
	    {
	    case SW_KB_ODOMETRY_SENSOR_TYPE:
	    case SW_KB_ENCODER_SENSOR_TYPE:
	    case SW_KB_INU_SENSOR_TYPE:
	      nameItr = senNameMap.begin();
	      while ( nameItr != senNameMap.end() ) 
		{
		  if( nameItr->second == itr->second->getSensorType())
		    {
		     ulapi_mutex_take(syncMutex);
  		     cmdStr << "SET {Type " << nameItr->first << "} {Name " << itr->first << "} {Opcode RESET} {Params 1}";

//fprintf(stderr,"\n\n!!!!!SENDING THE FOLLOWING TO USARSIM: \n%s\n\n", cmdStr.str().c_str());

		     writeMsg(cmdStr.str());
		     // Wait til RES  msg recieved
		     ulapi_cond_wait(syncCond, syncMutex);
		     ulapi_mutex_give(syncMutex);
		     cmdStr.str("");
		    }
		  nameItr++;
		}
	      break;
	    default:
	      break;
	    }
	  itr++;
	}
    case SW_SEN_CMD_RESET_TYPE:
      while ( itr != kb->senMap.end() ) 
	{
	  switch( itr->second->getSensorType())
	    {
	    case SW_KB_ODOMETRY_SENSOR_TYPE:
	      nameItr = senNameMap.begin();
	      while ( nameItr != senNameMap.end() ) 
		{
		  if( nameItr->second == itr->second->getSensorType())
		    {
		     ulapi_mutex_take(syncMutex);
  		     cmdStr << "SET {Type " << nameItr->first << "} {Name " << itr->first << "} {Opcode RESET} {Params 1}";

              //fprintf(stderr,"\n\nSENDING THE FOLLOWING TO USARSIM: \n%s\n\n", cmdStr.str().c_str());
//fflush(stdout);

		     writeMsg(cmdStr.str());
		     // Wait til RES  msg recieved
		     ulapi_cond_wait(syncCond, syncMutex);
		     ulapi_mutex_give(syncMutex);
		     cmdStr.str("");
		    }
		  nameItr++;
		}
	      break;
	    default:
	      break;
	    }
	  itr++;
	}
    case SW_SEN_CMD_POWERDOWN_TYPE:
      break;
    default:
      if(1)
	printf("** ERROR ** <USARSimInfSkin::doSenCmd> sensor command not recognized\n");
      break;

    }
}

bool USARSimInfSkin::startBot ( int vehID, string platformName, string robotName,  PM_POSE stPose)
{
  
  
  map<const char *, int, strCmp>::iterator itr;  
  PM_RPY rpy = stPose.rot;
  stringstream sStr;   
  
  if(platformName.empty())
    {
      printf("<USARSimInfSkin::startBot> ** ERROR ** need to specify platform name\n");
      return false;
    }
  if(robotName.empty())
    {
      sStr <<  vehID;
      robotName = platformName + sStr.str();
    }

  if(0)
    if( stPose.tran.x == 0 && stPose.tran.y == 0 ) 
      {
	printf("** WARNING ** <USARSimInfSkin.startBot> \n");
	printf("\tINIT Bug in USARSim prevents instantiation at xyz:[0,0,0].\n");
	printf("\tAdding 0.1 to x value to avoid bug.\n");
	stPose.tran.x+=0.1;
      }     


  if(1)
    {
      printf("<USARSimInfSkin::startBot>\n");
      printf("\tVehicle ID         : %d\n", vehID);
      printf("\tPlatform Name      : %s\n", platformName.c_str());
      printf("\tRobot Name         : %s\n", robotName.c_str());

      //correction to the P2AT start pos (Prevent bouncing)
      if(strcmp(platformName.c_str(),"P2AT")==0){
            printf("\tCorrecting start pos for the P2AT\n");
            stPose.tran.z -= 0.1;
      }

      printf("\tLocation           : %lf %lf %lf\n", 
	     stPose.tran.x, 
	     stPose.tran.y,  
	     stPose.tran.z);
      printf("\tOrientation        : %lf %lf %lf\n", rpy.r,rpy.p,rpy.y);
    }


  // construct init cmd
  sStr.str("");
  sStr << "INIT {ClassName USARBot." <<  platformName <<"}"
       << "{Name " << robotName << "}"
       << "{Location " << stPose.tran.x << "," <<  stPose.tran.y << "," <<  stPose.tran.z
       << "} {Rotation " << rpy.r << "," <<  rpy.p << "," <<  rpy.y << "}";
 
  // send init cmd
  this->writeMsg(sStr.str());
  
  //chill for a sec
  esleep(0.01);
  
  /* 
   * Sending Query Commands.
   *  The way this is set up is that the GETCONF will intialize object,
   *  so you need to call "GETCONF" first. 
   */
  
  //Query/Update robot
  if(0) printf("<USARSimInfSkin::startBot> query robot\n");
  sendQueryCmd( "GETCONF", "Robot");
  sendQueryCmd( "GETGEO", "Robot");
    
  // Now robot is initialized need to set robot ID
  kb->bot->setBotID(vehID);
  kb->bot->setName(robotName);
  
  //Query/Update Mission Packages - need to do before sensors and effecters
  if(0) printf("<USARSimInfSkin::startBot> query mission packages\n");
  sendQueryCmd( "GETCONF", "MisPkg");
  sendQueryCmd( "GETGEO", "MisPkg");
  
  //Query/Update Effecters
  if(0) printf("<USARSimInfSkin::startBot> query effecters\n");
  sendQueryCmd( "GETCONF", "Effecter");
  sendQueryCmd( "GETGEO", "Effecter");
  
  //Query/Update Sensors
  if(0) printf("<USARSimInfSkin::startBot> query sensorsn");
  itr=senNameMap.begin();
  while(itr != senNameMap.end())
    {
      sendQueryCmd("GETCONF", itr->first);
      sendQueryCmd("GETGEO", itr->first);
      itr++;
    }
  
  // Turn of trace function after robot is initialized
  if(traceInterval>0)
    {
      
      setTraceFunction(true,traceInterval);
    }
  
  kb->setKBState(SW_KB_STATE_INITIALIZED);
  return true;
}


bool
USARSimInfSkin::loadWorldInfo ()
{
  sendQueryCmd("GETSTARTPOSES");
  return true;

}


void USARSimInfSkin::setTraceFunction(bool turnOn, double interval)
{
#define TRACE_MAX_COLOR 6
  stringstream sStr;   
  if( ! turnOn )
    sStr << "Trace {On false}";
  else
    sStr << "Trace {On true}"
	 << "} {Interval " << interval
	 << "} {Color " << kb->bot->getBotID() % TRACE_MAX_COLOR << "}";
  
  sStr << "{Clear All}";
  // send trace function cmd
  this->writeMsg(sStr.str());
}

void USARSimInfSkin::runParser() { 
  // Message delimiters
  const static string mDelims = "\r\n";
  // Token delimiters
  const static string tDelims = "}{, \r\n";
  const static int bufSize = 512;
  // Message Buffer, recieves message
  //static char buf[bufSize];
  char buf[bufSize];
  // Input Buffer, concats messages received
  //static string inBuf;
  string inBuf;
  // container holding valid messages 
  //static vector < string > inMsgs;
  vector < string > inMsgs;
  // container holding valid tokens
  //static vector < string > toks;
  vector < string > toks;
  // inChar
  //static int inChar;
  int inChar;
  //static RCS_TIMER timer (0.01);
  RCS_TIMER timer (0.01);
  
  while (state != SW_KB_STATE_SHUTDOWN)
    {
      // Clear buffer
      memset (buf, 0, bufSize);

      // Read block from socket, 
      // read on char less than bufSize so string is NULL terminated
      inChar = ulapi_socket_read(socket_fd, buf, bufSize - 1);

      // Check for socket errors
      if (inChar == 0)
	{
	  printf ("** ERROR **<USARSimInfSkin::runParser> : Socket Closed\n");
	  state = SW_KB_STATE_SHUTDOWN;
	  //raise(SIGTERM);
	  //return;
	}
      if (inChar == -1)
	{
	  printf
	    ("** ERROR ** <USARSimInfSkin::runParser> : Error in Socket <errno : %d>\n",
	     errno);
	  state = SW_KB_STATE_SHUTDOWN;
	  //raise(SIGTERM);
	  //return;
	}
      
      // Concatenate received characters in input buffer
      inBuf += buf;
      
      //Put valid tokens in 'inMsgs', remaining characters left in input buffer
      Tokenize2 (inBuf, inMsgs, mDelims);
      
      // Parse all valid mags recieved from socket
      while (!inMsgs.empty ())
	{
	  toks.clear ();
	  
#ifdef VERBOSE
	  printf("<<**>> %s\n",inMsgs.back().c_str());
#endif
	  // Tokenize one message at a time
	  Tokenize2 (inMsgs.back (), toks, tDelims);
	  
	  // discard parsed message from vector
	  inMsgs.pop_back ();
	  
	  // No tokens might indicate internal errors.
	  if (toks.empty ())
	    {
	      printf
		("** WARNING ** <USARSimInfSkin::parseMsg> : No tokens found in message\n");
	      break;
	    }
	  
	  // Send message to appropriate parser
	  if (!nocase_cmp (toks[0], "NFO"))
	    parseMsg_NFO (toks);
 	  else if (!nocase_cmp (toks[0], "GEO"))
	    parseMsg_GEO (toks);
 	  else if (!nocase_cmp (toks[0], "CONF"))
 	    parseMsg_CONF (toks);
	  else if (!nocase_cmp (toks[0], "SEN"))
	    parseMsg_SEN (toks);
	  else if (!nocase_cmp (toks[0], "STA"))
	    parseMsg_STA (toks);
	  else if (!nocase_cmp (toks[0], "MISSTA"))
	    parseMsg_MIS (toks);
	  else if (!nocase_cmp (toks[0], "FIN"))
	    state = SW_KB_STATE_SHUTDOWN;
	  else if (!nocase_cmp (toks[0], "DIE"))
	    state = SW_KB_STATE_SHUTDOWN;
	  else if (!nocase_cmp (toks[0], "RES"))
	    parseMsg_RES(toks);
	  else if (!nocase_cmp (toks[0], "SEE"))
	    {
	    }
	  else
 	    {
	      if(0)
		{
 	      printf
 		("** WARNING ** <USARSimInfSkin::parseMsg> : Msg Type %s not handled\n",
 		 toks[0].c_str ());
		}
 	    }			//switch
	}			//while ( ! inMsgs.empty() )
      timer.wait ();
    }				// while ( state != SW_KB_STATE_SHUTDOWN
}




inline void
USARSimInfSkin::sendQueryCmd (string queryType , string typeName)
{
  string cmd;
  
  cmd = queryType;
  
  if( ! typeName.empty() )
      cmd+= " {Type " + typeName +"}";
  
  // conditional lock mutex
  ulapi_mutex_take(syncMutex);
  
  // formulate and send robot query commands
  writeMsg (cmd);
  
  //FIXME add timed wait here
  // Wait til the robot's CONF msg is parsed
  ulapi_cond_wait(syncCond, syncMutex);
  ulapi_mutex_give(syncMutex);    
}



void
USARSimInfSkin::parseMsg_NFO (const vector < string > &toks)
{
  unsigned int i=1;
  int ii=0;
  int c;
  
  if (0)
    printf ("USARSimInfSkin::parseMsg_NFO\n");
  if (0)
    dumpTokens (toks);


  while(i < toks.size() )
    {
      if( !nocase_cmp (toks[i], "Level"))
	{
	  kb->world.setWorldName(toks[++i]);
	  //	  kb->world.initWorld (toks[++i]);
	  state = SW_KB_STATE_INITIALIZED;
	}
      else if( !nocase_cmp (toks[i], "StartPoses"))
	{
	  c = atoi(toks[++i].c_str());
	  for(ii=0;ii<c;ii++)
	    {
	      kb->world.insertStartPose(toks[i+1],
					PM_POSE(
						PM_CARTESIAN(atof(toks[i+2].c_str()), 
							     atof(toks[i+3].c_str()), 
							     atof(toks[i+4].c_str())),
						PM_RPY(atof(toks[i+5].c_str()),
						       atof(toks[i+6].c_str()),
						       atof(toks[i+7].c_str()))
						));
	      i+=7;
	    }
	}
      i++;
    }

  ulapi_mutex_take(syncMutex);
  ulapi_cond_signal(syncCond);
  ulapi_mutex_give(syncMutex);
}


/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<   PARSING FUNCTIONS FOR CONF MESSAGES >>>>> */
void
USARSimInfSkin::parseMsg_CONF (const vector < string > &toks)
{
  map < const char *, int, strCmp >::iterator iter;
  int i=0;

  if (0)
    printf ("<USARSimInfSkin::parseMsg_CONF>\n");
  if (0)
    dumpTokens (toks);
  
  // Check for empty responses
  if (toks.size() <= 3 )
    {
      if(0) 
	printf
	  ("** Warning ** <USARSimInfSkin::parseMsg_CONF> Received empty message CONF message for %s\n",
	   toks[i].c_str ());
      ulapi_mutex_take(syncMutex);
      ulapi_cond_signal(syncCond);
      ulapi_mutex_give(syncMutex);
      return;
    }

  // find 'Type' in tokens
  if(-1 == (i=findToken(toks,"Type")))
    if(1)
      {
	printf("**Warning ** <USARSimInfSkin::parseMsg_CONF> No Type found in message, discard message\n");
	
	return;
      }
  
  //Increment to 'Type' value
  i++;
  
  // Mission Package
  if( !nocase_cmp (toks[i], "MisPkg"))
    pMisConf(toks);

  else if( !nocase_cmp (toks[i], "Effecter"))
    pEffConf(toks);
  
  // Robot
  else if (botTypeMap.end () != (iter = botTypeMap.find (toks[i].c_str ())))
    {
      switch(iter->second)
	{
	case SW_KB_GROUND_VEHICLE_TYPE:
	  pBotConfGroundVeh(toks);
	  break;

	case SW_KB_NAUTIC_ROBOT_TYPE:
	  pBotConfNauticBot(toks);
	  break;

	case SW_KB_AERIAL_ROBOT_TYPE:
	  pBotConfAerialBot(toks);
	  break;
	  
	case SW_KB_LEGGED_ROBOT_TYPE:
	  if(1)
	    printf ("** WARNING ** <USARSimInfSkin::parseMsg_CONF> Robot %s has not been implemented yet.\n",
		    toks[i].c_str ());
	  raise(SIGINT);
	  break;
	    
	default:
	  if(1)
	    printf ("** WARNING ** <USARSimInfSkin::parseMsg_CONF> Robot %s contains invalid robot type %d\n",
		    toks[i].c_str (), iter->second);
	  
	  break;	    
	}
    }  

  // Sensor
  else if (senNameMap.end () != (iter = senNameMap.find (toks[i].c_str ())))
    {
      switch (iter->second)
	{
	  
	case SW_KB_LINESCAN_SENSOR_TYPE:
	  pSenConfLinescan (toks);
	  break;
	  
	case SW_KB_SONAR_SENSOR_TYPE:
	  pSenConfSonar (toks);
	  break;
	  
	case SW_KB_INU_SENSOR_TYPE:
	  pSenConfINU (toks);
	  break;
	  
	case SW_KB_INS_SENSOR_TYPE:
	  pSenConfINS (toks);
	  break;
	  
	case SW_KB_ENCODER_SENSOR_TYPE:
	    printf("**WARNING** <USARSimInfSkin::parseMsg_CONF> Encoder Sensor currently not support\n");
	  //pSenConfEncoder (toks);
	    
	  break;
	  
	case SW_KB_ODOMETRY_SENSOR_TYPE:
	  pSenConfOdometry (toks);
	  break;
	  
	case SW_KB_TOUCH_SENSOR_TYPE:
	    printf("**WARNING** <USARSimInfSkin::parseMsg_CONF> Touch Sensor currently not support\n");
	  //pSenConfTouch (toks);
	  break;
	  
	case SW_KB_RFID_SENSOR_TYPE:
	  pSenConfRFID (toks);
	  break;
	  
	case SW_KB_VICTIM_SENSOR_TYPE:
	  pSenConfVictim (toks);
	  break;
	  
	case SW_KB_CAMERA_SENSOR_TYPE:
	  pSenConfCamera (toks);
	  break;

	case SW_KB_GRD_TRUTH_SENSOR_TYPE:
	  pSenConfGrdTruth (toks);
	  break;

	case SW_KB_GPS_SENSOR_TYPE:
	  pSenConfGPS (toks);
	  break;
	
	default:
	  printf
	    ("** WARNING ** <USARSimInfSkin::parseMsg_CONF> Sensor %s contains invalid sensor type %d\n",
	     toks[i].c_str (), iter->second);
	  break;
	}
    }
  
  else
    {
      if(1)
	    printf ("** WARNING ** <USARSimInfSkin::parseMsg_CONF> Type %s not in type/name maps\n",
		    toks[i].c_str ());
    }
  
  ulapi_mutex_take(syncMutex);
  ulapi_cond_signal(syncCond);
  ulapi_mutex_give(syncMutex);
}




void
USARSimInfSkin::pSenConfLinescan (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenLinescan *lsPtr;

  if (0)
    printf ("<USARSimInfSkin::pSenConfLinescan>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	lsPtr =
	  CLASS_CAST(SIMwareKBSenLinescan *, kb->getSensor (toks[++i], true,
							    SW_KB_LINESCAN_SENSOR_TYPE));
	if( lsPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfLinescan> Linescanner %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	while (++i < toks.size() )
	  {
	    if (!nocase_cmp (toks[i], "MaxRange"))
	      lsPtr->setMaxRange (atof (toks[++i].c_str ()));
	   
	    else if (!nocase_cmp (toks[i], "MinRange"))
	      lsPtr->setMinRange (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Resolution"))
	      lsPtr->setBeamSpacing (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Fov"))
	      lsPtr->setFov (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Paning"))
	      i++;
	    
	    else if (!nocase_cmp (toks[i], "Tilting"))
	      i++;
	    
	    else if (!nocase_cmp (toks[i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfLinescan> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	lsPtr=NULL;
      }
      else 
	i++;
    }
}

void
USARSimInfSkin::pSenConfSonar (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenSonar *sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenConfSonar>\n");
  if (0)
    dumpTokens (toks);

  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenSonar *, kb->getSensor (toks[++i], true,
							 SW_KB_SONAR_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfSonar> Sonar %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while ( ++i < toks.size() )
	  {
	    if (!nocase_cmp (toks[i], "MaxRange"))
	      sPtr->setMaxRange (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "MinRange"))
	      sPtr->setMinRange (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "BeamAngle"))
	      sPtr->setBeamDispersion (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Name")){
	      break;
	    }
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfSonar> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else 
	i++;
    }
}

void
USARSimInfSkin::pSenConfINS (const vector < string > &toks)
{
  unsigned int i=0;
  SIMwareKBSenINS *sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenConfINS>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenINS *, kb->getSensor (toks[++i], true,
						       SW_KB_INS_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfINU> INS %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while (++i < toks.size() )
	  { 
	    if (!nocase_cmp (toks[i], "ScanInterval"))
	      sPtr->setCycleTime (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfINS> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else
	i++;
    }
}

void
USARSimInfSkin::pSenConfINU (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenINU *sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenConfINU>\n");
  if (0)
    dumpTokens (toks);

  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenINU *, kb->getSensor (toks[++i], true,
						       SW_KB_INU_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfINU> INU %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while (++i < toks.size() )
	  { 
	    if (!nocase_cmp (toks[i], "ScanInterval"))
	      sPtr->setCycleTime (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfINU> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else
	i++;
    }
}

void
USARSimInfSkin::pSenConfEncoder (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenEncoder *sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenConfEncoder>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenEncoder *, kb->getSensor (toks[++i], true,
							   SW_KB_ENCODER_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfEncoder> Encoder %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while ( ++i < toks.size()) 
	  {  
	    if (!nocase_cmp (toks[i], "Resolution"))
	      sPtr->setResolution (atof (toks[++i].c_str ()));
	
	    else if (!nocase_cmp (toks[i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfEncoder> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else
	i++;
    }
}

void
USARSimInfSkin::pSenConfOdometry (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenOdometry *sPtr;

  if (0)
    printf ("<USARSimInfSkin::pSenConfOdometry>\n");
  if (0)
    dumpTokens (toks);
  
   while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenOdometry *, kb->getSensor (toks[++i], true,
							    SW_KB_ODOMETRY_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfOdometry> Odometer %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while (++i < toks.size())
	  {  
	    if (!nocase_cmp (toks[i], "ScanInterval"))
	      sPtr->setCycleTime (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "EncoderResolution"))
	      i++;
	    
	    else if (!nocase_cmp (toks[i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfOdometry> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else
	i++;
    }
}

void
USARSimInfSkin::pSenConfTouch (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenTouch *sPtr;

  if (0)
    printf ("<USARSimInfSkin::pSenConfTouch>\n");
  if (0)
    dumpTokens (toks);

  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenTouch *, kb->getSensor (toks[++i], true,
							 SW_KB_TOUCH_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfTouch> Touch sensor %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while (++i < toks.size()) 
	  { 
	    if (!nocase_cmp (toks[i], "Diameter"))
	      sPtr->setDiameter (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Name"))
	      break; 
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfTouch> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else
	i++;
    }
}

void
USARSimInfSkin::pSenConfRFID (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenRFID *sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenConfRFID>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenRFID *, kb->getSensor (toks[++i], true,
							SW_KB_RFID_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfRFID> RFID sensor %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while (++i < toks.size())
	  {  
	    if (!nocase_cmp (toks[i], "MaxRange"))
	      sPtr->setMaxRange (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[++i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfRFID> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else
	i++;
    }
}

void
USARSimInfSkin::pSenConfVictim (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenVictim *sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenConfVictim>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenVictim *, kb->getSensor (toks[++i], true,
							  SW_KB_VICTIM_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfVictim> Victim sensor %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while (++i < toks.size()) 
	  {  	    
	    if (!nocase_cmp (toks[i], "MaxRange"))
	      sPtr->setMaxRange (atof (toks[++i].c_str ())); 
	    else if (!nocase_cmp (toks[i], "HorizontalFOV"))
	      sPtr->setHorizontalFOV (atof (toks[++i].c_str ())); 
	    else if (!nocase_cmp (toks[i], "VerticalFOV"))
	      sPtr->setVerticalFOV (atof (toks[++i].c_str ())); 
	    else if (!nocase_cmp (toks[i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfRFID> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else 
	i++;
    }
}

void
USARSimInfSkin::pSenConfCamera (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenCamera *sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenConfCamera>\n");
  if (0)
    dumpTokens (toks);

  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenCamera *, kb->getSensor (toks[++i], true,
							  SW_KB_CAMERA_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfCamera> Camera %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
	
	while ( ++i < toks.size())
	  {  
	    
	    if (!nocase_cmp (toks[i], "CameraDefFov"))
	      {
		sPtr->setDefFov (atof (toks[++i].c_str ()));
		sPtr->setFov (atof (toks[i].c_str ()));
	      }
	    
	    else if (!nocase_cmp (toks[i], "CameraMinFov"))
	      sPtr->setMinFov (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "CameraMaxFov"))
	      sPtr->setMaxFov (atof (toks[++i].c_str ()));
	    
	    else if (!nocase_cmp (toks[i], "Name"))
	      break;
	    
	    else
	      printf
		("** WARNING ** <USARSimInfSkin::pSenConfCamera> Token %s not recognized\n",
		 toks[i].c_str ());
	  }
	sPtr=NULL;
      }
      else
	i++;
    }
}


void
USARSimInfSkin::pSenConfGrdTruth (const vector < string > &toks)
{
  unsigned int i=1;
  SIMwareKBSenGrdTruth * sPtr;

  if (0)
    printf ("<USARSimInfSkin::pSenConfGrdTruth>\n");
  if (0)
    dumpTokens (toks);

  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr =
	  CLASS_CAST(SIMwareKBSenGrdTruth *, kb->getSensor (toks[++i], true,
							    SW_KB_GRD_TRUTH_SENSOR_TYPE));
	
	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfCamera> Camera %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
      }
      else
	i++;
      
    }
}

void
USARSimInfSkin::pSenConfGPS (const vector < string > &toks)
{
  unsigned int i=1;
  SIMwareKBSenGPS * sPtr;

  if (0)
    printf ("<USARSimInfSkin::pSenConfGPS>\n");
  if (0)
    dumpTokens (toks);

  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Name")) {
	sPtr = (SIMwareKBSenGPS *) kb->getSensor (toks[++i], true, SW_KB_GPS_SENSOR_TYPE);	

	if( sPtr == NULL ) {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenConfGPS> GPS %s could not be found or added\n",
	     toks[i].c_str ());
	  continue;
	}
      }
      else
	i++;
      
    }
}

void
USARSimInfSkin::pEffConf (const vector < string > &toks)
{
  map < const char *, int, strCmp >::iterator iter;
  unsigned int i = 0; 
  SIMwareKBEff * effPtr;
  SIMwareEffOpcodeConfig * opConf; 

  if(0)
    printf("<USARSimInfSkin::pEffConf>\n");
  
  if(0)
    dumpTokens (toks);
  
  
  while ( i < toks.size() )
    {
      if (!nocase_cmp (toks[i], "Name"))
  	{
 	  effPtr = kb->getEffecter(toks[++i],true);
	  if( effPtr == NULL ) 
  	    {
  	      if(1)
 		printf
 		  ("** WARNING ** <USARSimInfSkin::pEffConf> Effector %s could not be found or added\n",
 		   toks[i].c_str ());
  	      continue;
  	    }
	  while ( i < toks.size() )
	    {
	      if (!nocase_cmp (toks[i], "Opcode"))
		{
		  if (effOpcodeNameMap.end () == (iter = effOpcodeNameMap.find (toks[++i].c_str ())))
		    {
		      if(0)
			printf("** WARNING ** <USARSimInfSkin::pEffConf> Opcode not recognized : %s\n",toks[i].c_str());
		      continue;
		    }
		  opConf = effPtr->getOpcodeConfig((SIMwareEffOpcodeType)iter->second,true);
		  if( opConf == NULL )
		    {
		      if(0) 
			printf("** WARNING ** USARSimInfSkin::pEffConf> Get/Insert opcode failed\n");
		      continue;
		    }
		  while ( ++i < toks.size() )
		    {
		      if (!nocase_cmp (toks[i], "MaxVal"))
			opConf->setMaxValue(atof(toks[++i].c_str()));
		      if (!nocase_cmp (toks[i], "MinVal"))
			opConf->setMinValue(atof(toks[++i].c_str()));
		      if (!nocase_cmp (toks[i], "Opcode") || !nocase_cmp (toks[i], "Name"))
			break;
		      
		    } // end second nested while 
		} // end if opcode

	      else if (!nocase_cmp (toks[i], "Name"))
		break;
	      else 
		i++;
	    } // end first nested while 
	} //end if name
      else 
	i++;
    }
}
  


void
USARSimInfSkin::pMisConf (const vector < string > &toks)
{
  unsigned int i = 0; 
  SIMwareKBMisPkg * misPtr;
  SIMwareKBLink * linkPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pMisConf>\n");
  if (0)
    dumpTokens (toks);
  
  while ( i < toks.size() )
    {
      if (!nocase_cmp (toks[i], "Name"))
 	{
	  misPtr = kb->getMisPkg(toks[++i],true);
	  if( misPtr == NULL ) 
	    {
	      if(1)
	      printf
		("** WARNING ** <USARSimInfSkin::pMisConf> Mission Package %s could not be found or added\n",
		 toks[i].c_str ());
	      continue;
	    }
	  while ( i < toks.size() )
	    {
	      if (!nocase_cmp (toks[i], "Link"))
		{
		  linkPtr = misPtr->getLink(atoi(toks[++i].c_str()), true);
		  if( linkPtr == NULL ) 
		    {
		      if(1)
			printf
			  ("** WARNING ** <USARSimInfSkin::pMisConf> Mission Package Link %s could not be found or added\n",
			   toks[i].c_str ());
		      continue;
		    }
		  while ( ++i < toks.size() )
		    {
		      if (!nocase_cmp (toks[i], "MaxRange"))
			linkPtr->setMaxRange (atof (toks[++i].c_str ()));
		      
		      else if (!nocase_cmp (toks[i], "MinRange"))
			linkPtr->setMinRange (atof (toks[++i].c_str ()));
		      
		      else if (!nocase_cmp (toks[i], "MaxSpeed"))
			linkPtr->setMaxSpeed (atof (toks[++i].c_str ()));
		      
		      else if (!nocase_cmp (toks[i], "MaxTorque"))
			linkPtr->setMaxTorque (atof (toks[++i].c_str ()));
		      
		      else if (!nocase_cmp (toks[i], "JointType"))
			{
			  i++;
			  if( !nocase_cmp (toks[i], "Revolute"))
			    linkPtr->setJointType(SW_KB_REVOLUTE_JOINT_TYPE);
			  else if( !nocase_cmp (toks[i], "Prismatic"))
			    linkPtr->setJointType(SW_KB_PRISMATIC_JOINT_TYPE);
			  else if( !nocase_cmp (toks[i], "Scissor"))
			    linkPtr->setJointType(SW_KB_SCISSOR_JOINT_TYPE);
			  else
			    linkPtr->setJointType(SW_KB_UNKNOWN_JOINT_TYPE);
			}
		      
		      else if (!nocase_cmp (toks[i], "Name") ||!nocase_cmp (toks[i], "Link") )
			break;
		      
		      else
			printf
			  ("** WARNING ** <USARSimInfSkin::pMisConf> Token %s not recognized\n",
			   toks[i].c_str ());
		    }
		  linkPtr=NULL;
		}
	      else if (!nocase_cmp (toks[i], "Name") )
		break;
	      else
		i++;
	    }
	  misPtr=NULL;
	}
      else
	i++;
    }
}


void
USARSimInfSkin::pBotConfAerialBot (const vector < string > &toks)
{
  unsigned int i = 2;
  SIMwareKBAerialRobot * aBot;

  if (0)
    printf ("<USARSimInfSkin::pBotConfAerialBot>\n");
  if (0)
    dumpTokens (toks);

  if( kb->bot == NULL)
    kb->bot = new SIMwareKBAerialRobot;
  
  aBot=CLASS_CAST(SIMwareKBAerialRobot *, kb->bot);
    
  while (++i < toks.size ())
    {
     if (!nocase_cmp (toks[i], "Name"))
       aBot->setPlatformName(toks[++i]);  
     else if (!nocase_cmp (toks[i], "SteeringType"))
       {
	 if(steerNameMap.end () == steerNameMap.find (toks[++i].c_str ()))
	   aBot->setSteerType (SW_KB_UNKNOWN_STEER_TYPE);
	 else
	   aBot->setSteerType ((SIMwareSteerType) steerNameMap[toks[i].c_str ()]); 
       }
      
       else if (!nocase_cmp (toks[i], "Mass"))
	 aBot->setMass (atof (toks[++i].c_str ()));

       else if (!nocase_cmp (toks[i], "MaxAltitudeVelocity"))
	 aBot->setMaxVertVel (atof (toks[++i].c_str ()));

       else if (!nocase_cmp (toks[i], "MaxLinearVelocity"))
	 aBot->setMaxLinVel (atof (toks[++i].c_str ()));

       else if (!nocase_cmp (toks[i], "MaxLateralVelocity"))
	 aBot->setMaxLatVel (atof (toks[++i].c_str ()));

       else if (!nocase_cmp (toks[i], "MaxRotationalVelocity"))
	 aBot->setMaxRotVel (atof (toks[++i].c_str ()));
       else
 	printf
 	  ("** WARNING ** <USARSimInfSkin::pBotConfAerialBot> Token not recognized : %s\n",
 	   toks[i].c_str ());
    }
}

void
USARSimInfSkin::pBotConfNauticBot (const vector < string > &toks)
{
  unsigned int i = 2;
  SIMwareKBNauticRobot * nBot;
  if (0)
    printf ("<USARSimInfSkin::pBotConfNauticBot>\n");
  if (0)
    dumpTokens (toks);

  if( kb->bot == NULL)
    kb->bot = new SIMwareKBNauticRobot;
  
  nBot=CLASS_CAST(SIMwareKBNauticRobot *, kb->bot);

  while (++i < toks.size ())
    {
     if (!nocase_cmp (toks[i], "Name"))
       nBot->setPlatformName(toks[++i]);  
     else if (!nocase_cmp (toks[i], "SteeringType"))
       {
	 if(steerNameMap.end () == steerNameMap.find (toks[++i].c_str ()))
	   nBot->setSteerType (SW_KB_UNKNOWN_STEER_TYPE);
	 else
	   nBot->setSteerType ((SIMwareSteerType) steerNameMap[toks[i].c_str ()]); 
       }
      
       else if (!nocase_cmp (toks[i], "Mass"))
	 nBot->setMass (atof (toks[++i].c_str ()));

       else
 	printf
 	  ("** WARNING ** <USARSimInfSkin::pBotConfNauticBot> Token not recognized : %s\n",
 	   toks[i].c_str ());
    }
}

void
USARSimInfSkin::pBotConfGroundVeh (const vector < string > &toks)
{
  unsigned int i = 2;
  SIMwareKBGroundVehicle * gVeh;
  if (0)
    printf ("<USARSimInfSkin::pBotConfGroundVeh>\n");
  if (0)
    dumpTokens (toks);

  if( kb->bot == NULL)
    kb->bot = new SIMwareKBGroundVehicle;
  
  gVeh=CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot);
  
  while (++i < toks.size ())
    {
      if (!nocase_cmp (toks[i], "Name"))
	gVeh->setPlatformName(toks[++i]);  
      else if (!nocase_cmp (toks[i], "SteeringType"))
	{
	  if(steerNameMap.end () == steerNameMap.find (toks[++i].c_str ()))
	    gVeh->setSteerType (SW_KB_UNKNOWN_STEER_TYPE);
	  else
	    gVeh->setSteerType ((SIMwareSteerType) steerNameMap[toks[i].c_str ()]);
	}
    
      else if (!nocase_cmp (toks[i], "Mass"))
	gVeh->setMass (atof (toks[++i].c_str ()));
      
      else if (!nocase_cmp (toks[i], "MaxSpeed"))
	gVeh->setMaxWheelRot (atof (toks[++i].c_str ()));
      
      else if (!nocase_cmp (toks[i], "MaxTorque"))
	gVeh->setMaxTorque (atof (toks[++i].c_str ()));
    
      else if (!nocase_cmp (toks[i], "MaxFrontSteer"))
	gVeh->setMaxSteerAngle (atof (toks[++i].c_str ()));
      
      else if (!nocase_cmp (toks[i], "MaxRearSteer"))
	gVeh->setMaxCrabAngle (atof (toks[++i].c_str ()));
    
      else if(!nocase_cmp (toks[i], "Flipper")) 
	{
	  SIMwareKBFlipper * rtFlipper;
	  SIMwareKBFlipper * ltFlipper;
	  i++;
	  if(!nocase_cmp (toks[i], "Front"))
	    {
	      rtFlipper=new SIMwareKBFlipper(SW_KB_FR_FLIPPER_TYPE);
	      ltFlipper=new SIMwareKBFlipper(SW_KB_FL_FLIPPER_TYPE);
	    }
	  else if(!nocase_cmp (toks[i], "Rear"))
	    {
	      rtFlipper=new SIMwareKBFlipper(SW_KB_RR_FLIPPER_TYPE);
	      ltFlipper=new SIMwareKBFlipper(SW_KB_RL_FLIPPER_TYPE);
	    }
	  else
	    continue;
	  
	  if(!nocase_cmp (toks[++i], "MinAngle"))
	    {
	      rtFlipper->setMinAng(atof(toks[++i].c_str()));
	      ltFlipper->setMinAng(atof(toks[i].c_str()));
	    } 
	  if(!nocase_cmp (toks[++i], "MaxAngle"))
	    {
	      rtFlipper->setMaxAng(atof(toks[++i].c_str()));
	      ltFlipper->setMaxAng(atof(toks[i].c_str()));
	    }
	  gVeh->addFlipper(rtFlipper);
	  gVeh->addFlipper(ltFlipper);
	    
	  
	}
      else
	printf
	  ("** WARNING ** <USARSimInfSkin::pBotConfGroundVeh> Token not recognized : %s\n",
	   toks[i].c_str ());
    }
}

/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<  PARSING FUNCTIONS FOR GEO MESSAGES  >>>>>> */
void
USARSimInfSkin::parseMsg_GEO (const vector < string > &toks)
{
  map < const char *, int, strCmp >::iterator iter;
  int i=0;


  if (0)
    printf ("<USARSimInfSkin::parseMsg_GEO>\n");
  if (0)
    dumpTokens (toks);
  
  // Check for empty responses
  if ( toks.size () <= 3)
    {
      if(0) 
	printf
	  ("** Warning ** <USARSimInfSkin::parseMsg_GEO> Received empty message GEO message for %s\n",
	   toks[i].c_str ());
      ulapi_mutex_take(syncMutex);
      ulapi_cond_signal(syncCond);
      ulapi_mutex_give(syncMutex);
      return;
    }
  
  // find 'Type' in tokens
  if(-1 == (i=findToken(toks,"Type")))
    if(1)
      {
	printf("**Warning ** <USARSimInfSkin::parseMsg_GEO> No Type found in message, discard message\n");
	return;
      }

  //Increment to 'Type' value
  i++;
  
  // Mission Package
  if( !nocase_cmp (toks[i], "MisPkg"))
    pMisGeo(toks);

  //Effecters
  else if( !nocase_cmp (toks[i], "Effecter"))
    pEffGeo(toks);

  // Sensor
  else if (senNameMap.end () != senNameMap.find (toks[i].c_str ()))
    pSenGeo (toks);
   
  else if (botTypeMap.end () != (iter = botTypeMap.find (toks[i].c_str ())))
    pBotGeo(toks);
  
  else
    {
      if(1)
	printf ("** WARNING ** <USARSimInfSkin::parseMsg_GEO> Type %s not in type/name maps\n",
		toks[i].c_str ());
    }
  
  ulapi_mutex_take(syncMutex);
  ulapi_cond_signal(syncCond);
  ulapi_mutex_give(syncMutex);
}

void
USARSimInfSkin::pMisGeo (const vector < string > &toks)
{
  unsigned int i = 2; 
  SIMwareKBMisPkg * misPtr;
  SIMwareKBLink * linkPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pMisGeo>\n");
  if (0)
    dumpTokens (toks);
   
  while ( i < toks.size() )
    {
      if (!nocase_cmp (toks[i], "Name"))
 	{
	  misPtr = kb->getMisPkg(toks[++i]);
	  if( misPtr == NULL ) 
	    {
	      if(1)
	      printf
		("** WARNING ** <USARSimInfSkin::pMisGeo> Mission Package %s was not found\n",
		 toks[i].c_str ());
	      continue;
	    }
	  while ( i < toks.size() )
	    {
	      if (!nocase_cmp (toks[i], "Link"))
		{
 		  linkPtr = misPtr->getLink(atoi(toks[++i].c_str()));
		  if( linkPtr == NULL ) 
		    {
		      if(1)
			printf
			  ("** WARNING ** <USARSimInfSkin::pMisGeo> Mission Package Link %s could not be found or added\n",
			   toks[i].c_str ());
		      continue;
		    }
 		  while ( ++i < toks.size() )
 		    {
		      if (!nocase_cmp (toks[i], "ParentLink"))
			linkPtr->setParentLink ( misPtr->getLink( atoi(toks[++i].c_str()) ) );
		      
		      else if (!nocase_cmp (toks[i], "Location"))
			{
			  linkPtr->setLocation ( atof (toks[i+1].c_str ()),
						 atof (toks[i+2].c_str ()),
						 atof (toks[i+3].c_str ())  );
			  i += 3;
			}
		      else if (!nocase_cmp (toks[i], "Orientation"))
			{
			  linkPtr->setOrientation ( atof (toks[i+1].c_str ()),
						    atof (toks[i+2].c_str ()),
						    atof (toks[i+3].c_str ())  );
			  i += 3;
			}
		      else if (!nocase_cmp (toks[i], "Name") ||!nocase_cmp (toks[i], "Link") )
 			break;
		      
 		      else
 			printf
 			  ("** WARNING ** <USARSimInfSkin::pMisGeo> Token %s not recognized\n",
 			   toks[i].c_str ());
 		    }
 		  linkPtr=NULL;
		}
	      else if (!nocase_cmp (toks[i], "Name") )
		break;
	      else
		i++;
	    }
	  misPtr=NULL;
	}
      else
	i++;
    }
}

void
USARSimInfSkin::pBotGeo (const vector < string > &toks)
{
  unsigned int i=2;
  SIMwareRobotType type;
  if (0)
    printf ("USARSimInfSkin::pBotGeo\n");
  if (0)
    dumpTokens (toks);
 
  if( kb->bot == NULL )
    {
      printf("** ERROR ** <USARSimInfSkin::pBotGeo> NULL pointer, CONF messages initializes robot\n");
      raise(SIGTERM);
    }
  
  type = kb->bot->getBotType();
    
  while (++i < toks.size ())
    {
      if (!nocase_cmp (toks[i], "Name"))
	kb->bot->setPlatformName(toks[++i]);  
      
      else if (!nocase_cmp (toks[i], "Dimensions"))
	{
	  kb->bot->setDimensions (atof (toks[i + 1].c_str ()),
				  atof (toks[i + 2].c_str ()),
				  atof (toks[i + 3].c_str ()));
	  i += 3;
	}

      else if (!nocase_cmp (toks[i], "COG"))
	{
	  kb->bot->setCentroid (atof (toks[i + 1].c_str ()),
				atof (toks[i + 2].c_str ()),
				atof (toks[i + 3].c_str ()));
	  i += 3;
	}
      
      // Ground Robot Configuration 
      else if (!nocase_cmp (toks[i], "WheelRadius") && type == SW_KB_GROUND_VEHICLE_TYPE)
      	CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->setWheelRadius (atof (toks[++i].c_str ()));

      else if (!nocase_cmp (toks[i], "WheelSeparation") && type == SW_KB_GROUND_VEHICLE_TYPE)
      	CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->setWheelSeparation (atof (toks[++i].c_str ()));
      
      else if (!nocase_cmp (toks[i], "WheelBase") && type == SW_KB_GROUND_VEHICLE_TYPE)
      	CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->setWheelBase (atof (toks[++i].c_str ()));

      else if (!nocase_cmp (toks[i], "Flipper") && type == SW_KB_GROUND_VEHICLE_TYPE)
      	{
	  SIMwareKBFlipper * rtFlipper=NULL;
	  SIMwareKBFlipper * ltFlipper=NULL;
	  if(!nocase_cmp (toks[++i], "Front"))
	    {
	      rtFlipper=CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_FR_FLIPPER_TYPE);
	      ltFlipper=CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_FL_FLIPPER_TYPE);
	      rtFlipper->setOrientation(PM_PI_2,0,0);
	      ltFlipper->setOrientation(PM_PI_2,0,0);
	    }
	  else if(!nocase_cmp (toks[i], "Rear"))
	    {
	      rtFlipper=CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_RR_FLIPPER_TYPE);
	      ltFlipper=CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_RL_FLIPPER_TYPE);
	      rtFlipper->setOrientation(-PM_PI_2,0,PM_PI);
	      ltFlipper->setOrientation(-PM_PI_2,0,PM_PI);
	    }
	  else
	    continue;
	  
	  if(!nocase_cmp (toks[++i], "Offset"))
	    {
	      rtFlipper->setLocation(atof(toks[i+1].c_str()),
				     atof(toks[i+2].c_str()),
				     atof(toks[i+3].c_str()));
	      ltFlipper->setLocation(atof(toks[i+1].c_str()),
				     -1*atof(toks[i+2].c_str()),
				     atof(toks[i+3].c_str()));
	      
	      i+=3;
	    }
	  if(!nocase_cmp (toks[++i], "Length"))
	    {
	      rtFlipper->setLength(atof(toks[++i].c_str()));
	      ltFlipper->setLength(atof(toks[i].c_str()));
	    }
	  if(!nocase_cmp (toks[++i], "Width"))
	    {
	      rtFlipper->setWidth(atof(toks[++i].c_str()));
	      ltFlipper->setWidth(atof(toks[i].c_str()));
	    }
	  rtFlipper=NULL;
	  ltFlipper=NULL;
	}
      
      else
	printf
	  ("** WARNING ** <USARSimInfSkin::pBotGeo> Token not recognized : %s\n",
	   toks[i].c_str ());
    }
}
 
void
USARSimInfSkin::pSenGeo (const vector < string > &toks)
{
  SIMwareKBSen *sensor;
  unsigned int i=0;
  SIMwareKBMount * mount=NULL;
  
  if (0)
    printf ("<USARSimInfSkin::pSenGeo>\n");
  if (0)
    dumpTokens (toks);

  while (i < toks.size()) {
    if (!nocase_cmp (toks[i], "Name")) {
      sensor = kb->getSensor (toks[++i]);
      if (sensor == NULL) {
	printf
	  ("** Warning ** <USARSimInfSkin::pSenGeo> Sensor %s not found\n",
	   toks[i].c_str ());
	continue;
      }
      while (++i < toks.size()) {
	if (!nocase_cmp (toks[i], "Location")) {
	  sensor->setSen2MountXYZ (atof (toks[i + 1].c_str ()),
				   atof (toks[i + 2].c_str ()),
				   atof (toks[i + 3].c_str ()));
	  i += 3;
	} else if (!nocase_cmp (toks[i], "Orientation")) {
	  sensor->setSen2MountRPY (atof (toks[i + 1].c_str ()),
				   atof (toks[i + 2].c_str ()),
				   atof (toks[i + 3].c_str ()));
	  i += 3;
	} else if (!nocase_cmp (toks[i], "Mount")) {
	  i++;
	  if (nocase_cmp (toks[i], "HARD")) {
	    mount = new SIMwareKBMount;
	    mount->misPkg=kb->getMisPkg(toks[i].substr(0,toks[i].find("_")));
		     
	    if (mount->misPkg == NULL) {
	      printf("**ERROR** <USARSimInfSkin::pSenGeo> Mount %s not recognized\n", toks[i].c_str());
	      delete mount;
	    } else {
	      PM_POSE pose;
	      PM_POSE cumpose = PM_POSE(PM_CARTESIAN(0,0,0),
					PM_QUATERNION(1,0,0,0));
	      PM_RPY rpy;
	      for (int t = 1; t <= mount->misPkg->getLinkCount(); t++) {
		pose.tran = mount->misPkg->getLink(t)->getLocation(); 
		pose.rot = mount->misPkg->getLink(t)->getOrientation(); 
		if (NULL != getenv("DEBUG")) {
		  rpy = pose.rot;
		  printf("link %d has pose %f %f %f, %f %f %f\n",
			 t, pose.tran.x, pose.tran.y, pose.tran.z,
			 rpy.r, rpy.p, rpy.y);
		}
		pose = inv(pose);
		cumpose = pose * cumpose;
	      }
	      pose = cumpose * sensor->getSen2Mount();
	      sensor->setSen2MountXYZ(pose.tran.x, pose.tran.y, pose.tran.z);
	      rpy = pose.rot;
	      sensor->setSen2MountRPY(rpy.r, rpy.p, rpy.y);
	      mount->link=
		mount->misPkg->getLink(atoi(toks[i].substr(toks[i].find_last_of("Link")+1).c_str()));
	      sensor->setMount(mount);
	      mount = NULL;
	    }
	  } else {
	    sensor->setMount(NULL);
	  }
	} else if (!nocase_cmp (toks[i], "Name")) {
	  break;
	} else {
	  printf
	    ("** WARNING ** <USARSimInfSkin::pSenGeo> Token %s not recognized\n",
	     toks[i].c_str ());
	}
      }
      sensor = NULL;
    } else {
      i++;
    }
  }
}



void
USARSimInfSkin::pEffGeo (const vector < string > &toks)
{
  SIMwareKBEff * effPtr;
  unsigned int i=0;
  SIMwareKBMount * mount=NULL;
  
  if (0)
    printf ("<USARSimInfSkin::pEffGeo>\n");
  if (0)
    dumpTokens (toks);

  while( i < toks.size () )
    {
      if (!nocase_cmp (toks[i], "Name"))
	{
	  effPtr = kb->getEffecter (toks[++i]);
	  if (effPtr == NULL)
	    {
	      printf
		("** Warning ** <USARSimInfSkin::pSenEff> Effecter %s not found\n",
		 toks[i].c_str ());
	      continue;
	    }
	  while( ++i < toks.size() )
	    {

 	      if (!nocase_cmp (toks[i], "Location"))
 		{
 		  effPtr->setEff2MountXYZ (atof (toks[i + 1].c_str ()),
 					   atof (toks[i + 2].c_str ()),
 					   atof (toks[i + 3].c_str ()));
 		  i += 3;
		  
 		}
 	      else if (!nocase_cmp (toks[i], "Orientation"))
 		{
 		  effPtr->setEff2MountRPY (atof (toks[i + 1].c_str ()),
 					   atof (toks[i + 2].c_str ()),
 					   atof (toks[i + 3].c_str ()));
 		  i += 3;
 		}
 	      else if (!nocase_cmp (toks[i], "Mount"))
 		{
 		  i++;
 		  if (nocase_cmp (toks[i], "HARD"))
 		     {
 		      mount = new SIMwareKBMount;
 		      mount->misPkg=kb->getMisPkg(toks[i].substr(0,toks[i].find("_")));
		     
 		      if( mount->misPkg == NULL ) 
 			{
 			  printf("**ERROR** <USARSimInfSkin::pEffGeo> Mount %s not recognized\n", toks[i].c_str());
 			  delete mount;
 			}
 		      else
 			{
 			  mount->link=
 			    mount->misPkg->getLink(atoi(toks[i].substr(toks[i].find_last_of("Link")+1).c_str()));
 			  effPtr->setMount(mount);
 			  mount = NULL;
 			}
 		    }
		  else
		    effPtr->setMount(NULL);
 		}
	      
 	      else if (!nocase_cmp (toks[i], "Name"))
 		break;
      
 	      else
 		printf
 		  ("** WARNING ** <USARSimInfSkin::pEffGeo> Token %s not recognized\n",
 		   toks[i].c_str ());
     
 	    }
 	  effPtr = NULL;
 	}
       else 
 	i++;
    }
}    




/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<  PARSING FUNCTIONS FOR SEN MESSAGES  >>>>>> */
void
USARSimInfSkin::parseMsg_SEN (const vector < string > &toks)
{
  map < const char *, int, strCmp >::iterator iter;
  int i=0;
  
  
  if (kb->getKBState () != SW_KB_STATE_INITIALIZED ) {
    if(0) printf ("<USARSimInfSkin::parseMsg_SEN> discarding message\n");
    return;
  }
  
  if (0)
    printf ("<USARSimInfSkin::parseMsg_SEN>\n");
  if (0)
    dumpTokens (toks);
  

  // find 'Type' in tokens
  if(-1 == (i=findToken(toks,"Type")))
    if(1)
      {
	printf("**Warning ** <USARSimInfSkin::parseMsg_SEN> No Type found in message, discard message\n");
	return;
      }
  
  //Increment to 'Type' value
  i++;
  
  if (senNameMap.end () == (iter = senNameMap.find (toks[i].c_str ())))
    {
      if(0)
	printf
	  ("** WARNING ** <USARSimInfSkin::parseMsg_SEN> %s not a recognized sensor type in sensor map\n",
	   toks[i].c_str ());
      return;
    }
  kb->lockSenMutex();
  switch (iter->second)
    {
    case SW_KB_LINESCAN_SENSOR_TYPE:
      pSenLinescan (toks);
      break;
    
    case SW_KB_SONAR_SENSOR_TYPE:
      pSenSonar (toks);
      break;
    
    case SW_KB_INS_SENSOR_TYPE:
      pSenINS (toks);
      break;

    case SW_KB_INU_SENSOR_TYPE:
      pSenINU (toks);
      break;
    
    case SW_KB_ENCODER_SENSOR_TYPE:
      //pSenEncoder (toks);
      break;
    
    case SW_KB_ODOMETRY_SENSOR_TYPE:
      pSenOdometry (toks);
      break;
    
    case SW_KB_TOUCH_SENSOR_TYPE:
      //pSenTouch (toks);
      break;
    
    case SW_KB_RFID_SENSOR_TYPE:
      pSenRFID (toks);
      break;
    
    case SW_KB_VICTIM_SENSOR_TYPE:
      pSenVictim (toks);
      break;
    
    case SW_KB_CAMERA_SENSOR_TYPE:
      pSenCamera (toks);
      break;

    case SW_KB_GRD_TRUTH_SENSOR_TYPE:
      pSenGrdTruth (toks);
      break;

    case SW_KB_GPS_SENSOR_TYPE: // Added by Ben
      pSenGPS (toks);            // Added by Ben
      break;                    // Added by Ben
    
    default:
      if(1)
	printf
	  ("** WARNING ** <USARSimInfSkin::parseMsg_SEN> Sensor %s is found in map but contains invalid sensor type %d\n",
	   toks[i].c_str (), iter->second);
      break;
    }
  kb->unlockSenMutex();
}

void
USARSimInfSkin::pSenLinescan (const vector < string > &toks)
{
  SIMwareKBSenLinescan *lsPtr = NULL;
  unsigned int i=0;
  double t = 0.00;
  static vector < double >returns;

  if (0)
    printf ("<USARSimInfSkin::pSenLinescan>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
	{
	  t = atof (toks[++i].c_str ());
	  i++;
	}
      else if (!nocase_cmp (toks[i], "Name")) 
	{
	  lsPtr = CLASS_CAST(SIMwareKBSenLinescan *, kb->getSensor (toks[++i].c_str ()));
	  if (lsPtr == NULL)
	    {
	      if(1) 
		printf
		  ("** WARNING ** <USARSimInfSkin::pSenLinescan> Linescanner %s not found\n",
		   toks[i].c_str ());
	      continue;
	    }

	  // Set Data
	  lsPtr->setTimeStamp (t);
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Resolution"))
		{
		  ++i;		  
		  //lsPtr->setBeamSpacing (atof (toks[++i].c_str ()));
		}
		  
	      else if (!nocase_cmp (toks[i], "FOV"))
		lsPtr->setFov (atof (toks[++i].c_str ()));
	      
	      else if (!nocase_cmp (toks[i], "Range"))
		{
		  while(++i< toks.size())
		    {
		      if (!nocase_cmp (toks[i], "Name"))
			break;
		      returns.push_back(atof (toks[i].c_str ()));
 		    }
		  lsPtr->setRangeReturn (returns);
 		  lsPtr->setBeamSpacing(lsPtr->getFov()/(lsPtr->getReturnCount()-1));
		  returns.clear();
		}
	      else if (!nocase_cmp (toks[i], "Name")) 
		break;
	      
	      else
		printf
		  ("** WARNING ** <USARSimInfSkin::pSenLinescan> Token <%s> is not recognized\n",
		   toks[i].c_str ());
	    }
	  lsPtr=NULL;
	}
      else 
	i++;
    }
}



void
USARSimInfSkin::pSenSonar (const vector < string > &toks)
{
  SIMwareKBSenSonar *sPtr;
  unsigned int i=0;
  double t = 0.00;
  
  if (0)
    printf ("<USARSimInfSkin::pSenSonar>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenSonar *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenSonar> Sonar %s not found\n",
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
      
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Range"))
		sPtr->setRangeReturn (atof (toks[++i].c_str ()));
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <USARSimInfSkin::pSenSonar> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}

void
USARSimInfSkin::pSenINS (const vector < string > &toks)
{
  unsigned int i=0;
  SIMwareKBSenINS *sPtr;
  double t = 0.00;
  if (0)
    printf ("<USARSimInfSkin::pSenINS>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenINS *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenINU> INS sensor %s not found\n",  
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
	  while (++i < toks.size()) 
	    {
	      
	      if (!nocase_cmp (toks[i], "Orientation"))
		{
		  sPtr->setOrientation (atof (toks[i + 1].c_str ()),
					atof (toks[i + 2].c_str ()),
					atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      else if (!nocase_cmp (toks[i], "Location"))
		{
		  sPtr->setLocation (atof (toks[i + 1].c_str ()),
				     atof (toks[i + 2].c_str ()),
				     atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <USARSimInfSkin::pSenINS> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}
void
USARSimInfSkin::pSenINU (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenINU *sPtr;
  double t = 0.00;

  if (0)
    printf ("<USARSimInfSkin::pSenINU>\n");
  if (0)
    dumpTokens (toks);
 
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenINU *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenINU> INU sensor %s not found\n",  
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
      
	  while (++i < toks.size()) 
	    {
	      
	      if (!nocase_cmp (toks[i], "Orientation"))
		{
		  sPtr->setOrientation (atof (toks[i + 1].c_str ()),
					atof (toks[i + 2].c_str ()),
					atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <USARSimInfSkin::pSenINU> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}


void
USARSimInfSkin::pSenEncoder (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenEncoder *sPtr;
  double t = 0.00;

  if (0)
    printf ("<USARSimInfSkin::pSenEncoder>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenEncoder *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenEncoder> Encoder sensor %s not found\n",  
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Tick"))
		sPtr->setTick (atoi (toks[++i].c_str ()));
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <USARSimInfSkin::pSenEncoder> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}


void
USARSimInfSkin::pSenOdometry (const vector < string > &toks)
{
  unsigned int i = 0;
  double t = 0.00;
  SIMwareKBSenOdometry *sPtr = NULL;

  if (0)
    printf ("<USARSimInfSkin::pSenOdometry>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenOdometry *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenOdometry> Odometry sensor %s not found\n",
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Pose"))
		{
		  sPtr->setOdometry (atof (toks[i + 1].c_str ()),
				     atof (toks[i + 2].c_str ()),
				     atof (toks[i + 3].c_str ()));
		  i += 3;
		}
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <USARSimInfSkin::pSenOdometry> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}

void
USARSimInfSkin::pSenTouch (const vector < string > &toks)
{
  unsigned int i=2;
  double t=0.00;
  SIMwareKBSenTouch * sPtr;
  
  if (0)
    printf ("<USARSimInfSkin::pSenTouch>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenTouch *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenTouch> Touch sensor %s not found\n",
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (t);
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "Touch"))
		sPtr->setTouch(((nocase_cmp (toks[++i], "True")) ? false : true));
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
 	      else
 		printf
		  ("** WARNING ** <USARSimInfSkin::pSenTouch> Token <%s> is not recognized\n",
 		   toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}
	

void
USARSimInfSkin::pSenRFID (const vector < string > &toks)
{
  unsigned int i=0;
  SIMwareKBSenRFID * sPtr;
  SIMwareRfidMsg rMsg; 
  
  if (0)
    printf ("<USARSimInfSkin::pSenRFID>\n");
  if (0)
    dumpTokens (toks);
  
  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  rMsg.setMsgTime( atof (toks[++i].c_str ()) );
 	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenRFID *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenRFID> Touch sensor %s not found\n",
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (rMsg.getMsgTime());
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "ID")) 
		{
		  rMsg.rfidID= atoi(toks[++i].c_str());
		  
		  if( !nocase_cmp (toks[++i], "Mem")) 
		    rMsg.rfidData = toks[++i];
		  
		  sPtr->addRfidMsg(rMsg);
		}
	      
	      else if (!nocase_cmp (toks[i], "Name")) 
 		break;
	      
	      else
		if(0)
		  printf
		    ("** WARNING ** <USARSimInfSkin::pSenRFID> Token <%s> is not recognized\n",
		     toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}

void
USARSimInfSkin::pSenVictim (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenVictim *sPtr = NULL;
  SIMwareVictimMsg vMsg;
  
  if (0)
    printf ("<USARSimInfSkin::pSenVictim>\n");
  if (0)
    dumpTokens (toks);


  while(i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  vMsg.setMsgTime( atof (toks[++i].c_str ()) );
	  i++;
 	}
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenVictim *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenVictim> Touch sensor %s not found\n",
       		   toks[i].c_str ());
       	      continue;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (vMsg.getMsgTime());
	  
	  while (++i < toks.size()) 
	    {
	      if (!nocase_cmp (toks[i], "PartName")) 
		{
		  i++;
		  if(!nocase_cmp (toks[i], "Head")) 
		    vMsg.partType = SW_KB_VICTIM_HEAD_PART_TYPE;
		  else if(!nocase_cmp (toks[i], "Chest"))
		    vMsg.partType = SW_KB_VICTIM_CHEST_PART_TYPE;
		  else if(!nocase_cmp (toks[i], "Pelvis"))
		    vMsg.partType = SW_KB_VICTIM_PELVIS_PART_TYPE;
		  else if(!nocase_cmp (toks[i], "Leg"))
		    vMsg.partType = SW_KB_VICTIM_LEG_PART_TYPE;  
		  else if(!nocase_cmp (toks[i], "Foot"))
		    vMsg.partType = SW_KB_VICTIM_FOOT_PART_TYPE;
		  else if(!nocase_cmp (toks[i], "Arm"))
		    vMsg.partType = SW_KB_VICTIM_ARM_PART_TYPE;
		  else if(!nocase_cmp (toks[i], "Hand"))
		    vMsg.partType = SW_KB_VICTIM_HAND_PART_TYPE;
		  else
		    vMsg.partType = SW_KB_VICTIM_UNKNOWN_PART_TYPE;
		    
		  if (!nocase_cmp (toks[++i], "Location"))
		    {
		      vMsg.partLoc.x=atof (toks[++i].c_str ());
		      vMsg.partLoc.y=atof (toks[++i].c_str ());
		      vMsg.partLoc.z=atof (toks[++i].c_str ());
		    }
		  sPtr->addVictMsg (vMsg);	  
		}
	      else if (!nocase_cmp (toks[i], "Name")) 
		break;
	      else
		if(0)
		  printf
		    ("** WARNING ** <USARSimInfSkin::pSenVictim> Token <%s> is not recognized\n",
		     toks[i].c_str ());
	    }
	  sPtr=NULL;
	}
      else 
	i++;
    }
}

void
USARSimInfSkin::pSenCamera (const vector < string > &toks)
{
  if (0)
    printf ("<USARSimInfSkin::pSenCamera>\n");
  if (0)
    dumpTokens (toks);
  //SIMwareKBSenCamera * sPtr;
}

void
USARSimInfSkin::pSenGrdTruth (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenGrdTruth *sPtr = NULL;
  double time=0.00;
  if (0)
    printf ("<USARSimInfSkin::pSenGrdTruth>\n");
  if (0)
    dumpTokens (toks);
  
  while(++i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
	time = atof (toks[++i].c_str ());
	
      else if (!nocase_cmp (toks[i], "Type"))
	i++;
 	
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = CLASS_CAST(SIMwareKBSenGrdTruth *, kb->getSensor (toks[++i].c_str ()));
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenGrdTruth> Touch sensor %s not found\n",
       		   toks[i].c_str ());
       	      return;
       	    }
	  
	  // Set Data
       	  sPtr->setTimeStamp (time);
	}
      else if (!nocase_cmp (toks[i], "Location"))
	{
	  sPtr->setLocation (atof (toks[i+1].c_str ()),
			     atof (toks[i+2].c_str ()),
			     atof (toks[i+3].c_str ()));
	  i+=3;
	}
      else if (!nocase_cmp (toks[i], "Orientation"))
	{
	  sPtr->setOrientation (atof (toks[i+1].c_str ()),
				atof (toks[i+2].c_str ()),
				atof (toks[i+3].c_str ()));
	  i+=3;
	}

      else 
	i++;
    }
}

void
USARSimInfSkin::pSenGPS (const vector < string > &toks)
{
  unsigned int i = 0;
  SIMwareKBSenGPS *sPtr = NULL;
  double time=0.00;
  if (0)
    printf ("<USARSimInfSkin::pSenGPS>\n");
  if (0)
    dumpTokens (toks);
  
  while(++i<toks.size()) 
    {
      if (!nocase_cmp (toks[i], "Time"))
	time = atof (toks[++i].c_str ());
	
      else if (!nocase_cmp (toks[i], "Type"))
	i++;
 	
      else if (!nocase_cmp (toks[i], "Name")) 
       	{
       	  sPtr = (SIMwareKBSenGPS *) kb->getSensor (toks[++i].c_str ());
       	  if (sPtr == NULL)
       	    {
       	      if(1) 
       		printf
		  ("** WARNING ** <USARSimInfSkin::pSenGPS> GPS sensor %s not found\n",
       		   toks[i].c_str ());
       	      return;
       	    }
	  // Set Data
       	  sPtr->setTimeStamp (time);
	}
      else if (!nocase_cmp (toks[i], "Latitude"))
	{
	  sPtr->setLatitudeDegrees (atoi (toks[i+1].c_str ()));
          sPtr->setLatitudeMinutes (atof (toks[i+2].c_str ()));
          sPtr->setLatitudeCardinality (toks[i+3][0]);
	  i+=3;
        }
      else if (!nocase_cmp (toks[i], "Longitude"))
	{
          sPtr->setLongitudeDegrees (atoi (toks[i+1].c_str ()));
          sPtr->setLongitudeMinutes (atof (toks[i+2].c_str ()));
	  sPtr->setLongitudeCardinality (toks[i+3][0]);
	  i+=3;
	}
      else if (!nocase_cmp (toks[i], "Fix"))
	{
	  sPtr->setFix (atoi(toks[++i].c_str()));
	}
      else if (!nocase_cmp (toks[i], "Satellites"))
	{
	  sPtr->setSatellites (atoi(toks[++i].c_str()));
	}

      else 
	i++;
    }
}


/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<  PARSING FUNCTIONS FOR STA MESSAGES  >>>>>> */
void
USARSimInfSkin::parseMsg_STA (const vector < string > &toks)
{
  
  map < const char *, int, strCmp >::iterator iter;
  int type;
  int i=0;
  double t;
  SIMwareKBFlipper * flipper=NULL;

  if (0)
    printf ("USARSimInfSkin::parseMsg_STA\n");
  if (0)
    dumpTokens (toks);
  
  if (kb->getKBState () != SW_KB_STATE_INITIALIZED ) {
    if(0) printf ("<USARSimInfSkin::parseMsg_STA> discarding message\n");
    return;
  }
  
  // find 'Type' in tokens
  if(-1 == (i=findToken(toks,"Type")))
    if(1)
      {
 	printf("**Warning ** <USARSimInfSkin::parseMsg_STA> No Type found in message, discard message\n");
 	return;
      }
  
  if (botTypeMap.end () == (iter = botTypeMap.find (toks[++i].c_str ())))
    {
      if(1)
	printf ("** WARNING ** <USARSimInfSkin::parseMsg_STA>  Robot %s is not in the bot type map, discard message\n",
		toks[i].c_str ());
      return;
    }
  
  type=iter->second;
  
  kb->lockBotMutex();
  
  while ( ++i < (int)toks.size() )
    {
      if (!nocase_cmp (toks[i], "Time"))
 	{
 	  t = atof (toks[++i].c_str ());
 	  kb->bot->setTimeStamp (t);
	}
      else if (!nocase_cmp (toks[i], "Location"))
	{
	  i += 3;
	}
      else if (!nocase_cmp (toks[i], "Orientation"))
	{ 
	  i += 3;
	}
      else if (!nocase_cmp (toks[i], "Velocity"))
	{
	  i += 3;
	}
      else if (!nocase_cmp (toks[i], "LightToggle"))
	{
	  kb->bot->setLightsOn( (!nocase_cmp (toks[++i], "True")) ? true : false );
	}
      else if (!nocase_cmp (toks[i], "LightIntensity"))
	{
	  kb->bot->setLightLevel (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "Battery"))
	{
	  kb->bot->setBatteryLevel (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "View"))
	{
	  //printf("> %s %s \n", toks[i].c_str(),toks[i+1].c_str());
	  ++i;
	}
      // Ground Vehicle specific parameters
      else if (!nocase_cmp (toks[i], "FrontSteer") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->setSteerAngle (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "RearSteer") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->setCrabAngle(atof (toks[++i].c_str ()));
	}

      else if (!nocase_cmp (toks[i], "FRFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_FR_FLIPPER_TYPE)))
	     flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      else if (!nocase_cmp (toks[i], "FLFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_FL_FLIPPER_TYPE)))
	    flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      else if (!nocase_cmp (toks[i], "RRFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_RR_FLIPPER_TYPE)))
	     flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      else if (!nocase_cmp (toks[i], "RLFlipper") && type==SW_KB_GROUND_VEHICLE_TYPE)
	{
	  if(NULL != (flipper = CLASS_CAST(SIMwareKBGroundVehicle *, kb->bot)->getFlipper(SW_KB_RL_FLIPPER_TYPE)))
	    flipper->setFlipperAngle(atof(toks[++i].c_str()));
	}

      // Nautical Robot specific parameters
      else if (!nocase_cmp (toks[i], "SternPlaneAngle") && type==SW_KB_NAUTIC_ROBOT_TYPE)
	{
	  CLASS_CAST(SIMwareKBNauticRobot *, kb->bot)->setSternPlaneAngle (atof (toks[++i].c_str ()));
	}
      else if (!nocase_cmp (toks[i], "RudderAngle") && type==SW_KB_NAUTIC_ROBOT_TYPE)
	{
	  CLASS_CAST(SIMwareKBNauticRobot *, kb->bot)->setRudderAngle(atof (toks[++i].c_str ()));
	}


      else 
	if(1)
	  printf
	    ("** WARNING ** <USARSimInfSkin::parseMsg_STA> Token <%s> is not a recognized token \n",
	     toks[i].c_str ());     
    }
  kb->unlockBotMutex();
}
 
  

/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<  PARSING FUNCTIONS FOR MIS MESSAGES  >>>>>> */
void
USARSimInfSkin::parseMsg_MIS (const vector < string > &toks)
{
  

  unsigned int i = 1; 
  SIMwareKBMisPkg * misPtr;
  SIMwareKBLink * linkPtr;
  double time;
  
  if (0)
    printf ("<USARSimInfSkin::parseMsg_MIS>\n");
  if (0)
    dumpTokens (toks);
  
  if (kb->getKBState () != SW_KB_STATE_INITIALIZED ) 
    {
      if(0) 
	printf ("<USARSimInfSkin::parseMsg_MIS> discarding message\n");
      return;
    }
  

  kb->lockMisMutex(); 
  
  while ( i < toks.size() )
    {
      if (!nocase_cmp (toks[i], "Time"))
	{
	  time=atof( toks[++i].c_str() );
	}
      if (!nocase_cmp (toks[i], "Name"))
 	{
	  misPtr = kb->getMisPkg(toks[++i]);
	  if( misPtr == NULL ) 
	    {
	      if(1)
		printf
		  ("** WARNING ** <USARSimInfSkin::parseMsg_MIS> Mission Package %s was not found\n",
		   toks[i].c_str ());
	      continue;
	    }
	  misPtr->setTimeStamp(time);
	  while ( i < toks.size() )
	    {
	      if (!nocase_cmp (toks[i], "Link"))
		{
 		  linkPtr = misPtr->getLink(atoi(toks[++i].c_str()));
		  if( linkPtr == NULL ) 
		    {
		      if(1)
			printf
			  ("** WARNING ** <USARSimInfSkin::parseMsg_MIS> Mission Package Link %s could not be found or added\n",
			   toks[i].c_str ());
		      continue;
		    }
		  while ( i < toks.size() )
		    {
		      if (!nocase_cmp (toks[i], "Link"))
			break;
		      else if (!nocase_cmp (toks[i], "Value"))
			linkPtr->setJointVal( atof( toks[++i].c_str()) );
		      else if (!nocase_cmp (toks[i], "Torque"))
			linkPtr->setTorque( atof( toks[++i].c_str()) );
		      else
			i++;
		    }
 		  linkPtr=NULL;
		}
	      else if (!nocase_cmp (toks[i], "Name") )
		break;
	      else
		i++;
	    }
	  misPtr=NULL;
	}
      else
	i++;
    }
  kb->unlockMisMutex();
}


/* <<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>> */
/* <<<<<  PARSING FUNCTIONS FOR RES MESSAGES  >>>>>> */
void
USARSimInfSkin::parseMsg_RES (const vector < string > &toks)
{
  static unsigned int i;
  
  if (0)
    printf ("<USARSimInfSkin::parseMsg_RES>\n");
  if (0)
    dumpTokens (toks);
  
  ulapi_mutex_take(syncMutex);
  for(i=0;i<toks.size();i++)
    {
      if (!nocase_cmp (toks[i], "Status"))
	{
	  if(!nocase_cmp (toks[++i], "ok"))
	    {
	      if(0)
		printf("<USARSimInfSkin::parseMsg_RES> operation successful\n");
	    }
	  else
	    printf("** WARNING ** <USARSimInfSkin::parseMsg_RES> operation failed\n");
	}
    }
  ulapi_cond_signal(syncCond);
  ulapi_mutex_give(syncMutex);  
}
