/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file moastSupSkin.hh

  \brief 
  \code CVS Status:
  $Author: proctor $
  $Revision: 1.24 $
  $Date: 2009/01/13 16:13:09 $
  \endcode

  \author Chris Scrapper
*/

#ifndef USARSIM_INF_SKIN_HH
#define USARSIM_INF_SKIN_HH

#include <string>
#include <map>

#include "swBaseSkin.hh"
#include "swKBTypes.hh"
#include "swKBUtils.hh"

using namespace std;

void startParserThr (void *tArgs);
void startCmdThr (void *tArgs);

class USARSimInfSkin : public SIMwareInfSkin {
private:
  int socket_fd;
  
  int portNumber;
  string hostname;
  
  double traceInterval;
  
  void * parThr;
  void * cmdThr;

  void * syncCond;
  void * syncMutex;
  
  map<const char *, int, strCmp> botTypeMap;
  map<const char *, int, strCmp> senNameMap;
  map<const char *, int, strCmp> steerNameMap;
  map<const char *, int, strCmp> effOpcodeNameMap;

  // Parsing utility function for new game message
  void parseMsg_NFO(const vector<string> &);
  
   // Parsing utility function for geography msgs
  void parseMsg_GEO(const vector<string> &);
  void pMisGeo(const vector<string> &);
  void pSenGeo(const vector<string> &);
  void pBotGeo(const vector<string> &);
  void pEffGeo(const vector<string> &);

  
  // Parsing utility function for sensor msgs
  void parseMsg_SEN(const vector<string> &);
  void pSenLinescan(const vector<string> &); 
  void pSenSonar(const vector<string> &); 
  void pSenINU(const vector<string> &); 
  void pSenINS(const vector<string> &); 
  void pSenEncoder(const vector<string> &); 
  void pSenOdometry(const vector<string> &); 
  void pSenTouch(const vector<string> &); // not finished
  void pSenRFID(const vector<string> &); 
  void pSenVictim(const vector<string> &); 
  void pSenCamera(const vector<string> &); 
  void pSenGrdTruth(const vector<string> &); 
  void pSenGPS (const vector < string > &); // Added by Ben
 
  // Parsing Utility for Robot Status Messages
  void parseMsg_STA(const vector<string> &); 

  
  // Parsing utility function for configuration msgs
  void parseMsg_CONF(const vector<string> &);
  void pMisConf(const vector<string> &);
  void pEffConf(const vector<string> &);

  void pBotConfGroundVeh(const vector<string> &);
  void pBotConfNauticBot(const vector<string> &);
  void pBotConfAerialBot(const vector<string> &);
  //void pBotConfLeggedBot(const vector<string> &);

  void pSenConfLinescan(const vector<string> &); 
  void pSenConfSonar(const vector<string> &); 
  void pSenConfINU(const vector<string> &); 
  void pSenConfINS(const vector<string> &); 
  void pSenConfEncoder(const vector<string> &);
  void pSenConfOdometry(const vector<string> &); 
  void pSenConfTouch(const vector<string> &); // not finished
  void pSenConfRFID(const vector<string> &); 
  void pSenConfVictim(const vector<string> &); 
  void pSenConfCamera(const vector<string> &);  
  void pSenConfGrdTruth(const vector<string> &);
  void pSenConfGPS(const vector<string> &);  


  void parseMsg_MIS(const vector<string> &); 
  void parseMsg_RES(const vector<string> &); 
  
  void sendQueryCmd( string , string="" );

  
  void doMobCmd(SIMwareMobCmd *);
  void doMisCmd(SIMwareMisCmd *);
  void doSenCmd(SIMwareSenCmd *);
  void doEffCmd(SIMwareEffCmd *);
  
  void setTraceFunction(bool,double=0);

public:
  USARSimInfSkin();
  ~USARSimInfSkin();
  bool configInfSkin(SIMwareKB *, SIMwareSupSkin *);
  bool openSkin();
  void closeSkin();
  void runParser();
  void runCmdHandler();
  int writeMsg(string);
  bool loadWorldInfo();
  bool startBot(int, string, string, PM_POSE);
  
};
#endif
