/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   nmlWrite.cc
  \brief example of writing an NML buffer

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.2 $
  $Date: 2007/10/16 12:53:48 $
  \endcode

  \author Stephen Balakisky
  \date   10/15/2007
*/

#include <stddef.h>		// NULL
#include <stdlib.h>		// getenv()
#include <stdio.h>
#include "rcs.hh"		// RCS_CMD,STAT_CHANNEL, rcs_print()
#include "skelCommon.hh"
#include "skel.hh"

static RCS_CMD_CHANNEL * skel_cmd_buf= NULL;

#define DEFAULT_NML_FILE "./skel.nml"
#define SKEL_CMD_NAME "skelCmd" // NML cmd buffer name
#define THIS_PROCESS "nmlWrite" // NML process name for the server

static int init_nml_buffers(char * config_file)
{
  if (NULL != skel_cmd_buf) delete skel_cmd_buf;
  skel_cmd_buf = new RCS_CMD_CHANNEL(skel_format, SKEL_CMD_NAME, THIS_PROCESS, config_file);
  if (! skel_cmd_buf->valid()) {
    delete skel_cmd_buf;
    skel_cmd_buf = 0;
    return 1;
  }
  return 0;
}

#define max(a,b) ((a) > (b) ? (a) : (b))

int main(int argc, char *argv[])
{
  char config_file_default[] = DEFAULT_NML_FILE;
  char * config_file_env;
  char * NML_FILE = config_file_default;
  int retval;
  SkelCmdGo goCmd;
  SkelCmdShutdown shutdownCmd;

  printf("usage: please run skelSvr before running this program.\n");
  printf("Next, run this program (nmlWrite) and nmlRead.\n");
  printf("Type a value <CR> and the value should appear at the nmlRead application.\n");
  printf("Type a negative value to exit.\n");
  // get the config file name from arg, if provided, next environment,
  // if there, else default
  if (argc > 1) {
    NML_FILE = argv[1];
    rcs_print("%s: using specified config file %s\n", argv[0], NML_FILE);
  } else {
    config_file_env = getenv("CONFIG_NML");
    if (NULL != config_file_env) {
      NML_FILE = config_file_env;
      rcs_print("%s: using environment config file %s\n", argv[0], NML_FILE);
    } else {
      rcs_print("%s: using default config file %s\n", argv[0], NML_FILE);
    }
  }

  // initial allocate all the buffers
  retval = init_nml_buffers(NML_FILE);

  if (retval) {
    rcs_print_error("%s: can't allocate NML buffers\n", argv[0]);
    return 1;
  }
  for(;;)
    {
      fscanf(stdin, "%lf", &goCmd.where);
      if( goCmd.where < 0 )
	{
	skel_cmd_buf->write(shutdownCmd);
	nml_cleanup();
	printf( "Exiting...\n" );
	return 0;
	}
      else
	skel_cmd_buf->write(goCmd);
    }

  // will never get here
  return 0;
}

/*
  Modification history:

  $Log: nmlWrite.cc,v $
  Revision 1.2  2007/10/16 12:53:48  dr_steveb
  Added the ability to have programs exit and some comments.

  Revision 1.1  2007/10/15 21:09:56  dr_steveb
  Changes made for IEEE workshop presentation

*/
