/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   skelSvr.cc
  \brief skeleton RCS server

  \code CVS Status:
  $Author: root $
  $Revision: 1.1.1.1 $
  $Date: 2005/10/12 20:50:32 $
  \endcode

  \author Fred Proctor
  \date   02/24/2005
*/

#include <stddef.h>		// NULL
#include <stdlib.h>		// getenv()
#include "rcs.hh"		// RCS_CMD,STAT_CHANNEL, rcs_print()
#include "skelCommon.hh"
#include "skel.hh"

static RCS_CMD_CHANNEL * skel_cmd_buf= NULL;
static RCS_STAT_CHANNEL * skel_stat_buf = NULL;
static RCS_CMD_CHANNEL * skel_cfg_buf= NULL;
static RCS_STAT_CHANNEL * skel_set_buf = NULL;

#define DEFAULT_NML_FILE "./skel.nml"
#define SKEL_CMD_NAME "skelCmd" // NML cmd buffer name
#define SKEL_STAT_NAME "skelStat" // NLM stat buffer name
#define SKEL_CFG_NAME "skelCfg" // NML stat buffer name
#define SKEL_SET_NAME "skelSet" // NML stat buffer name
#define THIS_PROCESS "skelSvr" // NML process name for the server

static int init_nml_buffers(char * config_file)
{
  if (NULL != skel_stat_buf) delete skel_stat_buf;
  skel_stat_buf = new RCS_STAT_CHANNEL(skel_format, SKEL_STAT_NAME, THIS_PROCESS, config_file);
  if (! skel_stat_buf->valid()) {
    delete skel_stat_buf;
    skel_stat_buf = 0;
    return 1;
  }

  if (NULL != skel_cmd_buf) delete skel_cmd_buf;
  skel_cmd_buf = new RCS_CMD_CHANNEL(skel_format, SKEL_CMD_NAME, THIS_PROCESS, config_file);
  if (! skel_cmd_buf->valid()) {
    delete skel_cmd_buf;
    skel_cmd_buf = 0;
    return 1;
  }

  if (NULL != skel_cfg_buf) delete skel_cfg_buf;
  skel_cfg_buf = new RCS_CMD_CHANNEL(skel_format, SKEL_CFG_NAME, THIS_PROCESS, config_file);
  if (! skel_cfg_buf->valid()) {
    delete skel_cfg_buf;
    skel_cfg_buf = 0;
    return 1;
  }

  if (NULL != skel_set_buf) delete skel_set_buf;
  skel_set_buf = new RCS_STAT_CHANNEL(skel_format, SKEL_SET_NAME, THIS_PROCESS, config_file);
  if (! skel_set_buf->valid()) {
    delete skel_set_buf;
    skel_set_buf = 0;
    return 1;
  }

  return 0;
}

#define max(a,b) ((a) > (b) ? (a) : (b))

int main(int argc, char *argv[])
{
  char config_file_default[] = DEFAULT_NML_FILE;
  char * config_file_env;
  char * NML_FILE = config_file_default;
  int retval;

  // get the config file name from arg, if provided, next environment,
  // if there, else default
  if (argc > 1) {
    NML_FILE = argv[1];
    rcs_print("%s: using specified config file %s\n", argv[0], NML_FILE);
  } else {
    config_file_env = getenv("CONFIG_NML");
    if (NULL != config_file_env) {
      NML_FILE = config_file_env;
      rcs_print("%s: using environment config file %s\n", argv[0], NML_FILE);
    } else {
      rcs_print("%s: using default config file %s\n", argv[0], NML_FILE);
    }
  }

  rcs_print("%s: max(sizeof(SKEL_CMD_XXX) = %d\n", 
	    argv[0], max(sizeof(SkelCmdInit), sizeof(SkelCmdGo)));
  rcs_print("%s:     sizeof(SKEL_STAT)    = %d\n", 
	    argv[0], sizeof(SkelStat));
  rcs_print("%s: max(sizeof(SKEL_CFG_XXX) = %d\n", 
	    argv[0], max(sizeof(SkelCfgLength), sizeof(SkelCfgWidth)));
  rcs_print("%s:     sizeof(SKEL_SET)     = %d\n", 
	    argv[0], sizeof(SkelSet));

  // initial allocate all the buffers
  retval = init_nml_buffers(NML_FILE);

  if (retval) {
    rcs_print_error("%s: can't allocate NML buffers\n", argv[0]);
    return 1;
  }

  // start the nml servers
  run_nml_servers();

  // will never get here
  return 0;
}

/*
  Modification history:

  $Log: skelSvr.cc,v $
  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.1  2005/10/11 18:45:03  stephen
  Initial addition of skeleton rcs controller.

*/
