/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file gtksplash.c

  \brief Generic splash screen.
  Syntax: splash <file> <sec>
  where <file> is some graphics file, e.g., .png, and <sec> is how
  many seconds you want it up, decimals OK.

  \code CVS Status:
  $Author: root $
  $Revision: 1.1.1.1 $
  $Date: 2005/10/12 20:50:32 $
  \endcode

  \author Fred Proctor
*/

#include <stdio.h>
#include <gtk/gtk.h>

static gint timeout_callback(gpointer data)
{
  gtk_main_quit();

  return 0;
}

int main(int argc, char *argv[])
{
  GtkWidget *window;
  GtkWidget *image;
  double sec;			/* how many sec to keep splash up */

  if (argc < 3 ||
      1 != sscanf(argv[2], "%lf", &sec) ||
      sec <= 0.0) {
    return 1;
  }

  gtk_init(&argc, &argv);
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width(GTK_CONTAINER(window), 1);
  gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);

  image = gtk_image_new_from_file(argv[1]);
  gtk_container_add(GTK_CONTAINER(window), image);

  gtk_widget_show_all(window);

  (void) g_timeout_add((int) (sec * 1000), timeout_callback, 0);

  gtk_main();

  return 0;
}

/*
  Modification history:

  $Log: gtksplash.c,v $
  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.3  2005/09/30 18:55:56  proctor
  Cleaned up unused variables, forgotten return values

  Revision 1.2  2005/06/29 14:34:05  proctor
  Changed a comment.

  Revision 1.1  2005/06/29 14:32:28  proctor
  Renamed the splash screen code to have gtk, tk prefixes

  Revision 1.3  2005/06/28 13:41:38  proctor
  Changed time from msec to secs

  Revision 1.2  2005/06/27 22:03:52  proctor
  Added CVS log.

*/
