/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   vectorWM.hh
  \brief  provides class to hold vertices of vector world model

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.1 $
  $Date: 2007/04/25 15:41:38 $
  \endcode

  \author Stephen Balakirsky
  \date   03/21/2007
*/
#ifndef __vectorWM__
#define __vectorWM__
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cluster.hh"


// structure declarations

// classes
class VectorWM
{
public:
  //! constructor
  VectorWM( int initialSize, int extentsIn, int exclusionIn);
  //! destructor
  ~VectorWM();
  //! add vertices that lie in the world model region to the array
  void addVerts( ClusterMember *toadd, ClusterMember &mapCenter,
		 bool useExclusion);
  //! assign clusters to data in world model
  void cluster();
  //! sort the vertices by x and then y
  void sort();

private:
  int compare( const void *vN1, const void *vN2 );
  void expandDataArray();
  int maxNumVertices;
  int numVertices;
  int extents;
  int exclusion;
  ClusterMember *wmData;
};


#endif
/*
  Modification history:

  $Log: vectorWM.hh,v $
  Revision 1.1  2007/04/25 15:41:38  dr_steveb
  First check-in of vector world model test.


*/
