/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   vehMobMap.hh
  \brief  scrolling map for vehicle planner
  This map is based on the scrolling map contained in scrollingMap.hh


  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.6 $
  $Date: 2007/02/27 14:46:26 $
  \endcode

  \author Stephen Balakirsky
  \date   09/20/2006
*/
#ifndef __vehMobMap
#define __vehMobMap
#define CELL_NAME VehMobCell
#include <rcs.hh>
#include "vehMobCell.hh"
#include "scrollingMap.hh"

class VehMobMap:public ScrollingMap
{
public:
  /*! vehMobMap constructor taking no arguments, does nothing except call
      the ScrollingMap constructor with no arguments. */
  VehMobMap():ScrollingMap(){}
  /*! Constructor taking two arguments creates a map based on an initial
      range, resolution */
  VehMobMap( int rangeIn, double resolutionIn );
  //! return the location of a cell
  PM_CARTESIAN graphToMap( int nodeIn );
  ~VehMobMap();

/*! Method setVehLoc sets the vehicle location at the center of the
  window, causing the window to scroll. */
  void setVehLoc(PM_POSE &vehLocIn)
  {
    PM_RPY vehRpy = vehLocIn.rot;
    vehLoc = vehLocIn.tran;
    vehYaw = vehRpy.y;
    //setCenter(vehLoc.x, vehLoc.y, &xDiff, &yDiff));
  }
  /*! 
    Copy the map to another map
    the new map must be at least as large as the old map.
    The routine takes care of the memory allocation
  */
  VehMobMap *copy( int newRange );
  void clearMap();
  unsigned char getMapObstacleMin( int row, int col );
  double getVehYaw(){ 
    return(vehYaw);
  }
  PM_CARTESIAN getVehLoc(){
    return(vehLoc);
  }
private:
  PM_CARTESIAN vehLoc;     //! vehicle location
  double vehYaw;
};
#endif
/*! CVS Log Information
 * $Log: vehMobMap.hh,v $
 * Revision 1.6  2007/02/27 14:46:26  dr_steveb
 * New interface between AM and VEH
 *
 * Revision 1.5  2006/10/18 00:18:21  dr_steveb
 * Added victim channel to vehicle level. It now hunts for victims. Also added CImg library from sourceforge to perform annotation. Modified make files to include this library.
 *
 * Revision 1.4  2006/10/13 16:45:30  tomrkramer
 * In the setVehLoc method, added one commented out line and modified another
 * to call setCenter correctly if the comment marks are removed.
 * if
 *
 * Revision 1.3  2006/10/10 13:30:19  dr_steveb
 * New vehicle level exploration based on camera FOV implemented.
 *
 * Revision 1.2  2006/10/03 20:24:25  dr_steveb
 * Exploration command added to vehicle level
 *
 * Revision 1.1  2006/09/30 16:05:30  dr_steveb
 * New world model files
 *
 * 
 * 
 */
