/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file moastError.hh

  \brief  NML declarations for error reporting channel

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.3 $
  $Date: 2006/10/24 13:09:48 $
  \endcode

  \author Fred Proctor
*/

#ifndef MOAST_ERROR_HH
#define MOAST_ERROR_HH

#include <rcs.hh>		// NMLmsg, PM_CARTESIAN,RPY,POSE
#include "moastNmlOffsets.hh"	// MOAST_ERROR_BASE

#define MOAST_ERROR_TYPE (MOAST_ERROR_BASE + 1)
#define MOAST_ERROR_NAME "moastError"

#define MOAST_ERROR_LEN 200

/*!
  \class MoastError
  \brief MOAST error messages
*/
class MoastError : public NMLmsg {
public:
  MoastError() : NMLmsg(MOAST_ERROR_TYPE, sizeof(MoastError)) {};
  void update(CMS *);

  int error;			// error code
  DECLARE_NML_DYNAMIC_LENGTH_ARRAY(char,message,MOAST_ERROR_LEN);
};

extern int moastError_format(NMLTYPE type, void *buf, CMS * cms);
extern const char *moastError_symbol_lookup(long type);

#endif // MOAST_ERROR_HH

/*
  Modification history:

  $Log: moastError.hh,v $
  Revision 1.3  2006/10/24 13:09:48  dr_steveb
  New vehicle level victim locator and output on moastError. MoastError is now read by the notifier program. Use notifier -bmoastError

  Revision 1.2  2006/04/25 12:46:01  proctor
  Changed 80 to MOAST_ERROR_LEN

  Revision 1.1  2006/04/24 21:02:44  proctor
  Initial checkin

*/
