// commStationShell interfaces with WSS using the wssIP found in the /etc/moast.ini file.
// It is used to send commands to each robot in the deployment challenge of Robocup 2009.

// User defines
#define COMM_STATION_PORT 39000
#define COMM_STATION_NAME "CommStation"
#define MOAST_INI_LOC "../etc/moast.ini"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stddef.h>		// sizeof(), NULL
#include <stdlib.h>		// getenv()
#include <fcntl.h>	        // open()
#include <sys/stat.h>           // S_IWUSR, etc.
#include <unistd.h>	        // close()
#include <string>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
#include "getopt.h"
#endif
#include <rcs.hh>               // INIFILE, etc.
#include <queue>
#include <iostream>
#include <fstream>

// Communications
#include "communications.h"

//deployment plan
#include "deploymentplan.h"

//robocup map
#include "robocupmap.h"

using namespace std;

int main(int argc, char * argv[]);
static bool readConfigFile();

static char wssIP[33];

int main(int argc, char * argv[])
{
	int* dummy;
	// Read in the configuration file
	if(!readConfigFile()) 
	{
		printf("Failed to read configuration file.\n");
		return 0;
	}

	printf("Config file read successfully.\n");

	// Set up Communications

	Communications comm(0, wssIP);

	cout << "input option" << endl;
	cout << "0 - send messages to robots" << endl;
	cout << "1 - start deployment (WITH moast path planning only)" << endl;
	cout << "2 - start deployment (WITH multiwaypoint generation)" << endl;
	cout << "3 - start Communications Station shell to send commands to specific robots" << endl;
	cout << "4 - start listen mode" << endl;

	int opt = 0;
	if(argc > 1) opt = atoi(argv[1]);
	else cin >> opt;

	if(opt == 0){
		printf("Inside If Statement\n\n");
		while(true){
			string msg;
			cin >> msg;
			for(int i=0;i<Communications::ROBOTCOUNT;i++){
				comm.sendMessage(i,(unsigned char*)(msg.c_str()), msg.size());
			}
		}
	}else if(opt == 1 || opt == 2){
		DeploymentPlan dp(150);
		dp.maxSearchNodes = 500;
		cout << "loading voronoi" << endl;
		dp.loadVoronoiFromBMP("voronoi_IMPRINT.BMP");


		RobocupMap m;
		m.readMapFromTIFF("CommOverhead.tif","CommOverhead.tfw");

		cout << "robocup map read" << endl;

		//find the comm station start location
		cout << "finding start location" << endl;
		SimplePoint *startLocation = new SimplePoint(0,0);
		string line;
		char startName[255];
		float startX;
		float startY;
		fstream startlocs("startlocations.txt"); // input
		while (! startlocs.eof() )
		{
			getline (startlocs,line);
			sscanf(line.c_str(),"%s %f %f",startName,&startX,&startY);
			if(strcmp(startName,"CommStation") == 0){
				startLocation->x = (int)((startY - m.xShift)/m.xScale);
				startLocation->y = (int)((startX - m.yShift+m.mapHeight*m.yScale)/m.yScale);;
				cout << "CommStation start location set to:(" << startX << "," << startY << ")" << endl;
			}
		}
		startlocs.close();

		cout << "begining  search for " << (Communications::ROBOTCOUNT - 1) << " robots" << endl;
		dp.runSearch(startLocation,0,Communications::ROBOTCOUNT - 1);
		dp.writeSolutionToBMP(dp.bestPoints,"planimages/CommPlan.bmp");
		float *destX = new float[Communications::ROBOTCOUNT];
		float *destY = new float[Communications::ROBOTCOUNT];
		cout << "search complete: reading in points" << endl;
		list<SimplePoint*>::iterator it;
		int rNum = 0;
		for ( it=dp.bestPoints->begin() ; it != dp.bestPoints->end(); it++ ){
			if(rNum>Communications::ROBOTCOUNT-1) break;
			destX[rNum] = (*it)->x;
			destY[rNum] = (*it)->y;
			rNum++;
		}
		if(opt == 1){
			//convert coords to map
			for(int r=1;r<Communications::ROBOTCOUNT;r++){
				float tx = destX[r];
				float ty = destY[r];
				destY[r] = m.xShift + tx * m.xScale;
				destX[r] = m.yShift + ty * m.yScale - m.mapHeight* m.yScale;
			}
		}

		//Tell all the robots to init themselves
		unsigned char* cmd = new unsigned char [4];
		cmd = (unsigned char*)"INI";
		for(int r=Communications::ROBOTCOUNT-1;r>0;r--){
			cout << "sending init command to robot " << r << endl;
			comm.sendMessage(r,cmd, 3);
		}
		delete cmd;

		sleep(20);
		//tell all the robots to start their sensors
		cmd = new unsigned char[4];
		cmd = (unsigned char*)"SPG";
		for(int r=Communications::ROBOTCOUNT-1;r>0;r--){
			cout << "sending spgo command to robot " << r << endl;
			comm.sendMessage(r, cmd, 3);
		}
		delete cmd;
		//sleep(5);

		//move everyone
		for(int r=Communications::ROBOTCOUNT-1;r>0;r--){
			sleep(25);
			char ccmd[200];
			if(opt == 1){
				cout << "sending move command to robot " << r << ": (" << destX[r] << "," << destY[r] << ")" << endl;
				sprintf(ccmd,"move_%f_%f",destX[r],destY[r]);
			}else{
				cout << "sending movepov command to robot " << r << ": (" << destX[r] << "," << destY[r] << ")" << endl;
				sprintf(ccmd,"movepov_%f_%f",destX[r],destY[r]);
			}
			string cmd(ccmd);
			comm.sendMessage(r,(unsigned char*)cmd.c_str(), cmd.size());
		}

		delete destX;
		delete destY;
		cout << "Press 1 then enter to quit the program" << endl;
		int q;
		cin >> q;
		comm.closeAllSockets();
	}
	else if(opt == 3)
	{
		bool userQuit = false;
		int id;
		int numArgs;
		char* curTok;
		char opcode[4];
		unsigned char* cmdMsg;
		float fTemp;

		id = 0;
		cout << "ID of Robot: ";
		cin >> id;

		while(!userQuit)
		{
			if(id==0)
			{
				break;
			}
			else
			{
				cin >> opcode;
			}

			if(strcmp(opcode, "Q") == 0 || strcmp(opcode, "q") == 0)
				break;

			for(int i=0; i<3; i++)
				opcode[i] = toupper(opcode[i]);

			// Set numArgs
			numArgs = 0;
			if(strcmp(opcode, "ARD")==0)
				numArgs = 4;
			else if(strcmp(opcode, "SKD")==0)
				numArgs = 2;

			//printf("numArgs: %d\n",numArgs);

			// Create and zero out the command message
			cmdMsg = new unsigned char[3+4*numArgs];
			bzero(cmdMsg, 3+4*numArgs);

			// Copy the opcode into the cmdMsg.
			memcpy(&cmdMsg[0], opcode, 3);
			int byteNum = 3; // Start writing the params after the opcode, which uses bytes 0-2.

			for(int entered = 0; entered<numArgs; entered++)
			{
				cin >> fTemp;
				//printf(" Read %f for parameter %d\n", fTemp, entered);
				memcpy(&cmdMsg[byteNum], &fTemp, 4);
				byteNum += 4;
			}

			printf("Sending %s command with length %d\n",opcode, byteNum);
			comm.sendMessage(id, cmdMsg, byteNum);

			delete cmdMsg; cmdMsg = NULL;
			bzero(opcode, 4);
		}
	}
	else if(opt == 4)
	{
		bool userQuit = false;
		CommMsg* commMsg;
		while(!userQuit)
		{
			// Wait for reply.
			int count, thisID;
			bool readSuccess=false;
			for(count=0; count<15; count++)
			{
				for(thisID=1; thisID<Communications::ROBOTCOUNT; thisID++)
				{
					if(comm.readIncomingMessage(thisID, &commMsg))
					{
						readSuccess = true;
						break;
					}
					else
					{
						sleep(1);
					}
				}
			}
			if(!readSuccess) // No response
			{
				printf("No response recieved from any robot in %d seconds\n", count);
				//userQuit = true;
			}
			else
			{
				printf("Response recieved from Robot %d: %s %f\n", thisID, (char*)commMsg->type, commMsg->params[0]);
				delete commMsg;
			}
		}
	}
	else if(opt == 5)
	{
		unsigned char msg[3];
		msg[0] = 'I';
		msg[1] = 'N';
		msg[2] = 'I';

		comm.sendMessage(1, msg, 3);

		sleep(6);
		msg[0] = 'S';
		msg[1] = 'P';
		msg[2] = 'G';
		comm.sendMessage(1, msg, 3);
		sleep(10);
	}

	return 0;
}

static bool readConfigFile()
{
	INIFILE if_d;
	const char *strPtr;

	if (if_d.open (MOAST_INI_LOC))
	{
		printf
			("** ERROR ** <commStationShell::readConfigFile> Inifile %s not Found \n\t",
			MOAST_INI_LOC);
		return false;
	}

	if (if_d.isSection ("USARSIM.INFSKIN"))
	{
		printf
			("** ERROR ** <commStationShell::readConfigFile> : Section USARSIM.INFSKIN not found\n");
		if_d.close();
		return false;
	}

	if (NULL == (strPtr = if_d.find ("HOST_NAME", "USARSIM.INFSKIN")))
	{
		printf("** WARNING ** <commStationShell::readConfigFile> : Parameter HOST_NAME not found\n");
	}
	else
	{
		strcpy(wssIP, strPtr);
	}

	printf("Read wssIP from config file: %s\n", wssIP);

	return true;
}
