#include "deploymentplan.h"

int DeploymentPlan::loadVoronoiFromBMP(char* filePath)
{
	fileName = filePath;
	// Init the FreeImage library
	FreeImage_Initialise();

	// Instantiate a FreeImagePlus Wrapper
	fipImage mapImage;

	// Get the file format of the provided file.
	FREE_IMAGE_FORMAT fif = FreeImage_GetFileType(filePath);

	// Failed to identify the provided file; fail out.
	if(fif == FIF_UNKNOWN)
	{
		printf("File not found or invalid file type.\n");
		return false;
	}

	// Load the image into memory.
	bool retVal = mapImage.load(filePath);

	if(!retVal)
	{
		printf("Failed to load image %s\n", filePath);
		FreeImage_DeInitialise();
		return 0;
	}

	RGBQUAD color;  

	int dummy;
	voronoiMap = new int*[mapImage.getWidth()];
	voronoiWidth = mapImage.getWidth();
	voronoiHeight = mapImage.getHeight();
	for(int i=0;i<mapImage.getWidth();i++) voronoiMap[i] = new int[mapImage.getHeight()];
	// Read the map image into the array.
	mapImage.convertTo32Bits();
	for(unsigned int i=mapImage.getHeight(); i>0; i--)
	{
		for(unsigned int j=0; j<mapImage.getWidth(); j++)
		{
			mapImage.getPixelColor(j, i-1, &color);
			if(color.rgbRed==0 && color.rgbBlue==0 && color.rgbGreen==0)
			{
				dummy = 0;
			}
			else if(color.rgbRed==1)
			{
				dummy = 1;

			}
			else
			{
				dummy = 2;
			}

			voronoiMap[j][mapImage.getHeight()-i] = dummy;
		}
	}

	scoring.map = voronoiMap;
	scoring.mapHeight = voronoiHeight;
	scoring.mapWidth = voronoiWidth;

	FreeImage_DeInitialise();

	return 0;
}
int DeploymentPlan::writeBestSolution()
{
	writeSolutionToBMP(bestPoints,NULL);
	return 0;
}
int DeploymentPlan::writeSolutionToBMP(list<SimplePoint*> *points, char* path)
{

	// Init the FreeImage library
	FreeImage_Initialise();

	// Instantiate a FreeImagePlus Wrapper
	fipImage mapImage;

	// Get the file format of the provided file.
	FREE_IMAGE_FORMAT fif = FreeImage_GetFileType(fileName);

	// Failed to identify the provided file; fail out.
	if(fif == FIF_UNKNOWN)
	{
		printf("File not found or invalid file type.\n");
		return false;
	}
	// Load the image into memory.
	bool retVal = mapImage.load(fileName);

	if(!retVal)
	{
		printf("Failed to load image %s\n", fileName);
		FreeImage_DeInitialise();
		return 0;
	}

	RGBQUAD color;  

	//Read the map image into the array.
	mapImage.convertTo32Bits();
	list<SimplePoint*>::iterator it;
	color.rgbRed = 255;
	color.rgbGreen = 0;
	color.rgbBlue = 0;
	for ( it=points->begin() ; it != points->end(); it++ ){
		mapImage.setPixelColor((*it)->x, mapImage.getHeight()-(*it)->y-1, &color);
		//draw a circle around the point ->circle render code taken from wikipedia
		int x0 = (*it)->x;
		int y0 = (*it)->y;
		//int radius = pixelRange;
		int radius = 10;
		int f = 1 - radius;
		int ddF_x = 1;
		int ddF_y = -2 * radius;
		int x = 0;
		int y = radius;

		mapImage.setPixelColor(x0, (mapImage.getHeight()+1)-y0 + radius, &color);
		mapImage.setPixelColor(x0, (mapImage.getHeight()+1)-y0 - radius, &color);
		mapImage.setPixelColor(x0 + radius, (mapImage.getHeight()+1)-y0, &color);
		mapImage.setPixelColor(x0 - radius, (mapImage.getHeight()+1)-y0, &color);

		while(x < y)
		{
			if(f >= 0) 
			{
				y--;
				ddF_y += 2;
				f += ddF_y;
			}
			x++;
			ddF_x += 2;
			f += ddF_x;    
			mapImage.setPixelColor(x0 + x, (mapImage.getHeight()+1)-y0 + y, &color);
			mapImage.setPixelColor(x0 - x, (mapImage.getHeight()+1)-y0 + y, &color);
			mapImage.setPixelColor(x0 + x, (mapImage.getHeight()+1)-y0 - y, &color);
			mapImage.setPixelColor(x0 - x, (mapImage.getHeight()+1)-y0 - y, &color);
			mapImage.setPixelColor(x0 + y, (mapImage.getHeight()+1)-y0 + x, &color);
			mapImage.setPixelColor(x0 - y, (mapImage.getHeight()+1)-y0 + x, &color);
			mapImage.setPixelColor(x0 + y, (mapImage.getHeight()+1)-y0 - x, &color);
			mapImage.setPixelColor(x0 - y, (mapImage.getHeight()+1)-y0 - x, &color);
		}
	}
	cout << endl;
	if(path == NULL){
		char mName[100];
		sprintf(mName,"map%d.bmp",solutionNumber++);
		mapImage.save(mName,0);
	}else{
		mapImage.save(path,0);
	}

	FreeImage_DeInitialise();

	return 0; 
}
void DeploymentPlan::printVoronoiToConsole()
{
	for(int y=0;y<voronoiHeight;y++){
		for(int x=0;x<voronoiWidth;x++){
			cout << voronoiMap[x][y];
		}
		cout << endl;
	}
}
bool DeploymentPlan::voronoiPointIntersect(int x,int y, bool removePointsNearRobots)
{
	bool locationNearRobot = false;
	int radiusSquared = pixelRange*pixelRange;
	if(removePointsNearRobots){
		list<SimplePoint*>::iterator it;
		for ( it=searchResults->begin() ; it != searchResults->end(); it++ ){
			int dx = (*it)->x - x;
			int dy = (*it)->y - y;
			if(dx*dx+dy*dy<radiusSquared){
				locationNearRobot = true;
				break;
			}
		}
	}
	if(x>=0 && x<voronoiWidth && y>=0 && y<voronoiHeight
		&& voronoiMap[x][y] == 1 && !locationNearRobot) return true;
	return false;
}
void DeploymentPlan::setPixel(int x,int y)
{
	//if(x>=0 && x<voronoiWidth && y>=0 && y<voronoiHeight) voronoiMap[x][y]=2;
}
void DeploymentPlan::runSearch(SimplePoint* p,int depth,int robotCount)
{
	if(depth == 0){
		bestScore=-1;
		solutionCount=0;
		searchCallCount=0;
	}
	searchCallCount++;
	if(maxSearchNodes > 0 && searchCallCount > maxSearchNodes){
		if(p != NULL) delete p;
		return;
	}
	if(searchCallCount % 1000 == 0) cout << "At Search:" << searchCallCount << endl;
	setPixel(p->x,p->y);
	searchResults->push_back(p);
	int radius = pixelRange;
	if(depth<robotCount){

		int f = 1 - radius;
		int ddF_x = 1;
		int ddF_y = -2 * radius;
		int x = 0;
		int y = radius;
		int x0 = p->x;
		int y0= p->y;
		//setPixel(x0, y0 + radius);
		if(voronoiPointIntersect(x0,y0+radius)) runSearch(new SimplePoint(x0,y0+radius),depth+1,robotCount);
		//setPixel(x0, y0 - radius);
		if(voronoiPointIntersect(x0, y0 - radius)) runSearch(new SimplePoint(x0, y0 - radius),depth+1,robotCount);
		//setPixel(x0 + radius, y0);
		if(voronoiPointIntersect(x0 + radius, y0)) runSearch(new SimplePoint(x0 + radius, y0),depth+1,robotCount);
		//setPixel(x0 - radius, y0);
		if(voronoiPointIntersect(x0 - radius, y0)) runSearch(new SimplePoint(x0 - radius, y0),depth+1,robotCount);

		while(x < y)
		{
			if(f >= 0) 
			{
				y--;
				ddF_y += 2;
				f += ddF_y;
			}
			x++;
			ddF_x += 2;
			f += ddF_x;
			//setPixel(x0 + x, y0 + y);
			//SimplePoint*p2=new SimplePoint(x0 + x, y0 + y);
			if(voronoiPointIntersect(x0 + x, y0 + y)) runSearch(new SimplePoint(x0 + x, y0 + y),depth+1,robotCount);
			// delete p2;
			//setPixel(x0 - x, y0 + y);
			if(voronoiPointIntersect(x0 - x, y0 + y)) runSearch(new SimplePoint(x0 - x, y0 + y),depth+1,robotCount);
			//setPixel(x0 + x, y0 - y);
			if(voronoiPointIntersect(x0 + x, y0 - y)) runSearch(new SimplePoint(x0 + x, y0 - y),depth+1,robotCount);
			//setPixel(x0 - x, y0 - y);
			if(voronoiPointIntersect(x0 - x, y0 - y)) runSearch(new SimplePoint(x0 - x, y0 - y),depth+1,robotCount);
			//setPixel(x0 + y, y0 + x);
			if(voronoiPointIntersect(x0 + y, y0 + x)) runSearch(new SimplePoint(x0 + y, y0 + x),depth+1,robotCount);
			//setPixel(x0 - y, y0 + x);
			if(voronoiPointIntersect(x0 - y, y0 + x)) runSearch(new SimplePoint(x0 - y, y0 + x),depth+1,robotCount);
			//setPixel(x0 + y, y0 - x);
			if(voronoiPointIntersect(x0 + y, y0 - x)) runSearch(new SimplePoint(x0 + y, y0 - x),depth+1,robotCount);
			//setPixel(x0 - y, y0 - x);
			if(voronoiPointIntersect(x0 - y, y0 - x)) runSearch(new SimplePoint(x0 - y, y0 - x),depth+1,robotCount);
		}
	}
	else{
		//if(solutionNumber<10)
		// writeSolutionToBMP(searchResults,NULL);
		//cout << "got a set of points" << endl;
		//evaluate their score
		int score = scoring.computeScore(searchResults,radius);
		solutionCount++;
		//if bigger than current best replace
		if(score>bestScore){
			cout << "got a new best score of " << score << endl;
			//copy search results into bestPoints
			while(!bestPoints->empty()){
				SimplePoint* tempPoint = bestPoints->front();
				delete tempPoint;
				bestPoints->pop_front();
			}
			list<SimplePoint*>::iterator it;

			for ( it=searchResults->begin() ; it != searchResults->end(); it++ ){
				SimplePoint* tempPoint = new SimplePoint((*it)->x,(*it)->y);
				//cout << "Point Added " << (*it)->x << " " << (*it)->y << endl;
				bestPoints->push_back(tempPoint);
			}
			bestScore = score;
		}
	}
	if(!searchResults->empty()){
		if(p != NULL) delete p;
		searchResults->pop_back();
	}
}

bool DeploymentPlan::generateWaypointsToGoal(SimplePoint *start, SimplePoint *goal)
{

	//need to find a starting points on the voronoi
	//this is because the robot will most likely not spawn ontop of the voronoi, but will be close to it
	list<int> temp; //stores locations when searching for a start location on the voronoi
	int temp_loc = start->x + (start->y)*voronoiWidth; //converts the x/y coords of the start to a single value
	int **viewedMap = new int*[voronoiWidth];       //used to remember which points on the map have been visited
	bool **junctionPoint = new bool*[voronoiWidth]; //used to remember if the voronoi branches at a point
	int sx = -1;  //start x value on the voronoi
	int sy = -1;  //start y value on the voronoi

	//initialize the map arrays
	for(int x=0;x<voronoiWidth;x++){
		viewedMap[x] = new int[voronoiHeight];
		junctionPoint[x] = new bool[voronoiHeight];
		for(int y=0;y<voronoiHeight;y++){
			viewedMap[x][y] = -1;
			junctionPoint[x][y] = 0;
		}
	}

	//search for a close start location on the voronoi (BFS)
	temp.push_back(temp_loc);
	while(!temp.empty()){
		int p = temp.front();
		int px = p % voronoiWidth;
		int py = p / voronoiWidth;
		temp.pop_front();
		if(voronoiPointIntersect(px,py,false)){
			sx = px;
			sy = py;
			break;
		}
		for(int x=-1;x<2;x++){
			for(int y=-1;y<2;y++){
				if(!(x == 0 && y == 0)){
					if(px+x >=0 && px+x < voronoiWidth && py+y >=0 && py+y < voronoiHeight && viewedMap[px+x][py+y] == -1
						&& voronoiMap[px+x][py+y] != 0){
							temp.push_back((px + x) + (py + y)*voronoiWidth);
							viewedMap[px+x][py+y] = px + (py)*voronoiWidth;
					}
				}
			}
		}
	}
	temp.clear();
	if(sx==-1 || sy==-1) return false;


	list<SimplePoint*> l;
	l.push_back((new SimplePoint(sx,sy)));

	//clear the visited map
	for(int x=0;x<voronoiWidth;x++){
		for(int y=0;y<voronoiHeight;y++){
			viewedMap[x][y] = -1;
		}
	}

	//search the voronoi for a path to the goal (BFS)
	bool goalFound = false;
	viewedMap[sx][sy] = -2;
	while(!l.empty())
	{
		if(l.size() > 10000) break;
		SimplePoint* p = l.front();
		l.pop_front();
		if(p->x == goal->x && p->y == goal->y){
			delete p;
			goalFound = true;
			break;
		}else{
			int intersectionCount = 0;
			for(int x=-1;x<2;x++){
				for(int y=-1;y<2;y++){
					if(!(x == 0 && y == 0)){
						if(voronoiPointIntersect(p->x+x,p->y+y,false) && viewedMap[p->x+x][p->y+y] == -1){
							l.push_back((new SimplePoint(p->x + x,p->y + y)));
							viewedMap[p->x+x][p->y+y] = p->x + (p->y)*voronoiWidth;
							intersectionCount++;
						}
					}
				}
			}
			if(intersectionCount > 1)junctionPoint[p->x][p->y] = true;
		}
		delete p;
	}
	//clean up any remaining elements
	while(!l.empty())
	{
		SimplePoint* p = l.front();
		l.pop_front();
		delete p;
	}

	//if a goal was found follow the viewed map backwards to create a series of waypoints for the robot to follow
	//currently a point is only registered as a waypoint if is a location where the voronoi branches
	if(goalFound){
		int loc = goal->x + (goal->y)*voronoiWidth;
		while(loc != -2){
			int x = loc % voronoiWidth;
			int y = loc / voronoiWidth;
			loc = viewedMap[x][y];
			if(junctionPoint[x][y]){
				waypoints->push_front(new SimplePoint(x,y));
			}
			if(loc == -1){
				goalFound = false;
				printf("search error\n");
				break;
			}
		}
		printf("# of points generated = %d\n", waypoints->size());
	}
	return goalFound;
}


//SCORE CODE
Score::Score()
{
}

Score::Score(int** newMap, int height, int width)
{
	Score();
	map = newMap;
	mapHeight = height;
	mapWidth = width;
}

int Score::computeScore(list<SimplePoint*> *points, int range)
{
	int score = 0; // This variable will contain the running total of the score.
	int deltaX, deltaY;
	list<SimplePoint*>::iterator curBot;
	//return 0;

	//calculate bounding box
	int xmin = mapWidth;
	int xmax = 0;
	int ymin = mapHeight;
	int ymax = 0;

	for(curBot = points->begin(); curBot != points->end(); curBot++)
	{
		if((*curBot)->x-range<xmin)xmin=(*curBot)->x-range;
		if((*curBot)->x+range>xmax)xmax=(*curBot)->x+range;
		if((*curBot)->y-range<ymin)ymin=(*curBot)->y-range;
		if((*curBot)->y+range>ymax)ymax=(*curBot)->y+range;
	}
	range = range*range;
	// For each point in the map
	if(xmin<0)xmin=0;
	if(xmax>mapWidth)xmax=mapWidth;
	if(ymin<0)ymin=0;
	if(ymax>mapHeight)ymax=mapHeight;
	//cout << ymax-ymin << endl;
	for(int x=xmin; x<xmax; x++)
	{
		for(int y=ymin; y<ymax; y++)
		{
			if(map[x][y] == 0) continue; // (x,y) is a wall.

			// Iterate through each location in List::points, checking to see if our
			// current location on the map is within range of one of the
			// robots. If so, increment the score counter and start the main loop over.
			for(curBot = points->begin(); curBot != points->end(); curBot++)
			{
				deltaX = x - (*curBot)->x;
				deltaY = y - (*curBot)->y;

				if( (deltaX*deltaX) + (deltaY*deltaY) < range)
				{
					score++;
					break;
				}
			}
		}
	}

	return score;
}
