/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   sectMobPL.cc
  \brief Simple section level that performs an exploration behavior

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.9 $
  $Date: 2007/12/17 21:36:03 $
  \endcode

  \author Stephen Balakirsky
  \date   10/14/2005
*/

#include <stddef.h>		// sizeof(), NULL
#include <string.h>		// strcpy()
#include <stdlib.h>		// getenv()
#include <stdio.h>              // fopen(), FILE
#include <signal.h>		// signal(), SIGINT
#include <unistd.h>		// sleep()
#include <math.h>               // floor(), sqrt()
#include <sys/types.h>          // open()
#include <sys/stat.h>           // open()
#include <fcntl.h>              // open()
#include "rcsSectMobPlEngine.hh"

/*
#include "rcs.hh"
#include "vehMobPL.hh"
#include "sectMobPL.hh"
#include "navDataExt.hh"
#include "amSP.hh"
#include "amSPMap.hh"
#include "config.h"
*/
#define MY_NAME_BASE "sectMobPL"

// globals
rcsEngine *sectRCSEngine;

/////////////////////////////////////////////////// forward declarations

////////////////////////////////////////////////////////////// routines

/***********************************************************************/
/*!
  Read in obstacles and populate world model

\param obsChan The channel to read from

\return Obstacle message on success 
        NULL on failure
*/
NMLmsg *readObstacles( NML *obsChan, int &type )
{
  type = obsChan->read();
  switch( type )
    {
    case 0: // no new data
      return(NULL);
    case SENSOR_DATA_LIST_TYPE:
      return(obsChan->get_address());
    case SENSOR_DATA_2D_TYPE:
      return(obsChan->get_address());
    default:
      fprintf( stderr, "obstacle channel error\n" );
      fprintf( stdout, "Command %s not yet supported\n", sensorData_symbol_lookup(type) );
      return(NULL);
    }
}

/***********************************************************************/
/*!
  Signal handler
  \param signum Signal that has been caught
*/
void sigHandler( int signum )
{
  switch( signum )
    {
    case SIGPIPE:
      fprintf( stderr, "Output pipe closed\n" );
      break;
    case SIGINT:
      fprintf( stderr, "Shutting down sectMobPL program\n" );
      sectRCSEngine->disconnect(0);
      exit(1);
      break;
    default:
      fprintf( stderr, "Unknown signal caught\n" );
      break;
    }
}


/***********************************************************************/
int main(int argc, char *argv[])
{
  char c;
  double cycleTime = .5;
  bool myVerbose = false;
  char *configNml = getenv("CONFIG_NML");
  char *myName = "sectMobPL1";
  double resolution = .5;

  // initialize some variables
  if(!configNml)
    {
      fprintf(stderr,"Please set CONFIG_NML environment variable.\n");
      return(false);
    }
  signal( SIGPIPE, sigHandler );
  signal( SIGINT, sigHandler );

  while((c=getopt(argc,argv,"c:r:v"))!=EOF) 
    {
      switch(c) 
	{
	case 'c':
	  // set cycle time
	  cycleTime = atof(optarg);
	  fprintf( stderr, "Cycle time set to %lf\n", cycleTime );
	  break;
	case 'r':
	  resolution = atof(optarg);
	  break;
	case 'v':
	  myVerbose = true;
	  break;
	default:
	  fprintf( stderr, "usage: %s -c<cycle time> -v (verbose)\n", argv[0] );
	  exit(1);
	}
    }
  sectRCSEngine = new rcsEngine(cycleTime, resolution);

  // initial allocate all the section buffers
  if( !sectRCSEngine->connect(configNml, myName, "1") )
    exit(1);

  for(;;)
    {
      sectRCSEngine->runStateMachine(myName);
    }
}

/*
  Modification history:

  $Log: sectMobPL.cc,v $
  Revision 1.9  2007/12/17 21:36:03  dr_steveb
  Initial check-in of section level manufacturing application

  Revision 1.8  2006/09/18 16:59:27  dr_steveb
  Added crash behavior

  Revision 1.7  2006/04/07 18:33:57  dr_steveb
  Check-in so compiles work for Fred. Still needs more work.

  Revision 1.6  2006/04/05 20:24:21  dr_steveb
  Start of new section algorithm for multi-vehicle control

  Revision 1.5  2005/11/22 21:33:40  dr_steveb
  Auto-generated file.

  Revision 1.4  2005/11/14 18:06:58  dr_steveb
  Simple exloration behavior added to section level.

  Revision 1.3  2005/10/21 14:44:31  dr_steveb
  Added world dimensions to settings message.

  Revision 1.2  2005/10/21 14:13:52  dr_steveb
  Changed the size of the default map and the mag of ppmsdisplay

  Revision 1.1  2005/10/21 12:43:22  dr_steveb
  Initial commit of section planner.

*/
