/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file matilda.h

  \brief Declarations for MATILDA parameters

  \code CVS Status:
  $Author: proctor $
  $Revision: 1.1 $
  $Date: 2008/03/07 17:45:41 $
  \endcode

  \author Fred Proctor
*/

#ifndef MATILDA_H
#define MATILDA_H

enum {
  /* number of bytes max in a messages to or from USARsim */
  USARSIM_MSG_LEN = 256,

  /* number of bytes max in a message to or from Smart Motor */
  SMART_MOTOR_MSG_LEN = 256
};

/* the incoming message delimiter */
#define USARSIM_MSG_DELIMITER '\n'

/* how many Smart Motors we have */
enum {SMART_MOTOR_NUM = 2, LEFT_MOTOR = 0, RIGHT_MOTOR = 1};

/* baud rate for serial ports */
#define SMART_MOTOR_BAUD 9600

/*
  Our Smart Motors use 4000 counts/rev encoders with an 8:1 reduction.
  The manual says,

  "If your motor has a 4000 count encoder (sizes 34, 42 & 56),
  multiply your desired acceleration in rev/sec2 by 15.82 to arrive at
  the number to set A to."

  15.82 is the speed conversion, square that for the accel conversion.
  The accel conversion is the inverse.
*/

/* how many counts are in one radian */
#define COUNTS_PER_RAD 5092.95817894065	/* 32000 counts/rev */

/* the vel and accel multipliers per the comment above */
#define SPEED_ADJUST (15.82)
#define ACCEL_ADJUST (1.0 / (SPEED_ADJUST * SPEED_ADJUST))

/* acceleration in rads/sec/sec */
#define SMART_MOTOR_ACC 100

#endif

/*
  Modification history:

  $Log: matilda.h,v $
  Revision 1.1  2008/03/07 17:45:41  proctor
  Added matilda code in servo, and socket, tty utils in tools

*/
