#ifndef _GMAPPER_HH
#define _GMAPPER_HH

#include <gridfastslam/gridslamprocessor.h>
#include <vector>
#include <map>
#include <deque>
#include <string>

using namespace std;
using namespace GMapping;

//const string VICTLMTYPE="VICTIM";

struct GrisettiParameters 
{
  int numberOfBeams;
  double angResRad;
  double maxRangeM;

  int numberOfParticles;

  bool queued;
  double minDist;
  unsigned int maxQueueSize;
  unsigned short tooBigHandler;
  double handlerParameter;

  //map size parameters
  double minX, maxX, minY, maxY;
  //size of cell in meters
  double delta;
	
  int skipBeams;

  //GrisettiParameters() 
  //{
  //  queued=false;
  //  numberOfParticles=16;
  //  minDist=500;
  //  maxQueueSize=5;
  //  tooBigHandler=0;
  //  handlerParameter=1.0;
  //}
};

class GMapper : public GridSlamProcessor
{
  public:
    GMapper(GrisettiParameters grisParam, OrientedPoint initPos = OrientedPoint(0.0,0.0,0.0));
    virtual ~GMapper();
    void UpdateMap(double Xpos,double Ypos,double Theta, vector<double> &laserData, double Xsen, double Ysen, double Tsen);
    bool newBestParticle();
    bool getPose(double &x, double &y, double &yaw);
    //virtual bool getPosemm(int &x, int &y, int &yaw);
    //virtual bool getPosemm(double &x, double &y, double &yaw);
    
    virtual void onOdometryUpdate();
    virtual void onScanmatchUpdate();
    virtual void onResampleUpdate();
    
protected:
    pthread_mutex_t data_mutex;
    RangeSensor *laser;
    OdometrySensor *odometry;

    ScanMatcherMap *bestmap;
    ScanMatcherMap *mapWriteBuffer;

    double resolution;
    int orig_beams_num;
    float max_range;
    bool newParticle;
    int skipBeams;   
    int bestParticleIndex;
    
    OrientedPoint bestpose;
    std::vector<OrientedPoint> bestpath;
    //std::vector<Landmark> bestvictims;
    
};

//void drawMap(const ScanMatcherMap & map);
#endif //_GMAPPER_HH
