/*
playerInfSkin.hh
Header file for playerInfSkin class used in SIMware inferior skin.
07/10/07
*/
#ifndef PLAYER_INF_SKIN_HH
#define PLAYER_INF_SKIN_HH


// TODO: Remove unnecessary headers.
#include <iostream>
#include <string>
#include <vector>
#include <fstream>

// Player-specific headers
#include <libplayerc/playerc.h>

// Boost headers.
#include <boost/bind.hpp>
#include <boost/signal.hpp>

// RCSLIB headers.
#include <timer.hh>

// Moast / SIMware headers.
#include "swBaseSkin.hh"
#include "swKBTypes.hh"
#include "swKBUtils.hh"

/********** Externs from simWare.cc **********/
extern SIMwareKB *kb;
extern SIMwareSupSkin *supSkin;
/********** End externs from simWare.cc **********/

/********** structors  and enums **********/
typedef enum { // bit fields for sensor roles
  ROLE_GROUND_TRUTH = 0x01, // ground truth sensor
  ROLE_RELATIVE_NAV = 0x02, // relative location sensor
  ROLE_ABSOLUTE_NAV = 0x04, // absolute location sensor
  ROLE_MOTOR_DRIVE  = 0x08  // where to send mobility commands
}PlayerDriverRoles;

// Callback prototypes
typedef void (*fndestroy_t) (void*);
typedef void (*fnupdate_t) (void*);

typedef struct  // borrowed from playerv.h
{
  // Device identifier.
  player_devaddr_t addr;

  // Driver name
  char *drivername;

  // Handle to the proxy for this device.
  void *proxy;

  // Callbacks
  fndestroy_t fndestroy;
  fnupdate_t fnupdate;

  // Non-zero if should be subscribed.
  int subscribe;

} device_t;

class ClientConnection{
public:
  ClientConnection( playerc_client_t *clientIn, int which );
  void laser_create();
  void sonar_create();
  void power_create();
  void position2d_create();
  playerc_client_t *client;
  device_t device;
  //  double nextTime;
  //  double period;
  string uniqueName;
};


/********** Forward Declarations **********/
ClientConnection *create_proxy(playerc_client_t *client, int);

/********** playerInfSkin class **********/
class PlayerInfSkin : public SIMwareInfSkin {
private:
  playerc_client_t *client;
  //  vector <ClientConnection> clientProxies;
  playerc_position2d_t *motorProxy;

  pthread_t parThr;
  pthread_t cmdThr;
  pthread_t playerThr;
  
  pthread_cond_t syncCond;
  pthread_mutex_t syncMutex;

  string cfg_file;
  void scanCfg (map<string, string> &botParamMap);

  bool doMobCmd(SIMwareMobCmd *);
  void doSenCmd(SIMwareSenCmd *);
  void doEffCmd(SIMwareEffCmd *);
  void doMisCmd(SIMwareMisCmd *);
  static void fwdJacobian(double w1, double w2, double R, double L_inv,
			  double *v, double *w);

public:
  PlayerInfSkin ();
  ~PlayerInfSkin ();

  bool configInfSkin(SIMwareKB *, SIMwareSupSkin *);
  //  void configInfSkin();
  bool loadWorldInfo();
  bool openSkin();
  void closeSkin();
  void runParser();
  void runCmdHandler();
  int writeMsg(string);
  bool startBot(int, string, string, PM_POSE);
  void configureKBLineScan (device_t *pLaser);
  void configureKBOdometry (device_t *p2d);
  void configPos2d (device_t *pPos);
  void readPower (device_t *powerProx );
  bool configGndRobot( int vehID, playerc_position2d_t *p2d, const char *platformName,
		       const char *position2DName );
  bool configLaser( device_t *pLaser, const char *name );
};
/********** End playerInfSkin class **********/
/********** Helper function prototypes. **********/
void
printPlayerClientDeviceList (  playerc_client_t *client );

void
printPose (device_t *pos);

void
tempFiducial();

void
fiducialLocations (device_t *fid);

void
groundTruth (device_t &sim);

void
printAllLaserConfig (device_t &pLaser);

void
initPos2d (device_t *p2d);

void
initNav2d (device_t *p2d);

void
readPower2 ();

void
initLaser (device_t *pLaser);

void
readLaserData (device_t *laser);

void
readOdometry (device_t *p2d);

void
printCameraConfig (device_t &camera);

void
startCmdThr (void *tArgs);

#endif // PLAYER_INF_SKIN_HH
