/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file swBaseSkin.hh

  \brief 
  \code CVS Status:
  $Author: cj_scrapper $
  $Revision: 1.8 $
  $Date: 2008/09/10 15:57:42 $
  \endcode

  \author Chris Scrapper
*/

#ifndef SIMWARE_BASE_SKIN_HH
#define SIMWARE_BASE_SKIN_HH

//Forward Declaration 
class SIMwareSupSkin;
class SIMwareInfSkin;


#include "swKBTypes.hh"
#include "simWareKB.hh"
#include "swKBUtils.hh"

class SIMwareBaseSkin {
protected:
  SIMwareKB * kb;
  SIMwareStateType state;
  bool skinOpen;
public:
  SIMwareBaseSkin() : state(SW_KB_STATE_UNINITIALIZED) {};
  virtual ~SIMwareBaseSkin() {};
  virtual bool openSkin()  = 0;
  virtual void closeSkin() = 0;
  SIMwareStateType getSkinState() { return state; };
  bool isSkinOpen() { return skinOpen; };
};

class SIMwareInfSkin : public SIMwareBaseSkin{
protected:
  SIMwareSupSkin * supSkin;
public:
  SIMwareInfSkin() {};
  virtual ~SIMwareInfSkin() {};
  virtual bool configInfSkin(SIMwareKB *, SIMwareSupSkin *) = 0; 
  virtual bool loadWorldInfo() = 0; 
  virtual bool startBot(int, string, string, PM_POSE) = 0;

  // virtual void sendMobCmd() {};
  // virtual void sendEffCmd() {};
  // virtual void sendSenCmd() {};
  // virtual void sendMisCmd() {};
};

class SIMwareSupSkin : public SIMwareBaseSkin{
protected:
  SIMwareInfSkin * infSkin;
public:
  SIMwareSupSkin() {};
  virtual ~SIMwareSupSkin() {};
  virtual bool configSupSkin(SIMwareKB *, SIMwareInfSkin *) = 0;
};


#endif
