#include "swKBTypes.hh"
#include "swServoMis.hh"
#include "moastSupSkin.hh"

SIMwareServoMis::SIMwareServoMis(MoastSupSkin * m, SIMwareKBMisPkg * mPkg) : SIMwareServoNodeBase(m)
{
  misPkg=mPkg;  
  nodeName="ServoMis";
  statMsg=new ServoMisJAStat;
  setMsg=new ServoMisJASet;
  statMsg->admin_state=ADMIN_UNINITIALIZED;
  CLASS_CAST(ServoMisJAStat *, statMsg)->heartbeat=0;
  CLASS_CAST(ServoMisJASet *, setMsg)->cycleTime=SERVO_NODE_CYCLE_TIME_DEFAULT;
  CLASS_CAST(ServoMisJASet *, setMsg)->debug=0;
}
SIMwareServoMis::~SIMwareServoMis() 
{
  if(0)
    printf("<SIMwareServoMis::~SIMwareServoMis>\n");
}
void SIMwareServoMis::doSingleStep() 
{
  if(0)
    printf("<SIMwareServoMis::doSingleStep> : %s\n", misPkg->getName().c_str());
  
  int serialNumber;
  int cfgType = 0;
  int cmdType = 0;
 
  // Read Configuration Message
  cfgType = cfgBuf->read();
  serialNumber = cfgBuf->get_address()->serial_number;
  switch (cfgType) {
    
  case 0:
    // no new cmd
    break;
    
  case -1:
    rcs_print_error("NML cmd read error\n");
    break;
    
  case SERVO_MIS_JA_CFG_CYCLE_TIME_TYPE:
  case SERVO_MIS_JA_CFG_DEBUG_TYPE:
    if (serialNumber != setMsg->echo_serial_number) 
      {
	setMsg->command_type = cfgType;
	setMsg->echo_serial_number = serialNumber;
	setMsg->state = NEW_COMMAND;
	setMsg->status = RCS_EXEC;
      }
    break;
    
  default:
    rcs_print_error("unknown cmd: %d\n", (int) cfgType);
    break;
  }	
  
  
  // Read Command Message 
  cmdType = cmdBuf->read();
  serialNumber = cmdBuf->get_address()->serial_number;
  switch (cmdType) {
    
  case 0:
    // no new cmd
    break;
    
  case -1:
    // comm error
    rcs_print_error("NML cmd read error\n");
    break;
    
  case SERVO_MIS_JA_CMD_INIT_TYPE:
  case SERVO_MIS_JA_CMD_HALT_TYPE:
  case SERVO_MIS_JA_CMD_ABORT_TYPE:
  case SERVO_MIS_JA_CMD_SHUTDOWN_TYPE:
  case SERVO_MIS_JA_CMD_MOVE_TYPE:
    if (serialNumber != statMsg->echo_serial_number) 
      {
	statMsg->command_type = cmdType;
	statMsg->echo_serial_number = serialNumber;
	statMsg->state = NEW_COMMAND;
	statMsg->status = RCS_EXEC;
	// error stuff
	CLASS_CAST(ServoMisJAStat *, statMsg)->error=0;
	statMsg->message_length=0;
      }
    break;
  
  default:
    rcs_print_error("unknown cmd: %d\n", (int) cmdType);
    break;
  }
  
  
  // Handle Configuration Message
  switch( setMsg->command_type) {
  case SERVO_MIS_JA_CFG_CYCLE_TIME_TYPE:
    break;

  case SERVO_MIS_JA_CFG_DEBUG_TYPE:
    break;

  default:
    // no cfg to handle
    break;
  }
  
  // Handle Command Message
  switch (statMsg->command_type) {
    
  case SERVO_MIS_JA_CMD_INIT_TYPE:
    doMisCmdInit();
    break;
    
  case SERVO_MIS_JA_CMD_HALT_TYPE:
    doMisCmdHalt();
    break;
    
  case SERVO_MIS_JA_CMD_ABORT_TYPE:
    doMisCmdAbort();
    break;
    
  case SERVO_MIS_JA_CMD_SHUTDOWN_TYPE:
    doMisCmdShutdown();
    break;

  case SERVO_MIS_JA_CMD_MOVE_TYPE:
    doMisCmdMove(CLASS_CAST(ServoMisJACmdMove *, cmdBuf->get_address()));
    break;
  
  default:
    // no cmd to handle
    break;
  }	
  
  if( cfgType > 0 )
    publishSettings();
  sendStatus();
}

void SIMwareServoMis::publishSettings()
{
  int count = 0;
  SIMwareKBLink * pLink;
  map <int, SIMwareKBLink *>::iterator itr = misPkg->linkMap.begin();
  CLASS_CAST(ServoMisJASet *, setMsg)->linkSet_length= misPkg->getLinkCount();
  while ( itr != misPkg->linkMap.end() ) 
    {
      CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].linkID=itr->second->getLinkID();
      if(NULL == (pLink = itr->second->getParentLink()) ) 
	CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].parentLinkID=0;
      else
	CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].parentLinkID=pLink->getLinkID();
      
      switch(itr->second->getJointType())
	{
	case SW_KB_PRISMATIC_JOINT_TYPE:
	  CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].jType=SERVO_MIS_JA_PRISMATIC_JOINT_TYPE;
	  break;
	  
	case SW_KB_REVOLUTE_JOINT_TYPE:
	  CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].jType=SERVO_MIS_JA_REVOLUTE_JOINT_TYPE;
	  break;

	case SW_KB_SCISSOR_JOINT_TYPE:
	  CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].jType=SERVO_MIS_JA_SCISSOR_JOINT_TYPE;
	  break;
	  
	default:
	  CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].jType=SERVO_MIS_JA_INVALID_JOINT_TYPE;
	  break;
	}
      
      CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].maxVelocity=itr->second->getMaxSpeed();
      CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].maxTorque=itr->second->getMaxTorque();
      CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].maxRange=itr->second->getMaxRange();
      CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].minRange=itr->second->getMinRange();
      CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].loc=itr->second->getLocation();
      CLASS_CAST(ServoMisJASet *, setMsg)->linkSet[count].rot=itr->second->getOrientation();
      itr++;
      count++;
    }
  setBuf->write(CLASS_CAST(ServoMisJASet *, setMsg));
}
 
void SIMwareServoMis::sendStatus()
{ 
  map <int, SIMwareKBLink *>::iterator itr = misPkg->linkMap.begin();
  CLASS_CAST(ServoMisJAStat *, statMsg)->heartbeat++;

  mSkin->kb->lockMisMutex();

  CLASS_CAST(ServoMisJAStat *, statMsg)->linkStat_length=misPkg->linkMap.size();
  CLASS_CAST(ServoMisJAStat *, statMsg)->time=misPkg->getTimeStamp();
  while ( itr != misPkg->linkMap.end() ) 
    {
      CLASS_CAST(ServoMisJAStat *, statMsg)->linkStat[itr->first-1].linkID=itr->second->getLinkID();
      CLASS_CAST(ServoMisJAStat *, statMsg)->linkStat[itr->first-1].jointVal=itr->second->getJointVal();
      CLASS_CAST(ServoMisJAStat *, statMsg)->linkStat[itr->first-1].torque=itr->second->getTorque();
      itr++;
    }

  mSkin->kb->unlockMisMutex();

  statBuf->write(CLASS_CAST(ServoMisJAStat *, statMsg));
}

void SIMwareServoMis::doMisCmdInit()
{
  if (state_match(statMsg, NEW_COMMAND)) 
    { 
      statMsg->admin_state=ADMIN_INITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMisCmd(SW_MIS_CMD_INIT_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);      
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMis::doMisCmdHalt() 
{
   if (state_match(statMsg, NEW_COMMAND)) 
     {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMisCmd(SW_MIS_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);
  
}

void SIMwareServoMis::doMisCmdAbort() 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMisCmd(SW_MIS_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMis::doMisCmdShutdown() 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMisCmd(SW_MIS_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);  
}
void SIMwareServoMis::doMisCmdMove(ServoMisJACmdMove * mCmd) 
{
  SIMwareMisMoveCmd * moveCmd = new SIMwareMisMoveCmd;
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      else 
	state_next(statMsg, S0);
    }
  
  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      moveCmd->linkCmd.resize(mCmd->linkCmd_length);
      moveCmd->misPkgName= misPkg->getName();
      for(int i=0;i<mCmd->linkCmd_length;i++)
	{
	  moveCmd->linkCmd[i].linkID=mCmd->linkCmd[i].linkID;
	  moveCmd->linkCmd[i].value=mCmd->linkCmd[i].value;
	  switch(mCmd->linkCmd[i].type)
	    { 
	    case SERVO_MIS_JA_LINK_CMD_ABS_VALUE_TYPE:
	      moveCmd->linkCmd[i].type=SW_MIS_LINK_CMD_ABS_VAL_TYPE;
	      break;
	    case SERVO_MIS_JA_LINK_CMD_VELOCITY_TYPE:
	      moveCmd->linkCmd[i].type=SW_MIS_LINK_CMD_VELOCITY_TYPE;
	      break;
	    case SERVO_MIS_JA_LINK_CMD_TORQUE_TYPE:
	      moveCmd->linkCmd[i].type=SW_MIS_LINK_CMD_TORQUE_TYPE;
	      break;
	    }
	}
       mSkin->kb->addCmdToQ(moveCmd);
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else
    state_default(statMsg);  
}

void SIMwareServoMis::errorHandler(int error) 
{
  switch(error)
    {
    case SE0:
      CLASS_CAST(ServoMisJAStat *, statMsg)->error=SE0;
      snprintf(statMsg->message,80,"Robot is not initialized%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;
    default:
      break;
    }

}

void SIMwareServoMis::doMisCfgCycleTime( ServoMisJACfgCycleTime * cTCmd ) 
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      state_new(setMsg);
      CLASS_CAST(ServoMisJASet *, setMsg)->cycleTime = cTCmd->cycleTime;
      setTimer(CLASS_CAST(ServoMisJASet *, setMsg)->cycleTime);
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);   
    }
  else
    state_default(setMsg);  
  
}

void SIMwareServoMis::doMisCfgDebug( ServoMisJACfgDebug * cTCmd ) 
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      state_new(setMsg);
      CLASS_CAST(ServoMisJASet *, setMsg)->debug = cTCmd->debug;
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);
    }
  else
    state_default(setMsg); 
}

