#include "swTraffic.hh"
#include "trafficSupSkin.hh"
#include "primMobEngine.hh"

SIMwareTraffic::SIMwareTraffic(TrafficSupSkin * m, double vmax, double amax, double wmax, double alphamax, double vCutoffAngle, double wCutoffAngle, PM_POSE controlPoint) 
{
  int botType;

  cmdBuf=NULL;
  statBuf=NULL;
  setBuf=NULL;
  cfgBuf=NULL;
  dataBuf=NULL;
  statMsg = NULL;
  setMsg=NULL;
  trafficData = NULL;
  mSkin=m;
  timer = new RCS_TIMER(TRAFFIC_CYCLE_TIME_DEFAULT);
  nodeName="Traffic";

  botType = mSkin->kb->bot->getBotType();
  switch(botType)
    {
    case SW_KB_GROUND_VEHICLE_TYPE:
      setMsg=new TrafficCtrlSet;
      statMsg=new TrafficCtrlStat;
      trafficData = new TrafficData;
      break;
    default:
      doTrafficCtrlCmdShutdown();
    }

  statMsg->admin_state=ADMIN_UNINITIALIZED;
  reset_time_tracker(&statMsg->tt);
  ((TrafficCtrlSet *)setMsg)->cycleTime=TRAFFIC_CYCLE_TIME_DEFAULT;
  ((TrafficCtrlSet *)setMsg)->debug=0;
  ((TrafficCtrlSet *)setMsg)->vmax = vmax;
  ((TrafficCtrlSet *)setMsg)->amax = amax;
  ((TrafficCtrlSet *)setMsg)->wmax = wmax;
  ((TrafficCtrlSet *)setMsg)->alphamax = alphamax;
  ((TrafficCtrlSet *)setMsg)->vCutoffAngle = vCutoffAngle;
  ((TrafficCtrlSet *)setMsg)->wCutoffAngle = wCutoffAngle;
  ((TrafficCtrlSet *)setMsg)->controlPoint = controlPoint;  

  primEngine = new PrimMobEngine[mSkin->getNumVeh()];
  for( int i=0;i<mSkin->getNumVeh();i++)
    {
      if( mSkin->kb_list[i]->bot->getBotType() != botType )
	{
	  printf( "SimwareTraffic: All robots must be the same; currently given %d and %d\n",
		  botType, mSkin->kb_list[i]->bot->getBotType());
	  doTrafficCtrlCmdShutdown();
	}
      switch( mSkin->kb_list[i]->bot->getSteerType() )
	{
	case SW_KB_ACKERMAN_STEER_TYPE:
	  //	  primEngine[i].setAckermanWheelbase(((SIMwareKBGroundVehicle*)mSkin->kb_list[i]->bot)->getWheelBase());
	  primEngine[i].setAckermanWheelbase(2);
	  break;
	case SW_KB_SKID_STEER_TYPE:
	  break;
	default:
	  printf( "<SIMwareTraffic:: bad steering type: %d\n",
		  mSkin->kb_list[i]->bot->getSteerType() );
	  break;
	}
      primEngine[i].setSettings(((TrafficCtrlSet *)setMsg)->cycleTime,
				((TrafficCtrlSet *)setMsg)->vmax,
				((TrafficCtrlSet *)setMsg)->amax,
				((TrafficCtrlSet *)setMsg)->wmax,
				((TrafficCtrlSet *)setMsg)->alphamax,
				((TrafficCtrlSet *)setMsg)->vCutoffAngle,
				((TrafficCtrlSet *)setMsg)->wCutoffAngle,
				((TrafficCtrlSet *)setMsg)->controlPoint,
				((TrafficCtrlSet *)setMsg)->debug );
    }
  switch(botType)
    {
    case SW_KB_GROUND_VEHICLE_TYPE:
      setMsg=new TrafficCtrlSet;
      statMsg=new TrafficCtrlStat;
      trafficData = new TrafficData;
      break;
    default:
      doTrafficCtrlCmdShutdown();
    }
}

SIMwareTraffic::~SIMwareTraffic() 
{
  if(0)
    printf("<SIMwareTraffic::~SIMwareTraffic>\n");
}
void SIMwareTraffic::run() 
{
  if(1)
    printf("<SIMwareTraffic::run> :  %s\n",nodeName.c_str());
  this->publishSettings();
  while( getNodeState() !=  SW_KB_STATE_SHUTDOWN) 
    {
    doSingleStep();
    timer->wait();
    }
}

void SIMwareTraffic::doSingleStep() 
{
  int i;
  int error;
  TrafficCtrlCmdMoveWaypoint *cmdMoveWaypointPtr;
  TrafficCtrlCmdMoveArcSegment *cmdMoveArcSegmentPtr;

  cycle_time_tracker(&statMsg->tt);
  if(0)
    printf("<SIMwareTraffic::doSingleStep>\n");
  int serialNumber;
  int cfgType = 0;
  int cmdType = 0;
  // Read Configuration Message
  cfgType = cfgBuf->read();
  serialNumber = cfgBuf->get_address()->serial_number;
  switch (cfgType) {
    
  case 0:
    // no new cmd
    break;

  case -1:
    // comm error
    rcs_print_error("trafficCtrl: NML config read error\n");
    break;
    
  case TRAFFIC_CTRL_CFG_NOP_TYPE:
    setMsg->command_type = cfgType;
    if (serialNumber != setMsg->echo_serial_number) {
      setMsg->echo_serial_number = serialNumber;
      setMsg->state = NEW_COMMAND;
      setMsg->status = RCS_EXEC;
    }
    break;

  case TRAFFIC_CTRL_CFG_CYCLE_TIME_TYPE:
    setMsg->command_type = cfgType;
    if (serialNumber != setMsg->echo_serial_number) {
      setMsg->echo_serial_number = serialNumber;
      setMsg->state = NEW_COMMAND;
      setMsg->status = RCS_EXEC;
    }
    break;

  case TRAFFIC_CTRL_CFG_MOVE_TYPE:
    setMsg->command_type = cfgType;
    if (serialNumber != setMsg->echo_serial_number) {
      setMsg->echo_serial_number = serialNumber;
      setMsg->state = NEW_COMMAND;
      setMsg->status = RCS_EXEC;
    }
    break;

  case TRAFFIC_CTRL_CFG_DEBUG_TYPE:
    setMsg->command_type = cfgType;
    if (serialNumber != setMsg->echo_serial_number) {
      setMsg->echo_serial_number = serialNumber;
      setMsg->state = NEW_COMMAND;
      setMsg->status = RCS_EXEC;
    }
    break;

  default:
    rcs_print_error("trafficCtrl: unknown config: %d\n", (int) cfgType);
    break;
  }	
  
  
  // Read Command Message 
  cmdType = cmdBuf->read();
  serialNumber = cmdBuf->get_address()->serial_number;
  switch (cmdType) 
    {
    case 0:
      // no new cmd
      break;
    case -1:
      // comm error
      rcs_print_error("trafficCtrl: NML command read error\n");
      break;

    case TRAFFIC_CTRL_CMD_NOP_TYPE:
    case TRAFFIC_CTRL_CMD_INIT_TYPE:
    case TRAFFIC_CTRL_CMD_HALT_TYPE:
    case TRAFFIC_CTRL_CMD_ABORT_TYPE:
    case TRAFFIC_CTRL_CMD_SHUTDOWN_TYPE:
    case TRAFFIC_CTRL_CMD_STOP_TYPE:
      // we can also accept servo type mobility commands
    case TRAFFIC_CTRL_CMD_ACKERMAN_TYPE:
    case TRAFFIC_CTRL_CMD_SUBMARINE_TYPE:
    case TRAFFIC_CTRL_CMD_SKID_TYPE:  
    case TRAFFIC_CTRL_CMD_FLIP_SKID_TYPE:  
    case TRAFFIC_CTRL_CMD_OMNI_TYPE:  
    case TRAFFIC_CTRL_CMD_ROTARY_WING_TYPE:  
      ((TrafficCtrlStat *)statMsg)->command_type = cmdType;
      if (serialNumber != statMsg->echo_serial_number) {
	statMsg->echo_serial_number = serialNumber;
	statMsg->state = NEW_COMMAND;
	statMsg->status = RCS_EXEC;
      }
      // error stuff
      ((TrafficCtrlStat *)statMsg)->error=0;
      statMsg->message_length=0;
      break;

    case TRAFFIC_CTRL_CMD_MOVE_WAYPOINT_TYPE:
      ((TrafficCtrlStat *)statMsg)->command_type = cmdType;
      if (serialNumber != statMsg->echo_serial_number) {
	statMsg->echo_serial_number = serialNumber;
	statMsg->state = NEW_COMMAND;
	statMsg->status = RCS_EXEC;
	cmdMoveWaypointPtr = (TrafficCtrlCmdMoveWaypoint*)cmdBuf->get_address();
	// set waypoints
	if( cmdMoveWaypointPtr->numVehicles != mSkin->getNumVeh() )
	  {
	    statMsg->state = S2;
	    statMsg->status = RCS_ERROR;
	  }
	for( i=0; i<mSkin->getNumVeh();i++ )
	  primEngine[i].setWaypoints( cmdMoveWaypointPtr->waypoint[i], cmdMoveWaypointPtr->waypoint_length[i]);
      }
      // error stuff
      ((TrafficCtrlStat *)statMsg)->error=0;
      statMsg->message_length=0;
      break;
	    
    case TRAFFIC_CTRL_CMD_MOVE_ARC_SEGMENT_TYPE:
      ((TrafficCtrlStat *)statMsg)->command_type = cmdType;
      if (serialNumber != statMsg->echo_serial_number) {
	statMsg->echo_serial_number = serialNumber;
	statMsg->state = NEW_COMMAND;
	statMsg->status = RCS_EXEC;
	cmdMoveArcSegmentPtr = (TrafficCtrlCmdMoveArcSegment*)cmdBuf->get_address();
			    
	// set waypoints	
	if( cmdMoveArcSegmentPtr->numVehicles != mSkin->getNumVeh() )
	  {
	    statMsg->state = S2;
	    statMsg->status = RCS_ERROR;
	  }
	for( i=0; i<mSkin->getNumVeh();i++ )
	  {
	    // rebuild the waypoint array with this fresh data
	    error=primEngine[i].arc_segments_to_waypoints(cmdMoveArcSegmentPtr->arcsegment[i],cmdMoveArcSegmentPtr->arcsegment_length[i]);
				    
	    if (error < 0)
	      rcs_print_error("Can't generate waypoint array\n");
				    
	    primEngine[i].resetWaypointIndex();
	  }
      }
      // error stuff
      ((TrafficCtrlStat *)statMsg)->error=0;
      statMsg->message_length=0;
      break;

    default:
      rcs_print_error("trafficCtrl: unknown command: %d\n", (int) cmdType);
      break;
    }
  
  
  // Handle Configuration Message
  switch( setMsg->command_type) 
    {
    case TRAFFIC_CTRL_CFG_NOP_TYPE:
      doTrafficCtrlCfgNop();
      break;

    case TRAFFIC_CTRL_CFG_CYCLE_TIME_TYPE:
      doTrafficCtrlCfgCycleTime((TrafficCtrlCfgCycleTime *)cfgBuf->get_address());
      break;

    case TRAFFIC_CTRL_CFG_MOVE_TYPE:
      doTrafficCtrlCfgMove((TrafficCtrlCfgMove *)cfgBuf->get_address());
      break;

    case TRAFFIC_CTRL_CFG_DEBUG_TYPE:
      doTrafficCtrlCfgDebug((TrafficCtrlCfgDebug *)cfgBuf->get_address());
      break;

    default:
      // no config to handle
      break;
  }
  
  // Handle Command Message
  switch (statMsg->command_type) {
    
  case TRAFFIC_CTRL_CMD_NOP_TYPE:
    doTrafficCtrlCmdNop();
    break;
    
  case TRAFFIC_CTRL_CMD_INIT_TYPE:
    doTrafficCtrlCmdInit();
    break;
    
  case TRAFFIC_CTRL_CMD_HALT_TYPE:
    doTrafficCtrlCmdHalt();
    break;
    
  case TRAFFIC_CTRL_CMD_ABORT_TYPE:
    doTrafficCtrlCmdAbort();
    break;
    
  case TRAFFIC_CTRL_CMD_SHUTDOWN_TYPE:
    doTrafficCtrlCmdShutdown();
    break;
    
  case TRAFFIC_CTRL_CMD_STOP_TYPE:
    doTrafficCtrlCmdStop();
    break;

  case TRAFFIC_CTRL_CMD_MOVE_WAYPOINT_TYPE:
    doTrafficCtrlCmdMoveWaypoint(); //(TrafficCtrlCmdMoveWaypoint *)cmdBuf->get_address());
    break;
    
  case TRAFFIC_CTRL_CMD_MOVE_ARC_SEGMENT_TYPE:
    doTrafficCtrlCmdMoveWaypoint(); //(TrafficCtrlCmdMoveWaypoint *)cmdBuf->get_address());
    break;

  case TRAFFIC_CTRL_CMD_ACKERMAN_TYPE:
    doMobCmdDrvAckerman((TrafficCtrlCmdAckerman *)cmdBuf->get_address());
    break;

  case TRAFFIC_CTRL_CMD_SUBMARINE_TYPE:
    doMobCmdDrvSubmarine((TrafficCtrlCmdSubmarine *)cmdBuf->get_address());
    break;
    
  case TRAFFIC_CTRL_CMD_SKID_TYPE:
    doMobCmdDrvSkid((TrafficCtrlCmdSkid *)cmdBuf->get_address());
    break;

  case TRAFFIC_CTRL_CMD_FLIP_SKID_TYPE:
    doMobCmdDrvFlipSkid((TrafficCtrlCmdFlipSkid *)cmdBuf->get_address());
    break;

  case TRAFFIC_CTRL_CMD_OMNI_TYPE:
    doMobCmdDrvOmni((TrafficCtrlCmdOmni *)cmdBuf->get_address());
    break;

  case TRAFFIC_CTRL_CMD_ROTARY_WING_TYPE:
    doMobCmdDrvRotaryWing((TrafficCtrlCmdRotaryWing *)cmdBuf->get_address());
    break;

  default:
    // no cmd to handle
    break;
  }	
  
  if( cfgType > 0 )
    publishSettings();
  sendStatus();
  sendData();
}

void SIMwareTraffic::publishSettings()
{
  int i;

  ((TrafficCtrlSet *)setMsg)->robotInfo_length = mSkin->getNumVeh();
  for( i=0; i<mSkin->getNumVeh();i++ )
    {
      // Get steering type
      switch(mSkin->kb_list[i]->bot->getSteerType())
	{
	case SW_KB_ACKERMAN_STEER_TYPE:
	  ((TrafficCtrlSet *)setMsg)->robotInfo[i].steerType=MOAST_ACKERMAN_STEER_TYPE;
	  break;
	case SW_KB_SKID_STEER_TYPE:
	  ((TrafficCtrlSet *)setMsg)->robotInfo[i].steerType=MOAST_SKID_STEER_TYPE;
	  break;
	case SW_KB_OMNI_STEER_TYPE:
	  ((TrafficCtrlSet *)setMsg)->robotInfo[i].steerType=MOAST_OMNI_STEER_TYPE;
	  break;
	case SW_KB_SUBMARINE_STEER_TYPE:
	  ((TrafficCtrlSet *)setMsg)->robotInfo[i].steerType=MOAST_SUBMARINE_STEER_TYPE;
	  break;
	case SW_KB_ROTARY_WING_STEER_TYPE:
	  ((TrafficCtrlSet *)setMsg)->robotInfo[i].steerType=MOAST_ROTARY_WING_STEER_TYPE;
	  break;
	case SW_KB_UNKNOWN_STEER_TYPE:
	  ((TrafficCtrlSet *)setMsg)->robotInfo[i].steerType=MOAST_UNKNOWN_STEER_TYPE;
	  break;
	default:
	  ((TrafficCtrlSet *)setMsg)->robotInfo[i].steerType=MOAST_INVALID_STEER_TYPE;
	  break;
	}
      // get name
      strcpy(((TrafficCtrlSet *)setMsg)->robotInfo[i].robotName, mSkin->kb_list[i]->bot->getName().c_str());
    }
  /*
  sprintf(((TrafficCtrlSet *)setMsg)->worldName,"%s",mSkin->kb->world.getWorldName().c_str());
  ((TrafficCtrlSet *)setMsg)->length=(mSkin->kb->bot->getDimensions()).x;
  ((TrafficCtrlSet *)setMsg)->width=(mSkin->kb->bot->getDimensions()).y;
  ((TrafficCtrlSet *)setMsg)->height=(mSkin->kb->bot->getDimensions()).z;
  ((TrafficCtrlSet *)setMsg)->mass=mSkin->kb->bot->getMass();
  ((TrafficCtrlSet *)setMsg)->cg.x=(mSkin->kb->bot->getCentroid()).x;
  ((TrafficCtrlSet *)setMsg)->cg.y=(mSkin->kb->bot->getCentroid()).y;
  ((TrafficCtrlSet *)setMsg)->cg.z=(mSkin->kb->bot->getCentroid()).z;
  ((TrafficCtrlSet *)setMsg)->steerType=mSkin->kb->bot->getSteerType();
  ((TrafficCtrlSet *)setMsg)->botType=mSkin->kb->bot->getBotType();
 
  // Get robot type
  switch(mSkin->kb->bot->getBotType())
    {
    case SW_KB_GROUND_VEHICLE_TYPE:
      ((TrafficCtrlSet *)setMsg)->botType=MOAST_GROUND_VEHICLE_TYPE;
      break;
    case SW_KB_NAUTIC_ROBOT_TYPE:
      ((TrafficCtrlSet *)setMsg)->botType=MOAST_NAUTIC_ROBOT_TYPE;
      break;
    case SW_KB_AERIAL_ROBOT_TYPE:
      ((TrafficCtrlSet *)setMsg)->botType=MOAST_AERIAL_ROBOT_TYPE;
      break;
    case SW_KB_UNKNOWN_ROBOT_TYPE:
      ((TrafficCtrlSet *)setMsg)->steerType=MOAST_UNKNOWN_ROBOT_TYPE;
      break;
    default:
      ((TrafficCtrlSet *)setMsg)->steerType=MOAST_INVALID_ROBOT_TYPE;
      break;
    }
  switch(setMsg->type)
    {
    case TRAFFIC_CTRL_SET_GRD_VEH_TYPE:
      
      ((TrafficCtrlSetGrdVeh *)setMsg)->maxSpeed=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getMaxSpeed();
      ((TrafficCtrlSetGrdVeh *)setMsg)->maxTorque=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getMaxTorque();
      ((TrafficCtrlSetGrdVeh *)setMsg)->wheelSeparation=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getWheelSeparation();
      ((TrafficCtrlSetGrdVeh *)setMsg)->wheelRadius=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getWheelRadius();
      ((TrafficCtrlSetGrdVeh *)setMsg)->wheelBase=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getWheelBase();
      ((TrafficCtrlSetGrdVeh *)setMsg)->maxSteerAngle=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getMaxSteerAngle();
      ((TrafficCtrlSetGrdVeh *)setMsg)->maxCrabAngle=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getMaxCrabAngle();
      ((TrafficCtrlSetGrdVeh *)setMsg)->maxTurningRadius=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getMaxTurnRadius();
      setBuf->write((TrafficCtrlSetGrdVeh *)setMsg);
      break;
    default:
      setBuf->write((TrafficCtrlSet *)setMsg);
      break;
    }
  */
  setBuf->write((TrafficCtrlSet *)setMsg);
}
 
void SIMwareTraffic::sendStatus()
{ 
  mSkin->kb->lockBotMutex();
  /*
  for( int count=0; count<mSkin->numVeh; count++ )
    {
      ((TrafficCtrlStat  mSkin->;
    }
  ((TrafficCtrlStat *)statMsg)->heartbeat++;
  ((TrafficCtrlStat *)statMsg)->time=mSkin->kb->bot->getTimeStamp();
  ((TrafficCtrlStat *)statMsg)->batteryVolt=mSkin->kb->bot->getBatteryLevel();
  ((TrafficCtrlStat *)statMsg)->fuelLevel=mSkin->kb->bot->getFuelLevel();
  ((TrafficCtrlStat *)statMsg)->headLights=mSkin->kb->bot->getLightsOn();
  switch(statMsg->type)
    {
    case TRAFFIC_CTRL_SET_GRD_VEH_TYPE:
      ((TrafficCtrlStatGrdVeh *)statMsg)->steerAngle=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getSteerAngle();
      ((TrafficCtrlStatGrdVeh *)statMsg)->crabAngle=((SIMwareKBGroundVehicle *)mSkin->kb->bot)->getCrabAngle();
      statBuf->write((TrafficCtrlStatGrdVeh *)statMsg);
      break;
    default:
      statBuf->write((TrafficCtrlStat *)statMsg);
      break;
    }
  */
  mSkin->kb->unlockBotMutex();
  statBuf->write((TrafficCtrlStat *)statMsg);
}

void SIMwareTraffic::errorHandler(int error) 
{
  switch(error)
    {
    case SE0:
      ((TrafficCtrlStat *)statMsg)->error=SE0;
      snprintf(statMsg->message,80,"Robot is not initialized%c",(char)NULL);
      statMsg->message_length=strlen(statMsg->message);
      break;
    default:
      break;
    }

}
void SIMwareTraffic::doTrafficCtrlCmdInit()
{
  int i;

  if (state_match(statMsg, NEW_COMMAND)) 
    { 
      statMsg->admin_state=ADMIN_INITIALIZED;
      state_new(statMsg);
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);      
      ((TrafficCtrlStat *)statMsg)->primState_length = mSkin->getNumVeh();
      ((TrafficCtrlStat *)statMsg)->pathIndex_length = mSkin->getNumVeh();
      ((TrafficCtrlStat *)statMsg)->primStatus_length = mSkin->getNumVeh();
      for( i=0; i<mSkin->getNumVeh();i++)
	{
	  ((TrafficCtrlStat *)statMsg)->primStatus[i] = primEngine[i].doCmdInit();
	  ((TrafficCtrlStat *)statMsg)->pathIndex[i] = -1;
	  ((TrafficCtrlStat *)statMsg)->primState[i] = S1;
	  if( ((TrafficCtrlStat *)statMsg)->primStatus[i] != RCS_DONE )
	    status_next(statMsg, RCS_ERROR);
	  mSkin->kb_list[i]->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_INIT_TYPE));
	}
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doTrafficCtrlCmdStop()
{
  if (state_match(statMsg, NEW_COMMAND)) 
    { 
      state_new(statMsg);
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);      
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doTrafficCtrlCmdNop() 
{
   if (state_match(statMsg, NEW_COMMAND)) 
     {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      //      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_NOP_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);
  
}

void SIMwareTraffic::doTrafficCtrlCmdHalt() 
{
   if (state_match(statMsg, NEW_COMMAND)) 
     {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);
  
}

void SIMwareTraffic::doTrafficCtrlCmdAbort() 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doTrafficCtrlCmdShutdown() 
{
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      statMsg->admin_state=ADMIN_UNINITIALIZED;
      state_new(statMsg);
      mSkin->kb->addCmdToQ(new SIMwareMobCmd(SW_MOB_CMD_POWERDOWN_TYPE));
      status_next(statMsg, RCS_DONE);  
      state_next(statMsg, S0);
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doTrafficCtrlCmdMoveWaypoint() //TrafficCtrlCmdMoveWaypoint * sWaypointCmd)
{
  int i;
  double trash;
  double velocity, steerAngle, sternAngle;

  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      statMsg->status = RCS_EXEC;
      state_next(statMsg, S0);
      if (statMsg->admin_state != ADMIN_INITIALIZED) 
	{
	  status_next(statMsg, RCS_ERROR);
	} 
      else 
	{
	  for( i=0; i<mSkin->getNumVeh();i++)
	    {
	      ((TrafficCtrlStat *)statMsg)->primStatus[i] = primEngine[i].doCmdMoveWaypoint(NEW_COMMAND, 
											    trafficData->vehicles[i].tranRelRate,
											    trafficData->vehicles[i].rpyRelRate,
											    trafficData->vehicles[i].tranRel,
											    trafficData->vehicles[i].rpyRel,
											    ((TrafficCtrlStat *)statMsg)->pathIndex[i], 
											    ((TrafficCtrlStat *)statMsg)->primState[i], 
											    trash, trash, trash);
	      if( ((TrafficCtrlStat *)statMsg)->primStatus[i] == RCS_ERROR )
		statMsg->status = RCS_ERROR;
	    }
	}
    }
  // and fall through to here if S0...
  for( i=0; i<mSkin->getNumVeh();i++)
    {
      if(((TrafficCtrlStat *)statMsg)->primState[i] == S0) 
	{
	  ((TrafficCtrlStat *)statMsg)->primStatus[i] = primEngine[i].doCmdMoveWaypoint(S0, 
											trafficData->vehicles[i].tranRelRate,
											trafficData->vehicles[i].rpyRelRate,
											trafficData->vehicles[i].tranRel,
											trafficData->vehicles[i].rpyRel,
											((TrafficCtrlStat *)statMsg)->pathIndex[i], 
											((TrafficCtrlStat *)statMsg)->primState[i], 
											velocity,
											steerAngle,
											sternAngle);
	  if( primEngine[i].isAckerman())
	    mSkin->kb_list[i]->addCmdToQ(new SIMwareAckermanCmd(velocity, steerAngle, 0));
	  else
	    mSkin->kb_list[i]->addCmdToQ(new SIMwareSkidCmd(velocity, steerAngle));
	}
      else if(((TrafficCtrlStat *)statMsg)->primState[i] == S1) 
	{			// S1 = stopping
	  // done
	  ((TrafficCtrlStat *)statMsg)->primStatus[i] = primEngine[i].doCmdMoveWaypoint(S1, 
											trafficData->vehicles[i].tranRelRate,
											trafficData->vehicles[i].rpyRelRate,
											trafficData->vehicles[i].tranRel,
											trafficData->vehicles[i].rpyRel,
											((TrafficCtrlStat *)statMsg)->pathIndex[i], 
											((TrafficCtrlStat *)statMsg)->primState[i], 
											velocity,
											steerAngle,
											sternAngle);
	  if( primEngine[i].isAckerman())
	    mSkin->kb_list[i]->addCmdToQ(new SIMwareAckermanCmd(velocity, steerAngle, 0));
	  else
	    mSkin->kb_list[i]->addCmdToQ(new SIMwareSkidCmd(velocity, steerAngle));
	} 
      else 
	{
	  state_default(statMsg);
	}
    }
  return;
}

void SIMwareTraffic::doMobCmdDrvSkid(TrafficCtrlCmdSkid * skidDrvCmd) 
{
  int i;

  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else 
	{
	  state_next(statMsg, S0);
	  for( i=0; i<skidDrvCmd->cmd_length; i++)
	    if( ((TrafficCtrlSet *)setMsg)->robotInfo[skidDrvCmd->cmd[i].vehID].steerType!= MOAST_SKID_STEER_TYPE) 
	      state_next(statMsg, SE1);
	}
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      for( i=0; i<skidDrvCmd->cmd_length; i++)
	mSkin->kb_list[skidDrvCmd->cmd[i].vehID]->addCmdToQ(new SIMwareSkidCmd(skidDrvCmd->cmd[i].wLeft,skidDrvCmd->cmd[i].wRight));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}



void SIMwareTraffic::doMobCmdDrvFlipSkid(TrafficCtrlCmdFlipSkid * fSkidDrvCmd) 
{
  SIMwareFlipperCmd * flipperCmds;
  SIMwareFlipperPair fPair;
  int i;

  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else 
	{
	  state_next(statMsg, S0);
	  for( i=0; i<fSkidDrvCmd->cmd_length; i++)
	    if( ((TrafficCtrlSet *)setMsg)->robotInfo[fSkidDrvCmd->cmd[i].vehID].steerType!= MOAST_SKID_STEER_TYPE) 
	      state_next(statMsg, SE1);
	}

      /*
	else if( ((TrafficCtrlSetGrdVeh *)setMsg)->fSet_length==0)
	state_next(statMsg, SE2);
      */
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);
      for( i=0; i<fSkidDrvCmd->cmd_length; i++)
	mSkin->kb_list[fSkidDrvCmd->cmd[i].vehID]->addCmdToQ(new SIMwareSkidCmd(fSkidDrvCmd->cmd[i].wLeft,fSkidDrvCmd->cmd[i].wRight));
      for( i=0; i<fSkidDrvCmd->fCmd_length; i++ )
	{
	  flipperCmds = new SIMwareFlipperCmd;
	  switch(fSkidDrvCmd->fCmd[i].fType)
	    {
	    case MOAST_FR_FLIPPER_TYPE:
	      fPair.fType=SW_KB_FR_FLIPPER_TYPE;
	      break;
	      
	    case MOAST_FL_FLIPPER_TYPE:
	      fPair.fType=SW_KB_FL_FLIPPER_TYPE;
	      break;

	    case MOAST_RR_FLIPPER_TYPE:
	      fPair.fType=SW_KB_RR_FLIPPER_TYPE;
	      break;

	    case MOAST_RL_FLIPPER_TYPE:
	      fPair.fType=SW_KB_RL_FLIPPER_TYPE;
	      break;
	      
	    default:
	      continue;
	    }
	  fPair.flipperAngle=fSkidDrvCmd->fCmd[i].flipperAngle;
	  flipperCmds->fCmd.push_back(fPair);
	  mSkin->kb_list[fSkidDrvCmd->fCmd[i].vehID]->addCmdToQ(flipperCmds);
	}
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    } 
  else if (state_match(statMsg, SE2))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE2);
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doMobCmdDrvAckerman(TrafficCtrlCmdAckerman * ackDrvCmd) 
{
  int i;

  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else 
	{
	  state_next(statMsg, S0);
	  for( i=0; i<ackDrvCmd->cmd_length; i++)
	    if( ((TrafficCtrlSet *)setMsg)->robotInfo[ackDrvCmd->cmd[i].vehID].steerType!= MOAST_ACKERMAN_STEER_TYPE) 
	      state_next(statMsg, SE1);
	}
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);

      for( i=0; i<ackDrvCmd->cmd_length;i++)
	mSkin->kb_list[i]->addCmdToQ(new SIMwareAckermanCmd(ackDrvCmd->cmd[i].velocity,
						      ackDrvCmd->cmd[i].steerAngle,
						      ackDrvCmd->cmd[i].crabAngle));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doMobCmdDrvSubmarine(TrafficCtrlCmdSubmarine * sDrvCmd) 
{
  int i;

  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else 
	{
	  state_next(statMsg, S0);
	  for( i=0; i<sDrvCmd->cmd_length; i++)
	    if( ((TrafficCtrlSet *)setMsg)->robotInfo[sDrvCmd->cmd[i].vehID].steerType!= MOAST_SUBMARINE_STEER_TYPE) 
	      state_next(statMsg, SE1);
	}
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);

      for( i=0; i<sDrvCmd->cmd_length;i++)
	mSkin->kb_list[i]->addCmdToQ(new SIMwareSubmarineCmd(sDrvCmd->cmd[i].propeller,
						       sDrvCmd->cmd[i].rudder,
						       sDrvCmd->cmd[i].sternPlane));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doMobCmdDrvOmni(TrafficCtrlCmdOmni * omniDrvCmd) 
{
  int i;
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);
      
      else 
	{
	  state_next(statMsg, S0);
	  for( i=0; i<omniDrvCmd->cmd_length; i++)
	    if( ((TrafficCtrlSet *)setMsg)->robotInfo[omniDrvCmd->cmd[i].vehID].steerType!= MOAST_OMNI_STEER_TYPE) 
	      state_next(statMsg, SE1);
	}
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);

      for( i=0; i<omniDrvCmd->cmd_length; i++)
	mSkin->kb_list[omniDrvCmd->cmd[i].vehID]->addCmdToQ(new SIMwareOmniCmd(omniDrvCmd->cmd[i].vLinear,
						omniDrvCmd->cmd[i].vLateral,
						omniDrvCmd->cmd[i].rot));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doMobCmdDrvRotaryWing(TrafficCtrlCmdRotaryWing * rwDrvCmd) 
{
  int i;
  if (state_match(statMsg, NEW_COMMAND)) 
    {
      state_new(statMsg);
      status_next(statMsg, RCS_EXEC);
      
      if( statMsg->admin_state != ADMIN_INITIALIZED ) 
	state_next(statMsg, SE0);

      else 
	{
	  state_next(statMsg, S0);
	  for( i=0; i<rwDrvCmd->cmd_length; i++)
	    if( ((TrafficCtrlSet *)setMsg)->robotInfo[rwDrvCmd->cmd[i].vehID].steerType!= MOAST_ROTARY_WING_STEER_TYPE) 
	      state_next(statMsg, SE1);
	}
    }

  if (state_match(statMsg, S0))
    {
      status_next(statMsg, RCS_EXEC);
      state_next(statMsg, NOP_STATE);

      
      for( i=0; i<rwDrvCmd->cmd_length; i++)
	mSkin->kb_list[rwDrvCmd->cmd[i].vehID]->addCmdToQ(new SIMwareRotaryWingCmd(rwDrvCmd->cmd[i].vLinear,
						      rwDrvCmd->cmd[i].vLateral,
						      rwDrvCmd->cmd[i].vVertical,
						      rwDrvCmd->cmd[i].vRotation));
    }
  else if (state_match(statMsg, SE0))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE0);
    }
  else if (state_match(statMsg, SE1))
    {
      status_next(statMsg, RCS_ERROR);
      state_next(statMsg, NOP_STATE);
      errorHandler(SE1);
    }
  else
    state_default(statMsg);  
}

void SIMwareTraffic::doTrafficCtrlCfgCycleTime( TrafficCtrlCfgCycleTime * cTCmd ) 
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      state_new(setMsg);
      ((TrafficCtrlSet *)setMsg)->cycleTime = cTCmd->cycleTime;
      setTimer(((TrafficCtrlSet *)setMsg)->cycleTime);
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);   
    }
  else
    state_default(setMsg);  
  
}

void SIMwareTraffic::doTrafficCtrlCfgMove( TrafficCtrlCfgMove * cTCmd ) 
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);   
    }
  else
    state_default(setMsg);  
}

void SIMwareTraffic::doTrafficCtrlCfgDebug( TrafficCtrlCfgDebug * cTCmd ) 
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      state_new(setMsg);
      ((TrafficCtrlSet *)setMsg)->debug = cTCmd->debug;
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);
    }
  else
    state_default(setMsg); 
}

void SIMwareTraffic::doTrafficCtrlCfgNop()
{
  if (state_match(setMsg, NEW_COMMAND)) 
    {
      state_new(setMsg);
      status_next(setMsg, RCS_DONE);
      state_next(setMsg, NOP_STATE);
    }
  else
    state_default(setMsg); 
}

void SIMwareTraffic::setTimer(double t) 
{
  delete timer;
  timer=new RCS_TIMER(t);
}

bool SIMwareTraffic::openNML(string cmdName, 
				   string statName, 
				   string cfgName,
				   string setName,
				   NML_FORMAT_PTR fPtr,
				   int compID) {
  stringstream str; 
  char suffix[10];
  
  if(1)
    printf("<SIMwareTraffic::openNML> Opening NML Buffers\n");
  
  // Check to see if there is a component id, this changes naming convention
  if(compID==0)
    sprintf(suffix,"%d",mSkin->kb->bot->getBotID());
  else
    sprintf(suffix,"%d_%d",mSkin->kb->bot->getBotID(),compID);
  
  str << cmdName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  cmdBuf = new RCS_CMD_CHANNEL(fPtr, 
			       str.str().c_str(), 
			       mSkin->nmlProcName.c_str(), 
			       mSkin->nmlCfgFile.c_str());
  if (!cmdBuf->valid()) 
    {
      delete cmdBuf;
      cmdBuf = NULL;
      return false;
    }
  cmdBuf->clear();
  str.str("");


  str << cfgName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  cfgBuf = new RCS_CMD_CHANNEL(fPtr, 
			       str.str().c_str(), 
			       mSkin->nmlProcName.c_str(), 
			       mSkin->nmlCfgFile.c_str());
  if (!cfgBuf->valid()) 
    {
      delete cfgBuf;
      cfgBuf = NULL;
      return false;
    }
  cfgBuf->clear();
  str.str("");
  
  str << statName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  statBuf = new RCS_STAT_CHANNEL(fPtr, 
				 str.str().c_str(), 
				 mSkin->nmlProcName.c_str(), 
				 mSkin->nmlCfgFile.c_str());
  if (!statBuf->valid()) 
    {
      delete statBuf;
      statBuf = NULL;
      return false;
    }
  statBuf->clear();
  str.str("");
  
  
  str << setName << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  setBuf = new RCS_STAT_CHANNEL(fPtr, 
				str.str().c_str(), 
				mSkin->nmlProcName.c_str(), 
				mSkin->nmlCfgFile.c_str());
  if (!setBuf->valid()) 
    {
      delete setBuf;
      setBuf = NULL;
      return false;
    }
  setBuf->clear();
  str.str("");

  str << TRAFFIC_DATA_NAME << suffix;
  if(1)
    printf("\t%s\n", str.str().c_str());
  dataBuf = new NML(trafficData_format, 
		    str.str().c_str(), 
		    mSkin->nmlProcName.c_str(), 
		    mSkin->nmlCfgFile.c_str());
  if (!dataBuf->valid()) 
    {
      delete dataBuf;
      dataBuf = NULL;
      return false;
    }
  dataBuf->clear();
  str.str("");
  return true;
}

void SIMwareTraffic::cleanUpNMLBuf()
{
  printf("<SIMwareTraffic::cleanUpNMLBuf> : %s\n",nodeName.c_str());  

  //nml_cleanup();
  // if (NULL != cmdBuf)
  //     delete cmdBuf;
  //   cmdBuf=NULL;
  
  //   if (NULL != statBuf)
  //     delete statBuf;
  //   statBuf=NULL;
  
  //   if (NULL != setBuf)
  //     delete setBuf;
  //   setBuf=NULL;
  
  //   if (NULL != cfgBuf)
  //     delete cfgBuf;
  //   cfgBuf=NULL;
} 

SIMwareStateType SIMwareTraffic::getNodeState() {
  switch(statMsg->admin_state) {
  case ADMIN_UNINITIALIZED:
    return SW_KB_STATE_UNINITIALIZED;
  case ADMIN_INITIALIZED:
    return SW_KB_STATE_INITIALIZED;
  case ADMIN_SHUT_DOWN:
    return SW_KB_STATE_SHUTDOWN;
  default:
    return SW_KB_STATE_NULL;
  }
}

void SIMwareTraffic::sendData() 
{
  bool needToWrite = false;
  int i;  
  double delta_t;
  
  if(0)
    printf("<SIMwareTraffic::sendData> :  %s\n",nodeName.c_str());

  delta_t = mSkin->kb->bot->getTimeStamp() - trafficData->time; 
  if( delta_t <= 0 )
    return;
  trafficData->time = mSkin->kb->bot->getTimeStamp();
  for(i=0;i<mSkin->getNumVeh();i++)
    {
      if( mSkin->grdTruth[i]->hasNewData())
	{
	  if( mSkin->grdTruth[i]->getSensorType() != SW_KB_GRD_TRUTH_SENSOR_TYPE )
	    continue;
	  needToWrite = true;
	  mSkin->kb_list[i]->lockSenMutex();
	  trafficData->vehicles[i].tranTrueRate = (mSkin->grdTruth[i]->getLocation() - 
						   trafficData->vehicles[i].tranTrue)/delta_t;
	  trafficData->vehicles[i].rpyTrueRate = (mSkin->grdTruth[i]->getOrientation() -
						  trafficData->vehicles[i].rpyTrue)/delta_t;
	  trafficData->vehicles[i].tranTrue = mSkin->grdTruth[i]->getLocation();
	  trafficData->vehicles[i].rpyTrue = mSkin->grdTruth[i]->getOrientation();
	  trafficData->vehicles[i].steeringCurvature = ((SIMwareKBGroundVehicle *)mSkin->kb_list[i]->bot)->getSteerAngle();

	  trafficData->vehicles[i].tranAbs = trafficData->vehicles[i].tranTrue;
	  trafficData->vehicles[i].rpyAbs = trafficData->vehicles[i].rpyTrue;
	  trafficData->vehicles[i].tranAbsRate = trafficData->vehicles[i].tranTrueRate;
	  trafficData->vehicles[i].rpyAbsRate = trafficData->vehicles[i].rpyTrueRate;

	  trafficData->vehicles[i].tranRel = trafficData->vehicles[i].tranTrue;
	  trafficData->vehicles[i].rpyRel = trafficData->vehicles[i].rpyTrue;
	  trafficData->vehicles[i].tranRelRate = trafficData->vehicles[i].tranTrueRate;
	  trafficData->vehicles[i].rpyRelRate = trafficData->vehicles[i].rpyTrueRate;

	  trafficData->vehicles[i].speed = sqrt(trafficData->vehicles[i].tranRelRate.x * trafficData->vehicles[i].tranRelRate.x + 
						trafficData->vehicles[i].tranRelRate.y * trafficData->vehicles[i].tranRelRate.y + 
						trafficData->vehicles[i].tranRelRate.z * trafficData->vehicles[i].tranRelRate.z); 
	  /*
	  printf( "xyz: <%.3f, %.3f, %.3f> speed: %.3f time: %.3f\n", 
		  trafficData->vehicles[i].tranTrueRate.x,
		  trafficData->vehicles[i].tranTrueRate.y,
		  trafficData->vehicles[i].tranTrueRate.z,
		  trafficData->vehicles[i].speed, delta_t);
	  */
	  mSkin->grdTruth[i]->resetNewDataFlag();
	  mSkin->kb_list[i]->unlockSenMutex();
	}
      //      else
      //	printf("<SIMwareTraffic::sendData> no new data from vehicle %d\n", i );
    }
  trafficData->vehicles_length = mSkin->getNumVeh();
  if(needToWrite)
    dataBuf->write(trafficData);
}
