/*
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*/

/*!
  \file skel.hh

  \brief NML declarations for skel application
  and settings.

  Synopsis:

  \code CVS Status:
  $Author: dr_steveb $
  $Revision: 1.3 $
  $Date: 2008/09/19 02:53:51 $
  \endcode

  \author Fred Proctor & Stephen Balakirsky
*/

#ifndef SKEL_HH
#define SKEL_HH

/*
  skel_stat.hh

  NML declarations for skel status.

  Modification history:

  24-Feb-2005  FMP created
*/

#include "rcs.hh"		// RCS_STAT_MSG, NMLTYPE, CMS
#include "skelCommon.hh"	// SKEL_STAT_BASE

#define SKEL_CMD_NAME "skelCmd"
#define SKEL_STAT_NAME "skelStat"
#define SKEL_CFG_NAME "skelCfg"
#define SKEL_SET_NAME "skelSet"

#define SKEL_STAT_TYPE (SKEL_STAT_BASE + 1)

#define SKEL_SET_TYPE (SKEL_SET_BASE + 1)

#define SKEL_CMD_INIT_TYPE          (SKEL_CMD_BASE + 1)
#define SKEL_CMD_GO_TYPE            (SKEL_CMD_BASE + 2)
#define SKEL_CMD_ABORT_TYPE         (SKEL_CMD_BASE + 3)
#define SKEL_CMD_HALT_TYPE          (SKEL_CMD_BASE + 4)
#define SKEL_CMD_SHUTDOWN_TYPE      (SKEL_CMD_BASE + 5)
#define SKEL_CMD_NOP_TYPE           (SKEL_CMD_BASE + 6)


#define SKEL_CFG_LENGTH_TYPE      (SKEL_CFG_BASE + 1)
#define SKEL_CFG_WIDTH_TYPE       (SKEL_CFG_BASE + 2)
#define SKEL_CFG_DEBUG_TYPE       (SKEL_CFG_BASE + 3)
#define SKEL_CFG_CYCLE_TIME_TYPE  (SKEL_CFG_BASE + 4)
#define SKEL_CFG_NOP_TYPE         (SKEL_CFG_BASE + 5)

class SkelStat : public RCS_STAT_MSG_V2 {
public:
  SkelStat() : RCS_STAT_MSG_V2(SKEL_STAT_TYPE, sizeof(SkelStat)) {};
  void update(CMS *);

  double where;
  double here;
};

class SkelSet : public RCS_STAT_MSG_V2 {
public:
  SkelSet() : RCS_STAT_MSG_V2(SKEL_SET_TYPE, sizeof(SkelSet)) {};
  void update(CMS *);

  double length;
  double lengthReq;
  double width;
  double widthReq;
  double cycleTime;
  int debug;
};

class SkelCmdNop : public RCS_CMD_MSG {
public:
  SkelCmdNop() : RCS_CMD_MSG(SKEL_CMD_NOP_TYPE, sizeof(SkelCmdNop)) {};
  void update(CMS *);
};

class SkelCmdInit : public RCS_CMD_MSG {
public:
  SkelCmdInit() : RCS_CMD_MSG(SKEL_CMD_INIT_TYPE, sizeof(SkelCmdInit)) {};
  void update(CMS *);
};

class SkelCmdGo : public RCS_CMD_MSG {
public:
  SkelCmdGo() : RCS_CMD_MSG(SKEL_CMD_GO_TYPE, sizeof(SkelCmdGo)) {};
  void update(CMS *);

  double where;
};

class SkelCmdHalt:public RCS_CMD_MSG {
public:
  SkelCmdHalt():RCS_CMD_MSG (SKEL_CMD_HALT_TYPE, sizeof(SkelCmdHalt)) {};
  void update(CMS *);
};

class SkelCmdAbort:public RCS_CMD_MSG {
public:
  SkelCmdAbort():RCS_CMD_MSG (SKEL_CMD_ABORT_TYPE, sizeof(SkelCmdAbort)) {};
  void update(CMS *);
};

class SkelCmdShutdown:public RCS_CMD_MSG {
public:
  SkelCmdShutdown():RCS_CMD_MSG (SKEL_CMD_SHUTDOWN_TYPE, sizeof(SkelCmdShutdown)) {};
  void update(CMS *);
};

class SkelCfgNop : public RCS_CMD_MSG {
public:
  SkelCfgNop() : RCS_CMD_MSG(SKEL_CFG_NOP_TYPE, sizeof(SkelCfgNop)) {};
  void update(CMS *);
};

class SkelCfgDebug : public RCS_CMD_MSG {
public:
  SkelCfgDebug() : RCS_CMD_MSG(SKEL_CFG_DEBUG_TYPE, sizeof(SkelCfgDebug)) {};
  void update(CMS *);

  int debug;
};

class SkelCfgCycleTime : public RCS_CMD_MSG {
public:
  SkelCfgCycleTime() : RCS_CMD_MSG(SKEL_CFG_CYCLE_TIME_TYPE, sizeof(SkelCfgCycleTime)) {};
  void update(CMS *);

  double cycleTime;
};

class SkelCfgLength : public RCS_CMD_MSG {
public:
  SkelCfgLength() : RCS_CMD_MSG(SKEL_CFG_LENGTH_TYPE, sizeof(SkelCfgLength)) {};
  void update(CMS *);

  double length;
};

class SkelCfgWidth : public RCS_CMD_MSG {
public:
  SkelCfgWidth() : RCS_CMD_MSG(SKEL_CFG_WIDTH_TYPE, sizeof(SkelCfgWidth)) {};
  void update(CMS *);

  double width;
};

extern int skel_format(NMLTYPE type, void *buf, CMS * cms);
extern const char * skel_symbol_lookup(long type);

#endif // SKEL_HH
