/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/

/*!
  \file moastscript.cc
  \brief Scripting interface to Moast NML commands, status.
  moastscript.cc is intended to be compiled and linked into scripting
  language modules for Tcl, Perl, Python, etc. The accompanying file
  moastscript.i is used by SWIG in a maintainer's system to build the
  script code necessary for each language.

  \code CVS Status:
  $Author: root $
  $Revision: 1.1.1.1 $
  $Date: 2005/10/12 20:50:32 $
  \endcode

  \author Fred Proctor
*/

#include <stddef.h>		// NULL
#include <rcs.hh>		// RCS_CMD,STAT_CHANNEL, rcs_print()
#include "moastTypes.hh"	// MOAST_NML_BUFFER_NAME_LEN
#include "moastNmlOffsets.hh"	// SERVO_MOB_JA_BASE, etc.
#include "moastNmlGlobal.hh"	// servoMobJABuf[], etc.
#include "servoMobJA.hh"	// servoMobJA_format()
#include "moastscript.hh"	// these decls

#define DEFAULT_NML_FILE "../etc/moast.nml"
#define THIS_PROCESS "moastscript"	// NML process name for the server

int nmlConnect(char * nml_file_given)
{
  char nml_file_default[] = DEFAULT_NML_FILE;
  char * nml_file;
  int retval;

  if (NULL == nml_file_given ||
      0 == nml_file_given[0]) {
    // no arg-- use default NML file
    nml_file = nml_file_default;
  } else {
    nml_file = nml_file_given;
  }

  // connect to the objective mob nav buffers
  retval = moastInitNmlBuffers(nml_file, THIS_PROCESS);
  if (retval) {
    return 1;
  }

  return 0;
}

int nmlDisconnect(void)
{
  (void) moastCleanupNmlBuffers();

  return 0;
}

/*
  moastChannelIndex is the index, starting at 0, for the NML channel
  to be looked at. The .nml file starts at 1 since it's human-readable.
*/

static int nmlChannelIndex = 0;

int nmlSetChannel(int channel)
{
  nmlChannelIndex = (channel <= 0 ? 0 : channel - 1);

  return 0;
}

int servoMobJACmdInit(void)
{
  ServoMobJACmdInit cmd;
  int channel;

  channel = nmlChannelIndex;
  if (channel >= NUM_SERVO_MOB_JA_BUFFERS) {
    channel = NUM_SERVO_MOB_JA_BUFFERS - 1;
  }

  if (NULL == servoMobJACmdBuf[channel] ||
      ! servoMobJACmdBuf[channel]->valid()) {
    return 1;
  }

  if (NULL == servoMobJAStatBuf[channel] ||
      ! servoMobJAStatBuf[channel]->valid() ||
      SERVO_MOB_JA_STAT_TYPE != servoMobJAStatBuf[channel]->peek()) {
    return 1;
  }

  cmd.serial_number = servoMobJAStatBuf[channel]->get_address()->echo_serial_number + 1;
  servoMobJACmdBuf[channel]->write(&cmd);

  return 0;
}

/*
  Modification history:

  $Log: moastscript.cc,v $
  Revision 1.1.1.1  2005/10/12 20:50:32  root
  initial import

  Revision 1.5  2005/09/30 18:55:56  proctor
  Cleaned up unused variables, forgotten return values

  Revision 1.4  2005/07/06 18:51:25  proctor
  Stripped Tcl-specific code out of moastscript.cc in favor of swig-generated
  code

  Revision 1.3  2005/06/08 14:31:11  proctor
  Added MobJA name changes

  Revision 1.2  2005/06/07 21:45:52  proctor
  Changed servoPL to servoMobJA

  Revision 1.1  2005/06/02 18:37:44  proctor
  Initial checkin

*/
