/*****************************************************************************
  DISCLAIMER:
  This software was produced by the National Institute of Standards
  and Technology (NIST), an agency of the U.S. government, and by statute is
  not subject to copyright in the United States.  Recipients of this software
  assume all responsibility associated with its operation, modification,
  maintenance, and subsequent redistribution.

  See NIST Administration Manual 4.09.07 b and Appendix I. 
*****************************************************************************/
/*!
  \file   sweep.hh
  \brief  provides an implementation of a line sweep algorithm for polygon intersection

  This algorithm may be found in the CS 373 Non-Lecture F: Line Segment 
  Intersecion document from the Fall 2002.

  \code CVS Status:
  $Author: proctor $
  $Revision: 1.3 $
  $Date: 2007/12/18 21:34:46 $
  \endcode

  \author Stephen Balakirsky
  \date   06/01/2007
*/
#ifndef __sweep__
#define __sweep__
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <list>
#include <vector>
#include "moastTypes.hh"
#include "moastGeom.hh"
using namespace std;

//////////////////////////////////////////////////////////////////////////////
// supporting classes
/////////////////////////////////////////////////////////////////////////////
class LinePoint
{
public:
  LinePoint();
  LinePoint(const LinePoint &);
  ~LinePoint(){};
  LinePoint &operator=(const LinePoint &rhs);
  int operator==(const LinePoint &rhs) const;
  int operator<(const LinePoint &rhs) const;

  bool isStart;
  //  LineSegment *myLine;
  LineSegment *myLine; // really contains a LineSegment pointer
};

class LabelSeq
{
private:
  list<LinePoint>::iterator findNode( LinePoint *point );
  //! list of sweep labels
public:
  list<LineSegment*> labels; 
  LabelSeq();
  bool CCW( Location2D &p1, Location2D &p2, Location2D &p3 );
  Location2D findIntersection( LineSegment *line1In, LineSegment *line2In );
  bool getNeighbors( LineSegment **pointIn, 
		     LineSegment **top,
		     LineSegment **bottom );
  bool isInterior( LineSegment *line, Location2D &point );
  double pointLineDistance( LineSegment *line, Location2D &point);
  void seqDelete(LineSegment *point);
  void seqInsert(LineSegment *point, Location2D &sweepValue);
  void clear();
};

//////////////////////////////////////////////////////////////////////////////
// end ofsupporting classes
/////////////////////////////////////////////////////////////////////////////

// note that points are treated as (x,y) pairs that correspond to (north, east)
class Sweep
{
public:
  Sweep();
  ~Sweep();
  void anyIntersections();
  void findNewEvent( LineSegment *upper, LineSegment *lower, Location2D &sweep );
  void handleEventPoint( Location2D &currentPoint );
  bool intersect( LineSegment *l1, LineSegment *l2 );
  //! raw segments of polygons
  vector<LineSegment> segments;
  //! list of polygon endpoints
  list<Location2D> pointList;
  //! label sequence for intersection detection
  LabelSeq labelseq;

};

#endif
/*
  Modification history:

  $Log: sweep.hh,v $
  Revision 1.3  2007/12/18 21:34:46  proctor
  In prim mob, read out the platform name from servo mob and used it to
  get platform-specific parameters V_CUTOFF_ANGLE et al; changed moast.ini
  accordingly; fixed a return value error in the segment sweep code, where
  anyIntersections doesn't really return anything

  Revision 1.2  2007/11/08 13:22:30  dr_steveb
  Not quite ready, but premature checking-in of other files means that these need to go to!

  Revision 1.1  2007/09/12 15:23:38  dr_steveb
  Added tools for generating sweep line for pologon world model

*/
